/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.caching;

import java.io.Serializable;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springmodules.cache.CacheException;


/**
 * Provides a namespacing utility for using a cache. The namespace can
 * be specified explicity using the <code>namespace</code> attribute or
 * it can be resolved dynamicly. If using dynamic resolution a stack trace
 * is generated for every request to the cache and the class name making
 * the call is used as the namespace key. Note the dynamic method is
 * <u>not</u> recomended and may be slow. 
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class NamespacingCacheWrapper implements ISimpleCacheProviderFacade {
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private ISimpleCacheProviderFacade wrappedCache;
    private String namespace;
    
    /**
     * @return Returns the namespace.
     */
    public String getNamespace() {
        return this.namespace;
    }
    /**
     * @param namespace The namespace to set.
     */
    public void setNamespace(String namespace) {
        this.namespace = namespace;
    }
    
    /**
     * @return Returns the wrappedCache.
     */
    public ISimpleCacheProviderFacade getWrappedCache() {
        return this.wrappedCache;
    }
    /**
     * @param wrappedCache The wrappedCache to set.
     */
    public void setWrappedCache(ISimpleCacheProviderFacade wrappedCache) {
        this.wrappedCache = wrappedCache;
    }
    
    
    
    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#cancelCacheUpdate(java.io.Serializable)
     */
    public void cancelCacheUpdate(Serializable key) throws CacheException {
        final String validNamespace = this.getValidNamespace();
        final NamespacedKey namespacedKey = new NamespacedKey(validNamespace, key);
        this.wrappedCache.cancelCacheUpdate(namespacedKey);
    }
    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#flushCache()
     */
    public void flushCache() {
        this.wrappedCache.flushCache();
    }
    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#getFromCache(java.io.Serializable)
     */
    public Object getFromCache(Serializable key) throws CacheException {
        final String validNamespace = this.getValidNamespace();
        final NamespacedKey namespacedKey = new NamespacedKey(validNamespace, key);
        return this.wrappedCache.getFromCache(namespacedKey);
    }
    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#isFailQuietlyEnabled()
     */
    public boolean isFailQuietlyEnabled() {
        return this.wrappedCache.isFailQuietlyEnabled();
    }
    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#putInCache(java.io.Serializable, java.lang.Object)
     */
    public void putInCache(Serializable key, Object obj) throws CacheException {
        final String validNamespace = this.getValidNamespace();
        final NamespacedKey namespacedKey = new NamespacedKey(validNamespace, key);
        this.wrappedCache.putInCache(namespacedKey, obj);
    }
    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#removeFromCache(java.io.Serializable)
     */
    public void removeFromCache(Serializable key) throws CacheException {
        final String validNamespace = this.getValidNamespace();
        final NamespacedKey namespacedKey = new NamespacedKey(validNamespace, key);
        this.wrappedCache.removeFromCache(namespacedKey);
    }
    
    
    private String getValidNamespace() {
        if (this.namespace != null) {
            return this.namespace + "_";
        }
        else {
            this.logger.warn("Dynamic namespacing is being used. The namespace property should be set for production use.");
            
            final Throwable throwable = new Throwable();
            throwable.fillInStackTrace();
            final StackTraceElement[] stack = throwable.getStackTrace();
            if (stack.length >= 3) {
                return stack[2].getClassName() + "_";
            }
            else {
                return "";
            }
        }
    }
    
    private static class NamespacedKey implements Serializable {
        private final String namespace; 
        private final Serializable realKey;
        
        public NamespacedKey(final String namespace, final Serializable realKey) {
            this.namespace = namespace;
            this.realKey = realKey;
        }

        /**
         * @see java.lang.Object#equals(Object)
         */
        public boolean equals(final Object object) {
            if (object == this) {
                return true;
            }
            if (!(object instanceof NamespacedKey)) {
                return false;
            }
            NamespacedKey rhs = (NamespacedKey)object;
            return new EqualsBuilder()
                .append(this.namespace, rhs.namespace)
                .append(this.realKey, rhs.realKey)
                .isEquals();
        }
        /**
         * @see java.lang.Object#hashCode()
         */
        public int hashCode() {
            return new HashCodeBuilder(1022327505, -274274227)
                .append(this.namespace)
                .append(this.realKey)
                .toHashCode();
        }
        /**
         * @see java.lang.Object#toString()
         */
        public String toString() {
            return this.namespace + ":" + this.realKey; 
        }
    }
}
