/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.caching;

import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.atomic.AtomicLong;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.springframework.beans.factory.annotation.Required;
import org.springmodules.cache.CacheException;


/**
 * Gathers statistics about a wrapped {@link ISimpleCacheProviderFacade}.
 * 
 * @author Eric Dalquist
 * @version $Revision: 11955 $
 */
public class StatisticsCacheWrapper implements ISimpleCacheProviderFacade, ICacheStatistics {
    private final Map<String, CacheQueryStatistics> queryStats = new HashMap<String, CacheQueryStatistics>();
    private final CacheQueryStatistics totalStats = new CacheQueryStatistics();

    private ISimpleCacheProviderFacade wrappedCache;
    private boolean keepDetails = false;
    

    /**
     * @return the keepDetails
     */
    public boolean isKeepDetails() {
        return this.keepDetails;
    }

    /**
     * @param keepDetails the keepDetails to set
     */
    @Required
    public void setKeepDetails(boolean keepDetails) {
        this.keepDetails = keepDetails;
    }

    /**
     * @return the wrappedCache
     */
    public ISimpleCacheProviderFacade getWrappedCache() {
        return this.wrappedCache;
    }

    /**
     * @param wrappedCache the wrappedCache to set
     */
    @Required
    public void setWrappedCache(ISimpleCacheProviderFacade wrappedCache) {
        this.wrappedCache = wrappedCache;
    }
    
    
    //********** ISimpleCacheProviderFacade methods **********//

    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#putInCache(java.io.Serializable, java.lang.Object)
     */
    public void putInCache(Serializable key, Object obj) throws CacheException {
        if (this.keepDetails) {
            final CacheQueryStatistics queryStats = this.getStatsForQuery(key);
            queryStats.puts.incrementAndGet();
        }
        
        this.totalStats.puts.incrementAndGet();
        
        this.wrappedCache.putInCache(key, obj);
    }
    
    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#getFromCache(java.io.Serializable)
     */
    public Object getFromCache(Serializable key) throws CacheException {
        final Object obj = this.wrappedCache.getFromCache(key);
        
        if (this.keepDetails) {
            final CacheQueryStatistics queryStats = this.getStatsForQuery(key);
            
            //Update all stats
            if (obj != null) {
                this.totalStats.hits.incrementAndGet();
                queryStats.hits.incrementAndGet();
            }
            else {
                this.totalStats.misses.incrementAndGet();
                queryStats.misses.incrementAndGet();
            }
        }
        else {
            //Update overall stats
            if (obj != null) {
                this.totalStats.hits.incrementAndGet();
            }
            else {
                this.totalStats.misses.incrementAndGet();
            }
        }
        
        return obj;
    }
    
    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#cancelCacheUpdate(java.io.Serializable)
     */
    public void cancelCacheUpdate(Serializable key) throws CacheException {
        this.wrappedCache.cancelCacheUpdate(key);
    }

    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#flushCache()
     */
    public void flushCache() {
        this.wrappedCache.flushCache();
    }

    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#isFailQuietlyEnabled()
     */
    public boolean isFailQuietlyEnabled() {
        return this.wrappedCache.isFailQuietlyEnabled();
    }

    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#removeFromCache(java.io.Serializable)
     */
    public void removeFromCache(Serializable key) throws CacheException {
        this.wrappedCache.removeFromCache(key);
    }
    

    /**
     * Gets the Collection of {@link CacheQueryStatistics} objects for all queries that
     * have been made against this cache. 
     * 
     * @return The Collection of {@link CacheQueryStatistics} objects. 
     */
    public Collection<CacheQueryStatistics> getCacheQueryStatistics() {
        return Collections.unmodifiableCollection(this.queryStats.values());
    }
    
    
    //********** ISimpleCacheProviderFacade methods **********//
    
    /**
     * @see org.jasig.portal.caching.ICacheStatistics#getCacheExpirations()
     */
    public long getCacheExpirations() {
        return -1;
    }

    /**
     * @see org.jasig.portal.caching.ICacheStatistics#getCacheHits()
     */
    public long getCacheHits() {
        return this.totalStats.getHits();
    }

    /**
     * @see org.jasig.portal.caching.ICacheStatistics#getCacheMisses()
     */
    public long getCacheMisses() {
        return this.totalStats.getMisses();
    }

    /**
     * @see org.jasig.portal.caching.ICacheStatistics#getCachePuts()
     */
    public long getCachePuts() {
        return this.totalStats.getPuts();
    }

    /**
     * @see org.jasig.portal.caching.ICacheStatistics#getSize()
     */
    public int getSize() {
        return -1;
    }
    
    
    //********** Internal Methods **********//

    /**
     * Generates a String to use for tracking key specific statistics. Only used if
     * <code>keepDetails</code> is set to <code>true</code>.
     */
    protected String getKeyString(Serializable key) {
        if (key == null || key instanceof String) {
            return (String)key;
        }
        else {
            return key.toString();
        }
    }

    private CacheQueryStatistics getStatsForQuery(Serializable key) {
        final String keyString = this.getKeyString(key);
        
        CacheQueryStatistics queryStats;
        synchronized (this.queryStats) {
            queryStats = this.queryStats.get(keyString);
            if (queryStats == null) {
                queryStats = new CacheQueryStatistics();
                this.queryStats.put(keyString, queryStats);
            }
        }
        return queryStats;
    }
    
    
    public static class CacheQueryStatistics {
        private final AtomicLong hits = new AtomicLong();
        private final AtomicLong misses = new AtomicLong();
        private final AtomicLong puts = new AtomicLong();
        
        /**
         * @return Returns the hits.
         */
        public long getHits() {
            return this.hits.get();
        }

        /**
         * @return Returns the misses.
         */
        public long getMisses() {
            return this.misses.get();
        }
        
        /**
         * @return Returns the puts
         */
        public long getPuts() {
            return this.puts.get();
        }

        /**
         * @see java.lang.Object#toString()
         */
        public String toString() {
            return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
                .append("puts", this.puts)
                .append("hits", this.hits)
                .append("misses", this.misses)
                .toString();
        }
    }
}
