/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.collections;

import org.apache.commons.collections.map.LRUMap;

/**
 * An extension of the commons collections LRUMap that also uses
 * creation and idle time for removing Map elements.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class LruExpirationMap extends LRUMap {
    private int timeToLive = 0;
    private int timeToIdle = 0;
    
    private long hitCount = 0;
    private long missCount = 0;
    private long expirationCount = 0;
    
    
    /**
     * @see LRUMap#LRUMap(int)
     */
    public LruExpirationMap(int maxSize) {
        super(maxSize);
    }
    /**
     * @see LRUMap#LRUMap(int, boolean)
     */
    public LruExpirationMap(int maxSize, boolean scanUntilRemovable) {
        super(maxSize, scanUntilRemovable);
    }
    
    
    /**
     * @return Returns the timeToIdle.
     */
    public int getTimeToIdle() {
        return this.timeToIdle;
    }
    /**
     * @param timeToIdle The timeToIdle to set.
     */
    public void setTimeToIdle(int timeToIdle) {
        this.timeToIdle = timeToIdle;
    }

    /**
     * @return Returns the timeToLive.
     */
    public int getTimeToLive() {
        return this.timeToLive;
    }
    /**
     * @param timeToLive The timeToLive to set.
     */
    public void setTimeToLive(int timeToLive) {
        this.timeToLive = timeToLive;
    }
    
    /**
     * @return Returns the expirationCount.
     */
    public long getExpirationCount() {
        return this.expirationCount;
    }
    /**
     * @return Returns the hitCount.
     */
    public long getHitCount() {
        return this.hitCount;
    }
    /**
     * @return Returns the missCount.
     */
    public long getMissCount() {
        return this.missCount;
    }
    
    
    /**
     * @see org.apache.commons.collections.map.AbstractLinkedMap#clear()
     */
    public void clear() {
        this.hitCount = 0;
        this.missCount = 0;
        this.expirationCount = 0;

        super.clear();
    }

    /**
     * @see org.apache.commons.collections.map.AbstractHashedMap#getEntry(java.lang.Object)
     */
    protected HashEntry getEntry(Object key) {
        final TrackingHashEntry entry = (TrackingHashEntry)super.getEntry(key);
        if (entry == null) {
            this.missCount++;
            return null;
        }
        
        final long now = System.currentTimeMillis();
        
        if ((this.timeToLive > 0 && (now - entry.getCreationTime()) / 1000 >= this.timeToLive) ||
            (this.timeToIdle > 0 && (now - entry.getAccessTime()) / 1000 >= this.timeToIdle)) {
            
            this.remove(key);
            this.missCount++;
            this.expirationCount++;
            return null;
        }

        entry.setAccessTime(now);
        this.hitCount++;
        return entry;
    }

    /**
     * @see org.apache.commons.collections.map.AbstractHashedMap#reuseEntry(org.apache.commons.collections.map.AbstractHashedMap.HashEntry, int, int, java.lang.Object, java.lang.Object)
     */
    protected void reuseEntry(HashEntry entry, int hashIndex, int hashCode, Object key, Object value) {
        //Update the time of the entry to make it look new
        final TrackingHashEntry trackingEntry = (TrackingHashEntry)entry;
        trackingEntry.setCreationTime(System.currentTimeMillis());
        trackingEntry.setAccessTime(trackingEntry.getCreationTime());
        
        super.reuseEntry(entry, hashIndex, hashCode, key, value);
    }
    
    /**
     * @see org.apache.commons.collections.map.AbstractLinkedMap#createEntry(org.apache.commons.collections.map.AbstractHashedMap.HashEntry, int, java.lang.Object, java.lang.Object)
     */
    protected HashEntry createEntry(HashEntry next, int hashCode, Object key, Object value) {
        return new TrackingHashEntry(next, hashCode, key, value);
    }

    /**
     * Entry that tracks when it was created and the last time it was
     * accessed.
     */
    protected static class TrackingHashEntry extends LinkEntry {
        private long creationTime;
        private long accessTime;
        
        protected TrackingHashEntry(HashEntry next, int hashCode, Object key, Object value) {
            super(next, hashCode, key, value);
            this.creationTime = System.currentTimeMillis();
            this.accessTime = this.creationTime;
        }

        /**
         * @return Returns the accessTime.
         */
        public long getAccessTime() {
            return this.accessTime;
        }
        /**
         * @param accessTime The accessTime to set.
         */
        public void setAccessTime(long accessTime) {
            this.accessTime = accessTime;
        }

        /**
         * @return Returns the creationTime.
         */
        public long getCreationTime() {
            return this.creationTime;
        }
        /**
         * @param creationTime The creationTime to set.
         */
        public void setCreationTime(long creationTime) {
            this.creationTime = creationTime;
        }
        
    }
}
