/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.context;

import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.core.exception.PortalException;

/**
 * ServletPathContextResolver resolves context name based on the first
 * path element of the HTTP servlet request.
 * <p>
 * For example, the following URL would yield a
 * context name of <i>dummy</i>:<br>
 * <code>http://localhost:8080/uPortal/dummy/render.uP</code>
 * <p>
 * In this case, <i>uPortal</i> is the web application context name
 * and <i>dummy</i> is the portal context name.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class ServletPathContextResolver implements IContextResolver {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(ServletPathContextResolver.class);
    
    public static final String PORTAL_CONTEXT_ATTRIBUTE_NAME="uPortalContextNameAttribute"; 

	private IContextFactory contextFactory;
    private String defaultContext=null;
	
	public IContext resolveContext(HttpServletRequest req) throws PortalException {
		String path = req.getServletPath();
        
        IContext context=null;
        String contextName = null;
        
        int secondSlash=path.indexOf("/",1);
        if(secondSlash!=-1) {
            contextName=path.substring(1, secondSlash);
            context=getContext(contextName, req.getSession(true));
        }
        
        if (logger.isDebugEnabled()) {
            logger.debug("resolveContext(HttpServletRequest) -  : path=" + path + ", contextName=" + contextName+", contextPath="+req.getContextPath()+", pathInfo="+req.getPathInfo()+", pathTranslated="+req.getPathTranslated()+", servletName"+req.getServerName()+", defaultContext="+defaultContext);
        }
        
        if(context==null) {
            if (logger.isDebugEnabled()) {
                logger.debug("resolveContext(HttpServletRequest) - invalid or empty context specified: contextName=" + contextName+", defaulting to "+defaultContext);
            }
            recordContextPathElement(null,req);
            context=getContext(defaultContext, req.getSession(true));
            if(context==null) {
                logger.error("resolveContext(HttpServletRequest) - unable to obtain default context : defaultContext=" + defaultContext);
            }
        } else {
            recordContextPathElement(contextName,req);
        }
        return context;
	}
    
	public Collection getAvailableContexts() {
		return contextFactory.getKnownContextNames();
	}
    
	public IContext getContext(String contextName, HttpSession session) throws PortalException {
        return contextFactory.getContext(contextName);
	}
    
	public IContextFactory getContextFactory() {
		return contextFactory;
	}
    
	public void setContextFactory(IContextFactory contextFactory) {
		this.contextFactory = contextFactory;
	}

    /**
     * Specify default context name. If set to <code>null</code> context resolver will thow an error.
     * @param defaultContext
     */
    public void setDefaultContext(String defaultContext) {
        this.defaultContext = defaultContext;
    }
    
    
    /**
     * Record context path element from the current request.
     * @param req
     */
    private static void recordContextPathElement(String contextPathElement,HttpServletRequest req) {
        req.setAttribute(PORTAL_CONTEXT_ATTRIBUTE_NAME,contextPathElement);
    }
    
    /**
     * Returns context path element for the current request. For instance, if the current context
     * is uP3, the method will return "uP3" unless uP3 context was selected because it is the default
     * context and either no context was specified in the request, or a wrong context was specified.
     * @param req
     * @return
     */
    public static String getContextPathElement(HttpServletRequest req) {
        return (String) req.getAttribute(PORTAL_CONTEXT_ATTRIBUTE_NAME);
    }

}
