/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.context.common;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.portlet.rendering.IPortletWindowManager;
import org.jasig.portal.portlet.rendering.IPortletWindowManagerEventListener;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.rendering.attributes.IElementResolver;
import org.jasig.portal.rendering.attributes.ElementResolverKey;
import org.jasig.portal.rendering.attributes.IRenderingAttributeProvider;
import org.jasig.portal.rendering.attributes.IRenderingAttributeValues;
import org.jasig.portal.rendering.attributes.NoElementResolver;
import org.jasig.portal.rendering.attributes.ElementResolverKey;
import org.xml.sax.Attributes;

/**
 * A base class for rendering attribute providers that inject portlet information.
 * The deriving classes should implement updateValues() and receivePortletWindowManagerEvent() methods. 
 * If attributes have default values, the implementation should also override getDefaultAttributeValueMap() and getDefaultParamValueMap() methods.
 * 
 * Note: implementation assumes that element resolver key will be a {@link org.jasig.portal.rendering.attributes.ElementResolverKey} corresponding to portlet window id. If that's
 * not the case you must derive a child class and override getPortletWindowId() and getElementResolverKey() methods.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public abstract class PortletStateRenderingAttributeProvider implements IRenderingAttributeProvider, IPortletWindowManagerEventListener {
    protected IPortletWindowManager portletWindowManager;
    protected IElementResolver elementResolver=new NoElementResolver();
    protected String providerName;
    protected IRenderingAttributeValues renderingAttributeValues;
    boolean updateValuesOnGetAttributes=false;
    boolean updateValuesonGetCacheKey=false;
    
    
    /**
     * Update the rendering attribute values for a given portlet window id.
     * @param elementKey key of a resolved element
     */
    protected abstract void updateValues(ElementResolverKey elementKey);
    
    public boolean removeParameter(String parameterName) {
        return renderingAttributeValues.removeParameter(parameterName);
    }
    
    public void setElementResolver(IElementResolver elementResolver) {
        this.elementResolver = elementResolver;
    }

    public String getProviderName() {
        return providerName;
    }

    public void setProviderName(String providerName) {
        this.providerName = providerName;
    }

    public void setRenderingAttributeValues(IRenderingAttributeValues renderingAttributeValues) {
        this.renderingAttributeValues = renderingAttributeValues;
    }

    public ICacheKey generateKey() {
        if(updateValuesonGetCacheKey) {
            // retrieve attribute map
            Set<ElementResolverKey> resolverKeys=renderingAttributeValues.getElementResolverKeys();
            for(ElementResolverKey key: resolverKeys) {
                this.updateValues(key);
            }
        }
        return renderingAttributeValues.generateKey();
    }
    
    protected Map getDefaultAttributeValueMap() {
        // no default values
        return null;
    }
    
    protected Map getDefaultParamValueMap() {
        // no default values
        return null;
    }

    public boolean isCacheValid(Serializable validityObject) {
        return renderingAttributeValues.isCacheValid(validityObject);
    }

    public void clearAttributes() {
        // nothing needs to be done here
    }

    public void clearAttributes(String namespaceUri, String localName, String qName, Attributes atts) {
        // nothing needs to be done here
    }

    public void clearParameters() {
        // nothing needs to be done here
    }

    public Map getParameterValues() {
        Map defaultValues=this.getDefaultParamValueMap();
        Map values=renderingAttributeValues.getParameterValues();
        if(defaultValues!=null) {
            if(values!=null) {
                Map initialValues=new HashMap(defaultValues);
                initialValues.putAll(values);
                return initialValues;
            } else {
                return defaultValues;
            }
        } else {
            return values;
        }
    }

    public boolean removeAttribute(String attributeName, String namespaceUri, String localName, String qName, Attributes atts) {
        // nothing needs to be done here        
        return false;
    }

    public void setAttributeValue(String attributeName, String attributeValue, String namespaceUri, String localName, String qName, Attributes atts) {
        // nothing needs to be done here
    }

    public void setParameterValue(String parameterName, String parameterValue) {
        // nothing needs to be done here
    }
    
    /**
     * Obtain portlet window id for a resolved element key
     * @param elementKey key of a resolved portlet element
     * @return portlet window id of a resolved portlet element
     */
    protected PortletWindowId getPortletWindowId(ElementResolverKey elementKey) {
        return new PortletWindowId(elementKey.toString());
    }
    
    /**
     * Obtain element resolver key corresponding to a given portlet window id
     * @param portletWindowId
     * @return
     */
    protected ElementResolverKey getElementResolverKey(PortletWindowId portletWindowId) {
        return new ElementResolverKey(portletWindowId.toString());
    }

    public Map getAttributeValues(String namespaceUri, String localName, String qName, Attributes atts) {
        // obtain portlet window
        ElementResolverKey elementKey=elementResolver.resolveElementKey(namespaceUri,localName,qName,atts);
        if(elementKey!=null) {
            // create portlet window id
            if(updateValuesOnGetAttributes) {
                this.updateValues(elementKey);
            }
            // return values for this window
            Map defaultValueMap=this.getDefaultAttributeValueMap();
            Map values=renderingAttributeValues.getElementAttributeValues(elementKey);
            if(defaultValueMap!=null) {
                if(values!=null) {
                    Map initialMap=new HashMap(defaultValueMap);
                    initialMap.putAll(values);
                    return initialMap;
                } else {
                    return defaultValueMap;
                }
            } else {
                return values;
            }
        } else {
            return null;
        }
    }

    /**
     * Specify if attribute values should be updated (by consulting portlet window manager) whenever attributes for a given
     * portlet window are requested (default <code>false</code>).
     * @param updateValuesOnGetAttributes
     */
    public void setUpdateValuesOnGetAttributes(boolean updateValuesOnGetAttributes) {
        this.updateValuesOnGetAttributes = updateValuesOnGetAttributes;
    }

    /**
     * Specify whether rendering attribute values should be updated (by consulting portlet window manager
     * prior to returning a cache key (default <code>false</code>).
     * @param updateValuesonGetCacheKey
     */
    public void setUpdateValuesonGetCacheKey(boolean updateValuesonGetCacheKey) {
        this.updateValuesonGetCacheKey = updateValuesonGetCacheKey;
    }

    protected IPortletWindowManager getPortletWindowManager() {
        return portletWindowManager;
    }

    public void setPortletWindowManager(IPortletWindowManager portletWindowManager) {
        if(this.portletWindowManager!=portletWindowManager) {
            if(this.portletWindowManager!=null) {
                this.portletWindowManager.removePortletWindowManagerEventListener(this);
            }
            this.portletWindowManager = portletWindowManager;
            portletWindowManager.addPortletWindowManagerEventListener(this);
        }
    }

    protected IRenderingAttributeValues getRenderingAttributeValues() {
        return renderingAttributeValues;
    }
}
