/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.context.common.persistence;

import java.util.Collections;
import java.util.Iterator;
import java.util.Set;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.url.IRequestParameterProcessor;
import org.jasig.portal.url.IWritableHttpServletRequest;
import org.jasig.portal.url.syntax.IUrlBasicSyntaxProvider;
import org.jasig.portal.utils.WeakHashSet;

/**
 * PersistenceControllerImpl is a simple persistence controller that relies on a single request param.
 * Note that the implementation uses {@link org.jasig.portal.utils.WeakHashSet} to keep track of listeners,
 * which means that while one does not have to worry about calling <code>removePersistenceControllerListener</code> method,
 * listeners must be hard-refenced by some other means to prevent them from being gc'ed at once.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class PersistenceControllerImpl implements IRequestParameterProcessor, IPersistenceController {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PersistenceControllerImpl.class);

    public final static String COMMAND_PARAM="command";
    public final static String PERSIST_COMMAND="persist";
    public final static String RESET_COMMAND="reset";
    
    final PersistenceControllerEvent persistEvent=new PersistenceControllerEvent(this,PersistenceControllerEvent.persistPreferencesEventType);
    final PersistenceControllerEvent resetEvent=new PersistenceControllerEvent(this,PersistenceControllerEvent.resetPreferencesEventType);
    
    Set listeners=Collections.synchronizedSet(new WeakHashSet());
    protected String processorPrefix="persistence";
    protected IUrlBasicSyntaxProvider syntaxProvider;

    /* (non-Javadoc)
     * @see org.jasig.portal.url.IUrlParameterProcessor#canEvaluateVariables()
     */
    public boolean canEvaluateVariables() {
        return false;
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.url.IUrlParameterProcessor#getProcessorName()
     */
    public String getProcessorPrefix() {
        return this.processorPrefix;
    }
    public void persist() {
        this.distributeEvent(persistEvent);
    }
    
    public void reset() {
        this.distributeEvent(resetEvent);
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.url.IUrlParameterProcessor#processParameters(org.jasig.portal.url.IWritableHttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res) {
        String commandValue=req.getParameter(syntaxProvider.incorporateProcessorPrefix(processorPrefix,COMMAND_PARAM));
        if(commandValue!=null) {
            if(PERSIST_COMMAND.equals(commandValue)) {
                this.persist();
            } else if(RESET_COMMAND.equals(commandValue)) {
                this.reset();
            } else {
                logger.warn("processParameters() - invalid persistence command passed : String commandValue = " + commandValue, null);
            }
        }
        return true;
    }
    public void addPersistenceControllerListener(IPersistenceControllerEventListener listener) {
        listeners.add(listener);
    }
    public void removePersistenceControllerListener(IPersistenceControllerEventListener listener) {
        listeners.remove(listener);
    }
    
    /**
     * Distribute persistence event
     * @param event - event object
     */
    void distributeEvent(PersistenceControllerEvent event) {
        if (logger.isDebugEnabled()) {
            logger.debug("distributeEvent(PersistenceControllerEvent event = " + event.getEventType().toString() + ") - distributing persistence controller event");
        }

        for(Iterator i=this.listeners.iterator();i.hasNext();) {
            ((IPersistenceControllerEventListener)i.next()).receivePersistenceControllerEvent(event);
        }
    }
    /**
     * @param processorName The processorName to set.
     */
    public void setProcessorPrefix(String processorName) {
        this.processorPrefix = processorName;
    }
    /**
     * @param syntaxProvider The syntaxProvider to set.
     */
    public void setSyntaxProvider(IUrlBasicSyntaxProvider syntaxProvider) {
        this.syntaxProvider = syntaxProvider;
    }
}
