/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.context.common.profile;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.context.common.profile.dao.IProfilePreferenceDao;
import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.security.UserId;
import org.springframework.beans.factory.annotation.Required;

/**
 * A profile selector that records preferred profile in a provided DAO.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 1.0$
 */
public class DaoProfileSelector implements IProfileSelector {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(DaoProfileSelector.class);

    ProfileId defaultProfileId;
    IProfilePreferenceDao profilePreferenceDao;
    IUserController userController;
    boolean persistDefaultProfileChoice = false;

    public IProfile getCurrentProfile(IProfileConfig config) {
        // determine profile id
        ProfileId profileId;
        // attempt to get from DAO
        UserId userId = userController.getUserId();
        if (userId == null) {
            logger.warn("getCurrentProfile(IProfileConfig) - null user id returned by user controller, unable to retrieve profile preference");
            return this.getDefaultProfile(config);
        }
        else {
            profileId = profilePreferenceDao.getPreferredProfile(userId);
            if (profileId == null) {
                return this.getDefaultProfile(config);
            }
            // obtain profile itself
            IProfile currentProfile = config.getProfile(profileId);
            if (currentProfile == null) {
                logger.warn("getCurrentProfile(IProfileConfig) - unable to obtain selected profile from profile config : profileId="
                        + profileId);
                return this.getDefaultProfile(config);
            }
            else {
                return currentProfile;
            }
        }

    }

    private IProfile getDefaultProfile(IProfileConfig config) {
        IProfile defaultProfile = config.getProfile(this.defaultProfileId);
        if (defaultProfile == null) {
            logger.warn("returnDefaultProfile(IProfileConfig) - unable to obtain default profile from profile config : defaultProfileId="
                    + defaultProfileId);
            return null;
        }
        else {
            return defaultProfile;
        }
    }

    public void setCurrentProfile(ProfileId profileId, IProfileConfig config) {
        if (persistDefaultProfileChoice || profileId != defaultProfileId) {
            UserId userId = userController.getUserId();
            if (userId == null) {
                logger.warn("getCurrentProfile(IProfileConfig) - null user id returned by user controller, unable to persist profile preference",
                            null);
            }
            else {
                this.profilePreferenceDao.setPreferredProfile(userId, profileId);
            }
        }
    }

    /**
     * Specify default profile id, to be used when no preferred
     * profile is available from DAO
     * @param defaultProfileId
     */
    public void setDefaultProfileId(String defaultProfileId) {
        this.defaultProfileId = new ProfileId(defaultProfileId);
    }

    /**
     * Specify DAO to be used for retrieving/storing preferred profile ids
     * @param profilePreferenceDao
     */
    @Required
    public void setProfilePreferenceDao(IProfilePreferenceDao profilePreferenceDao) {
        this.profilePreferenceDao = profilePreferenceDao;
    }

    /**
     * Specify whether the choice of the default profile should be persisted.
     * Defaults to <code>false</code>
     * @param persistDefaultProfileChoice <code>true</code> if choice of the default profile
     * by the user should be persisted, <code>false</code> otherwise
     */
    public void setPersistDefaultProfileChoice(boolean persistDefaultProfileChoice) {
        this.persistDefaultProfileChoice = persistDefaultProfileChoice;
    }

    /**
     * Specify user controller
     * @param userController
     */
    @Required
    public void setUserController(IUserController userController) {
        this.userController = userController;
    }
}
