/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.context.common.profile;

import java.util.List;
import java.util.Map;
import java.util.WeakHashMap;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.context.common.user.IUserControllerEventListener;
import org.jasig.portal.context.common.user.UserControllerEvent;
import org.jasig.portal.url.IRequestParameterProcessor;
import org.jasig.portal.url.IWritableHttpServletRequest;
import org.springframework.beans.factory.annotation.Required;

/**
 * Default {@link org.jasig.portal.context.common.profile.IProfileController} implementation.
 * 
 * Note: the implementation is also a {@link org.jasig.portal.url.IRequestParameterProcessor},
 * and if configured in a context will re-determine current profile every rendering cycle. Otherwise
 * it will rely on calls to setCurrentProfile()
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class ProfileControllerImpl implements IRequestParameterProcessor, IProfileController, IUserControllerEventListener {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(ProfileControllerImpl.class);
    
    protected final ProfileControllerEvent profileSwitchedEvent=new ProfileControllerEvent(this,ProfileControllerEvent.profileSwitched);
    private IProfileSelector profileSelector;
    private IProfileConfig profileConfig;
    private IUserController userController;
    
    protected final Map<IProfileControllerEventListener,Boolean> eventListeners = new WeakHashMap<IProfileControllerEventListener,Boolean>();
    
    protected IProfile currentProfile=null;
    
    /* (non-Javadoc)
     * @see org.jasig.portal.url.IUrlParameterProcessor#canEvaluateVariables()
     */
    public boolean canEvaluateVariables() {
        return false;
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.url.IUrlParameterProcessor#processParameters(org.jasig.portal.url.IWritableHttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res) {
        IProfile newProfile=profileSelector.getCurrentProfile(this.profileConfig);
        setCurrentProfile(newProfile);
        
        return true;
    }
    
    private void setCurrentProfile(IProfile newProfile) {
        if(newProfile!=null && newProfile.getId()!=null) {
            if(currentProfile==null || !newProfile.getId().equals(currentProfile.getId())) {
                currentProfile=newProfile;
                distributeEvent(profileSwitchedEvent);
            }
        }
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.context.common.profile.IProfileController#addProfileControllerEventListener(org.jasig.portal.context.common.profile.IProfileControllerEventListener)
     */
    public void addProfileControllerEventListener(IProfileControllerEventListener listener) {
        eventListeners.put(listener,Boolean.TRUE);
    }

    public IProfile getCurrentProfile() {
        if(currentProfile==null) {
            this.setCurrentProfile(profileSelector.getCurrentProfile(this.profileConfig));
        }
        return currentProfile;
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.context.common.profile.IProfileController#removeProfileControllerEventListener(org.jasig.portal.context.common.profile.IProfileControllerEventListener)
     */
    public void removeProfileControllerEventListener(IProfileControllerEventListener listener) {
        eventListeners.remove(listener);
    }
    
    /**
     * Distribute persistance event
     * @param event - event object
     */
    void distributeEvent(ProfileControllerEvent event) {
        if (logger.isDebugEnabled()) {
            logger.debug("distributeEvent(ProfileControllerEvent event = " + event.getEventType().toString() + ") - distributing profile controller event");
        }

        for(IProfileControllerEventListener listener: eventListeners.keySet()) {
            if(listener!=null) {
                listener.receiveProfileControllerEvent(event);
            }
        }
    }
    
    /**
     * Specify initial set of controller event listeners
     * @param listeners
     */
    public void setProfileControllerEventListeners(List<IProfileControllerEventListener> listeners) {
        for(IProfileControllerEventListener listener: listeners) {
            eventListeners.put(listener,Boolean.TRUE);
        }
    }
    
    @Required
    public void setProfileSelector(IProfileSelector profileSelector) {
        this.profileSelector = profileSelector;
    }
    
    @Required
    public void setProfileConfig(IProfileConfig profileConfig) {
        this.profileConfig = profileConfig;
    }
    
    public IProfile getProfile(ProfileId profileId) {
        return this.profileConfig.getProfile(profileId);
    }
    
    public void setCurrentProfile(ProfileId profileId) {
        IProfile profile=profileConfig.getProfile(profileId);
        if(profile!=null) {
            this.setCurrentProfile(profile);
            this.profileSelector.setCurrentProfile(profileId,this.profileConfig);
        } else {
            logger.warn("setCurrentProfile(ProfileId) - attempting to set profile that doesn't exist in the configuration : profileId=" + profileId, null);
        }
    }
    
    public IProfileConfig getProfileConfig() {
        //TODO: would be nice to make this immutable
        return profileConfig;
    }
    public void receiveUserControllerEvent(UserControllerEvent event) {
        // reset current profile upon any user controller event

        if (logger.isDebugEnabled()) {
            logger.debug("receiveUserControllerEvent(UserControllerEvent) - resetting profile upon user controller event");
        }

        IProfile newProfile=profileSelector.getCurrentProfile(this.profileConfig);
        setCurrentProfile(newProfile);
    }
    public void setUserController(IUserController userController) {
        if(this.userController!=null) {
            this.userController.removeUserControllerEventListener(this);
        }
        this.userController=userController;
        this.userController.addUserControllerEventListener(this);
    }
    
}
