/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.context.common.user;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.acegisecurity.Authentication;
import org.acegisecurity.AuthenticationTrustResolver;
import org.acegisecurity.context.SecurityContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.security.IUser;
import org.jasig.portal.security.UserId;
import org.jasig.portal.security.PortalUserNotFoundException;
import org.jasig.portal.security.acegi.TrackingContextHolder;
import org.jasig.portal.security.dao.IUserDao;
import org.jasig.portal.url.IRequestParameterProcessor;
import org.jasig.portal.url.IWritableHttpServletRequest;

/**
 * AcegiSessionUserController is a user controller implementation that acquires security context
 * from the {@link org.jasig.portal.security.acegi.TrackingContextHolder} (which is initialized by 
 * the {@link org.jasig.portal.security.acegi.HttpRequestContextIntegrationFilter}
 * 
 * The controller will return valid information within each request, however controller events 
 * are sent out during processParameters() call. Hence for the events to be active, controller has
 * to be placed into the request parameter processor list (usually in a first position, since
 * other components may depend on it).
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class AcegiSessionUserController extends AbstractUserController implements IUserController, IRequestParameterProcessor {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(AcegiSessionUserController.class);
    
    protected IUserDao userDao;
    protected AuthenticationTrustResolver authenticationTrustResolver;
    
    private String previousSessionId=null;
    private IUser previousUser=null;
    private String previousPrincipal=null;
    private Boolean previousAuthenticationState=null;
    
    /* (non-Javadoc)
     * @see org.jasig.portal.context.common.user.IUserController#getUserId()
     */
    public UserId getUserId() {
        IUser user=this.getUser();
        if(user!=null) {
            return user.getId();
        } else {
            return null;
        }
    }
    public IUser getUser() {
        SecurityContext securityContext=getSecurityContext();
        if(securityContext!=null) {
            Authentication authentication=securityContext.getAuthentication();
            if(authentication!=null) {
                String principal=authentication.getName();
                if(principal!=null) {
                    if(principal.equals(previousPrincipal)) {
                        return previousUser;
                    } else {
                        // TODO: add caching to the userDao
                        if (logger.isDebugEnabled()) {
                            logger.debug("getUser() - acquiring IUser for changed principal : principal=" + principal +", previous principal="+(previousPrincipal==null?"null":previousPrincipal));
                        }
                        return userDao.getUser(principal);
                    }
                } else {
                    logger.error("getUser() - unable to acqiure principal from a valid authentication", null);
                }
            } else {
                logger.error("getUser() - unable to acquire authentication from a valid SecureContext", null);
            }
        }
        return null;
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.context.common.user.IUserController#isUserAuthenticated()
     */
    public boolean isUserAuthenticated() {
        SecurityContext securityContext=getSecurityContext();
        if(securityContext!=null) {
            Authentication authentication=securityContext.getAuthentication();
            if(authentication!=null) {
                return authentication.isAuthenticated();
            } else {
                logger.error("isUserAuthenticated() - unable to acquire authentication from a valid SecureContext", null);
            }
        }
        return false;
    }
    
    /**
     * Acquire ACEGI security context for a particular session
     * @param session
     * @return ACEGI secutiry context
     */
    protected SecurityContext getSecurityContext(HttpSession session) {
    	SecurityContext securityContext=getSecurityContext();
        if(securityContext==null) {
            logger.warn("getSecureContext(HttpSession) - unable to obtain ACEGI context : sessionId=" + (session==null?"null":session.getId()) , null);
        }
        return securityContext;
    }
    
    /**
     * Acquire security context from {@link org.jasig.portal.security.acegi.TrackingContextHolder}
     * @return
     */
    protected SecurityContext getSecurityContext() {
    	return TrackingContextHolder.getContext();
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.url.IUrlParameterProcessor#canEvaluateVariables()
     */
    public boolean canEvaluateVariables() {
        return false;
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.url.IUrlParameterProcessor#processParameters(org.jasig.portal.url.IWritableHttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res) {
        HttpSession session=req.getSession(false);
        String sessionId="null";
        if(session!=null) {
            sessionId=session.getId();
            if(this.previousSessionId!=null) {
                // check that session id remained unchanged
                if(!this.previousSessionId.equals(sessionId)) {
                    // session id changed, this shouldn't happen
                    final IllegalStateException ise = new IllegalStateException("user controller instance is being used in conjunction with multiple sessions! : sessionId = " + sessionId+", previous sessionId="+previousSessionId);
                    logger.error(ise.getMessage(), ise);
                    throw ise;
                }
            } else {
                // record current session id
                this.previousSessionId=sessionId;
            }
        }
        
        SecurityContext securityContext=getSecurityContext(session);
        if(securityContext!=null) {
            Authentication authentication=securityContext.getAuthentication();
            //  see if the principal has changed
            String principal=authentication.getName();
            if(principal!=null) {
                if(!principal.equals(previousPrincipal)) {
                    // update authentication
                    this.previousPrincipal=principal;
                    this.previousAuthenticationState=new Boolean(authentication.isAuthenticated());
                    // obtain user id
                    IUser user=null;
                    try {
                        user=userDao.getUser(principal);
                        
                    } catch (Exception e) {
                        logger.error("unable to obtain portal user information : principal=" + principal, e);
                        throw new PortalUserNotFoundException(principal,e);
                    }
                    
                    // distribute user change event
                    if(user!=null) {
                        // user-changed event is not distributed upon the initialization
                        if(previousUser!=null) {
                            this.previousUser=user;
                            distributeUserChangedEvent(previousUser,previousAuthenticationState,req);
                        } else {
                            this.previousUser=user;
                        }
                        
                        if (logger.isInfoEnabled()) {
                            logger.info("user logged in : sessionId=" + (session==null?"null":session.getId()) + ", principal=" + principal + ", userId=" + previousUser.getId());
                        }
                    } else {
                        throw new PortalUserNotFoundException(principal);
                    }
                } else if(this.previousAuthenticationState.booleanValue()!=authentication.isAuthenticated()) {
                    this.previousAuthenticationState=new Boolean(authentication.isAuthenticated());
                    distributeAuthenticationChangedEvent(req);
                }
            } else {
                logger.error("unable to validate authentication because ACEGI authentication principal is NULL for session : sessionId=" + sessionId);
            }
        } else {
            if (logger.isDebugEnabled()) {
                logger.debug("successfully processed ACEGI authentication for session : sessionId=" + sessionId);
            }
        }
        
        return true;
    }

  
    public void setUserDao(IUserDao userDao) {
        this.userDao = userDao;
    }
    public void setAuthenticationTrustResolver(AuthenticationTrustResolver authenticationTrustResolver) {
        this.authenticationTrustResolver = authenticationTrustResolver;
    }
    public boolean isAnonymous() {
        SecurityContext securityContext=getSecurityContext();
        if(securityContext!=null) {
            Authentication authentication=securityContext.getAuthentication();
            if(authentication!=null) {
                return authenticationTrustResolver.isAnonymous(authentication);
            } else {
                final IllegalStateException ise = new IllegalStateException("unable to acquire authentication from a valid SecureContext");
                logger.error(ise.getMessage(), ise);
                throw ise; 
            }
        } else {
            throw new IllegalStateException("SecureContext is null");
        }
    }
}
