/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.context.uportal2;

import java.util.HashMap;
import java.util.Map;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.layout.uportal2.IUserLayoutChannelDescription;
import org.jasig.portal.layout.uportal2.IUserLayoutManager;
import org.jasig.portal.layout.uportal2.IUserLayoutNodeDescription;
import org.jasig.portal.portlet.rendering.IPortletWindowManager;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.rendering.attributes.IRenderingAttributeProvider;
import org.jasig.portal.url.IRequestParameterProcessor;
import org.jasig.portal.url.IWritableHttpServletRequest;


/**
 * A processor that watches for URL parameters from the uP2 style sheets that change modes and
 * states on portlets.<br>
 * <br>
 * Maps:<br>
 * uP_edit_target to PortletMode.EDIT<br>
 * uP_help_target to PortletMode.HELP<br>
 * uP_about_target to PortletMode.VIEW<br>
 * minimized to WindowState.MINIMIZED<br>
 * maximized to WindowState.MAXIMIZED<br>
 * <br>
 * The processor supports multiple targets and operations in a single request although the default
 * uP2 style sheets don't provide a way to have multiple targets for these requests.
 * 
 * @author Allen Haws <joeshmoe@unicon.net>
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision: 11955 $
 */
public class LegacyChannelStateParameterProcessor implements IRequestParameterProcessor {
    private static final String LEGACY_EDIT_COMMAND = "uP_edit_target";
    private static final String LEGACY_HELP_COMMAND = "uP_help_target";
    private static final String LEGACY_ABOUT_COMMAND = "uP_about_target";

    private static final String UP_THEME_CHANNEL_ATTRIBUTE = "uP_tcattr";
    private static final String MINIMIZED_CHANNEL_ATTRIBUTE = "minimized";
    private static final String MINIMIZED_CHANNEL_ID = "minimized_channelId";
    private static final String MINIMIZED_VALUE_SUFIX = "_value";
    private static final String MINIMIZED_VALUE_PREFIX = "minimized_";
    
    private static final String ROOT_NODE_LAYOUT_PARAMETER = StructureTransformPortletStateRenderingAttributeProvider.ROOT_NODE_PARAMETER;
    private static final String ROOT_NODE_KEY = "root";
    private static final String ROOT_NODE_ATTRIBUTE = "uP_root";


    protected final Log logger = LogFactory.getLog(this.getClass());
    
	private IPortletWindowManager portletWindowManager = null;
	private IUserLayoutManager userLayoutManager = null;
    private IRenderingAttributeProvider structureRenderingAttributeProvider = null;

	/** {@inheritDoc} */
	public boolean canEvaluateVariables() {
		return false;
	}
    
	/** {@inheritDoc} */
	public boolean processParameters(IWritableHttpServletRequest pReq, HttpServletResponse pRes) {
        final Map<String, PortletChange> changes = new HashMap<String, PortletChange>();
        
        //Check for PortletMode modifications by the chrome
        this.parsePortletModeChanges(pReq, LEGACY_EDIT_COMMAND, PortletMode.EDIT, changes);
        this.parsePortletModeChanges(pReq, LEGACY_HELP_COMMAND, PortletMode.HELP, changes);
        this.parsePortletModeChanges(pReq, LEGACY_ABOUT_COMMAND, PortletMode.VIEW, changes);
        
        //Check if the chrome modified the WindowState for any portlet
        this.parseMinimizedToggle(pReq, changes);
        this.parseMaximizedToggle(pReq, changes);
		
        //Apply the changes to the portlets based on layout nodeIds
		for (final PortletChange change : changes.values()) {
            final String nodeId = change.getNodeId();
            //TODO nodeID was root when using Chrome going from Max to Norm
			final IUserLayoutNodeDescription node;
            try {
                node = this.userLayoutManager.getNode(nodeId);
            }
            catch (PortalException pe) {
                final IllegalStateException ise = new IllegalStateException("Could not retrieve 'IUserLayoutNodeDescription' for nodeId='" + nodeId + "'", pe);
                this.logger.error(ise.getMessage(), ise);
                throw ise;
            }
			
            final PortletWindowId portletId;
            if (node instanceof IUserLayoutChannelDescription) {
				portletId = ((IUserLayoutChannelDescription)node).getPortletWindowId();

                final PortletMode portletMode = change.getNewMode();
                if (portletMode != null) {
                    this.portletWindowManager.setPortletMode(portletId, portletMode);
                }
                
                final WindowState windowState = change.getNewState();
                if (windowState != null) {
                    this.portletWindowManager.setWindowState(portletId, windowState);
                }
			}
            else {
                final ClassCastException cce = new ClassCastException("'IUserLayoutNodeDescription' for nodeId='" + nodeId + "' is of type '" + node.getClass() + "' not '" + IUserLayoutChannelDescription.class + "' as expected");
                this.logger.error(cce.getMessage(), cce);
                throw cce;
            }
		}

		return true;
	}
    
    /**
     * For the specified parameter, gets the array of String values and assumes
     * they are portlet node ids. Adds each to the changes map with the specified
     * target PortletMode
     */
    protected void parsePortletModeChanges(HttpServletRequest pReq, String parameter, PortletMode targetMode, Map<String, PortletChange> changesMap) {
        final String[] nodeIds = pReq.getParameterValues(parameter);
        if (nodeIds == null) {
            return;
        }
        
        for (final String nodeId : nodeIds) {
            final PortletChange change = this.getPortletChange(changesMap, nodeId);
            change.setNewMode(targetMode);
        }
        
    }

    /**
     * Parses attributes related to changing a portlet's WindowState between Normal and Minimized
     */
    protected void parseMinimizedToggle(HttpServletRequest pReq, Map<String, PortletChange> changes) {
        final String channelAttributeParameter = pReq.getParameter(UP_THEME_CHANNEL_ATTRIBUTE);
        if (channelAttributeParameter != null && channelAttributeParameter.equals(MINIMIZED_CHANNEL_ATTRIBUTE)) {
            
            final String[] minimizedNodeIds = pReq.getParameterValues(MINIMIZED_CHANNEL_ID);
            if (minimizedNodeIds != null) {
                
                for (final String nodeId : minimizedNodeIds) {
                    final PortletChange change = this.getPortletChange(changes, nodeId);
                    
                    final String minimizedParam = pReq.getParameter(MINIMIZED_VALUE_PREFIX + nodeId + MINIMIZED_VALUE_SUFIX);
                    final boolean minimized = Boolean.valueOf(minimizedParam);
                    
                    if (minimized) {
                        change.setNewState(WindowState.MINIMIZED);
                    }
                    else {
                        change.setNewState(WindowState.NORMAL);
                    }
                }
            }
        }
    }

    /**
     * Parses attributes related to changing a portlet's WindowState between Normal and Maximized
     */
    protected void parseMaximizedToggle(HttpServletRequest pReq, Map<String, PortletChange> changes) {
        final String rootNodeId = pReq.getParameter(ROOT_NODE_ATTRIBUTE);
        if (rootNodeId != null) {
            if (ROOT_NODE_KEY.equals(rootNodeId)) {
                final Map structureParameters = this.structureRenderingAttributeProvider.getParameterValues();
                final String windowId = (String)structureParameters.get(ROOT_NODE_LAYOUT_PARAMETER);
                if(windowId!=null && !ROOT_NODE_KEY.equals(windowId)) {
                    PortletWindowId wId=new PortletWindowId(windowId);
                    portletWindowManager.setWindowState(wId,WindowState.NORMAL);
                }
            }
            else {
                final PortletChange change = this.getPortletChange(changes, rootNodeId);
                change.setNewState(WindowState.MAXIMIZED);
            }
        }
    }

    /**
     * Gets a PortletChange bean from the changes Map for a specified nodeId,
     * creating one and storing it in the Map if it did not yet exist.
     */
    protected PortletChange getPortletChange(Map<String, PortletChange> changesMap, String nodeId) {
        PortletChange change = changesMap.get(nodeId);
        if (change == null) {
            change = new PortletChange(nodeId);
            changesMap.put(change.getNodeId(), change);
        }
        return change;
    }

    /**
     * @return the portletWindowManager
     */
    public IPortletWindowManager getPortletWindowManager() {
        return this.portletWindowManager;
    }

    /**
     * @param portletWindowManager the portletWindowManager to set
     */
    public void setPortletWindowManager(IPortletWindowManager portletWindowManager) {
        this.portletWindowManager = portletWindowManager;
    }

    /**
     * @return the structureRenderingAttributeProvider
     */
    public IRenderingAttributeProvider getStructureRenderingAttributeProvider() {
        return this.structureRenderingAttributeProvider;
    }

    /**
     * @param structureRenderingAttributeProvider the structureRenderingAttributeProvider to set
     */
    public void setStructureRenderingAttributeProvider(IRenderingAttributeProvider structureRenderingAttributeProvider) {
        this.structureRenderingAttributeProvider = structureRenderingAttributeProvider;
    }

    /**
     * @return the userLayoutManager
     */
    public IUserLayoutManager getUserLayoutManager() {
        return this.userLayoutManager;
    }

    /**
     * @param userLayoutManager the userLayoutManager to set
     */
    public void setUserLayoutManager(IUserLayoutManager userLayoutManager) {
        this.userLayoutManager = userLayoutManager;
    }
    
    

    /**
     * Internal class to track changes made to Portlets by request parameters
     */
    private class PortletChange {
        private final String nodeId;
        private PortletMode newMode;
        private WindowState newState;
        
        public PortletChange(String nodeId) {
            this.nodeId = nodeId;
        }

        /**
         * @return the newMode
         */
        public PortletMode getNewMode() {
            return this.newMode;
        }
        /**
         * @param newMode the newMode to set
         */
        public void setNewMode(PortletMode newMode) {
            this.newMode = newMode;
        }

        /**
         * @return the newState
         */
        public WindowState getNewState() {
            return this.newState;
        }
        /**
         * @param newState the newState to set
         */
        public void setNewState(WindowState newState) {
            this.newState = newState;
        }

        /**
         * @return the nodeId
         */
        public String getNodeId() {
            return this.nodeId;
        }
    }
}
