/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.context.uportal2;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.portlet.WindowState;

import org.jasig.portal.context.common.PortletStateRenderingAttributeProvider;
import org.jasig.portal.portlet.rendering.PortletWindowManagerEvent;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.rendering.attributes.ElementResolverKey;

/**
 * Provides rendering attributes describing portlet state information necessary for the uP2 structure transform.
 * Specifically, keeps track of <code>focusedPortletWindowId</code> param based on the portlet that is currently in the MAXIMIZED
 * WindowState. The provider ensures that only one portlet is in the MAXIMIZED state (by setting previously maximized portlet
 * to a normal window state).
 * 
 * Note: Original uP2 stylesheets use a different param (userLayoutRoot) which carries layout node id, and have to be patched to
 * resolve userLayoutRoot from focusedPortletWindowId param. 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class StructureTransformPortletStateRenderingAttributeProvider extends PortletStateRenderingAttributeProvider {
    static final String ROOT_NODE_PARAMETER = "focusedPortletWindowId";
    static final String DEFAULT_ROOT_NODE="root";
    PortletWindowId maximizedPortletWindow=null;
    Map defaultParamValuesMap;
    
    public StructureTransformPortletStateRenderingAttributeProvider() {
        super();
        Map defaults=new HashMap();
        defaults.put(ROOT_NODE_PARAMETER,DEFAULT_ROOT_NODE);
        this.defaultParamValuesMap=Collections.unmodifiableMap(defaults);
    }

    @Override
    protected void updateValues(ElementResolverKey elementKey) {
        // nothing to do here, maximized portlet has to be determined from events
    }

    public void receivePortletWindowManagerEvent(PortletWindowManagerEvent event) {
        if(PortletWindowManagerEvent.windowStateChangedEventType.equals(event.getEventType())) {
            PortletWindowId windowId=event.getWindowId();
            if(windowId!=null) {
                boolean isSwitchingToMaximized=WindowState.MAXIMIZED.equals(event.getNewWindowState());
                if(maximizedPortletWindow!=null && maximizedPortletWindow.equals(event.getWindowId()) && !isSwitchingToMaximized) {
                    // the window state of a currently focused portlet window has changed to a non-maximized state
                    maximizedPortletWindow=null;
                    renderingAttributeValues.setParameterValue(ROOT_NODE_PARAMETER,DEFAULT_ROOT_NODE);
                }
                
                if(isSwitchingToMaximized) {
                    PortletWindowId oldWindowId=maximizedPortletWindow;
                    maximizedPortletWindow=windowId;
                    renderingAttributeValues.setParameterValue(ROOT_NODE_PARAMETER,windowId.toString());
                    // demote old window to normal state
                    if(oldWindowId!=null) {
                        this.portletWindowManager.setWindowState(oldWindowId,WindowState.NORMAL);
                    }
                }
            }
        }
    }

    /**
     * Allow to set maximized set maximized state by setting root parameter
     */
    @Override
    public void setParameterValue(String parameterName, String parameterValue) {
        if(ROOT_NODE_PARAMETER.equals(parameterName)) {
            if(parameterValue!=null) {
                if(DEFAULT_ROOT_NODE.equals(parameterValue)) {
                    if(maximizedPortletWindow!=null) {
                        PortletWindowId portletWindowId=new PortletWindowId(parameterValue);
                        this.portletWindowManager.setWindowState(portletWindowId,WindowState.NORMAL);
                    }
                } else {
                    PortletWindowId portletWindowId=new PortletWindowId(parameterValue);
                    this.portletWindowManager.setWindowState(portletWindowId,WindowState.MAXIMIZED);
                }
            }
        }
    }

    @Override
    protected Map getDefaultParamValueMap() {
        return this.defaultParamValuesMap;
    }
}
