/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.core;

import java.util.Collections;
import java.util.Iterator;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import org.apache.commons.lang.Validate;

/**
 * Creates an ObjectId from a composite of any number of ObjectIds. The component
 * ids are tracked via their natural ordering to ensure maximum compatability.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class CompositeObjectId extends ObjectId<CompositeObjectId> {
    private final SortedSet<ObjectId> componentIds;
    
    public CompositeObjectId(final Set<ObjectId> componentIds) {
        Validate.notNull(componentIds, "component id Set may not be null or empty");
        if (componentIds.contains(null)) {
            throw new IllegalArgumentException("Null component ObjectIds are not allowed");
        }
        
        final SortedSet<ObjectId> workingIdSet = new TreeSet<ObjectId>(componentIds);
        this.componentIds = Collections.unmodifiableSortedSet(workingIdSet);
    }
    
    public CompositeObjectId(ObjectId... componentIds) {
        Validate.notNull(componentIds, "component id array may not be null or empty");
        
        final SortedSet<ObjectId> workingIdSet = new TreeSet<ObjectId>();
        for (final ObjectId id : componentIds) {
            if (id == null) {
                throw new IllegalArgumentException("Null component ObjectIds are not allowed");
            }
            
            workingIdSet.add(id);
        }
        
        this.componentIds = Collections.unmodifiableSortedSet(workingIdSet);
    }
    
    /**
     * Creates a String composed of the String representations of the component ObjectIds
     * seperated by the . character.
     * 
     * @see org.jasig.portal.core.ObjectId#toString()
     */
    public String toString() {
        final StringBuilder buf = new StringBuilder();
        
        for (final Iterator<ObjectId> idItr = this.componentIds.iterator(); idItr.hasNext();) {
            final ObjectId id = idItr.next();
            buf.append(id);
            
            if (idItr.hasNext()) {
                buf.append(".");
            }
        }
        
        return buf.toString();
    }

    /**
     * If o is null {@link Integer#MIN_VALUE} is returned.
     * <br>
     * The number of components for each CompositeObjectId is then compared, if not equal
     * the difference is returned.
     * <br>
     * The component ObjectIds are then compared to each other, in there natural ordering
     * using {@link ObjectId#compareTo(ObjectId)} on each pair. The first non-zero compare
     * value is returned.
     * <br>
     * If all above tests result in equality 0 is returned.
     * 
     * @see org.jasig.portal.core.ObjectId#compareTo(org.jasig.portal.core.ObjectId)
     */
    @SuppressWarnings("unchecked")
    @Override
    public int compareTo(CompositeObjectId o) {
        if (o == null) {
            return Integer.MIN_VALUE;
        }
        if (this == o) {
            return 0;
        }
        
        final int sizeDiff = o.componentIds.size() - this.componentIds.size();
        if (sizeDiff != 0) {
            return sizeDiff;
        }
        
        final Iterator<ObjectId> thisItr = this.componentIds.iterator();
        final Iterator<ObjectId> oItr = o.componentIds.iterator();
        
        while (thisItr.hasNext()) {
            final ObjectId oid1 = thisItr.next();
            final ObjectId oid2 = oItr.next();
            
            final int diff = oid1.compareTo(oid2);
            if (diff != 0) {
                return diff;
            }
        }
        
        return 0;
    }

    /**
     * Calculates the sum of the hashcodes of the component ObjectIds
     * 
     * @see org.jasig.portal.core.ObjectId#hashCode()
     */
    @Override
    public int hashCode() {
        return this.componentIds.hashCode();
    }
}
