/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.core;

import java.io.Serializable;


/**
 * Base class for strongly typed object identifiers. Sub-classes must provide a {@link #toString()} implementation.
 * <br>
 * <br>
 * Default implementations are provided for the following methods:
 * <table>
 *  <tr>
 *      <td>{@link #compareTo(ObjectId)}</td>
 *      <td>Calls {@link #toString()} on this object and the object being compared and {@link String#compareTo(String)} is used.</td>
 *  </tr>
 *  <tr>
 *      <td>{@link #equals(Object)}</td>
 *      <td>Calls {@link #toString()} on this object and the object being compared and {@link String#equals(Object)} is used.</td>
 *  </tr>
 *  <tr>
 *      <td>{@link #hashCode()}</td>
 *      <td>Calls {@link #toString()} on this object and {@link String#hashCode()} is used.</td>
 *  </tr>
 * </table>
 * If a sub-class overrides any of these methods the new behavior should be very well
 * documented and completely covered by unit tests.
 * 
 * @version $Revision: 11942 $
 */
public abstract class ObjectId<T extends ObjectId> implements Serializable, Comparable<T> {

    /**
     * Compares this ObjectId with another ObjectId. Converts both ObjectIds to
     * Strings using {@link #toString()} and uses {@link String#compareTo(String)}.
     * If the other ObjectId is null {@link Integer#MIN_VALUE} is returned.
     * 
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(T o) {
        if (o == null) {
            return Integer.MIN_VALUE;
        }
        if (this == o) {
            return 0;
        }
        
        return this.toString().compareTo(o.toString());
    }

    /**
     * Checks if this ObjectId is equal to another Object. If the other object
     * is assignable to the same Class as the type of this class {@link #compareTo(ObjectId)}
     * is used and if it returns 0 both classes are deemed equal.
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @SuppressWarnings("unchecked")
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        final Class<? extends ObjectId> thisClass = this.getClass();
        final Class<? extends Object> objClass = obj.getClass();
        if (!(thisClass.isAssignableFrom(objClass) || objClass.isAssignableFrom(thisClass))) {
            return false;
        }
        
        return this.compareTo((T)obj) == 0;
    }

    /**
     * Converts the ObjectId to a String using {@link #toString()} and uses
     * {@link String#hashCode()}.
     * 
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        return this.toString().hashCode();
    }

    /**
     * Sub-classes must provide an implementation that returns a String version
     * of the ObjectId. This may <b>never</b> return null.
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public abstract String toString();
    
}
