/* Copyright 2001-2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.event;

import java.util.EventObject;


/**
 * Events which may be arguments to the IChannel receiveEvent() method.
 * @author <a href="mailto:pkharchenko@interactivebusiness.com">Peter Kharchenko</a>
 * @author <a href="mailto:mivanov@inicon.net>Michael Ivanov</a>
 * @version $Revision: 11911 $
 */
public class PortalEvent extends EventObject {

    /**
     * String representation of the event.
     */
    private final String eventType;
    
  
    // framework-generated events
    
    public static final String SESSION_DONE_EVENT = "sessionDoneEvent";
    public static final String UNSUBSCRIBE_EVENT = "unsubscribeEvent";

    // layout-generated events
    public static final String EDIT_BUTTON_EVENT = "editButtonEvent";
    public static final String HELP_BUTTON_EVENT = "helpButtonEvent";
    public static final String ABOUT_BUTTON_EVENT = "aboutButtonEvent";
    public static final String DETACH_BUTTON_EVENT = "detachButtonEvent";
    public static final String MINIMIZE_WINDOW_EVENT = "minimizeWindowEvent";
    public static final String MAXIMIZE_WINDOW_EVENT = "maximizeWindowEvent";
    
    
  /**
   * The framework-generated event that is broadcast to
   * channels that were used for a user session which is now ending.
   * Typical usage is to trigger state cleanup in channels that are not
   * user-session-scoped.
   */
  public static final PortalEvent SESSION_DONE = 
      new PortalEvent(SESSION_DONE_EVENT,PortalEventSource.FRAMEWORK);

  /**
   * The framework-generated event that is sent to
   * a channel when the user unsubscribes from that channel.
   */
  public static final PortalEvent UNSUBSCRIBE = 
      new PortalEvent(UNSUBSCRIBE_EVENT,PortalEventSource.FRAMEWORK);

  /**
   * The layout-generated event that is sent to
   * a channel when the user actuates its edit control.
   */
  public static final PortalEvent EDIT_BUTTON = 
      new PortalEvent(EDIT_BUTTON_EVENT,PortalEventSource.LAYOUT);
  
  /**
   * The layout-generated event that is sent to
   * a channel when the user actuates its help control.
   */
  public static final PortalEvent HELP_BUTTON = 
      new PortalEvent(HELP_BUTTON_EVENT,PortalEventSource.LAYOUT);
  
  /**
   * The layout-generated event that is sent to
   * a channel when the user actuates its about control.
   */
  public static final PortalEvent ABOUT_BUTTON = 
      new PortalEvent(ABOUT_BUTTON_EVENT,PortalEventSource.LAYOUT);
  
  /**
   * The layout-generated event that is sent to
   * a channel when the user actuates its detach control.
   */
  public static final PortalEvent DETACH_BUTTON = 
      new PortalEvent(DETACH_BUTTON_EVENT,PortalEventSource.LAYOUT);
  
  /**
   * The layout-generated event that is sent to
   * a channel when the user actuates its minimize control.
   */
  public static final PortalEvent MINIMIZE_WINDOW = 
      new PortalEvent(MINIMIZE_WINDOW_EVENT,PortalEventSource.LAYOUT);
  
  /**
   * The layout-generated event that is sent to
   * a channel when the user actuates its maximize control.
   */
  public static final PortalEvent MAXIMIZE_WINDOW = 
      new PortalEvent(MAXIMIZE_WINDOW_EVENT,PortalEventSource.LAYOUT);
  
  /**
   * Construct a PortalEvent instance from parameters.
   * @param eventType - Type of event
   * @param source - source type of event
   */
  private PortalEvent(String eventType, Object source) {
      super(source);
      this.eventType = eventType;
  }

  
  /**
   * Get a String representing this event.
   * @return a String representing this event.
   */
  public String getEventType() {
    return eventType;
  }
 
  /**
   * Two PortalEvents are equal if they have the same eventNumber.
 * @param other - another object
 * @return true if other is a PortalEvent with the same eventNumber,
 * false otherwise
   */
  public boolean equals(Object other) {
    if ( other == null || !(other instanceof PortalEvent) )
      return false;
    PortalEvent otherEvent = (PortalEvent) other;
    return getEventType().equals(otherEvent.getEventType());
  }
  
  public String toString() {
    return eventType;
  }
  
  public int hashCode() {
    return eventType.hashCode();
  }
  
}