/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/


package  org.jasig.portal.groups.services;

import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.gap.services.caching.CachingException;
import org.jasig.gap.services.caching.IEntityCachingService;
import org.jasig.gap.services.entitytypes.IEntityIdentifier;


/**
 * A Service to allow the querying and storing of properties relating
 * to portal entities.  Configured using /properties/EntityPropertyRegistry.xml
 *
 * see dtds/EntityPropertyRegistry.dtd for configuration file grammar
 *
 * @author Alex Vigdor av317@columbia.edu
 * @version $Revision: 11911 $
 *
 * @author Don Fracapane df7@columbia.edu
 * Removed caching from this class and delegated it to the finder classes. Each
 * finder can choose the method of caching if caching is appropriate.
 */
public class EntityPropertyRegistry {
    
    private static final Log log = LogFactory.getLog(EntityPropertyRegistry.class);
    
    private Class propsType;

    // IoC injections
    private IEntityPropertyStore entityPropertyStore;
    private Map<String, IEntityPropertyFinder> finders;
    private IEntityCachingService entityCachingService;
    
    public EntityPropertyRegistry() {
        propsType = EntityProperties.class;
    }
    
    public void setFinders(Map<String, IEntityPropertyFinder> finders) {
        this.finders = finders;
    }

    public String[] getPropertyNames(IEntityIdentifier entityID) {
        return  getProperties(entityID).getPropertyNames();
    }

    public String getProperty(IEntityIdentifier entityID, String name) {
        return getProperties(entityID).getProperty(name);
    }

    public void storeProperty(IEntityIdentifier entityID, String name, String value) {
        entityPropertyStore.storeProperty(entityID, name, value);
    }

    public void removeProperty (IEntityIdentifier entityID, String name) {
        entityPropertyStore.removeProperty(entityID, name);
    }

    protected String getPropsKey(IEntityIdentifier entityID) {
       return entityID.getType().getName() + "." + entityID.getKey(); 
    }

   protected EntityProperties getProperties(IEntityIdentifier entityID) {
      EntityProperties ep = new EntityProperties(getPropsKey(entityID));
      IEntityPropertyFinder potentialFinder;
      IEntityPropertyFinder finder = null;
      
      for (String finderType : finders.keySet()) {
         finder = null;
         potentialFinder = finders.get(finderType);
         if ( potentialFinder instanceof IEntityPropertyStore ) {
            finder = entityPropertyStore;
         } else {
            if ( potentialFinder != null && (finderType.equals("*") || finderType.equals(entityID.getType().getName())))  {
               finder = potentialFinder;
            }
         }
         if ( finder != null ) {
            String[] names = finder.getPropertyNames(entityID);
            for (int j = 0; j < names.length; j++) {
               ep.setProperty(names[j], finder.getProperty(entityID,names[j]));
            }
         }
      }
      return  ep;
   }

   public void clearCache(IEntityIdentifier entityID) {
      try {
    	  entityCachingService.remove(propsType, getPropsKey(entityID));
      } catch (CachingException e) {
         log.error("Error clearing cache for entity ID [" + entityID + "]", e);
      }
   }

   public void addToCache(EntityProperties ep) {
      try {
    	  entityCachingService.add(ep);
      } catch (CachingException e) {
         log.error("Error adding entity properties [" + ep + "] to the cache", e);
      }
   }

   public EntityProperties getCachedProperties(IEntityIdentifier entityID) {
      EntityProperties ep = null;
      try {
         ep = (EntityProperties) entityCachingService.get(propsType,entityID.getKey());
      } catch (CachingException e) {
         log.error("Error getting cached properties for entity [" + entityID + "]", e);
      }
      return ep;
   }
   
    /**
     * @param entityPropertyStore The entityPropertyStore to set.
     */
    public void setEntityPropertyStore(IEntityPropertyStore entityPropertyStore) {
        this.entityPropertyStore = entityPropertyStore;
    }

    public void setEntityCachingService(IEntityCachingService entityCachingService) {
        this.entityCachingService = entityCachingService;
    }
}



