/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.i18n;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Set;

/**
 * Locale resolution provider that finds the best matched locale by matching 
 * the exact Locale, the language and country or the language.  If there is 
 * not an exact match of the locale, it defaults to finding a locale that
 * matches the language and country.  When it does not find a match for 
 * language and country, it then finds the first locale that matches the 
 * language.  The order of the preference is important here as the search 
 * order.  It cycles through the preference order looking for matches.  When
 * it finds something that matches, it returns.  If for some reason there are
 * no matches at all null is returned.
 * 
 * @author David Young, dyoung@unicon.net
 * @version $Revision: 11911 $
 */
public class MatchLocaleResolutionProvider implements ILocaleResolutionProvider {

	/**
     * The resolution logic loops in
     * <pre>O(sizeof(unique Locales in preferences) * sizeof(availableLocales))</pre>
     * time.
     * 
	 * @see org.jasig.portal.i18n.ILocaleResolutionProvider#resolveLocale(java.util.List, java.util.Set)
	 */
	public Locale resolveLocale(List preferences, Set availableLocales) {
        if (preferences == null) {
            throw new IllegalArgumentException("preferences may not be null");
        }
        if (preferences.size() == 0) {
            throw new IllegalArgumentException("preferences must contain at least one Locale");
        }
        if (availableLocales == null) {
            throw new IllegalArgumentException("availableLocales may not be null");
        }
        if (availableLocales.size() == 0) {
            throw new IllegalArgumentException("availableLocales must contain at least one Locale");
        }
        
        final Set checkedLocales = new HashSet();
        Locale matchesLangCountry = null;
        Locale matchesLang = null;
        
        //Loop through the prefered locales in order of most to least prefered.
        for (final Iterator prefLocaleItr = preferences.iterator(); prefLocaleItr.hasNext();) {
            final Locale prefLocale = (Locale)prefLocaleItr.next();
            
            //Keep track of locales that have already been tested since dupes may appear in the List
            //If this locale has been tested add returns false and we skip the testing
            if (checkedLocales.add(prefLocale)) {
                
                //Loop through the available locales to resolve from.
                for (final Iterator availLocaleItr = availableLocales.iterator(); availLocaleItr.hasNext();) {
                    final Locale availLocale = (Locale)availLocaleItr.next();
                    
                    //If the locales are an exact match return immediatly
                    if (prefLocale.equals(availLocale)) {
                        return availLocale;
                    }
                    //If the language matches
                    else if (prefLocale.getLanguage().equals(availLocale.getLanguage())) {
                        //If a language match hasn't been found set this as the language match
                        if (matchesLang == null) {
                            matchesLang = availLocale;
                        }
                        
                        //If the country matches set this as the language and country match
                        if (prefLocale.getCountry().equals(availLocale.getCountry()) && matchesLangCountry == null) {
                            matchesLangCountry = availLocale;
                        }
                    }
                }
            }
        }
        
        if (matchesLangCountry != null) {
            return matchesLangCountry;
        }
        
        if (matchesLang != null) {
            return matchesLang;
        }
        
        return null;
	}
}
