/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.i18n;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import org.jasig.portal.portlet.window.IPortletWindow;

/**
 * Simple locale manager to resolve locales based on preferences.  The list of
 * preference locales and resolution logic are injected into the manager.
 * 
 * @author David Young, dyoung@unicon.net
 * @version $Revision: 11911 $
 */
public class SimpleLocaleManager implements ILocaleManager {
	private List localePreferenceProviders;
	private ILocaleResolutionProvider localeResolutionProvider;
	
	/**
     * @return Returns the localePreferenceProviders.
     */
    public List getLocalePreferenceProviders() {
        return this.localePreferenceProviders;
    }
    /**
     * @param localePreferenceProviders The localePreferenceProviders to set.
     */
    public void setLocalePreferenceProviders(List localePreferenceProviders) {
        this.localePreferenceProviders = localePreferenceProviders;
    }

    /**
     * @return Returns the localeResolutionProvider.
     */
    public ILocaleResolutionProvider getLocaleResolutionProvider() {
        return this.localeResolutionProvider;
    }
    /**
     * @param localeResolutionProvider The localeResolutionProvider to set.
     */
    public void setLocaleResolutionProvider(ILocaleResolutionProvider localeResolutionProvider) {
        this.localeResolutionProvider = localeResolutionProvider;
    }

    
	public Locale resolveLocale(Set availableLocales) {
		if (availableLocales == null) {
            throw new IllegalArgumentException("availableLocales may not be null");
		}
        if (availableLocales.size() == 0) {
            throw new IllegalArgumentException("availableLocales must contain at least one Locale");
        }
        
        //Create the ordered list of prefered Locale objects. Done here since some preference
        //provider may have dynamic information
        final List preferences = new ArrayList(this.localePreferenceProviders.size() * 5);
        for (final Iterator providers = this.localePreferenceProviders.iterator(); providers.hasNext();) {
            final ILocalePreferenceProvider provider = (ILocalePreferenceProvider)providers.next();
            final List prefOrder = provider.getLocalePreferenceOrder();
            
            if (prefOrder != null && prefOrder.size() > 0) {
                preferences.addAll(prefOrder);
            }
        }
        
		return this.localeResolutionProvider.resolveLocale(preferences, availableLocales);
	}
    
    public Locale resolveLocale() {
        for (final Iterator providerItr = this.localePreferenceProviders.iterator(); providerItr.hasNext();) {
            final ILocalePreferenceProvider provider = (ILocalePreferenceProvider)providerItr.next();
            final List locales = provider.getLocalePreferenceOrder();
            
            if (locales.size() > 0) {
                return (Locale)locales.get(0);
            }
        }
        
        throw new IllegalStateException("At least one Locale must be provided via the configured ILocalePreferenceProviders='" + this.localePreferenceProviders + "'");
    }
    
    public Locale resolveLocale(IPortletWindow portletWindow) {
        if (portletWindow == null) {
            throw new IllegalArgumentException("PortletWindow may not be null");
        }
        
		final Set<Locale> supportLocales = portletWindow.getSupportedLocales();
		return this.resolveLocale(supportLocales);
	}
}
