/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.i18n.dao.hibernate;

/**
 * Dao implementation of portlet user locale preferences dao using hibernate.
 * 
 * @author David Young, dyoung@unicon.net
 * @version $Revision: 11955 $
 */
import java.util.List;
import java.util.Locale;

import org.jasig.portal.i18n.IUserLocalePreferenceTransfer;
import org.jasig.portal.i18n.IUserLocalePreferencesDao;
import org.jasig.portal.i18n.UserLocalePreferenceTransferImpl;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.security.UserId;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.springframework.util.Assert;

public class HibernateUserLocalePreferencesDao extends HibernateDaoSupport implements IUserLocalePreferencesDao {

	static final String QUERY_FIND_LOCALE_PREF_BY_USER_ID = "FIND_LOCALE_PREF_BY_USER_ID";
	static final String QUERY_FIND_LOCALE_PREF_BY_USER_WIN_ID = "FIND_LOCALE_PREF_BY_USER_PORT_WIN_ID";
	static final String QUERY_PARAM_USER_ID = "userId";
	static final String QUERY_PARAM_WIN_ID = "winId";
	

	/**
	 * Get the portal-wide user preferences.
	 * @param userId user id
	 * @return IUserLocalePreferencesTransfer
	 */
	@SuppressWarnings("unchecked")
    public List<IUserLocalePreferenceTransfer> getUserLocalePreferences(UserId userId) {
		final HibernateTemplate hibernateTemplate = getHibernateTemplate();
        List<IUserLocalePreferenceTransfer> prefs = hibernateTemplate.findByNamedQueryAndNamedParam(QUERY_FIND_LOCALE_PREF_BY_USER_ID, QUERY_PARAM_USER_ID, userId);

		return prefs;
	}

	/**
	 * Persist the user's locale preferences.
	 * @param userLocalePreferenceTransfer object
	 */
	public void storeUserLocalePreference(IUserLocalePreferenceTransfer userLocalePreferenceTransfer) {
		if (userLocalePreferenceTransfer != null) {
			getHibernateTemplate().saveOrUpdate(userLocalePreferenceTransfer);
		}
	}

	/**
	 * Removes a locale preference from a user.
	 * @param userLocalePreferenceTransfer
	 */
	public void removeUserLocalePreference(IUserLocalePreferenceTransfer userLocalePreferenceTransfer) {
		if (userLocalePreferenceTransfer != null) {
			getHibernateTemplate().delete(userLocalePreferenceTransfer);
		}

	}

	/**
	 * Get the locale preferences for a specific piece of content.
	 * @param userId user id
	 * @param portletWindowId content id
	 * @return IUserLocalePreferencesTransfer
	 */
	@SuppressWarnings("unchecked")
    public List<IUserLocalePreferenceTransfer> getContentUserLocalePreferences(UserId userId, PortletWindowId portletWindowId) {
		String[] params = {QUERY_PARAM_USER_ID, QUERY_PARAM_WIN_ID};
		Object[] obj = {userId , portletWindowId};
		List<IUserLocalePreferenceTransfer> prefs = getHibernateTemplate().findByNamedQueryAndNamedParam(QUERY_FIND_LOCALE_PREF_BY_USER_WIN_ID, params, obj);
		return prefs;
	}

	/**
	 * Persist the user's locale prefernces for a content.
	 * @param userLocalePreferenceTransfer
	 */
	public void setContentUserLocalePreference(IUserLocalePreferenceTransfer userLocalePreferenceTransfer) {
		if (userLocalePreferenceTransfer != null) {
			getHibernateTemplate().saveOrUpdate(userLocalePreferenceTransfer);
		}

	}

	/**
	 * Removes a locale preference from the user for a content.
	 * @param userLocalePreferenceTransfer
	 */
	public void removeContentUserLocalePreference(IUserLocalePreferenceTransfer userLocalePreferenceTransfer) {
		if (userLocalePreferenceTransfer != null) {
			getHibernateTemplate().delete(userLocalePreferenceTransfer);
		}
	}

	/**
	 * Creates a new IUserLocalePreferenceTransfer object.
	 * @param userId user id
	 * @param portletWindowId window id
	 * @param locale 
	 */
	public IUserLocalePreferenceTransfer createContentUserLocalePreference(UserId userId, PortletWindowId portletWindowId, Locale locale) {
        Assert.notNull(userId, "userId may not be null");
        Assert.notNull(locale, "locale may not be null");
        
		final IUserLocalePreferenceTransfer pref = new UserLocalePreferenceTransferImpl();
		pref.setUserId(userId);
		pref.setPortletWindowId(portletWindowId);
		pref.setLocale(locale);
        this.storeUserLocalePreference(pref);
		return pref;
	}

	/**
	 * Creates a new IUserLocalePreferenceTransfer object.
	 * @param userId user id
	 * @param locale
	 */
	public IUserLocalePreferenceTransfer createUserLocalePreference(UserId userId, Locale locale) {
		return this.createContentUserLocalePreference(userId, null, locale);
	}

}
