/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.layout;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.acegisecurity.AccessDeniedException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.portlet.entity.IPortletApplicationEntity;
import org.jasig.portal.portlet.entity.IPortletEntity;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.registry.general.IPortletApplicationEntityRegistry;
import org.jasig.portal.portlet.registry.general.IPortletEntityRegistry;
import org.jasig.portal.portlet.registry.general.IPortletWindowRegistry;
import org.jasig.portal.portlet.transfer.IPortletApplicationEntityTransfer;
import org.jasig.portal.portlet.transfer.IPortletEntityTransfer;
import org.jasig.portal.portlet.transfer.IPortletWindowTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletApplicationEntityTransferImpl;
import org.jasig.portal.portlet.transfer.impl.PortletEntityTransferImpl;
import org.jasig.portal.portlet.transfer.impl.PortletWindowTransferImpl;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.security.UserId;

/**
 * A default implementation of {@link org.jasig.portal.layout.IPortletWindowMapping}
 * 
 * This implementation will remove windows mapped from non-existent template window Ids
 * in the update() method.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletWindowMappingImpl implements IPortletWindowMapping, Serializable {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PortletWindowMappingImpl.class);

    protected transient boolean removeUnusedWindows=true;
    protected transient boolean removeEmptyEntities=true;
    
    Map windowMapping;
    Map entityMapping;
    Map applicationEntityMapping;
    
    // TODO: this is just to make hibernate work .. would be nice to avoid having
    // to declare this explicitly.
    private Long id=null;
    
    protected transient IPortletWindowRegistry windowRegistry,templateUserWindowRegistry;
    protected transient IPortletEntityRegistry entityRegistry,templateUserEntityRegistry;
    protected transient IPortletApplicationEntityRegistry applicationEntityRegistry,templateUserApplicationEntityRegistry;
    
    UserId userId,templateUserId;
    
    public PortletWindowId getCreatePortletWindowId(PortletWindowId templateWindowId) {
        PortletWindowId portletWindowId=(PortletWindowId) windowMapping.get(templateWindowId);
        if(portletWindowId!=null) {
            if (logger.isDebugEnabled()) {
                logger.debug("encountered mapped tempalte window : templateWindowId=" + templateWindowId +" userId="+userId.toString()+" templateUserId="+templateUserId.toString());
            }
        } else {
            portletWindowId=createNewPortletWindow(templateWindowId);
        }
        return portletWindowId;
    }

    public PortletWindowId getPortletWindowId(PortletWindowId templateWindowId) {
        return (PortletWindowId) windowMapping.get(templateWindowId);
    }

    /**
     * This implementation creates mappings for all listed template windows
     * (creating windows and entities when necessary), and removes unused/empty
     * windows and entities according to removeUnusedWindows and removeEmptyEntities settings.
     */
    public void updateMapping(Set portletWindowIds) {
        Set unencounteredIds=new HashSet(windowMapping.keySet());
        for (Iterator iter = portletWindowIds.iterator(); iter.hasNext();) {
            PortletWindowId templateWindowId = (PortletWindowId) iter.next();
            unencounteredIds.remove(templateWindowId);
            getCreatePortletWindowId(templateWindowId);
        }
        
        // clean up unused window mappings
        for (Iterator iter = unencounteredIds.iterator(); iter.hasNext();) {
            PortletWindowId templateWindowId = (PortletWindowId) iter.next();
            PortletWindowId windowId=(PortletWindowId) windowMapping.remove(templateWindowId);
            if(removeUnusedWindows) {
                windowRegistry.removePortletWindow(windowId);
            }

            if (logger.isDebugEnabled()) {
                logger.debug("updateMapping(List) - removed ununsed window : windowId=" + windowId+ ", templateWindowId=" + templateWindowId + ", userId="+userId+", templateUserId="+templateUserId);
            }
        }
        
        
        // clean up unused entity mappings
        for (Iterator iter = entityMapping.entrySet().iterator(); iter.hasNext();) {
            Map.Entry entry = (Map.Entry) iter.next();
            PortletEntityId targetEntityId=(PortletEntityId) entry.getValue();
            IPortletEntity targetEntity=entityRegistry.getPortletEntity(targetEntityId);
            
            final Set targetEntityWindowSet;
            if (targetEntity == null) {
                targetEntityWindowSet = Collections.emptySet();
            }
            else {
                targetEntityWindowSet=targetEntity.getPortletWindows();
            }
            
            if(targetEntityWindowSet.isEmpty()) {
                iter.remove();
                if(removeEmptyEntities) {
                    entityRegistry.removePortletEntity(targetEntityId);
                }

                if (logger.isDebugEnabled()) {
                    logger.debug("updateMapping(List) - removed empty portlet entity : targetEntityId=" + targetEntityId+ ", userId="+userId+", templateUserId="+templateUserId);
                }
            }
        }
        
        // clean up unused application entities
        for (Iterator iter = applicationEntityMapping.entrySet().iterator(); iter.hasNext();) {
            Map.Entry entry = (Map.Entry) iter.next();
            PortletApplicationEntityId targetAppplicationEntityId=(PortletApplicationEntityId) entry.getValue();
            Set entities=entityRegistry.getPortletEntities(targetAppplicationEntityId);
            if(entities.isEmpty()) {
                iter.remove();
                if(removeEmptyEntities) {
                    applicationEntityRegistry.removePortletApplicationEntity(targetAppplicationEntityId);
                }
                if (logger.isDebugEnabled()) {
                    logger.debug("updateMapping(List) - removed empty portlet application Aentity : targepplicationtEntityId=" + targetAppplicationEntityId+ ", userId="+userId+", templateUserId="+templateUserId);
                }
            }
        }
        
        
    }
    
    /**
     * Will create new portlet window and map it from the templateWindowId;
     * Will also create new entity if necessary (and establish the mapping)
     */
    protected PortletWindowId createNewPortletWindow(PortletWindowId templateWindowId) {
        // retrieve template window
        
        IPortletWindow templateWindow=null;
        try {
            templateWindow=templateUserWindowRegistry.getPortletWindow(templateWindowId);
        } catch (AccessDeniedException ade) {
            logger.error("template user layout contains a window to which the user doesn't have access! : templateWindowId=" + templateWindowId + ", templateUserId=" + templateUserId+", userId="+this.userId, ade);
            return null;
        }
        
        if(templateWindow==null) {
            logger.error("unable to retrieve template window from the registry : templateWindowId=" + templateWindowId+ ", userId="+userId.toString()+", templateUserId="+templateUserId.toString(), null);
            // skip window mapping
            return null;
        }
        
        // check if the entity has been mapped
        PortletEntityId entityId=(PortletEntityId) entityMapping.get(templateWindow.getPortletEntityId());
        if(entityId==null) {
            IPortletEntity templateEntity=templateUserEntityRegistry.getPortletEntity(templateWindow.getPortletEntityId());
            // check if the application entity has been mapped
            PortletApplicationEntityId templateApplicationEntityId=templateEntity.getPortletApplicationEntityId();
            IPortletApplicationEntity templateApplicationEntity=templateUserApplicationEntityRegistry.getPortletApplicationEntity(templateApplicationEntityId);
            PortletApplicationEntityId applicationEntityId=(PortletApplicationEntityId) applicationEntityMapping.get(templateApplicationEntityId);
            if(applicationEntityId==null) {
                // create application entity
                IPortletApplicationEntityTransfer newTransfer=new PortletApplicationEntityTransferImpl(templateApplicationEntity.getPortletApplicationEntityTransfer());
                newTransfer.setUserId(userId);
                newTransfer.setPortletApplicationEntityId(null);
                applicationEntityId=applicationEntityRegistry.createPortletApplicationEntity(newTransfer).getPortletApplicationEntityId();
                applicationEntityMapping.put(templateApplicationEntityId,applicationEntityId);
                if (logger.isDebugEnabled()) {
                    logger.debug("created new portlet application entity : applicationEntityId=" + applicationEntityId + ", templateApplicationEntityId=" + templateApplicationEntityId+ ", templateWindowId=" + templateWindowId + ", userId="+userId+", templateUserId="+templateUserId);
                }
            }
            
            // create new entity
            IPortletEntityTransfer newTransfer=new PortletEntityTransferImpl(templateEntity.getPortletEntityTransfer());
            newTransfer.setUserId(userId);
            newTransfer.setPortletEntityId(null);
            newTransfer.setPortletApplicationEntityId(applicationEntityId);
            entityId= entityRegistry.createPortletEntity(newTransfer).getPortletEntityId();
            entityMapping.put(templateEntity.getPortletEntityId(),entityId);

            if (logger.isDebugEnabled()) {
                logger.debug("created new portlet entity : entityId=" + entityId + ", templateEntityId=" + templateEntity.getPortletEntityId()+ ", templateWindowId=" + templateWindowId + ", userId="+userId+", templateUserId="+templateUserId);
            }
        }
        
        // create new window
        IPortletWindowTransfer newTransfer=new PortletWindowTransferImpl(templateWindow.getPortletWindowTransfer());
        // set entity id to point to the target one, instead of the template entity id 
        newTransfer.setPortletEntityId(entityId);
        newTransfer.setPortletWindowId(null);
        IPortletWindow newWindow=windowRegistry.createPortletWindow(newTransfer);
        windowMapping.put(templateWindowId,newWindow.getPortletWindowId());

        if (logger.isDebugEnabled()) {
            logger.debug("created new portlet window : entityId=" + entityId + ", newWindow=" + newWindow +", templateWindowId=" + templateWindowId + ", userId="+userId+", templateUserId="+templateUserId);
        }

        return newWindow.getPortletWindowId();
    }

    /**
     * Specify whether windows mapped from non-existent template Ids should be deleted
     * (default true)
     * @param removeUnusedWindows
     */
    public void setRemoveUnusedWindows(boolean removeUnusedWindows) {
        this.removeUnusedWindows = removeUnusedWindows;
    }

    /**
     * Specify whether entities contained in the map that no longer have any windows
     * associated with them should be deleted (default true)
     * @param removeEmptyEntities
     */
    public void setRemoveEmptyEntities(boolean removeEmptyEntities) {
        this.removeEmptyEntities = removeEmptyEntities;
    }

    public void setEntityRegistry(IPortletEntityRegistry entityRegistry) {
        this.entityRegistry = entityRegistry;
    }

    public void setTemplateUserEntityRegistry(IPortletEntityRegistry templateUserEntityRegistry) {
        this.templateUserEntityRegistry = templateUserEntityRegistry;
    }

    public void setTemplateUserId(UserId templateUserId) {
        this.templateUserId = templateUserId;
    }

    public void setTemplateUserWindowRegistry(IPortletWindowRegistry templateUserWindowRegistry) {
        this.templateUserWindowRegistry = templateUserWindowRegistry;
    }

    public void setUserId(UserId userId) {
        this.userId = userId;
    }

    public void setWindowMapping(Map windowMapping) {
        this.windowMapping = windowMapping;
    }

    public void setWindowRegistry(IPortletWindowRegistry windowRegistry) {
        this.windowRegistry = windowRegistry;
    }

    public Map getEntityMapping() {
        return entityMapping;
    }

    public void setEntityMapping(Map entityMapping) {
        this.entityMapping = entityMapping;
    }

    public Map getWindowMapping() {
        return windowMapping;
    }

    private Long getId() {
        return id;
    }

    private void setId(Long id) {
        this.id = id;
    }
    
    public UserId getTemplateUserId() {
        return templateUserId;
    }

    public UserId getUserId() {
        return userId;
    }

    public Map getApplicationEntityMapping() {
        return applicationEntityMapping;
    }

    public void setApplicationEntityMapping(Map applicationEntityMapping) {
        this.applicationEntityMapping = applicationEntityMapping;
    }

    public void setApplicationEntityRegistry(IPortletApplicationEntityRegistry applicationEntityRegistry) {
        this.applicationEntityRegistry = applicationEntityRegistry;
    }

    public void setTemplateUserApplicationEntityRegistry(IPortletApplicationEntityRegistry templateUserApplicationEntityRegistry) {
        this.templateUserApplicationEntityRegistry = templateUserApplicationEntityRegistry;
    }
}
