/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.layout.transitory;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheable;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.rendering.sax.SaxFilterImpl;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 * Inserts transient layout manager nodes using a special hidden folder.
 * The default settings mimic uP2 TLF insertion.
 * 
 * The containing and portlet element names and their attributes can be injected.
 * 
 * The implementation decides if transient content needs to be written out at the
 * time of startDocument() call.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class LegacyTransientLayoutInjector extends SaxFilterImpl implements ICacheable {
    // injected properties
    ITransientLayoutManager transientLayoutManager;
    
    // optionally injected properties
    protected String containingElementName,portletElementName;
    protected Attributes containingElementAttributes, portletElementAttributes;
    protected String portletWindowIdAttributeName,functionalNameAttributeName,portletTitleAttributeName;
    protected String parentNodeName;
    
    
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(LegacyTransientLayoutInjector.class);
    private boolean writtenOut=false;
    
    protected final static ICacheKey noPortletCacheKey=new CacheKeyImpl("");
    
    public LegacyTransientLayoutInjector() {
        parentNodeName="folder";
        
        containingElementName="folder";
        AttributesImpl folderAtts=new AttributesImpl();
        folderAtts.addAttribute("","ID","ID","ID","ft1");
        folderAtts.addAttribute("","type","type","CDATA","regular" );
        folderAtts.addAttribute("","hidden","hidden","CDATA","true");
        folderAtts.addAttribute("","unremovable","unremovable","CDATA","true");
        folderAtts.addAttribute("","immutable","immutable","CDATA","true");
        folderAtts.addAttribute("","name","name","CDATA","Transient Folder");
        containingElementAttributes=folderAtts;
        
        portletElementName="channel";
        AttributesImpl channelAttrs = new AttributesImpl();
        channelAttrs.addAttribute("","ID","ID","ID","pt1");
        channelAttrs.addAttribute("","hidden","hidden","CDATA","false");
        channelAttrs.addAttribute("","unremovable","unremovable","CDATA","true");
        portletElementAttributes=channelAttrs;
        
        portletWindowIdAttributeName="portletWindowId"; 
        functionalNameAttributeName="fname";
        portletTitleAttributeName="name";
    }

    /**
     * Set qName of the containing element (defaults to "folder")
     * @param containingElementName
     */
    public void setContainingElementName(String containingElementName) {
        this.containingElementName = containingElementName;
    }

    /**
     * Set a map of containing element attributes (defaults to ID="ft1" type="regular" hidden="true" unremovable="true" immutable="true" name="Transient Folder")
     * @param containingElementAttributes a <code>Map</code> from <code>String</code> attribute qNames to <code>String</code> values
     */
    public void setContainingElementAttributes(Map containingElementAttributes) {
        this.containingElementAttributes = constructAttributes(containingElementAttributes);
    }
    
    /**
     * Set a map of static portlet element attributes (defaults to ID="pt1" hidden="true" unremovable="true")
     * Note: portletWindowId and functional name attributes will be added at runtime.
     * @param containingElementAttributes a <code>Map</code> from <code>String</code> attribute qNames to <code>String</code> values
     */
    public void setPortletElementAttributes(Map portletElementAttributes) {
        this.portletElementAttributes = constructAttributes(portletElementAttributes);
    }

    /**
     * Set qName of the transient portlet element.
     * @param portletElementName
     */
    public void setPortletElementName(String portletElementName) {
        this.portletElementName = portletElementName;
    }
    
    protected static Attributes constructAttributes(Map attributeValueMap) {
        AttributesImpl ai=new AttributesImpl();
        for (Iterator iter = attributeValueMap.entrySet().iterator(); iter.hasNext();) {
            Map.Entry entry = (Map.Entry) iter.next();
            String key=(String)entry.getKey(); String value=(String)entry.getValue();
            ai.addAttribute("",key,key,"CDATA",value);
        }
        return ai;
    }

    /**
     * Set portlet element functional name attribute name (defaults to "fname")
     * @param functionalNameAttributeName
     */
    public void setFunctionalNameAttributeName(String functionalNameAttributeName) {
        this.functionalNameAttributeName = functionalNameAttributeName;
    }

    /**
     * Set portlet element portlet window id attribute name (defaults to "portletWindowId";
     * @param portletWindowIdAttributeName
     */
    public void setPortletWindowIdAttributeName(String portletWindowIdAttributeName) {
        this.portletWindowIdAttributeName = portletWindowIdAttributeName;
    }

    public void startDocument() throws SAXException {
        super.startDocument();
        // determine if there is a transient window
        if(transientLayoutManager.getPortletWindowId()!=null) {
            writtenOut=false;
        } else {
            writtenOut=true;
        }
    }

    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
        super.startElement(uri, localName, qName, atts);
        if(!writtenOut) {
            // transient information still needs to be written out
            String elementName=qName; if(elementName==null) { elementName=localName; }
            if(parentNodeName.equals(elementName)) {
                IPortletWindow portletWindow=transientLayoutManager.getPortletWindow();
                if(portletWindow!=null) {
                    // output transient information
                    super.startElement("",containingElementName,containingElementName,containingElementAttributes);
                    // construct transient portlet attributes
                    AttributesImpl ai=new AttributesImpl(portletElementAttributes);
                    if(functionalNameAttributeName!=null) { ai.addAttribute("",functionalNameAttributeName,functionalNameAttributeName,"CDATA",portletWindow.getFunctionalName()); }
                    if(portletWindowIdAttributeName!=null) {ai.addAttribute("",portletWindowIdAttributeName,portletWindowIdAttributeName,"CDATA",portletWindow.getPortletWindowId().toString());}
                    if(portletTitleAttributeName!=null) {ai.addAttribute("",portletTitleAttributeName,portletTitleAttributeName,"CDATA",portletWindow.getPortletName()); }
                    super.startElement("",portletElementName,portletElementName,ai);
                    super.endElement("",portletElementName,portletElementName);
                    super.endElement("",containingElementName,containingElementName);
                    
                    if (logger.isDebugEnabled()) {
                        logger.debug("startElement(String, String, String, Attributes) - wrote out transient information into the layout stream : parent elementName=" + elementName + ", portletWindowId=" + portletWindow.getPortletWindowId().toString());
                    }
                }
                writtenOut=true;
            }
        }
    }

    public void setTransientLayoutManager(ITransientLayoutManager transientLayoutManager) {
        this.transientLayoutManager = transientLayoutManager;
    }

    /**
     * Set the name of the node under which containing element should be placed.
     * The containing element will be written out into the first occurrence of the parent element.
     * @param parentNodeName
     */
    public void setParentNodeName(String parentNodeName) {
        this.parentNodeName = parentNodeName;
    }

    public ICacheKey generateKey() {
        PortletWindowId windowId=transientLayoutManager.getPortletWindowId();
        if(windowId!=null) {
            return new CacheKeyImpl(windowId.toString());
        } else {
            return noPortletCacheKey;
        }
    }

    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }

    public void setPortletTitleAttributeName(String portletTitleAttributeName) {
        this.portletTitleAttributeName = portletTitleAttributeName;
    }
}
