/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.layout.uportal2;

import java.io.Serializable;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheable;
import org.jasig.portal.context.common.persistence.IPersistenceController;
import org.jasig.portal.context.common.persistence.IPersistenceControllerEventListener;
import org.jasig.portal.context.common.persistence.PersistenceControllerEvent;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.layout.IUserLayoutRootNodeAccessor;
import org.jasig.portal.rendering.IPortalRenderer;
import org.jasig.portal.rendering.sax.ISaxConsumer;
import org.jasig.portal.rendering.sax.ISaxRenderer;

/**
 * LegacyLayoutSource is a facade for the legacy IUserLayoutManager
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public class LegacyLayoutSource implements IPortalRenderer, ISaxRenderer, ICacheable, IPersistenceControllerEventListener, ILegacyLayoutSourceCommands {
    // injected properties
    INodeAdditionManager nodeAdditionManager;
    IPersistenceController persistenceController;
    IUserLayoutManager userLayoutManager;
    IUserLayoutRootNodeAccessor userLayoutRootNodeAccessor;
    
    
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(LegacyLayoutSource.class);
    ISaxConsumer saxConsumer;
    
    
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.IPortalRenderer#render(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public void render(HttpServletRequest req, HttpServletResponse res) throws PortalException {
        String userLayoutRootNode=null;
        if(userLayoutRootNodeAccessor==null || (userLayoutRootNode=userLayoutRootNodeAccessor.getUserLayoutRootNode(req))==null) {
            userLayoutManager.getUserLayout(saxConsumer);
        } else {
            userLayoutManager.getUserLayout(userLayoutRootNode,saxConsumer);
        }
    }

    
    /**
     * Reload layout from store
     */
    public void reloadLayout() {
        try {
            userLayoutManager.loadUserLayout();
        } catch (PortalException e) {
            logger.error("reloadLayout() - exception raised while trying to reload user layout", e);
        }
    }
    
    /**
     * Clear add/move node markings
     */
    public void clearMarkings() {
        try {
            userLayoutManager.markAddTargets(null);
            userLayoutManager.markMoveTargets(null);
        } catch (PortalException e) {
            logger.error("clearMarkings() - exception raised while trying to clear markings", e);
        }
    }
    
    /**
     * Perform move operations on a user layout node
     * @param nodeId
     * @param parentId
     * @param nextId
     */
    public void moveNode(String nodeId, String parentId, String nextId) {
        try {
            userLayoutManager.moveNode(nodeId,parentId,nextId);
        } catch (PortalException e) {
            logger.error("moveNode(String nodeId = " + nodeId + ", String parentId = " + parentId + ", String nextId = " + nextId + ") - exception raised while performing move operation on a node", e);
        }
    }
    
    /**
     * Remove a user layout node
     * @param nodeId
     */
    public void removeNode(String nodeId) {
        try {
            userLayoutManager.deleteNode(nodeId);
        } catch (PortalException e) {
            logger.error("removeNode(String nodeId = " + nodeId + ") - exception raised while trying to delete user layout node", e);
        }
    }
    
    /**
     * Change name attribute of a user layout node
     * @param nodeId
     * @param newName
     */
    public void renameNode(String nodeId, String newName) {
        try {
            IUserLayoutNodeDescription nodeDesc = userLayoutManager.getNode(nodeId);
            if (nodeDesc != null) {
                nodeDesc.setName(newName);
                userLayoutManager.updateNode(nodeDesc);
            }
        } catch (PortalException e) {
            logger.error("renameNode(String nodeId = " + nodeId + ", String newName = " + newName + ") - exception raised while renaming a node", e);
        }
    }
    
    /**
     * Show where current subscription-selected portlet can be placed
     */
    public void addMarkings() {
        IUserLayoutNodeDescription node=nodeAdditionManager.getNode();
        if(node!=null) {
            try {
                userLayoutManager.markAddTargets(node);
            } catch (PortalException e) {
                logger.error("addMarkings() - unable to set add markings.", e);
            }
        } else {
            logger.warn("addMarkings() - no node has been specified for addition.", null);
        }
    }
    
    /**
     * Insert a current subscribtion-selected portlet at a given location
     * @param parentId
     * @param nextId
     */
    public String addNode(String parentId, String nextId) {
        IUserLayoutNodeDescription node=nodeAdditionManager.getNode();
        if(node!=null) {
            try {
                IUserLayoutNodeDescription d=userLayoutManager.addNode(node,parentId,nextId);
                nodeAdditionManager.nodeAdded(d);
                if(d!=null) {
                    return d.getId();
                } else {
                    return null;
                }
            } catch (PortalException e) {
                logger.error("addNode(String parentId = " + parentId + ", String nextId = " + nextId + ") - unable to add node.", e);
            }
        } else {
            logger.warn("addNode(String parentId = " + parentId + ", String nextId = " + nextId + ") - no node has been specified for addition.", null);
        }
        return null;
    }
    
    /**
     * Show move markings for a given node
     * @param nodeId
     */
    public void moveMarkings(String nodeId) {
        try {
            userLayoutManager.markMoveTargets(nodeId);
        } catch (PortalException e) {
            logger.error("moveMarkings(String nodeId = " + nodeId + ") - exception raised while marking move targets", e);
        }
    }
    

    /* (non-Javadoc)
     * @see org.jasig.portal.caching.ICacheable#generateKey()
     */
    public ICacheKey generateKey() {
        if(userLayoutManager instanceof ICacheable) {
            return ((ICacheable)userLayoutManager).generateKey();
        } else {
            return null;
        }
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.caching.ICacheable#isCacheValid(java.io.Serializable)
     */
    public boolean isCacheValid(Serializable validityObject) {
        if(userLayoutManager instanceof ICacheable) {
            return ((ICacheable)userLayoutManager).isCacheValid(validityObject);
        } else {
            return true;
        }
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.ISaxProducer#setSaxConsumer(org.jasig.portal.rendering.sax.ISaxConsumer)
     */
    public void setSaxConsumer(ISaxConsumer consumer) {
        this.saxConsumer=consumer;
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.ISaxPipelineComponent#canRenderSax()
     */
    public boolean canRenderSax() {
        return this.saxConsumer!=null;
    }
    
    /**
     * @return Returns the nodeAdditionManager.
     */
    public INodeAdditionManager getNodeAdditionManager() {
        return nodeAdditionManager;
    }
    /**
     * @param subscriptionManager The nodeAdditionManager to set.
     */
    public void setNodeAdditionManager(INodeAdditionManager subscriptionManager) {
        this.nodeAdditionManager = subscriptionManager;
    }

    public void setPersistenceController(IPersistenceController persistanceController) {
        this.persistenceController = persistanceController;
        persistanceController.addPersistenceControllerListener(this);
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.context.common.persistance.IPersistanceControllerEventListener#receivePersistanceControllerEvent(org.jasig.portal.context.common.persistance.PersistanceControllerEvent)
     */
    public void receivePersistenceControllerEvent(PersistenceControllerEvent event) {
        if(event.getSource()==this.persistenceController) {
            try {
                if(PersistenceControllerEvent.persistPreferencesEventType.equals(event.getEventType())) {
                    // save layout
                    userLayoutManager.saveUserLayout();
                } else if(PersistenceControllerEvent.resetPreferencesEventType.equals(event.getEventType())) {
                    // load layout
                    userLayoutManager.loadUserLayout();
                }
            } catch (PortalException e) {
                logger.error("receivePersistanceControllerEvent(PersistenceControllerEvent event = " + event.getEventType().toString() + ") - exception raised while processing persistence event", e);
            }
        }
    }


    public void setUserLayoutManager(IUserLayoutManager userLayoutManager) {
        this.userLayoutManager = userLayoutManager;
    }


    public void setUserLayoutRootNodeAccessor(IUserLayoutRootNodeAccessor userLayoutRootNodeAccessor) {
        this.userLayoutRootNodeAccessor = userLayoutRootNodeAccessor;
    }


    public void processAddCommand(String command) {
        throw new UnsupportedOperationException("processAddCommands() should never be called on the legacy layout source directly, and handled by parameter processor instead");
    }
}
