/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.layout.uportal2;

import java.io.IOException;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.digester.AbstractObjectCreationFactory;
import org.apache.commons.digester.Digester;
import org.apache.commons.digester.Rule;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.core.exception.PortalRuntimeException;
import org.jasig.portal.layout.uportal2.url.AttributeCommandDescription;
import org.jasig.portal.layout.uportal2.url.IAddCommandsDigesterFactoryProvider;
import org.jasig.portal.url.IRequestParameterProcessor;
import org.jasig.portal.url.IWritableHttpServletRequest;
import org.jasig.portal.url.syntax.IUrlBasicSyntaxProvider;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * LegacyLayoutSourceRequestParameterProcessor is a user layout parameter processor, implementing uP2 URL syntax
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public class LegacyLayoutSourceRequestParameterProcessor implements IRequestParameterProcessor, ILegacyLayoutSourceCommands {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(LegacyLayoutSourceRequestParameterProcessor.class);
    
    public final static String RELOAD_LAYOUT_COMMAND="uP_reload_layout";
    public final static String CLEAR_MARAKINGS_COMMAND="uP_cancel_targets";
    public final static String REMOVE_TARGET_COMMAND="uP_remove_target";
    public final static String RENAME_TARGET_COMMAND="uP_rename_target";
    public final static String RENAME_NAME_PARAM="uP_target_name";
    public final static String MOVE_TARGET_COMMAND="uP_move_target";
    public final static String NEXT_ID_PARAM="targetNextID";
    public final static String PARENT_ID_PARAM="targetParentID";
    public final static String REQUEST_ADD_TARGETS_COMMAND="uP_request_add_targets";
    public final static String REQUEST_MOVE_TARGETS_COMMAND="uP_request_move_targets";
    public final static String CHANNEL_PUBLISH_ID_PARAM="channelPublishID";
    public final static String CHANNEL_NODE_TYPE="channel";
    public final static String FOLDER_NODE_TYPE="folder";
    public final static String ADD_TARGET_COMMAND="uP_add_target";
    public final static String ADD_COMMANDS_COMMAND="uP_add_commands";
    
    
    // injected properties
    LegacyLayoutSource layoutSource;
    INodeAdditionManager nodeAdditionManager;
    IAddCommandsDigesterFactoryProvider addCommandsDigesterFactoryProvider;

    
    /* (non-Javadoc)
     * @see org.jasig.portal.url.IUrlParameterProcessor#processParameters(org.jasig.portal.url.IWritableHttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res) {
        if(req.getParameterValues(RELOAD_LAYOUT_COMMAND)!=null) {
            reloadLayout();
        }
        
        if(req.getParameterValues(CLEAR_MARAKINGS_COMMAND)!=null) {
            clearMarkings();
        }
        
        {
            String[] nodeIds = req.getParameterValues(REMOVE_TARGET_COMMAND);
            if (nodeIds != null) {
                for (int i = 0; i < nodeIds.length; i++) {
                    String nodeId=nodeIds[i];
                    if(nodeId!=null && !nodeId.equals("")) {
                        removeNode(nodeIds[i]);
                    }
                }
            }
        }
        
        {
            String nodeId=getNonEmptyParam(RENAME_TARGET_COMMAND,req);
            if(nodeId!=null) {
                String newName=getNonEmptyParam(RENAME_NAME_PARAM,req);
                if(newName!=null) {
                    renameNode(nodeId,newName);
                }
            }
        }
        
        {
            String nodeId=getNonEmptyParam(MOVE_TARGET_COMMAND,req);
            if(nodeId!=null) {
                String parentId=getNonEmptyParam(PARENT_ID_PARAM,req);
                String nextId=getNonEmptyParam(NEXT_ID_PARAM,req);
                if(parentId!=null) { 
                    moveNode(nodeId,parentId,nextId);
                }
            }
        }
        
        {
            String nodeType=req.getParameter(REQUEST_ADD_TARGETS_COMMAND);
            if(nodeType!=null) {
                // if a folder is supposed to be added, and
                // no node is present in the node addition manager
                // add empty folder
                if(FOLDER_NODE_TYPE.equals(nodeType)) {
                    INodeAdditionManager nam=layoutSource.getNodeAdditionManager();
                    if(nam.getNode()==null) {
                        if (logger.isDebugEnabled()) {
                            logger.debug("processParameters(IWritableHttpServletRequest, HttpServletResponse) - adding an empty folder to node addition manager, as no node is currently present there, and the node type is folder");
                        }
                    }
                }
                addMarkings();
            }
        }
        
        {
            String nodeId=getNonEmptyParam(ADD_TARGET_COMMAND,req);
            if(nodeId!=null) {
                String parentId=getNonEmptyParam(PARENT_ID_PARAM,req);
                String nextId=getNonEmptyParam(NEXT_ID_PARAM,req);
                if(parentId!=null) { 
                    addNode(parentId,nextId);
                }
            }
        }
        
        
        {
            String nodeId=getNonEmptyParam(REQUEST_MOVE_TARGETS_COMMAND,req);
            if(nodeId!=null) {
                moveMarkings(nodeId);
            }
        }
        
        {
            String addCommand=getNonEmptyParam(ADD_COMMANDS_COMMAND,req);
            if(addCommand!=null) {
                processAddCommand(addCommand);
            }
        }
        
        
        return true;
    }
    
    public String getNonEmptyParam(String pName, HttpServletRequest req) {
        String pValue=req.getParameter(pName);
        if("".equals(pValue)) {
            return null;
        } else {
            return pValue;
        }
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.url.IUrlParameterProcessor#canEvaluateVariables()
     */
    public boolean canEvaluateVariables() {
        return false;
    }

    /**
     * @return Returns the layoutSource.
     */
    public LegacyLayoutSource getLayoutSource() {
        return layoutSource;
    }
    /**
     * @param layoutSource The layoutSource to set.
     */
    public void setLayoutSource(LegacyLayoutSource layoutSource) {
        this.layoutSource = layoutSource;
    }

    // layout source delegate methods
    
    public void addMarkings() {
        layoutSource.addMarkings();
    }

    public String addNode(String parentId, String nextId) {
        return layoutSource.addNode(parentId, nextId);
    }

    public void clearMarkings() {
        layoutSource.clearMarkings();
    }

    public void moveMarkings(String nodeId) {
        layoutSource.moveMarkings(nodeId);
    }

    public void moveNode(String nodeId, String parentId, String nextId) {
        layoutSource.moveNode(nodeId, parentId, nextId);
    }

    public void reloadLayout() {
        layoutSource.reloadLayout();
    }

    public void removeNode(String nodeId) {
        layoutSource.removeNode(nodeId);
    }

    public void renameNode(String nodeId, String newName) {
        layoutSource.renameNode(nodeId, newName);
    }

    public void processAddCommand(String command) {
        try {
            String commandXml=URLDecoder.decode(command,IUrlBasicSyntaxProvider.DEFAULT_ENCODING);
            
            // configure digester
            Digester digester=new Digester();
            digester.setValidating(false);
            
            // folder addition
            digester.addFactoryCreate("*/addFolder",addCommandsDigesterFactoryProvider.getFolderCreationFactory());
            // portlet addition
            digester.addFactoryCreate("*/addPortlet",addCommandsDigesterFactoryProvider.getPortletCreationFactory());
            
            // handling of outer elements (to record parentId and nextId)
            OuterElementRule oer=new OuterElementRule();
            digester.addRule("addFolder",oer);
            digester.addFactoryCreate("addFolder",addCommandsDigesterFactoryProvider.getFolderCreationFactory());
            digester.addRule("addPortlet",oer);
            digester.addFactoryCreate("addPortlet",addCommandsDigesterFactoryProvider.getPortletCreationFactory());
            
            // attribute handling
            digester.addFactoryCreate("*/attribute",new AbstractObjectCreationFactory() {
                public Object createObject(Attributes attributes) throws Exception {
                    AttributeCommandDescription d=new AttributeCommandDescription();
                    d.setProvider(attributes.getValue("provider"));
                    d.setAttributeName(attributes.getValue("name"));
                    d.setAttributeValue(attributes.getValue("value"));
                    return d;
                }
            });
            digester.addSetNext("*/attribute","setAttribute");
            
            // parse and add node structure
            try {
                IUserLayoutNodeDescription node=(IUserLayoutNodeDescription) digester.parse(new StringReader(commandXml));
                if(node==null) {
                    logger.error("processAddCommand(String) - no node resulted from processing add command : commandXml=" + commandXml, null);
                } else {
                    nodeAdditionManager.addNode(node);
                    // see if the position of the node has been specified, or if the user still needs to choose it
                    if(oer.parentId!=null) {
                        // add the node to the layout
                        layoutSource.addNode(oer.parentId,oer.nextId);
                    } else {
                        // turn on markings so user can pick a location
                        layoutSource.addMarkings();
                    }
                }
                
            } catch (IOException e) {
                logger.error("processAddCommand(String) - unable to parse add command xml : commandXml=" + commandXml, e);
                throw new PortalRuntimeException("unable to parse add command xml",e);
            } catch (SAXException e) {
                logger.error("processAddCommand(String) - unable to parse add command xml : commandXml=" + commandXml, e);
                throw new PortalRuntimeException("unable to parse add command xml",e);
            }
        } catch (UnsupportedEncodingException e) {
            logger.error("processAddCommand(String) - unable to URL decode using default encoding : command=" + command+" encoding="+IUrlBasicSyntaxProvider.DEFAULT_ENCODING, e);
            throw new PortalRuntimeException("unable to URL decode using default URL encoding",e);
        }
    }
    
    /**
     * A utility class used to track parentId and nextId settings for the  
     */
    private static class OuterElementRule extends Rule {
        /**
         * Logger for this class
         */
        private static final Log logger = LogFactory.getLog(OuterElementRule.class);

        public String parentId=null;
        public String nextId=null;
        @Override
        public void begin(String arg0, String arg1, Attributes arg2) throws Exception {
            super.begin(arg0, arg1, arg2);
            parentId=arg2.getValue("parentId");
            nextId=arg2.getValue("nextId");
        }
    }
    

    public void setAddCommandsDigesterFactoryProvider(IAddCommandsDigesterFactoryProvider addCommandsDigesterFactoryProvider) {
        this.addCommandsDigesterFactoryProvider = addCommandsDigesterFactoryProvider;
    }

    public void setNodeAdditionManager(INodeAdditionManager nodeAdditionManager) {
        this.nodeAdditionManager = nodeAdditionManager;
    }
}
