/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.layout.uportal2;

import java.util.Stack;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xalan.extensions.XSLProcessorContext;
import org.apache.xalan.templates.ElemExtensionCall;
import org.apache.xalan.transformer.TransformerImpl;
import org.jasig.portal.url.IPortalUrl;
import org.jasig.portal.url.PortalUrlXalanElements;
import org.w3c.dom.Node;

/**
 * Xalan extension elements for constructing legacy layout source URL commands.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11917 $
 */
public class LegacyLayoutSourceXalanElements {
    /**
     * Transformer parameter name used to store add commands buffer
     */
    final static String LAYOUT_ADD_STRINGBUFFER_PARAMETER_NAME="addCommandsStringBuffer";
    final static String LAYOUT_ADD_STRINGBUFFER_CLEARED_PARAMETER_NAME="addCommandsStringBuffer.cleared";
    
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(LegacyLayoutSourceXalanElements.class);

    public void clearMarkings(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl=context.getTransformer();
        ILegacyLayoutSourceCommands llsc=getLegacyLayoutSourceCommands(transformerImpl);
        llsc.clearMarkings();
    }
    
    public void reloadLayout(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl=context.getTransformer();
        ILegacyLayoutSourceCommands llsc=getLegacyLayoutSourceCommands(transformerImpl);
        llsc.reloadLayout();
    }
    
    public void addMarkings(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl=context.getTransformer();
        ILegacyLayoutSourceCommands llsc=getLegacyLayoutSourceCommands(transformerImpl);
        llsc.addMarkings();
    }
    
    public void addNode(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl=context.getTransformer();
        Node contextNode=context.getContextNode();
        ILegacyLayoutSourceCommands llsc=getLegacyLayoutSourceCommands(transformerImpl);
        String parentId=elem.getAttribute("parentId",contextNode,transformerImpl);
        String nextId=elem.getAttribute("nextId",contextNode,transformerImpl);
        llsc.addNode(parentId,nextId);
    }
    
    public void moveMarkings(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl=context.getTransformer();
        Node contextNode=context.getContextNode();
        ILegacyLayoutSourceCommands llsc=getLegacyLayoutSourceCommands(transformerImpl);
        String nodeId=elem.getAttribute("nodeId",contextNode,transformerImpl);
        llsc.moveMarkings(nodeId);
    }
    
    public void removeNode(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl=context.getTransformer();
        Node contextNode=context.getContextNode();
        ILegacyLayoutSourceCommands llsc=getLegacyLayoutSourceCommands(transformerImpl);
        String nodeId=elem.getAttribute("nodeId",contextNode,transformerImpl);
        llsc.removeNode(nodeId);
    }
    
    public void renameNode(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl=context.getTransformer();
        Node contextNode=context.getContextNode();
        ILegacyLayoutSourceCommands llsc=getLegacyLayoutSourceCommands(transformerImpl);
        String nodeId=elem.getAttribute("nodeId",contextNode,transformerImpl);
        String name=elem.getAttribute("name",contextNode,transformerImpl);
        llsc.renameNode(nodeId,name);
    }
    
    public void moveNode(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl=context.getTransformer();
        Node contextNode=context.getContextNode();
        ILegacyLayoutSourceCommands llsc=getLegacyLayoutSourceCommands(transformerImpl);
        String nodeId=elem.getAttribute("nodeId",contextNode,transformerImpl);
        String parentId=elem.getAttribute("parentId",contextNode,transformerImpl);
        String nextId=elem.getAttribute("nextId",contextNode,transformerImpl);
        llsc.moveNode(nodeId,parentId,nextId);
    }
    
    
    public void addFolder(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        // TODO: add other standard layout attribute support here
        addExtensionElement("addFolder", new String[] {"name","hidden","parentId","nextId"}, true, context, elem,true);
    }
    
    public void addPortlet(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        // TODO: add other standard layout attribute support here
        addExtensionElement("addPortlet", new String[] {"portletWindowId","fname","parentId","nextId"}, true, context, elem,true);
    }
    
    public void attribute(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        // TODO: add other standard layout attribute support here
        addExtensionElement("attribute", new String[] {"provider","name","value"}, false, context, elem,false);
    }

    /**
     * A generic utility method for processing add command extension elements
     * 
     * @param eeName name of the extension element
     * @param eeAttributes attributes that should be carried over
     * @param canBeOuterElement whether the element can be a start of the add commands
     * @param context context
     * @param elem extension call
     * @param processChildren whether the children elements should be processed
     * @throws TransformerException
     */
    private void addExtensionElement(String eeName, String[] eeAttributes, boolean canBeOuterElement, XSLProcessorContext context, ElemExtensionCall elem, boolean processChildren) throws TransformerException {
        TransformerImpl transformerImpl = context.getTransformer();
        // check for existing add buffer
        StringBuffer sb=getAddStringBuffer(transformerImpl);
        boolean outerAddElement=false;
        if(sb==null) {
            if(canBeOuterElement) {
                outerAddElement=true;
                // start recording add commands from this context on
                sb=createStringBuffer(transformerImpl);
            } else {
                logger.error("addExtensionElement(): "+eeName+" can not be an outer element");
            }
        }
        
        Node contextNode=context.getContextNode();
        // output opening string
        sb.append("<").append(eeName);
        for (int i = 0; i < eeAttributes.length; i++) {
            appendAttr(eeAttributes[i],sb,contextNode,elem,transformerImpl);
        }
        if(processChildren) {
            sb.append(">");
            transformerImpl.executeChildTemplates(elem,true);
            sb.append("</").append(eeName).append(">");
        } else {
            sb.append("/>");
        }

        if(outerAddElement) {
            // output collected add commands
            ILegacyLayoutSourceCommands llsc=getLegacyLayoutSourceCommands(transformerImpl);
            llsc.processAddCommand(sb.toString());
            clearAddStringBuffer(transformerImpl);
        }
    }
    
    
    /**
     * Append attribute expression to the string buffer if the attribute exists on the context node
     * @param sb string buffer
     * @param attrName attribute name
     * @param contextNode context node
     * @param elem extension call
     * @param transformer tranformer
     * @throws TransformerException 
     */
    protected static void appendAttr(String attrName, StringBuffer sb, Node contextNode, ElemExtensionCall elem, TransformerImpl transformer) throws TransformerException {
        String value=elem.getAttribute(attrName,contextNode,transformer);
        if(value!=null && !value.equals("")) {
            sb.append(" ").append(attrName).append("=\"").append(value).append("\"");
        }
    }

    
    /**
     * A helper method to obtain string buffer representing current add commands
     * 
     * @param transformer
     * @return add commands <code>StringBuffer</code> or <code>null</code> if it hasn't been created.
     */
     protected static StringBuffer getAddStringBuffer(Transformer transformer) {
        Boolean bufferCleared=(Boolean) transformer.getParameter(LAYOUT_ADD_STRINGBUFFER_CLEARED_PARAMETER_NAME);
        if(bufferCleared!=null && bufferCleared.booleanValue()) {
            // buffer has been cleared
            return null;
        } else {
            return (StringBuffer) transformer.getParameter(LAYOUT_ADD_STRINGBUFFER_PARAMETER_NAME);
        }
    }
    
    
    /**
     * A helper method to create string buffer representing current add commands
     * 
     * @param transformer
     * @return add commands <code>StringBuffer</code>.
     */
     protected static StringBuffer createStringBuffer(Transformer transformer) {
         StringBuffer sb=new StringBuffer();
         transformer.setParameter(LAYOUT_ADD_STRINGBUFFER_PARAMETER_NAME,sb);
         transformer.setParameter(LAYOUT_ADD_STRINGBUFFER_CLEARED_PARAMETER_NAME, Boolean.FALSE);
         return sb;
     }
     
     /**
      * A helper method to clear add command string buffer
      * 
      * @param transformer
      */
     protected static void clearAddStringBuffer(Transformer transformer) {
         transformer.setParameter(LAYOUT_ADD_STRINGBUFFER_CLEARED_PARAMETER_NAME, Boolean.TRUE);
         // this doesn't work, hence the cleared state
         // transformer.setParameter(LAYOUT_ADD_STRINGBUFFER_PARAMETER_NAME,null);
     }
     
    protected ILegacyLayoutSourceCommands getLegacyLayoutSourceCommands(Transformer transformer) {
        Stack urlStack=PortalUrlXalanElements.getUrlStack(transformer);
        if(urlStack!=null) {
            IPortalUrl portalUrl=(IPortalUrl) urlStack.peek();
            if(portalUrl!=null) {
                ILegacyLayoutSourceCommands llsc=(ILegacyLayoutSourceCommands) portalUrl.getUrlConstructor(ILegacyLayoutSourceCommands.class.getName());
                if(llsc==null) {
                    final IllegalStateException ise = new IllegalStateException("legacy layout source commands url constructor must be registered");
                    logger.error("unable to obtain legacy layout source commands constructor", ise);
                    throw ise;
                } else {
                    return llsc;
                }
            } else {
                final IllegalStateException ise = new IllegalStateException("url stack on the transformer is empty");
                logger.error("unable to obtain portal url from the url stack", ise);
                throw ise;
            }
        } else {
            final IllegalStateException ise = new IllegalStateException("url stack is null");
            logger.error("unable to obtain url stack from the transformer", ise);
            throw ise;
        }
    }
}
