/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.layout.uportal2;

import java.io.Serializable;
import java.util.Enumeration;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheable;
import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.context.common.user.IUserControllerEventListener;
import org.jasig.portal.context.common.user.UserControllerEvent;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.layout.uportal2.simple.dao.IUserLayoutDao;
import org.w3c.dom.Document;
import org.xml.sax.ContentHandler;

/**
 * A wrapper that constructs user layout from a specified {@link org.jasig.portal.layout.uportal2.ILegacyLayoutManagerFactory}
 * based on {@link org.jasig.portal.context.common.user.IUserController} information.
 * 
 *  Does not allow layout changes for anonymous users.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class UserAwareLayoutManager implements IUserLayoutManager, IUserControllerEventListener, ICacheable {
    // injected properties
    IUserController userController;
    ILegacyLayoutManagerFactory layoutManagerFactory;
    
    
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(UserAwareLayoutManager.class);
    
    protected IUserLayoutManager currentLayoutManager;
    
    protected IUserLayoutManager getCurrentLayoutManager() {
        if(currentLayoutManager==null) {
            currentLayoutManager=layoutManagerFactory.createUserLayoutManager(userController.getUserId());
        }
        return currentLayoutManager;
    }
    
    protected void resetCurrentLayoutManager() {
        this.currentLayoutManager = null;
    }
    
    /**
     * @param layoutManagerFactory The layoutManagerFactory to set.
     */
    public void setLayoutManagerFactory(ILegacyLayoutManagerFactory layoutManagerFactory) {
        this.layoutManagerFactory = layoutManagerFactory;
    }
    
    /**
     * @param userController The userController to set.
     */
    public void setUserController(IUserController userController) {
        this.userController = userController;
        userController.addUserControllerEventListener(this);
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.caching.ICacheable#generateKey()
     */
    public ICacheKey generateKey() {
        if(currentLayoutManager==null) {
            return null;
        }
        
        try {
            return new CacheKeyImpl(currentLayoutManager.getCacheKey());
        } catch (PortalException e) {
            logger.error("generateKey() - exception raised while trying to obtain user layout manager cache key", e);
            return null;
        }
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.caching.ICacheable#isCacheValid(java.io.Serializable)
     */
    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }
    
    public void receiveUserControllerEvent(UserControllerEvent event) {
        // TODO: handle persistance of the previous session
        currentLayoutManager=null;
    }

    // delegate methods
    public boolean addLayoutEventListener(LayoutEventListener l) {
        return getCurrentLayoutManager().addLayoutEventListener(l);
    }

    public IUserLayoutNodeDescription addNode(IUserLayoutNodeDescription node, String parentId, String nextSiblingId) throws PortalException {
        if(!userController.isAnonymous()) {
            return getCurrentLayoutManager().addNode(node, parentId, nextSiblingId);
        } else {
            return null;
        }
    }

    public boolean canAddNode(IUserLayoutNodeDescription node, String parentId, String nextSiblingId) throws PortalException {
        if(!userController.isAnonymous()) {
            return getCurrentLayoutManager().canAddNode(node, parentId, nextSiblingId);
        } else {
            return false;
        }
    }

    public boolean canDeleteNode(String nodeId) throws PortalException {
        if(!userController.isAnonymous()) {
            return getCurrentLayoutManager().canDeleteNode(nodeId);
        } else {
            return false;
        }
    }

    public boolean canMoveNode(String nodeId, String parentId, String nextSiblingId) throws PortalException {
        if(!userController.isAnonymous()) {
            return getCurrentLayoutManager().canMoveNode(nodeId, parentId, nextSiblingId);
        } else {
            return false;
        }
    }

    public boolean canUpdateNode(IUserLayoutNodeDescription node) throws PortalException {
        if(!userController.isAnonymous()) {
            return getCurrentLayoutManager().canUpdateNode(node);
        } else {
            return false;
        }
    }

    public IUserLayoutNodeDescription createNodeDescription(int nodeType) throws PortalException {
        if(!userController.isAnonymous()) {
            return getCurrentLayoutManager().createNodeDescription(nodeType);
        } else {
            return null;
        }
    }

    public boolean deleteNode(String nodeId) throws PortalException {
        if(!userController.isAnonymous()) {
            return getCurrentLayoutManager().deleteNode(nodeId);
        } else {
            return false;
        }
    }

    public String getCacheKey() throws PortalException {
        return getCurrentLayoutManager().getCacheKey();
    }

    public Enumeration getChildIds(String nodeId) throws PortalException {
        return getCurrentLayoutManager().getChildIds(nodeId);
    }

    public int getDepth(String nodeId) throws PortalException {
        return getCurrentLayoutManager().getDepth(nodeId);
    }

    public String getNextSiblingId(String nodeId) throws PortalException {
        return getCurrentLayoutManager().getNextSiblingId(nodeId);
    }

    public IUserLayoutNodeDescription getNode(String nodeId) throws PortalException {
        return getCurrentLayoutManager().getNode(nodeId);
    }

    public String getParentId(String nodeId) throws PortalException {
        return getCurrentLayoutManager().getParentId(nodeId);
    }

    public String getPreviousSiblingId(String nodeId) throws PortalException {
        return getCurrentLayoutManager().getPreviousSiblingId(nodeId);
    }

    public String getRootFolderId() {
        return getCurrentLayoutManager().getRootFolderId();
    }

    public String getSubscribeId(String fname) throws PortalException {
        return getCurrentLayoutManager().getSubscribeId(fname);
    }

    public void getUserLayout(ContentHandler ch) throws PortalException {
        getCurrentLayoutManager().getUserLayout(ch);
    }

    public void getUserLayout(String nodeId, ContentHandler ch) throws PortalException {
        getCurrentLayoutManager().getUserLayout(nodeId, ch);
    }

    public Document getUserLayoutDOM() throws PortalException {
        return getCurrentLayoutManager().getUserLayoutDOM();
    }

    public void loadUserLayout() throws PortalException {
        if(currentLayoutManager==null) {
            getCurrentLayoutManager();
        } else {
            getCurrentLayoutManager().loadUserLayout();
        }
    }

    public void markAddTargets(IUserLayoutNodeDescription node) throws PortalException {
        if(!userController.isAnonymous()) {
            getCurrentLayoutManager().markAddTargets(node);
        }
    }

    public void markMoveTargets(String nodeId) throws PortalException {
        if(!userController.isAnonymous()) {
            getCurrentLayoutManager().markMoveTargets(nodeId);
        }
    }

    public boolean moveNode(String nodeId, String parentId, String nextSiblingId) throws PortalException {
        if(!userController.isAnonymous()) {
            return getCurrentLayoutManager().moveNode(nodeId, parentId, nextSiblingId);
        } else {
            return false;
        }
    }

    public boolean removeLayoutEventListener(LayoutEventListener l) {
        return getCurrentLayoutManager().removeLayoutEventListener(l);
    }

    public void saveUserLayout() throws PortalException {
        if(!userController.isAnonymous()) {
            getCurrentLayoutManager().saveUserLayout();
        }
    }

    public void setLayoutStore(IUserLayoutDao ls) {
        getCurrentLayoutManager().setLayoutStore(ls);
    }

    public boolean updateNode(IUserLayoutNodeDescription node) throws PortalException {
        if(!userController.isAnonymous()) {
            return getCurrentLayoutManager().updateNode(node);
        } else {
            return false;
        }
    }
}
