/*
 * Copyright 2002 The JA-SIG Collaborative. All rights reserved. See license distributed with this
 * file and available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.layout.uportal2.simple;

import java.net.URI;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.sax.SAXResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.core.exception.PortalRuntimeException;
import org.jasig.portal.layout.IPortletWindowMapping;
import org.jasig.portal.layout.uportal2.IUserLayoutChannelDescription;
import org.jasig.portal.layout.uportal2.IUserLayoutFolderDescription;
import org.jasig.portal.layout.uportal2.IUserLayoutFolderHierarchicalDescription;
import org.jasig.portal.layout.uportal2.IUserLayoutManager;
import org.jasig.portal.layout.uportal2.IUserLayoutNodeDescription;
import org.jasig.portal.layout.uportal2.LayoutEvent;
import org.jasig.portal.layout.uportal2.LayoutEventListener;
import org.jasig.portal.layout.uportal2.LayoutMoveEvent;
import org.jasig.portal.layout.uportal2.UserLayoutChannelDescription;
import org.jasig.portal.layout.uportal2.UserLayoutFolderDescription;
import org.jasig.portal.layout.uportal2.UserLayoutNodeDescription;
import org.jasig.portal.layout.uportal2.simple.dao.ITemplateUserWindowMappingDao;
import org.jasig.portal.layout.uportal2.simple.dao.IUserLayoutDao;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.rendering.sax.xslt.ITransformerHelper;
import org.jasig.portal.security.UserId;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.ContentHandler;

/**
 * A port of uP2 simple user layout manager
 * 
 * @author <a href="mailto:pkharchenko@interactivebusiness.com">Peter Kharchenko</a>
 * @version $Revision: 11955 $
 */
public class SimpleUserLayoutManager implements IUserLayoutManager {
    protected static final String           TEMPLATE_USER_ID_ATTRIBUTE_NAME           = "templateUserId";
    protected static final String           TEMPLATE_PORTLET_WINDOW_ID_ATTRIBUTE_NAME = "templatePortletWindowId";
    protected static final String           CHANNEL_ELEMENT_NAME                      = "channel";
    protected static final String           PORTLET_WINDOW_ID_ATTRIBUTE_NAME          = "portletWindowId";

    private static final Log                log                                       = LogFactory.getLog(SimpleUserLayoutManager.class);

    protected final UserId                 owner;
    protected final UserId                 templateUserId;
    protected IUserLayoutDao                store                                     = null;
    protected ITemplateUserWindowMappingDao templateUserWindowMappingDao;
    protected Set                           listeners                                 = new HashSet();

    protected ITransformerHelper            transformerHelper;

    protected Document                      userLayoutDocument                        = null;
    protected Document                      markedUserLayout                          = null;
    protected IPortletWindowMapping         windowMapping                             = null;

    protected static Random                 rnd                                       = new Random();
    protected String                        cacheKey                                  = "initialKey";
    protected String                        rootNodeId;

    protected Map                           idByFname                                 = new HashMap();
    protected Set                           absentFnames                              = new HashSet();

    private boolean                         dirtyState                                = false;
    /**
     * This is for testing purposes only - forces implementation to use ordered set when mapping
     * template windows
     */
    static boolean                          useOrderedSet                             = false;

    // marking mode variables
    private String                          markingMode                               = null;                                            // null
                                                                                                                                            // means
                                                                                                                                            // markings
                                                                                                                                            // are
                                                                                                                                            // turned
                                                                                                                                            // off
    private String                          markingNode;

    // The names for marking nodes
    private static final String             ADD_COMMAND                               = "add";
    private static final String             MOVE_COMMAND                              = "move";

    // marking stylesheet
    private static final String             MARKING_XSLT_LOCATION                     = "MarkUserLayout.xsl";


    public SimpleUserLayoutManager(UserId owner, IUserLayoutDao store, UserId templateUserId,
                                   ITemplateUserWindowMappingDao templateUserWindowMappingDao) throws PortalException {
        if (owner == null) {
            throw new PortalException("A non-null owner needs to be specified.");
        }


        this.owner = owner;
        this.templateUserId = templateUserId;
        this.rootNodeId = null;
        this.setLayoutStore(store);
        this.templateUserWindowMappingDao = templateUserWindowMappingDao;
        this.loadUserLayout();
        this.markingMode = null;
        this.markingNode = null;
    }

    private void setUserLayoutDOM(Document doc) {
        this.userLayoutDocument = doc;
        this.markedUserLayout = null;
        this.updateCacheKey();
    }

    public Document getUserLayoutDOM() {
        return this.userLayoutDocument;
    }

    public void getUserLayout(ContentHandler ch) throws PortalException {
        Document ulm = this.getUserLayoutDOM();
        if (ulm == null) {
            throw new PortalException("User layout has not been initialized");
        }
        else {
            getUserLayout(ulm, ch);
        }
    }

    public void getUserLayout(String nodeId, ContentHandler ch) throws PortalException {
        Document ulm = this.getUserLayoutDOM();

        if (ulm == null) {
            throw new PortalException("User layout has not been initialized");
        }
        else {
            Node rootNode = ulm.getElementById(nodeId);

            if (rootNode == null) {
                throw new PortalException("A requested root node (with id=\"" + nodeId
                                          + "\") is not in the user layout.");
            }
            else {
                getUserLayout(rootNode, ch);
            }
        }
    }

    protected void getUserLayout(Node n, ContentHandler ch) throws PortalException {
        // do a DOM2SAX transformation, invoking marking transformation if necessary
        try {
            if (markingMode != null) {
                Hashtable stylesheetParams = new Hashtable(1);
                stylesheetParams.put("operation", markingMode);
                if (markingNode != null) {
                    stylesheetParams.put("targetId", markingNode);
                }
                Transformer transformer = getTransformerHelper().getTemplates(new URI(this.getClass()
                        .getResource(MARKING_XSLT_LOCATION)
                        .toString())).newTransformer();
                for (Iterator iter = stylesheetParams.keySet().iterator(); iter.hasNext();) {
                    String paramName = (String)iter.next();
                    transformer.setParameter(paramName, stylesheetParams.get(paramName));
                }
                transformer.transform(new DOMSource(n), new SAXResult(ch));
            }
            else {
                Transformer emptyt = TransformerFactory.newInstance().newTransformer();
                emptyt.transform(new DOMSource(n), new SAXResult(ch));
            }
        }
        catch (Exception e) {
            throw new PortalException("Encountered an exception trying to output user layout", e);
        }
    }


    public void setLayoutStore(IUserLayoutDao store) {
        this.store = store;
    }

    protected IUserLayoutDao getLayoutStore() {
        return this.store;
    }


    public void loadUserLayout() throws PortalException {
        if (this.getLayoutStore() == null) {
            throw new PortalException("Store implementation has not been set.");
        }
        else {
            try {
                Document uli = null;
                try {
                    uli = this.getLayoutStore().getUserLayout(this.owner);
                }
                catch (Exception e) {
                    // TODO: find a way to differentiate absence of layout from other DB errors
                    if (log.isDebugEnabled()) {
                        log.debug("unable to retrieve user layout for userId="
                                          + this.owner
                                          + ". The user may have not established the layout yet, or this could be a DB error.",
                                  e);
                    }
                }
                if (uli == null) {
                    // try loading template user layout
                    if (log.isDebugEnabled()) {
                        log.debug("trying to load user layout for user=" + this.owner + " based on the templateUserId="
                                  + this.templateUserId);
                    }
                    uli = this.getLayoutStore().getUserLayout(templateUserId);
                    if (uli == null) {
                        throw new IllegalStateException("template user layout can not be found; userId=" + owner
                                                        + "; templateUserId=" + this.templateUserId);
                    }
                    // load portlet window mapping
                    windowMapping = templateUserWindowMappingDao.getTemplateUserWindowMapping(templateUserId, owner);
                    // synchronize window mapping with the layout
                    NodeList portletElements = uli.getElementsByTagName(CHANNEL_ELEMENT_NAME);
                    // compile template window id set
                    Set templatePortletWindowIdSet;
                    if (useOrderedSet) {
                        templatePortletWindowIdSet = new TreeSet();
                    }
                    else {
                        templatePortletWindowIdSet = new HashSet();
                    }
                    for (int i = 0; i < portletElements.getLength(); i++) {
                        Element portletElement = (Element)portletElements.item(i);
                        String str_templateNodeId = portletElement.getAttribute(PORTLET_WINDOW_ID_ATTRIBUTE_NAME);
                        if (str_templateNodeId != null) {
                            PortletWindowId templateWindowId = new PortletWindowId(str_templateNodeId);
                            templatePortletWindowIdSet.add(templateWindowId);
                        }
                        else {
                            log.warn("portletWindowId attribute is missing on a portlet element. userId = "
                                     + owner.toString() + " templateUserId = " + templateUserId.toString()
                                     + " portlet ID = " + portletElement.getAttribute("ID"), null);
                        }
                    }
                    // update window mapping
                    windowMapping.updateMapping(templatePortletWindowIdSet);

                    // save window mapping
                    templateUserWindowMappingDao.setTemplateUserWindowMapping(templateUserId, owner, windowMapping);

                    // translate DOM to remapped window Ids
                    for (int i = 0; i < portletElements.getLength(); i++) {
                        Element portletElement = (Element)portletElements.item(i);
                        String str_templateNodeId = portletElement.getAttribute(PORTLET_WINDOW_ID_ATTRIBUTE_NAME);
                        if (str_templateNodeId != null) {
                            PortletWindowId templateWindowId = new PortletWindowId(str_templateNodeId);
                            portletElement.setAttribute(TEMPLATE_PORTLET_WINDOW_ID_ATTRIBUTE_NAME, str_templateNodeId);
                            portletElement.setAttribute(TEMPLATE_USER_ID_ATTRIBUTE_NAME, templateUserId.toString());
                            PortletWindowId windowId = windowMapping.getPortletWindowId(templateWindowId);
                            if (windowId != null) {
                                // windowId may be null if we were unable to remap the window. This
                                // could happen, for instance,
                                // if the user doesn't have access to the original window.
                                // TODO: put in an error portlet instead
                                portletElement.setAttribute(PORTLET_WINDOW_ID_ATTRIBUTE_NAME, windowId.toString());
                            }
                        }
                    }
                }
                if (uli != null) {
                    this.setUserLayoutDOM(uli);
                    clearDirtyFlag();
                    // inform listeners
                    for (Iterator i = listeners.iterator(); i.hasNext();) {
                        LayoutEventListener lel = (LayoutEventListener)i.next();
                        lel.layoutLoaded();
                    }
                    updateCacheKey();
                }
                else {
                    throw new PortalException("Null user layout returned for ownerId=\"" + owner.toString()
                                              + "\", and templateUserId=\"" + templateUserId.toString());
                }
            }
            catch (PortalException pe) {
                throw pe;
            }
            catch (Exception e) {
                throw new PortalException("Exception encountered while reading a layout for userId=" + this.owner, e);
            }
        }
    }

    public void saveUserLayout() throws PortalException {
        if (isLayoutDirty()) {
            Document ulm = this.getUserLayoutDOM();
            if (ulm == null) {
                throw new PortalException("UserLayout has not been initialized.");
            }
            else {
                if (this.getLayoutStore() == null) {
                    throw new PortalException("Store implementation has not been set.");
                }
                else {
                    try {
                        this.getLayoutStore().setUserLayout(this.owner, ulm, true);
                        // inform listeners
                        for (Iterator i = listeners.iterator(); i.hasNext();) {
                            LayoutEventListener lel = (LayoutEventListener)i.next();
                            lel.layoutSaved();
                        }
                    }
                    catch (PortalException pe) {
                        throw pe;
                    }
                    catch (Exception e) {
                        throw new PortalException("Exception encountered while trying to save a layout for userId="
                                                  + this.owner.toString(), e);
                    }
                }
            }
        }
    }

    public IUserLayoutNodeDescription getNode(String nodeId) throws PortalException {
        Document ulm = this.getUserLayoutDOM();
        if (ulm == null) {
            throw new PortalException("UserLayout has not been initialized.");
        }

        // find an element with a given id
        Element element = (Element)ulm.getElementById(nodeId);

        if (element == null) {
            throw new PortalException("Element with ID=\"" + nodeId + "\" doesn't exist.");
        }
        return UserLayoutNodeDescription.createUserLayoutNodeDescription(element);
    }

    /**
     * Returns the depth of a node in the layout tree.
     * 
     * @param nodeId a <code>String</code> value
     * @return a depth value
     * @exception PortalException if an error occurs
     */
    public int getDepth(String nodeId) throws PortalException {
        int depth = 0;
        for (String parentId = getParentId(nodeId); parentId != null; parentId = getParentId(parentId), depth++)
            ;
        return depth;
    }

    public IUserLayoutNodeDescription addNode(IUserLayoutNodeDescription node, String parentId, String nextSiblingId) throws PortalException {
        boolean isChannel = false;
        IUserLayoutNodeDescription parent = this.getNode(parentId);
        if (canAddNode(node, parent, nextSiblingId)) {
            // assign new Id

            if (this.getLayoutStore() == null) {
                throw new PortalException("Store implementation has not been set.");
            }
            else {
                try {
                    if (node instanceof IUserLayoutChannelDescription) {
                        isChannel = true;
                        String newId = this.getLayoutStore().generateNewChannelSubscribeId(owner);
                        node.setId(newId);
                        // register fname mapping
                        String fname = ((IUserLayoutChannelDescription)node).getFunctionalName();
                        if (fname != null) {
                            idByFname.put(fname, newId);
                            absentFnames.remove(fname);
                        }
                    }
                    else {
                        node.setId(this.getLayoutStore().generateNewFolderId(owner));
                    }
                }
                catch (PortalException pe) {
                    throw pe;
                }
                catch (Exception e) {
                    throw new PortalException("Exception encountered while generating new usre layout node Id for userId="
                                                      + this.owner.toString(),
                                              e);
                }
            }

            Document ulm = this.getUserLayoutDOM();
            Element childElement = node.getXML(ulm);
            Element parentElement = (Element)ulm.getElementById(parentId);
            if (nextSiblingId == null) {
                parentElement.appendChild(childElement);
            }
            else {
                Node nextSibling = ulm.getElementById(nextSiblingId);
                parentElement.insertBefore(childElement, nextSibling);
            }
            markLayoutDirty();

            // register element id

            childElement.setIdAttribute("ID", true);
            childElement.setAttribute("ID", node.getId());

            this.updateCacheKey();
            this.clearMarkings();

            // inform the listeners
            LayoutEvent ev = new LayoutEvent(this, node);
            for (Iterator i = listeners.iterator(); i.hasNext();) {
                LayoutEventListener lel = (LayoutEventListener)i.next();
                if (isChannel) {
                    lel.channelAdded(ev);
                }
                else {
                    lel.folderAdded(ev);
                }
            }

            // process children
            if (node instanceof IUserLayoutFolderHierarchicalDescription) {
                List children = ((IUserLayoutFolderHierarchicalDescription)node).getChildren();
                if (children != null) {
                    for (Iterator iter = children.iterator(); iter.hasNext();) {
                        IUserLayoutNodeDescription c = (IUserLayoutNodeDescription)iter.next();
                        this.addNode(c, node.getId(), null);
                    }
                }
            }

            return node;
        }
        return null;
    }

    public void clearMarkings() {
        markingMode = null;
        markingNode = null;
    }

    public boolean moveNode(String nodeId, String parentId, String nextSiblingId) throws PortalException {

        IUserLayoutNodeDescription parent = this.getNode(parentId);
        IUserLayoutNodeDescription node = this.getNode(nodeId);
        String oldParentNodeId = getParentId(nodeId);
        if (canMoveNode(node, parent, nextSiblingId)) {
            // must be a folder
            Document ulm = this.getUserLayoutDOM();
            Element childElement = (Element)ulm.getElementById(nodeId);
            Element parentElement = (Element)ulm.getElementById(parentId);
            if (nextSiblingId == null) {
                parentElement.appendChild(childElement);
                if (log.isDebugEnabled()) {
                    log.debug("moveNode(String, String, String) - moved node to the last position under the parent : nodeId="
                              + nodeId + ", parentId=" + parentId);
                }
            }
            else {
                Node nextSibling = ulm.getElementById(nextSiblingId);
                if (nextSibling != null) {
                    parentElement.insertBefore(childElement, nextSibling);
                    if (log.isDebugEnabled()) {
                        log.debug("moveNode(String, String, String) - moved node : nodeId=" + nodeId + ", parentId="
                                  + parentId + ", nextSiblingId=" + nextSiblingId);
                    }
                }
                else {
                    log.warn("moveNode(String, String, String) - invalid nextSiblingId specified, moving node to the last position in the parent : nodeId="
                             + nodeId + ", parentId=" + parentId + ", nextSiblingId=" + nextSiblingId);
                    parentElement.appendChild(childElement);
                }
            }
            markLayoutDirty();
            clearMarkings();
            updateCacheKey();

            // inform the listeners
            boolean isChannel = false;
            if (node instanceof IUserLayoutChannelDescription) {
                isChannel = true;
            }
            LayoutMoveEvent ev = new LayoutMoveEvent(this, node, oldParentNodeId);
            for (Iterator i = listeners.iterator(); i.hasNext();) {
                LayoutEventListener lel = (LayoutEventListener)i.next();
                if (isChannel) {
                    lel.channelMoved(ev);
                }
                else {
                    lel.folderMoved(ev);
                }
            }
            return true;
        }
        else {
            return false;
        }
    }

    public boolean deleteNode(String nodeId) throws PortalException {
        if (canDeleteNode(nodeId)) {
            IUserLayoutNodeDescription nodeDescription = this.getNode(nodeId);
            String parentNodeId = this.getParentId(nodeId);

            // remove fname cache
            if (nodeDescription instanceof IUserLayoutChannelDescription) {
                String fname = ((IUserLayoutChannelDescription)nodeDescription).getFunctionalName();
                if (fname != null) {
                    idByFname.remove(fname);
                }
            }

            Document ulm = this.getUserLayoutDOM();
            Element childElement = (Element)ulm.getElementById(nodeId);
            Node parent = childElement.getParentNode();
            if (parent != null) {
                parent.removeChild(childElement);
            }
            else {
                throw new PortalException("Node \"" + nodeId + "\" has a NULL parent !");
            }
            markLayoutDirty();
            // clearMarkings(); // this one is questionable
            this.updateCacheKey();

            // inform the listeners
            boolean isChannel = false;
            if (nodeDescription instanceof IUserLayoutChannelDescription) {
                isChannel = true;
            }
            LayoutMoveEvent ev = new LayoutMoveEvent(this, nodeDescription, parentNodeId);
            for (Iterator i = listeners.iterator(); i.hasNext();) {
                LayoutEventListener lel = (LayoutEventListener)i.next();
                if (isChannel) {
                    lel.channelDeleted(ev);
                }
                else {
                    lel.folderDeleted(ev);
                }
            }

            return true;
        }
        else {
            return false;
        }
    }

    public String getRootFolderId() {
        try {
            if (rootNodeId == null) {
                // Element rootNode = (Element)
                // XPathAPI.selectSingleNode(this.getUserLayoutDOM(),"//layout/folder");
                Element rootNode = this.getNodeByExpression(this.getUserLayoutDOM(), "//layout/folder");
                rootNodeId = rootNode.getAttribute("ID");
            }
        }
        catch (Exception e) {
            log.error(e);
        }
        return rootNodeId;
    }

    public synchronized boolean updateNode(IUserLayoutNodeDescription node) throws PortalException {
        if (canUpdateNode(node)) {
            // normally here, one would determine what has changed
            // but we'll just make sure that the node type has not
            // changed and then regenerate the node Element from scratch,
            // and attach any children it might have had to it.

            String nodeId = node.getId();
            String nextSiblingId = getNextSiblingId(nodeId);
            Element nextSibling = null;
            if (nextSiblingId != null) {
                Document ulm = this.getUserLayoutDOM();
                nextSibling = ulm.getElementById(nextSiblingId);
            }

            IUserLayoutNodeDescription oldNode = getNode(nodeId);

            if (oldNode instanceof IUserLayoutChannelDescription) {
                if (node instanceof IUserLayoutChannelDescription) {
                    Document ulm = this.getUserLayoutDOM();
                    // generate new XML Element
                    Element newChannelElement = node.getXML(ulm);
                    Element oldChannelElement = (Element)ulm.getElementById(nodeId);
                    Node parent = oldChannelElement.getParentNode();
                    parent.removeChild(oldChannelElement);
                    parent.insertBefore(newChannelElement, nextSibling);
                    // register new child instead
                    newChannelElement.setIdAttribute("ID", true);
                    newChannelElement.setAttribute("ID", node.getId());

                    // inform the listeners
                    LayoutEvent ev = new LayoutEvent(this, node);
                    for (Iterator i = listeners.iterator(); i.hasNext();) {
                        LayoutEventListener lel = (LayoutEventListener)i.next();
                        lel.channelUpdated(ev);
                    }
                }
                else {
                    throw new PortalException("Change channel to folder is not allowed by updateNode() method!");
                }
            }
            else {
                // must be a folder
                if (node instanceof IUserLayoutFolderDescription) {
                    Document ulm = this.getUserLayoutDOM();
                    // generate new XML Element
                    Element newFolderElement = node.getXML(ulm);
                    Element oldFolderElement = (Element)ulm.getElementById(nodeId);
                    Node parent = oldFolderElement.getParentNode();

                    // move children
                    Vector children = new Vector();
                    for (Node n = oldFolderElement.getFirstChild(); n != null; n = n.getNextSibling()) {
                        children.add(n);
                    }

                    for (int i = 0; i < children.size(); i++) {
                        newFolderElement.appendChild((Node)children.get(i));
                    }

                    // replace the actual node
                    parent.removeChild(oldFolderElement);
                    parent.insertBefore(newFolderElement, nextSibling);

                    // register new child instead
                    newFolderElement.setIdAttribute("ID", true);
                    newFolderElement.setAttribute("ID", node.getId());

                    // inform the listeners
                    LayoutEvent ev = new LayoutEvent(this, node);
                    for (Iterator i = listeners.iterator(); i.hasNext();) {
                        LayoutEventListener lel = (LayoutEventListener)i.next();
                        lel.folderUpdated(ev);
                    }
                }
            }
            markLayoutDirty();
            this.updateCacheKey();

            return true;
        }
        else {
            return false;
        }
    }


    public boolean canAddNode(IUserLayoutNodeDescription node, String parentId, String nextSiblingId) throws PortalException {
        return this.canAddNode(node, this.getNode(parentId), nextSiblingId);
    }

    protected boolean canAddNode(IUserLayoutNodeDescription node, IUserLayoutNodeDescription parent,
                                 String nextSiblingId) throws PortalException {
        // make sure sibling exists and is a child of nodeId
        if (nextSiblingId != null) {
            IUserLayoutNodeDescription sibling = getNode(nextSiblingId);
            if (sibling == null) {
                throw new PortalException("Unable to find a sibling node with id=\"" + nextSiblingId + "\"");
            }
            if (!parent.getId().equals(getParentId(nextSiblingId))) {
                throw new PortalException("Given sibling (\"" + nextSiblingId
                                          + "\") is not a child of a given parentId (\"" + parent.getId() + "\")");
            }
        }

        return (parent != null && parent instanceof IUserLayoutFolderDescription && !parent.isImmutable());
    }

    public boolean canMoveNode(String nodeId, String parentId, String nextSiblingId) throws PortalException {
        return this.canMoveNode(this.getNode(nodeId), this.getNode(parentId), nextSiblingId);
    }

    protected boolean canMoveNode(IUserLayoutNodeDescription node, IUserLayoutNodeDescription parent,
                                  String nextSiblingId) throws PortalException {
        // is the current parent immutable ?
        IUserLayoutNodeDescription currentParent = getNode(getParentId(node.getId()));
        if (currentParent == null) {
            throw new PortalException("Unable to determine a parent node for node with id=\"" + node.getId() + "\"");
        }
        return (!currentParent.isImmutable() && canAddNode(node, parent, nextSiblingId));
    }

    public boolean canDeleteNode(String nodeId) throws PortalException {
        return canDeleteNode(this.getNode(nodeId));
    }

    protected boolean canDeleteNode(IUserLayoutNodeDescription node) throws PortalException {
        return !(node == null || node.isUnremovable());
    }

    public boolean canUpdateNode(String nodeId) throws PortalException {
        return canUpdateNode(this.getNode(nodeId));
    }

    public boolean canUpdateNode(IUserLayoutNodeDescription node) {
        return !(node == null || node.isImmutable());
    }

    public void markAddTargets(IUserLayoutNodeDescription node) throws PortalException {
        if (node != null) {
            this.markingMode = ADD_COMMAND;
        }
        else {
            clearMarkings();
        }
    }

    public void markMoveTargets(String nodeId) throws PortalException {
        if (nodeId != null) {
            this.markingMode = MOVE_COMMAND;
            this.markingNode = nodeId;
        }
        else {
            clearMarkings();
        }
    }

    public String getParentId(String nodeId) throws PortalException {
        Document ulm = this.getUserLayoutDOM();
        Element nelement = (Element)ulm.getElementById(nodeId);
        if (nelement != null) {
            Node parent = nelement.getParentNode();
            if (parent != null) {
                if (parent.getNodeType() != Node.ELEMENT_NODE) {
                    throw new PortalException("Node with id=\"" + nodeId
                                              + "\" is attached to something other then an element node.");
                }
                else {
                    Element e = (Element)parent;
                    return e.getAttribute("ID");
                }
            }
            else {
                return null;
            }
        }
        else {
            throw new PortalException("Node with id=\"" + nodeId + "\" doesn't exist.");
        }
    }

    public String getNextSiblingId(String nodeId) throws PortalException {
        Document ulm = this.getUserLayoutDOM();
        Element nelement = (Element)ulm.getElementById(nodeId);
        if (nelement != null) {
            Node nsibling = nelement.getNextSibling();
            // scroll to the next element node
            while (nsibling != null && nsibling.getNodeType() != Node.ELEMENT_NODE) {
                nsibling = nsibling.getNextSibling();
            }
            if (nsibling != null) {
                Element e = (Element)nsibling;
                return e.getAttribute("ID");
            }
            else {
                return null;
            }
        }
        else {
            throw new PortalException("Node with id=\"" + nodeId + "\" doesn't exist.");
        }
    }

    public String getPreviousSiblingId(String nodeId) throws PortalException {
        Document ulm = this.getUserLayoutDOM();
        Element nelement = (Element)ulm.getElementById(nodeId);
        if (nelement != null) {
            Node nsibling = nelement.getPreviousSibling();
            // scroll to the next element node
            while (nsibling != null && nsibling.getNodeType() != Node.ELEMENT_NODE) {
                nsibling = nsibling.getNextSibling();
            }
            if (nsibling != null) {
                Element e = (Element)nsibling;
                return e.getAttribute("ID");
            }
            else {
                return null;
            }
        }
        else {
            throw new PortalException("Node with id=\"" + nodeId + "\" doesn't exist.");
        }
    }

    public Enumeration getChildIds(String nodeId) throws PortalException {
        Vector v = new Vector();
        IUserLayoutNodeDescription node = getNode(nodeId);
        if (node instanceof IUserLayoutFolderDescription) {
            Document ulm = this.getUserLayoutDOM();
            Element felement = (Element)ulm.getElementById(nodeId);
            for (Node n = felement.getFirstChild(); n != null; n = n.getNextSibling()) {
                if (n.getNodeType() == Node.ELEMENT_NODE) {
                    Element e = (Element)n;
                    if (e.getAttribute("ID") != null) {
                        v.add(e.getAttribute("ID"));
                    }
                }
            }
        }
        return v.elements();
    }

    public String getCacheKey() {
        if (markingMode == null) {
            return this.cacheKey;
        }
        else {
            if (markingNode != null) {
                return this.cacheKey + this.markingMode + this.markingNode;
            }
            else {
                return this.cacheKey + this.markingMode;
            }
        }
    }

    /**
     * This is outright cheating ! We're supposed to analyze the user layout tree and return a key
     * that corresponds uniqly to the composition and the sturcture of the tree. Here we just return
     * a different key wheneever anything changes. So if one was to move a node back and forth, the
     * key would always never (almost) come back to the original value, even though the changes to
     * the user layout are cyclic.
     */
    private void updateCacheKey() {
        this.cacheKey = Long.toString(rnd.nextLong());
    }

    /**
     * Returns a subscription id given a functional name. note: what it really returns is layout id
     * 
     * @param fname the functional name to lookup.
     * @return a <code>String</code> subscription id.
     */
    public String getSubscribeId(String fname) throws PortalException {
        // check cache
        String layoutId = (String)idByFname.get(fname);
        if (layoutId == null && !absentFnames.contains(fname)) {
            try {
                // Element fnameNode = (Element)
                // XPathAPI.selectSingleNode(this.getUserLayoutDOM(),"//channel[@fname=\'"+fname+"\']");
                Element fnameNode = this.getNodeByExpression(this.getUserLayoutDOM(), "//channel[@fname=\'" + fname
                                                                                      + "\']");
                if (fnameNode != null) {
                    layoutId = fnameNode.getAttribute("ID");
                    idByFname.put(fname, layoutId);
                }
                else {
                    absentFnames.add(fname);
                }
            }
            catch (Exception e) {
                log.error("encountered the following exception, while trying to identify subscribe channel id for the fname='" + fname + "'", e);
                throw new PortalRuntimeException("unable to scan for an fname in the layout", e);
            }
        }
        return layoutId;
    }

    public boolean addLayoutEventListener(LayoutEventListener l) {
        return listeners.add(l);
    }

    public boolean removeLayoutEventListener(LayoutEventListener l) {
        return listeners.remove(l);
    }

    /**
     * A factory method to create an empty <code>IUserLayoutNodeDescription</code> instance
     * 
     * @param nodeType a node type value
     * @return an <code>IUserLayoutNodeDescription</code> instance
     * @exception PortalException if the error occurs.
     */
    public IUserLayoutNodeDescription createNodeDescription(int nodeType) throws PortalException {
        switch (nodeType) {
            case IUserLayoutNodeDescription.FOLDER:
                return new UserLayoutFolderDescription();
            case IUserLayoutNodeDescription.CHANNEL:
                return new UserLayoutChannelDescription();
            default:
                return null;
        }
    }

    protected boolean isLayoutDirty() {
        return dirtyState;
    }

    private void markLayoutDirty() {
        dirtyState = true;
    }

    private void clearDirtyFlag() {
        dirtyState = false;
    }

    /**
     * @param transformerHelper The transformerHelper to set.
     */
    public void setTransformerHelper(ITransformerHelper transformerHelper) {
        this.transformerHelper = transformerHelper;
    }

    /**
     * @return Returns the transformerHelper.
     */
    public ITransformerHelper getTransformerHelper() {
        return this.transformerHelper;
    }

    public void setTemplateUserWindowMappingDao(ITemplateUserWindowMappingDao templateUserWindowMappingDao) {
        this.templateUserWindowMappingDao = templateUserWindowMappingDao;
    }

    protected Element getNodeByExpression(Document doc, String xPathExpression) throws XPathExpressionException {
        XPathFactory fac = XPathFactory.newInstance();
        XPath xPath = fac.newXPath();
        Element node = (Element)xPath.evaluate(xPathExpression, doc, XPathConstants.NODE);
        return node;
    }
}
