/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.layout.uportal2.url;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.digester.AbstractObjectCreationFactory;
import org.apache.commons.digester.ObjectCreationFactory;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.core.exception.PortalRuntimeException;
import org.jasig.portal.layout.uportal2.UserLayoutChannelDescription;
import org.jasig.portal.layout.uportal2.UserLayoutFolderDescription;
import org.jasig.portal.layout.uportal2.UserLayoutNodeDescription;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.rendering.attributes.IRenderingAttributeProvider;
import org.jasig.portal.utils.XmlUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.Attributes;

/**
 * An implementation of {@link org.jasig.portal.layout.uportal2.url.IAddCommandsDigesterFactoryProvider}
 * that generates digester objects directly executing add commands.
 * 
 * 
 * Note: implementation normally sets attributes for the standard layout xml elements (i.e. "channel" and "folder"),
 * as specified by the {@link org.jasig.portal.layout.uportal2.UserLayoutChannelDescription} etc.. If these are
 * renamed (i.e. "channel" elements become "portlets" by the time rendering attributes are injected), then it
 * is necessary to set portletElementName (or folderElementName) properties to override normal element names when
 * setting attributes.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class AddCommandsDigesterFactoryProvider implements IAddCommandsDigesterFactoryProvider {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(AddCommandsDigesterFactoryProvider.class);
    
    // optionally injected properties
    String portletElementName=null;
    String folderElementName=null;

    //  internal properties
    HashMap renderingAttributeProviderMap=null;
    
    Document fakeRoot=null;
    
    public AddCommandsDigesterFactoryProvider() {
        try {
            fakeRoot=DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        } catch (ParserConfigurationException e) {
            logger.error("AddCommandsDigesterFactoryProvider() - unable to create empty DOM Document", e);
            throw new PortalRuntimeException("unable to create new DOM Document",e);
        }
    }
    
    public ObjectCreationFactory getFolderCreationFactory() {
        return new FolderCreationFactory();
    }

    public ObjectCreationFactory getPortletCreationFactory() {
        return new PortletCreationFactory();
    }
    
    class FolderCreationFactory extends AbstractObjectCreationFactory {
        public Object createObject(Attributes attrs) throws Exception {
            Object parent=super.getDigester().peek();
            if(parent instanceof AttributeProcessingFolderDescription) {
                return new AttributeProcessingFolderDescription(attrs,(AttributeProcessingFolderDescription)parent);
            } else {
                return new AttributeProcessingFolderDescription(attrs,null);
            }
        }
    }
    
    class PortletCreationFactory extends AbstractObjectCreationFactory {
        public Object createObject(Attributes attrs) throws Exception {
            Object parent=super.getDigester().peek();
            if(parent instanceof AttributeProcessingFolderDescription) {
                return new AttributeProcessingPortletDescription(attrs,(AttributeProcessingFolderDescription)parent);
            } else {
                return new AttributeProcessingPortletDescription(attrs,null);
            }
        }
    }

    private void setNodeDescriptionProperties(UserLayoutNodeDescription node, Attributes atts) {
        String name=atts.getValue("name");
        if(name!=null) {
            node.setName(name);
        }
        
        String immutable=atts.getValue("immutable");
        if(immutable!=null && immutable.equals("true")) {
            node.setImmutable(true);
        }
        
        String unremovable=atts.getValue("unremovable");
        if(unremovable!=null && unremovable.equals("true")) {
            node.setUnremovable(true);
        }
        
        String hidden=atts.getValue("hidden");
        if(hidden!=null && hidden.equals("true")) {
            node.setHidden(true);
        }
    }
    
    /**
     * Set rendering attribute for a node
     * @param provider
     * @param name
     * @param value
     */
    void setAttributeValue(AttributeCommandDescription d, String qName, String localName, String namespace, Attributes attributes) {
        String providerName=d.getProvider();
        String attName=d.getAttributeName();
        String attValue=d.getAttributeValue();
        
        IRenderingAttributeProvider provider=(IRenderingAttributeProvider) renderingAttributeProviderMap.get(providerName);
        if(provider==null) {
            logger.error("setAttribute(String,String,String) - Ivalid rendering attribute provider specified : providerName=" + providerName+" attName="+attName+" attValue="+attValue, null);
        } else {
            if (logger.isDebugEnabled()) {
                String element="<"+qName;
                for(int j=0;j<attributes.getLength();j++) {
                    element+=" "+attributes.getQName(j)+"="+attributes.getValue(j);
                }
                element+="/>";
                logger.debug("setAttribute(String,String,String) - setting attribute value : providerName=" + providerName + ", attName=" + attName + ", attValue=" + attValue + ", element="+element);
            }
            provider.setAttributeValue(attName,attValue,namespace,localName,qName,attributes);
        }
    }
    
    class AttributeProcessingFolderDescription extends UserLayoutFolderDescription {
        AttributeProcessingFolderDescription parentFolder;
        List<AttributeCommandDescription> attributeCommands;
        
        public AttributeProcessingFolderDescription(Attributes atts, AttributeProcessingFolderDescription parentFolder) {
            this.parentFolder=parentFolder;
            if(parentFolder!=null) {
                parentFolder.addChild(this);
            }
            this.attributeCommands=null;
            setNodeDescriptionProperties(this,atts);
        }
        
        public void setAttribute(AttributeCommandDescription d) {
            if(attributeCommands==null) {
                attributeCommands=new LinkedList<AttributeCommandDescription>();
            }
            attributeCommands.add(d);
        }

        @Override
        public void setId(String id) {
            super.setId(id);
            if(attributeCommands!=null) {
                // set attributes
                // obtain full element from the node addition manager (with newly determined layout id)
                Element folderElement=this.getXML(fakeRoot);
                
                // set element information
                String qName=folderElementName==null?folderElement.getTagName():folderElementName;
                String localName=qName;
                String namespace=folderElement.getNamespaceURI();
                // record attributes
                Attributes attributes=XmlUtils.getAttributes(folderElement);
                
                for (Iterator<AttributeCommandDescription> iter = attributeCommands.iterator(); iter.hasNext();) {
                    AttributeCommandDescription element =  iter.next();
                    setAttributeValue(element,qName,localName,namespace,attributes);
                }
            }
        }

        public AttributeProcessingFolderDescription getParentFolder() {
            return parentFolder;
        }
    }
    
    class AttributeProcessingPortletDescription extends UserLayoutChannelDescription {
        AttributeProcessingFolderDescription parentFolder;
        List<AttributeCommandDescription> attributeCommands;
        public AttributeProcessingPortletDescription(Attributes atts, AttributeProcessingFolderDescription parentFolder) {
            this.parentFolder=parentFolder;
            if(parentFolder!=null) {
                parentFolder.addChild(this);
            }
            this.attributeCommands=null;
            setNodeDescriptionProperties(this,atts);
            
            // TODO: automatic id generation/determination based on fname
            // determine portlet window id
            String str_portletWindowId=atts.getValue("portletWindowId");
            if(str_portletWindowId==null) {
                final IllegalArgumentException iae = new IllegalArgumentException("portletWindowId may not be null");
                logger.error("AttributeProcessingPortletDescription() - no portletWindowId specified for portlet", iae);
                throw iae;
            }
            this.setPortletWindowId(new PortletWindowId(str_portletWindowId));
            // fname
            this.setFunctionalName(atts.getValue("fname"));
        }
        
        public void setAttribute(AttributeCommandDescription d) {
            if(attributeCommands==null) {
                attributeCommands=new LinkedList<AttributeCommandDescription>();
            }
            attributeCommands.add(d);
        }

        @Override
        public void setId(String id) {
            super.setId(id);
            if(attributeCommands!=null) {
                // set attributes
                // obtain full element from the node addition manager (with newly determined layout id)
                Element portletElement=this.getXML(fakeRoot);
                
                // set element information
                String qName=portletElementName==null?portletElement.getTagName():portletElementName;
                String localName=qName;
                String namespace=portletElement.getNamespaceURI();
                // record attributes
                Attributes attributes=XmlUtils.getAttributes(portletElement);
                
                for (Iterator<AttributeCommandDescription> iter = attributeCommands.iterator(); iter.hasNext();) {
                    AttributeCommandDescription element =  iter.next();
                    setAttributeValue(element,qName,localName,namespace,attributes);
                }
            }
        }
    }

    
    /**
     * Specify a list of rendering attribute providers to which the events should be delivered
     * @param renderingAttributeProviders
     */
    public void setRenderingAttributeProviders(Set renderingAttributeProviders) {
        // establish name->instance map
        renderingAttributeProviderMap=new HashMap();
        for (Iterator iter = renderingAttributeProviders.iterator(); iter.hasNext();) {
            IRenderingAttributeProvider provider = (IRenderingAttributeProvider) iter.next();
            renderingAttributeProviderMap.put(provider.getProviderName(),provider);
        }
    }

    /**
     * Specify folder element name (defaults to "folder")
     * @param folderElementName
     */
    public void setFolderElementName(String folderElementName) {
        this.folderElementName = folderElementName;
    }

    /**
     * Specify portlet element name (defaults to "portlet") 
     * @param portletElementName
     */
    public void setPortletElementName(String portletElementName) {
        this.portletElementName = portletElementName;
    }

}
