/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.util.Collections;
import java.util.List;



/**
 * The merging portlet info implementation specifies a targeted
 * portlet info and a default portlet info. Any modifications made
 * via this class will only happen on the targeted portlet info, the default
 * portlet info cannot be modified via this class.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class LayeredPortletInfo implements IPortletInfo {
    private final IPortletInfo targetPortletInfo;
    private final IPortletInfo defaultPortletInfo;
    
    /**
     * Creates a new merging portlet info class.
     * 
     * @param targetPortletInfo The portlet info to add/remove entries on, may not be null.
     * @param defaultPortletInfo The portlet info to use as a fall back when finding entries.
     */
    public LayeredPortletInfo(IPortletInfo targetPortletInfo, IPortletInfo defaultPortletInfo) {
        if (targetPortletInfo == null) {
            throw new IllegalArgumentException("targetPortletInfo cannot be null");
        }
        if (defaultPortletInfo == null) {
            throw new IllegalArgumentException("defaultPortletInfo cannot be null");
        }
        
        this.targetPortletInfo = targetPortletInfo;
        this.defaultPortletInfo = defaultPortletInfo;
    }

    public String getTitle() {
        final String targetTitle = this.targetPortletInfo.getTitle();
        
        if (targetTitle != null) {
            return targetTitle;
        }
        else {
            return this.defaultPortletInfo.getTitle();
        }
    }

    public String getShortTitle() {
        final String targetShortTitle = this.targetPortletInfo.getShortTitle();
        
        if (targetShortTitle != null) {
            return targetShortTitle;
        }
        else {
            return this.defaultPortletInfo.getShortTitle();
        }
    }

    public String getKeywordsString() {
        final String targetKeywords = this.targetPortletInfo.getKeywordsString();
        
        if (targetKeywords != null) {
            return targetKeywords;
        }
        else {
            return this.defaultPortletInfo.getKeywordsString();
        }
    }

    public List<String> getKeywords() {
        final List<String> targetKeywords = this.targetPortletInfo.getKeywords();
        
        if (targetKeywords != null) {
            return Collections.unmodifiableList(targetKeywords);
        }
        else {
            return Collections.unmodifiableList(this.defaultPortletInfo.getKeywords());
        }
    }

    public void setTitle(String title) {
        if (title != null && !title.equals(this.defaultPortletInfo.getTitle())) {
            this.targetPortletInfo.setTitle(title);
        }
        else {
            this.targetPortletInfo.setTitle(null);
        }
    }

    public void setShortTitle(String shortTitle) {
        if (shortTitle != null && !shortTitle.equals(this.defaultPortletInfo.getShortTitle())) {
            this.targetPortletInfo.setShortTitle(shortTitle);
        }
        else {
            this.targetPortletInfo.setShortTitle(null);
        }
    }

    public void setKeywords(List<String> keywords) {
        if (keywords != null && !keywords.equals(this.defaultPortletInfo.getKeywords())) {
            this.targetPortletInfo.setKeywords(keywords);
        }
        else {
            this.targetPortletInfo.setKeywords(null);
        }
    }

    public void setKeywordsString(String keywords) {
        if (keywords != null && !keywords.equals(this.defaultPortletInfo.getKeywordsString())) {
            this.targetPortletInfo.setKeywordsString(keywords);
        }
        else {
            this.targetPortletInfo.setKeywordsString(null);
        }
    }
}
