/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.common;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jasig.portal.utils.LayeredMap;

/**
 *  A utility class implementing behavior of nested preferences.
 *  Two {@link org.jasig.portal.portlet.common.IPreferences} instances are supplied,
 *  with values in one overwriting the values in the other (if specified). Control
 *  methods operate only on the overwriting preferences.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class LayeredPreferences implements IPreferences, Serializable {
    private static final long serialVersionUID = 1L;

    protected final IPreferences targetPreferences;
    protected final IPreferences defaultPreferences;
    protected final Map<String, IPreference> preferences;
    
    /**
     * Construct layered preferences from a base and a current preferences layer.
     * TODO: implement PortletPreferences instead of the old Pluto classes
     * 
     * @param targetPreferences overwriting preferences set, may not be null
     * @param defaultPreferences base preferences set, may be null
     */
    public LayeredPreferences(IPreferences targetPreferences, IPreferences defaultPreferences) {
        if (targetPreferences == null) {
            throw new IllegalArgumentException("targetPreferences preferences cannot be null");
        }
        if (defaultPreferences == null) {
            throw new IllegalArgumentException("defaultPreferences preferences cannot be null");
        }
        
        this.targetPreferences=targetPreferences;
        this.defaultPreferences=defaultPreferences;
        
        this.preferences = new LayeredMap<String, IPreference>(this.targetPreferences.getPreferencesMap(), this.defaultPreferences.getPreferencesMap());
    }
    
    public long getId() {
        return this.targetPreferences.getId();
    }

    public void setId(long id) {
        this.targetPreferences.setId(id);
    }

    public void addPreference(IPreference preference) {
        if (preference == null) {
            throw new IllegalArgumentException("preference may not be null");
        }
        
        final IPreference basePref = this.defaultPreferences.getPreference(preference.getName());
        if (!preference.equals(basePref)) {
            this.targetPreferences.addPreference(preference);
        }
    }

    public void addPreferences(List preferences) {
        if (preferences == null) {
            throw new IllegalArgumentException("preferences may not be null");
        }
        
        for (final Iterator prefItr = preferences.iterator(); prefItr.hasNext();) {
            this.addPreference((IPreference)prefItr.next());
        }
    }

    public IPreference getPreference(String preferenceName) {
        final IPreference pref = this.targetPreferences.getPreference(preferenceName);
        if (pref != null) {
            return pref;
        }
        else {
            return this.defaultPreferences.getPreference(preferenceName);
        }
    }

    public List<IPreference> getPreferences() {
        return new ArrayList<IPreference>(this.preferences.values());
    }

    public String getPreferencesValidatorClassName() {
        String validatorClassName = 
                this.targetPreferences.getPreferencesValidatorClassName();
        if (validatorClassName == null) {
            validatorClassName = 
                this.defaultPreferences.getPreferencesValidatorClassName();
        }
        return validatorClassName;
    }

    public IPreference removePreference(String preferenceName) {
        return this.targetPreferences.removePreference(preferenceName);
    }

    public void setPreferences(List<IPreference> preferences) {
        this.targetPreferences.setPreferences(preferences);
    }

    public void setPreferencesValidatorClassName(String preferencesValidatorClassName) {
        this.targetPreferences.setPreferencesValidatorClassName(preferencesValidatorClassName);
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreferences#getPreferencesMap()
     */
    public Map<String, IPreference> getPreferencesMap() {
        return this.preferences;
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreferences#setPreferencesMap(java.util.Map)
     */
    public void setPreferencesMap(Map<String, IPreference> preferences) {
        this.preferences.clear();
        this.preferences.putAll(preferences);
    }
}
