/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * Implementation of IPortletInfo.
 * @author Michael Ivanov, mivanov@unicon.net
 * @version $Revision: 11911 $
 */
public class PortletInfoImpl implements IPortletInfo, Serializable {
    private String title = null;
    private String shortTitle = null;
    private List<String> keywords = new ArrayList<String>();
    private String keywordsStr = "";
    
    public PortletInfoImpl() {
    }
    
    public PortletInfoImpl(IPortletInfo portletInfo) {
        this.title = portletInfo.getTitle();
        this.shortTitle = portletInfo.getShortTitle();
        this.setKeywordsString(portletInfo.getKeywordsString());
    }

    public String getTitle() {
        return title;
    }

    public String getShortTitle() {
        return shortTitle;
    }

    public synchronized List<String> getKeywords() {
        return this.keywords;
    }

    public synchronized String getKeywordsString() {
        return this.keywordsStr;
    }

    public void setTitle(final String title) {
        this.title = title;
    }

    public void setShortTitle(final String shortTitle) {
        this.shortTitle = shortTitle;
    }

    public synchronized void setKeywordsString(String keywordsStr) {
        if (keywordsStr == null) {
            keywordsStr = "";
        }

        this.keywords = this.parseKeywords(keywordsStr);
        this.keywordsStr = keywordsStr;
    }

    public synchronized void setKeywords(List<String> keywords) {
        if (keywords == null) {
            keywords = new ArrayList<String>();
        }

        this.keywords = keywords;
        this.keywordsStr = formatKeywords(keywords);
    }

    /**
     * Parses a comma-separated keywords string into 
     * a list of keyword strings.
     * @param keywordString the comma-separated keyword string
     * @return a list of keyword strings
     */
    private List<String> parseKeywords(final String keywordString) {
        final String[] keywordsArray = keywordString.split(",");
        final List<String> keywords = new ArrayList<String>(keywordsArray.length);
        for (final String keyword : keywordsArray) {
            keywords.add(keyword.trim());
        }
        
        return keywords;
    }
    
    /**
     * Formats a List of String keywords into a comma seperated
     * String.
     * @param keywords The List of keywords to process.
     * @return A comma seperated String of keywords.
     */
    private String formatKeywords(final List<String> keywords) {
        final StringBuilder keywordStrBuilder = new StringBuilder(10 * keywords.size());
        
        for (final Iterator keywordItr = keywords.iterator(); keywordItr.hasNext();) { 
            final String keyword = (String)keywordItr.next();
            keywordStrBuilder.append(keyword);
            
            if (keywordItr.hasNext()) {
                keywordStrBuilder.append(", ");
            }
        }

        return keywordStrBuilder.toString();
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof IPortletInfo)) {
            return false;
        }
        IPortletInfo rhs = (IPortletInfo)object;
        return new EqualsBuilder()
            .append(this.title, rhs.getTitle())
            .append(this.keywords, rhs.getKeywords())
            .append(this.shortTitle, rhs.getShortTitle())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(594801933, 824451589)
            .appendSuper(super.hashCode())
            .append(this.title)
            .append(this.keywords)
            .append(this.shortTitle)
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
        .appendSuper(super.toString())
        .append("shortTitle", this.shortTitle)
        .append("keywords", this.keywords)
        .append("title", this.title)
        .toString();
    }
}
