/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * Implementation of IPreferences.
 * TODO: implement PortletPreferences instead of the old Pluto classes
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11911 $
 */
public class PreferencesImpl implements IPreferences, Serializable {
    /** Transient objects are marked with this value */
    public static final long UNPERSISTED_ID = -1L;
    private static final long serialVersionUID = 1L;

    private long id = UNPERSISTED_ID;
    private Map<String, IPreference> preferences = new LinkedHashMap<String, IPreference>();
    private String preferencesValidatorClassName;
    
    public PreferencesImpl() {
    }
    
    public PreferencesImpl(final IPreferences preferences) {
        if (preferences == null) {
            throw new IllegalArgumentException("preferences may not be null");
        }
        
        this.preferencesValidatorClassName = preferences.getPreferencesValidatorClassName();
        for (final IPreference preference : preferences.getPreferencesMap().values()) {
            final IPreference preferenceCopy = new PreferenceImpl(preference);
            this.addPreference(preferenceCopy);
        }
    }

    
    //********** IPreferences Methods **********//
    public long getId() {
        return this.id;
    }
    public void setId(long id) {
        this.id = id;
    }

    
    public void addPreference(IPreference preference) {
        if (preference == null) {
            throw new IllegalArgumentException("preference may not be null");
        }

        this.preferences.put(preference.getName(), preference);        
    }

    public void addPreferences(List<IPreference> preferences) {
        if (preferences == null) {
            throw new IllegalArgumentException("preferences may not be null");
        }
        
        for (final IPreference preference : preferences) {
            this.addPreference(preference);
        }
    }

    public IPreference getPreference(String preferenceName) {
        if (preferenceName == null) {
            throw new IllegalArgumentException("preferenceName may not be null");
        }
        
        return this.preferences.get(preferenceName);
    }

    public IPreference removePreference(String preferenceName) {
        if (preferenceName == null) {
            throw new IllegalArgumentException("preferenceName may not be null");
        }
        
        return this.preferences.remove(preferenceName);
    }

    /**
     * <b>Warning</b> - manipulating this List directly is not syncronized with modifications
     * to this List made by this class or in other classes that have called getPreferences.
     * 
     *@see org.jasig.portal.portlet.common.IPreferences#getPreferences()
     */
    public List<IPreference> getPreferences() {
        return new ArrayList<IPreference>(this.preferences.values());
    }

    public void setPreferences(List<IPreference> preferences) {
        if (preferences == null) {
            throw new IllegalArgumentException("preferences may not be null");
        }
        
        synchronized (this.preferences) {
            this.preferences.clear();
            this.addPreferences(preferences);
        }
    }

    public String getPreferencesValidatorClassName() {
        return this.preferencesValidatorClassName;
    }

    public void setPreferencesValidatorClassName(String preferencesValidatorClassName) {
        this.preferencesValidatorClassName = preferencesValidatorClassName;
    }
    
    public Map<String, IPreference> getPreferencesMap() {
        return this.preferences;
    }
    
    public void setPreferencesMap(Map<String, IPreference> preferences) {
        this.preferences = preferences;
    }
    
    
    //********** Object Methods **********//

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof PreferencesImpl)) {
            return false;
        }
        PreferencesImpl rhs = (PreferencesImpl)object;
        return new EqualsBuilder()
            .append(this.preferences, rhs.preferences)
            .append(this.preferencesValidatorClassName, rhs.preferencesValidatorClassName)
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(1689162999, 442794499)
            .append(this.preferences)
            .append(this.preferencesValidatorClassName)
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
            .appendSuper(super.toString())
                .append("preferencesValidatorClassName", this.preferencesValidatorClassName)
                .append("preferences", this.preferences)
                .toString();
    }
}
