/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.pluto.descriptors.common.DescriptionDD;
import org.apache.pluto.descriptors.common.DisplayNameDD;
import org.apache.pluto.descriptors.common.UserDataConstraintDD;
import org.apache.pluto.descriptors.portlet.PortletCollectionDD;
import org.apache.pluto.descriptors.portlet.SecurityConstraintDD;
import org.jasig.portal.portlet.deployment.PlutoDescriptionImpl;
import org.jasig.portal.portlet.deployment.PlutoDisplayNameImpl;

/**
 * Implementation of ISecurityConstraint.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11911 $
 */
public class SecurityConstraintImpl implements ISecurityConstraint {
    private final SecurityConstraintDD securityConstraintDD;
    
    public SecurityConstraintImpl(SecurityConstraintDD securityConstraintDD) {
        if (securityConstraintDD == null) {
            throw new IllegalArgumentException("SecurityConstraintDD can not be null");
        }
        
        this.securityConstraintDD = securityConstraintDD;
    }

    /**
     * @see org.jasig.portal.portlet.common.ISecurityConstraint#getDisplayNames()
     */
    @SuppressWarnings("unchecked")
    public Map<Locale, IDisplayName> getDisplayNames() {
        final List<DisplayNameDD> plutoDisplayNames = this.securityConstraintDD.getDisplayNames();
        final Map<Locale, IDisplayName> upDisplayNames = new HashMap<Locale, IDisplayName>(plutoDisplayNames.size());
        
        for (final DisplayNameDD plutoDisplayName : plutoDisplayNames) {
            final IDisplayName upDisplayName = new PlutoDisplayNameImpl(plutoDisplayName);
            upDisplayNames.put(upDisplayName.getLocale(), upDisplayName);
        }
        
        return Collections.unmodifiableMap(upDisplayNames);
    }

    /**
     * @see org.jasig.portal.portlet.common.ISecurityConstraint#getPortletCollection()
     */
    @SuppressWarnings("unchecked")
    public List<String> getPortletCollection() {
        final PortletCollectionDD portletCollection = this.securityConstraintDD.getPortletCollection();
        return Collections.unmodifiableList(portletCollection.getPortletNames());
    }

    /**
     * @see org.jasig.portal.portlet.common.ISecurityConstraint#getUserDataConstraint()
     */
    @SuppressWarnings("unchecked")
    public IUserDataConstraint getUserDataConstraint() {
        final UserDataConstraintDD userDataConstraint = this.securityConstraintDD.getUserDataConstraint();

        final String transportGuaranteeStr = userDataConstraint.getTransportGuarantee();
        final TransportGuarantee transportGuarantee = TransportGuarantee.valueOf(transportGuaranteeStr);
        
        final List<DescriptionDD> plutoDescriptions = userDataConstraint.getDescriptions();
        final Map<Locale, IDescription> upDescriptions = new HashMap<Locale, IDescription>(plutoDescriptions.size());
        for (final DescriptionDD plutoDescription : plutoDescriptions) {
            final IDescription upDescription = new PlutoDescriptionImpl(plutoDescription);
            upDescriptions.put(upDescription.getLocale(), upDescription);
        }
        
        final IUserDataConstraint upUserConstraint = new UserDataConstraintImpl(transportGuarantee, Collections.unmodifiableMap(upDescriptions));
        return upUserConstraint;
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof ISecurityConstraint)) {
            return false;
        }
        ISecurityConstraint rhs = (ISecurityConstraint)object;
        return new EqualsBuilder()
            .append(this.getDisplayNames(), rhs.getDisplayNames())
            .append(this.getPortletCollection(), rhs.getPortletCollection())
            .append(this.getUserDataConstraint(), rhs.getUserDataConstraint())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(1509965201, -1626671541)
            .append(this.getDisplayNames())
            .append(this.getPortletCollection())
            .append(this.getUserDataConstraint())
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this).appendSuper(super.toString())
            .append("displayNames", this.getDisplayNames())
            .append("portletCollection", this.getPortletCollection())
            .append("userDataConstraint", this.getUserDataConstraint())
            .toString();
    }
}
