/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.container.services;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.portlet.RenderResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.lang.ArrayUtils;
import org.apache.pluto.PortletWindow;
import org.apache.pluto.spi.PortalCallbackService;
import org.apache.pluto.spi.PortletURLProvider;
import org.apache.pluto.spi.ResourceURLProvider;
import org.jasig.portal.portlet.IPlutoDomainObjectHelper;
import org.jasig.portal.portlet.IRedirectUrlConstructorsAccessor;
import org.jasig.portal.portlet.url.IPortletUrl;
import org.jasig.portal.portlet.url.IPortletUrlFactory;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.springframework.beans.factory.annotation.Required;

/**
 * Default implementation of <code>PortalCallbackServiceImpl</code> 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 1.0$
 */
public class PortalCallbackServiceImpl implements PortalCallbackService {
    private IPortletUrlFactory portletUrlFactory;
    private IRedirectUrlConstructorsAccessor redirectUrlConstructorAccessor;
    IPlutoDomainObjectHelper plutoDomainObjectHelper;
    
    /**
     * A property name that can be used to retrieve portlet window id
     * TODO: check if this is still necessary - I think it was only used by previous
     *       servant implementation 
     */
    public static final String WINDOW_ID_PARAM_NAME = "javax.portlet.window.id";
    
    public void addResponseProperty(HttpServletRequest req, PortletWindow portletWindow, String property, String value) {
        if (portletWindow == null) {
            throw new IllegalArgumentException("window cannot be null");
        }
        if (req == null) {
            throw new IllegalArgumentException("request cannot be null");
        }
        if(property==null) {
            throw new IllegalArgumentException("property cannot be null");
        }
        
        String key=getPropertiesAttributeKey(portletWindow);
        final HttpSession session = req.getSession();
        Map<String,String[]> savedProps = (Map<String,String[]>) session.getAttribute(key);
        if(savedProps==null) {
            savedProps=Collections.synchronizedMap(new HashMap<String,String[]>());
            session.setAttribute(key,savedProps);
        }
        String[] values=savedProps.get(property);
        if(values==null) {
            values=new String[] {value};
        } else {
            values=(String[]) ArrayUtils.add(values,value);
        }
        savedProps.put(property,values);
    }

    public PortletURLProvider getPortletURLProvider(HttpServletRequest req, PortletWindow portletWindow) {
        IPortletUrl portletUrl=portletUrlFactory.createPortletUrl(plutoDomainObjectHelper.getPortletWindow(portletWindow),req,false);
        return new PortletUrlProviderImpl(portletUrl,getWindowId(portletWindow),redirectUrlConstructorAccessor);
    }

    public Map getRequestProperties(HttpServletRequest req, PortletWindow portletWindow) {
        if (portletWindow == null) {
            throw new IllegalArgumentException("window cannot be null");
        }
        if (req == null) {
            throw new IllegalArgumentException("request cannot be null");
        }
        
        final HttpSession session = req.getSession();
        final Map<String,String[]> savedProps = (Map<String,String[]>) session.getAttribute(getPropertiesAttributeKey(portletWindow));

        final Map<String,String[]> properties;
        if (savedProps != null) {
            properties = new HashMap<String,String[]>(savedProps);
        }
        else {
            properties = new HashMap<String,String[]>();
        }
        
        // add dynamic values
        
        //Make sure the EXPIRATION_CACHE property is set to whatever the current
        //cache timeout for the portlet is. This is not a required property
        final String[] exprTime = properties.get(RenderResponse.EXPIRATION_CACHE);
        if (exprTime == null) {
            IPortletWindow window=plutoDomainObjectHelper.getPortletWindow(portletWindow);
            Integer expirationCache=window.getPortletEntity().getPortletDefinition().getPortletDeployment().getPortletExpirationCache();
            if(expirationCache!=null) {
                properties.put(RenderResponse.EXPIRATION_CACHE, new String[] {expirationCache.toString()});
            }
        }
        
        //Standard CGI variables that the portlet spec doesn't provide.
        properties.put("REMOTE_ADDR",  new String[] {req.getRemoteAddr()});
        properties.put("REQUEST_METHOD", new String[] {req.getMethod()});
        // Adding the portlet window ID for the portlet itself
        properties.put(WINDOW_ID_PARAM_NAME, new String[] {portletWindow.getId().toString()});
                        
        return properties;
    }

    public ResourceURLProvider getResourceURLProvider(HttpServletRequest req, PortletWindow portletWindow) {
        return new ResourceUrlProviderImpl(req);
    }

    public void setResponseProperty(HttpServletRequest req, PortletWindow portletWindow, String property, String value) {
        if (portletWindow == null)
            throw new IllegalArgumentException("portlet window cannot be null");
        if (req == null)
            throw new IllegalArgumentException("request cannot be null");
        if (property == null)
            throw new IllegalArgumentException("property cannot be null");

        final HttpSession session = req.getSession();
        String key=getPropertiesAttributeKey(portletWindow);
        Map<String,String[]> savedProps = (Map<String,String[]>) session.getAttribute(key);
        if(savedProps==null) {
            savedProps=Collections.synchronizedMap(new HashMap<String,String[]>());
            session.setAttribute(key,savedProps);
        }
        savedProps.put(property,new String[] {value});
        session.setAttribute(getPropertiesAttributeKey(portletWindow), savedProps);
        
    }

    public void setTitle(HttpServletRequest req, PortletWindow portletWindow, String title) {
        // title will be set on portlet render response instead
    }
    
    private PortletWindowId getWindowId(PortletWindow portletWindow) {
        return new PortletWindowId(portletWindow.getId().toString());
    }
    /**
     * Constructs and returns the attribute key of the properties for the given portlet window.
     */
    private String getPropertiesAttributeKey(PortletWindow window) {
       return PortalCallbackService.class.getName() + "_"  + window.getId().toString();
    }
    
    
    @Required
    public void setPortletUrlFactory(IPortletUrlFactory protletUrlFactory) {
        this.portletUrlFactory = protletUrlFactory;
    }

    @Required
    public void setPlutoDomainObjectHelper(IPlutoDomainObjectHelper plutoDomainObjectHelper) {
        this.plutoDomainObjectHelper = plutoDomainObjectHelper;
    }

    @Required
    public void setRedirectUrlConstructorAccessor(IRedirectUrlConstructorsAccessor redirectUrlConstructorAccessor) {
        this.redirectUrlConstructorAccessor = redirectUrlConstructorAccessor;
    }

}
