/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.container.services;

import java.util.Collections;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

import javax.portlet.PortalContext;
import javax.portlet.PortletMode;
import javax.portlet.WindowState;

/**
 * Implementation of Apache Pluto object model.
 * @author Michael Ivanov, mvi@immagic.coms
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 1386 $
 */
public class PortalContextImpl implements PortalContext {
    
    private Properties portalProperties = new Properties();
    private Set<PortletMode> supportedPortletModes = Collections.emptySet();
    private Set<WindowState> supportedWindowStates = Collections.emptySet();
    private String portalInfo;
    
    /**
     * Set the properties to use for the portal context. A defensive copy is
     * not made.
     * 
     * @param props The Properties to use.
     * @throws IllegalArgumentException If props is null.
     */
    public void setPortalProperties(Properties props) {
        if (props == null)
            throw new IllegalArgumentException("props cannot be null.");

        this.portalProperties = props;
    }
    
    /**
     * Sets the list of supported PortletModes.
     * 
     * @param modes A List of PortletMode names (String) to create PortletMode objects from.
     * @throws IllegalArgumentException If modes is null.
     */
    public void setSupportedPortletModeNames(Set<String> modes) {
        if (modes == null)
            throw new IllegalArgumentException("modes cannot be null");
        
        final Set<PortletMode> portletModes = new HashSet<PortletMode>(modes.size());
        for (final String modeName : modes) {
            portletModes.add(new PortletMode(modeName.toLowerCase()));
        }
        
        this.supportedPortletModes = Collections.unmodifiableSet(portletModes);
    }
    
    /**
     * Sets the list of supported WindowStates.
     * 
     * @param states A List of WindowState names (String) to create WindowState objects from.
     * @throws IllegalArgumentException If states is null.
     */
    public void setSupportedWindowStateNames(Set<String> states) {
        if (states == null)
            throw new IllegalArgumentException("states cannot be null");
        
        final Set<WindowState> windowStates = new HashSet<WindowState>(states.size());
        for (final String stateName : states) {
            windowStates.add(new WindowState(stateName.toLowerCase()));
        }
        
        this.supportedWindowStates = Collections.unmodifiableSet(windowStates);
    }
    
    /**
     * Sets the portal information string which will be provided to portlets
     * 
     * @param info The information string.
     * @throws IllegalArgumentException If info is null.
     */
    public void setPortalInfo(String info) {
        this.portalInfo = info;
    }

    // PortalContextProvider methods
    /**
     * @see org.apache.pluto.services.information.PortalContextProvider#getProperty(java.lang.String)
     * @throws IllegalArgumentException If name is null.
     */
    public String getProperty(String name) {
        if (name == null)
            throw new IllegalArgumentException("name cannot be null.");

        return (String)portalProperties.get(name);
    }

    public Enumeration getPropertyNames() {
        return Collections.enumeration(portalProperties.keySet());
    }

    public Enumeration getSupportedPortletModes() {
        return Collections.enumeration(supportedPortletModes);
    }

    public Enumeration getSupportedWindowStates() {
        return Collections.enumeration(supportedWindowStates);
    }

    /**
     * @see org.apache.pluto.services.information.PortalContextProvider#getPortalInfo()
     */
    public String getPortalInfo() {
        return this.portalInfo;
    }
}
