/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.dao;

import java.util.Set;

import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.transfer.IPortletEntityTransfer;
import org.jasig.portal.security.UserId;

/**
 * Portlet entity data access object.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public interface IPortletEntityDao {

    /**
     * Returns a portlet entity.
     * @param userId the user ID, may not be null.
     * @param portletEntityId the portlet entity object identifier, may note be null.
     * @return the portlet entity, null if not entity exists for the user id and entity id.
     */
    public IPortletEntityTransfer getPortletEntity(UserId userId, PortletEntityId portletEntityId);

    /**
     * Persists the portlet entity. It does not have to already exist.
     * @param userId the user ID, may not be null.
     * @param portletEntity the portlet entity, may not be null.
     */
    public void storePortletEntity(UserId userId, IPortletEntityTransfer portletEntity);
    
    /**
     * Removes the portlet entity.
     * @param userId the user ID, may not be null.
     * @param portletEntityId the portlet entity object identifier, may not be null., may not be null.
     */
    public void removePortletEntity(UserId userId, PortletEntityId portletEntityId);
    
    /**
     * Returns the portlet entities.  The set contains IPortletEntityTransfer objects.
     * @param userId the user ID, may not be null.
     * @return the portlet entities or empty collection.
     */
    public Set<IPortletEntityTransfer> getPortletEntities(UserId userId);
    
    /**
     * Returns the portlet entities for a given portlet application entity.  The set contains
     * IPortletEntityTransfer objects.
     * @param userId the user ID, may not be null.
     * @param portletApplicationEntityId the portlet application entity object identifier, may not be null.
     * @return the portlet entities or empty collection.
     */
    public Set<IPortletEntityTransfer> getPortletEntities(UserId userId, PortletApplicationEntityId portletApplicationEntityId);

    /**
     * Returns the portlet entities for a given portlet application entity and portlet definition.
     * The set contains IPortletEntityTransfer objects.
     * @param userId the user ID, may not be null.
     * @param portAppEntId the portlet application entity object identifier, may not be null.
     * @param portDefId the portlet definition object identifier, may not be null.
     * @return the portlet entities or empty collection.
     */
    public Set<IPortletEntityTransfer> getPortletEntities(UserId userId, PortletApplicationEntityId portAppEntId, PortletDefinitionId portDefId);
    
    /**
     * Creates a new minimal portlet entity and persists it.
     * @param userId the user ID, may not be null.
     * @param portletApplicationEntityId the portlet application entity identifier, may not be null.
     * @param portletDefinitionId the portlet definition identifer, may not be null.
     * @return the new portlet entity, may not be null.
     */
    public IPortletEntityTransfer createPortletEntity(UserId userId, PortletApplicationEntityId portletApplicationEntityId, PortletDefinitionId portletDefinitionId);

    /**
     * Removes from the persistent store all portlet entities related to the 
     * specified portlet application definition
     * 
     * @param applDefId identifier for a parent portlet application definition, 
     *   child portlet entities of which should be deleted. May not be null
     */
    public void removePortletEntities(PortletApplicationDefinitionId applDefId);

    /**
     * Removes from the persistent store all portlet entities related to the 
     * specified portlet application entity
     * 
     * @param applEntityId identifier for a parent portlet application entity, 
     *   child portlet entities of which should be deleted. May not be null
     */
    public void removePortletEntities(PortletApplicationEntityId applEntityId);

    /**
     * Removes from the persistent store all portlet entities related to the 
     * specified portlet definition
     * 
     * @param portletDefId identifier for a parent portlet definition, 
     *   child portlet entities of which should be deleted. May not be null
     */
    public void removePortletEntities(PortletDefinitionId portletDefId);
}
