/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Random;

import javax.sql.DataSource;

import org.apache.commons.lang.Validate;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.utils.IIncrementer;
import org.springframework.jdbc.core.SqlParameter;
import org.springframework.jdbc.object.MappingSqlQuery;
import org.springframework.jdbc.object.SqlUpdate;
import org.springframework.jdbc.support.incrementer.AbstractDataFieldMaxValueIncrementer;

/**
 * Provide a database independent incrementer used for id sequence generation
 * <br>Creation Date: May 20, 2004
 *
 * @author Mike DeSimone, mike@rsmart.com
 * @version $Revision: 11955 $
 */
public class IncrementerImpl extends AbstractDataFieldMaxValueIncrementer implements IIncrementer {
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private final Random rnd = new Random();
    private int minimumSleep = 10;
    private int maximumSleep = 500;
    private int collisionAttempts = 20;

    private boolean validated = false;
    private IncrementerValueQuery incrementerValue;
    private IncrementerUpdate updateQuery;
    private IncrementerCreate createQuery;
    private IncrementerUpdateForce updateForceQuery;

    /**
     * Determine the next key to use, as a long.
     *
     * @return the key to use as a long. It will eventually be converted later
     *         in another format by the public concrete methods of this class.
     */
    public synchronized long getNextKey() {
        this.afterPropertiesSet();

        final String incrName = getIncrementerName();
        Validate.notEmpty(incrName, "incrementerName can not be null or zero length");
        
        for (int attempt = 0; attempt < this.collisionAttempts; attempt++) {
            final Long oldId = (Long)this.incrementerValue.findObject(incrName);
            if (oldId == null) {
                // incrementer not in DB!
                throw new IllegalStateException("incrementer '" + incrName + "' does not exist");
            }
            
            final Long newId = oldId + 1;
        
            // try to increment and check rows udpated count
            int rows = this.updateQuery.update(newId, incrName, oldId);
            if (rows == 1) {
                return newId;
            }
            
            try {
                final int sleep = this.rnd.nextInt(this.maximumSleep - this.minimumSleep) + this.minimumSleep;
                Thread.sleep(sleep);
            }
            catch (InterruptedException ie) {
                this.logger.warn("Failed to sleep between increment generation attempts", ie);
            }
        }
        
        throw new RuntimeException("could not increment counter after " + this.collisionAttempts + " attempts");
    }

    /**
     * create a new incrementer with given starting value
     * it is an error to call this if the incrementer already exists
     * @param startVal value to set the incrementer to
     */
    public void createIncrementer(long startVal) {
        this.afterPropertiesSet();
        
        final String incrName = getIncrementerName();
        Validate.notEmpty(incrName, "incrementerName can not be null or zero length");
        
        this.createQuery.update(incrName, startVal);
    }

    /**
     * set incrementer to a new value in the database
     * WARNING: use with care
     *
     * @param newValue
     */
    public void setIncrementerValue(long newValue) {
        this.afterPropertiesSet();
        
        final String incrName = getIncrementerName();
        Validate.notEmpty(incrName, "incrementerName can not be null or zero length");
        
        if (!this.existsInDb()) {
            throw new IllegalStateException("cannot set incrementer value: incrementer does not exist");
        }
        
        this.updateForceQuery.update(newValue, incrName);
    }

    /**
     * Check to see if this incrementer exists
     * 
     * @return true If the incrementer exists, false if not.
     */
    public boolean existsInDb() {
        this.afterPropertiesSet();
        
        final String incrName = getIncrementerName();
        if (this.incrementerValue.findObject(incrName) != null) {
            return true;
        }
        return false;
    }

    public synchronized void afterPropertiesSet() {
        if (!this.validated) {
            super.afterPropertiesSet();
            
            // instantiate queries we use
            this.incrementerValue = new IncrementerValueQuery(getDataSource());
            this.updateQuery = new IncrementerUpdate(getDataSource());
            this.createQuery = new IncrementerCreate(getDataSource());
            this.updateForceQuery = new IncrementerUpdateForce(getDataSource());
            
            //Mark validated
            this.validated = true;
        }
    }
    
    
    

    protected static class IncrementerValueQuery extends MappingSqlQuery {
        IncrementerValueQuery(DataSource ds) {
            super(ds, "SELECT SEQ_VL FROM UP_SEQ WHERE SEQ_NM=?");
            super.declareParameter(new SqlParameter(Types.VARCHAR));
            compile();
        }

        protected Object mapRow(ResultSet rs, int rowNum) throws SQLException {
            final long l = rs.getLong(1);
            return Long.valueOf(l);
        }
    }

    /**
     * update the incrementer
     * Will fail if current value isn't what we expect (i.e., updated by someone else)
     */
    protected static class IncrementerUpdate extends SqlUpdate {

        IncrementerUpdate(DataSource ds) {
            super(ds, "UPDATE UP_SEQ SET SEQ_VL=? WHERE SEQ_NM=? AND SEQ_VL=?");
            super.declareParameter(new SqlParameter(Types.BIGINT));
            super.declareParameter(new SqlParameter(Types.VARCHAR));
            super.declareParameter(new SqlParameter(Types.BIGINT));
            compile();
        }

        protected int update(long newSequenceValue, String sequenceName, long oldSequenceValue) {
            final Object[] args = new Object[] { new Long(newSequenceValue), sequenceName, new Long(oldSequenceValue) };
            return update(args);
        }
    }

    /**
     * unconditionally update the incrementer
     */
    protected class IncrementerUpdateForce extends SqlUpdate {

        IncrementerUpdateForce(DataSource ds) {
            super(ds, "UPDATE UP_SEQ SET SEQ_VL=? WHERE SEQ_NM=?");
            super.declareParameter(new SqlParameter(Types.BIGINT));
            super.declareParameter(new SqlParameter(Types.VARCHAR));
            compile();
        }

        protected int update(long newSequenceValue, String sequenceName) {
            final Object[] args = new Object[] { new Long(newSequenceValue), sequenceName };
            return update(args);
        }
    }

    protected static class IncrementerCreate extends SqlUpdate {

        IncrementerCreate(DataSource ds) {
            super(ds, "INSERT INTO UP_SEQ (SEQ_VL, SEQ_NM) VALUES(?, ?)");
            super.declareParameter(new SqlParameter(Types.BIGINT));
            super.declareParameter(new SqlParameter(Types.VARCHAR));
            compile();
        }

        protected int update(String sequenceName, long startVal) {
            final Object[] args = new Object[] { new Long(startVal), sequenceName };
            return update(args);
        }
    }
}
