/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.dao.hibernate;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.dao.IPortletDefinitionDao;
import org.jasig.portal.portlet.dao.IPortletEntityDao;
import org.jasig.portal.portlet.dao.IPortletWindowDao;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.deployment.PortletDeploymentId;
import org.jasig.portal.portlet.transfer.IPortletDefinitionTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletDefinitionTransferImpl;
import org.jasig.portal.utils.IIncrementer;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

/**
 * Dao implementation of portlet definition dao using hibernate.
 * 
 * @author David Young, dyoung@unicon.net
 * @version $Revision: 11955 $
 */
public class HibernatePortletDefinitionDao extends HibernateDaoSupport implements IPortletDefinitionDao {
    private static final String QUERY_FIND_PORT_DEF_BY_ID = "FIND_PORT_DEF_BY_ID";
    private static final String QUERY_PARAM_PORT_DEF_ID = "portDefId";
    
    private static final String QUERY_FIND_PORT_DEF_BY_APPL_DEF_ID = "FIND_PORT_DEF_BY_APPL_DEF_ID";
    private static final String QUERY_PARAM_PORT_APP_DEF_ID = "portAppDefId";
    
    private static final String QUERY_FIND_PORT_DEF_BY_FNAME = "FIND_PORT_DEF_BY_FNAME";
    private static final String QUERY_PARAM_PORT_DEF_FNAME = "fName";
    
    private static final long UNPERSISTED_PREF_ID = -1;
    private static final long UNPERSISTED_PREF_VAL_ID = -1;
    
    
    

    private IIncrementer portletDefinitionIdIncrementer;
    private IIncrementer portletPreferenceIdIncrementer;
    private IIncrementer portletPreferenceValueIdIncrementer;
    private long unpersistedPrefId = UNPERSISTED_PREF_ID;
    private long unpersistedPrefValueId = UNPERSISTED_PREF_VAL_ID;
    private IPortletWindowDao portletWindowDao;
    private IPortletEntityDao portletEntityDao;

    public IIncrementer getPortletDefinitionIdIncrementer() {
        return portletDefinitionIdIncrementer;
    }

    public void setPortletDefinitionIdIncrementer(IIncrementer incrementer) {
        this.portletDefinitionIdIncrementer = incrementer;
    }
    
	public IIncrementer getPortletPreferenceIdIncrementer() {
		return portletPreferenceIdIncrementer;
	}

	public void setPortletPreferenceIdIncrementer(
			IIncrementer portletPreferenceIdIncrementer) {
		this.portletPreferenceIdIncrementer = portletPreferenceIdIncrementer;
	}

	public IPortletEntityDao getPortletEntityDao() {
        return portletEntityDao;
    }

    public void setPortletEntityDao(IPortletEntityDao portletEntityDao) {
        this.portletEntityDao = portletEntityDao;
    }

    public IPortletWindowDao getPortletWindowDao() {
        return portletWindowDao;
    }

    public void setPortletWindowDao(IPortletWindowDao portletWindowDao) {
        this.portletWindowDao = portletWindowDao;
    }

    /**
	 * Unpersisted IPreferences will have this value assigned
	 * as their current Id.
	 * 
	 * @return the value used to indicate unpersisted IPreferences
	 */
	public long getUnpersistedPrefId() {
		return unpersistedPrefId;
	}

	/**
	 * Assign the Id value used to indicate an unpersisted IPreference
	 * 
	 * @param unpersistedPrefId
	 */
	public void setUnpersistedPrefId(long unpersistedPrefId) {
		this.unpersistedPrefId = unpersistedPrefId;
	}

	public IIncrementer getPortletPreferenceValueIdIncrementer() {
        return portletPreferenceValueIdIncrementer;
    }

    public void setPortletPreferenceValueIdIncrementer(
            IIncrementer portletPreferenceValueIdIncrementer) {
        this.portletPreferenceValueIdIncrementer = portletPreferenceValueIdIncrementer;
    }

    public long getUnpersistedPrefValueId() {
        return unpersistedPrefValueId;
    }

    public void setUnpersistedPrefValueId(long unpersistedPrefValueId) {
        this.unpersistedPrefValueId = unpersistedPrefValueId;
    }

    public IPortletDefinitionTransfer getPortletDefinition(PortletDefinitionId portletDefinitionId) {
		if (portletDefinitionId == null) {
			throw new IllegalArgumentException("portletDefinitionId may not be null");
		}

        final HibernateTemplate template = this.getHibernateTemplate();
        final List portDefList = template.findByNamedQueryAndNamedParam(
            QUERY_FIND_PORT_DEF_BY_ID,
            QUERY_PARAM_PORT_DEF_ID,
            portletDefinitionId);
        
        final IPortletDefinitionTransfer portDefTrans;
        if (portDefList.size() > 0) {
            portDefTrans = (IPortletDefinitionTransfer)portDefList.get(0);
        }
        else {
            portDefTrans = null;
        }
        if (logger.isDebugEnabled()) {
            logger.debug("Retrieved (portDefId=" + portletDefinitionId + "): " + portDefTrans);
        }
        return portDefTrans;
	}
    
    public IPortletDefinitionTransfer getPortletDefinitionByFunctionalName(String functionalName) {
        if (functionalName == null) {
            throw new IllegalArgumentException("functionalName may not be null");
        }

        final HibernateTemplate template = this.getHibernateTemplate();
        final List portDefList = template.findByNamedQueryAndNamedParam(
            QUERY_FIND_PORT_DEF_BY_FNAME,
            QUERY_PARAM_PORT_DEF_FNAME,
            functionalName);
        
        final IPortletDefinitionTransfer portDefTrans;
        if (portDefList.size() > 0) {
            portDefTrans = (IPortletDefinitionTransfer)portDefList.get(0);
        }
        else {
            portDefTrans = null;
        }
        if (logger.isDebugEnabled()) {
            logger.debug("Retrieved (fName=" + functionalName + "): " + portDefTrans);
        }
        return portDefTrans;
    }

    @SuppressWarnings("unchecked")
    public Set<IPortletDefinitionTransfer> getPortletDefinitions(PortletApplicationDefinitionId portletApplicationDefinitionId) {
        if (portletApplicationDefinitionId == null) {
            throw new IllegalArgumentException("portletApplicationDefinitionId may not be null");
        }
        
        final HibernateTemplate template = this.getHibernateTemplate();
        final List<IPortletDefinitionTransfer> portDefList = template.findByNamedQueryAndNamedParam(
            QUERY_FIND_PORT_DEF_BY_APPL_DEF_ID,
            QUERY_PARAM_PORT_APP_DEF_ID,
            portletApplicationDefinitionId);
        final Set<IPortletDefinitionTransfer> portDefsTrans = new HashSet<IPortletDefinitionTransfer>(portDefList);
        
        if (logger.isDebugEnabled()) {
            logger.debug("Retrieved (portAppDefId=" + portletApplicationDefinitionId + "): " + portDefsTrans);
        }
        
        return portDefsTrans;
	}

    /**
     * Removes the portlet definition.
     * @param portletDefinitionId the portlet definition object identifier
     */
	public void removePortletDefinition(PortletDefinitionId portletDefinitionId) {
        if (portletDefinitionId == null) {
            throw new IllegalArgumentException("portletDefinitionId may not be null");
        }
        
        final IPortletDefinitionTransfer portDefTrans = getPortletDefinition(portletDefinitionId);
        
        if (portDefTrans != null) {
            
            portletWindowDao.removePortletWindows(portletDefinitionId);
            portletEntityDao.removePortletEntities(portletDefinitionId);
            
    	    if (portDefTrans.getPortletInfo().getKeywords() != null && portDefTrans.getPortletInfo().getKeywords().size() == 0) {
    		    portDefTrans.getPortletInfo().setKeywords(null);
    	    }
    	    
            final HibernateTemplate template = this.getHibernateTemplate();
            template.delete(portDefTrans);
            
            if (logger.isDebugEnabled()) {
                logger.debug("Removed (portDefId=" + portletDefinitionId + "): " + portletDefinitionId);
            }
        }
	}
    
    /**
     * Implemented to use ORM-style deletes, i.e. load up each portlet definition
     * and delete it using a Hibernate delete() call. This allows us to reuse
     * Hibernate mappings for cascaded delete of dependent objects, and should
     * be acceptable in large installations since portlet definitions are
     * globally scoped DOs (unlike per-user XxxEntity and PortletWindows).
     * 
     */
    public void removePortletDefinitions(PortletApplicationDefinitionId applDefId) {
        if ( applDefId == null ) {
            throw new IllegalArgumentException("applDefId may not be null");
        }
        
        Set<IPortletDefinitionTransfer> portletDefs = this.getPortletDefinitions(applDefId);
        for ( IPortletDefinitionTransfer portletDef : portletDefs ) {
            PortletDefinitionId portletDefId = portletDef.getPortletDefinitionId();
            portletWindowDao.removePortletWindows(portletDefId);
            portletEntityDao.removePortletEntities(portletDefId);
        }
        
        final HibernateTemplate template = this.getHibernateTemplate();
        template.deleteAll(portletDefs);
        
        if ( logger.isDebugEnabled() ) {
            logger.debug("Removed (portAppDefId=" + applDefId + "): " + portletDefs);
        }
    }

    /**
     * Persists the portlet definition.
     * @param portletDefinition the portlet definition
     */
	public void storePortletDefinition(IPortletDefinitionTransfer portletDefinition) {
        if (portletDefinition == null) {
            throw new IllegalArgumentException("portletDefinition may not be null");
        }
        
        final HibernateTemplate template = this.getHibernateTemplate();
        
        final IPreferences preferences = portletDefinition.getPreferences();
        if (preferences != null) {
            template.saveOrUpdate(preferences);
        }
        
        template.saveOrUpdate(portletDefinition);
        
        // explicitly delete prefs, otherwise will end up with orphans in the db
//        template.deleteAll(template.findByNamedQuery(QUERY_FIND_ORPHANED_PORT_PREFS));
        
        if (logger.isDebugEnabled()) {
            logger.debug("Stored (portDefId=" + portletDefinition.getPortletDefinitionId() + "): " + portletDefinition);
        }
	}

    

	public IPortletDefinitionTransfer createPortletDefinition(PortletApplicationDefinitionId portletApplicationDefintionId, PortletDeploymentId portletDeploymentId, String functionalName) {
        if (portletApplicationDefintionId == null) {
            throw new IllegalArgumentException("portletApplicationDefintionId may not be null");
        }
        if (portletDeploymentId == null) {
            throw new IllegalArgumentException("portletDeploymentId may not be null");
        }
        if (functionalName == null) {
            throw new IllegalArgumentException("functionalName may not be null");
        }
        
        final IPortletDefinitionTransfer portDefTrans = new PortletDefinitionTransferImpl();
        final long nextKey = this.portletDefinitionIdIncrementer.getNextKey();
        final PortletDefinitionId portDefId = new PortletDefinitionId(Long.toString(nextKey));
        
        portDefTrans.setPortletDefinitionId(portDefId);
        portDefTrans.setPortletApplicationDefinitionId(portletApplicationDefintionId);
        portDefTrans.setPortletDeploymentId(portletDeploymentId);
        portDefTrans.setFunctionalName(functionalName);
        
        this.storePortletDefinition(portDefTrans);
        
        return portDefTrans;
    }
}
