/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.dao.hibernate;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;

import org.hibernate.Query;
import org.jasig.portal.core.ObjectId;
import org.jasig.portal.portlet.dao.IPortletWindowDao;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.transfer.IPortletWindowTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletWindowTransferImpl;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.utils.IIncrementer;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

/**
 * Dao implementation of portlet window dao using hibernate.
 * 
 * @author David Young, dyoung@unicon.net
 * @version $Revision: 11955 $
 */
public class HibernatePortletWindowDao extends HibernateDaoSupport implements IPortletWindowDao {
    private static final String QUERY_FIND_PORT_WIN_BY_ID = "FIND_PORT_WIN_BY_ID";
    private static final String QUERY_PARAM_WIN_ID = "portWinId";

    private static final String QUERY_FIND_PORT_WIN_BY_ENT_ID = "FIND_PORT_WIN_BY_ENT_ID";
    private static final String QUERY_PARAM_ENT_ID = "portEntId";
    
    private static final String QUERY_DELETE_PORT_WINS_BY_APP_DEF_ID = "DELETE_PORT_WINS_BY_APP_DEF_ID";
    
    private static final String QUERY_DELETE_PORT_WINS_BY_APP_ENT_ID = "DELETE_PORT_WINS_BY_APP_ENT_ID";
    
    private static final String QUERY_DELETE_PORT_WINS_BY_DEF_ID = "DELETE_PORT_WINS_BY_DEF_ID";
    
    private static final String QUERY_DELETE_PORT_WINS_BY_ENT_ID = "DELETE_PORT_WINS_BY_ENT_ID";

    
    private IIncrementer portletWindowIdIncrementer;

    public IIncrementer getPortletWindowIdIncrementer() {
        return portletWindowIdIncrementer;
    }

    public void setPortletWindowIdIncrementer(IIncrementer incrementer) {
        this.portletWindowIdIncrementer = incrementer;
    }

	public IPortletWindowTransfer getPortletWindow(PortletWindowId portletWindowId) {
        if (portletWindowId == null) {
            throw new IllegalArgumentException("portletWindowId may not be null");
        }
        
        final HibernateTemplate template = this.getHibernateTemplate();
        final List portWins = template.findByNamedQueryAndNamedParam(
            QUERY_FIND_PORT_WIN_BY_ID,
            QUERY_PARAM_WIN_ID,
            portletWindowId);
    
        final IPortletWindowTransfer portWinTrans = (IPortletWindowTransfer)DataAccessUtils.uniqueResult(portWins);
        if (logger.isDebugEnabled()) {
            logger.debug("Retrieved (portWinId=" + portletWindowId + "): " + portWinTrans);
        }
        return portWinTrans;
	}

	@SuppressWarnings("unchecked")
    public Set<IPortletWindowTransfer> getPortletWindows(PortletEntityId portletEntityId) {
        if (portletEntityId == null) {
            throw new IllegalArgumentException("portletEntityId may not be null");
        }
        
        final HibernateTemplate template = this.getHibernateTemplate();
        final List<IPortletWindowTransfer> portWins = template.findByNamedQueryAndNamedParam(
            QUERY_FIND_PORT_WIN_BY_ENT_ID,
            QUERY_PARAM_ENT_ID,
            portletEntityId);
    
        final Set<IPortletWindowTransfer> portWinsTrans = new HashSet<IPortletWindowTransfer>(portWins);
        if (logger.isDebugEnabled()) {
            logger.debug("Retrieved (portEntId=" + portletEntityId + "): " + portWinsTrans);
        }
        return portWinsTrans;
	}

	public IPortletWindowTransfer createPortletWindow(PortletEntityId portletEntityId) {
        if (portletEntityId == null) {
            throw new IllegalArgumentException("portletEntityId may not be null");
        }
        
        final IPortletWindowTransfer portWinTrans = new PortletWindowTransferImpl();
        final long nextKey = this.portletWindowIdIncrementer.getNextKey();
        final PortletWindowId portWinId = new PortletWindowId(Long.toString(nextKey));
        
        portWinTrans.setPortletWindowId(portWinId);
        portWinTrans.setPortletEntityId(portletEntityId);
        portWinTrans.setPortletMode(PortletMode.VIEW);
        portWinTrans.setWindowState(WindowState.NORMAL);
        
        this.storePortletWindow(portWinTrans);

        return portWinTrans;
	}

	public void removePortletWindow(PortletWindowId portletWindowId) {
        if (portletWindowId == null) {
            throw new IllegalArgumentException("portletWindowId may not be null");
        }
        
        final IPortletWindowTransfer portWinTrans = this.getPortletWindow(portletWindowId);
        
        if (portWinTrans != null) {
            final HibernateTemplate template = this.getHibernateTemplate();
            template.delete(portWinTrans);
            
            if (logger.isDebugEnabled()) {
                logger.debug("Removed (portWinId=" + portletWindowId + "): " + portWinTrans);
            }
        }
	}

	public void storePortletWindow(IPortletWindowTransfer portletWindow) {
        if (portletWindow == null) {
            throw new IllegalArgumentException("portletWindow may not be null");
        }
        
        final HibernateTemplate template = this.getHibernateTemplate();
        template.saveOrUpdate(portletWindow);
        
        if (logger.isDebugEnabled()) {
            logger.debug("Stored (portWinId=" + portletWindow.getPortletWindowId() + "): " + portletWindow);
        }
	}

    public void removePortletWindows(PortletApplicationDefinitionId applDefId) {
        bulkUpdateByObjectId(QUERY_DELETE_PORT_WINS_BY_APP_DEF_ID, applDefId);
    }

    public void removePortletWindows(PortletApplicationEntityId applEntityId) {
        bulkUpdateByObjectId(QUERY_DELETE_PORT_WINS_BY_APP_ENT_ID, applEntityId);
    }

    public void removePortletWindows(PortletDefinitionId portletDefId) {
        bulkUpdateByObjectId(QUERY_DELETE_PORT_WINS_BY_DEF_ID, portletDefId);
    }

    public void removePortletWindows(PortletEntityId portletEntityId) {
        bulkUpdateByObjectId(QUERY_DELETE_PORT_WINS_BY_ENT_ID, portletEntityId);
    }
    
    /**
     * Performs a bulk update operation using a named query and an
     * ObjectId parameter.
     * 
     * @param queryName
     * @param fk
     */
    private void bulkUpdateByObjectId(String queryName, ObjectId id) {
        
        final HibernateTemplate template = this.getHibernateTemplate();
        
        Query query = template.getSessionFactory()
            .getCurrentSession()
            .getNamedQuery(queryName);
        String queryString = query.getQueryString();
        int result = template.bulkUpdate(queryString, id);
        
        if ( logger.isDebugEnabled() ) {
            logger.debug("Performed bulk update by object key [object key type: " 
                    + id.getClass() + "][foreign key value: " 
                    + id + "][instances deleted: " + result + 
                    "][query: " + queryString + "]");
        }
    }
}
