/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.definition;

import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.common.BaseLocalizedBean;
import org.jasig.portal.portlet.common.IContentType;
import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.common.IInitParameter;
import org.jasig.portal.portlet.common.IPortletInfo;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.common.ISecurityRoleReference;
import org.jasig.portal.portlet.deployment.IPortletDeployment;
import org.jasig.portal.portlet.deployment.PortletDeploymentId;
import org.jasig.portal.portlet.transfer.IPortletDefinitionTransfer;
import org.jasig.portal.portlet.window.IPortletWindow;


/**
 * Localized portlet definition. Wraps a {@link IPortletDefinition}
 * with a specified locale. The specified locale is used by the
 * {@link ILocalizedPortletDefinition} methods when they are called.
 * <br>
 * <br>
 * There is a default locale defined of Locale.ENGLISH. If specified 
 * in the constructor the localized methods will fall back to the default
 * locale if no value is found for the configured locale.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class LocalizedPortletDefinitionImpl extends BaseLocalizedBean implements ILocalizedPortletDefinition {
    private final IPortletDefinition portletDefinition;
    
    
    public LocalizedPortletDefinitionImpl(IPortletDefinition portletDefinition, ILocaleManager localeManager) {
        this(portletDefinition, localeManager, null);
    }
    
    public LocalizedPortletDefinitionImpl(IPortletDefinition portletDefinition, ILocaleManager localeManager, IPortletWindow portWin) {
        super(localeManager, portWin);

        if (portletDefinition == null) {
            throw new IllegalArgumentException("portletDefinition may not be null");
        }

        this.portletDefinition = portletDefinition;
    }

    //********** Impls fopr ILocalizedPortletDefintion **********\\
    
    public String getLocalDisplayName() {
        final Map<Locale, IDisplayName> displayNames = this.portletDefinition.getDisplayNames();
        final IDisplayName displayName = this.resolveLocalDisplayName(displayNames);
        
        if (displayName != null) {
            return displayName.getDisplayNameText();
        }
        else {
            return null;
        }
    }
    
    public String getLocalDescription() {
        final Map<Locale, IDescription> descriptions = this.portletDefinition.getDescriptions();
        final IDescription description = this.resolveLocalDescription(descriptions);
        
        if (description != null) {
            return description.getDescriptionText();
        }
        else {
            return null;
        }
    }
    
    //********** Delegates for IPortletDefintion **********\\
    
    public Map<Locale, IDescription> getDescriptions() {
        return this.portletDefinition.getDescriptions();
    }

    public Map<Locale, IDisplayName> getDisplayNames() {
        return this.portletDefinition.getDisplayNames();
    }

    public String getFunctionalName() {
        return this.portletDefinition.getFunctionalName();
    }

    public Map<String, IInitParameter> getInitParameters() {
        return this.portletDefinition.getInitParameters();
    }

    public PortletDefinitionId getPortletDefinitionId() {
        return this.portletDefinition.getPortletDefinitionId();
    }

    public PortletApplicationDefinitionId getPortletApplicationDefinitionId() {
        return this.portletDefinition.getPortletApplicationDefinitionId();
    }

    public String getPortletClassName() {
        return this.portletDefinition.getPortletClassName();
    }

    public IPortletDefinitionTransfer getPortletDefinitionTransfer() {
        return this.portletDefinition.getPortletDefinitionTransfer();
    }

    public PortletDeploymentId getPortletDeploymentId() {
        return this.portletDefinition.getPortletDeploymentId();
    }

    public Integer getPortletExpirationCache() {
        return this.portletDefinition.getPortletExpirationCache();
    }

    public IPortletInfo getPortletInfo() {
        return this.portletDefinition.getPortletInfo();
    }

    public String getPortletName() {
        return this.portletDefinition.getPortletName();
    }

    public IPreferences getPreferences() {
        return this.portletDefinition.getPreferences();
    }

    public long getRenderTimeOut() {
        return this.portletDefinition.getRenderTimeOut();
    }

    public Map<String, ISecurityRoleReference> getSecurityRoleReferences() {
        return this.portletDefinition.getSecurityRoleReferences();
    }

    public Map<String, IContentType> getSupportedContentTypes() {
        return this.portletDefinition.getSupportedContentTypes();
    }

    public Set<Locale> getSupportedLocales() {
        return this.portletDefinition.getSupportedLocales();
    }

    public IPortletDeployment getPortletDeployment() {
        return this.portletDefinition.getPortletDeployment();
    }
}
