/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.definition;

import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.jasig.portal.portlet.common.IContentType;
import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.common.IInitParameter;
import org.jasig.portal.portlet.common.IPortletInfo;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.common.ISecurityRoleReference;
import org.jasig.portal.portlet.common.LayeredPortletInfo;
import org.jasig.portal.portlet.common.LayeredPreferences;
import org.jasig.portal.portlet.deployment.IPortletDeployment;
import org.jasig.portal.portlet.deployment.PortletDeploymentId;
import org.jasig.portal.portlet.transfer.IPortletDefinitionTransfer;
import org.jasig.portal.utils.LayeredMap;

/**
 * Portlet definition implementation.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public class PortletDefinitionImpl implements IPortletDefinitionControl {
    private final IPortletDefinitionTransfer portletDefinitionTransfer;
    private final IPortletDeployment portletDeployment;
    
    
    public PortletDefinitionImpl(IPortletDefinitionTransfer portletDefinitionTransfer,  
                                 IPortletDeployment portletDeployment) {
        
        if (portletDefinitionTransfer == null) {
            throw new IllegalArgumentException("portletDefinitionTransfer may not be null");
        }
        if (portletDeployment == null) {
            throw new IllegalArgumentException("portletDeployment may not be null");
        }
        if (!portletDefinitionTransfer.getPortletDeploymentId().equals(portletDeployment.getPortletDeploymentId())) {
            throw new IllegalArgumentException("portletDefinitionTransfer.getPortletDeploymentId() must equal portletDeployment.getPortletDeploymentId(). '" + portletDefinitionTransfer.getPortletDeploymentId() + "' != '" + portletDeployment.getPortletDeploymentId() + "'");
        }
        
        this.portletDefinitionTransfer = portletDefinitionTransfer;
        this.portletDeployment = portletDeployment;
    }


    // IPortletDefinitionTransfer methods
    
    public PortletDefinitionId getPortletDefinitionId() {
        return this.portletDefinitionTransfer.getPortletDefinitionId();
    }
    
    public void setPortletDefinitionId(PortletDefinitionId id) {
        this.portletDefinitionTransfer.setPortletDefinitionId(id);
    }
    
    public String getFunctionalName() {
        return this.portletDefinitionTransfer.getFunctionalName();
    }

    public void setFunctionalName(String functionalName) {
        this.portletDefinitionTransfer.setFunctionalName(functionalName);
    }

    public void setPreferences(IPreferences preferences) {
        final IPreferences layeredPrefs = this.getPreferences();
        layeredPrefs.addPreferences(preferences.getPreferences());
    }
    
    public void setPortletInfo(IPortletInfo portletInfo) {
        final IPortletInfo layeredInfo = this.getPortletInfo();
        layeredInfo.setTitle(portletInfo.getTitle());
        layeredInfo.setShortTitle(portletInfo.getShortTitle());
        layeredInfo.setKeywords(portletInfo.getKeywords());
    }
    
    public void setDescriptions(Map<Locale, IDescription> descriptions) {
        final Map<Locale, IDescription> layeredDescs = this.getDescriptions();
        layeredDescs.putAll(descriptions);
    }

    public void setDisplayNames(Map<Locale, IDisplayName> displayNames) {
        final Map<Locale, IDisplayName> layeredNames = this.getDisplayNames();
        layeredNames.putAll(displayNames);
    }
    
    public void setRenderTimeOut(long renderTimeOut) {
        this.portletDefinitionTransfer.setRenderTimeOut(renderTimeOut);
    }

    public IPortletDefinitionTransfer getPortletDefinitionTransfer() {
        return this.portletDefinitionTransfer;
    }

    public PortletDeploymentId getPortletDeploymentId() {
        return this.portletDefinitionTransfer.getPortletDeploymentId();
    }
    
    public IPortletDeployment getPortletDeployment() {
        return this.portletDeployment;
    }
    
    public long getRenderTimeOut() {
        return this.portletDefinitionTransfer.getRenderTimeOut();
    }

    public PortletApplicationDefinitionId getPortletApplicationDefinitionId() {
        return this.portletDefinitionTransfer.getPortletApplicationDefinitionId();
    }
    
    public void setPortletApplicationDefinitionId(PortletApplicationDefinitionId portletApplicationDefinitionId) {
        this.portletDefinitionTransfer.setPortletApplicationDefinitionId(portletApplicationDefinitionId);
    }


    // IPortletApplicationDeploymentInformationProvider methods

    
    public Map<Locale, IDescription> getDescriptions() {
        final Map<Locale, IDescription> defDescs = this.portletDefinitionTransfer.getDescriptions();
        final Map<Locale, IDescription> depDescs = this.portletDeployment.getDescriptions();
        return new LayeredMap<Locale, IDescription>(defDescs, depDescs);
    }

    public String getPortletName() {
        return this.portletDeployment.getPortletName();
    }

    public Map<Locale, IDisplayName> getDisplayNames() {
        final Map<Locale, IDisplayName> defNames = this.portletDefinitionTransfer.getDisplayNames();
        final Map<Locale, IDisplayName> depNames = this.portletDeployment.getDisplayNames();
        return new LayeredMap<Locale, IDisplayName>(defNames, depNames);
    }

    public String getPortletClassName() {
        return this.portletDeployment.getPortletClassName();
    }

    public Map<String, IInitParameter> getInitParameters() {
        return this.portletDeployment.getInitParameters();
    }

    public Integer getPortletExpirationCache() {
        return this.portletDeployment.getPortletExpirationCache();
    }

    public Map<String, IContentType> getSupportedContentTypes() {
        return this.portletDeployment.getSupportedContentTypes();
    }

    public Set<Locale> getSupportedLocales() {
        return this.portletDeployment.getSupportedLocales();
    }

    public IPortletInfo getPortletInfo() {
        final IPortletInfo defInfo = this.portletDefinitionTransfer.getPortletInfo();
        final IPortletInfo depInfo = this.portletDeployment.getPortletInfo();
        return new LayeredPortletInfo(defInfo, depInfo);
    }
    
    public IPreferences getPreferences() {
        final IPreferences defPrefs = this.portletDefinitionTransfer.getPreferences();
        final IPreferences depPrefs = this.portletDeployment.getPreferences();
    	return new LayeredPreferences(defPrefs, depPrefs);
    }

    public Map<String, ISecurityRoleReference> getSecurityRoleReferences() {
        return this.portletDeployment.getSecurityRoleReferences();
    }
    

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (!(obj instanceof IPortletDefinition)) {
            return false;
        }
        if (this == obj) {
            return true;
        }
        
        IPortletDefinition other = (IPortletDefinition)obj;
        return new EqualsBuilder()
            .append(this.portletDefinitionTransfer, other.getPortletDefinitionTransfer())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return this.portletDefinitionTransfer.hashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
        .append("portletDefinitionTransfer", this.portletDefinitionTransfer)
        .toString();
    }
}