/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.deployment;

import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.portlet.common.IContentType;
import org.jasig.portal.portlet.common.IDescribable;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.common.IInitParameter;
import org.jasig.portal.portlet.common.IPortletInfo;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.common.ISecurityRoleReference;

/**
 * Knows how to supply basic portlet information
 * as specified in the portlet application deployment
 * descriptor (portlet.xml) file.
 * @author Michael Ivanov, mivanov@unicon.net
 * @version $Revision: 11911 $
 */
public interface IPortletDeploymentInformationProvider extends IDescribable {
    
    /**
     * Leaves timeout value to the framework
     */
    public static final long UNDEFINED_TIMEOUT=-1;
    /**
     * Specifies that no timeout checking should be performed
     */
    public static final long INFINITE_TIMEOUT=-2;

    /**
     * Returns the canonical portlet name that is
     * unique within the portlet application.
     * The return value cannot be NULL.
     * @return the canonical portlet name
     */
    public String getPortletName();

    /**
     * Returns the display names of this portlet.
     * The return value may not be null.
     * @return display names for this portlet
     */
    public Map<Locale, IDisplayName> getDisplayNames();

    /**
     * Returns the portlet class name.
     * The return value cannot be NULL.
     * @return the portlet class name
     */
    public String getPortletClassName();
      
    /**
     * Returns all parameters of this portlet.
     * The return value may not be null.
     * @return the parameters of this portlet
     */
    public Map<String, IInitParameter> getInitParameters();

    /**
     * Returns duration (in seconds) of expiration cache.
     * The return value may be NULL.
     * @return duration of expiration cache in seconds
     */
    public Integer getPortletExpirationCache();

    /**
     * Returns all content types and related portlet modes
     * supported by this portlet.
     * Content types are referred to as mime types in the
     * portlet application deployment descriptor.
     * The return value cannot be NULL.
     * @return the content types supported by this portlet, the keys are the mime/type string.
     */
    public Map<String, IContentType> getSupportedContentTypes();

    /**
     * Returns a set of locales supported by this portlet
     * The return value may not be null.
     * @return the set of locales supported by this portlet
     */
    public Set<Locale> getSupportedLocales();

    /**
     * Returns the portlet information which contains
     * title, short title, and keyword information.
     * The return value cannot be NULL.
     * @return the portlet metadata information
     */
    public IPortletInfo getPortletInfo();
    
    /**
     * Returns all preferences of this portlet
     * The return value may not be null.
     * @return the preference set
     */
    public IPreferences getPreferences();

    /**
     * Returns all security role refs of this portlet.
     * @return all security role refs of this portlet
     */
    public Map<String, ISecurityRoleReference> getSecurityRoleReferences();
}
