/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.deployment;

import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.common.BaseLocalizedBean;
import org.jasig.portal.portlet.common.IContentType;
import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.common.IInitParameter;
import org.jasig.portal.portlet.common.IPortletInfo;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.common.ISecurityRoleReference;
import org.jasig.portal.portlet.definition.ILocalizedPortletDeployment;
import org.jasig.portal.portlet.transfer.IPortletDeploymentTransfer;
import org.jasig.portal.portlet.window.IPortletWindow;


/**
 * Localized portlet deployment. Wraps a {@link IPortletDeployment}
 * with a specified locale. The specified locale is used by the
 * {@link ILocalizedPortletDeployment} methods when they are called.
 * <br>
 * <br>
 * There is a default locale defined of Locale.ENGLISH. If specified 
 * in the constructor the localized methods will fall back to the default
 * locale if no value is found for the configured locale.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class LocalizedPortletDeploymentImpl extends BaseLocalizedBean implements ILocalizedPortletDeployment {
    private final IPortletDeployment portletDeployment;
    
    public LocalizedPortletDeploymentImpl(IPortletDeployment portletDeployment, ILocaleManager localeManager) {
        this(portletDeployment, localeManager, null);
    }
    
    public LocalizedPortletDeploymentImpl(IPortletDeployment portletDeployment, ILocaleManager localeManager, IPortletWindow portWin) {
        super(localeManager, portWin);

        if (portletDeployment == null) {
            throw new IllegalArgumentException("portletDeployment may not be null");
        }

        this.portletDeployment = portletDeployment;
    }

    //********** Impls fopr ILocalizedPortletDeployment **********\\
    
    public String getLocalDisplayName() {
        final Map<Locale, IDisplayName> displayNames = this.portletDeployment.getDisplayNames();
        final IDisplayName displayName = this.resolveLocalDisplayName(displayNames);
        
        if (displayName != null) {
            return displayName.getDisplayNameText();
        }
        else {
            return null;
        }
    }
    
    public String getLocalDescription() {
        final Map<Locale, IDescription> descriptions = this.portletDeployment.getDescriptions();
        final IDescription description = this.resolveLocalDescription(descriptions);
        
        if (description != null) {
            return description.getDescriptionText();
        }
        else {
            return null;
        }
    }

    
    //********** Delegates for IPortletDeployment **********\\

    public Map<Locale, IDescription> getDescriptions() {
        return this.portletDeployment.getDescriptions();
    }

    public Map<Locale, IDisplayName> getDisplayNames() {
        return this.portletDeployment.getDisplayNames();
    }

    public Map<String, IInitParameter> getInitParameters() {
        return this.portletDeployment.getInitParameters();
    }

    public PortletDeploymentId getPortletDeploymentId() {
        return this.portletDeployment.getPortletDeploymentId();
    }

    public PortletApplicationDeploymentId getPortletApplicationDeploymentId() {
        return this.portletDeployment.getPortletApplicationDeploymentId();
    }

    public String getPortletClassName() {
        return this.portletDeployment.getPortletClassName();
    }

    public Integer getPortletExpirationCache() {
        return this.portletDeployment.getPortletExpirationCache();
    }

    public IPortletInfo getPortletInfo() {
        return this.portletDeployment.getPortletInfo();
    }

    public String getPortletName() {
        return this.portletDeployment.getPortletName();
    }

    public IPreferences getPreferences() {
        return this.portletDeployment.getPreferences();
    }

    public Map<String, ISecurityRoleReference> getSecurityRoleReferences() {
        return this.portletDeployment.getSecurityRoleReferences();
    }

    public Map<String, IContentType> getSupportedContentTypes() {
        return this.portletDeployment.getSupportedContentTypes();
    }

    public Set<Locale> getSupportedLocales() {
        return this.portletDeployment.getSupportedLocales();
    }

    public IPortletDeploymentTransfer getPortletDeploymentTransfer() {
        return this.portletDeployment.getPortletDeploymentTransfer();
    }
}
    
    

