/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.deployment;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.pluto.descriptors.portlet.PortletPreferenceDD;
import org.apache.pluto.descriptors.portlet.PortletPreferencesDD;
import org.jasig.portal.portlet.common.IPreference;
import org.jasig.portal.portlet.common.IPreferences;

/**
 * A IPreferences implementation that wraps the pluto PortletPreferencesDD object.
 * 
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public class PlutoPortletPreferences implements IPreferences {
    private final PortletPreferencesDD portletPreferencesDD;
    
    public PlutoPortletPreferences(PortletPreferencesDD portletPreferencesDD) {
        if (portletPreferencesDD == null) {
            throw new IllegalArgumentException("PortletPreferencesDD can not be null");
        }
        
        this.portletPreferencesDD = portletPreferencesDD;
    }

    
    /**
     * @see org.jasig.portal.portlet.common.IPreferences#getId()
     */
    public long getId() {
        return this.portletPreferencesDD.hashCode();
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreferences#setId(long)
     */
    public void setId(long id) {
        throw new UnsupportedOperationException("PlutoPortletPreferences are read only");
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreferences#addPreference(org.jasig.portal.portlet.common.IPreference)
     */
    public void addPreference(IPreference preference) {
        throw new UnsupportedOperationException("PlutoPortletPreferences are read only");
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreferences#addPreferences(java.util.List)
     */
    public void addPreferences(List preferences) {
        throw new UnsupportedOperationException("PlutoPortletPreferences are read only");
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreferences#getPreference(java.lang.String)
     */
    @SuppressWarnings("unchecked")
    public IPreference getPreference(String preferenceName) {
        if (preferenceName == null) {
            throw new IllegalArgumentException("preferenceName can not be null");
        }
        
        final List<PortletPreferenceDD> portletPreferences = this.portletPreferencesDD.getPortletPreferences();
        for (final PortletPreferenceDD plutoPortletPreference : portletPreferences) {
            if (preferenceName.equals(plutoPortletPreference.getName())) {
                final PlutoPortletPreference upPortletPreference = new PlutoPortletPreference(plutoPortletPreference);
                return upPortletPreference;
            }
        }

        return null;
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreferences#getPreferences()
     */
    @SuppressWarnings("unchecked")
    public List<IPreference> getPreferences() {
        final List<PortletPreferenceDD> plutoPortletPreferences = this.portletPreferencesDD.getPortletPreferences();
        final List<IPreference> upPortletPreferences = new ArrayList<IPreference>(plutoPortletPreferences.size());
        for (final PortletPreferenceDD plutoPortletPreference : plutoPortletPreferences) {
            final PlutoPortletPreference upPortletPreference = new PlutoPortletPreference(plutoPortletPreference);
            upPortletPreferences.add(upPortletPreference);
        }
    
        return Collections.unmodifiableList(upPortletPreferences);
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreferences#getPreferencesValidatorClassName()
     */
    public String getPreferencesValidatorClassName() {
        return this.portletPreferencesDD.getPreferencesValidator();
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreferences#removePreference(java.lang.String)
     */
    public IPreference removePreference(String preferenceName) {
        throw new UnsupportedOperationException("PlutoPortletPreferences are read only");
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreferences#setPreferences(java.util.List)
     */
    public void setPreferences(List<IPreference> preferences) {
        throw new UnsupportedOperationException("PlutoPortletPreferences are read only");
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreferences#setPreferencesValidatorClassName(java.lang.String)
     */
    public void setPreferencesValidatorClassName(String preferencesValidatorClassName) {
        throw new UnsupportedOperationException("PlutoPortletPreferences are read only");
    }
    
    /**
     * @see org.jasig.portal.portlet.common.IPreferences#getPreferencesMap()
     */
    @SuppressWarnings("unchecked")
    public Map<String, IPreference> getPreferencesMap() {
        final List<PortletPreferenceDD> plutoPortletPreferences = portletPreferencesDD.getPortletPreferences();
        final Map<String, IPreference> preferencesMap = new HashMap<String, IPreference>(plutoPortletPreferences.size());
        
        for (final PortletPreferenceDD plutoPortletPreference : plutoPortletPreferences) {
            final PlutoPortletPreference upPortletPreference = new PlutoPortletPreference(plutoPortletPreference);
            preferencesMap.put(upPortletPreference.getName(), upPortletPreference);
        }
        
        return Collections.unmodifiableMap(preferencesMap);
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreferences#setPreferencesMap(java.util.Map)
     */
    public void setPreferencesMap(Map<String, IPreference> preferences) {
        throw new UnsupportedOperationException("PlutoPortletPreferences are read only");
    }


    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof IPreferences)) {
            return false;
        }
        IPreferences rhs = (IPreferences)object;
        return new EqualsBuilder()
            .append(this.getPreferences(), rhs.getPreferences())
            .append(this.getPreferencesValidatorClassName(), rhs.getPreferencesValidatorClassName())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(1459015719, -340708575)
            .append(this.getPreferences())
            .append(this.getPreferencesValidatorClassName())
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this).appendSuper(super.toString())
            .append("preferences", this.getPreferences())
            .append("preferencesValidatorClassName", this.getPreferencesValidatorClassName())
            .toString();
    }
}
