/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.entity;

import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.jasig.portal.portlet.common.IContentType;
import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.common.IInitParameter;
import org.jasig.portal.portlet.common.IPortletInfo;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.common.ISecurityRoleReference;
import org.jasig.portal.portlet.common.LayeredPreferences;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.deployment.IPortletDeploymentInformationProvider;
import org.jasig.portal.portlet.registry.general.IPortletWindowRegistry;
import org.jasig.portal.portlet.transfer.IPortletEntityTransfer;
import org.jasig.portal.security.UserId;


/**
 * Portlet entity implementation.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public class PortletEntityImpl implements IPortletEntityControl {
    private IPortletEntityTransfer portletEntityTransfer;
    private IPortletDefinition portletDefinition;
    
    private final IPortletWindowRegistry portletWindowRegistry;
    
    public PortletEntityImpl (IPortletEntityTransfer portletEntityTransfer,
                              IPortletDefinition portletDefinition,
                              IPortletWindowRegistry portletWindowRegistry) {
        
        if (portletEntityTransfer == null) {
            throw new IllegalArgumentException("portletEntityTransfer may not be null");
        }
        if (portletDefinition == null) {
            throw new IllegalArgumentException("portletDefinition may not be null");
        }
        if (portletWindowRegistry == null) {
            throw new IllegalArgumentException("portletWindowRegistry may not be null");
        }
        if (!portletEntityTransfer.getPortletDefinitionId().equals(portletDefinition.getPortletDefinitionId())) {
            throw new IllegalArgumentException("portletEntityTransfer.getPortletDefinitionId() must equal portletDefinition.getPortletDefinitionId(). '" + portletEntityTransfer.getPortletDefinitionId() + "' != '" + portletDefinition.getPortletDefinitionId() + "'");
        }
        
        this.portletEntityTransfer = portletEntityTransfer;
        this.portletDefinition = portletDefinition;
        this.portletWindowRegistry = portletWindowRegistry;
    }

    
    
    //********** IPortletEntity Methods **********\\
    
    public PortletApplicationEntityId getPortletApplicationEntityId() {
        return this.portletEntityTransfer.getPortletApplicationEntityId();
    }

    public PortletDefinitionId getPortletDefinitionId() {
        return this.portletEntityTransfer.getPortletDefinitionId();
    }
    
    public UserId getUserId() {
        return this.portletEntityTransfer.getUserId();
    }

    public PortletEntityId getPortletEntityId() {
        return this.portletEntityTransfer.getPortletEntityId();
    }

    public Set getPortletWindows() {
        return this.portletWindowRegistry.getPortletWindows(this.getPortletEntityId());
    }

    public IPortletEntityTransfer getPortletEntityTransfer() {
        return this.portletEntityTransfer;
    }
    
    
    //********** IPortletEntityControl Methods **********\\

    public void setPortletEntityId(PortletEntityId portEntId) {
        this.portletEntityTransfer.setPortletEntityId(portEntId);
    }

    public void setPreferences(IPreferences preferences) {
        final IPreferences layeredPrefs = this.getPreferences();
        layeredPrefs.addPreferences(preferences.getPreferences());
    }
    
    
    //********** IPortletDeploymentInformationProvider Methods **********\\

    public Map<Locale, IDescription> getDescriptions() {
        return this.portletDefinition.getDescriptions();
    }

    public Map<Locale, IDisplayName> getDisplayNames() {
        return this.portletDefinition.getDisplayNames();
    }

    public Map<String, IInitParameter> getInitParameters() {
        return this.portletDefinition.getInitParameters();
    }

    public String getPortletClassName() {
        return this.portletDefinition.getPortletClassName();
    }

    public Integer getPortletExpirationCache() {
        return this.portletDefinition.getPortletExpirationCache();
    }

    public IPortletInfo getPortletInfo() {
        return this.portletDefinition.getPortletInfo();
    }

    public String getPortletName() {
        return this.portletDefinition.getPortletName();
    }

    public IPreferences getPreferences() {
        final IPreferences entPrefs = this.portletEntityTransfer.getPreferences();
        final IPreferences defPrefs = this.portletDefinition.getPreferences();
        return new LayeredPreferences(entPrefs, defPrefs);
    }

    public long getRenderTimeOut() {
        long timeOut=portletEntityTransfer.getRenderTimeOut();
        if(timeOut==IPortletDeploymentInformationProvider.UNDEFINED_TIMEOUT) {
            // if timeout has not been defined for the enity, use timeout value from the
            // portlet definition
            return portletDefinition.getRenderTimeOut();
        } else {
            return timeOut;
        }
    }

    public Map<String, ISecurityRoleReference> getSecurityRoleReferences() {
        return this.portletDefinition.getSecurityRoleReferences();
    }

    public Map<String, IContentType> getSupportedContentTypes() {
        return this.portletDefinition.getSupportedContentTypes();
    }

    public Set<Locale> getSupportedLocales() {
        return this.portletDefinition.getSupportedLocales();
    }

    public IPortletDefinition getPortletDefinition() {
        return this.portletDefinition;
    }
    
    
    //********** IPortletDefinitionInformationProvider Methods **********\\

    public String getFunctionalName() {
        return this.portletDefinition.getFunctionalName();
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (!(obj instanceof IPortletEntity)) {
            return false;
        }
        if (this == obj) {
            return true;
        }
        
        IPortletEntity other = (IPortletEntity)obj;
        return new EqualsBuilder()
            .append(this.portletEntityTransfer, other.getPortletEntityTransfer())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return this.portletEntityTransfer.hashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
        .append("portletEntityTransfer", this.portletEntityTransfer)
        .toString();
    }
}
