/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.core.ObjectId;
import org.jasig.portal.utils.NamespacingKeyWrapper;
import org.jasig.portal.utils.Tuple;

/**
 * Provides the basic logic for storing objects with parents and secondary keys
 * in a registry. Implementations must provide an IRegistryStore implementation
 * for storing the objects.
 * <ol>
 *   <li>GT_StageId - id interface/class, example - {@link org.jasig.portal.portlet.entity.PortletEntityId}</li>
 *   <li>GT_StageTransfer - transfer interface/class, example - {@link org.jasig.portal.portlet.transfer.IPortletEntityTransfer}</li>
 * </ol>
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public abstract class AbstractRegistry<GT_StageTransfer, GT_StageId extends ObjectId> {
    private static final String ID_SET_NAMESPACE = "ID_SET__";
    private static final String SECONDARY_KEY_NAMESPACE = "SECONDARY_KEY_MAPPING__";

    
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    /**
     * @return The registry store implementation to use for storage.
     */
    protected abstract IRegistryStore getRegistryStore();
    
    /**
     * When an ID is being added to the registry the implementer may determine
     * how the ownedIdSet is updated. This Set should contain all the ObjectId primary
     * keys currently in the registry store the the specified key.
     * 
     * @param primaryId id being added to the set
     * @param parentId parent id
     * @param ownedIdSet current set of ids for the given parent
     * @return a tuple with updated set and a boolean flag which is <code>true</code> if the set has been modified
     */
    protected abstract Tuple<Set<GT_StageId>, Boolean> updateIdSetOnAdd(GT_StageId primaryId, ObjectId parentId, Set<GT_StageId> ownedIdSet);
    
    /**
	 * When an ID is being removed from the registry the implementer may determine 
     * how the ownedIdSet is updated. This Set should contain all ObjectId primary
     * keys currently in the registry store for the specified key.
     * @param primaryId id being removed from the set
     * @param parentId parent id
     * @param ownedIdSet current set of ids for a given parent
     * @return a tuple with updated set and a boolean flag which is <code>true</code> if the set has been modified
     */
    protected abstract Tuple<Set<GT_StageId>, Boolean> updateIdSetOnRemove(GT_StageId primaryId, ObjectId parentId, Set<GT_StageId> ownedIdSet);
    
    /**
     * Adds the object to the registry. Creates all the secondary key mappings and
     * updates all the owned ID Sets.
     * <br>
     * The parentIds Set is used if the registered object is a member of some group or groups. An example
     * would be a IPortletDefinition. A IPortletDefinition has its own ID and has a parent
     * IPortletApplicationDefinition and a parent IPortletDeployment. The parentIds set would contain the
     * IDs of the IPortletApplicationDefinition and the IPortletDeployment. This allows direct access to
     * the registred IPortletDefinitions for either type of parent object.
     * 
     * @param registeredObject object to register, may not be null
     * @param primaryId objects' primary id, may not be null
     * @param parentIds ids of object sets to which the registering object should be added, may be null
     * @param secondaryKeys unique string secondary keys that may be associated with the object, may be null
     */
    protected final void registerObject(GT_StageTransfer registeredObject, GT_StageId primaryId, Set<? extends ObjectId> parentIds, Set<String> secondaryKeys) {
        if (logger.isDebugEnabled()) {
            logger.debug("Storing '" + registeredObject + "' for key='" + primaryId + "'");
        }
        final IRegistryStore registryStore = this.getRegistryStore();
        
        //Put the object in the cache
        registryStore.put(primaryId, registeredObject);
        
        if (parentIds != null) {
            for (final ObjectId parentId : parentIds) {
                //Check the idSet for each parent to see if it contains the registered id
                this.checkIdSetContains(parentId, primaryId);
            }
        }
        
        //Check the default idSet to see if it contains the registered id
        this.checkIdSetContains(primaryId);
        
        if (secondaryKeys != null) {
            //Create secondary key mappings
            for (final String secKey : secondaryKeys) {
                final String fullKey = SECONDARY_KEY_NAMESPACE + secKey;
                registryStore.put(fullKey, primaryId);
                this.addMapping(primaryId, fullKey);
            }
        }
    }
    
    /**
     * Retrieves the object for the id. May be null even if the object has not
     * been explicitly removed.
     * 
     * @param primaryId The primary key of the object to retrieve.
     * @return The object associated with the id.
     */
    @SuppressWarnings("unchecked")
    protected final GT_StageTransfer getObject(GT_StageId primaryId) {
        final IRegistryStore registryStore = this.getRegistryStore();
        return (GT_StageTransfer)registryStore.get(primaryId);
    }
    
    
    /**
     * Retrieves the object for the secondary key. May be null even if the
     * object has not been explicitly removed.
     * 
     * @param secondaryKey The secondary key of the object to retrieve.
     * @return The object associated with the id.
     */
    @SuppressWarnings("unchecked")
    protected final GT_StageTransfer getObjectBySecondaryKey(String secondaryKey) {
        final IRegistryStore registryStore = this.getRegistryStore();
        final GT_StageId primaryId = (GT_StageId)registryStore.get(SECONDARY_KEY_NAMESPACE + secondaryKey);
        if (primaryId != null) {
            return this.getObject(primaryId);
        }
        else {
            return null;
        }
    }


    /**
     * Removes the object for the id from the registry as well as all the
     * secondary mappings and updates the owned id Sets.
     * 
     * @param primaryId The id to remove.
     */
    protected final void invalidateObject(GT_StageId primaryId) {
        if (logger.isDebugEnabled()) {
            logger.debug("Invalidating key='" + primaryId + "'");
        }
        
        //Remove the object from the cache
        final IRegistryStore registryStore = this.getRegistryStore();
        registryStore.remove(primaryId);
        
        //Clear any invalid id mappings
        this.removeMappings(primaryId);
    }
    
    
    /**
     * Same as calling {@link #getIdSet(ObjectId)} with a <code>null</code> argument.
     * 
     * @return The immutable Set of ObjectIds, may be null if no Set exists for the parent.
     */
    @SuppressWarnings("unchecked")
    protected final Set<GT_StageId> getIdSet() {
        return (Set<GT_StageId>)this.getIdSet(null);
    }
    
    /**
     * Gets a Set of ObjectIds for a specified parent. The parent may be null
     * which is the same as calling {@link #getIdSet()}
     * 
     * @param parentId The id of the Set parent, may be null.
     * @return The immutable Set of ObjectIds, may be null if no Set exists for the parent.
     */
    @SuppressWarnings("unchecked")
    protected final Set<GT_StageId> getIdSet(ObjectId parentId) {
        final NamespacingKeyWrapper cacheKey = new NamespacingKeyWrapper(ID_SET_NAMESPACE, parentId);
            
        final IRegistryStore registryStore = this.getRegistryStore();
        final Set<GT_StageId> primaryIdSet = (Set<GT_StageId>)registryStore.get(cacheKey);
        if (primaryIdSet != null) {
            return Collections.unmodifiableSet(primaryIdSet);
        }
        else {
            return null;
        }
    }
    
    /**
     * Same as calling {@link #setIdSet(ObjectId, Set)} with a <code>null</code> ObjectId.
     * 
     * @param primaryIdSet The ID Set to store.
     */
    protected final void setIdSet(Set<GT_StageId> primaryIdSet) {
        this.setIdSet(null, primaryIdSet);
    }
    
    /**
     * Stores an id Set for the specified parent in the cache. If the specified Set is
     * <code>null</code> {@link #clearIdSet(ObjectId)} is called.
     * 
     * @param parentId The key to store the Set for, null values ok.
     * @param primaryIdSet The ID Set to store.
     */
    protected final void setIdSet(ObjectId parentId, Set<GT_StageId> primaryIdSet) {
        if (logger.isDebugEnabled()) {
            logger.debug("Storing IDSet='" + primaryIdSet + "' for parent='" + parentId + "'");
        }
        
        if (primaryIdSet == null) {
            this.clearIdSet(parentId);
        }
        else {
            final NamespacingKeyWrapper cacheKey = new NamespacingKeyWrapper(ID_SET_NAMESPACE, parentId);
            
            //Synchronize to ensure the set isn't being modified while updating the cache & mappings 
            synchronized (primaryIdSet) {
                final IRegistryStore registryStore = this.getRegistryStore();
                registryStore.put(cacheKey, primaryIdSet);
            
                for (final GT_StageId primaryId : primaryIdSet) {
                    this.addMapping(primaryId, cacheKey);
                }
            }
        }
    }
    
    
    /**
     * Same as calling {@link #clearIdSet(ObjectId)} with a <code>null</code>
     * argument.
     */
    protected final void clearIdSet() {
        this.clearIdSet(null);
    }
    
    /**
     * Removes the id set for the specified parent.
     * 
     * @param parentId The parent of the id set, may be null.
     */
    @SuppressWarnings("unchecked")
    protected final void clearIdSet(ObjectId parentId) {
        if (logger.isDebugEnabled()) {
            logger.debug("Clearing IDSet for parent='" + parentId + "'");
        }
        
        final NamespacingKeyWrapper cacheKey = new NamespacingKeyWrapper(ID_SET_NAMESPACE, parentId);
        
        final IRegistryStore registryStore = this.getRegistryStore();
        final Set<GT_StageId> idSet = (Set<GT_StageId>)registryStore.get(cacheKey);
        registryStore.remove(cacheKey);

        if (idSet != null) {
            // The leftover mappings to non-existing cache key set shouldn't matter,
            // as these are used only to cleanup sets upon DO removal, and
            // that is already being done here (by clearing the whole set)
            idSet.clear();
        }
    }
    

    /**
     * Same as calling {@link #checkIdSetContains(ObjectId, ObjectId)} with a null key.
     */
    private void checkIdSetContains(GT_StageId primaryId) {
        this.checkIdSetContains(null, primaryId);
    }
    
    /**
     * If the idSet for the parent ID exists and the specified id does not
     * exist in the set the set is cleared from the cache.
     * 
     * @param parentId The parent of the id set, may be null.
     * @param primaryId The id to check the set for.
     */
    @SuppressWarnings("unchecked")
    private void checkIdSetContains(ObjectId parentId, GT_StageId primaryId) {
        final NamespacingKeyWrapper cacheKey = new NamespacingKeyWrapper(ID_SET_NAMESPACE, parentId);
        final Set<GT_StageId> idSet = (Set<GT_StageId>)this.getRegistryStore().get(cacheKey);
        
        final Tuple<Set<GT_StageId>, Boolean> newIdSet = this.updateIdSetOnAdd(primaryId, parentId, idSet);
        if (newIdSet.getSecond()) {
            this.setIdSet(parentId, newIdSet.getFirst());
        }
    }
    
    
    /**
     * Adds a mapping from the id to the cache key. Used to keep track of all
     * the locations a primary ID is stored in the cache.
     * 
     * @param primaryId The id to store store the mapping for.
     * @param key The key that maps to the id.
     */
    private void addMapping(GT_StageId primaryId, Serializable key) {
        final IRegistryStore registryStore = this.getRegistryStore();
        
        Set<Serializable> mappings = registryStore.getKeySet(primaryId);
        if (mappings == null) {
            mappings = new HashSet<Serializable>();
            registryStore.putKeySet(primaryId, mappings);
        }

        mappings.add(key);
    }
    
    /**
     * Removes all mappings for a specified id. Also removes the mapped
     * keys from the cache.
     * 
     * @param primaryId The id to remove the mappings for.
     */
    @SuppressWarnings("unchecked")
    private void removeMappings(GT_StageId primaryId) {
        final IRegistryStore registryStore = this.getRegistryStore();
        
        final Set<Serializable> mappings = registryStore.removeKeySet(primaryId);
        if (mappings == null) {
            return;
        }
        
        for (final Serializable key : mappings) {
            if (key instanceof NamespacingKeyWrapper) {
                final NamespacingKeyWrapper setKey = (NamespacingKeyWrapper)key;
                final ObjectId parentId = (ObjectId)setKey.getKey();

                final NamespacingKeyWrapper cacheKey = new NamespacingKeyWrapper(ID_SET_NAMESPACE, parentId);
                final Set<GT_StageId> idSet = (Set<GT_StageId>)registryStore.get(cacheKey);

                final Tuple<Set<GT_StageId>, Boolean> newIdSet = this.updateIdSetOnRemove(primaryId, parentId, idSet);
                if (newIdSet.getSecond()) {
                    this.setIdSet(parentId, newIdSet.getFirst());
                }
            }
            else {
                registryStore.remove(key);
            }
        }
    }
    
    
    /**
     * Represents an object the registry can store data in.
     */
    protected interface IRegistryStore {
        /**
         * Stores an object in the registry for the specified key. This store
         * is not required to be permanent and may drop entries as needed,
         * (cache behavior).
         * 
         * @param key The key to store the value for, may not be null.
         * @param value The value to store in the registry.
         */
        public void put(Serializable key, Object value);
        
        /**
         * Retrieves the object from the registry. The object may not exist in
         * the registry even if it has not been explicity removed (cache behavior).
         * 
         * @param key The key to get the object for, may not be null.
         * @return The registered object.
         */
        public Object get(Serializable key);
        
        /**
         * Removes the object from the registry for the key. If the object does
         * not exist in the registry no action is taken.
         * 
         * @param key The key to remove the object for, may not be null.
         */
        public void remove(Serializable key);

        
        /**
         * Stores a Set of key mappings. This store may not drop entries as needed.
         * 
         * @param key The key to store the Set with.
         * @param idSet The idSet to store.
         */
        public void putKeySet(Object key, Set<Serializable> idSet);
        
        /**
         * Retrieves the idSet from the store.
         * 
         * @param key The key to store the Set with.
         * @return The idSet for the key, may be null.
         */
        public Set<Serializable> getKeySet(Object key);
        
        /**
         * Removes the idSet from the store.
         * 
         * @param key The key to remove the Set for.
         * @return The idSet exists in the store.
         */
        public Set<Serializable> removeKeySet(Object key);
    }
}
