/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.registry;

import java.io.Serializable;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.core.ObjectId;
import org.jasig.portal.portlet.registry.event.IRegistryEventListener;
import org.jasig.portal.utils.Tuple;
import org.springframework.beans.factory.annotation.Required;

/**
 * An abstract registry implementation that uses {@link org.jasig.portal.caching.ISimpleCacheProviderFacade} based store.
 * Note: this serves as a base to the regular (i.e. DAO-backed) registries. Transient registries extend a sibling GenericTransientRegistry class.
 * <ol>
 *   <li>GT_StageId - id interface/class, example - {@link org.jasig.portal.portlet.entity.PortletEntityId}</li>
 *   <li>GT_StageTransfer - transfer interface/class, example - {@link org.jasig.portal.portlet.transfer.IPortletEntityTransfer}</li>
 *   <li>GT_Registry - interface/class of the specific registry. This is used to deliver a type-safe registry reference when 
 *      notifying registry listeners of an event, example - {@link org.jasig.portal.portlet.registry.general.IPortletEntityRegistry}.</li>
 *   <li>GT_RegistryListener - interface/class of an event listener. Must extend {@link org.jasig.portal.portlet.registry.event.IRegistryEventListener}<GT_StageId, GT_Registry>.
 *      example - {@link org.jasig.portal.portlet.registry.event.IPortletEntityEventListener}</li>
 * </ol>
 * 
 * @author Eric Dalquist
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public abstract class GenericCachingRegistry<GT_StageTransfer, GT_StageId extends ObjectId, GT_Registry, GT_RegistryListener extends IRegistryEventListener<GT_StageId, GT_Registry>>
        extends GenericNotifyingRegistry<GT_StageTransfer, GT_StageId, GT_Registry, GT_RegistryListener> {
    
    private final IRegistryStore registryStore = new CachingRegistryStore();
    private final Map<Object, Set<Serializable>> idMappings = new HashMap<Object, Set<Serializable>>();
    private ISimpleCacheProviderFacade cache;
    
    /**
     * Gets/Creates a Set from the specified Map for using the ownerId as the key. The
     * childId is then added to the Set.
     * 
     * @param idSets The Map of ownedId Set objects.
     * @param ownerId The Id to get the ownedId Set with.
     * @param childId The Id to add to the ownedId Set.
     */
    protected static final <K extends ObjectId, V extends ObjectId> void addToIdSet(Map<K, Set<V>> idSets, K ownerId, V childId) {
        Set<V> ownedIdSet = idSets.get(ownerId);
        if (ownedIdSet == null) {
            ownedIdSet = new HashSet<V>();
            idSets.put(ownerId, ownedIdSet);
        }
        ownedIdSet.add(childId);
    }
    
    
    /**
     * @return Returns the cache.
     */
    public ISimpleCacheProviderFacade getCache() {
        return this.cache;
    }
    /**
     * @param cache The cache to set.
     */
    @Required
    public void setCache(ISimpleCacheProviderFacade cache) {
        this.cache = cache;
    }
    
    public Map<Object, Set<Serializable>> getIdMappings() {
        return this.idMappings;
    }


    protected final IRegistryStore getRegistryStore() {
        return this.registryStore;
    }
    
    /**
     * @see org.jasig.portal.portlet.registry.AbstractRegistry#updateIdSetOnAdd(org.jasig.portal.core.ObjectId, org.jasig.portal.core.ObjectId, java.util.Set)
     */
    @Override
    protected final Tuple<Set<GT_StageId>, Boolean> updateIdSetOnAdd(GT_StageId primaryId, ObjectId parentId, Set<GT_StageId> ownedIdSet) {
        if (ownedIdSet != null && !ownedIdSet.contains(primaryId)) {
            if (logger.isDebugEnabled()) {
                logger.debug("Clearing id Set for owner='" + parentId + "' because it did not contain id='" + primaryId + "'");
            }
            
            return new Tuple<Set<GT_StageId>, Boolean>(null, Boolean.TRUE);
        }
        else {
            return new Tuple<Set<GT_StageId>, Boolean>(ownedIdSet, Boolean.FALSE);
        }
    }

    /**
     * @see org.jasig.portal.portlet.registry.AbstractRegistry#updateIdSetOnRemove(org.jasig.portal.core.ObjectId, org.jasig.portal.core.ObjectId, java.util.Set)
     */
    @Override
    protected final Tuple<Set<GT_StageId>, Boolean> updateIdSetOnRemove(GT_StageId primaryId, ObjectId parentId, Set<GT_StageId> ownedIdSet) {
        if (ownedIdSet != null && ownedIdSet.contains(primaryId)) {
            if (logger.isDebugEnabled()) {
                logger.debug("Clearing id Set for owner='" + parentId + "' because it did contain id='" + primaryId + "'");
            }

            return new Tuple<Set<GT_StageId>, Boolean>(null, Boolean.TRUE);
        }
        else {
            return new Tuple<Set<GT_StageId>, Boolean>(ownedIdSet, Boolean.FALSE);
        }
    }
    
    /**
     * Stores objects in the cache and key sets in a local Map.
     */
    private final class CachingRegistryStore implements IRegistryStore {
        public Object get(Serializable key) {
            final ISimpleCacheProviderFacade cache = GenericCachingRegistry.this.getCache();
            return cache.getFromCache(key);
        }

        public void put(Serializable key, Object value) {
            final ISimpleCacheProviderFacade cache = GenericCachingRegistry.this.getCache();
            cache.putInCache(key, value);
        }

        public void remove(Serializable key) {
            final ISimpleCacheProviderFacade cache = GenericCachingRegistry.this.getCache();
            cache.removeFromCache(key);
        }

        public Set<Serializable> getKeySet(Object key) {
            final Map<Object, Set<Serializable>> idMappings = GenericCachingRegistry.this.getIdMappings();
            return (Set<Serializable>)idMappings.get(key);
        }

        public void putKeySet(Object key, Set<Serializable> idSet) {
            final Map<Object, Set<Serializable>> idMappings = GenericCachingRegistry.this.getIdMappings();
            idMappings.put(key, idSet);
        }

        public Set<Serializable> removeKeySet(Object key) {
            final Map<Object, Set<Serializable>> idMappings = GenericCachingRegistry.this.getIdMappings();
            return (Set<Serializable>)idMappings.remove(key);
        }
    }
}
