/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.registry;

import java.util.List;
import java.util.NoSuchElementException;

import org.jasig.portal.core.CompositeObjectId;
import org.jasig.portal.core.ObjectId;
import org.jasig.portal.core.ObjectIdNamespaceWrapper;
import org.jasig.portal.portlet.registry.event.IRegistryEventListener;
import org.jasig.portal.portlet.registry.event.IRegistryEventSource;
import org.jasig.portal.portlet.registry.event.RegistryEvent;
import org.jasig.portal.utils.WeakLinkedList;

/**
 * Abstract extension of {@link org.jasig.portal.portlet.registry.AbstractRegistry} that
 * implements event notification logic. 
 * Template types: 
 * <ol>
 *   <li>GT_StageId - id interface/class, example - {@link org.jasig.portal.portlet.entity.PortletEntityId}</li>
 *   <li>GT_StageTransfer - transfer interface/class, example - {@link org.jasig.portal.portlet.transfer.IPortletEntityTransfer}</li>
 *   <li>GT_Registry - interface/class of the specific registry. This is used to deliver a type-safe registry reference when 
 *      notifying registry listeners of an event, example - {@link org.jasig.portal.portlet.registry.general.IPortletEntityRegistry}.</li>
 *   <li>GT_RegistryListener - interface/class of an event listener. Must extend {@link org.jasig.portal.portlet.registry.event.IRegistryEventListener}<GT_StageId, GT_Registry>.
 *      example - {@link org.jasig.portal.portlet.registry.event.IPortletEntityEventListener}</li>
 * </ol>
 *  
 * The class also includes id-scoping constants used by derivative registries.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public abstract class GenericNotifyingRegistry<GT_StageTransfer, GT_StageId extends ObjectId, GT_Registry, GT_RegistryListener extends IRegistryEventListener<GT_StageId, GT_Registry>>
        extends AbstractRegistry<GT_StageTransfer, GT_StageId>
        implements IRegistryEventSource<GT_RegistryListener> {
    /* ============= event listener management and notification ============ */
    
    // I guess typedefs won't appear until java10
    protected final List<IRegistryEventListener<GT_StageId, GT_Registry>> eventListeners = new WeakLinkedList<IRegistryEventListener<GT_StageId, GT_Registry>>();
    
    /**
     * Sets the event listeners to this registry.
     * @param listeners
     */
    public void setRegistryEventListeners(List<GT_RegistryListener> listeners) {
        this.eventListeners.clear();
        this.eventListeners.addAll(listeners);
    }
    
    /**
     * Adds an event listener to this registry.
     * @param listener
     */
    public void addRegistryEventListener(GT_RegistryListener listener) {
        this.eventListeners.add(listener);
    }
    
    /**
     * Adds a generic event listener to this registry.
     * @param listener
     */
    public void addGenericRegistryEventListener(IRegistryEventListener<GT_StageId, GT_Registry> listener) {
        this.eventListeners.add(listener);
    }
    
    /**
     * Adds the event listeners for this registry.
     * @param listeners
     */
    public void addRegistryEventListeners(List<? extends GT_RegistryListener> listeners) {
        this.eventListeners.addAll(listeners);
    }
    
    /**
     * Removes an event listener from this registry.
     * @param listener
     */
    public void removeRegistryEventListener(GT_RegistryListener listener) {
        this.eventListeners.remove(listener);
    }
    
    /**
     * Removes an event listener from this registry.
     * @param listener
     */
    public void removeGenericRegistryEventListener(IRegistryEventListener<GT_StageId,GT_Registry> listener) {
        this.eventListeners.remove(listener);
    }
    
    /**
     * Notify listeners that a DO with a specific id has been updated
     * @param primaryId id of a DO that has been updated
     */
    @SuppressWarnings("unchecked")
    protected final void notifyOnUpdate(GT_StageId primaryId) {
        final RegistryEvent<GT_StageId, GT_Registry> event = new RegistryEvent<GT_StageId, GT_Registry>(primaryId, (GT_Registry)this);
        try {
            for (final IRegistryEventListener<GT_StageId, GT_Registry> listener : this.eventListeners) {
                if (listener != null) {
                    listener.objectUpdated(event);
                }
            }
        }
        catch (NoSuchElementException nsee) {
            //Ignore exception as the WeakLinkedList may throw it even if hasNext returned true.
        }
    }
    
    /**
     * Notify listeners that a DO with a specific id has been loaded into registry
     * @param id id of a DO that was loaded
     */
    @SuppressWarnings("unchecked")
    protected final void notifyOnLoad(GT_StageId id) {
        final RegistryEvent<GT_StageId, GT_Registry> event = new RegistryEvent<GT_StageId, GT_Registry>(id, (GT_Registry)this);
        try {
            for (final IRegistryEventListener<GT_StageId, GT_Registry> listener: this.eventListeners) {
                if (listener != null) {
                    listener.objectLoaded(event);
                }
            }
        }
        catch (NoSuchElementException nsee) {
            //Ignore exception as the WeakLinkedList may throw it even if hasNext returned true.
        }
    };
    
    /**
     * Notify listeners that a DO with a specific id has been removed
     * @param id id of an removed DO
     */
    @SuppressWarnings("unchecked")
    protected final void notifyOnRemove(GT_StageId id) {
        final RegistryEvent<GT_StageId, GT_Registry> event = new RegistryEvent<GT_StageId, GT_Registry>(id, (GT_Registry)this);
        try {
            for (final IRegistryEventListener<GT_StageId, GT_Registry> listener: this.eventListeners) {
                if (listener != null) {
                    listener.objectRemoved(event);
                }
            }
        }
        catch (NoSuchElementException nsee) {
            //Ignore exception as the WeakLinkedList may throw it even if hasNext returned true.
        }
    };
    
    /**
     * Notify listeners that a new DO has been created
     * @param id id of a newly created DO
     */
    @SuppressWarnings("unchecked")
    protected final void notifyOnCreate(GT_StageId id) {
        final RegistryEvent<GT_StageId, GT_Registry> event = new RegistryEvent<GT_StageId, GT_Registry>(id, (GT_Registry)this);
        try {
            for (final IRegistryEventListener<GT_StageId, GT_Registry> listener: this.eventListeners) {
                if (listener != null) {
                    listener.objectCreated(event);
                }
            }
        }
        catch (NoSuchElementException nsee) {
            //Ignore exception as the WeakLinkedList may throw it even if hasNext returned true.
        }
    };
    
    /* ============== id scoping constants & methods ================ */
    
    protected static final String PARENT_DO_NAMESPACE = "PARENT_DO_";
    protected static final String APPLICATION_DO_NAMESPACE = "APPLICATION_DO_";
    protected static final String PARENT_AND_APPLICATION_DO_NAMESPACE="PARENT_AND_APP_DO_";
    
    /**
     * Generates a namespaced ObjectId for the parent's ObjectId
     */
    protected ObjectId getParentNamespacedId(ObjectId parentId) {
        if (parentId == null) {
            return null;
        }

        return new ObjectIdNamespaceWrapper(PARENT_DO_NAMESPACE, parentId);
    }
    
    /**
     * Generates a namespaced ObjectId for the parent application's ObjectId
     */
    protected ObjectId getParentApplicationNamespacedId(ObjectId parentApplicationId) {
        if (parentApplicationId == null) {
            return null;
        }

        return new ObjectIdNamespaceWrapper(APPLICATION_DO_NAMESPACE, parentApplicationId);
    }
    
    /**
     * Generates a namespaced ObjectId for the parent's ObjectId and the parent application's ObjectId
     */
    protected ObjectId getParentsNamespacedId(ObjectId parentId, ObjectId parentApplicationId) {
        if (parentId == null || parentApplicationId == null) {
            return null;
        }
        
        final CompositeObjectId compositeObjectId = new CompositeObjectId(parentId, parentApplicationId);
        return new ObjectIdNamespaceWrapper(PARENT_AND_APPLICATION_DO_NAMESPACE, compositeObjectId);
    }
}
