/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.registry.general;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.core.ObjectId;
import org.jasig.portal.portlet.registry.GenericCachingRegistry;
import org.jasig.portal.portlet.registry.event.IRegistryEventListener;
import org.springframework.util.Assert;

/**
 * A generic implementation of a portlet registry on which all specific registry implementations
 * can be based. 
 * 
 * This implementation includes all the primary logic, leaving simple abstract methods to be implemented
 * by the stage-specific registries. Most of these methods are proxies to DAO calls and methods for obtaining
 * information from the current stage transfer object. The current class takes care of coordinating cache retrieval,
 * DAO calls and event notifications.
 * 
 * Template types (illustrated on a portlet entity registry example):
 * <ol>
 *   <li>GT_Stage - domain object interface/class, example - {@link org.jasig.portal.portlet.entity.IPortletEntity}</li>
 *   <li>GT_StageId - id interface/class, example - {@link org.jasig.portal.portlet.entity.PortletEntityId}</li>
 *   <li>GT_StageTransfer - transfer interface/class, example - {@link org.jasig.portal.portlet.transfer.IPortletEntityTransfer}</li>
 *   <li>GT_ParentStageId - id interface/class of a previous portlet stage, example - {@link org.jasig.portal.portlet.definition.PortletDefinitionId}</li>
 *   <li>GT_ParentApplicationStageId - id interface/class of an applicaiton at a current stage, example - {@link org.jasig.portal.portlet.entity.PortletApplicationEntityId}.
 *      If not applicable (e.g. Stage is already describes portlet application) just use ObjectId for the template type.</li>
 *   <li>GT_Registry - interface/class of the specific registry. This is used to deliver a type-safe registry reference when 
 *      notifying registry listeners of an event, example - {@link org.jasig.portal.portlet.registry.general.IPortletEntityRegistry}.</li>
 *   <li>GT_RegistryListener - interface/class of an event listener. Must extend {@link org.jasig.portal.portlet.registry.event.IRegistryEventListener}<Stage,Registry>.
 *      example - {@link org.jasig.portal.portlet.registry.event.IPortletEntityEventListener}</li>
 * </ol>
 *      
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @author Michael Ivanov
 * @author Eric Dalquist
 * @version $Revision: 11955 $
 */
public abstract class GenericPortletRegistry<GT_Stage, GT_StageId extends ObjectId, GT_StageTransfer, GT_ParentStageId extends ObjectId, GT_ParentApplicationStageId extends ObjectId, GT_Registry, GT_RegistryListener extends IRegistryEventListener<GT_StageId, GT_Registry>>
    extends GenericCachingRegistry<GT_StageTransfer, GT_StageId, GT_Registry, GT_RegistryListener> {
    
    //********** Methods for subclasses to override depending on needed functionality **********//
    
    /**
     * Subtypes must implement retrieval of a stage transfer object from the DAO
     * @param id id of a domain object at a current stage
     * @return stage transfer object for the given id or <code>null</code> if it doesn't exist
     */
    abstract GT_StageTransfer getTransferObjectFromDAO(GT_StageId id);
    
    /**
     * Permanently delete a domain object with a given id
     * @param id id of a domain object at a current stage to be deleted
     */
    abstract void removeObjectFromDAO(GT_StageId id);
    
    /**
     * Update domain object transfer in DAO.
     * @param transfer
     */
    abstract void updateObjectInDAO(GT_StageTransfer transfer);
    
    /**
     * Obtain a set of current stage transfer objects for a parent stage id
     * @param parentId object id of a parent stage for which current stage transfer objects should be fetched
     * @return a set of current stage transfer objects for a given parent stage object id
     */
    abstract Set<GT_StageTransfer> getTransferObjectsForParentIdFromDAO(GT_ParentStageId parentId);
    
    /**
     * Obtain a set of current stage transfer objects for an application stage id
     * @param applicationId object id of a corresponding application stage for which current stage transfer objects should be fetched
     * @return a set of current stage transfer objects for a given application stage object id
     */
    abstract Set<GT_StageTransfer> getTransferObjectsForApplicationIdFromDAO(GT_ParentApplicationStageId applicationId);
    
    /**
     * Obtain a set of transfer objects for a specified application and parent ids
     * @param applicationId
     * @param parentId
     * @return a set of current transfer objects for specified application and parent ids
     */
    abstract Set<GT_StageTransfer> getTransferObjectsForApplicationIdAndParentIdFromDAO(GT_ParentApplicationStageId applicationId,GT_ParentStageId parentId);
    
    /**
     * Obtain a transfer object for a specified secondary key
     * @param key
     * @return transfer object for a specified secondary key
     */
    abstract GT_StageTransfer getTransferObjectBySecondaryKeyFromDAO(String key);
    
    /**
     * Return a set of all current stage transfer objects
     * @return a set of all current stage transfer objects
     */
    abstract Set<GT_StageTransfer> getTransferObjectsFromDAO();
    
    /**
     * Add a new DO transfer to the DAO. This typically involves three steps:
     * 1. creating new transfer object (to obtain new id)
     * 2. copying all the properties into the new transfer object
     * 3. persisting complete transfer object
     * @param transfer transfer object containing information for new DO
     * @return updated transfer object (with newly created id)
     */
    abstract GT_StageTransfer addTransferObjectToDAO(GT_StageTransfer transfer);
    
    /**
     * Subtypes must implement creation of the domain object from transfer object.
     * This typically involves a call to an appropriate factory.
     * @param transfer transfer object
     * @return new DO instance
     */
    abstract GT_Stage createDomainObjectFromTransfer(GT_StageTransfer transfer);
    
    /**
     * Obtain domain object id given a transfer object
     * @param transfer transfer object
     * @return id of a given transfer object
     */
    abstract GT_StageId getId(GT_StageTransfer transfer);
    
    /**
     * Obtain object id of a parent stage (e.g. if current stage is portletEntity,
     * this method would return corresponding portletDefinitionId)
     * @param transfer
     * @return id of a parent domain object or <code>null</code> if parent doesnt' exist
     */
    abstract GT_ParentStageId getParentId(GT_StageTransfer transfer);
    
    /**
     * Obtain object id of a corresponding application stage (e.g. if current stage is portletEntity,
     * this method would return corresponding portletApplicationEntityId).
     * @param transfer
     * @return id of a corresponding stage application domain object, or <code>null</code> if this is not applicable
     */
    abstract GT_ParentApplicationStageId getApplicationId(GT_StageTransfer transfer);
    
    
    /**
     * Obtain secondary keys for the current stage
     * @param transfer
     * @return a set of secondary keys, such as functionalName
     */
    abstract Set<String> getSecondaryKeys(GT_StageTransfer transfer);
    
   
    /* ==============  implementation of core registry methods ============ */
    
    /**
     * A generic method for obtaining a domain object given its id
     * @param primaryId id of the domain object at a current stage
     * @return DO instance or <code>null</code> if it can not be found
     */
    protected final synchronized GT_Stage get(GT_StageId primaryId) {
        final GT_StageTransfer transfer = this.getTransfer(primaryId);
        if (transfer != null) {
            //Create and return the domain object
            return createDomainObjectFromTransfer(transfer);
        }
        else {
            return null;
        }
    }
    
    protected final synchronized GT_Stage getBySecondaryKey(String secondaryKey) {
        Assert.notNull(secondaryKey, "secondaryKey can not be null");

        // try getting from cache
        GT_StageTransfer stageTransfer = (GT_StageTransfer)this.getObjectBySecondaryKey(secondaryKey);

        if (stageTransfer == null) {
            // check DAO
            stageTransfer = this.getTransferObjectBySecondaryKeyFromDAO(secondaryKey);
            if (stageTransfer != null) {
                GT_StageId primaryId = this.getId(stageTransfer);
                this.invalidateObject(primaryId);
                this.registerObject(stageTransfer);
                this.notifyOnLoad(primaryId);
            }
        }

        if (stageTransfer != null) {
            return this.createDomainObjectFromTransfer(stageTransfer);
        }
        else {
            return null;
        }
    }
    
    private final synchronized GT_StageTransfer getTransfer(GT_StageId primaryId) {
        Assert.notNull(primaryId, "primaryId can not be null");

        //Get the transfer object from the cache
        GT_StageTransfer stageTransfer = (GT_StageTransfer)this.getObject(primaryId);

        if (stageTransfer == null) {
            //Get the transfer object from the DAO
            stageTransfer = this.getTransferObjectFromDAO(primaryId);

            if (stageTransfer == null) {
                if (logger.isDebugEnabled()) {
                    logger.debug("Could not find a transfer object in DAO for id='" + primaryId + "'");
                }
                this.invalidateObject(primaryId);
                return null;
            }
            this.registerObject(stageTransfer);
            this.notifyOnLoad(primaryId);
        }
        return stageTransfer;
    }
    
    protected final synchronized void remove(GT_StageId primaryId) {
        Assert.notNull(primaryId, "primaryId can not be null");

        //Remove from the DAO
        removeObjectFromDAO(primaryId);

        //Remove the Entintion from the registry
        this.invalidateObject(primaryId);
        this.notifyOnRemove(primaryId);
    }
    
    /**
     * Create a new DO given appropriate transfer object
     * @param stageTransfer transfer object of the same stage containing information for the new DO
     * @return newly created DO instance
     */
    protected final synchronized GT_Stage create(GT_StageTransfer stageTransfer) {
        Assert.notNull(stageTransfer, "stageTransfer can not be null");
        final GT_StageTransfer updatedTransfer = this.addTransferObjectToDAO(stageTransfer);

        //Register the new transfer object
        this.registerObject(updatedTransfer);
        final GT_StageId primaryId = this.getId(updatedTransfer);
        this.notifyOnCreate(primaryId);
        return createDomainObjectFromTransfer(updatedTransfer);
    }
    
    protected final synchronized void update(GT_StageTransfer stageTransfer) {
        Assert.notNull(stageTransfer, "stageTransfer can not be null");
        
        final GT_StageId primaryId = this.getId(stageTransfer);
        final GT_Stage oldDomainObject = this.get(primaryId);
        if (oldDomainObject == null) {
            throw new IllegalArgumentException("There is no existing domain object for the transfer object '" + stageTransfer +"'");
        }
        //Update the DAO
        this.updateObjectInDAO(stageTransfer);

        //Update the registry
        this.invalidateObject(primaryId);
        this.registerObject(stageTransfer);
        this.notifyOnUpdate(primaryId);
    }
    
    /**
     * Register given transfer object
     * @param stageTransfer
     */
     private void registerObject(GT_StageTransfer stageTransfer) {
        // determine owner id sets
        final GT_ParentStageId parentId = this.getParentId(stageTransfer);
        final GT_ParentApplicationStageId parentApplicationId = this.getApplicationId(stageTransfer);
        
        Set<ObjectId> parentIds = null;
        if (parentId != null) {
            if (parentApplicationId != null) {
                parentIds = new HashSet<ObjectId>(5);
                parentIds.add(this.getParentNamespacedId(parentId));
                parentIds.add(this.getParentApplicationNamespacedId(parentApplicationId));
                parentIds.add(this.getParentsNamespacedId(parentId, parentApplicationId));
            }
            else {
                final ObjectId parentNamespacedId = this.getParentNamespacedId(parentId);
                parentIds = Collections.singleton(parentNamespacedId);
            }
        }
        else if (parentApplicationId != null) {
            final ObjectId parentApplicationNamespacedId = this.getParentApplicationNamespacedId(parentApplicationId);
            parentIds = Collections.singleton(parentApplicationNamespacedId);
        }
        
        final GT_StageId primaryId = getId(stageTransfer);
        final Set<String> secondaryKeys = this.getSecondaryKeys(stageTransfer); // determine seconday keys
        this.registerObject(stageTransfer, primaryId, parentIds, secondaryKeys);
    }
    
    /**
     * Obtain all domain objects for a given parent id
     * @param parentId
     * @return set of all domain objects for a given parent id
     */
    protected synchronized Set<GT_Stage> getForParent(GT_ParentStageId parentId) {
        Assert.notNull(parentId, "parentId can not be null");
        final ObjectId namespacedParentId = this.getParentNamespacedId(parentId);

        Set<GT_Stage> stageSet = this.getSetFromCache(namespacedParentId);

        //No domain objects have been loaded for one reason or another
        if (stageSet == null) {
            //Get the set of entity transfers from the dao
            final Set<GT_StageTransfer> transfers = this.getTransferObjectsForParentIdFromDAO(parentId);
            if (transfers == null) {
                return Collections.emptySet();
            }

            stageSet = this.updateSetInCache(namespacedParentId, transfers);
        }

        return Collections.unmodifiableSet(stageSet);
    }

    /**
     * Fetch all domain objects for a given application id
     * @param applicationId
     * @return set of all domain objects for a given application id
     */
    protected synchronized Set<GT_Stage> getForApplication(GT_ParentApplicationStageId applicationId) {
        Assert.notNull(applicationId, "applicationId can not be null");

        final ObjectId namespacedApplicationId = this.getParentApplicationNamespacedId(applicationId);

        Set<GT_Stage> stageSet = getSetFromCache(namespacedApplicationId);

        //No domain objects have been loaded for one reason or another
        if (stageSet == null) {
            //Get the set of entity transfers from the dao
            final Set<GT_StageTransfer> transfers = this.getTransferObjectsForApplicationIdFromDAO(applicationId);
            if (transfers == null) {
                return Collections.emptySet();
            }

            stageSet = this.updateSetInCache(namespacedApplicationId, transfers);
        }

        return Collections.unmodifiableSet(stageSet);
    }
    
    /**
     * Fetch all domain objects for a given application id and parent id
     * @param parentApplicationId
     * @param parentId
     * @return set of all domain objects for a given application id and parent id
     */
    protected synchronized Set<GT_Stage> getForApplicationAndParent(GT_ParentApplicationStageId parentApplicationId, GT_ParentStageId parentId) {
        Assert.notNull(parentApplicationId, "parentApplicationId can not be null");
        Assert.notNull(parentId, "parentId can not be null");

        final ObjectId parentsNamespacedId = this.getParentsNamespacedId(parentId, parentApplicationId);

        Set<GT_Stage> stageSet = getSetFromCache(parentsNamespacedId);

        //No domain objects have been loaded for one reason or another
        if (stageSet == null) {
            // clear sets for parentId and applicationId individually
            this.clearIdSet(this.getParentNamespacedId(parentId));
            this.clearIdSet(this.getParentApplicationNamespacedId(parentApplicationId));

            //Get the set of entity transfers from the dao
            final Set<GT_StageTransfer> stageTransfers = this.getTransferObjectsForApplicationIdAndParentIdFromDAO(parentApplicationId, parentId);
            if (stageTransfers == null) {
                return Collections.emptySet();
            }

            stageSet = this.updateSetInCache(parentsNamespacedId, stageTransfers);
        }

        return Collections.unmodifiableSet(stageSet);
    }
    
    /**
     * Get a set of all domain objects at a current stage
     * @return set of all existing domain objects at a current stage
     */
     protected synchronized Set<GT_Stage> getAll() {
        // Get the set of loaded ids from cache
        Set<GT_StageId> primaryIdSet = this.getIdSet();

        //Domain object set
        Set<GT_Stage> stageSet = null;

        if (primaryIdSet != null) {
            if (logger.isDebugEnabled()) {
                logger.debug("getAll() - Found Set of loaded ids '" + primaryIdSet + "'");
            }
            stageSet = new HashSet<GT_Stage>(primaryIdSet.size());

            //Iterate through the IDs
            for (final GT_StageId primaryId : primaryIdSet) {
                //Try to get the cached version of the transfer object
                final GT_Stage stage = this.get(primaryId);
                if (stage == null) {
                    // invalidate global set
                    this.clearIdSet();
                    // give up on the cached id set, break out of the loop
                    stageSet = null;
                    break;
                }
                else {
                    stageSet.add(stage);
                }
            }
        }

        // No domain objects have been loaded for one reason or another
        if (stageSet == null) {
            //Get the set of entity transfers from the dao
            final Set<GT_StageTransfer> stageTransfers = getTransferObjectsFromDAO();

            if (stageTransfers == null || stageTransfers.size() == 0) {
                return Collections.emptySet();
            }

            primaryIdSet = new HashSet<GT_StageId>(stageTransfers.size());
            stageSet = new HashSet<GT_Stage>(stageTransfers.size());
            final Map<ObjectId, Set<GT_StageId>> onwerIdSets = new HashMap<ObjectId, Set<GT_StageId>>();

            //Iterate through the transfers
            for (final GT_StageTransfer stageTransfer : stageTransfers) {
                final GT_StageId primaryId = this.getId(stageTransfer);
                final GT_Stage stage = this.get(primaryId);
                
                if (stage != null) {
                    primaryIdSet.add(primaryId);
                    stageSet.add(stage);
                    
                    // add to owner id sets
                    ObjectId applicationId = this.getParentApplicationNamespacedId(this.getApplicationId(stageTransfer));
                    if (applicationId != null) {
                        addToIdSet(onwerIdSets, applicationId, primaryId);
                    }

                    ObjectId parentId = this.getParentNamespacedId(this.getParentId(stageTransfer));
                    if (parentId != null) {
                        addToIdSet(onwerIdSets, parentId, primaryId);

                        if (applicationId != null) {
                            final ObjectId compositeId = this.getParentsNamespacedId(this.getParentId(stageTransfer), this.getApplicationId(stageTransfer));
                            addToIdSet(onwerIdSets, compositeId, primaryId);
                        }
                    }
                }
            }

            this.setIdSet(primaryIdSet);

            // cache owner sets
            for (Map.Entry<ObjectId, Set<GT_StageId>> entry : onwerIdSets.entrySet()) {
                this.setIdSet(entry.getKey(), entry.getValue());
            }

            //Cache the Set of loaded transfer IDs
            this.setIdSet(primaryIdSet);
        }

        return Collections.unmodifiableSet(stageSet);
    }
    
    /* =============== private methods ================== */
    
    /**
     * A utility method for updating a set in cache given owner id and transfer object set, and obtaining
     * domain object set along the way.
     * @param parentId owner for which the id set should be recorded
     * @param stageTransfers transfer objects whose ids should be included in the set
     * @return set of domain objects corresponding to the provided set of transfer objects
     */
    private Set<GT_Stage> updateSetInCache(final ObjectId parentId, final Set<GT_StageTransfer> stageTransfers) {
        final Set<GT_StageId> primaryIdSet = Collections.synchronizedSet(new HashSet<GT_StageId>(stageTransfers.size()));
        final Set<GT_Stage> stageSet = new HashSet<GT_Stage>(stageTransfers.size());

        //Iterate through the transfer
        for (final GT_StageTransfer stageTransfer : stageTransfers) {
            final GT_StageId primaryId = this.getId(stageTransfer);
            final GT_Stage stage = this.get(primaryId);
            if (stage != null) {
                primaryIdSet.add(primaryId);
                stageSet.add(stage);
            }
        }

        //Cache the Set of loaded transfer IDs
        this.setIdSet(parentId, primaryIdSet);
        return stageSet;
    }

    /**
     * A utility method to get a set from cache
     * @param parentId owner id for which the set should be retrieved (properly namespaced)
     * @return valid DO set for a given owner id, or <code>null</code> if the retreival from cache failed
     */
    private Set<GT_Stage> getSetFromCache(final ObjectId parentId) {
        
        //Get the set of loaded ids from cache
        final Set<GT_StageId> primarIdSet = this.getIdSet(parentId);

        //Domain object set
        Set<GT_Stage> stageSet = null;
        if (primarIdSet != null) {
            if (logger.isDebugEnabled()) {
                logger.debug("getSetFromCache(ObjectId) - Found Set of loaded ids '" + primarIdSet + "' for id='" + parentId + "'");
            }

            stageSet = new HashSet<GT_Stage>(primarIdSet.size());
            //Iterate through the IDs
            for (final GT_StageId primaryId : primarIdSet) {
                //Try to get the cached version of the transfer object
                final GT_Stage stage = this.get(primaryId);
                if (stage == null) {
                    // invalidate current set
                    this.clearIdSet(parentId);
                    // invalidate global set, since it's likely to contain the same DO
                    this.clearIdSet();
                    // give up on the cached id set, break out of the loop
                    stageSet = null;
                    break;
                }
                else {
                    stageSet.add(stage);
                }
            }
        }

        return stageSet;
    }
}
