/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.registry.general;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.core.ObjectId;
import org.jasig.portal.portlet.registry.GenericNotifyingRegistry;
import org.jasig.portal.portlet.registry.event.IRegistryEventListener;
import org.jasig.portal.portlet.registry.event.RegistryEvent;
import org.jasig.portal.utils.NamespacingKeyWrapper;
import org.jasig.portal.utils.Tuple;
import org.springframework.util.Assert;

/**
 * An abstract base class for transient registries.
 * Provides proxy implementations of registry methods from {@link org.jasig.portal.portlet.registry.general.GenericPortletRegistry},
 * and handles chaining of event notifications. 
 * 
 * Template types (illustrated on a portlet entity registry example):
 * <ol>
 *   <li>GT_Stage - domain object interface/class, example - {@link org.jasig.portal.portlet.entity.IPortletEntity}</li>
 *   <li>GT_StageId - id interface/class, example - {@link org.jasig.portal.portlet.entity.PortletEntityId}</li>
 *   <li>GT_StageTransfer - transfer interface/class, example - {@link org.jasig.portal.portlet.transfer.IPortletEntityTransfer}</li>
 *   <li>GT_ParentStageId - id interface/class of a previous portlet stage, example - {@link org.jasig.portal.portlet.definition.PortletDefinitionId}</li>
 *   <li>GT_ParentApplicationStageId - id interface/class of an applicaiton at a current stage, example - {@link org.jasig.portal.portlet.entity.PortletApplicationEntityId}.
 *      If not applicable (e.g. Stage is already describes portlet application) just use ObjectId for the template type.</li>
 *   <li>GT_Registry - interface/class of the specific registry. This is used to deliver a type-safe registry reference when 
 *      notifying registry listeners of an event, example - {@link org.jasig.portal.portlet.registry.general.IPortletEntityRegistry}.</li>
 *   <li>GT_RegistryListener - interface/class of an event listener. Must extend {@link org.jasig.portal.portlet.registry.event.IRegistryEventListener}<Stage,Registry>.
 *      example - {@link org.jasig.portal.portlet.registry.event.IPortletEntityEventListener}</li>
 * </ol>
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public abstract class GenericTransientRegistry<GT_Stage, GT_StageId extends ObjectId, GT_StageTransfer, GT_ParentStageId extends ObjectId, GT_ParentApplicationStageId extends ObjectId, GT_Registry, GT_RegistryListener extends IRegistryEventListener<GT_StageId, GT_Registry>>
        extends GenericNotifyingRegistry<GT_StageTransfer, GT_StageId, GT_Registry, GT_RegistryListener>
        implements IRegistryEventListener<GT_StageId, GT_Registry> {
    
    private long idCounter = 0;
    private final IRegistryStore mapRegistryStore = new MapRegistryStore();
    
    // the real (persistent) registry is maintained using a transient reference here (no pun indended) this is needed for the session serialization
    private transient GenericPortletRegistry<GT_Stage, GT_StageId, GT_StageTransfer, GT_ParentStageId, GT_ParentApplicationStageId, GT_Registry, GT_RegistryListener> generalRegistry;
    
    /**
     * Request an instance of the registry being wrapped
     * @return non-null instance of a registry being wrapped
     */
    protected abstract GenericPortletRegistry<GT_Stage, GT_StageId, GT_StageTransfer, GT_ParentStageId, GT_ParentApplicationStageId, GT_Registry, GT_RegistryListener> getGeneralRegistry();
    
    private synchronized GenericPortletRegistry<GT_Stage, GT_StageId, GT_StageTransfer, GT_ParentStageId, GT_ParentApplicationStageId, GT_Registry, GT_RegistryListener> generalRegistryInstance() {
        if (this.generalRegistry == null) {
            this.setGeneralRegistry(this.getGeneralRegistry());
        }
        return this.generalRegistry;
    }
    
    
    //***** abstract methods that concrete classes will have to implement *****//
    
    /**
     * Set id on a transfer object
     * @param stageTransfer
     * @param primaryId a unique counter value that should be used for id generation
     */
    protected abstract void setId(GT_StageTransfer stageTransfer, long primaryId);
    
    /**
     * Subtypes must implement creation of the domain object from transfer object.
     * This typically involves a call to an appropriate factory.
     * @param stageTransfer transfer object
     * @return new DO instance
     */
    protected abstract GT_Stage createDomainObjectFromTransfer(GT_StageTransfer stageTransfer);
    
    
    //***** Transient registry specific methods *****//
    
    /**
     * Specify a general (i.e persistent) portlet registry that the transient registry should wrap
     * @param generalRegistry
     */
    protected void setGeneralRegistry(GenericPortletRegistry<GT_Stage, GT_StageId, GT_StageTransfer, GT_ParentStageId, GT_ParentApplicationStageId, GT_Registry,GT_RegistryListener> generalRegistry) {
        if (this.generalRegistry != null) {
            this.generalRegistry.removeGenericRegistryEventListener(this);
        }

        this.generalRegistry = generalRegistry;
        if (generalRegistry != null) {
            generalRegistry.addGenericRegistryEventListener(this);
        }
    }
    
    /**
     * @return The next ID in the sequence for this registry.
     */
    protected final long getNextId() {
        return ++this.idCounter; //increment the value then return the new value
    }
    
    protected GT_StageTransfer getTransientTransfer(GT_StageId primaryId) {
        return this.getObject(primaryId);
    }
    
    /* =============== registry proxy method implementation =============== */
    protected GT_Stage get(GT_StageId primaryId) {
        Assert.notNull(primaryId, "primaryId can not be null");
        
        // try local first
        final GT_StageTransfer localTransfer = this.getObject(primaryId);
        if (localTransfer != null) {
            final GT_Stage domainObject = this.createDomainObjectFromTransfer(localTransfer);
            if (domainObject == null) {
                if (logger.isInfoEnabled()) {
                    logger.info("unable to create domain object from an existing transfer object (could've gotten deleted along the way, invalidating): id=" + primaryId);
                }
                this.invalidateObject(primaryId);
            }
            return domainObject;
        }
        else {
            // delegate to general registry
            final GenericPortletRegistry<GT_Stage, GT_StageId, GT_StageTransfer, GT_ParentStageId, GT_ParentApplicationStageId, GT_Registry, GT_RegistryListener> generalRegistryInstance = this.generalRegistryInstance();
            return generalRegistryInstance.get(primaryId);
        }
    }
    
    protected GT_Stage getBySecondaryKey(String key) {
        Assert.notNull(key, "key can not be null");
        // try local first from cache
        final GT_StageTransfer localTransfer = this.getObjectBySecondaryKey(key);
        if (localTransfer != null) {
            return this.createDomainObjectFromTransfer(localTransfer);
        }
        else {
            // delegate to general registry
            final GenericPortletRegistry<GT_Stage, GT_StageId, GT_StageTransfer, GT_ParentStageId, GT_ParentApplicationStageId, GT_Registry, GT_RegistryListener> generalRegistryInstance = this.generalRegistryInstance();
            return generalRegistryInstance.getBySecondaryKey(key);
        }
    }

    protected void remove(GT_StageId id) {
        Assert.notNull(id, "id can not be null");

        //  try local first
        final GT_StageTransfer localTransfer = this.getObject(id);
        if (localTransfer != null) {
            this.invalidateObject(id);
        }
        else {
            final GenericPortletRegistry<GT_Stage, GT_StageId, GT_StageTransfer, GT_ParentStageId, GT_ParentApplicationStageId, GT_Registry, GT_RegistryListener> generalRegistryInstance = this.generalRegistryInstance();
            generalRegistryInstance.remove(id);
        }
    }
    
    /**
     * Create a transient DO given a complete transfer object (with the exception of the id)
     * @param transfer transfer object with all information necessary to create new DO
     * @return a newly created DO with an generated id
     */
    protected GT_Stage createTransient(GT_StageTransfer transfer) {
        Assert.notNull(transfer, "transfer can not be null");

        final long nextId = this.getNextId();
        // set generated id
        this.setId(transfer, nextId);
        GT_Stage domainObject = this.createDomainObjectFromTransfer(transfer);
        if (domainObject != null) {
            this.registerObject(transfer);
        }
        return domainObject;
    }
    
    protected GT_Stage create(GT_StageTransfer transfer) {
        return this.generalRegistryInstance().create(transfer);
    }
    
    protected synchronized void update(GT_StageTransfer transfer) {
        Assert.notNull(transfer, "transfer can not be null");
        
        final GenericPortletRegistry<GT_Stage, GT_StageId, GT_StageTransfer, GT_ParentStageId, GT_ParentApplicationStageId, GT_Registry, GT_RegistryListener> generalRegistryInstance = this.generalRegistryInstance();
        
        final GT_StageId id = generalRegistryInstance.getId(transfer);
        if (this.getObject(id) != null) {
            this.invalidateObject(id);
            this.registerObject(transfer);
        }
        else {
            generalRegistryInstance.update(transfer);
        }
    }
    
    protected synchronized Set<GT_Stage> getForParent(GT_ParentStageId parentId) {
        Assert.notNull(parentId, "parentId can not be null");
        
        final ObjectId namespacedParentId = this.getParentNamespacedId(parentId);

        // determine local set
        final Set<GT_StageId> localSet = this.getIdSet(namespacedParentId);
        final GenericPortletRegistry<GT_Stage, GT_StageId, GT_StageTransfer, GT_ParentStageId, GT_ParentApplicationStageId, GT_Registry, GT_RegistryListener> generalRegistryInstance = this.generalRegistryInstance();
        final Set<GT_Stage> generalSet = generalRegistryInstance.getForParent(parentId);
        return this.createJoinedSet(localSet,generalSet);
    }

    protected synchronized Set<GT_Stage> getForApplication(GT_ParentApplicationStageId applicationId) {
        Assert.notNull(applicationId, "applicationId can not be null");
        
        final ObjectId namespacedApplicationId = this.getParentApplicationNamespacedId(applicationId);
        // determine local set
        final Set<GT_StageId> localSet = this.getIdSet(namespacedApplicationId);
        final GenericPortletRegistry<GT_Stage, GT_StageId, GT_StageTransfer, GT_ParentStageId, GT_ParentApplicationStageId, GT_Registry, GT_RegistryListener> generalRegistryInstance = this.generalRegistryInstance();
        final Set<GT_Stage> generalSet = generalRegistryInstance.getForApplication(applicationId);
        return this.createJoinedSet(localSet,generalSet);
    }
    
    protected synchronized Set<GT_Stage> getForApplicationAndParent(GT_ParentApplicationStageId applicationId, GT_ParentStageId parentId) {
        Assert.notNull(applicationId, "applicationId can not be null");
        Assert.notNull(parentId, "parentId can not be null");
        
        final ObjectId compositeId = this.getParentsNamespacedId(parentId, applicationId);

        // determine local set
        final Set<GT_StageId> localSet = this.getIdSet(compositeId);
        final GenericPortletRegistry<GT_Stage, GT_StageId, GT_StageTransfer, GT_ParentStageId, GT_ParentApplicationStageId, GT_Registry, GT_RegistryListener> generalRegistryInstance = this.generalRegistryInstance();
        final Set<GT_Stage> generalSet = generalRegistryInstance.getForApplicationAndParent(applicationId, parentId);
        return this.createJoinedSet(localSet,generalSet);
    }
    
    /**
     * Get a set of all domain objects at a current stage
     * @return set of all existing domain objects at a current stage
     */
    protected synchronized Set<GT_Stage> getAll() {
        // get local id set
        final Set<GT_StageId> localSet = this.getIdSet();
        final GenericPortletRegistry<GT_Stage, GT_StageId, GT_StageTransfer, GT_ParentStageId, GT_ParentApplicationStageId, GT_Registry, GT_RegistryListener> generalRegistryInstance = this.generalRegistryInstance();
        final Set<GT_Stage> generalSet = generalRegistryInstance.getAll();
        return this.createJoinedSet(localSet,generalSet);
    }
    
    /**
     * Create a combined set of domain objects from a set of local ids and a general set
     * @param localSet set of transient DO ids
     * @param generalSet set of general registry DOs
     * @return a combined set of DOs
     */
    private Set<GT_Stage> createJoinedSet(Set<GT_StageId> localSet, Set<GT_Stage> generalSet) {
        if (localSet != null && !localSet.isEmpty()) {
            final int generalSetSize;
            if (generalSet != null) {
                generalSetSize = generalSet.size();
            }
            else {
                generalSetSize = 0;
            }
            
            final Set<GT_Stage> compositePortEnts = new HashSet<GT_Stage>(generalSetSize + localSet.size());
            if (generalSet != null) {
                compositePortEnts.addAll(generalSet);
            }

            for (final GT_StageId primaryId : localSet) {
                final GT_StageTransfer localTransfer = this.getObject(primaryId);
                if (localTransfer != null) {
                    final GT_Stage domainObject = this.createDomainObjectFromTransfer(localTransfer);
                    if (domainObject != null) {
                        compositePortEnts.add(domainObject);
                    }
                    else {
                        if (logger.isInfoEnabled()) {
                            logger.info("unable to create domain object from an existing transfer object (could've gotten deleted along the way, invalidating): id=" + primaryId);
                        }
                        this.invalidateObject(primaryId);
                    }
                }
                else {
                    if (logger.isInfoEnabled()) {
                        logger.info("unable to find transfer object for an id from an owner-mapped set (could've gotten deleted earlier, invalidating): id=" + primaryId);
                    }
                    this.invalidateObject(primaryId);
                }
            }
            return Collections.unmodifiableSet(compositePortEnts);
        }
        else {
            return generalSet;
        }
    }
    
    private void registerObject(GT_StageTransfer transfer) {
        final GenericPortletRegistry<GT_Stage, GT_StageId, GT_StageTransfer, GT_ParentStageId, GT_ParentApplicationStageId, GT_Registry, GT_RegistryListener> generalRegistryInstance = this.generalRegistryInstance();

        // determine owner id set
        final ObjectId parentId = generalRegistryInstance.getParentId(transfer);
        final ObjectId parentApplicationId = generalRegistryInstance.getApplicationId(transfer);
        Set<ObjectId> ownerIds = null;
        if (parentId != null) {
            if (parentApplicationId != null) {
                ownerIds = new HashSet<ObjectId>(3);
                ownerIds.add(this.getParentNamespacedId(parentId));
                ownerIds.add(this.getParentApplicationNamespacedId(parentApplicationId));
                ownerIds.add(this.getParentsNamespacedId(parentId, parentApplicationId));
            }
            else {
                final ObjectId namepsacedParentId = this.getParentNamespacedId(parentId);
                ownerIds = Collections.singleton(namepsacedParentId);
            }
        }
        else if (parentApplicationId != null) {
            final ObjectId namespacedParentApplicationId = this.getParentApplicationNamespacedId(parentApplicationId);
            ownerIds = Collections.singleton(namespacedParentApplicationId);
        }
        
        // determine seconday keys
        final Set<String> secondaryKeys = generalRegistryInstance.getSecondaryKeys(transfer);
        final GT_StageId primaryId = generalRegistryInstance.getId(transfer);
        this.registerObject(transfer, primaryId, ownerIds, secondaryKeys);
    }
    
    /* =============== even notification proxy stuff ================== */
    
    
    public void objectCreated(RegistryEvent<GT_StageId, GT_Registry> event) {
       this.notifyOnCreate(event.getId());
    }

    public void objectLoaded(RegistryEvent<GT_StageId, GT_Registry> event) {
        this.notifyOnLoad(event.getId());
    }

    public void objectRemoved(RegistryEvent<GT_StageId, GT_Registry> event) {
        this.notifyOnRemove(event.getId());
    }

    public void objectUpdated(RegistryEvent<GT_StageId, GT_Registry> event) {
       this.notifyOnUpdate(event.getId());
    }

    /* =============== AbstractRegistry method implementations ============ */
    protected final IRegistryStore getRegistryStore() {
        return this.mapRegistryStore;
    }
    
    /**
     * Creates the Set if it does not exist, adds the ID to the Set.
     * 
     * @see org.jasig.portal.portlet.registry.AbstractRegistry#updateIdSetOnAdd(org.jasig.portal.core.ObjectId, org.jasig.portal.core.ObjectId, java.util.Set)
     */
    protected Tuple<Set<GT_StageId>, Boolean> updateIdSetOnAdd(GT_StageId primaryId, ObjectId parentId, Set<GT_StageId> ownedIdSet) {
        Boolean changed = Boolean.FALSE;
        if (ownedIdSet == null) {
            ownedIdSet = new HashSet<GT_StageId>();
            ownedIdSet.add(primaryId);
            changed = Boolean.TRUE;
        }
        else if (!ownedIdSet.contains(primaryId)) {
            ownedIdSet.add(primaryId);
            changed = Boolean.TRUE;
        }

        return new Tuple<Set<GT_StageId>, Boolean>(ownedIdSet, changed);
    }

    /**
     * Removes the ID from the Set, returns null if there are no entries left in the Set.
     * 
     * @see org.jasig.portal.portlet.registry.AbstractRegistry#updateIdSetOnRemove(org.jasig.portal.core.ObjectId, org.jasig.portal.core.ObjectId, java.util.Set)
     */
    protected Tuple<Set<GT_StageId>, Boolean> updateIdSetOnRemove(GT_StageId primaryId, ObjectId parentId, Set<GT_StageId> ownedIdSet) {
        Boolean changed = Boolean.FALSE;
        if (ownedIdSet != null && ownedIdSet.contains(primaryId)) {
            ownedIdSet.remove(primaryId);
            changed = Boolean.TRUE;
        }
        return new Tuple<Set<GT_StageId>, Boolean>(ownedIdSet, changed);
    }
    
    /**
     * A registry store implementation that uses a {@link Map} to store objects
     * and key sets.
     */
    private class MapRegistryStore implements IRegistryStore, Serializable {
        private static final String KEY_SET_NAMESPACE = "KEY_SET__";
        private final Map<Serializable, Object> registry = Collections.synchronizedMap(new HashMap<Serializable, Object>());
        
        public void put(Serializable key, Object value) {
            this.registry.put(key, value);
        }

        public Object get(Serializable key) {
            return this.registry.get(key);
        }

        public void remove(Serializable key) {
            this.registry.remove(key);
        }

        public void putKeySet(Object key, Set<Serializable> idSet) {
            final NamespacingKeyWrapper setKey = this.getSetKey(key);
            this.registry.put(setKey, idSet);
        }

        @SuppressWarnings("unchecked")
        public Set<Serializable> getKeySet(Object key) {
            final NamespacingKeyWrapper setKey = this.getSetKey(key);
            return (Set<Serializable>)this.registry.get(setKey);
        }

        @SuppressWarnings("unchecked")
        public Set<Serializable> removeKeySet(Object key) {
            final NamespacingKeyWrapper setKey = this.getSetKey(key);
            return (Set<Serializable>)this.registry.remove(setKey);
        }

        private NamespacingKeyWrapper getSetKey(Object key) {
            return new NamespacingKeyWrapper(KEY_SET_NAMESPACE, key);
        }
    }
}
