/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.general;

import java.util.Set;

import org.jasig.portal.core.ObjectId;
import org.jasig.portal.portlet.dao.IPortletApplicationDefinitionDao;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.factory.IPortletDefinitionFactory;
import org.jasig.portal.portlet.registry.DomainObjectCreationException;
import org.jasig.portal.portlet.registry.event.IPortletApplicationDefinitionEventListener;
import org.jasig.portal.portlet.transfer.IPortletApplicationDefinitionTransfer;
import org.jasig.portal.portlet.transfer.impl.TransferUtils;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.util.Assert;

/**
 * The portlet application definition registry implementation.
 * 
 * Template types to actual types
 * <ol>
 *   <li>GT_Stage - {@link IPortletApplicationDefinition}</li>
 *   <li>GT_StageId - {@link PortletApplicationDefinitionId}</li>
 *   <li>GT_StageTransfer - {@link IPortletApplicationDefinitionTransfer}</li>
 *   <li>GT_ParentStageId - {@link PortletApplicationDeploymentId}</li>
 *   <li>GT_ParentApplicationStageId - {@link ObjectId}</li>
 *   <li>GT_Registry - {@link IPortletApplicationDefinitionRegistry}</li> 
 *   <li>GT_RegistryListener - {@link IPortletApplicationDefinitionEventListener}</li>
 * </ol>
 * 
 * @author Michael Ivanov
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletApplicationDefinitionRegistryImpl
        extends GenericPortletRegistry<IPortletApplicationDefinition, PortletApplicationDefinitionId, IPortletApplicationDefinitionTransfer, PortletApplicationDeploymentId, ObjectId, IPortletApplicationDefinitionRegistry, IPortletApplicationDefinitionEventListener>
        implements IPortletApplicationDefinitionRegistry {

    private IPortletApplicationDefinitionDao portletApplicationDefinitionDao;
    private IPortletDefinitionFactory portletDefinitionFactory;
    
    
    /**
     * @param portletApplicationDefinitionDao The portletApplicationDefinitionDao to set.
     */
    @Required
    public void setPortletApplicationDefinitionDao(IPortletApplicationDefinitionDao portletApplicationDefinitionDao) {
        this.portletApplicationDefinitionDao = portletApplicationDefinitionDao;
    }
    /**
     * @param portletDefinitionFactory The portletDefinitionFactory to set.
     */
    @Required
    public void setPortletDefinitionFactory(IPortletDefinitionFactory portletDefinitionFactory) {
        this.portletDefinitionFactory = portletDefinitionFactory;
    }

    //********** IPortletApplicationDefinitionRegistry Methods **********//
    
    public Set<IPortletApplicationDefinition> getPortletApplicationDefinitions() {
        return this.getAll(); 
    }
    
    public Set<IPortletApplicationDefinition> getPortletApplicationDefinitions(PortletApplicationDeploymentId portAppDepId) {
        return this.getForParent(portAppDepId);
    }

    public IPortletApplicationDefinition createPortletApplicationDefinition(IPortletApplicationDefinitionTransfer transfer) {
        return this.create(transfer);
    }
    
    public IPortletApplicationDefinition getPortletApplicationDefinition(PortletApplicationDefinitionId portAppDefId) {
        return this.get(portAppDefId);
    }

    public void updatePortletApplicationDefinition(IPortletApplicationDefinition newPortAppDef) {
        Assert.notNull(newPortAppDef, "newPortAppDef can not be null");
        this.update(newPortAppDef.getPortletApplicationDefinitionTransfer());
    }

    public void removePortletApplicationDefinition(PortletApplicationDefinitionId portAppDefId) {
        this.remove(portAppDefId);
    }
    

    /* =========== GenericPortletRegistry Methods ==================== */

    @Override
    IPortletApplicationDefinitionTransfer addTransferObjectToDAO(IPortletApplicationDefinitionTransfer transfer) {
        // create new transfer
        final IPortletApplicationDefinitionTransfer newTransfer=this.portletApplicationDefinitionDao.createPortletApplicationDefinition(transfer.getPortletApplicationDeploymentId());
        TransferUtils.deepCopyPortletApplicationDefinitionTransferProperties(transfer, newTransfer);
        // persist the new transfer
        this.portletApplicationDefinitionDao.storePortletApplicationDefinition(newTransfer);
        return newTransfer;
    }

    @Override
    IPortletApplicationDefinition createDomainObjectFromTransfer(IPortletApplicationDefinitionTransfer transfer) {
        try {
            return this.portletDefinitionFactory.createPortletApplicationDefinition(transfer);
        }
        catch (DomainObjectCreationException e) {
            if (logger.isInfoEnabled()) {
                logger.info("error creating application definition : id=" + transfer.getPortletApplicationDefinitionId(), e);
            }
            return null;
        }
    }
    
    
    @Override
    ObjectId getApplicationId(IPortletApplicationDefinitionTransfer transfer) {
        return null;
    }

    @Override
    PortletApplicationDefinitionId getId(IPortletApplicationDefinitionTransfer transfer) {
        return transfer.getPortletApplicationDefinitionId();
    }

    @Override
    Set<String> getSecondaryKeys(IPortletApplicationDefinitionTransfer transfer) {
        return null;
    }

    @Override
    IPortletApplicationDefinitionTransfer getTransferObjectBySecondaryKeyFromDAO(String key) {
        return null;
    }

    @Override
    IPortletApplicationDefinitionTransfer getTransferObjectFromDAO(PortletApplicationDefinitionId id) {
        return this.portletApplicationDefinitionDao.getPortletApplicationDefinition(id);
    }

    @Override
    Set<IPortletApplicationDefinitionTransfer> getTransferObjectsForApplicationIdAndParentIdFromDAO(ObjectId applicationId, PortletApplicationDeploymentId parentId) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    Set<IPortletApplicationDefinitionTransfer> getTransferObjectsForApplicationIdFromDAO(ObjectId applicationId) {
        throw new UnsupportedOperationException("already at application stage");
    }

    @Override
    Set<IPortletApplicationDefinitionTransfer> getTransferObjectsForParentIdFromDAO(PortletApplicationDeploymentId parentId) {
        return this.portletApplicationDefinitionDao.getPortletApplicationDefinitions(parentId);
    }

    @Override
    Set<IPortletApplicationDefinitionTransfer> getTransferObjectsFromDAO() {
        return this.portletApplicationDefinitionDao.getPortletApplicationDefinitions();
    }

    @Override
    void removeObjectFromDAO(PortletApplicationDefinitionId id) {
        this.portletApplicationDefinitionDao.removePortletApplicationDefinition(id);
    }

    @Override
    void updateObjectInDAO(IPortletApplicationDefinitionTransfer transfer) {
        this.portletApplicationDefinitionDao.storePortletApplicationDefinition(transfer);
    }

    @Override
    PortletApplicationDeploymentId getParentId(IPortletApplicationDefinitionTransfer transfer) {
        return transfer.getPortletApplicationDeploymentId();
    }
}
