/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.general;

import java.util.Set;

import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.core.ObjectId;
import org.jasig.portal.portlet.dao.IPortletApplicationEntityDao;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.entity.IPortletApplicationEntity;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.factory.IPortletEntityFactory;
import org.jasig.portal.portlet.registry.DomainObjectCreationException;
import org.jasig.portal.portlet.registry.event.IPortletApplicationEntityEventListener;
import org.jasig.portal.portlet.transfer.IPortletApplicationEntityTransfer;
import org.jasig.portal.portlet.transfer.impl.TransferUtils;
import org.jasig.portal.security.UserId;

/**
 * The portlet application entity registry implementation.
 * 
 * Template types to actual types
 * <ol>
 *   <li>GT_Stage - {@link IPortletApplicationEntity}</li>
 *   <li>GT_StageId - {@link PortletApplicationEntityId}</li>
 *   <li>GT_StageTransfer - {@link IPortletApplicationEntityTransfer}</li>
 *   <li>GT_ParentStageId - {@link PortletApplicationDefinitionId}</li>
 *   <li>GT_ParentApplicationStageId - {@link ObjectId}</li>
 *   <li>GT_Registry - {@link IPortletApplicationEntityRegistry}</li> 
 *   <li>GT_RegistryListener - {@link IPortletApplicationEntityEventListener}</li>
 * </ol>
 * 
 * 
 * @author Michael Ivanov
 * @author Eric Dalquist
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletApplicationEntityRegistryImpl
        extends GenericPortletRegistry<IPortletApplicationEntity, PortletApplicationEntityId, IPortletApplicationEntityTransfer, PortletApplicationDefinitionId, ObjectId, IPortletApplicationEntityRegistry, IPortletApplicationEntityEventListener>
        implements IPortletApplicationEntityRegistry {
    
    private final UserId userId;
    private final IPortletApplicationEntityDao portletApplicationEntityDao;
    private final IPortletEntityFactory portletEntityFactory;
    
    public PortletApplicationEntityRegistryImpl(UserId userId, IPortletApplicationEntityDao portletApplicationEntityDao, IPortletEntityFactory portletEntityFactory, ISimpleCacheProviderFacade cache) {
        if (userId == null) {
            throw new IllegalArgumentException("userId can not be null");
        }
        if (portletApplicationEntityDao == null) {
            throw new IllegalArgumentException("portletApplicationEntityDao can not be null");
        }
        if (portletEntityFactory == null) {
            throw new IllegalArgumentException("portletEntityFactory can not be null");
        }
        if (cache == null) {
            throw new IllegalArgumentException("cache can not be null");
        }
        
        this.userId = userId;
        this.portletApplicationEntityDao = portletApplicationEntityDao;
        this.portletEntityFactory = portletEntityFactory;
        this.setCache(cache);
    }
    
    
    //********** IPortletApplicationDeploymentRegistry Methods **********//
    
    public UserId getUserId() {
        return this.userId;
    }

    public Set<IPortletApplicationEntity> getPortletApplicationEntities(PortletApplicationDefinitionId portAppDefId) {
        return this.getForParent(portAppDefId);
    }

    public IPortletApplicationEntity createPortletApplicationEntity(IPortletApplicationEntityTransfer portAppEntTransfer) {
        return this.create(portAppEntTransfer);
    }
    
    public IPortletApplicationEntity getPortletApplicationEntity(PortletApplicationEntityId portAppEntId) {
        return this.get(portAppEntId);
    }

    public void removePortletApplicationEntity(PortletApplicationEntityId portAppEntId) {
        this.remove(portAppEntId);
    }
    
    
    /* ================ abstract method implementation ================= */

    @Override
    IPortletApplicationEntityTransfer addTransferObjectToDAO(IPortletApplicationEntityTransfer transfer) {
        // create new transfer
        final IPortletApplicationEntityTransfer newTransfer=this.portletApplicationEntityDao.createPortletApplicationEntity(this.userId,transfer.getPortletApplicationDefinitionId());
        TransferUtils.deepCopyPortletApplicationEntityTransferProperties(transfer, newTransfer);
        // persist the new transfer
        this.portletApplicationEntityDao.storePortletApplicationEntity(this.userId,newTransfer);
        return newTransfer;
    }

    @Override
    IPortletApplicationEntity createDomainObjectFromTransfer(IPortletApplicationEntityTransfer transfer) {
        try {
            return this.portletEntityFactory.createPortletApplicationEntity(transfer);
        }
        catch (DomainObjectCreationException e) {
            if (logger.isInfoEnabled()) {
                logger.info("error creating portlet application entity : id=" + transfer.getPortletApplicationEntityId(), e);
            }
            return null;
        }
    }

    @Override
    ObjectId getApplicationId(IPortletApplicationEntityTransfer transfer) {
        return null;
    }

    @Override
    PortletApplicationEntityId getId(IPortletApplicationEntityTransfer transfer) {
        return transfer.getPortletApplicationEntityId();
    }

    @Override
    PortletApplicationDefinitionId getParentId(IPortletApplicationEntityTransfer transfer) {
        return transfer.getPortletApplicationDefinitionId();
    }

    @Override
    Set<String> getSecondaryKeys(IPortletApplicationEntityTransfer transfer) {
        return null;
    }

    @Override
    IPortletApplicationEntityTransfer getTransferObjectBySecondaryKeyFromDAO(String key) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    IPortletApplicationEntityTransfer getTransferObjectFromDAO(PortletApplicationEntityId id) {
        return this.portletApplicationEntityDao.getPortletApplicationEntity(this.userId, id);
    }

    @Override
    Set<IPortletApplicationEntityTransfer> getTransferObjectsForApplicationIdAndParentIdFromDAO(ObjectId applicationId, PortletApplicationDefinitionId parentId) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    Set<IPortletApplicationEntityTransfer> getTransferObjectsForApplicationIdFromDAO(ObjectId applicationId) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    Set<IPortletApplicationEntityTransfer> getTransferObjectsForParentIdFromDAO(PortletApplicationDefinitionId parentId) {
        return this.portletApplicationEntityDao.getPortletApplicationEntities(this.userId, parentId);
    }

    @Override
    Set<IPortletApplicationEntityTransfer> getTransferObjectsFromDAO() {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    void removeObjectFromDAO(PortletApplicationEntityId id) {
        this.portletApplicationEntityDao.removePortletApplicationEntity(this.userId, id);
        
    }

    @Override
    void updateObjectInDAO(IPortletApplicationEntityTransfer transfer) {
        this.portletApplicationEntityDao.storePortletApplicationEntity(this.userId, transfer);
    }
}