/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.general;

import java.util.Collections;
import java.util.Set;

import org.jasig.portal.portlet.dao.IPortletDefinitionDao;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.deployment.PortletDeploymentId;
import org.jasig.portal.portlet.factory.IPortletDefinitionFactory;
import org.jasig.portal.portlet.registry.DomainObjectCreationException;
import org.jasig.portal.portlet.registry.event.IPortletDefinitionEventListener;
import org.jasig.portal.portlet.transfer.IPortletDefinitionTransfer;
import org.jasig.portal.portlet.transfer.impl.TransferUtils;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.util.Assert;

/**
 * The portlet definition registry implementation.
 * 
 * Template types to actual types
 * <ol>
 *   <li>GT_Stage - {@link IPortletDefinition}</li>
 *   <li>GT_StageId - {@link PortletDefinitionId}</li>
 *   <li>GT_StageTransfer - {@link IPortletDefinitionTransfer}</li>
 *   <li>GT_ParentStageId - {@link PortletDeploymentId}</li>
 *   <li>GT_ParentApplicationStageId - {@link PortletApplicationDefinitionId}</li>
 *   <li>GT_Registry - {@link IPortletDefinitionRegistry}</li> 
 *   <li>GT_RegistryListener - {@link IPortletDefinitionEventListener}</li>
 * </ol>
 * 
 * The portlet definition registry implementation.
 * @author Michael Ivanov
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletDefinitionRegistryImpl
        extends GenericPortletRegistry<IPortletDefinition, PortletDefinitionId, IPortletDefinitionTransfer, PortletDeploymentId, PortletApplicationDefinitionId, IPortletDefinitionRegistry, IPortletDefinitionEventListener>
        implements IPortletDefinitionRegistry {

    private IPortletDefinitionDao portletDefinitionDao;
    private IPortletDefinitionFactory portletDefinitionFactory;
    
    /**
     * @param portletDefinitionDao The portletDefinitionDao to set.
     */
    @Required
    public void setPortletDefinitionDao(IPortletDefinitionDao portletDefinitionDao) {
        this.portletDefinitionDao = portletDefinitionDao;
    }
    
    /**
     * @param portletDefinitionFactory The portletDefinitionFactory to set.
     */
    @Required
    public void setPortletDefinitionFactory(IPortletDefinitionFactory portletDefinitionFactory) {
        this.portletDefinitionFactory = portletDefinitionFactory;
    }
    
    //********** IPortletDefinitionRegistry Methods **********//
    
    public Set<IPortletDefinition> getPortletDefinitions(PortletApplicationDefinitionId portAppDefId) {
        return this.getForApplication(portAppDefId);
    }
    
    public IPortletDefinition createPortletDefinition(IPortletDefinition portDef) {
        Assert.notNull(portDef, "portDef can not be null");
        return this.create(portDef.getPortletDefinitionTransfer());
    }
    
    public IPortletDefinition getPortletDefinition(PortletDefinitionId portDefId) {
        return this.get(portDefId);
    }
    
    public IPortletDefinition getPortletDefinitionByFunctionalName(String funcName) {
        return this.getBySecondaryKey(funcName);
    }
    
    public void updatePortletDefinition(IPortletDefinition newPortDef) {
        Assert.notNull(newPortDef, "newPortDef can not be null");
        this.update(newPortDef.getPortletDefinitionTransfer());
    }

    public void removePortletDefinition(PortletDefinitionId portDefId) {
        this.remove(portDefId);
    }
    

    /* =========== GenericPortletRegistry Methods ==================== */

    @Override
    IPortletDefinitionTransfer addTransferObjectToDAO(IPortletDefinitionTransfer transfer) {
        // create new transfer
        final IPortletDefinitionTransfer newTransfer = this.portletDefinitionDao.createPortletDefinition(transfer.getPortletApplicationDefinitionId(),
                                                                                                      transfer.getPortletDeploymentId(),
                                                                                                      transfer.getFunctionalName());
        TransferUtils.deepCopyPortletDefinitionTransferProperties(transfer, newTransfer);
        // persist the new transfer
        this.portletDefinitionDao.storePortletDefinition(newTransfer);
        return newTransfer;
    }

    @Override
    IPortletDefinition createDomainObjectFromTransfer(IPortletDefinitionTransfer transfer) {
        try {
            return this.portletDefinitionFactory.createPortletDefinition(transfer);
        }
        catch (DomainObjectCreationException e) {
            if (logger.isInfoEnabled()) {
                logger.info("error creating portlet definition : id=" + transfer.getPortletDefinitionId(), e);
            }
            return null;
        }
    }

    @Override
    PortletApplicationDefinitionId getApplicationId(IPortletDefinitionTransfer transfer) {
        return transfer.getPortletApplicationDefinitionId();
    }

    @Override
    PortletDefinitionId getId(IPortletDefinitionTransfer transfer) {
        return transfer.getPortletDefinitionId();
    }

    @Override
    PortletDeploymentId getParentId(IPortletDefinitionTransfer transfer) {
        return transfer.getPortletDeploymentId();
    }

    @Override
    Set<String> getSecondaryKeys(IPortletDefinitionTransfer transfer) {
        return Collections.<String>singleton(transfer.getFunctionalName());
    }

    @Override
    IPortletDefinitionTransfer getTransferObjectBySecondaryKeyFromDAO(String key) {
        return this.portletDefinitionDao.getPortletDefinitionByFunctionalName(key);
    }

    @Override
    IPortletDefinitionTransfer getTransferObjectFromDAO(PortletDefinitionId id) {
        return this.portletDefinitionDao.getPortletDefinition(id);
    }

    @Override
    Set<IPortletDefinitionTransfer> getTransferObjectsForApplicationIdAndParentIdFromDAO(PortletApplicationDefinitionId applicationId, PortletDeploymentId parentId) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    Set<IPortletDefinitionTransfer> getTransferObjectsForApplicationIdFromDAO(PortletApplicationDefinitionId applicationId) {
        return this.portletDefinitionDao.getPortletDefinitions(applicationId);
    }

    @Override
    Set<IPortletDefinitionTransfer> getTransferObjectsForParentIdFromDAO(PortletDeploymentId parentId) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    Set<IPortletDefinitionTransfer> getTransferObjectsFromDAO() {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    void removeObjectFromDAO(PortletDefinitionId id) {
       this.portletDefinitionDao.removePortletDefinition(id);
        
    }

    @Override
    void updateObjectInDAO(IPortletDefinitionTransfer transfer) {
        this.portletDefinitionDao.storePortletDefinition(transfer);
        
    }
}
