/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.general;

import java.util.Set;

import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.portlet.dao.IPortletEntityDao;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.entity.IPortletEntity;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.factory.IPortletEntityFactory;
import org.jasig.portal.portlet.registry.DomainObjectCreationException;
import org.jasig.portal.portlet.registry.event.IPortletEntityEventListener;
import org.jasig.portal.portlet.transfer.IPortletEntityTransfer;
import org.jasig.portal.portlet.transfer.impl.TransferUtils;
import org.jasig.portal.security.UserId;
import org.springframework.util.Assert;

/**
 * The portlet entity registry implementation.
 * 
 * Template types to actual types
 * <ol>
 *   <li>GT_Stage - {@link IPortletEntity}</li>
 *   <li>GT_StageId - {@link PortletEntityId}</li>
 *   <li>GT_StageTransfer - {@link IPortletEntityTransfer}</li>
 *   <li>GT_ParentStageId - {@link PortletDefinitionId}</li>
 *   <li>GT_ParentApplicationStageId - {@link PortletApplicationEntityId}</li>
 *   <li>GT_Registry - {@link IPortletEntityRegistry}</li> 
 *   <li>GT_RegistryListener - {@link IPortletEntityEventListener}</li>
 * </ol>
 * 
 * The portlet entity registry implementation.
 * @author Michael Ivanov
 * @author Eric Dalquist 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletEntityRegistryImpl
        extends GenericPortletRegistry<IPortletEntity, PortletEntityId, IPortletEntityTransfer, PortletDefinitionId, PortletApplicationEntityId, IPortletEntityRegistry, IPortletEntityEventListener>
        implements IPortletEntityRegistry {

    private final IPortletEntityDao portletEntityDao;
    private final IPortletEntityFactory portletEntityFactory;
    private final UserId userId;
    
    public PortletEntityRegistryImpl(UserId userId, IPortletEntityDao portletEntityDao, IPortletEntityFactory portletEntityFactory, ISimpleCacheProviderFacade cache) {
        if (userId == null) {
            throw new IllegalArgumentException("userId can not be null");
        }
        if (portletEntityDao == null) {
            throw new IllegalArgumentException("portletEntityDao can not be null");
        }
        if (portletEntityFactory == null) {
            throw new IllegalArgumentException("portletEntityFactory can not be null");
        }
        if (cache == null) {
            throw new IllegalArgumentException("cache can not be null");
        }
        
        this.userId = userId;
        this.portletEntityDao = portletEntityDao;
        this.portletEntityFactory = portletEntityFactory;
        this.setCache(cache);
    }
    
    
    //********** IPortletEntityRegistry Methods **********//
    
    public UserId getUserId() {
        return this.userId;
    }

    public Set<IPortletEntity> getPortletEntities() {
        return this.getAll();
    }
    
    public Set<IPortletEntity> getPortletEntities(PortletApplicationEntityId portAppEntId) {
        return this.getForApplication(portAppEntId);
    }
    
    public Set<IPortletEntity> getPortletEntities(PortletApplicationEntityId portAppEntId, PortletDefinitionId portDefId) {
        return this.getForApplicationAndParent(portAppEntId, portDefId);
    }

    public IPortletEntity createPortletEntity(IPortletEntityTransfer portEntityTransfer) {
        return this.create(portEntityTransfer);
    }
    
    public  IPortletEntity getPortletEntity(PortletEntityId portEntId) {
        return this.get(portEntId);
    }
    
    public void updatePortletEntity(IPortletEntity newPortEnt) {
        Assert.notNull(newPortEnt, "newPortEnt can not be null");
        this.update(newPortEnt.getPortletEntityTransfer());
    }

    public void removePortletEntity(PortletEntityId portEntId) {
        this.remove(portEntId);
    }
    
    public IPortletEntity clonePortletEntity(PortletEntityId portEntId) {
        Assert.notNull(portEntId, "portEntId can not be null");

        final IPortletEntity baseEnt = this.getPortletEntity(portEntId);
        if (baseEnt != null) {
            return this.create(baseEnt.getPortletEntityTransfer());
        }
        else {
            return null;
        }
    }
    
    
    /* ================ abstract method implementation ================= */

    @Override
    IPortletEntityTransfer addTransferObjectToDAO(IPortletEntityTransfer transfer) {
        // create new transfer
        final IPortletEntityTransfer newTransfer = this.portletEntityDao.createPortletEntity(this.userId,
                                                                                          transfer.getPortletApplicationEntityId(),
                                                                                          transfer.getPortletDefinitionId());
        TransferUtils.deepCopyPortletEntityTransferProperties(transfer, newTransfer);
        // persist the new transfer
        this.portletEntityDao.storePortletEntity(this.userId, newTransfer);
        return newTransfer;
    }

    @Override
    IPortletEntity createDomainObjectFromTransfer(IPortletEntityTransfer transfer) {
        try {
            return this.portletEntityFactory.createPortletEntity(this.userId, transfer);
        }
        catch (DomainObjectCreationException e) {
            if (logger.isInfoEnabled()) {
                logger.info("error creating portlet entity : id=" + transfer.getPortletEntityId(), e);
            }
            return null;
        }
    }

    @Override
    PortletApplicationEntityId getApplicationId(IPortletEntityTransfer transfer) {
        return transfer.getPortletApplicationEntityId();
    }

    @Override
    PortletEntityId getId(IPortletEntityTransfer transfer) {
        return transfer.getPortletEntityId();
    }

    @Override
    PortletDefinitionId getParentId(IPortletEntityTransfer transfer) {
        return transfer.getPortletDefinitionId();
    }

    @Override
    Set<String> getSecondaryKeys(IPortletEntityTransfer transfer) {
        return null;
    }

    @Override
    IPortletEntityTransfer getTransferObjectBySecondaryKeyFromDAO(String key) {
        return null;
    }

    @Override
    IPortletEntityTransfer getTransferObjectFromDAO(PortletEntityId id) {
        return this.portletEntityDao.getPortletEntity(this.userId, id);
    }

    @Override
    Set<IPortletEntityTransfer> getTransferObjectsForApplicationIdAndParentIdFromDAO(PortletApplicationEntityId applicationId, PortletDefinitionId parentId) {
        return this.portletEntityDao.getPortletEntities(this.userId, applicationId, parentId);
    }

    @Override
    Set<IPortletEntityTransfer> getTransferObjectsForApplicationIdFromDAO(PortletApplicationEntityId applicationId) {
        return this.portletEntityDao.getPortletEntities(this.userId, applicationId);
    }

    @Override
    Set<IPortletEntityTransfer> getTransferObjectsForParentIdFromDAO(PortletDefinitionId parentId) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    Set<IPortletEntityTransfer> getTransferObjectsFromDAO() {
        return this.portletEntityDao.getPortletEntities(this.userId);
    }

    @Override
    void removeObjectFromDAO(PortletEntityId id) {
        this.portletEntityDao.removePortletEntity(this.userId, id);
    }

    @Override
    void updateObjectInDAO(IPortletEntityTransfer transfer) {
        this.portletEntityDao.storePortletEntity(this.userId, transfer);
    }
}