/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.general;

import java.util.Set;

import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.core.ObjectId;
import org.jasig.portal.portlet.dao.IPortletWindowDao;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.factory.IPortletWindowFactory;
import org.jasig.portal.portlet.registry.DomainObjectCreationException;
import org.jasig.portal.portlet.registry.event.IPortletWindowEventListener;
import org.jasig.portal.portlet.transfer.IPortletWindowTransfer;
import org.jasig.portal.portlet.transfer.impl.TransferUtils;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.security.UserId;
import org.springframework.util.Assert;

/**
 * The portlet entity registry implementation.
 * 
 * Template types to actual types
 * <ol>
 *   <li>GT_Stage - {@link IPortletWindow}</li>
 *   <li>GT_StageId - {@link PortletWindowId}</li>
 *   <li>GT_StageTransfer - {@link IPortletWindowTransfer}</li>
 *   <li>GT_ParentStageId - {@link PortletEntityId}</li>
 *   <li>GT_ParentApplicationStageId - {@link ObjectId}</li>
 *   <li>GT_Registry - {@link IPortletWindowRegistry}</li> 
 *   <li>GT_RegistryListener - {@link IPortletWindowEventListener}</li>
 * </ol>
 * 
 * The portlet window registry implementation.
 * @author Michael Ivanov, mvi@immagic.com
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */

public class PortletWindowRegistryImpl
        extends GenericPortletRegistry<IPortletWindow, PortletWindowId, IPortletWindowTransfer, PortletEntityId, ObjectId, IPortletWindowRegistry, IPortletWindowEventListener>
        implements IPortletWindowRegistry {

    private final IPortletWindowDao portletWindowDao;
    private final IPortletWindowFactory portletWindowFactory;
    private final UserId userId;
    
    public PortletWindowRegistryImpl(UserId userId, IPortletWindowDao portletWindowDao, IPortletWindowFactory portletWindowFactory, ISimpleCacheProviderFacade cache) {
        if (userId == null) {
            throw new IllegalArgumentException("userId can not be null");
        }
        if (portletWindowDao == null) {
            throw new IllegalArgumentException("portletWindowDao can not be null");
        }
        if (portletWindowFactory == null) {
            throw new IllegalArgumentException("portletWindowFactory can not be null");
        }
        if (cache == null) {
            throw new IllegalArgumentException("cache can not be null");
        }
        
        this.userId = userId;
        this.portletWindowDao = portletWindowDao;
        this.portletWindowFactory = portletWindowFactory;
        this.setCache(cache);
    }
    
    
    //********** IPortletWindowRegistry Methods **********//
    
    public UserId getUserId() {
        return this.userId;
    }
    
    public Set<IPortletWindow> getPortletWindows(PortletEntityId portEntId) {
        return this.getForParent(portEntId);
    }
    
    public IPortletWindow createPortletWindow(IPortletWindowTransfer portWindowTransfer) {
        return this.create(portWindowTransfer);
    }

    public IPortletWindow getPortletWindow(PortletWindowId portWinId) {
        return this.get(portWinId);
    }
    
    public void updatePortletWindow(IPortletWindow newPortWin) {
        Assert.notNull(newPortWin, "newPortWin can not be null");
        this.update(newPortWin.getPortletWindowTransfer());
    }
    
    public synchronized void removePortletWindow(PortletWindowId portWinId) {
        this.remove(portWinId);
    }
    
    
    /* ================ abstract method implementation ================= */

    @Override
    IPortletWindowTransfer addTransferObjectToDAO(IPortletWindowTransfer transfer) {
        // create new transfer
        final IPortletWindowTransfer newTransfer = this.portletWindowDao.createPortletWindow(transfer.getPortletEntityId());
        TransferUtils.deepCopyPortletWindowTransferProperties(transfer, newTransfer);
        // persist the new transfer
        this.portletWindowDao.storePortletWindow(newTransfer);
        return newTransfer;
    }

    @Override
    IPortletWindow createDomainObjectFromTransfer(IPortletWindowTransfer transfer) {
        try {
            return this.portletWindowFactory.createPortletWindow(this.userId, transfer);
        }
        catch (DomainObjectCreationException e) {
            if (logger.isInfoEnabled()) {
                logger.info("error creating portlet window : id=" + transfer.getPortletWindowId(), e);
            }
            return null;
        }
    }

    @Override
    ObjectId getApplicationId(IPortletWindowTransfer transfer) {
        return null;
    }

    @Override
    PortletWindowId getId(IPortletWindowTransfer transfer) {
        return transfer.getPortletWindowId();
    }

    @Override
    PortletEntityId getParentId(IPortletWindowTransfer transfer) {
        return transfer.getPortletEntityId();
    }

    @Override
    Set<String> getSecondaryKeys(IPortletWindowTransfer transfer) {
        return null;
    }

    @Override
    IPortletWindowTransfer getTransferObjectBySecondaryKeyFromDAO(String key) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    IPortletWindowTransfer getTransferObjectFromDAO(PortletWindowId id) {
        return this.portletWindowDao.getPortletWindow(id);
    }

    @Override
    Set<IPortletWindowTransfer> getTransferObjectsForApplicationIdAndParentIdFromDAO(ObjectId applicationId, PortletEntityId parentId) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    Set<IPortletWindowTransfer> getTransferObjectsForApplicationIdFromDAO(ObjectId applicationId) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    Set<IPortletWindowTransfer> getTransferObjectsForParentIdFromDAO(PortletEntityId parentId) {
        return this.portletWindowDao.getPortletWindows(parentId);
    }

    @Override
    Set<IPortletWindowTransfer> getTransferObjectsFromDAO() {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    void removeObjectFromDAO(PortletWindowId id) {
       this.portletWindowDao.removePortletWindow(id);
        
    }

    @Override
    void updateObjectInDAO(IPortletWindowTransfer transfer) {
        this.portletWindowDao.storePortletWindow(transfer);
    }
}