/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.rendering;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.servlet.http.HttpServletResponse;

import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.url.IRequestParameterProcessor;
import org.jasig.portal.url.IUrlConstructor;
import org.jasig.portal.url.IUrlConstructorProvider;
import org.jasig.portal.url.IWritableHttpServletRequest;
import org.jasig.portal.url.UrlInfo;
import org.jasig.portal.url.syntax.IUrlBasicSyntaxProvider;

/**
 *  Parses and directs requests to refresh/restart protlets that are
 *  currently being replaced by the error portlet.
 *  The class also implements associated {@link org.jasig.portal.url.IUrlConstructorProvider}
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class ErrorPortletParameterProcessor implements IRequestParameterProcessor, IUrlConstructorProvider {
    String processorPrefix="error";
    IPortletWindowManager portletWindowManager;
    IUrlBasicSyntaxProvider syntaxProvider;
    
    final static String RESTART_COMMAND="restart";
    final static String RETRY_COMMAND="retry";
    final static String REFRESH_COMMAND="refresh";


    public boolean canEvaluateVariables() {
        return false;
    }

    public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res) {
        // process restarts
        String[] restartValues=req.getParameterValues(syntaxProvider.incorporateProcessorPrefix(processorPrefix,RESTART_COMMAND));
        if(restartValues!=null && restartValues.length>0) {
            for (int i = 0; i < restartValues.length; i++) {
                PortletWindowId portletWindowId=new PortletWindowId(restartValues[i]);
                portletWindowManager.restartPortletWindow(portletWindowId);
            }
        }
        
        // process retry requests
        String[] retryValues=req.getParameterValues(syntaxProvider.incorporateProcessorPrefix(processorPrefix,RETRY_COMMAND));
        if(retryValues!=null && retryValues.length>0) {
            for (int i = 0; i < retryValues.length; i++) {
                PortletWindowId portletWindowId=new PortletWindowId(retryValues[i]);
                portletWindowManager.refreshPortletWindow(portletWindowId,true);
            }
        }
        
        // process refresh requests
        String[] refreshValues=req.getParameterValues(syntaxProvider.incorporateProcessorPrefix(processorPrefix,REFRESH_COMMAND));
        if(refreshValues!=null && refreshValues.length>0) {
            for (int i = 0; i < refreshValues.length; i++) {
                PortletWindowId portletWindowId=new PortletWindowId(refreshValues[i]);
                portletWindowManager.refreshPortletWindow(portletWindowId,false);
            }
        }
        
        return true;
    }

    public void setSyntaxProvider(IUrlBasicSyntaxProvider syntaxProvider) {
        this.syntaxProvider = syntaxProvider;
    }


    /**
     * Return the url constructor name
     * @see org.jasig.portal.url.IUrlConstructorProvider#getName()
     */
    public String getName() {
        return IErrorPortletCommands.class.getName();
    }

    public IUrlConstructor getUrlConstructor() {
        return new ErrorUrlConstructor(syntaxProvider,processorPrefix);
    }
    
    static class ErrorUrlConstructor implements IErrorPortletCommands, IUrlConstructor {
        IUrlBasicSyntaxProvider syntaxProvider;
        String processorPrefix;
        Set refreshIds, retryIds, restartIds;
        
        
        public ErrorUrlConstructor(IUrlBasicSyntaxProvider provider, String processorPrefix) {
            super();
            this.syntaxProvider = provider;
            this.processorPrefix=processorPrefix;
        }

        public void refreshPortletWindow(PortletWindowId windowId) {
            if(refreshIds==null) { refreshIds=new HashSet(); }
            refreshIds.add(windowId.toString());
        }

        public void retryPortletWindow(PortletWindowId windowId) {
            if(retryIds==null) { retryIds=new HashSet(); }
            retryIds.add(windowId.toString());
        }
        
        public void restartPortletWindow(PortletWindowId windowId) {
            if(restartIds==null) { restartIds=new HashSet(); }
            restartIds.add(windowId.toString());
        }

        public void writeToUrl(UrlInfo urlInfo) {
            // add restart commands
            if(restartIds!=null) {
                String restartCommand=syntaxProvider.incorporateProcessorPrefix(processorPrefix,RESTART_COMMAND);
                for (Iterator iter = restartIds.iterator(); iter.hasNext();) {
                    String id = (String) iter.next();
                    urlInfo.addParameterValue(restartCommand,id);
                }
            }
            // add retry commands
            if(retryIds!=null) {
                String retryCommand=syntaxProvider.incorporateProcessorPrefix(processorPrefix,RETRY_COMMAND);
                for (Iterator iter = retryIds.iterator(); iter.hasNext();) {
                    String id = (String) iter.next();
                    urlInfo.addParameterValue(retryCommand,id);
                }
            }
            
            // add refresh commands
            if(refreshIds!=null) {
                String refreshCommand=syntaxProvider.incorporateProcessorPrefix(processorPrefix,REFRESH_COMMAND);
                for (Iterator iter = refreshIds.iterator(); iter.hasNext();) {
                    String id = (String) iter.next();
                    urlInfo.addParameterValue(refreshCommand,id);
                }
            }
        }
        
    }

    public void setProcessorPrefix(String processorPrefix) {
        this.processorPrefix = processorPrefix;
    }

    public void setPortletWindowManager(IPortletWindowManager portletWindowManager) {
        this.portletWindowManager = portletWindowManager;
    }


}
