/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.portlet.rendering;

import java.util.Map;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;

import org.jasig.portal.portlet.window.PortletWindowId;

/**
 * IPortletWindowManager describes a session-scoped entity that keeps track of <code>IPortletWindow</code> instances,
 * serving as a link between uPortal framework and Pluto container.
 * 
 * It maintains references to <code>IPortletWindow</code> instances, gathers render and action request parameters, initiates action
 * and render requests on the portlet windows and records portlet renering.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface IPortletWindowManager {
    /**
     * Specify all action request parameters incoming with the current request to
     * a targeted portlet window.
     * 
     * Note: this method would normally cause PortletWindowManager to invovke action request on the portlet
     * 
     * @param windowId id of the target portlet window
     * @param parameters map of parameter names to parameter value <code>String[]</code> arrays.
     */
    void setPortletActionRequestParameters(PortletWindowId windowId, Map<String,String[]> parameters);
    
    
    /**
     * Specify all render request parameters incoming with the current request to
     * a targeted portlet window.
     * 
     * @param windowId id of the target portlet window
     * @param parameters map of parameter names to parameter value <code>String[]</code> arrays.
     */
    void setPortletRenderRequestParameters(PortletWindowId windowId, Map<String,String[]> parameters);
    
    /**
     * Obtain content of a portlet window for a given rendering cycle.
     * @param windowId window whose content is requested
     * @return content of the current portlet state rendering
     */
    IPortletRenderContent getPortletRenderContent(PortletWindowId windowId);
    
    /**
     * Signals that render request processing on a given portlet window can be initiated 
     * 
     * Note: this method should be called when it is known with high probability that the
     * the <code>getPortletRendering()</code> method will be called on that portlet window.
     *  
     * @param windowId window that should be rendered
     */
    void startPortletRendering(PortletWindowId windowId);
    
    /**
     * Change portlet window mode for the current rendering cycle
     * @param windowId window id
     * @param mode new portlet mode
     */
    void setPortletMode(PortletWindowId windowId, PortletMode mode);
    
    /**
     * Change portlet window state for the current rendering cycle
     * @param windowId window id
     * @param state new portlet window state
     */
    void setWindowState(PortletWindowId windowId, WindowState state);
    
    /**
     * Returns portlet window mode for the current rendering cycle
     * @param windowId window id
     * @return mode a portlet mode
     */
    PortletMode getPortletMode(PortletWindowId windowId);
    
    /**
     * Returns portlet window state for the current rendering cycle
     * @param windowId window id
     * @return state a portlet window state
     */
    WindowState getWindowState(PortletWindowId windowId);
    
    /**
     * Obtain all portlet render parameters outstanding for this request.
     * This method is used to reconstruct additional render request parameters
     * that were passed alongside an action request to other portlets.
     * 
     * @return <code>Map</code>: <code>PortletWindowId</code>->(<code>Map</code>: <code>String</code>(parameter name) -> <code>String[]</code>(parameter values)),
     * or <code>null</code> if no render parameters were passed to any other portlets
     */
    Map<PortletWindowId,Map<String,String[]>> getPortletRenderParameters();
    
    /**
     * Replaces ErrorPortlet with the original window.
     * Session is kept the same as it was during the last request to that portlet window.
     * 
     * @param windowId window id
     * @param passLastRequestParameters <code>true</code> if the action or render parameters should be passed again, <code>false</code> if the parameter maps should be cleared
     */
    public void refreshPortletWindow(PortletWindowId windowId, boolean passLastRequestParameters);

    /**
     * Replaces ErrorPortlet with the original window.
     * Portlet window to be returned to its default state (with session and params cleared).
     * 
     * @param windowId window id
     */
    public void restartPortletWindow(PortletWindowId windowId);
    
    /**
     * Add a new portlet window manager event listener
     * @param listener
     */
    public void addPortletWindowManagerEventListener(IPortletWindowManagerEventListener listener);
    
    /**
     * Remove a portlet window manager event listener
     * @param listener
     */
    public void removePortletWindowManagerEventListener(IPortletWindowManagerEventListener listener);
}
