/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.portlet.rendering;

import java.util.Hashtable;
import java.util.Map;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.acegisecurity.context.SecurityContextHolder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.pluto.PortletContainer;
import org.apache.pluto.PortletWindow;
import org.jasig.portal.portlet.deployment.IPortletDeploymentInformationProvider;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.security.IUser;
import org.jasig.portal.security.acegi.TrackingContextHolder;
import org.jasig.portal.spring.RequestContextAwareThreadLocalCopyRunnable;

/**
 * MultithreadedPortletRenderingModel implements a basic multi-threaded portlet rendering model that
 * accounts for timeouts and thrown exceptions. The model is based on the uPortal2 functionality.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class MultithreadedPortletRenderingModel extends LinearPortletRenderingModel {
    
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(MultithreadedPortletRenderingModel.class);
    
    private ExecutorService executorService;
    private long defaultRenderTimeOutValue = 10000L;

    
    private static final String WORKER_MAP_KEY=".workers";
    
    // TODO: should have a separate thread that would clean up timed out, unrequested portlet rendering jobs
    
    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletRenderingModel#startPortletRendering(org.jasig.portal.portlet.window.IPortletWindow, java.util.Map, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public void startPortletRendering(IPortletWindow window, Map<String, String[]> renderParams, IUser user, HttpServletRequest req, HttpServletResponse res) throws PortletRenderingException {
        // check if the rendering has already been started
        RenderingWorker worker=getWorker(window.getPortletWindowId(),req);
        if(worker==null) {
            
            final PortletWindow plutoWindow=plutoDomainObjectHelper.getPortletWindow(window);
            
            // construct wrapped request and response
            PortletHttpServletRequestWrapper wrappedRequest=new PortletHttpServletRequestWrapper(req, renderParams, user);
            PortletHttpServletResponseWrapper wrappedResponse=new PortletHttpServletResponseWrapper(plutoWindow.getId(),req,res,renderContentAccessor);
            
            long timeOut=window.getRenderTimeOut();
            if(timeOut==IPortletDeploymentInformationProvider.UNDEFINED_TIMEOUT) {
                // use the default value
                timeOut=defaultRenderTimeOutValue;
            }
            worker=new RenderingWorker(plutoWindow,wrappedRequest,wrappedResponse,portletContainer,timeOut);
            setWorker(worker,window.getPortletWindowId(),req);
            // start rendering
            Future tracker=executorService.submit(new RequestContextAwareThreadLocalCopyRunnable(worker, true));
            worker.setTracker(tracker);
        }
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletRenderingModel#getPortletRenderContent(org.jasig.portal.portlet.window.IPortletWindow, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public IPortletRenderContent getPortletRenderContent(IPortletWindow window, HttpServletRequest req, HttpServletResponse res) throws PortletRenderingException {
        RenderingWorker worker=getWorker(window.getPortletWindowId(),req);
        
        if(worker!=null) {
            long timeOut=worker.getTimeOut();
            Future tracker=worker.getTracker();
           
            try {
                if (timeOut>0) { // time-limited rendering
                    long timeLeft = 0;
                    if(worker.isStarted()) {
                        timeLeft=timeOut-System.currentTimeMillis()+worker.getStartTime();
                    } else {
                        timeLeft=timeOut;
                    }
                    
                    if (timeLeft<0) timeLeft=0;
                    logger.debug("getPortletRenderContent(window ID = " + window.getPortletWindowId() + ") - timeOut="+timeOut+", used time="+(System.currentTimeMillis()-worker.getStartTime())+", time left="+timeLeft);
                    tracker.get(timeLeft, TimeUnit.MILLISECONDS);
                } else { // unlimited rendering time
                    tracker.get();
                }
            } catch (TimeoutException te) {
                // don't do anything yet, wait for cancel
                logger.info("getPortletRenderContent(window ID = " + window.getPortletWindowId() + ") - Portlet rendering timed out (allowed time="+timeOut+", used time="+(System.currentTimeMillis()-worker.getStartTime()));
            } catch (InterruptedException e) {
                tracker.cancel(true);
                throw new PortletRenderingInterruptedException(window.getPortletWindowId(),timeOut,System.currentTimeMillis()-worker.getStartTime());
            } catch (ExecutionException e) {
                tracker.cancel(true);
                throw new PortletRenderingException(window.getPortletWindowId(),e);
            }
            
            if(!tracker.isDone()) {
                long timeOfCancel=System.currentTimeMillis()-worker.getStartTime();
                tracker.cancel(true);
                throw new PortletRenderingTimeoutException(window.getPortletWindowId(),timeOut,timeOfCancel);
            } else {
                // check if the rendering has been successful
                if(worker.isDone()) {
                    if(worker.isSuccessful()) {
                        // return successful result
                        return worker.getRenderContent();
                    } else {
                        // report the problem
                        Throwable problem=worker.getProblem();
                        throw new PortletRenderingException(window.getPortletWindowId(),problem);
                    }
                } else {
                    // this probably means it was cancelled or killed in some other way
                    throw new PortletRenderingException(window.getPortletWindowId());
                }
            }
        } else {
            throw new PortletRenderingNotStartedException(window.getPortletWindowId());
        }
    }
    
    
    /**
     * Retreive a rendering worker associated with a given window id and a given http request
     * @param windowId window id
     * @param req http request
     * @return rendering worker if one was already created, or <code>null</code> if one has not been created
     */
    protected RenderingWorker getWorker(PortletWindowId windowId, HttpServletRequest req) {
        Map workerMap=(Map) req.getAttribute(MultithreadedPortletRenderingModel.class.toString()+WORKER_MAP_KEY);
        if(workerMap!=null) {
            return (RenderingWorker) workerMap.get(windowId);
        } else {
            return null;
        }
    }
    
    /**
     * Associate a rendering worker for a given window id with a given request
     * @param worker worker
     * @param windowId window id
     * @param req http request
     */
    @SuppressWarnings("unchecked")
    protected void setWorker(RenderingWorker worker,PortletWindowId windowId, HttpServletRequest req) {
        Map<PortletWindowId, RenderingWorker> workerMap=(Map<PortletWindowId, RenderingWorker>) req.getAttribute(MultithreadedPortletRenderingModel.class.toString()+WORKER_MAP_KEY);
        if(workerMap==null) {
            workerMap=new Hashtable<PortletWindowId, RenderingWorker>();
            req.setAttribute(MultithreadedPortletRenderingModel.class.toString()+WORKER_MAP_KEY, workerMap);
        }
        
        workerMap.put(windowId,worker);
    }
    
    protected class RenderingWorker implements Runnable {
        boolean successful;
        boolean done;
        boolean started;
        
        long startTime;
        long timeOut;
        
        Throwable problem;
        
        PortletWindow portletWindow;
        HttpServletRequest req;
        IPortletRenderContent renderContent;
        PortletHttpServletResponseWrapper res;
        PortletContainer portletContainer;
        
        Future tracker;
        
        
        
        /**
         * Construct a rendering worker
         * 
         * @param portletWindow portlet window (Pluto's object)
         * @param req http request
         * @param res http response
         * @param portletContainer Pluto container
         * @param timeOut maximum time (in milliseconds) that portlet is given to render itself
         */
        public RenderingWorker(PortletWindow portletWindow, HttpServletRequest req, PortletHttpServletResponseWrapper res, PortletContainer portletContainer, long timeOut) {
            super();
            this.timeOut = timeOut;
            this.portletWindow = portletWindow;
            this.req = req;
            this.res = res;
            this.portletContainer = portletContainer;
        }
        
        public void run() {
            try {
                
                // set start timestamp
                this.startTime = System.currentTimeMillis();
                
                // ACEGI-specific thingy: copy security context
                SecurityContextHolder.setContext(TrackingContextHolder.getContext());
                
                started=true;
                
                // invoke container call
                portletContainer.doRender(portletWindow,req,res);
                renderContent=res.getPortletRenderContent();
                successful=true;
                done=true;
                
            } catch (Throwable t) {
                problem=t;
                successful=false;
                done=true;
            }
        }
        
        
        /**
         * @return Returns the startTime.
         */
        public long getStartTime() {
            return startTime;
        }
        /**
         * @param startTime The startTime to set.
         */
        public void setStartTime(long startTime) {
            this.startTime = startTime;
        }
        /**
         * @return Returns the timeOut.
         */
        public long getTimeOut() {
            return timeOut;
        }
        /**
         * @param timeOut The timeOut to set.
         */
        public void setTimeOut(long timeOut) {
            this.timeOut = timeOut;
        }
        /**
         * @return Returns the done.
         */
        public boolean isDone() {
            return done;
        }
        /**
         * @return Returns the problem.
         */
        public Throwable getProblem() {
            return problem;
        }
        /**
         * @return Returns the successful.
         */
        public boolean isSuccessful() {
            return successful;
        }
        /**
         * @param req The httpServletRequest to set.
         */
        public void setHttpServletRequest(HttpServletRequest req) {
            this.req = req;
        }
        /**
         * @param res The httpServletResponse to set.
         */
        public void setHttpServletResponse(PortletHttpServletResponseWrapper res) {
            this.res = res;
        }
        /**
         * @param portletContainer The portletContainer to set.
         */
        public void setPortletContainer(PortletContainer portletContainer) {
            this.portletContainer = portletContainer;
        }
        /**
         * @param portletWindow The portletWindow to set.
         */
        public void setPortletWindow(PortletWindow portletWindow) {
            this.portletWindow = portletWindow;
        }
        /**
         * @return Returns the started.
         */
        public boolean isStarted() {
            return started;
        }
        /**
         * @return Returns the tracker.
         */
        public Future getTracker() {
            return tracker;
        }
        /**
         * @param tracker The tracker to set.
         */
        public void setTracker(Future tracker) {
            this.tracker = tracker;
        }
        /**
         * @return Returns the renderContent.
         */
        public IPortletRenderContent getRenderContent() {
            return renderContent;
        }
    }
    /**
     * @return Returns the defaultRenderTimeOutValue.
     */
    public long getDefaultRenderTimeOutValue() {
        return defaultRenderTimeOutValue;
    }
    /**
     * @param defaultRenderTimeOutValue The defaultRenderTimeOutValue to set.
     */
    public void setDefaultRenderTimeOutValue(long defaultRenderTimeOutValue) {
        this.defaultRenderTimeOutValue = defaultRenderTimeOutValue;
    }
    /**
     * @param executorService The executorService to set.
     */
    public void setExecutorService(ExecutorService executorService) {
        this.executorService = executorService;
    }
}
