/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.rendering;

import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.url.IUrlConstructorProviderFactoryServiceAccessor;
import org.jasig.portal.url.IUrlDecoratorListAccessor;
import org.jasig.portal.url.UrlCacheUtils;

/**
 * Default portlet cache manager implementation
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletCacheManagerImpl implements IPortletCacheManager {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PortletCacheManagerImpl.class);
    
    // url constructor/decorator accessors are needed to determine url cache keys
    IUrlConstructorProviderFactoryServiceAccessor constructorProviderFactoryServiceAccessor;
    IUrlDecoratorListAccessor urlDecoratorListAccessor;

    protected ISimpleCacheProviderFacade portletCache;

    private final static String RETRIEVED_CONTENT_ATTRIBUTE_NAME_BASE=PortletCacheManagerImpl.class.toString(); 
    
    public boolean isCachedContentAvailable(PortletWindowId portletWindowId, HttpServletRequest req) {
        // try to fetch content from cache
        SerializablePortletRenderContent content=(SerializablePortletRenderContent) portletCache.getFromCache(portletWindowId.toString());
        if(content!=null) {
            // check cache validity
            String currentCacheKey=UrlCacheUtils.getCacheKey(constructorProviderFactoryServiceAccessor,urlDecoratorListAccessor,req).getKey();
            if(content.cacheKey==null || content.cacheKey.equals(currentCacheKey)) {
                if(isCacheValid(content.getPortletExpirationCache(),content.getTimestamp())) {
                    // record cache in a request attribute for further retrieval
                    setRetrievedContent(portletWindowId,content,req);
                    if (logger.isInfoEnabled()) {
                        logger.info("retrieved valid portlet cache : portletWindowId=" + portletWindowId + " cacheExpiration="+content.getPortletExpirationCache()+", cacheKey="+content.cacheKey);
                    }
                    return true;
                }
            } else {
                if (logger.isDebugEnabled()) {
                    logger.debug("cache key has changed : portletWindowId=" + portletWindowId + ", currentCacheKey=" + currentCacheKey+", oldCacheKey="+content.cacheKey);
                }
            }
            // remove invalid cache entry
            invalidateCache(portletWindowId);
            return false;
        } else {
            return false;
        }
    }
    
    protected boolean isCacheValid(String cacheExpiration, long timeStamp) {
        try {
            int cacheExpiration_int=Integer.parseInt(cacheExpiration);
            if(cacheExpiration_int==-1) {
                return true;
            } else if(cacheExpiration_int>0) {
                // check timeout
                long elapsedTime=System.currentTimeMillis()-timeStamp;
                if(elapsedTime<=cacheExpiration_int) {
                    return true;
                } else {
                    return false;
                }
            } else {
                logger.warn("isCacheValid(String, long) - invalid cache expiration found in an existing cache : cacheExpiration_int=" + cacheExpiration_int, null);
                return false;
            }
        } catch (Exception e) {
            logger.warn("isCacheValid(String, long) - unable to parse cache expiration directive of a cached entiry : cacheExpiration=" + cacheExpiration, null);
            return false;
        }
    }
    
    /**
     * Determine if the content should be cached based on the cache expiration directive
     * @param cacheExpiration
     * @return <code>true</code> if the content should be cached, <code>false</code> if it should be disregarded
     */
    protected boolean shouldCache(String cacheExpiration) {
        try {
            int cacheExpiration_int=Integer.parseInt(cacheExpiration);
            return (cacheExpiration_int>0 || cacheExpiration_int==-1);
        } catch (Exception e) {
            return false;
        }
    }
    
    public IPortletRenderContent getPortletRenderContent(PortletWindowId windowId, HttpServletRequest req) {
        return getRetrievedContent(windowId,req);
    }

    public void setPortletRenderContent(IPortletRenderContent renderContent, PortletWindowId windowId, HttpServletRequest req) {
        try {
            if(shouldCache(renderContent.getPortletExpirationCache())) {
                String cacheKey=UrlCacheUtils.getCacheKey(constructorProviderFactoryServiceAccessor,urlDecoratorListAccessor,req).getKey();
                portletCache.putInCache(windowId.toString(),new SerializablePortletRenderContent(renderContent,cacheKey));
                if (logger.isInfoEnabled()) {
                    logger.info("cached portlet content : windowId=" + windowId +" cacheExpiration="+renderContent.getPortletExpirationCache()+", cacheKey="+cacheKey);
                }
            }
        } catch (UnsupportedEncodingException e) {
            logger.error("unable to retrieve specified content type as characters", e);
        }
        
    }
    
    protected static void setRetrievedContent(PortletWindowId portletWindowId, SerializablePortletRenderContent content, HttpServletRequest req) {
        req.setAttribute(RETRIEVED_CONTENT_ATTRIBUTE_NAME_BASE+portletWindowId.toString(),content);
    }
    
    protected static SerializablePortletRenderContent getRetrievedContent(PortletWindowId portletWindowId, HttpServletRequest req) {
        return (SerializablePortletRenderContent) req.getAttribute(RETRIEVED_CONTENT_ATTRIBUTE_NAME_BASE+portletWindowId.toString());
    }
    
    /**
     * A serializable version of the portlet render content, based on
     * the character output. 
     */
    protected static class SerializablePortletRenderContent implements IPortletRenderContent, Serializable {

        String title;
        String content;
        String contentType;
        Locale locale;
        String portletExpirationCache;
        String cacheKey;
        long timestamp;
        
        public SerializablePortletRenderContent(IPortletRenderContent portletRenderContent, String cacheKey) throws UnsupportedEncodingException {
            this.content=portletRenderContent.getCharacterOutput();
            this.contentType=portletRenderContent.getContentType();
            this.title=portletRenderContent.getTitle();
            this.locale=portletRenderContent.getLocale();
            this.portletExpirationCache=portletRenderContent.getPortletExpirationCache();
            this.cacheKey=cacheKey;
            // set timestamp to creation time
            this.timestamp=System.currentTimeMillis();
        }
        
        
        public byte[] getBinaryOutput() {
            return content.getBytes();
        }

        public String getCharacterOutput() throws UnsupportedEncodingException {
            return content;
        }

        public String getContentType() {
            return contentType;
        }

        public Locale getLocale() {
            return locale;
        }

        public String getPortletExpirationCache() {
            return portletExpirationCache;
        }

        public String getTitle() {
            return title;
        }

        public void setTimestamp(long timestamp) {
            this.timestamp = timestamp;
        }

        /**
         * Get cache creation time
         */
        public long getTimestamp() {
            return timestamp;
        }
    }

    public void invalidateCache(PortletWindowId windowId) {
        portletCache.removeFromCache(windowId.toString());
        if (logger.isInfoEnabled()) {
            logger.info("invalidated portlet cache : portletWindowId=" + windowId);
        }
    }

    public void setPortletCache(ISimpleCacheProviderFacade portletCache) {
        this.portletCache = portletCache;
    }

    public void setConstructorProviderFactoryServiceAccessor(IUrlConstructorProviderFactoryServiceAccessor constructorProviderFactoryServiceAccessor) {
        this.constructorProviderFactoryServiceAccessor = constructorProviderFactoryServiceAccessor;
    }

    public void setUrlDecoratorListAccessor(IUrlDecoratorListAccessor urlDecoratorListAccessor) {
        this.urlDecoratorListAccessor = urlDecoratorListAccessor;
    }

}
