/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.rendering;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.context.common.user.IUserControllerEventListener;
import org.jasig.portal.context.common.user.UserControllerEvent;
import org.jasig.portal.portlet.registry.transitory.ITransientPortletWindowRegistry;
import org.jasig.portal.portlet.session.IPortletSessionActionManager;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.portlets.error.bean.ErrorData;
import org.jasig.portal.portlets.error.bean.ErrorPhase;

/**
 *  Default impelementation of {@link org.jasig.portal.portlet.rendering.IPortletErrorHandler}
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletErrorHandlerImpl implements IPortletErrorHandler, IUserControllerEventListener {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PortletErrorHandlerImpl.class);
    // session key for a map maintaining spawned ErrorPortlet window ids
    protected final static String ERROR_MAP_SESSION_KEY=PortletErrorHandlerImpl.class.getName()+".errorMap";
    // session key for a map maintaining error portlet window ids
    protected final static String WINDOW_MAP_SESSION_KEY=PortletErrorHandlerImpl.class.getName()+".windowMap";
      

    private IPortletWindowManager portletWindowManager;
    private ITransientPortletWindowRegistry transientPortletWindowRegistry;
    
    private String errorPortletFunctionalName="uP3.ErrorPortlet";
    private IPortletSessionActionManager portletSessionActionManager;
    private IUserController userController;
    
    //  fallback error content
    // TODO: add localization, etc. support to the fallback content. It can be made a lot smarter, yet robust.
    protected static final StaticContent fallbackErrorContent=new StaticContent("Unable to render error portlet");

    public IPortletRenderContent handleProblem(PortletWindowId windowId, HttpServletRequest req, HttpServletResponse res, ErrorData ed) {
        if(isErrorPortlet(windowId,req)) {
            // error portlet failed to render
            return getFallbackErrorContent(windowId,req,res,ed);
        }
        // substitute error portlet instead of the original window
        try {
            PortletWindowId errorPortletWindowId=replacePortletWindowWithErrorPortlet(windowId,ed,req,res);
            ErrorPhase phase=ed.getErrorPhase();
            if(phase==ErrorPhase.CONTENT_RETRIEVAL_PHASE || phase==ErrorPhase.RENDERING_INITIATION_PHASE) {
                // initiate rendering
                if(errorPortletWindowId!=null) {
                    portletWindowManager.startPortletRendering(errorPortletWindowId);
                }
                
                if(phase==ErrorPhase.CONTENT_RETRIEVAL_PHASE) {
                    // produce error content
                    IPortletRenderContent errorContent=null;
                    if(errorPortletWindowId!=null) {
                        // render error portlet
                        portletWindowManager.startPortletRendering(errorPortletWindowId);
                        errorContent=portletWindowManager.getPortletRenderContent(errorPortletWindowId);
                    }
                    if(errorContent!=null) {
                        return errorContent;
                    } else {
                        logger.error("handleProblem(windowId = " + windowId + ") - error portlet rendering returned null");
                        return getFallbackErrorContent(windowId,req,res,ed);
                    }
                }
            }
            // content return is not required at this phase
            return null;
        } catch (Exception ne) {
            logger.error("handleProblem(windowId = " + windowId + ") - exception raised while rendering error portlet",ne);
            return getFallbackErrorContent(windowId,req,res,new ErrorData(windowId,ne));
        }
    }
    
    public PortletWindowId getReplacingErrorPortletWindowId(PortletWindowId windowId, HttpServletRequest req) {
        PortletWindowId portletWindowId=null;
        // attempt retrieve an error portlet replacing the specified window id
        Map errorPortletMap=getErrorPortletMapping(req);
        if(errorPortletMap!=null) {
            ErrorReplacementInformation replacement=(ErrorReplacementInformation) errorPortletMap.get(windowId);
            if(replacement!=null) {
                // retrieve an error portlet specified by the replacement
                portletWindowId=replacement.getErrorPortletWindowId();
                if(portletWindowId==null) {
                    logger.error("unable to find error portlet specified by a valid replacement mapping : original windowId=" + windowId + ", replacement error portlet windowId=" + replacement.getErrorPortletWindowId(), null);
                } else {
                    if (logger.isDebugEnabled()) {
                        logger.debug("returning a replacement error portlet window : error portlet windowId=" + portletWindowId + ", replaced windowId=" + windowId);
                    }
                }
            }
        }
        return portletWindowId;
    }

    /**
     * Checks if a given windowId belongs to a spawned error portlet
     * @param windowId
     * @param req
     * @return <code>true</code> if the windowId corresponds to a spawned error portlet
     */
    protected boolean isErrorPortlet(PortletWindowId windowId, HttpServletRequest req) {
        Set windowSet=getErrorWindowSet(req);
        if(windowSet!=null) {
            return windowSet.contains(windowId);
        } else {
            return false;
        }
    }
    /**
     * Retrieve window map, creating it if necessary
     * @param req
     * @return a {@link Set} of error portlet {@link PortletWindowId}  
     */
    protected Set getCreateErrorWindowSet(HttpServletRequest req) {
        HttpSession session=req.getSession(true);
        synchronized (session) {
            Set windowSet=(Set)session.getAttribute(WINDOW_MAP_SESSION_KEY);
            if(windowSet==null) {
                windowSet=Collections.synchronizedSet(new HashSet());
                session.setAttribute(WINDOW_MAP_SESSION_KEY,windowSet);
            }
            return windowSet;
        }
    }
    
    /**
     * Retrieve window map, without creating it
     * @param req
     * @return a {@link Set} of error portlet {@link PortletWindowId}  
     */
    protected Set getErrorWindowSet(HttpServletRequest req) {
        HttpSession session=req.getSession();
        if(session!=null) {
            return (Set)session.getAttribute(WINDOW_MAP_SESSION_KEY);
        } else {
            return null;
        }
    }

    public void refreshPortletWindow(PortletWindowId windowId, HttpServletRequest req, HttpServletResponse res, boolean passLastRequestParameters) {
        // remove error portlet id mapping and the error portlet window id
        if (logger.isDebugEnabled()) {
            logger.debug("removing error portlet replacement for windowId=" + windowId, null);
        }
        
        Map errorPortletMap=getErrorPortletMapping(req);
        if(errorPortletMap!=null) {
            ErrorReplacementInformation replacement=(ErrorReplacementInformation) errorPortletMap.remove(windowId);
            if(replacement!=null) {
                // remove from a window map
                Set windowSet=getErrorWindowSet(req);
                if(windowSet==null) {
                    windowSet.remove(replacement.getErrorPortletWindowId());
                }
                
                // redo render/action parameters if required
                if(passLastRequestParameters) {
                    if(replacement.errorPhase==ErrorPhase.ACTION_PHASE) {
                        // retry portlet action
                        Map actionParameters=replacement.getParams();
                        portletWindowManager.setPortletActionRequestParameters(windowId,actionParameters);
                    } else {
                        Map renderParameters=replacement.getParams();
                        portletWindowManager.setPortletRenderRequestParameters(windowId,renderParameters);
                    }
                }
            } else {
                logger.warn("unable to locate replacing error portlet in the internal maps : windowId=" + windowId, null);
            }
        }
    }

    public void restartPortletWindow(PortletWindowId windowId, HttpServletRequest req, HttpServletResponse res) {
        // clean up portlet session
        try {
            IPortletWindow window=transientPortletWindowRegistry.getPortletWindow(windowId);
            if(window!=null) {
                this.portletSessionActionManager.clear(window, req, res);
            }
        } catch (Exception e) {
            logger.warn("encountered exception while trying to clear session for portlet restart : windowId=" + windowId, e);
        }
        refreshPortletWindow(windowId,req,res,false);
    }


    public IPortletWindowManager getPortletWindowManager() {
        return portletWindowManager;
    }

    public void setPortletWindowManager(IPortletWindowManager portletWindowManager) {
        this.portletWindowManager = portletWindowManager;
    }

    /**
     * Replace an existing window with an error portlet
     * 
     * @param windowId id of a window to be replaced
     * @param data error data summary for the error portlet
     * @param req current request
     * @param res current response
     * @return portlet window id of an error portlet, or null if the supplied window id is already an instance of an error portlet
     */
    public PortletWindowId replacePortletWindowWithErrorPortlet(PortletWindowId windowId, ErrorData data, HttpServletRequest req, HttpServletResponse res) {
        // check if the window is a remapped error channel, and if so fall back to some other type of error reporting
        if(isErrorPortlet(windowId,req)) {
            return null;
        }
        

        if (logger.isDebugEnabled()) {
            logger.debug("replacing window with error portlet : windowId=" + windowId);
        }

        // retrieve map containing spawned error portlets
        Map errorPortletMap=getCreateErrorPortletMapping(req);
        
        // this shouldn't normally happen - if a window is already replaced
        // by an error portlet, there should not be additional requests to replace it
        ErrorReplacementInformation replacement=(ErrorReplacementInformation) errorPortletMap.get(windowId);
        
        if(replacement==null) {
            // create a new error portlet window
            IPortletWindow errorWindow=transientPortletWindowRegistry.createTransientPortletWindowControl(errorPortletFunctionalName);
            replacement=new ErrorReplacementInformation(data.getErrorPhase(),errorWindow.getPortletWindowId(),data.getParameterMap());
            // record replacement
            errorPortletMap.put(windowId,replacement);
            // add newly created window to the error portlet window map
            Set windowSet=getCreateErrorWindowSet(req);
            windowSet.add(errorWindow.getPortletWindowId());
            
            // setup action attributes for the error channel and invoke portlet action
            try {
                this.portletSessionActionManager.setAttribute(errorWindow, req, res, ErrorData.ERROR_DATA_SESSION_ATTRIBUTE, data);
            }
            catch (Exception e) {
                logger.error("unable to initialize session variable for the error portlet : original windowId=" + windowId + ", error portlet windowId=" + errorWindow.getPortletWindowId(), e);
            }
        }
        
        return replacement.getErrorPortletWindowId();
    }
    
    
    /**
     * Retrieve windowId to error portlet window Id mapping, creating it if necessary
     * @param req
     * @return a {@link Map} from {@link PortletWindowId} of windows being replaced to the ids of the error portlet window replacing them  
     */
    protected Map getCreateErrorPortletMapping(HttpServletRequest req) {
        HttpSession session=req.getSession(true);
        synchronized (session) {
            Map errorPortletMap=(Map)session.getAttribute(ERROR_MAP_SESSION_KEY);
            if(errorPortletMap==null) {
                errorPortletMap=Collections.synchronizedMap(new HashMap());
                session.setAttribute(ERROR_MAP_SESSION_KEY,errorPortletMap);
            }
            return errorPortletMap;
        }
    }
    
    /**
     * Retrieve windowId to error portlet window Id mapping, without creating it
     * @param req
     * @return a {@link Map} from {@link PortletWindowId} of windows being replaced to the ids of the error portlet window replacing them  
     */
    protected Map getErrorPortletMapping(HttpServletRequest req) {
        HttpSession session=req.getSession();
        if(session!=null) {
            return (Map)session.getAttribute(ERROR_MAP_SESSION_KEY);
        } else {
            return null;
        }
    }
    

    
    /**
     * A datastructure containing information about an error portlet replacing
     * another portlet.
     */
    static class ErrorReplacementInformation {
        /**
         * Logger for this class
         */
        private static final Log logger = LogFactory.getLog(ErrorReplacementInformation.class);

        ErrorPhase errorPhase;
        PortletWindowId errorPortletWindowId;
        Map params;
        
        
        public ErrorReplacementInformation(ErrorPhase phase, PortletWindowId id, Map params) {
            super();
            // TODO Auto-generated constructor stub
            errorPhase = phase;
            errorPortletWindowId = id;
            this.params = params;
        }
        
        public ErrorPhase getErrorPhase() {
            return errorPhase;
        }
        public void setErrorPhase(ErrorPhase errorPhase) {
            this.errorPhase = errorPhase;
        }
        public PortletWindowId getErrorPortletWindowId() {
            return errorPortletWindowId;
        }
        public void setErrorPortletWindowId(PortletWindowId errorPortletWindowId) {
            this.errorPortletWindowId = errorPortletWindowId;
        }
        public Map getParams() {
            return params;
        }
        public void setParams(Map params) {
            this.params = params;
        }
    }
    
    /**
     * This is a fallback error output method. It should never return null or throw an exception.
     * @param windowId error portlet window id
     * @param req current request
     * @param res current response
     * @param errorData error data
     * @return fallback error content report
     */
    protected IPortletRenderContent getFallbackErrorContent(PortletWindowId windowId, HttpServletRequest req, HttpServletResponse res, ErrorData errorData) {
        logger.warn("returning fallback error content : windowId=" + windowId, null);
        return fallbackErrorContent;
    }
    
    /**
     * @param transientPortletWindowRegistry The transientPortletWindowRegistry to set.
     */
    public void setTransientPortletWindowRegistry(ITransientPortletWindowRegistry transientPortletWindowRegistry) {
        this.transientPortletWindowRegistry = transientPortletWindowRegistry;
    }

    public void setPortletSessionActionManager(IPortletSessionActionManager portletSessionActionManager) {
        this.portletSessionActionManager = portletSessionActionManager;
    }
    
    public void setUserController(IUserController userController) {
        this.userController = userController;
        userController.addUserControllerEventListener(this);
    }

    public void receiveUserControllerEvent(UserControllerEvent event) {
        if(userController==event.getSource()) {
            HttpServletRequest req=event.getHttpServletRequest();
            // clean up session
            if(req!=null) {
                HttpSession session=req.getSession(false);
                if(session!=null) {
                    session.removeAttribute(WINDOW_MAP_SESSION_KEY);
                    session.removeAttribute(ERROR_MAP_SESSION_KEY);
                }
            }
        }
    }

    public void setErrorPortletFunctionalName(String errorPortletFunctionalName) {
        this.errorPortletFunctionalName = errorPortletFunctionalName;
    }

}
