/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.rendering;

import java.security.Principal;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

import org.apache.commons.collections.iterators.EnumerationIterator;
import org.apache.commons.collections.iterators.IteratorChain;
import org.apache.commons.collections.iterators.IteratorEnumeration;
import org.apache.commons.collections.iterators.UniqueFilterIterator;
import org.jasig.portal.security.IUser;

/**
 * An http request wrapper that is used in portlet action and render processing.
 * Allows to substitute request parameters and attributes.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class PortletHttpServletRequestWrapper extends HttpServletRequestWrapper {
    private final Map<String, Object> attributeValueMap = Collections.synchronizedMap(new HashMap<String, Object>());
    private final Map<String, String[]> parameterMap;
    private final IUser user;

    /**
     * Construct a wrapper for a given request
     * @param user The IUser the request is for. NULL implies an unauthenticated user.
     */
    public PortletHttpServletRequestWrapper(HttpServletRequest req, Map<String, String[]> parameterMap, IUser user) {
        super(req);
        
        if (parameterMap == null) {
            throw new IllegalArgumentException("parameterMap may not be null");
        }
        
        this.parameterMap = parameterMap;
        this.user = user;
    }
    
    /**
     * @see javax.servlet.ServletRequest#getParameter(java.lang.String)
     */
    public String getParameter(String paramName) {
        final String[] parameterValues = this.getParameterValues(paramName);
        if (parameterValues != null && parameterValues.length > 0) {
            return parameterValues[0];
        }
        else {
            return null;
        }
    }
    
    /**
     * @see javax.servlet.ServletRequest#getParameterMap()
     */
    public Map getParameterMap() {
        return this.parameterMap;
    }

    /**
     * @see javax.servlet.ServletRequest#getParameterNames()
     */
    public Enumeration getParameterNames() {
        return new IteratorEnumeration(this.parameterMap.keySet().iterator());
    }

    /**
     * @see javax.servlet.ServletRequest#getParameterValues(java.lang.String)
     */
    public String[] getParameterValues(String paramName) {
        return this.parameterMap.get(paramName);
    }

    /**
     * @see javax.servlet.ServletRequest#getAttribute(java.lang.String)
     */
    public Object getAttribute(String name) {
        if (name == null)
            throw new IllegalArgumentException("Attribute name cannot be null");
        
        final Object value = this.attributeValueMap.get(name);

        if (value != null) {
            return value;
        }
        else {
            return super.getAttribute(name);
        }
    }

    /**
     * @see javax.servlet.ServletRequest#getAttributeNames()
     */
    public Enumeration getAttributeNames() {
        final Iterator<String> attrNameItr = this.attributeValueMap.keySet().iterator();
        
        final Enumeration superAttrNameEnum = super.getAttributeNames();
        final Iterator superAttrNameItr = new EnumerationIterator(superAttrNameEnum);
        
        final Iterator[] allAttrNameItrs = new Iterator[] {attrNameItr, superAttrNameItr};
        final Iterator allAttrNameItr = new IteratorChain(allAttrNameItrs);
        
        final Iterator filteredAttrNameItr = new UniqueFilterIterator(allAttrNameItr);
        return new IteratorEnumeration(filteredAttrNameItr);
    }

    /**
     * @see javax.servlet.ServletRequest#removeAttribute(java.lang.String)
     */
    public void removeAttribute(String name) {
        if (name == null)
            throw new IllegalArgumentException("Attribute name cannot be null");
        
        //TODO is this the desired behavior?
        this.attributeValueMap.remove(name);
        super.removeAttribute(name);
    }

    /**
     * @see javax.servlet.ServletRequest#setAttribute(java.lang.String, java.lang.Object)
     */
    public void setAttribute(String name, Object value) {
        if (name == null)
            throw new IllegalArgumentException("Attribute name cannot be null");
        
        if (value == null) {
            this.removeAttribute(name);
        }
        else {
            this.attributeValueMap.put(name, value);
        }
    }

    public String getRemoteUser() {
        if (this.user != null) {
            return this.user.getUserName();
        }
        else {
            return null;
        }
    }

    public Principal getUserPrincipal() {
        if (this.user != null) {
            return this.user.getPrincipal();
        }
        else {
            return null;
        }
    }
}
