/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.rendering;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Locale;
import java.util.Map;

import javax.portlet.PortletURL;
import javax.portlet.RenderResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.pluto.PortletContainer;
import org.apache.pluto.PortletWindow;
import org.apache.pluto.internal.InternalPortletWindow;
import org.apache.pluto.internal.impl.RenderResponseImpl;
import org.apache.pluto.spi.PortalCallbackService;
import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.IPlutoDomainObjectHelper;
import org.jasig.portal.portlet.definition.LocalizedPortletDefinitionImpl;
import org.jasig.portal.portlet.url.IPortletUrlFactory;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.utils.RecordingHttpServletResponseWrapper;


/**
 * Default implementation of portlet render response.
 * 
 * The implementation will use {@link org.jasig.portal.utils.RecordingHttpServletResponseWrapper}
 * to record content rendered by the portlet (which includes character encoding handling and buffering).
 * The implementation also captures locale, title, error and status information returned by the portlet.
 * 
 * The HttpServletResponse provided to the constructor must be an instance of {@link org.jasig.portal.portlet.rendering.PortletHttpServletResponseWrapper}
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11917 $
 */
public class PortletRenderResponseImpl extends RenderResponseImpl implements IPortletRenderResponse, IPortletRenderContent {
    /**
     * Default value of cache expiration
     */
    private static final String DEFAULT_EXPIRATION_CACHE = "0";

    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PortletRenderResponseImpl.class);
    
    boolean commited=false;
    String title;
    RecordingHttpServletResponseWrapper wrappedResponse;
    int status=HttpServletResponse.SC_OK;
    String error;
    HttpServletRequest req;
    IPortletWindow window;
    PortletWindow plutoWindow;
    
    IPortletUrlFactory portletUrlFactory;
    PortalCallbackService callbackService;
    
  
    Locale locale;

    /**
     * Create a new render response instance
     * @param internalWindow portlet window
     * @param req http request
     * @param res http response - must be an instance of {@link PortletHttpServletResponseWrapper}
     */
    public PortletRenderResponseImpl(PortletContainer container, InternalPortletWindow internalWindow, HttpServletRequest req, HttpServletResponse res, ILocaleManager localeManager,IPortletUrlFactory portletUrlFactory,IPlutoDomainObjectHelper plutoHelper) {
        //TODO look at how to deal with containerSupportsBuffering
        super(container, internalWindow, req, new RecordingHttpServletResponseWrapper(res));
        // argh !
        this.wrappedResponse=(RecordingHttpServletResponseWrapper)getResponse();
        this.req=req;
        this.plutoWindow=internalWindow.getOriginalPortletWindow();
        this.window=plutoHelper.getPortletWindow(plutoWindow);
        
        this.portletUrlFactory=portletUrlFactory;
        this.callbackService=container.getRequiredContainerServices().getPortalCallbackService();
        
        // associate content with the response
        if(res instanceof PortletHttpServletResponseWrapper) {
            ((PortletHttpServletResponseWrapper)res).setPortletRenderContent(this);
        } else {
            throw new IllegalArgumentException("response must be an instance of PortletHttpServletResponseWrapper");
        }
        
        // set up title
        LocalizedPortletDefinitionImpl lpd=new LocalizedPortletDefinitionImpl(window.getPortletEntity().getPortletDefinition(),localeManager);
        this.title=lpd.getLocalDisplayName();
        this.locale=lpd.resolveLocal();
    }
  
    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletRenderContent#getBinaryOutput()
     */
    public byte[] getBinaryOutput() {
        return wrappedResponse.getContentAsByteArray();
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletRenderContent#getCharacterOutput()
     */
    public String getCharacterOutput() throws UnsupportedEncodingException {
        return wrappedResponse.getContentAsString();
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletRenderContent#getTitle()
     */
    public String getTitle() {
        return this.title;
    }
    /* (non-Javadoc)
     * @see javax.portlet.RenderResponse#isCommitted()
     */
    public boolean isCommitted() {
        return commited;
    }
    /* (non-Javadoc)
     * @see javax.portlet.RenderResponse#setTitle(java.lang.String)
     */
    public void setTitle(String title) {
        if(title!=null) {
            this.title=title;
        }
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletRenderResponse#getRenderContent()
     */
    public IPortletRenderContent getRenderContent() {
        return this;
    }
    /* (non-Javadoc)
     * @see javax.servlet.http.HttpServletResponse#sendError(int, java.lang.String)
     */
    public void sendError(int sc, String m) throws IOException {
        this.status=sc;
        this.error=m;
    }
    /* (non-Javadoc)
     * @see javax.servlet.http.HttpServletResponse#sendError(int)
     */
    public void sendError(int sc) throws IOException {
        this.status=sc;
    }
    /* (non-Javadoc)
     * @see javax.servlet.http.HttpServletResponse#sendRedirect(java.lang.String)
     */
    public void sendRedirect(String location) throws IOException {
        super.sendRedirect(location);
    }
    /* (non-Javadoc)
     * @see javax.servlet.ServletResponse#setContentLength(int)
     */
    public void setContentLength(int length) {
        super.setContentLength(length);
    }
    /* (non-Javadoc)
     * @see javax.servlet.ServletResponse#setLocale(java.util.Locale)
     */
    public void setLocale(Locale locale) {
        this.locale=locale;
    }
    /* (non-Javadoc)
     * @see javax.servlet.http.HttpServletResponse#setStatus(int, java.lang.String)
     */
    public void setStatus(int sc, String sm) {
        this.status=sc;
        this.error=sm;
    }
    /* (non-Javadoc)
     * @see javax.servlet.http.HttpServletResponse#setStatus(int)
     */
    public void setStatus(int sc) {
        this.status=sc;
    }

    /* (non-Javadoc)
     * @see javax.servlet.ServletResponse#getLocale()
     */
    public Locale getLocale() {
        if(this.locale==null) {
            return super.getLocale();
        } else {
            return this.locale;
        }
    }
    /* (non-Javadoc)
     * @see javax.servlet.ServletResponse#getBufferSize()
     */
    public int getBufferSize() {
        return wrappedResponse.getBufferSize();
    }
    /* (non-Javadoc)
     * @see javax.servlet.ServletResponse#resetBuffer()
     */
    public void resetBuffer() {
        wrappedResponse.resetBuffer();
    }
    /* (non-Javadoc)
     * @see javax.servlet.ServletResponse#setBufferSize(int)
     */
    public void setBufferSize(int arg0) {
        wrappedResponse.setBufferSize(arg0);
    }

    public String getPortletExpirationCache() {
        // check if it was set in the deployment
        if(window.getPortletEntity().getPortletDefinition().getPortletDeployment().getPortletExpirationCache()==null) {
            return DEFAULT_EXPIRATION_CACHE;
        } else {
            // return the value set in the property
            Map<String,String[]> props=callbackService.getRequestProperties(req,plutoWindow);
            if(props!=null) {
                String[] ec=props.get(RenderResponse.EXPIRATION_CACHE);
                if(ec!=null && ec.length>0) {
                    return ec[0];
                }
            }
            return DEFAULT_EXPIRATION_CACHE;
        }
    }

    @Override
    public PortletURL createActionURL() {
        return portletUrlFactory.createPortletUrl(window,req,true);
    }

    @Override
    public PortletURL createRenderURL() {
        return portletUrlFactory.createPortletUrl(window,req,false);
    }

}
