/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.rendering;

import java.util.EventObject;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import javax.servlet.http.HttpServletRequest;

import org.jasig.portal.portlet.window.PortletWindowId;

/**
 * Events sent out by portlet window manager to notify about changes in window states and porlet modes.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletWindowManagerEvent extends EventObject {
	final EventType eventType;
	HttpServletRequest httpServletRequest=null;
    PortletMode oldPortletMode=null;
    PortletMode newPortletMode=null;
    WindowState oldWindowState=null;
    WindowState newWindowState=null;
    PortletWindowId windowId=null;
    
	
	/**
	 * An event fired when portlet mode is changed
	 */
	public static final EventType portletModeChangedEventType=new EventType("portletModeChanged");

	/**
	 * An event fired when window state is changed
	 */
	public static final EventType windowStateChangedEventType=new EventType("windowStateChanged");
	
    /**
     * Create a portlet window manager event
     * @param source - event source
     * @param eventType - event type
     */
	protected PortletWindowManagerEvent(Object source,EventType eventType) {
		super(source);
		this.eventType=eventType;
	}
    
    public static PortletWindowManagerEvent createModeChangeEvent(PortletWindowId windowId,Object source, PortletMode oldMode, PortletMode newMode, HttpServletRequest req) {
        PortletWindowManagerEvent ev=new PortletWindowManagerEvent(source,portletModeChangedEventType,req);
        ev.windowId=windowId;
        ev.oldPortletMode=oldMode;
        ev.newPortletMode=newMode;
        return ev;
    }
    
    public static PortletWindowManagerEvent createWindowStateChangeEvent(PortletWindowId windowId,Object source, WindowState oldState, WindowState newState, HttpServletRequest req) {
        PortletWindowManagerEvent ev=new PortletWindowManagerEvent(source,windowStateChangedEventType,req);
        ev.windowId=windowId;
        ev.oldWindowState=oldState;
        ev.newWindowState=newState;
        return ev;
    }
    

	/**
	 * Create a portlet window manager event
	 * @param source - event source
	 * @param eventType - event type
	 * @param req - http request associated with the current cycle
	 * @param res - http response associated with the current cycle
	 */
	protected PortletWindowManagerEvent(Object source,EventType eventType, HttpServletRequest req) {
		this(source,eventType);		
		this.httpServletRequest=req;
	}
	
	/**
	 * @return Returns the http request associated with the current cycle
	 */
	public HttpServletRequest getHttpServletRequest() {
		return httpServletRequest;
	}

	/**
	 * @return Returns the eventType.
	 */
	public EventType getEventType() {
		return eventType;
	}

	public static class EventType {
		final String name;
		
		public EventType(String name) {
			this.name=name;
		}
		
		/**
		 * @return Returns the name.
		 */
		public String getName() {
			return name;
		}

        @Override
        public String toString() {
            return name;
        }
		
	}

    /**
     * Obtain current portlet window mode associated with the portlet mode change event
     * @return
     */
    public PortletMode getNewPortletMode() {
        return newPortletMode;
    }

    /**
     * Obtain current portlet window state associated with state change event
     * @return
     */
    public WindowState getNewWindowState() {
        return newWindowState;
    }

    /**
     * Obtain previous portlet mode
     * @return
     */
    public PortletMode getOldPortletMode() {
        return oldPortletMode;
    }

    /**
     * Obtain previous window state of the portlet window
     * @return
     */
    public WindowState getOldWindowState() {
        return oldWindowState;
    }

    public PortletWindowId getWindowId() {
        return windowId;
    }

    @Override
    public String toString() {
        StringBuffer sb=new StringBuffer();
        sb.append("eventType=").append(this.getEventType().toString());
        if(windowId!=null) { 
            sb.append(" portletWindowId="+windowId.toString());
        }
        if(oldWindowState!=null || newWindowState!=null) {
            sb.append(" windowState: ").append(oldWindowState).append("->").append(newWindowState);
        }
        if(oldPortletMode!=null || newPortletMode!=null) {
            sb.append(" portletMode: ").append(oldPortletMode).append("->").append(newPortletMode);
        }
        return sb.toString();
    }
    
    
}

