/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.rendering;

import java.util.Collections;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.WeakHashMap;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.context.common.IHttpRequestResponseAccessor;
import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.context.common.user.IUserControllerEventListener;
import org.jasig.portal.context.common.user.UserControllerEvent;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.registry.transitory.ITransientPortletWindowRegistry;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.portlets.error.bean.ErrorData;
import org.jasig.portal.portlets.error.bean.ErrorPhase;
import org.jasig.portal.security.IUser;
import org.springframework.beans.factory.annotation.Required;

/**
 * Default implementation of the portlet window manager
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletWindowManagerImpl implements IPortletWindowManager, IUserControllerEventListener {

    private IPortletWindowManagerAccessor windowManagerAccessor;
    private IUserController userController;
    private ITransientPortletWindowRegistry transientPortletWindowRegistry;
    private IPortletRenderingModel portletRenderingModel;
    private IPortletErrorHandler portletErrorHandler;
    private IPortletCacheManager portletCacheManager;
    private IHttpRequestResponseAccessor requestAccessor;
    
    protected final Map<IPortletWindowManagerEventListener,Boolean> eventListeners = new WeakHashMap<IPortletWindowManagerEventListener,Boolean>();


    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PortletWindowManagerImpl.class);
    
     // http request attribute key under which portlet param map is maintained
    protected final static String PORTLET_RENDERING_PARAM_MAP_SESSION_KEY=PortletWindowManagerImpl.class.getName()+".portletParamMap";
    
    // attribute prefix used to keep track of window ids for which rendering has been started
    protected final static String PORTLET_RENDERING_STARTED_ATTRIBUTE_PREFIX=PortletWindowManagerImpl.class.getName()+".renderingStarted.";
        
    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletWindowManager#setPortletActionRequestParameters(org.jasig.portal.portlet.window.PortletWindowId, java.util.Map, javax.servlet.http.HttpServletRequest)
     */
    public void setPortletActionRequestParameters(PortletWindowId windowId, Map parameters) {
        HttpServletRequest req=requestAccessor.getCurrentHttpServletRequest();
        HttpServletResponse res=requestAccessor.getCurrentHttpServletResponse();
        IPortletWindow window=null;
        try {
            window=getPortletWindow(windowId,req);
            if(window!=null) {
                // associate current window manager with the request
                windowManagerAccessor.setPortletWindowManager(this,req);
                portletCacheManager.invalidateCache(windowId);
                
                // clear old rendering parameters
                clearPortletRenderParameters(windowId,req);
                
                portletRenderingModel.processPortletAction(window,parameters,getValidUser(),req,res);
            } else {
                logger.error("windowId = " + windowId.toString() + " - unable to obtain portlet window with a specified id");
            }
        } catch (Exception e) {
            PortletWindowId trueWindowId=windowId;
            if(window!=null) { trueWindowId=window.getPortletWindowId(); }
            logger.error("exception raised while trying to process portlet action : windowId=" + windowId + ", trueWindowId=" + trueWindowId, e); 
            ErrorData ed=new ErrorData(windowId,e);
            ed.setErrorPhase(ErrorPhase.ACTION_PHASE);
            ed.setParameterMap(parameters);
            ed.setWindowTitle(getPortletTitle(window,res.getLocale()));
            portletErrorHandler.handleProblem(windowId, req, res, ed);
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletWindowManager#startPortletRendering(org.jasig.portal.portlet.window.PortletWindowId, javax.servlet.http.HttpServletRequest)
     */
    public void startPortletRendering(PortletWindowId windowId) {
        HttpServletRequest req=requestAccessor.getCurrentHttpServletRequest();
        HttpServletResponse res=requestAccessor.getCurrentHttpServletResponse();
        // check if rendering has already been started
        if(req.getAttribute(PORTLET_RENDERING_STARTED_ATTRIBUTE_PREFIX+windowId.toString())==null) {
            Map<String,String[]> portletRenderParams=null;
            IPortletWindow window=null;
            try {
                window=getPortletWindow(windowId,req);
                if(window!=null) {
                    windowManagerAccessor.setPortletWindowManager(this,req);
                    portletRenderParams=retrievePortletRenderRequestParameters(windowId,req);
                    // check if cached content is available
                    if(!portletCacheManager.isCachedContentAvailable(windowId,req)) {
                        portletRenderingModel.startPortletRendering(window,portletRenderParams,getValidUser(),req,res);
                    }
                    // remember that rendering was started for this windowId
                    req.setAttribute(PORTLET_RENDERING_STARTED_ATTRIBUTE_PREFIX+windowId.toString(),true);
                } else {
                    logger.error("unable to obtain portet window : windowId=" + windowId, null); 
                    ErrorData ed=new ErrorData(windowId,"unable to obtain portlet window",ErrorData.INVALID_PORTLET_WINDOW,null);
                    ed.setErrorPhase(ErrorPhase.RENDERING_INITIATION_PHASE);
                    ed.setParameterMap(portletRenderParams);
                    portletErrorHandler.handleProblem(windowId, req, res, ed);
                }
            } catch (Exception e) {
                PortletWindowId trueWindowId=windowId;
                if(window!=null) { trueWindowId=window.getPortletWindowId(); }
                logger.error("exception encountered during render initiation : windowId=" + windowId + ", trueWindowId=" + trueWindowId, e);
                ErrorData ed=new ErrorData(windowId,e);
                ed.setErrorPhase(ErrorPhase.RENDERING_INITIATION_PHASE);
                ed.setParameterMap(portletRenderParams);
                ed.setWindowTitle(getPortletTitle(window,res.getLocale()));
                portletErrorHandler.handleProblem(windowId, req, res, ed);
            }   
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletWindowManager#getPortletRenderContent(org.jasig.portal.portlet.window.PortletWindowId, javax.servlet.http.HttpServletRequest)
     */
    public IPortletRenderContent getPortletRenderContent(PortletWindowId windowId) {
        HttpServletRequest req=requestAccessor.getCurrentHttpServletRequest();
        HttpServletResponse res=requestAccessor.getCurrentHttpServletResponse();
        IPortletWindow window=null;
        // make sure that the rendering has been started
        if(req.getAttribute(PORTLET_RENDERING_STARTED_ATTRIBUTE_PREFIX+windowId.toString())==null) {
            if (logger.isInfoEnabled()) {
                logger.info("getPortletRenderContent(PortletWindowId, HttpServletRequest, HttpServletResponse) - portlet rendering content requested without prior call to startPortletRendering(), calling automatically : windowId=" + windowId.toString());
            }
            this.startPortletRendering(windowId);
        }
        
        // check if the content is available
        try {
            window=getPortletWindow(windowId,req);
            if(window!=null) {
                IPortletRenderContent content=portletCacheManager.getPortletRenderContent(window.getPortletWindowId(),req);
                if(content==null) {
                    content=portletRenderingModel.getPortletRenderContent(window,req,res);
                    if(content!=null) {
                        // record cache
                        portletCacheManager.setPortletRenderContent(content,window.getPortletWindowId(),req);
                        return content;
                    }
                } else {
                    return content;
                }
                // if content is null
                logger.error("null returned for portlet content : windowId=" + windowId + ", trueWindowId=" + window.getPortletWindowId());
                ErrorData ed=new ErrorData(windowId,"portlet returned null content",ErrorData.RENDERING_EXCEPTION,null);
                ed.setErrorPhase(ErrorPhase.CONTENT_RETRIEVAL_PHASE);
                ed.setParameterMap(retrievePortletRenderRequestParameters(windowId,req));
                ed.setWindowTitle(getPortletTitle(window,res.getLocale()));
                return portletErrorHandler.handleProblem(windowId, req, res, ed);
            } else {
                logger.error("unable to obtain portet window : windowId=" + windowId, null);
                ErrorData ed=new ErrorData(windowId,"unable to obtain portlet window",ErrorData.INVALID_PORTLET_WINDOW,null);
                ed.setErrorPhase(ErrorPhase.CONTENT_RETRIEVAL_PHASE);
                ed.setWindowTitle(getPortletTitle(window,res.getLocale()));
                ed.setParameterMap(retrievePortletRenderRequestParameters(windowId,req));
                return portletErrorHandler.handleProblem(windowId, req, res, ed);
            }
        } catch (Exception e) {
            PortletWindowId trueWindowId=windowId;
            if(window!=null) { trueWindowId=window.getPortletWindowId(); }
            logger.error("portlet rendering failed : windowId=" + windowId + ", trueWindowId=" + trueWindowId, e);
            ErrorData ed=new ErrorData(windowId,e);
            ed.setErrorPhase(ErrorPhase.CONTENT_RETRIEVAL_PHASE);
            ed.setParameterMap(retrievePortletRenderRequestParameters(windowId,req));
            ed.setWindowTitle(getPortletTitle(window,res.getLocale()));
            return portletErrorHandler.handleProblem(windowId, req, res, ed);
        }
    }
    
    /**
     * A helper method to obtain title string for a failed portlet
     * @param portletWindow
     * @return
     */
    private static String getPortletTitle(IPortletWindow portletWindow, Locale l) {
        if(portletWindow==null) { return null; }
        
        String title=null;
        if(l!=null) {
            Map<Locale,IDisplayName> displayNames=portletWindow.getDisplayNames();
            if(displayNames!=null) {
                IDisplayName displayName=displayNames.get(l);
                if(displayName!=null) {
                    title=displayName.getDisplayNameText();
                }
            }
        }
        if(title==null) {
            title=portletWindow.getPortletName();
        }
        return title;
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletWindowManager#getWindowState(org.jasig.portal.portlet.window.PortletWindowId, javax.servlet.http.HttpServletRequest)
     */
    public WindowState getWindowState(PortletWindowId windowId) {
        HttpServletRequest req=requestAccessor.getCurrentHttpServletRequest();
        IPortletWindow window=getPortletWindow(windowId,req);
        if(window!=null) {
            return window.getWindowState();
        } else {
            return null;
        }
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletWindowManager#setWindowState(org.jasig.portal.portlet.window.PortletWindowId, javax.portlet.WindowState, javax.servlet.http.HttpServletRequest)
     */
    public void setWindowState(PortletWindowId windowId, WindowState state) {
        HttpServletRequest req=requestAccessor.getCurrentHttpServletRequest();
        IPortletWindow window=getPortletWindow(windowId,req);
        if(window!=null) {
            WindowState oldState=window.getWindowState();
            if(!oldState.equals(state)) {
                window.setWindowState(state);
                portletCacheManager.invalidateCache(windowId);
                WindowState newState=window.getWindowState();
                if(!oldState.equals(newState)) {
                    this.distributeEvent(PortletWindowManagerEvent.createWindowStateChangeEvent(windowId,this,oldState,newState,req));
                }
            }
        }
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletWindowManager#getPortletMode(org.jasig.portal.portlet.window.PortletWindowId, javax.servlet.http.HttpServletRequest)
     */
    public PortletMode getPortletMode(PortletWindowId windowId) {
        HttpServletRequest req=requestAccessor.getCurrentHttpServletRequest();
        IPortletWindow window=getPortletWindow(windowId,req);
        if(window!=null) {
            return window.getPortletMode();
        } else {
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletWindowManager#setPortletMode(org.jasig.portal.portlet.window.PortletWindowId, javax.portlet.PortletMode, javax.servlet.http.HttpServletRequest)
     */
    public void setPortletMode(PortletWindowId windowId, PortletMode mode) {
        HttpServletRequest req=requestAccessor.getCurrentHttpServletRequest();
        IPortletWindow window=getPortletWindow(windowId,req);
        if(window!=null) {
            PortletMode oldMode=window.getPortletMode();
            if(!oldMode.equals(mode)) {
                portletCacheManager.invalidateCache(windowId);
                window.setPortletMode(mode);
                PortletMode newMode=window.getPortletMode();
                if(!oldMode.equals(newMode)) {
                    this.distributeEvent(PortletWindowManagerEvent.createModeChangeEvent(windowId,this,oldMode,newMode,req));
                }
            }
        }
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletWindowManager#setPortletRenderRequestParameters(org.jasig.portal.portlet.window.PortletWindowId, java.util.Map, javax.servlet.http.HttpServletRequest)
     */
    public void setPortletRenderRequestParameters(PortletWindowId windowId, Map<String,String[]> parameters) {
        HttpServletRequest req=requestAccessor.getCurrentHttpServletRequest();
        Map<PortletWindowId,Map<String,String[]>> parameterMap=getCreateRenderParameterMap(req);
        // obtain parameter map for the window
        parameterMap.put(windowId,parameters);
        portletCacheManager.invalidateCache(windowId);
    }
    
    protected IPortletWindow getPortletWindow(PortletWindowId windowId, HttpServletRequest req) {        
        // is error portlet replacing this window ?
        PortletWindowId replacingErrorWindowId=portletErrorHandler.getReplacingErrorPortletWindowId(windowId,req);
        if(replacingErrorWindowId!=null) {
            return transientPortletWindowRegistry.getPortletWindow(replacingErrorWindowId);
        } else {
            return transientPortletWindowRegistry.getPortletWindow(windowId);
        }
    }
    
    protected IUser getValidUser() {
        if(userController.isUserAuthenticated() && !userController.isAnonymous()) {
            return userController.getUser();
        } else {
            return null;
        }
    }
    
    /**
     * Obtain render request parameter map.
     * @param windowId window id
     * @param req http request
     * @return <code>Map</code> of <code>String</code> parameter names to <code>String[]</code> value arrays,
     * or <code>null</code> if no request parameters have been specified for this window
     */
    protected Map<String,String[]> retrievePortletRenderRequestParameters(PortletWindowId windowId,HttpServletRequest req) {
        Map<PortletWindowId,Map<String,String[]>> parameterMap=getPortletRenderParameters(req);
        if(parameterMap!=null) {
            // obtain parameter map for the window
            Map<String,String[]> windowParameterMap=parameterMap.get(windowId);
            if(windowParameterMap!=null) {
                return windowParameterMap;
            }
        }
        return Collections.EMPTY_MAP;
    }
    
    /**
     * Helper method to retrieve map of render parameter attribute maps.
     * Creates the map if it hasn't yet been initialized.
     * @param req http request
     * @return Map: PortletWindowId->{Map:String->String[]}
     */
    protected static Map<PortletWindowId,Map<String,String[]>> getCreateRenderParameterMap(HttpServletRequest req) {
        Map<PortletWindowId,Map<String,String[]>> parameterMap=(Map<PortletWindowId,Map<String,String[]>>)req.getSession().getAttribute(PORTLET_RENDERING_PARAM_MAP_SESSION_KEY);
        if(parameterMap==null) {
            parameterMap=new HashMap<PortletWindowId,Map<String,String[]>>();
            req.getSession().setAttribute(PORTLET_RENDERING_PARAM_MAP_SESSION_KEY,parameterMap);
        } 
        return parameterMap;
    }
    
    protected void clearPortletRenderParameters(PortletWindowId windowId,HttpServletRequest req) {
        Map parameterMap = (Map)req.getSession().getAttribute(PORTLET_RENDERING_PARAM_MAP_SESSION_KEY);
        if(parameterMap!=null) {
            parameterMap.remove(windowId);
        }
    }
    
    protected Map<PortletWindowId,Map<String,String[]>> getPortletRenderParameters(HttpServletRequest req) {
        return (Map<PortletWindowId,Map<String,String[]>>)req.getSession().getAttribute(PORTLET_RENDERING_PARAM_MAP_SESSION_KEY);
    }

    public Map<PortletWindowId,Map<String,String[]>> getPortletRenderParameters() {
        HttpServletRequest req=requestAccessor.getCurrentHttpServletRequest();
        return this.getPortletRenderParameters(req);
    }
    
    /**
     * @param windowManagerAccessor The windowManagerAccessor to set.
     */
    @Required
    public void setWindowManagerAccessor(IPortletWindowManagerAccessor windowManagerAccessor) {
        this.windowManagerAccessor = windowManagerAccessor;
    }

    /**
     * @param portletRenderingModel The portletRenderingModel to set.
     */
    @Required
    public void setPortletRenderingModel(IPortletRenderingModel portletRenderingModel) {
        this.portletRenderingModel = portletRenderingModel;
    }
    
    @Required
    public void setUserController(IUserController userController) {
        this.userController = userController;
        userController.addUserControllerEventListener(this);
    }

    @Required
    public void setPortletErrorHandler(IPortletErrorHandler portletErrorHandler) {
        this.portletErrorHandler = portletErrorHandler;
    }

    public void refreshPortletWindow(PortletWindowId windowId, boolean passLastRequestParameters) {
        HttpServletRequest req=requestAccessor.getCurrentHttpServletRequest();
        HttpServletResponse res=requestAccessor.getCurrentHttpServletResponse();
        portletErrorHandler.refreshPortletWindow(windowId,req,res,passLastRequestParameters);
        
    }

    public void restartPortletWindow(PortletWindowId windowId) {
        HttpServletRequest req=requestAccessor.getCurrentHttpServletRequest();
        HttpServletResponse res=requestAccessor.getCurrentHttpServletResponse();
        portletErrorHandler.restartPortletWindow(windowId,req,res);
    }

    public void receiveUserControllerEvent(UserControllerEvent event) {
        if(userController==event.getSource()) {
            // clean up request
            HttpServletRequest req=event.getHttpServletRequest();
            req.getSession().removeAttribute(PORTLET_RENDERING_PARAM_MAP_SESSION_KEY);
        }
    }

    public void setPortletCacheManager(IPortletCacheManager portletCacheManager) {
        this.portletCacheManager = portletCacheManager;
    }
    
    @Required
    public void setPortletWindowRegistry(ITransientPortletWindowRegistry transientPortletWindowRegistry) {
        this.transientPortletWindowRegistry = transientPortletWindowRegistry;
    }

    public void addPortletWindowManagerEventListener(IPortletWindowManagerEventListener listener) {
        eventListeners.put(listener,Boolean.TRUE);
    }

    public void removePortletWindowManagerEventListener(IPortletWindowManagerEventListener listener) {
        eventListeners.remove(listener);
    }
    
    protected void distributeEvent(PortletWindowManagerEvent event) {
        if(event!=null) {
            if (logger.isDebugEnabled()) {
                logger.debug("distributeEvent(PortletWindowManagerEvent) - distributing event : " + event.toString());
            }
            
            for(IPortletWindowManagerEventListener listener: eventListeners.keySet()) {
                if(listener!=null) {
                    listener.receivePortletWindowManagerEvent(event);
                }
            }
        }
    }

    /**
     * Specifies accessor used to obtain request and response objects
     * @param requestAccessor
     */
    @Required
    public void setRequestAccessor(IHttpRequestResponseAccessor requestAccessor) {
        this.requestAccessor = requestAccessor;
    }
    
}
