/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.servant;

import java.io.IOException;
import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.portal.portlet.rendering.IPortletRenderContent;

/** An interface that a Servant Portlet must implement.  A Servant Portlet is capable of providing some type of
 * interactive service within the flow of another Channel's use.  Originally designed for GroupsManager
 * and PermissionsManager, which can function both as standalone channels and provide the functions of 
 * selecting groups and people, and assigning permissions to them (respectively) for other channels
 *
 * @author Alex Vigdor - av317@columbia.edu
 * @author Michael Ivanov - mivanov at unicon.net
 * @version $Revision: 11911 $
 */
public interface IPortletServant {
    
    /**
     * Returns the servant ID given by the portlet servant locator.
     * @return a <code>String</code> value
     */
    public String getId();
    
    /** Allows the Client Portlet to ascertain if the Servant has accomplished the requested task 
     * (Note that the way which a certain task is requested is not specified by this interface; 
     * normally it will be documented by a particular IPortletServant and require some particular 
     * configuration paramaters used to initialize the servant)
     * @return boolean value
     */    
    public boolean isComplete(PortletRequest request);
    
    /** Allows the Client Portlet to ascertain if the Servant has been cancelled.
     * @return boolean value
     */    
    public boolean isCancelled(PortletRequest request);
    
    /**
     * Resets this servant for reuse.
     */
    public void reset(PortletRequest request);
    
    /**
     * Obtain servant shared attribute
     * @param request current portlet request
     * @param key attribute key
     * @return attribute value
     */
    public Object getAttribute(PortletRequest request, String key);

    /**
     * Set servant shared attribute. The attribute will be accessible by the servant portlet
     * through {@link ServantUtil}
     * @param request current portlet request
     * @param key attribute key
     * @param value attribute value
     */
    public void setAttribute(PortletRequest request, String key, Object value);

    /**
     * Render servant portlet into the current response.
     * Note: as this method renders directly to the response output stream, it is recommended
     * to use getRenderContent method instead (see below), and incorporate portlet content into
     * the overall portlet rendering manually.
     * @param request current portlet request
     * @param response current portlet reponse
     * @throws PortletException
     * @throws IOException
     */
    public void render(RenderRequest request, RenderResponse response) throws PortletException, IOException;
    
    /**
     * Initiate action processing on the servant portlet.
     * Note: the use of this method is not recommended. Servant portlet would normally process the portlet action requests
     * through the usual means (i.e. if an action request for the servant portlet comes in with the incoming HttpServletRequest).
     * Calling this method (during client portlet action or render request processing) goes against JSR168 spec (but technically, works).
     * @param actionRequestParameterMap
     * @param request
     * @param response
     * @throws PortletException
     * @throws IOException
     */
    public void processAction(Map<String, String[]> actionRequestParameterMap, ActionRequest request, ActionResponse response) throws PortletException, IOException;
    
    /**
     * Obtain content of the servant portlet for the current request.
     * @param request current portlet render request
     * @return content of the servant portlet
     */
    public IPortletRenderContent getRenderContent(RenderRequest request, RenderResponse response);
    
    
    /**
     * Return the servant result, if available.
     * @param request current portlet render request
     * @return the servant result.
     */
    public Object getResult(PortletRequest request);
    
}

