/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.transfer.impl;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.pluto.descriptors.portlet.CustomPortletModeDD;
import org.apache.pluto.descriptors.portlet.CustomWindowStateDD;
import org.apache.pluto.descriptors.portlet.PortletAppDD;
import org.apache.pluto.descriptors.portlet.SecurityConstraintDD;
import org.apache.pluto.descriptors.portlet.UserAttributeDD;
import org.jasig.portal.portlet.common.CustomPortletModeImpl;
import org.jasig.portal.portlet.common.CustomWindowStateImpl;
import org.jasig.portal.portlet.common.ICustomPortletMode;
import org.jasig.portal.portlet.common.ICustomWindowState;
import org.jasig.portal.portlet.common.ISecurityConstraint;
import org.jasig.portal.portlet.common.IUserAttribute;
import org.jasig.portal.portlet.common.SecurityConstraintImpl;
import org.jasig.portal.portlet.common.UserAttributeImpl;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.transfer.IPortletApplicationDeploymentTransfer;

/**
 * Implementation of a portlet application deployment transfer object.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11955 $
 */
public class PortletApplicationDeploymentTransferImpl implements IPortletApplicationDeploymentTransfer {
    private final String portletApplicationName;
    private final PortletAppDD portletAppDD;

    public PortletApplicationDeploymentTransferImpl(final String portletApplicationName, final PortletAppDD portletAppDD) {
        if (portletApplicationName == null) {
            throw new IllegalArgumentException("portletApplicationName can not be null");
        }
        if (portletAppDD == null) {
            throw new IllegalArgumentException("PortletAppDD can not be null");
        }
        
        this.portletApplicationName = portletApplicationName;
        this.portletAppDD = portletAppDD;
    }

    /**
     * @see org.jasig.portal.portlet.transfer.IPortletApplicationDeploymentTransfer#getPortletApplicationDeploymentId()
     */
    public PortletApplicationDeploymentId getPortletApplicationDeploymentId() {
        return new PortletApplicationDeploymentId(this.portletApplicationName);
    }

    /**
     * @see org.jasig.portal.portlet.deployment.IPortletApplicationDeploymentInformationProvider#getCustomPortletModes()
     */
    @SuppressWarnings("unchecked")
    public Set<ICustomPortletMode> getCustomPortletModes() {
        final List<CustomPortletModeDD> plutoCustomPortletModes = this.portletAppDD.getCustomPortletModes();
        final Set<ICustomPortletMode> upCustomPortletModes = new HashSet<ICustomPortletMode>(plutoCustomPortletModes.size());
        
        for (final CustomPortletModeDD plutoCustomPortletMode : plutoCustomPortletModes) {
            final CustomPortletModeImpl upCustomPortletMode = new CustomPortletModeImpl(plutoCustomPortletMode);
            upCustomPortletModes.add(upCustomPortletMode);
        }
        
        return upCustomPortletModes;
    }

    /**
     * @see org.jasig.portal.portlet.deployment.IPortletApplicationDeploymentInformationProvider#getCustomWindowStates()
     */
    @SuppressWarnings("unchecked")
    public Set<ICustomWindowState> getCustomWindowStates() {
        final List<CustomWindowStateDD> plutoCustomWindowStates = this.portletAppDD.getCustomWindowStates();
        final Set<ICustomWindowState> upCustomWindowStates = new HashSet<ICustomWindowState>(plutoCustomWindowStates.size());

        for (final CustomWindowStateDD plutoCustomWindowState : plutoCustomWindowStates) {
            final CustomWindowStateImpl upCustomWindowState = new CustomWindowStateImpl(plutoCustomWindowState);
            upCustomWindowStates.add(upCustomWindowState);
        }
        
        return upCustomWindowStates;
    }

    /**
     * @see org.jasig.portal.portlet.deployment.IPortletApplicationDeploymentInformationProvider#getSecurityConstraints()
     */
    @SuppressWarnings("unchecked")
    public Set<ISecurityConstraint> getSecurityConstraints() {
        final List<SecurityConstraintDD> plutoSecurityConstraints = this.portletAppDD.getSecurityConstraints();
        final Set<ISecurityConstraint> upSecurityConstraints = new HashSet<ISecurityConstraint>(plutoSecurityConstraints.size());

        for (final SecurityConstraintDD plutoSecurityConstraint : plutoSecurityConstraints) {
            final SecurityConstraintImpl upSecurityConstraint = new SecurityConstraintImpl(plutoSecurityConstraint);
            upSecurityConstraints.add(upSecurityConstraint);
        }
        
        return upSecurityConstraints;
    }

    /**
     * @see org.jasig.portal.portlet.deployment.IPortletApplicationDeploymentInformationProvider#getUserAttributes()
     */
    @SuppressWarnings("unchecked")
    public Set<IUserAttribute> getUserAttributes() {
        final List<UserAttributeDD> plutoUserAttributes = this.portletAppDD.getUserAttributes();
        final Set<IUserAttribute> upUserAttributes = new HashSet<IUserAttribute>(plutoUserAttributes.size());

        for (final UserAttributeDD plutoUserAttribute : plutoUserAttributes) {
            final UserAttributeImpl upUserAttribute = new UserAttributeImpl(plutoUserAttribute);
            upUserAttributes.add(upUserAttribute);
        }
        
        return upUserAttributes;
    }

    /**
     * @see org.jasig.portal.portlet.deployment.IPortletApplicationDeploymentInformationProvider#getVersion()
     */
    public String getVersion() {
        return this.portletAppDD.getVersion();
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof IPortletApplicationDeploymentTransfer)) {
            return false;
        }
        IPortletApplicationDeploymentTransfer rhs = (IPortletApplicationDeploymentTransfer)object;
        return new EqualsBuilder()
            .append(this.getPortletApplicationDeploymentId(), rhs.getPortletApplicationDeploymentId())
            .append(this.getVersion(), rhs.getVersion())
            .append(this.getCustomWindowStates(), rhs.getCustomWindowStates())
            .append(this.getCustomPortletModes(), rhs.getCustomPortletModes())
            .append(this.getUserAttributes(), rhs.getUserAttributes())
            .append(this.getSecurityConstraints(), rhs.getSecurityConstraints())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(-766564685, 470385279)
            .append(this.getPortletApplicationDeploymentId())
            .append(this.getVersion())
            .append(this.getCustomWindowStates())
            .append(this.getCustomPortletModes())
            .append(this.getUserAttributes())
            .append(this.getSecurityConstraints())
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this).appendSuper(super.toString())
            .append("portletApplicationDeploymentId", this.getPortletApplicationDeploymentId())
            .append("version", this.getVersion())
            .append("customWindowStates", this.getCustomWindowStates())
            .append("customPortletModes", this.getCustomPortletModes())
            .append("userAttributes", this.getUserAttributes())
            .append("securityConstraints", this.getSecurityConstraints())
            .toString();
    }
}
