/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.transfer.impl;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.pluto.descriptors.common.DescriptionDD;
import org.apache.pluto.descriptors.common.DisplayNameDD;
import org.apache.pluto.descriptors.common.InitParamDD;
import org.apache.pluto.descriptors.common.SecurityRoleRefDD;
import org.apache.pluto.descriptors.portlet.PortletDD;
import org.apache.pluto.descriptors.portlet.PortletInfoDD;
import org.apache.pluto.descriptors.portlet.PortletPreferencesDD;
import org.apache.pluto.descriptors.portlet.SupportsDD;
import org.jasig.portal.portlet.common.ContentTypeImpl;
import org.jasig.portal.portlet.common.IContentType;
import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.common.IInitParameter;
import org.jasig.portal.portlet.common.IPortletInfo;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.common.ISecurityRoleReference;
import org.jasig.portal.portlet.common.InitParameterImpl;
import org.jasig.portal.portlet.common.SecurityRoleReferenceImpl;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.deployment.PortletDeploymentId;
import org.jasig.portal.portlet.deployment.PlutoDescriptionImpl;
import org.jasig.portal.portlet.deployment.PlutoDisplayNameImpl;
import org.jasig.portal.portlet.deployment.PlutoPortletInfo;
import org.jasig.portal.portlet.deployment.PlutoPortletPreferences;
import org.jasig.portal.portlet.transfer.IPortletDeploymentTransfer;



/**
 * Implementation of a portlet definition transfer object.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11955 $
 */
public class PortletDeploymentTransferImpl implements IPortletDeploymentTransfer {
    private final PortletDD portletDD;
    private final PortletDeploymentId portletDeploymentId;
    
    public PortletDeploymentTransferImpl(PortletDeploymentId portletDeploymentId, PortletDD portletDD) {
        if (portletDD == null) {
            throw new IllegalArgumentException("PortletDD can not be null");
        }
        if (portletDeploymentId == null) {
            throw new IllegalArgumentException("PortletApplicationDeploymentId can not be null");
        }
        
        this.portletDD = portletDD;
        this.portletDeploymentId = portletDeploymentId;
    }

    /**
     * @see org.jasig.portal.portlet.transfer.IPortletDeploymentTransfer#getPortletApplicationDeploymentId()
     */
    public PortletApplicationDeploymentId getPortletApplicationDeploymentId() {
        return this.portletDeploymentId.getPortletApplicationDeploymentId();
    }

    /**
     * @see org.jasig.portal.portlet.transfer.IPortletDeploymentTransfer#getPortletDeploymentId()
     */
    public PortletDeploymentId getPortletDeploymentId() {
        return this.portletDeploymentId;
    }

    /**
     * @see org.jasig.portal.portlet.deployment.IPortletDeploymentInformationProvider#getDisplayNames()
     */
    @SuppressWarnings("unchecked")
    public Map<Locale, IDisplayName> getDisplayNames() {
        final List<DisplayNameDD> plutoDisplayNames = this.portletDD.getDisplayNames();
        final Map<Locale, IDisplayName> upDisplayNames = new HashMap<Locale, IDisplayName>(plutoDisplayNames.size());
        
        for (final DisplayNameDD plutoDisplayName : plutoDisplayNames) {
            final IDisplayName upDisplayName = new PlutoDisplayNameImpl(plutoDisplayName);
            upDisplayNames.put(upDisplayName.getLocale(), upDisplayName);
        }
        
        return Collections.unmodifiableMap(upDisplayNames);
    }

    /**
     * @see org.jasig.portal.portlet.deployment.IPortletDeploymentInformationProvider#getInitParameters()
     */
    @SuppressWarnings("unchecked")
    public Map<String, IInitParameter> getInitParameters() {
        final List<InitParamDD> plutoInitParams = this.portletDD.getInitParams();
        final Map<String, IInitParameter> upInitParams = new HashMap<String, IInitParameter>(plutoInitParams.size());
        for (final InitParamDD plutoInitParam : plutoInitParams) {
            final IInitParameter upInitParam = new InitParameterImpl(plutoInitParam);
            upInitParams.put(upInitParam.getName(), upInitParam);
        }
        return upInitParams;
    }

    /**
     * @see org.jasig.portal.portlet.deployment.IPortletDeploymentInformationProvider#getPortletClassName()
     */
    public String getPortletClassName() {
        return this.portletDD.getPortletClass();
    }

    /**
     * @see org.jasig.portal.portlet.deployment.IPortletDeploymentInformationProvider#getPortletExpirationCache()
     */
    public Integer getPortletExpirationCache() {
        return this.portletDD.getExpirationCache();
    }

    /**
     * @see org.jasig.portal.portlet.deployment.IPortletDeploymentInformationProvider#getPortletInfo()
     */
    public IPortletInfo getPortletInfo() {
        final PortletInfoDD plutoPortletInfo = this.portletDD.getPortletInfo();
        if (plutoPortletInfo == null) {
            return null;
        }
        else {
            final IPortletInfo upPortletInfo = new PlutoPortletInfo(plutoPortletInfo);
            return upPortletInfo;
        }
    }

    /**
     * @see org.jasig.portal.portlet.deployment.IPortletDeploymentInformationProvider#getPortletName()
     */
    public String getPortletName() {
        return this.portletDD.getPortletName();
    }

    /**
     * @see org.jasig.portal.portlet.deployment.IPortletDeploymentInformationProvider#getPreferences()
     */
    public IPreferences getPreferences() {
        final PortletPreferencesDD plutoPortletPreferences = this.portletDD.getPortletPreferences();
        return new PlutoPortletPreferences(plutoPortletPreferences);
    }

    /**
     * @see org.jasig.portal.portlet.deployment.IPortletDeploymentInformationProvider#getSecurityRoleReferences()
     */
    @SuppressWarnings("unchecked")
    public Map<String, ISecurityRoleReference> getSecurityRoleReferences() {
        final List<SecurityRoleRefDD> plutoSecurityRoleRefs = this.portletDD.getSecurityRoleRefs();
        final Map<String, ISecurityRoleReference> upSecurityRoleRefs = new HashMap<String, ISecurityRoleReference>(plutoSecurityRoleRefs.size());
        
        for (final SecurityRoleRefDD securityRoleRefDD : plutoSecurityRoleRefs) {
            final ISecurityRoleReference securityRoleReference = new SecurityRoleReferenceImpl(securityRoleRefDD);
            upSecurityRoleRefs.put(securityRoleReference.getRoleName(), securityRoleReference);
        }

        return upSecurityRoleRefs;
    }

    /**
     * @see org.jasig.portal.portlet.deployment.IPortletDeploymentInformationProvider#getSupportedContentTypes()
     */
    @SuppressWarnings("unchecked")
    public Map<String, IContentType> getSupportedContentTypes() {
        final List<SupportsDD> plutoSupportedContentTypes = this.portletDD.getSupports();
        final Map<String, IContentType> upSupportedContentTypes = new HashMap<String, IContentType>(plutoSupportedContentTypes.size());
        
        for (final SupportsDD supportsDD : plutoSupportedContentTypes) {
            final IContentType contentType = new ContentTypeImpl(supportsDD);
            upSupportedContentTypes.put(contentType.getMimeType(), contentType);
        }

        return upSupportedContentTypes;
    }

    /**
     * @see org.jasig.portal.portlet.deployment.IPortletDeploymentInformationProvider#getSupportedLocales()
     */
    @SuppressWarnings("unchecked")
    public Set<Locale> getSupportedLocales() {
        final List<String> plutoSupportedLocales = this.portletDD.getSupportedLocales();
        final Set<Locale> upSupportedLocales = new HashSet<Locale>(plutoSupportedLocales.size());
        
        for (final String plutoLocale : plutoSupportedLocales) {
            final Locale upLocale = new Locale(plutoLocale);
            upSupportedLocales.add(upLocale);
        }

        return Collections.unmodifiableSet(upSupportedLocales);
    }

    /**
     * @see org.jasig.portal.portlet.common.IDescribable#getDescriptions()
     */
    @SuppressWarnings("unchecked")
    public Map<Locale, IDescription> getDescriptions() {
        final List<DescriptionDD> plutoDescriptions = this.portletDD.getDescriptions();
        final Map<Locale, IDescription> upDescriptions = new HashMap<Locale, IDescription>(plutoDescriptions.size());
        
        for (final DescriptionDD plutoDescription : plutoDescriptions) {
            final IDescription upDescription = new PlutoDescriptionImpl(plutoDescription);
            upDescriptions.put(upDescription.getLocale(), upDescription);
        }
        
        return Collections.unmodifiableMap(upDescriptions);
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof IPortletDeploymentTransfer)) {
            return false;
        }
        IPortletDeploymentTransfer rhs = (IPortletDeploymentTransfer)object;
        return new EqualsBuilder()
            .append(this.portletDeploymentId, rhs.getPortletDeploymentId())
            .append(this.getPortletApplicationDeploymentId(), rhs.getPortletApplicationDeploymentId())
            .append(this.getPortletName(), rhs.getPortletName())
            .append(this.getDisplayNames(), rhs.getDisplayNames())
            .append(this.getDescriptions(), rhs.getDescriptions())
            .append(this.getSupportedLocales(), rhs.getSupportedLocales())
            .append(this.getSupportedContentTypes(), rhs.getSupportedContentTypes())
            .append(this.getPortletClassName(), rhs.getPortletClassName())
            .append(this.getPortletExpirationCache(), rhs.getPortletExpirationCache())
            .append(this.getInitParameters(), rhs.getInitParameters())
            .append(this.getPortletInfo(), rhs.getPortletInfo())
            .append(this.getPreferences(), rhs.getPreferences())
            .append(this.getSecurityRoleReferences(), rhs.getSecurityRoleReferences())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(-204673741, -1615737423)
            .append(this.portletDeploymentId)
            .append(this.getPortletApplicationDeploymentId())
            .append(this.getPortletName())
            .append(this.getDisplayNames())
            .append(this.getDescriptions())
            .append(this.getSupportedLocales())
            .append(this.getSupportedContentTypes())
            .append(this.getPortletClassName())
            .append(this.getPortletExpirationCache())
            .append(this.getInitParameters())
            .append(this.getPortletInfo())
            .append(this.getPreferences())
            .append(this.getSecurityRoleReferences())
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this).appendSuper(super.toString())
            .append("portletDeploymentId", this.portletDeploymentId)
            .append("portletApplicationDeploymentId", this.getPortletApplicationDeploymentId())
            .append("portletName", this.getPortletName())
            .append("displayNames", this.getDisplayNames())
            .append("descriptions", this.getDescriptions())
            .append("supportedLocales", this.getSupportedLocales())
            .append("supportedContentTypes", this.getSupportedContentTypes())
            .append("portletClassName", this.getPortletClassName())
            .append("portletExpirationCache", this.getPortletExpirationCache())
            .append("initParameters", this.getInitParameters())
            .append("portletInfo", this.getPortletInfo())
            .append("preferences", this.getPreferences())
            .append("securityRoleReferences", this.getSecurityRoleReferences())
            .toString();
    }
    
    
}
