/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.url.impl;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jasig.portal.portlet.url.IPortletParameterSyntaxProvider;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.url.IRequestParameterProcessor;
import org.jasig.portal.url.IWritableHttpServletRequest;

/**
 * Processes form submitted portlet parameters. This needs to be one of the first parameter
 * processesors to run as it will be affected by other processesors that add parameters to the
 * request that weren't on the original request.
 * <br>
 * <br>
 * This processor works on requests targeting portlets. If there are parameters in the request
 * Map that were not on the request query string {@link javax.servlet.http.HttpServletRequest#getQueryString()}
 * it is assumed these are from the BODY of a POST request from a form. These parameters are renamed
 * to the correct portlet parameter namespace so they will be handled correctly by other request
 * parameter processors.
 *
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class PortletRequestSubmitParameterProcessor implements IRequestParameterProcessor {
	private IPortletParameterSyntaxProvider parameterSyntax;
	
	/**
	 * @param parameterSyntax The parameterSyntax to set.
	 */
	public void setParameterSyntaxProvider(IPortletParameterSyntaxProvider parameterSyntax) {
		this.parameterSyntax = parameterSyntax;
	}
    
    private static final Pattern URL_PARAM_NAME = Pattern.compile("&([^&?=\n]*)");
	
    public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res) {
        // See if the request targets a portlet
        PortletWindowId targetWindow = this.parameterSyntax.getTargetWindow(req);
        if (targetWindow != null) {
            final Set urlParameterNames = this.getUrlParameterNames(req);
            final Map parameters = req.getParameterMap();
            
            //If there are more parameters in the Map than were on the URL parameter re-writing should be done
            if (urlParameterNames.size() != parameters.size()) {
                final Set bodyParameterNames = new HashSet();
                
                //Populate all the parameter names that don't appear in the URL
                for (final Iterator paramNameItr = parameters.keySet().iterator(); paramNameItr.hasNext();) {
                    final String paramName = (String)paramNameItr.next();
                    
                    if (!urlParameterNames.contains(paramName)) {
                        bodyParameterNames.add(paramName);
                    }
                }
    
                //Each parameter that was not on the URL is removed from the request, re-named
                //with the portlet parameter syntax and added back to the request
                for (final Iterator bodyParamNameItr = bodyParameterNames.iterator(); bodyParamNameItr.hasNext();) {
                    final String paramName = (String)bodyParamNameItr.next();
                    final String prfxdParamName = this.parameterSyntax.getRequestParameterName(paramName, targetWindow);
                    
                    final String[] paramValue = req.getParameterValues(paramName);
                    req.deleteParameter(paramName);
                    req.setParameterValues(prfxdParamName, paramValue);
                }
            }
        }  
             
        return true;
    }
    
    public boolean canEvaluateVariables() {
    	return true;
    }


    /**
     * Parses the request URL to return a Set of the parameter names that appeared on the URL string.
     * 
     * @param req The request to look at.
     * @return The Set of parameter names from the URL.
     */
    private Set getUrlParameterNames(HttpServletRequest req) {
        final Set urlParameterNames = new HashSet();
        
        final String queryString = req.getQueryString();
        final Matcher paramNameMatcher = URL_PARAM_NAME.matcher("&" + queryString);

        final String reqEncoding = req.getCharacterEncoding();
        final String encoding;
        if (reqEncoding == null) {
            encoding = "UTF-8";
        }
        else {
            encoding = reqEncoding;
        }
        
        while (paramNameMatcher.find()) {
            final String paramName = paramNameMatcher.group(1);
            String decParamName;
            try {
                decParamName = URLDecoder.decode(paramName, encoding);
            }
            catch (UnsupportedEncodingException uee) {
                decParamName = paramName;
            }
            
            urlParameterNames.add(decParamName);
        }
        
        return urlParameterNames;
    }
}
