/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.url.impl;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.core.exception.PortalRuntimeException;
import org.jasig.portal.portlet.registry.general.IPortletWindowRegistry;
import org.jasig.portal.portlet.url.IPortletParameterSyntaxProvider;
import org.jasig.portal.portlet.url.IPortletUrlConstructor;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.url.UrlInfo;

/**
 * The PortletURL constructor implementation.
 * It is used by PortletURLProvider.
 *
 * @author Michael Ivanov: mivanov at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletUrlConstructorImpl implements IPortletUrlConstructor {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PortletUrlConstructorImpl.class);
	
	private IPortletParameterSyntaxProvider syntaxProvider;
	private IPortletWindowRegistry portletWindowRegistry;
	private Map<PortletWindowId, PortletWindowUrl> windowUrls = new HashMap<PortletWindowId, PortletWindowUrl>();
    private Boolean setSecure=null;
    private String requestUrl;
    
    private String secureProtocolName=UrlInfo.HTTPS_PROTOCOL;
    private String unsecureProtocolName=UrlInfo.HTTP_PROTOCOL;
    private String securePortNumber=null;
    private String unsecurePortNumber=null;
    private PortletWindowId targetWindowId=null;
    private boolean isAction=false;
	
	private class PortletWindowUrl {
		private PortletWindowId portletWindowId;
        // default mode
		private transient PortletMode portletMode;
		// default window state
		private transient WindowState windowState;
		private boolean isTarget = false;
        private boolean isAction=false;
		private Map parameters;
		
		public PortletWindowUrl() {}
		
		/**
		 * @return Returns the isTarget.
		 */
		public boolean isTarget() {
			return isTarget;
		}
		/**
		 * @param isTarget The isTarget to set.
		 */
		public void setTarget(boolean isTarget) {
			this.isTarget = isTarget;
		}
		/**
		 * @return Returns the parameters.
		 */
		public Map getParameters() {
			return parameters;
		}
		/**
		 * @param parameters The parameters to set.
		 */
		public void setParameters(Map parameters) {
			this.parameters = parameters;
		}
		/**
		 * @return Returns the portletMode.
		 */
		public PortletMode getPortletMode() {
			return portletMode;
		}
		/**
		 * @param portletMode The portletMode to set.
		 */
		public void setPortletMode(PortletMode portletMode) {
			this.portletMode = portletMode;
		}
		/**
		 * @return Returns the portletWindowId.
		 */
		public PortletWindowId getPortletWindowId() {
			return portletWindowId;
		}
		/**
		 * @param portletWindowId The portletWindowId to set.
		 */
		public void setPortletWindowId(PortletWindowId portletWindowId) {
			this.portletWindowId = portletWindowId;
		}
		/**
		 * @return Returns the windowState.
		 */
		public WindowState getWindowState() {
			return windowState;
		}
		/**
		 * @param windowState The windowState to set.
		 */
		public void setWindowState(WindowState windowState) {
			this.windowState = windowState;
		}
		
		public void addParametersToUrl(UrlInfo url) {
			
		 if ( url != null ) {
            if( isTarget ) {
                url.addParameterValue(syntaxProvider.getTargetWindowCommandName(),portletWindowId.toString());
            }
			if ( isAction ) {
			  url.addParameterValue(syntaxProvider.getActionTargetCommandName(),"true");
			}
			if ( portletMode != null ) {
			  url.addParameterValue(syntaxProvider.getPortletModeCommandName(portletWindowId),portletMode.toString());
			}
			if ( windowState != null ) {
			  url.addParameterValue(syntaxProvider.getWindowStateCommandName(portletWindowId),windowState.toString());
			}
			
            if ( parameters != null ) { 
			 for ( Iterator keys = parameters.keySet().iterator(); keys.hasNext(); ) {
				String name = (String) keys.next();
				Object value = parameters.get(name);
	            String[] values = (value instanceof String[]) ? (String[]) value : new String[] {value.toString()};
				for ( int i = 0; i < values.length; i++ ) {
				  url.addParameterValue(syntaxProvider.getRequestParameterName(name,portletWindowId),values[i]);
				} 
			 }
            } 
		 } 
            
		}

        public boolean isAction() {
            return isAction;
        }

        public void setAction(boolean isAction) {
            this.isAction = isAction;
        }
		
	}
	
	
	private PortletWindowUrl getWindowUrl(PortletWindowId windowId) {
		if ( portletWindowRegistry != null ) {
		 if ( portletWindowRegistry.getPortletWindow(windowId) == null )
		  	throw new PortalRuntimeException ( "The portlet window with ID="+windowId+" does not exist in the registry!");
		}
		PortletWindowUrl windowUrl = (PortletWindowUrl) windowUrls.get(windowId);
		if ( windowUrl == null ) {
		  windowUrl = new PortletWindowUrl();
		  windowUrl.setPortletWindowId(windowId);
		  windowUrls.put(windowId,windowUrl);
		}
		return windowUrl;
	}
	
    /**
     * Extract the real path element from the request URL.
     * Extracts {"this" and "path"} from "http://host/this/path/index.html"
     * @param requestUrl
     * @return
     */
    protected String[] getValidPathElements(URL requestUrl) {
        String raw=requestUrl.getPath();
        
        String[] pathElements=raw.split("/");
        if(raw.endsWith("/")) {
            // no file spec
            return pathElements;
        } else {
            if(pathElements.length>1) {
                // some path elements exist
                String[] validPathElements=new String[pathElements.length-1];
                for (int i = 0; i < pathElements.length-1; i++) {
                    validPathElements[i]=pathElements[i];
                }
                return validPathElements;
            } else {
                // just the file spec, no path elements
                return null;
            }
        }
    }
    
	/**
	 * Commits the changes to a given instance of UrlInfo
	 * @param urlInfo a UrlInfo instance
	 */
	public void writeToUrl(UrlInfo urlInfo) {
	    // adjust security level if necessary
        if(setSecure!=null) {
            // parse out request URL
            try {
                URL url=new URL(requestUrl);
                
                if(setSecure.booleanValue()) {
                    if(!secureProtocolName.equals(url.getProtocol())) {
                        // not already secure, need to adjust
                        urlInfo.setProtocol(secureProtocolName);
                        urlInfo.setHost(url.getHost());
                        if(securePortNumber!=null) {
                            urlInfo.setPort(securePortNumber);
                        }
                        String[] pathElements=getValidPathElements(url);
                        for (int i = 0; i < pathElements.length; i++) {
                            urlInfo.insertPathElement(i,pathElements[i]);
                        }
                    }
                } else {
                    // move to unsecure URL
                    if(!unsecureProtocolName.equals(url.getProtocol())) {
                        // not already unsecure, need to adjust
                        urlInfo.setProtocol(unsecureProtocolName);
                        urlInfo.setHost(url.getHost());
                        if(unsecurePortNumber!=null) {
                            urlInfo.setPort(unsecurePortNumber);
                        }
                        String[] pathElements=getValidPathElements(url);
                        for (int i = 0; i < pathElements.length; i++) {
                            urlInfo.insertPathElement(i,pathElements[i]);
                        }
                    }
                }
            } catch (MalformedURLException e) {
                logger.error("writeToUrl(UrlInfo) - unable to adjust secure mode because supplied request URL is invalid: requestUrl="+requestUrl, e);
            }
        }
        
      for ( Iterator i =  windowUrls.values().iterator(); i.hasNext(); ) {
	    ((PortletWindowUrl)i.next()).addParametersToUrl(urlInfo);		
	  }
	}
	
	/**
     * Change portlet window mode in the portlet URL for the next request.
     * @param windowId window id
     * @param mode new portlet mode
     */
    public void setPortletMode(PortletWindowId windowId, PortletMode mode) {
       getWindowUrl(windowId).setPortletMode(mode);    	
    }
    
    /**
     * Change portlet window state in the portlet URL for the next request.
     * @param windowId window id
     * @param state new portlet window state
     */
    public void setWindowState(PortletWindowId windowId, WindowState state) {
       getWindowUrl(windowId).setWindowState(state);    	
    }

    
    
    public void setActionUrl(boolean value) {
        isAction=value;
        if(targetWindowId!=null) {
            getWindowUrl(targetWindowId).setAction(value);
        }
    }

    /**
     * Sets the parameters in the portlet URL for the next request. 
     * Overwrites previously added parameters for the given window.
     * @param windowId window id
     * @param parameters a map of portlet URL parameters
     */
    public void setParameters(PortletWindowId windowId, Map parameters) {
       getWindowUrl(windowId).setParameters(parameters);
    }
    
    /**
     * Adds the parameters to existing parameters in the portlet URL for the next request.
     * @param windowId window id
     * @param parameters a map of portlet URL parameters
     */
    public void addParameters(PortletWindowId windowId, Map parameters) {
    	Map windowParams = getWindowUrl(windowId).getParameters();
    	if ( windowParams == null )
    	  setParameters(windowId,parameters);
    	else
    	  windowParams.putAll(parameters);
    }
    
    /**
     * Sets the parameter in the portlet URL for the next request.
     * @param windowId window id
     * @param value a parameter value
     */
    public void setParameter(PortletWindowId windowId, String paramName, String value) {
    	Map windowParams = getWindowUrl(windowId).getParameters();
    	if ( windowParams == null ) {
    		windowParams = new HashMap();
    		getWindowUrl(windowId).setParameters(windowParams);
    	}
    	windowParams.put(paramName,new String[] {value});
    }
    
    /**
     * Sets the parameter in the portlet URL for the next request.
     * @param windowId window id
     * @param values a parameter values
     */
    public void setParameter(PortletWindowId windowId, String paramName, String[] values) {
    	Map windowParams = getWindowUrl(windowId).getParameters();
    	if ( windowParams == null ) {
    		windowParams = new HashMap();
    		getWindowUrl(windowId).setParameters(windowParams);
    	}
    	windowParams.put(paramName,values);
    }
    
    
    /**
     * Removes portlet URL parameters for the given window.
     * @param windowId window id
     */
    public void clearParameters(PortletWindowId windowId) {
    	Map windowParams = getWindowUrl(windowId).getParameters();
    	if ( windowParams != null )
    	  windowParams.clear();
    }
    
    /**
     * Clears the entire portlet URL
     */
    public void clearUrl() {
       windowUrls.clear();
    }
    
    /**
     * Clears the window portlet URL
     * @param windowId window id
     */
    public void clearUrl(PortletWindowId windowId) {
       windowUrls.remove(windowId);
    }
    
	/**
	 * @param portletWindowRegistry The portletWindowRegistry to set.
	 */
	public void setPortletWindowRegistry(IPortletWindowRegistry portletWindowRegistry) {
		this.portletWindowRegistry = portletWindowRegistry;
	}
	/**
	 * @param syntaxProvider The syntaxProvider to set.
	 */
	public void setSyntaxProvider(IPortletParameterSyntaxProvider syntaxProvider) {
		this.syntaxProvider = syntaxProvider;
	}

    public void setSecure(boolean secure) {
        this.setSecure=new Boolean(secure);
    }

    public void setRequestUrl(String requestUrl) {
        this.requestUrl = requestUrl;
    }

    /**
     * Set the secure protocol (defaults to "https")
     * @param secureProtocolName
     */
    public void setSecureProtocolName(String secureProtocolName) {
        this.secureProtocolName = secureProtocolName;
    }

    /**
     * Set unsecure protocol (defaults to "http")
     * @param unsecureProtocolName
     */
    public void setUnsecureProtocolName(String unsecureProtocolName) {
        this.unsecureProtocolName = unsecureProtocolName;
    }

    /**
     * Set secure port number (defaults to <code>null</code>)
     * @param securePortNumber port number, or <code>null</code> if no port should be secified
     */
    public void setSecurePortNumber(String securePortNumber) {
        this.securePortNumber = securePortNumber;
    }

    /**
     * Set unsecure port number (defaults to <code>null</code>)
     * @param unsecurePortNumber port number, or <code>null</code> if no port should be secified
     */
    public void setUnsecurePortNumber(String unsecurePortNumber) {
        this.unsecurePortNumber = unsecurePortNumber;
    }

    public PortletWindowId getTargetWindowId() {
        return targetWindowId;
    }

    public void setTargetWindowId(PortletWindowId targetWindowId) {
        this.targetWindowId = targetWindowId;
        getWindowUrl(targetWindowId).setTarget(true);
        getWindowUrl(targetWindowId).setAction(isAction);
    }

}
