/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.userattributes;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.portlet.PortletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.portlet.common.IUserAttribute;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.entity.IPortletApplicationEntity;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.factory.IPortletApplicationEntityRegistryFactory;
import org.jasig.portal.portlet.registry.general.IPortletApplicationDefinitionRegistry;
import org.jasig.portal.portlet.registry.general.IPortletApplicationEntityRegistry;
import org.jasig.portal.security.IUser;
import org.jasig.services.persondir.IPersonAttributeDao;
import org.springframework.beans.factory.annotation.Required;

/**
 * Gets user attributes who's names have been mapped from the uportal specific names to the
 * portlet specific names.
 * 
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public class PersonDirectoryMappedUserAttributeProvider implements IMappedUserAttributeProvider {
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private IPortletApplicationEntityRegistryFactory portletApplicationEntityRegistryFactory;
    private IPortletApplicationDefinitionRegistry portletApplicationDefinitionRegistry;
    private IPersonAttributeDao personAttributeDao;
    
    public IPersonAttributeDao getPersonAttributeDao() {
        return this.personAttributeDao;
    }

    @Required
    public void setPersonAttributeDao(IPersonAttributeDao personAttributeDao) {
        this.personAttributeDao = personAttributeDao;
    }

    public IPortletApplicationDefinitionRegistry getPortletApplicationDefinitionRegistry() {
        return this.portletApplicationDefinitionRegistry;
    }

    @Required
    public void setPortletApplicationDefinitionRegistry(IPortletApplicationDefinitionRegistry portletApplicationDefinitionRegistry) {
        this.portletApplicationDefinitionRegistry = portletApplicationDefinitionRegistry;
    }

    public IPortletApplicationEntityRegistryFactory getPortletApplicationEntityRegistryFactory() {
        return this.portletApplicationEntityRegistryFactory;
    }

    @Required
    public void setPortletApplicationEntityRegistryFactory(IPortletApplicationEntityRegistryFactory portletApplicationEntityRegistryFactory) {
        this.portletApplicationEntityRegistryFactory = portletApplicationEntityRegistryFactory;
    }

    /**
     * @see org.jasig.portal.portlet.userattributes.IMappedUserAttributeProvider#getMappedUserAttributes(org.jasig.portal.security.IUser, org.jasig.portal.portlet.entity.PortletApplicationEntityId)
     */
    public Map<String, String> getMappedUserAttributes(IUser user, PortletApplicationEntityId portAppEntId) {
        //Get the app ent registry and app ent to get the user attributes set from
        final IPortletApplicationEntityRegistry portletApplicationEntityRegistry = this.portletApplicationEntityRegistryFactory.getPortletApplicationEntityRegistry(user.getId());
        final IPortletApplicationEntity portletApplicationEntity = portletApplicationEntityRegistry.getPortletApplicationEntity(portAppEntId);
        final Set<IUserAttribute> userAttributes = portletApplicationEntity.getUserAttributes();

        //Get the app def to get the user attribute mappings from
        final PortletApplicationDefinitionId portletApplicationDefinitionId = portletApplicationEntity.getPortletApplicationDefinitionId();
        final IPortletApplicationDefinition portletApplicationDefinition = this.portletApplicationDefinitionRegistry.getPortletApplicationDefinition(portletApplicationDefinitionId);
        final Map<String, String> userAttributeMappings = portletApplicationDefinition.getUserAttributeMappings();
        
        //Get the mapped attributes
        final Map<String, String> mappedUserAttributes = this.getMappedUserAttributes(user, userAttributes, userAttributeMappings);
        
        if (this.logger.isInfoEnabled()) {
            final String mapInfo;
            if (mappedUserAttributes == null) {
                mapInfo = "null";
            }
            else {
                mapInfo = Integer.toString(mappedUserAttributes.size());
            }
            
            this.logger.info("Returning " + mapInfo + " mapped user attributes for IUser='" + user + "' and PortletApplicationEntityId='" + portAppEntId + "'");
        }
        
        return mappedUserAttributes;
    }

    /**
     * @see org.jasig.portal.portlet.userattributes.IMappedUserAttributeProvider#getMappedUserAttributes(org.jasig.portal.security.UserId, org.jasig.portal.portlet.definition.PortletApplicationDefinitionId)
     */
    public Map<String, String> getMappedUserAttributes(IUser user, PortletApplicationDefinitionId portAppDefId) {
        //Get the app def to get the user attributes and user attribute mappings from
        final IPortletApplicationDefinition portletApplicationDefinition = this.portletApplicationDefinitionRegistry.getPortletApplicationDefinition(portAppDefId);
        final Set<IUserAttribute> userAttributes = portletApplicationDefinition.getUserAttributes();
        final Map<String, String> userAttributeMappings = portletApplicationDefinition.getUserAttributeMappings();
        
        //Get the mapped attributes
        final Map<String, String> mappedUserAttributes = this.getMappedUserAttributes(user, userAttributes, userAttributeMappings);
        
        if (this.logger.isInfoEnabled()) {
            final String mapInfo;
            if (mappedUserAttributes == null) {
                mapInfo = "null";
            }
            else {
                mapInfo = Integer.toString(mappedUserAttributes.size());
            }
            
            this.logger.info("Returning " + mapInfo + " mapped user attributes for IUser='" + user + "' and PortletApplicationDefinitionId='" + portAppDefId + "'");
        }
        
        return mappedUserAttributes;
    }
    
    /**
     * Gets the a user attribute map that compiles with the Map returned by {@link PortletRequest#USER_INFO}.
     * User attribute name mappings are honored to convert the map from portal centric attribute names to
     * portlet centric attribute names.
     * 
     * @param user The user to get attributes for
     * @param userAttributes The user attributes the portlet expectes
     * @param userAttributeMappings The mapping from portlet user attribute names to portal user attribute names
     * @return a Map of names to values for the user. The names will have been mapped if possible. Will only be null if IPersonAttributeDao returns null
     */
    @SuppressWarnings("unchecked")
    protected Map<String, String> getMappedUserAttributes(IUser user, Set<IUserAttribute> userAttributes, Map<String, String> userAttributeMappings) {
        final String userName = user.getUserName();
        final Map<String, Object> userAttributeValues = this.personAttributeDao.getUserAttributes(userName);
        if (userAttributeValues == null) {
            if (this.logger.isInfoEnabled()) {
                this.logger.debug("IPersonAttributeDao returned null for user='" + user + "'.");
            }

            return null;
        }
        
        final Map<String, String> mappedUserAttributes = new HashMap<String, String>(); 
        for (final IUserAttribute userAttribute : userAttributes) {
            final String portletAttributeName = userAttribute.getName();
            final String portalAttributeName = userAttributeMappings.get(portletAttributeName);
            
            //Try the portal mapped attribute name first if it exists
            final boolean containsAttribute;
            final Object attributeValue;
            if (portalAttributeName != null) {
                attributeValue = userAttributeValues.get(portalAttributeName);
                containsAttribute = attributeValue != null || userAttributeValues.containsKey(portalAttributeName);
                
                if (this.logger.isDebugEnabled()) {
                    if (containsAttribute) {
                        this.logger.debug("Mapping portal attribute '" + portalAttributeName + "' to portlet attribute '" + portletAttributeName + "'");
                    }
                    else {
                        this.logger.debug("The user did not have requested mapped portal attribute '" + portalAttributeName + "' for the portlet attribute '" + portletAttributeName + "' it will not be included");
                    }
                }
            }
            else {
                attributeValue = userAttributeValues.get(portletAttributeName);
                containsAttribute = attributeValue != null || userAttributeValues.containsKey(portletAttributeName);
                
                if (this.logger.isDebugEnabled()) {
                    if (containsAttribute) {
                        this.logger.debug("Getting the un-mapped portlet attribute '" + portletAttributeName + "'");
                    }
                    else {
                        this.logger.debug("The user did not have requested un-mapped portlet attribute '" + portletAttributeName + "' it will not be included");
                    }
                }
            }
            
            final String stringValue = getAttributeStringValue(attributeValue);
            if (containsAttribute) {
                mappedUserAttributes.put(portletAttributeName, stringValue);
            }
        }
        
        return mappedUserAttributes;
    }

    /**
     * Handling code for IPersonAttributeDao values. Does its best to get a single String
     * attribute values. If the attributeValue is a String it is returned. If it is null,
     * null is returned. If it is a List and the first item in the List is a String that
     * String is returned. Otherwise it returns null.
     * 
     * @param attributeValue The generic Object to get a String for
     * @return a String for the value, may be null.
     */
    protected String getAttributeStringValue(final Object attributeValue) {
        if (attributeValue == null) {
            return null;
        }
        else if (attributeValue instanceof String) {
            return (String)attributeValue;
        }
        else if (attributeValue instanceof List) {
            final List attributeValueList = ((List)attributeValue);
            
            if (attributeValueList.size() > 0) {
                final Object firstValue = attributeValueList.get(0);
                if (firstValue instanceof String) {
                    return (String)firstValue;
                }
            }
        }
        
        return null;
    }
}
