package org.jasig.portal.portlets.cachemanager.beans;

import java.util.Collection;

import org.jasig.portal.caching.ICacheStatistics;
import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.caching.StatisticsCacheWrapper;
import org.jasig.portal.caching.StatisticsCacheWrapper.CacheQueryStatistics;

/**
 * Utility bean for wrapping ICache object that provide differing levels
 * of information.
 */
public class CacheInfoBean {
    private final String cacheName;
    private final ISimpleCacheProviderFacade targetCache;
    private final ICacheStatistics targetStatsCache;
    private final StatisticsCacheWrapper targetStatsCacheWrapper;
    
    public CacheInfoBean(String cacheName, ISimpleCacheProviderFacade targetCache) {
        this.cacheName = cacheName;
        this.targetCache = targetCache;
        
        if (this.targetCache instanceof ICacheStatistics) {
            this.targetStatsCache = (ICacheStatistics)this.targetCache;
        }
        else {
            this.targetStatsCache = null;
        }
        
        if (this.targetCache instanceof StatisticsCacheWrapper) {
            this.targetStatsCacheWrapper = (StatisticsCacheWrapper)this.targetCache;
        }
        else {
            this.targetStatsCacheWrapper = null;
        }
    }
    
    public String getName() {
        return this.cacheName;
    }
    
    public int getSize() {
        if (this.targetStatsCache != null) {
            return this.targetStatsCache.getSize();
        }
        else {
            return -1;
        }
    }
    
    public long getHits() {
        if (this.targetStatsCache != null) {
            return this.targetStatsCache.getCacheHits();
        }
        else {
            return -1l;
        }
    }
    
    public long getMisses() {
        if (this.targetStatsCache != null) {
            return this.targetStatsCache.getCacheMisses();
        }
        else {
            return -1l;
        }
    }
    
    public long getExpirations() {
        if (this.targetStatsCache != null) {
            return this.targetStatsCache.getCacheExpirations();
        }
        else {
            return -1l;
        }
    }
    
    public double getHitRatio() {
        final long queries = this.getQueryCount();
        final long hits = this.getHits();
        
        if (queries > 0 && hits >= 0) {
            return ((double)hits) / ((double)queries);
        }
        else {
            return -1d;
        }
    }
    
    public double getMissRatio() {
        final long queries = this.getQueryCount();
        final long misses = this.getMisses();

        if (queries > 0 && misses >= 0) {
            return ((double)misses) / ((double)queries);
        }
        else {
            return -1d;
        }
    }
    
    public double getExpirationMissRatio() {
        final long misses = this.getMisses();
        final long expirations = this.getExpirations();

        if (misses > 0 && expirations >= 0) {
            return ((double)expirations) / ((double)misses);
        }
        else {
            return -1d;
        }
    }
    
    public long getQueryCount() {
        final long hits = this.getHits();
        final long misses = this.getMisses();
        
        if (hits >= 0 || misses >= 0) {
            return Math.max(0, hits) + Math.max(0, misses);
        }
        else {
            return -1l;
        }
    }
    
    public Collection<CacheQueryStatistics> getQueryStatistics() {
        if (this.targetStatsCacheWrapper != null) {
            return this.targetStatsCacheWrapper.getCacheQueryStatistics();
        }
        else {
            return null;
        }
    }
}