/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.cachemanager.web;

import java.util.Comparator;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.Transformer;
import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.portlets.cachemanager.beans.CacheInfoBean;
import org.jasig.portal.spring.TypeAwareBeanRegistry;
import org.springframework.web.portlet.ModelAndView;
import org.springframework.web.portlet.mvc.AbstractController;

/**
 * Controller that handles only render requests. The controller populates the {@link java.util.Map}
 * model with a {@link java.util.SortedSet} of {@link org.jasig.portal.portlets.cachemanager.beans.CacheInfoBean}
 * objects. Each CacheInfoBean wraps a {@link org.jasig.portal.caching.ISimpleCacheProviderFacade}. The 
 * {@link #VIEW_NAME} view is returned.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class CachesController extends AbstractController {
    private static final Comparator<CacheInfoBean> CACHE_INFO_COMPARATOR = new Comparator<CacheInfoBean>() {
        
        /**
         * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
         */
        public int compare(CacheInfoBean o1, CacheInfoBean o2) {
            return o1.getName().compareTo(o2.getName());
        }
    };
    
    
    public static final String MODEL_NAME = "caches";
    public static final String VIEW_NAME = "caches";
    
    private TypeAwareBeanRegistry<ISimpleCacheProviderFacade> typeAwareBeanRegistry;
    
    /**
     * @return the typeAwareBeanRegistry
     */
    public TypeAwareBeanRegistry<ISimpleCacheProviderFacade> getTypeAwareBeanRegistry() {
        return this.typeAwareBeanRegistry;
    }
    /**
     * @param typeAwareBeanRegistry the typeAwareBeanRegistry to set
     */
    public void setTypeAwareBeanRegistry(TypeAwareBeanRegistry<ISimpleCacheProviderFacade> typeAwareBeanRegistry) {
        this.typeAwareBeanRegistry = typeAwareBeanRegistry;
    }


    /**
     * Exposes a defensive copy of this controller's current ISimpleCacheProviderFacade collection,
     * with each object wrapped in a CacheInfoBean. The new collection comprises
     * the entire model and is keyed using the value of {@link #MODEL_NAME}.
     * 
     * @return a ModelAndView with a CacheInfoBean collection. 
     * @see org.springframework.web.portlet.mvc.AbstractController#handleRenderRequestInternal(javax.portlet.RenderRequest, javax.portlet.RenderResponse)
     */
    protected ModelAndView handleRenderRequestInternal(RenderRequest request, RenderResponse response) throws Exception {
        final Map<String, ISimpleCacheProviderFacade> caches = this.typeAwareBeanRegistry.getBeansOfType(ISimpleCacheProviderFacade.class);
    	final SortedSet<CacheInfoBean> sortedCaches = new TreeSet<CacheInfoBean>(CACHE_INFO_COMPARATOR);
    	
    	// expects an ISimpleCacheProviderFacade name as the object to transform. Returns
    	// a CacheInfoBean comprised of the name received and the
    	// ISimpleCacheProviderFacade retrieved by that name
    	Transformer wrappingTransformer = new Transformer() {
    		public Object transform(Object input) {
        		return new CacheInfoBean((String)input, caches.get(input));
        	}
    	};
    	
    	CollectionUtils.collect(caches.keySet(), wrappingTransformer, sortedCaches);
        return new ModelAndView(VIEW_NAME, MODEL_NAME, sortedCaches);
        
    }
}
