/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlets.channeladapter;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.Serializable;
import java.io.StringWriter;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.portlet.WindowState;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.jasig.portal.ChannelCacheKey;
import org.jasig.portal.ChannelRuntimeData;
import org.jasig.portal.ChannelStaticData;
import org.jasig.portal.ICacheable;
import org.jasig.portal.IChannel;
import org.jasig.portal.PortalException;
import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.xml.sax.ContentHandler;

/**
 * Channel adapter portlet 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class ChannelAdapterPortletBean extends GenericPortlet {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(ChannelAdapterPortletBean.class);

    protected final static String CHANNEL_INSTANCE_SESSION_ATTRIBUTE_NAME="channel instance";
    protected final static String CHANNEL_INSTANCE_ID_ATTRIBUTE_NAME="uP3_channelId";
    protected final static String CHANNEL_ID_SEPARATOR=".";
    protected final static String PORTLET_KEYWORDS_RESOURCE_STRING="javax.portlet.keywords";
    
    
    // standard portlet preference names
    
    /**
     * Prefix of adapter-specific preference names 
     */
    protected final static String ADAPTER_PREFIX="uP3_";
    protected final static String CHANNEL_CLASSNAME_PREFERENCE_NAME="uP3_channelClassName";
    protected final static String RUNTIME_DATA_SESSION_ATTRIBUTE_NAME="uP3_runtimeData";
    
    
    
    // injected properties
    protected ISimpleCacheProviderFacade channelSystemCache;
    protected ISimpleCacheProviderFacade channelInstanceCache;
    protected OutputFormat outputFormat;
    
    
    private static long uid=0;
    
    public void processAction(ActionRequest req, ActionResponse res) throws PortletException, IOException {
        // construct runtime data and leave it in the session
        ChannelRuntimeData runtimeData=getRuntimeData(req);
        PortletSession session=req.getPortletSession();
        session.setAttribute(RUNTIME_DATA_SESSION_ATTRIBUTE_NAME,runtimeData);
    }

    public void render(RenderRequest req, RenderResponse res) throws PortletException, IOException {
        XMLSerializer serializer=new XMLSerializer(outputFormat);
        
        // TODO: add support for content type selection
        res.setContentType(req.getResponseContentType());
        
        try {
            IChannel channel=getChannelInstance(req);
            // see if the runtime data needs to be set again
            PortletSession session=req.getPortletSession();
            ChannelRuntimeData runtimeData=(ChannelRuntimeData) session.getAttribute(RUNTIME_DATA_SESSION_ATTRIBUTE_NAME);
            if(runtimeData==null) {
                runtimeData=getRuntimeData(req);
            } else {
                session.removeAttribute(RUNTIME_DATA_SESSION_ATTRIBUTE_NAME);
            }
            runtimeData.setBaseActionURL(res.createActionURL().toString());
            channel.setRuntimeData(runtimeData);
            
            boolean rendered=false;            
            // check if cache should be replayed
            if(channel instanceof ICacheable) {
                PortletPreferences preferences=req.getPreferences();
                String channelClassName=preferences.getValue(CHANNEL_CLASSNAME_PREFERENCE_NAME,"");
                
                // check if cache is available
                ChannelCacheKey key=((ICacheable)channel).generateKey();
                if(key!=null) {
                    ChannelCacheEntry entry=null;
                    String channelId=(String) session.getAttribute(CHANNEL_INSTANCE_ID_ATTRIBUTE_NAME);
                    String channelInstanceCacheKey=null;
                    if(key.getKeyScope()==ChannelCacheKey.SYSTEM_KEY_SCOPE) {
                        entry=(ChannelCacheEntry)channelSystemCache.getFromCache(key.getKey());
                    } else {
                        channelInstanceCacheKey=session.getAttribute(CHANNEL_INSTANCE_ID_ATTRIBUTE_NAME)+CHANNEL_ID_SEPARATOR+key.getKey();
                        entry=(ChannelCacheEntry)channelInstanceCache.getFromCache(channelInstanceCacheKey);
                    }
                    
                    // check cache validity
                    if(entry!=null && ((ICacheable)channel).isCacheValid(entry.validity) && (entry.buffer!=null)) {
                        // use cache
                        PrintWriter responseWriter=res.getWriter();
                        responseWriter.write(entry.buffer);
                        rendered=true;
                        if (logger.isDebugEnabled()) {
                            logger.debug("render(RenderRequest, RenderResponse) - replayed valid channel cache : channelClassName=" + channelClassName + ", channelId=" + channelId+", key="+key.getKey().toString());
                        }
                    } else {
                        // render and record cache
                        StringWriter sw=new StringWriter();
                        try {
                            serializer.setOutputCharStream(sw);
                            ContentHandler serializerHandler=serializer.asContentHandler();
                            channel.renderXML(serializerHandler);
                            sw.flush();
                            // record cache
                            try {
                                entry=new ChannelCacheEntry(sw.toString(),(Serializable) key.getKeyValidity());
                                if(key.getKeyScope()==ChannelCacheKey.SYSTEM_KEY_SCOPE) {
                                    channelSystemCache.putInCache(key.getKey(),entry);
                                    if (logger.isDebugEnabled()) {
                                        logger.debug("render(RenderRequest, RenderResponse) - recorded system-scoped channel cache : channelClassName=" + channelClassName + ", channelId=" + channelId+", key="+key.getKey().toString());
                                    }
                                } else {
                                    channelInstanceCacheKey=channelId+CHANNEL_ID_SEPARATOR+key.getKey();
                                    channelInstanceCache.putInCache(channelInstanceCacheKey,entry);
                                    if (logger.isDebugEnabled()) {
                                        logger.debug("render(RenderRequest, RenderResponse) - recorded instance-scoped channel cache : channelClassName=" + channelClassName + ", channelId=" + channelId+", key="+key.getKey().toString());
                                    }
                                }
                            } catch (ClassCastException cce) {
                                // report error
                                logger.warn("render(RenderRequest, RenderResponse) - unable to cast key validity object to Serializable interface, cache not recorded. ChannelAdapter supports caching only with Serializable validity objects. channel classname="+channelClassName +", channel uid="+channelId+", key="+key.getKey().toString(), cce);
                            }
                            PrintWriter responseWriter=res.getWriter();
                            responseWriter.write(sw.toString());
                            rendered=true;
                        } catch (PortalException e) {
                            throw new PortletException("channel failed to render",e);
                        }
                    }
                }
            } 
            if(!rendered) {
                // proceed with regular rendering
                try {
                    serializer.setOutputCharStream(res.getWriter());
                    ContentHandler serializerHandler=serializer.asContentHandler();
                    channel.renderXML(serializerHandler);
                } catch (PortalException e) {
                    throw new PortletException("channel failed to render",e);
                }
            }
        } catch (PortalException e) {
            throw new PortletException("channel failed to accept runtime data",e);
        }
    }
    
    /**
     * Acquire channel instance from the session, or create a new one if necessary
     * @return instance of a configured channel
     * @throws PortalException 
     */
    protected IChannel getChannelInstance(PortletRequest req) throws PortalException {
        PortletSession session=req.getPortletSession();
        IChannel channel=(IChannel) session.getAttribute(CHANNEL_INSTANCE_SESSION_ATTRIBUTE_NAME);
        if(channel==null) {
            // create new channel instance
            PortletPreferences preferences=req.getPreferences();
            String channelClassName=preferences.getValue(CHANNEL_CLASSNAME_PREFERENCE_NAME,null);
            if(channelClassName==null) {
                throw new IllegalStateException("PortletPreference channelClassName can not be null");
            }
            String channelId=Long.toString(++uid);
            channel=ChannelFactory.instantiateChannel(channelClassName,channelId);
            
            // set static data
            ChannelStaticData staticData=getStaticData(req);
            channel.setStaticData(staticData);
            session.setAttribute(CHANNEL_INSTANCE_ID_ATTRIBUTE_NAME,channelId);
            session.setAttribute(CHANNEL_INSTANCE_SESSION_ATTRIBUTE_NAME,channel);
        }
        return channel;
    }
    
    /**
     * Create runtime data for a given request
     * @param req current request
     * @return channel runtime data for the current request
     */
    protected ChannelRuntimeData getGeneralRuntimeData(PortletRequest req) {
        Locale locale=req.getLocale();
        
        // select out parameters from the request
        PortletAdapterChannelRuntimeData rd;
        rd=new PortletAdapterChannelRuntimeData(req.getParameterMap());

        // set browser info
        rd.setBrowserInfo(new PortletBrowserInfo(req));
        // set is rendering as root
        rd.setRenderingAsRoot(WindowState.MAXIMIZED.equals(req.getWindowState()));
        // set keywords
        ResourceBundle rb=getPortletConfig().getResourceBundle(locale);
        if(rb!=null) {
            rd.setKeywords(rb.getString(PORTLET_KEYWORDS_RESOURCE_STRING));
        }
        //TODO:  set remote address
        //TODO: set http method
        
        // set locales
        rd.setLocales(new Locale[]{locale});
        
        return rd;
    }
    
    protected ChannelRuntimeData getRuntimeData(RenderRequest req) {
        return getGeneralRuntimeData(req);
    }
    
    protected ChannelRuntimeData getRuntimeData(ActionRequest req) {
        ChannelRuntimeData rd=getGeneralRuntimeData(req);
        rd.setTargeted(true);
        return rd;
    }
    
    
    
    /**
     * Create static data
     * @param req portlet request
     * @return channel runtime data
     */
    protected ChannelStaticData getStaticData(PortletRequest req) {
        // retrieve channel subscribe parameters from preferences
        PortletPreferences prefrences=req.getPreferences();
        Map channelParameters=new HashMap();
        for(Enumeration e=prefrences.getNames();e.hasMoreElements();) {
            String parameterName=(String) e.nextElement();
            if(!parameterName.startsWith(ADAPTER_PREFIX)) {
                channelParameters.put(parameterName,prefrences.getValue(parameterName,null));
            }
        }
        ChannelStaticData staticData=new org.jasig.portal.portlets.channeladapter.ChannelStaticData(channelParameters);
        return staticData;
    }

    public void setOutputFormat(OutputFormat outputFormat) {
        this.outputFormat = outputFormat;
    }

    /**
     * Specify cache in which channel system-scoped entries should be kept
     * @param channelCache
     */
    public void setChannelSystemCache(ISimpleCacheProviderFacade channelCache) {
        this.channelSystemCache = channelCache;
    }

    /**
     * Specify cache in which channel instance-scoped entries should be kept
     * @param channelInstanceCache
     */
    public void setChannelInstanceCache(ISimpleCacheProviderFacade channelInstanceCache) {
        this.channelInstanceCache = channelInstanceCache;
    }
    
    /**
     * A combination of cached content (buffer) and validity object. The validity object is assumed to
     * be serializable (which is not the case in uPortal2).
     */
    protected static class ChannelCacheEntry implements Serializable {
        static final long serialVersionUID=-1;
        protected String buffer;
        protected final Serializable validity;
        public ChannelCacheEntry() {
            buffer = null;
            validity = null;
        }
        public ChannelCacheEntry(String buffer, Serializable validity) {
            this.buffer = buffer;
            this.validity = validity;
        }
    }
}
