/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.error.bean;

import java.util.Map;
import java.util.Vector;

import org.jasig.portal.portlet.rendering.PortletRenderingException;
import org.jasig.portal.portlet.rendering.PortletRenderingTimeoutException;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.utils.ResourceNotFoundException;

/**
 * This is a simple data structure for the ErrorPortlet and
 * ErrorAction classes.  There is no behavior in any methods.
 *
 * @author Cris J Holdorph, holdorph@unicon.net
 * @version $Revision: 11955 $
 */
public class ErrorData
{
	private PortletWindowId windowId=null;
	/**
	 * High-level message on what has happened
	 */
	private String message=null;
	/**
	 * A constant describing error type
	 */
	private ErrorType errorType=null;
	/**
	 * Description of a missing resource (if there is one)
	 */
	private String resourceDescription=null;
	/**
	 * URI of a missing resource (if there is one)
	 */
	private String resourceUri=null;
	/**
	 * A detailed description of a problem. Obtained from problem message by default.
	 */
	private String detailedDescription=null;
	/**
	 * A <code>Throwable</code> providing the details of the problem
	 */
	private Throwable problem=null;
    /**
     * Title of the failed portlet
     */
    private String windowTitle=null;
    
    /**
     * View flag for showing stack trace
     */
    private boolean showStackTrace=false;
    
    /**
     * Portlet rendering phase at which the error occurred.
     */
    private ErrorPhase errorPhase=null;
    
    /**
     * A map of name to String[] parameter values. 
     * Contains render or action parameters, depending on the error phase.
     */
    private Map parameterMap;
    
    /**
     * Attribute name under which error data is stored in the portlet session.
     * Note: this is used externally to initialize error data for new error portlet
     * window instances.
     */
    public static final String ERROR_DATA_SESSION_ATTRIBUTE = "data";
    
    /**
     * @return Returns the showStackTrace.
     */
    public boolean isShowStackTrace() {
        return showStackTrace;
    }
    /**
     * @param showStackTrace The showStackTrace to set.
     */
    public void setShowStackTrace(boolean showStackTrace) {
        this.showStackTrace = showStackTrace;
    }
    // below are error types that the error portlet should be able to handle (i.e. tailor rendering to the problem type)
    // NOTE: detailed information should be included in the exception
    
    /**
     * Portlet timed out
     */
    public static final ErrorType PORTLET_TIMEOUT=new ErrorType("timeout");
    /**
     * General rendering problem (i.e. exception raised)
     */
    public static final ErrorType RENDERING_EXCEPTION=new ErrorType("general rendering problem");
    /**
     * A specific resource was missing
     */
    public static final ErrorType RESOURCE_MISSING=new ErrorType("resource missing");
    /**
     * An portlet window with a specified id could not be found
     */
    public static final ErrorType INVALID_PORTLET_WINDOW=new ErrorType("invalid portlet window");
    /**
     * Authorziation problem
     */
    public static final ErrorType AUTHORIZATION_EXCEPTION=new ErrorType("not authorized to view the portlet");
    
    
    
    /**
     * A constructor for an error data in a basic case
     * 
     * @param windowId
     * @param message
     * @param errorType
     * @param problem
     */
    public ErrorData(PortletWindowId windowId, String message, ErrorType errorType, Throwable problem) {
        super();
        this.windowId = windowId;
        this.message = message;
        this.errorType = errorType;
        this.problem = problem;
    }
    
    /**
     * Tries to reconstruct error type and message from the throwable.
     * 
     * @param windowId
     * @param problem
     */
    public ErrorData(PortletWindowId windowId, Throwable problem) {
        super();
        this.windowId=windowId;
        this.problem=problem;
        this.errorType=RENDERING_EXCEPTION;
        this.message=problem.getMessage();
        if(problem instanceof PortletRenderingException) {
            if(problem instanceof PortletRenderingTimeoutException) {
                this.errorType=PORTLET_TIMEOUT;
            }
        } else if(problem instanceof ResourceNotFoundException) {
            ResourceNotFoundException rnfe=(ResourceNotFoundException) problem;
            this.errorType=RESOURCE_MISSING;
            this.resourceDescription=rnfe.getResourceName();
        }
    }
    
    
    /**
     * A constructor for an error data in a case of a missing resource
     * 
     * @param windowId
     * @param message
     * @param errorType
     * @param resourceDescription
     * @param resourceUri
     * @param problem
     */
    public ErrorData(PortletWindowId windowId, String message, ErrorType errorType, String resourceDescription, String resourceUri, Throwable problem) {
        super();
        this.windowId = windowId;
        this.message = message;
        this.errorType = errorType;
        this.resourceDescription = resourceDescription;
        this.resourceUri = resourceUri;
        this.problem = problem;
    }
    public static class ErrorType {
        final String typeName;
        public ErrorType(String typeName) {
            this.typeName=typeName;
        }
        public String toString() {
            return this.typeName;
        }
    }

    /**
     * @return Returns the detailedDescription.
     */
    public String getDetailedDescription() {
        if(detailedDescription==null) {
            if(problem!=null) {
                return problem.getMessage();
            } else {
                return null;
            }
        } else {
            return detailedDescription;
        }
    }
    /**
     * @param detailedDescription The detailedDescription to set.
     */
    public void setDetailedDescription(String detailedDescription) {
        this.detailedDescription = detailedDescription;
    }
    /**
     * @return Returns the errorType.
     */
    public ErrorType getErrorType() {
        return errorType;
    }
    /**
     * @param errorType The errorType to set.
     */
    public void setErrorType(ErrorType errorType) {
        this.errorType = errorType;
    }
    /**
     * @return Returns the message.
     */
    public String getMessage() {
        return message;
    }
    /**
     * @param message The message to set.
     */
    public void setMessage(String message) {
        this.message = message;
    }
    /**
     * @return Returns the problem.
     */
    public Throwable getProblem() {
        return problem;
    }
    
    /**
     * Obtain a causal chain of problems as an array
     */
    public Throwable[] getProblems() {
        Vector throwables=new Vector();
        Throwable cause=problem;
        while(cause!=null) {
            throwables.add(cause);
            cause=cause.getCause();
        }
        return (Throwable[]) throwables.toArray(new Throwable[throwables.size()]);
    }
    
    /**
     * @param problem The problem to set.
     */
    public void setProblem(Throwable problem) {
        this.problem = problem;
    }
    /**
     * @return Returns the resourceDescription.
     */
    public String getResourceDescription() {
        return resourceDescription;
    }
    /**
     * @param resourceDescription The resourceDescription to set.
     */
    public void setResourceDescription(String resourceDescription) {
        this.resourceDescription = resourceDescription;
    }
    /**
     * @return Returns the resourceUri.
     */
    public String getResourceUri() {
        return resourceUri;
    }
    /**
     * @param resourceUri The resourceUri to set.
     */
    public void setResourceUri(String resourceUri) {
        this.resourceUri = resourceUri;
    }
    /**
     * @return Returns the windowId.
     */
    public PortletWindowId getWindowId() {
        return windowId;
    }
    /**
     * @param windowId The windowId to set.
     */
    public void setWindowId(PortletWindowId windowId) {
        this.windowId = windowId;
    }
    public ErrorPhase getErrorPhase() {
        return errorPhase;
    }
    public void setErrorPhase(ErrorPhase errorPhase) {
        this.errorPhase = errorPhase;
    }
    public Map getParameterMap() {
        return parameterMap;
    }
    public void setParameterMap(Map parameterMap) {
        this.parameterMap = parameterMap;
    }
    public String getWindowTitle() {
        return windowTitle;
    }
    public void setWindowTitle(String windowTitle) {
        this.windowTitle = windowTitle;
    }
}
