/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.manager.web.pages.application;

import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.lang.StringUtils;
import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.common.DescriptionImpl;
import org.jasig.portal.portlet.common.DisplayNameImpl;
import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinitionControl;
import org.jasig.portal.portlet.definition.LocalizedPortletApplicationDefinitionControlImpl;
import org.jasig.portal.portlet.deployment.IPortletApplicationDeployment;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.registry.general.IPortletApplicationDeploymentRegistry;
import org.jasig.portal.portlet.registry.transitory.ITransientPortletApplicationDefinitionRegistry;
import org.jasig.portal.portlets.utils.ConfigurationException;


/**
 * Displayes the General information wizard page for portlet application
 * definition mangement.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class GeneralWizardPage extends BasePortletApplicationWizardPage {
    private static final String APP_NAME = "APP_NAME";
    private static final String APP_DESC = "APP_DESC";
    private static final String BASE_APP_ID = "BASE_APP_ID";
    
    private IPortletApplicationDeploymentRegistry portletApplicationDeploymentRegistry;
    private ITransientPortletApplicationDefinitionRegistry transientPortletApplicationDefinitionRegistry;
    private ILocaleManager localeManager;
    
    
    /**
     * @return Returns the portletApplicationDeploymentRegistry.
     */
    public IPortletApplicationDeploymentRegistry getPortletApplicationDeploymentRegistry() {
        return this.portletApplicationDeploymentRegistry;
    }
    /**
     * @param portletApplicationDeploymentRegistry The portletApplicationDeploymentRegistry to set.
     */
    public void setPortletApplicationDeploymentRegistry(IPortletApplicationDeploymentRegistry portletApplicationDeploymentRegistry) {
        this.portletApplicationDeploymentRegistry = portletApplicationDeploymentRegistry;
    }
    
    /**
     * @return Returns the transientPortletApplicationDefinitionRegistry.
     */
    public ITransientPortletApplicationDefinitionRegistry getTransientPortletApplicationDefinitionRegistry() {
        return this.transientPortletApplicationDefinitionRegistry;
    }
    /**
     * @param transientPortletApplicationDefinitionRegistry The transientPortletApplicationDefinitionRegistry to set.
     */
    public void setTransientPortletApplicationDefinitionRegistry(ITransientPortletApplicationDefinitionRegistry transientPortletApplicationDefinitionRegistry) {
        this.transientPortletApplicationDefinitionRegistry = transientPortletApplicationDefinitionRegistry;
    }

    /**
     * @return Returns the localeManager.
     */
    public ILocaleManager getLocaleManager() {
        return this.localeManager;
    }
    /**
     * @param localeManager The localeManager to set.
     */
    public void setLocaleManager(ILocaleManager localeManager) {
        this.localeManager = localeManager;
    }
    
    public String getTitle() {
        return "General";
    }

    public void renderPage(RenderRequest request, RenderResponse response) throws IOException, PortletException {
        final Set portletAppDeployments = this.portletApplicationDeploymentRegistry.getPortletApplicationDeployments();
        request.setAttribute("PORT_APP_DEPS", portletAppDeployments);
        
        super.renderPage(request, response);
    }

    public void processResults(ActionRequest request, ActionResponse response) throws IOException, PortletException, ConfigurationException {
        this.validateRequest(request);

        final String portAppName = StringUtils.trimToNull(request.getParameter(APP_NAME));
        final String portAppDepIdStr = StringUtils.trimToNull(request.getParameter(BASE_APP_ID));
        final String portAppDesc = StringUtils.trimToNull(request.getParameter(APP_DESC));
        
        final PortletApplicationDeploymentId portAppDepId = new PortletApplicationDeploymentId(portAppDepIdStr);

        final PortletSession session = request.getPortletSession();
        IPortletApplicationDefinitionControl portAppDefControl = (IPortletApplicationDefinitionControl)session.getAttribute(PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF);
        
        if (session.getAttribute(PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF_ID) == null && 
                (portAppDefControl == null || !portAppDefControl.getPortletApplicationDeploymentId().equals(portAppDepId))) {

            final IPortletApplicationDeployment portAppDep = this.portletApplicationDeploymentRegistry.getPortletApplicationDeployment(portAppDepId);
            portAppDefControl = this.transientPortletApplicationDefinitionRegistry.createTransientPortletApplicationDefinitionControl(portAppDep.getPortletApplicationDeploymentId());
            session.setAttribute(PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF, new LocalizedPortletApplicationDefinitionControlImpl(portAppDefControl, this.localeManager));
        }
        
        final Map<Locale, IDisplayName> displayNames = portAppDefControl.getDisplayNames();
        final DisplayNameImpl newDisplayName = new DisplayNameImpl(portAppName, request.getLocale());
        displayNames.put(newDisplayName.getLocale(), newDisplayName);

        final Map<Locale, IDescription> descriptions =  portAppDefControl.getDescriptions();
        final DescriptionImpl newDescription = new DescriptionImpl(portAppDesc, request.getLocale());
        descriptions.put(newDescription.getLocale(), newDescription);
    }
    
    public Map isCompleted(ActionRequest request, ActionResponse response) throws IOException, PortletException {
        final Map<String, String> errors = new HashMap<String, String>();

        final PortletSession session = request.getPortletSession();
        final IPortletApplicationDefinitionControl portAppDefControl = (IPortletApplicationDefinitionControl)session.getAttribute(PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF);
        
        if (portAppDefControl != null) {
            if (portAppDefControl.getDisplayNames().size() == 0) {
                errors.put(APP_NAME, "No name specified for the PortletApplicationDefinition");
            }

            if (portAppDefControl.getPortletApplicationDeploymentId() == null) {
                errors.put(APP_NAME, "No deployment ID specified for the PortletApplicationDefinition");
            }
        }
        else {
            errors.put("", "No PortletApplicationDefinition in the session.");
        }
        
        return Collections.unmodifiableMap(errors);
    }
    
    /**
     * Validates the request parameters for this wizard page. Throwing a ConfigurationException for
     * an invalid request.
     * 
     * @param request The request to get the parameters to validate from.
     * @throws ConfigurationException If the parameter values are invalid.
     */
    private void validateRequest(ActionRequest request) throws ConfigurationException {
        final Map<String, String> dataErrors = new HashMap<String, String>();
        
        final String portAppName = StringUtils.trimToNull(request.getParameter(APP_NAME));
        final String portAppDepIdStr = StringUtils.trimToNull(request.getParameter(BASE_APP_ID));
        
        if (portAppName == null) {
            dataErrors.put(APP_NAME, "A 'Name' must be specified.");
        }
        if (portAppDepIdStr == null) {
            dataErrors.put(BASE_APP_ID, "A 'Portlet App' must be specified.");
        }
        else {
            final PortletSession session = request.getPortletSession();
            final IPortletApplicationDefinitionControl portAppDefControl = (IPortletApplicationDefinitionControl)session.getAttribute(PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF);
            final PortletApplicationDeploymentId portAppDepId = new PortletApplicationDeploymentId(portAppDepIdStr);
            
            if (portAppDefControl != null && 
                    session.getAttribute(PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF_ID) != null && 
                    !portAppDefControl.getPortletApplicationDeploymentId().equals(portAppDepId)) {
                
                dataErrors.put(BASE_APP_ID, "The base PortletApplicationDefinitionId cannot be changed when modifying a published application.");
            }
            else {
                final IPortletApplicationDeployment portAppDep = this.portletApplicationDeploymentRegistry.getPortletApplicationDeployment(portAppDepId);
                
                if (portAppDep == null) {
                    dataErrors.put(BASE_APP_ID, "Invalid 'Portlet App' specified (does not exist).");
                }
            }
        }

        if (dataErrors.size() > 0) {
            throw new ConfigurationException(dataErrors);
        }
    }
}
