/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.manager.web.pages.application;

import java.io.IOException;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.lang.StringUtils;
import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinitionControl;
import org.jasig.portal.portlet.definition.LocalizedPortletApplicationDefinitionControlImpl;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.deployment.IPortletApplicationDeployment;
import org.jasig.portal.portlet.registry.general.IPortletApplicationDefinitionRegistry;
import org.jasig.portal.portlet.registry.general.IPortletApplicationDeploymentRegistry;
import org.jasig.portal.portlet.registry.transitory.ITransientPortletApplicationDefinitionRegistry;
import org.jasig.portal.portlet.transfer.impl.TransferUtils;
import org.jasig.portal.portlets.utils.BaseWizard;


/**
 * Wizard implementation for PortletApplicationDefinitions. Provides the
 * action methods for starting/stopping the wizard and maintaining the
 * session.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class PortletApplicationDefinitionWizard extends BaseWizard {
    static final String PARAM_PORT_APP_DEF = "PORT_APP_DEF";
    static final String PARAM_PORT_APP_DEF_ID = "PORT_APP_DEF_ID";
    
    private static final String PARAM_WIZ_MODE = "WIZ_MODE";
    private static final String MODE_NEW    = "MODE_NEW";
    private static final String MODE_EDIT   = "MODE_EDIT";
    
    private ITransientPortletApplicationDefinitionRegistry transientPortletApplicationDefinitionRegistry;
    private IPortletApplicationDefinitionRegistry portletApplicationDefinitionRegistry;
    private IPortletApplicationDeploymentRegistry portletApplicationDeploymentRegistry;
    private ILocaleManager localeManager;
    
    
    /**
     * @return Returns the portletApplicationDefinitionRegistry.
     */
    public IPortletApplicationDefinitionRegistry getPortletApplicationDefinitionRegistry() {
        return this.portletApplicationDefinitionRegistry;
    }
    /**
     * @param portletApplicationDefinitionRegistry The portletApplicationDefinitionRegistry to set.
     */
    public void setPortletApplicationDefinitionRegistry(IPortletApplicationDefinitionRegistry portletApplicationDefinitionRegistry) {
        this.portletApplicationDefinitionRegistry = portletApplicationDefinitionRegistry;
    }

    /**
     * @return Returns the portletApplicationDeploymentRegistry.
     */
    public IPortletApplicationDeploymentRegistry getPortletApplicationDeploymentRegistry() {
        return this.portletApplicationDeploymentRegistry;
    }
    /**
     * @param portletApplicationDeploymentRegistry The portletApplicationDeploymentRegistry to set.
     */
    public void setPortletApplicationDeploymentRegistry(IPortletApplicationDeploymentRegistry portletApplicationDeploymentRegistry) {
        this.portletApplicationDeploymentRegistry = portletApplicationDeploymentRegistry;
    }
    
    /**
     * @return Returns the transientPortletApplicationDefinitionRegistry.
     */
    public ITransientPortletApplicationDefinitionRegistry getTransientPortletApplicationDefinitionRegistry() {
        return this.transientPortletApplicationDefinitionRegistry;
    }
    /**
     * @param transientPortletApplicationDefinitionRegistry The transientPortletApplicationDefinitionRegistry to set.
     */
    public void setTransientPortletApplicationDefinitionRegistry(ITransientPortletApplicationDefinitionRegistry transientPortletApplicationDefinitionRegistry) {
        this.transientPortletApplicationDefinitionRegistry = transientPortletApplicationDefinitionRegistry;
    }
    
    /**
     * @return Returns the localeManager.
     */
    public ILocaleManager getLocaleManager() {
        return this.localeManager;
    }
    /**
     * @param localeManager The localeManager to set.
     */
    public void setLocaleManager(ILocaleManager localeManager) {
        this.localeManager = localeManager;
    }
    
    
    public void renderPage(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        final PortletSession session = request.getPortletSession();
        final String mode = (String)session.getAttribute(PARAM_WIZ_MODE);
        if (mode != null) {
            request.setAttribute(PARAM_WIZ_MODE, mode);
        }
        
        super.renderPage(request, response);
    }
    
    public void newPortletApp(ActionRequest request, ActionResponse response) {
        this.clearSession(request);
        
        final PortletSession session = request.getPortletSession();
        session.setAttribute(PARAM_WIZ_MODE, MODE_NEW);
    }
    
    public void editPortletApp(ActionRequest request, ActionResponse response) {
        this.clearSession(request);
        
        final String portAppDefIdStr = StringUtils.trimToNull(request.getParameter(PARAM_PORT_APP_DEF_ID));
        
        if (portAppDefIdStr != null) {
            //Get the portlet application definition from the registry
            final PortletApplicationDefinitionId portAppDefId = new PortletApplicationDefinitionId(portAppDefIdStr);
            final IPortletApplicationDefinition portAppDef = this.portletApplicationDefinitionRegistry.getPortletApplicationDefinition(portAppDefId);
            
            if (portAppDef != null) {
                //Get the IPortletApplicationDeployment the app def is based on
                final IPortletApplicationDeployment portAppDep = this.portletApplicationDeploymentRegistry.getPortletApplicationDeployment(portAppDef.getPortletApplicationDeploymentId());
                
                //Use the temp registry to create a app def control for the deployment 
                final IPortletApplicationDefinitionControl portAppDefControl = this.transientPortletApplicationDefinitionRegistry.createTransientPortletApplicationDefinitionControl(portAppDep.getPortletApplicationDeploymentId());
                
                //Clone the data from the real app def to the control for in-memory manipulation
                TransferUtils.deepCopyPortletApplicationDefinitionTransferProperties(portAppDef.getPortletApplicationDefinitionTransfer(), portAppDefControl.getPortletApplicationDefinitionTransfer());
                
                this.transientPortletApplicationDefinitionRegistry.updatePortletApplicationDefinition(portAppDefControl);
               
                final PortletSession session = request.getPortletSession(true);
                //Set the portlet app def in the session
                session.setAttribute(PARAM_PORT_APP_DEF, new LocalizedPortletApplicationDefinitionControlImpl(portAppDefControl, this.localeManager));
                //Store the original app def id in the session
                session.setAttribute(PARAM_PORT_APP_DEF_ID, portAppDefId);
                
                session.setAttribute(PARAM_WIZ_MODE, MODE_EDIT);
            }
            else {
                throw new IllegalStateException("No PortletApplicationDefinition found for ID '" + portAppDef + "' specified by parameter '" + PARAM_PORT_APP_DEF_ID + "'");
            }
        }
        else {
            throw new IllegalArgumentException("'" + PARAM_PORT_APP_DEF_ID + "' may not be null");
        }
    }
    
    public void deletePortletApp(ActionRequest request, ActionResponse response) {
        final String portAppDefIdStr = StringUtils.trimToNull(request.getParameter(PARAM_PORT_APP_DEF_ID));
        
        if (portAppDefIdStr != null) {
            final PortletApplicationDefinitionId portAppDefId = new PortletApplicationDefinitionId(portAppDefIdStr);
            this.portletApplicationDefinitionRegistry.removePortletApplicationDefinition(portAppDefId);
        }
        else {
            throw new IllegalArgumentException("'" + PARAM_PORT_APP_DEF_ID + "' may not be null");
        }
    }
    
    public void submitPortletApp(ActionRequest request, ActionResponse response) {
        final PortletSession session = request.getPortletSession();
        final PortletApplicationDefinitionId portAppDefId = (PortletApplicationDefinitionId)session.getAttribute(PARAM_PORT_APP_DEF_ID);
        final IPortletApplicationDefinitionControl portAppDef = (IPortletApplicationDefinitionControl)session.getAttribute(PARAM_PORT_APP_DEF);
        
        if (portAppDef == null) {
            throw new IllegalStateException("No IPortletApplicationDefinitionControl found in session for key '" + PARAM_PORT_APP_DEF + "'");
        }

        this.transientPortletApplicationDefinitionRegistry.removePortletApplicationDefinition(portAppDef.getPortletApplicationDefinitionId());

        if (portAppDefId == null) {
            this.portletApplicationDefinitionRegistry.createPortletApplicationDefinition(portAppDef.getPortletApplicationDefinitionTransfer());
        }
        else {
            portAppDef.setPortletApplicationDefinitionId(portAppDefId);
            this.portletApplicationDefinitionRegistry.updatePortletApplicationDefinition(portAppDef);
        }
        
        session.removeAttribute(PARAM_PORT_APP_DEF_ID);
        session.removeAttribute(PARAM_PORT_APP_DEF);
        session.removeAttribute(PARAM_WIZ_MODE);
    }
    
    public void cancelPortletApp(ActionRequest request, ActionResponse response) {
        this.clearSession(request);
    }
    
    private void clearSession(PortletRequest request) {
        final PortletSession session = request.getPortletSession();

        //Ensure the session is clean and ready to go
        final IPortletApplicationDefinitionControl portAppDef = (IPortletApplicationDefinitionControl)session.getAttribute(PARAM_PORT_APP_DEF);
        if (portAppDef != null) {
            this.transientPortletApplicationDefinitionRegistry.removePortletApplicationDefinition(portAppDef.getPortletApplicationDefinitionId());
            session.removeAttribute(PARAM_PORT_APP_DEF);
        }
        
        //Make sure there is no base ID in the session, this signifies a new app
        session.removeAttribute(PARAM_PORT_APP_DEF_ID);
        
        //Clear the mode from the session
        session.removeAttribute(PARAM_WIZ_MODE);
    }
}
