/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.manager.web.pages.application;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortalContext;
import javax.portlet.PortletException;
import javax.portlet.PortletMode;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.jasig.portal.portlet.common.ICustomPortletMode;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinitionControl;
import org.jasig.portal.portlets.utils.ConfigurationException;
import org.springframework.beans.factory.annotation.Required;


/**
 * Displayes the Portlet Mode mapping wizard page for portlet application
 * definition mangement.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class PortletModesWizardPage extends BasePortletApplicationWizardPage {
    private static final String MAPPING_PREFIX = "MODE_MAPPING__";
    private static final List<PortletMode> SPEC_MODES;
    
    static {
        final List<PortletMode> specModesBuilder = new ArrayList<PortletMode>(3);
        specModesBuilder.add(PortletMode.VIEW);
        specModesBuilder.add(PortletMode.EDIT);
        specModesBuilder.add(PortletMode.HELP);
        SPEC_MODES = Collections.unmodifiableList(specModesBuilder);
    }
    
    private PortalContext portalContext;
    
    /**
     * @return the portalContext
     */
    public PortalContext getPortalContext() {
        return this.portalContext;
    }
    /**
     * @param portalContext the portalContext to set
     */
    @Required
    public void setPortalContext(PortalContext portalContext) {
        this.portalContext = portalContext;
    }

    public String getTitle() {
        return "Portlet Modes";
    }

    public void renderPage(RenderRequest request, RenderResponse response) throws IOException, PortletException {
        final PortletSession session = request.getPortletSession();
        final IPortletApplicationDefinitionControl portAppDefControl = (IPortletApplicationDefinitionControl)session.getAttribute(PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF);
        
        final Set<PortletMode> upSupportedModes = this.portalSupportedModes();
        
        final Set<PortletMode> mappableModes = new HashSet<PortletMode>(upSupportedModes);
        for (final ICustomPortletMode customMode : portAppDefControl.getCustomPortletModes()) {
            mappableModes.add(customMode.getPortletMode());
        }

        request.setAttribute("UP_MODES", upSupportedModes);
        request.setAttribute("SPEC_MODES", SPEC_MODES);
        request.setAttribute("MAPPABLE_MODES", mappableModes);
        
        super.renderPage(request, response);
    }

    public void processResults(ActionRequest request, ActionResponse response) throws IOException, PortletException, ConfigurationException {
        final PortletSession session = request.getPortletSession();
        final IPortletApplicationDefinitionControl portAppDefControl = (IPortletApplicationDefinitionControl)session.getAttribute(PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF);
        
        if (portAppDefControl == null) {
            throw new IllegalStateException("IPortletApplicationDefinitionControl cannot be found in session for key '" + PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF + "'");
        }
        
        final Map<PortletMode, PortletMode> portletModeMappings = new HashMap<PortletMode, PortletMode>();
        final Set<PortletMode> upSupportedModes = this.portalSupportedModes();
        
        for (final Enumeration mappingEnum = request.getParameterNames(); mappingEnum.hasMoreElements();) {
            final String paramName = (String)mappingEnum.nextElement();
            
            if (paramName.startsWith(MAPPING_PREFIX)) {
                final String upModeName = paramName.substring(MAPPING_PREFIX.length());
                final String customModeName = StringUtils.trimToNull(request.getParameter(paramName));
                
                final PortletMode upMode = new PortletMode(upModeName);
                
                if (!upSupportedModes.contains(upMode)) {
                    throw new ConfigurationException(paramName, "Unknown uPortal PortletMode specified '" + upMode + "'");
                }
                
                final PortletMode customMode;
                if (customModeName != null) {
                    customMode = new PortletMode(customModeName);
                }
                else {
                    customMode = null;
                }
                
                portletModeMappings.put(upMode, customMode);
            }
        }

        final Map<String, String> mappingErrors = this.checkPortletModeMappings(portletModeMappings);
        if (mappingErrors.size() > 0) {
            throw new ConfigurationException(mappingErrors);
        }

        portAppDefControl.setPortletModeMappings(portletModeMappings);
    }
    
    public Map isCompleted(ActionRequest request, ActionResponse response) throws IOException, PortletException {
        final Map<String, String> errors = new HashMap<String, String>();

        final PortletSession session = request.getPortletSession();
        final IPortletApplicationDefinitionControl portAppDefControl = (IPortletApplicationDefinitionControl)session.getAttribute(PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF);
        
        if (portAppDefControl != null) {
            final Map<PortletMode, PortletMode> portletModeMappings = portAppDefControl.getPortletModeMappings();

            if (portletModeMappings == null) {
                errors.put(MAPPING_PREFIX, "No PortletMode mappings specified for the PortletApplicationDefinition");
            }
            else {
                final Map<String, String> mappingErrors = this.checkPortletModeMappings(portletModeMappings);
                errors.putAll(mappingErrors);
            }
        }
        else {
            errors.put("", "No PortletApplicationDefinition in the session.");
        }
        
        return Collections.unmodifiableMap(errors);
    }
    
    protected Set<PortletMode> portalSupportedModes() {
        final Set<PortletMode> upSupportedModes = new HashSet<PortletMode>();
        final Enumeration upSupportedModesEnum = this.portalContext.getSupportedPortletModes();
        CollectionUtils.addAll(upSupportedModes, upSupportedModesEnum);
        return upSupportedModes;
    }
    
    private Map<String, String> checkPortletModeMappings(Map<PortletMode, PortletMode> portletModeMappings) {
        final Map<String, String> errors = new HashMap<String, String>();

        for (final PortletMode specMode : SPEC_MODES) { 
            if (portletModeMappings.get(specMode) == null) {
                errors.put(MAPPING_PREFIX + specMode, "PortletMode mappings must map PortletMode." + specMode + " to another PortletMode");
            }
        }
        
        return Collections.unmodifiableMap(errors);
    }
}
