/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.manager.web.pages.application;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortalContext;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.portlet.WindowState;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.jasig.portal.portlet.common.ICustomWindowState;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinitionControl;
import org.jasig.portal.portlets.utils.ConfigurationException;
import org.springframework.beans.factory.annotation.Required;


/**
 * Displayes the Window State mapping wizard page for portlet application
 * definition mangement.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class WindowStatesWizardPage extends BasePortletApplicationWizardPage {
    private static final String MAPPING_PREFIX = "STATE_MAPPING__";
    private static final List<WindowState> SPEC_STATES;
    
    static {
        final List<WindowState> specStatesBuilder = new ArrayList<WindowState>(3);
        specStatesBuilder.add(WindowState.NORMAL);
        specStatesBuilder.add(WindowState.MAXIMIZED);
        specStatesBuilder.add(WindowState.MINIMIZED);
        SPEC_STATES = Collections.unmodifiableList(specStatesBuilder);
    }
    
    private PortalContext portalContext;
    
    /**
     * @return the portalContext
     */
    public PortalContext getPortalContext() {
        return this.portalContext;
    }

    /**
     * @param portalContext the portalContext to set
     */
    @Required
    public void setPortalContext(PortalContext portalContext) {
        this.portalContext = portalContext;
    }

    public String getTitle() {
        return "Window States";
    }

    public void renderPage(RenderRequest request, RenderResponse response) throws IOException, PortletException {
        final PortletSession session = request.getPortletSession();
        final IPortletApplicationDefinitionControl portAppDefControl = (IPortletApplicationDefinitionControl)session.getAttribute(PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF);
        
        final Set<WindowState> upSupportedStates = this.portalSupportedStates();
        
        final Set<WindowState> mappableStates = new HashSet<WindowState>();
        for (final ICustomWindowState customState : portAppDefControl.getCustomWindowStates()) {
            mappableStates.add(customState.getWindowState());
        }
        
        request.setAttribute("UP_STATES", upSupportedStates);
        request.setAttribute("SPEC_STATES", SPEC_STATES);
        request.setAttribute("MAPPABLE_STATES", mappableStates);
        
        super.renderPage(request, response);
    }

    public void processResults(ActionRequest request, ActionResponse response) throws IOException, PortletException, ConfigurationException {
        final PortletSession session = request.getPortletSession();
        final IPortletApplicationDefinitionControl portAppDefControl = (IPortletApplicationDefinitionControl)session.getAttribute(PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF);
        
        if (portAppDefControl == null) {
            throw new IllegalStateException("IPortletApplicationDefinitionControl cannot be found in session for key '" + PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF + "'");
        }
        
        final Map<WindowState, WindowState> windowStateMappings = new HashMap<WindowState, WindowState>();
        final Set<WindowState> upSupportedStates = this.portalSupportedStates();
        
        for (final Enumeration mappingEnum = request.getParameterNames(); mappingEnum.hasMoreElements();) {
            final String paramName = (String)mappingEnum.nextElement();
            
            if (paramName.startsWith(MAPPING_PREFIX)) {
                final String upStateName = paramName.substring(MAPPING_PREFIX.length());
                final String customStateName = StringUtils.trimToNull(request.getParameter(paramName));
                
                final WindowState upState = new WindowState(upStateName);
                
                if (!upSupportedStates.contains(upState)) {
                    throw new ConfigurationException(paramName, "Unknown uPortal WindowState specified '" + upState + "'");
                }
                
                final WindowState customState;
                if (customStateName != null) {
                    customState = new WindowState(customStateName);
                }
                else {
                    customState = null;
                }
                
                windowStateMappings.put(upState, customState);
            }
        }

        final Map mappingErrors = this.checkWindowStateMappings(windowStateMappings);
        if (mappingErrors.size() > 0) {
            throw new ConfigurationException(mappingErrors);
        }
        
        portAppDefControl.setWindowStateMappings(windowStateMappings);
    }
    
    public Map isCompleted(ActionRequest request, ActionResponse response) throws IOException, PortletException {
        final Map<String, String> errors = new HashMap<String, String>();

        final PortletSession session = request.getPortletSession();
        final IPortletApplicationDefinitionControl portAppDefControl = (IPortletApplicationDefinitionControl)session.getAttribute(PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF);
        
        if (portAppDefControl != null) {
            final Map<WindowState, WindowState> windowStateMappings = portAppDefControl.getWindowStateMappings();

            if (windowStateMappings == null) {
                errors.put(MAPPING_PREFIX, "No WindowState mappings specified for the PortletApplicationDefinition");
            }
            else {
                final Map<String, String> mappingErrors = this.checkWindowStateMappings(windowStateMappings);
                errors.putAll(mappingErrors);
            }
        }
        else {
            errors.put("", "No PortletApplicationDefinition in the session.");
        }
        
        return Collections.unmodifiableMap(errors);
    }
    
    protected Set<WindowState> portalSupportedStates() {
        final Set<WindowState> upSupportedModes = new HashSet<WindowState>();
        final Enumeration upSupportedModesEnum = this.portalContext.getSupportedWindowStates();
        CollectionUtils.addAll(upSupportedModes, upSupportedModesEnum);
        return upSupportedModes;
    }
    
    private Map<String, String> checkWindowStateMappings(Map<WindowState, WindowState> windowStateMappings) {
        final Map<String, String> errors = new HashMap<String, String>();

        for (final WindowState specState : SPEC_STATES) {
            if (windowStateMappings.get(specState) == null) {
                errors.put(MAPPING_PREFIX + specState, "WindowState mappings must map WindowState." + specState + " to another WindowState");
            }
        }
        
        return Collections.unmodifiableMap(errors);
    }
}
