/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.manager.web.pages.portlet;

import java.io.IOException;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortalContext;
import javax.portlet.PortletException;
import javax.portlet.PortletMode;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.portlet.WindowState;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.math.NumberUtils;
import org.jasig.portal.portlet.common.IPortletInfo;
import org.jasig.portal.portlet.definition.ILocalizedPortletDefinitionControl;
import org.jasig.portal.portlets.utils.ConfigurationException;
import org.springframework.beans.factory.annotation.Required;


/**
 * Displayes the Display information wizard page for portlet
 * definition mangement.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class DisplayWizardPage extends BasePortletWizardPage {
    private static final String TITLE = "TITLE";
    private static final String SHORT_TITLE = "SHORT_TITLE";
    private static final String RENDER_TIME_OUT = "RENDER_TIME_OUT";
    private static final String MODE_BUTTON_PRFX = "MODE_BUTTON__";
    private static final String STATE_BUTTON_PRFX = "STATE_BUTTON__";

    
    private PortalContext portalContext;
    
    /**
     * @return the portalContext
     */
    public PortalContext getPortalContext() {
        return this.portalContext;
    }

    /**
     * @param portalContext the portalContext to set
     */
    @Required
    public void setPortalContext(PortalContext portalContext) {
        this.portalContext = portalContext;
    }


    public String getTitle() {
        return "Display";
    }

    public void renderPage(RenderRequest request, RenderResponse response) throws IOException, PortletException {
        //TODO pass in selected mode/state buttons here
        
        final Set<PortletMode> upSupportedModes = this.portalSupportedModes();
        request.setAttribute("PORTAL_MODES", upSupportedModes);
        
        final Set<WindowState> upSupportedStates = this.portalSupportedStates();
        request.setAttribute("PORTAL_STATES", upSupportedStates);        
        
        super.renderPage(request, response);
    }

    public void processResults(ActionRequest request, ActionResponse response) throws IOException, PortletException, ConfigurationException {
        final PortletSession session = request.getPortletSession();
        final ILocalizedPortletDefinitionControl localPortDef = (ILocalizedPortletDefinitionControl)session.getAttribute(PortletDefinitionWizard.PARAM_PORT_DEF);
        
        if (localPortDef == null) {
            throw new IllegalStateException("ILocalizedPortletDefinitionControl cannot be found in session for key '" + PortletDefinitionWizard.PARAM_PORT_DEF + "'");
        }
        
        this.validateRequest(request);

        final String title = StringUtils.trimToNull(request.getParameter(TITLE));
        final String shortTitle = StringUtils.trimToNull(request.getParameter(SHORT_TITLE));
        final String renderTimeOut = StringUtils.trimToNull(request.getParameter(RENDER_TIME_OUT));
        
        final IPortletInfo portletInfo = localPortDef.getPortletInfo();
        portletInfo.setTitle(title);
        portletInfo.setShortTitle(shortTitle);
        
        localPortDef.setRenderTimeOut(NumberUtils.toLong(renderTimeOut, -1));
        
        for (final Enumeration paramNameEnum = request.getParameterNames(); paramNameEnum.hasMoreElements();) {
            final String paramName = (String)paramNameEnum.nextElement();
            
            if (paramName.startsWith(STATE_BUTTON_PRFX)) {
                final String stateName = paramName.substring(STATE_BUTTON_PRFX.length());
                final String value = StringUtils.trimToNull(request.getParameter(paramName));
                
                System.out.println(stateName + " -> " + value);
            }
            else if (paramName.startsWith(MODE_BUTTON_PRFX)) {
                final String modeName = paramName.substring(MODE_BUTTON_PRFX.length());
                final String value = StringUtils.trimToNull(request.getParameter(paramName));
                
                System.out.println(modeName + " -> " + value);
            }
        }
    }
    
    public Map isCompleted(ActionRequest request, ActionResponse response) throws IOException, PortletException {
        final Map<String, String> errors = new HashMap<String, String>();
        
        final PortletSession session = request.getPortletSession();
        final ILocalizedPortletDefinitionControl localPortDef = (ILocalizedPortletDefinitionControl)session.getAttribute(PortletDefinitionWizard.PARAM_PORT_DEF);
        
        if (localPortDef != null) {
            if (localPortDef.getPortletInfo().getTitle() == null) {
                errors.put(TITLE, "No title specified for the PortletDefinition");
            }
        }
        else {
            errors.put("", "No PortletDefinition in the session.");
        }
        
        return Collections.unmodifiableMap(errors);
    }
    
    protected Set<WindowState> portalSupportedStates() {
        final Set<WindowState> upSupportedModes = new HashSet<WindowState>();
        final Enumeration upSupportedModesEnum = this.portalContext.getSupportedWindowStates();
        CollectionUtils.addAll(upSupportedModes, upSupportedModesEnum);
        return upSupportedModes;
    }
    
    protected Set<PortletMode> portalSupportedModes() {
        final Set<PortletMode> upSupportedModes = new HashSet<PortletMode>();
        final Enumeration upSupportedModesEnum = this.portalContext.getSupportedPortletModes();
        CollectionUtils.addAll(upSupportedModes, upSupportedModesEnum);
        return upSupportedModes;
    }

    /**
     * Validates the request parameters for this wizard page. Throwing a ConfigurationException for
     * an invalid request.
     * 
     * @param request The request to get the parameters to validate from.
     * @throws ConfigurationException If the parameter values are invalid.
     */
    private void validateRequest(ActionRequest request) throws ConfigurationException {
        final Map<String, String> dataErrors = new HashMap<String, String>();
        
        final String title = StringUtils.trimToNull(request.getParameter(TITLE));
        final String renderTimeOut = StringUtils.trimToNull(request.getParameter(RENDER_TIME_OUT));
        
        if (title == null) {
            dataErrors.put(TITLE, "A 'Title' must be specified.");
        }
        if (renderTimeOut != null && !NumberUtils.isNumber(renderTimeOut)) {
            dataErrors.put(RENDER_TIME_OUT, "A 'Render Time Out' must be a number specified.");
        }

        if (dataErrors.size() > 0) {
            throw new ConfigurationException(dataErrors);
        }
    }
}
