/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.manager.web.pages.portlet;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.portlet.common.IPreference;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.common.PreferenceImpl;
import org.jasig.portal.portlet.definition.ILocalizedPortletDefinitionControl;
import org.jasig.portal.portlets.utils.ConfigurationException;

import java.util.TreeMap;


/**
 * Displayes the Preferences information wizard page for portlet
 * definition mangement.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class PreferencesWizardPage extends BasePortletWizardPage {
    private static final Log LOG = LogFactory.getLog(PreferencesWizardPage.class);
    
    private static final String ADD_PREF = "ADD_PREF";
    private static final String RESET_PREFS = "RESET_PREFS";
    private static final String REFRESH = "REFRESH";
    private static final String DEL_PREF_PRFX = "DEL_PREF__";
    private static final String ADD_VAL_PRFX = "ADD_VAL__";
    private static final String DEL_VAL_PRFX = "DEL_VAL__";
    
    private static final String PREF_NM_PRFX = "PREF_NM__";
    private static final String PREF_VAL_PRFX = "PREF_VAL__";
    private static final String PREF_RO_PRFX = "PREF_RO__";
    private static final String PREF_ID_PRFX = "PREF_ID__";
    
    
    public String getTitle() {
        return "Preferences";
    }

    public void processResults(ActionRequest request, ActionResponse response) throws IOException, PortletException, ConfigurationException {
        final PortletSession session = request.getPortletSession();
        final ILocalizedPortletDefinitionControl localPortDef = (ILocalizedPortletDefinitionControl)session.getAttribute(PortletDefinitionWizard.PARAM_PORT_DEF);
        
        if (localPortDef == null) {
            throw new IllegalStateException("ILocalizedPortletDefinitionControl cannot be found in session for key '" + PortletDefinitionWizard.PARAM_PORT_DEF + "'");
        }
        
        final IPreferences prefs = localPortDef.getPreferences();

        if (request.getParameterMap().containsKey(RESET_PREFS)) {
            prefs.getPreferences().clear();
        }
        else {
            //Set a render flag to allow new preference
            if (request.getParameterMap().containsKey(ADD_PREF)) {
                response.setRenderParameter("ADD_NEW_PREF", "TRUE");
            }
            
            //Loop through the parameters and build a representation of the submitted
            //preference tree in memory
            final Map prefNameMap = new HashMap();
            for (final Enumeration nameEnum = request.getParameterNames(); nameEnum.hasMoreElements();) {
                final String paramName = (String)nameEnum.nextElement();
                
                if (paramName.startsWith(PREF_NM_PRFX)) {
                    final Integer prefId = this.getPrefId(paramName, PREF_NM_PRFX);
                    final String prefName = StringUtils.trim(request.getParameter(paramName));
                    
                    if (prefName == null) {
                        throw new ConfigurationException(paramName, "Preference names may not be null");
                    }
                    
                    final IPreference pref = this.getPref(prefId, prefNameMap);
                    pref.setName(prefName);
                }
                else if (paramName.startsWith(PREF_RO_PRFX)) {
                    final Integer prefId = this.getPrefId(paramName, PREF_RO_PRFX);
                    final String readOnlyStr = request.getParameter(paramName);
                    final boolean readOnly = Boolean.valueOf(readOnlyStr).booleanValue();

                    final IPreference pref = this.getPref(prefId, prefNameMap);
                    pref.setReadOnly(readOnly);
                }
                else if (paramName.startsWith(PREF_VAL_PRFX)) {
                    final Integer prefId = this.getPrefId(paramName, PREF_VAL_PRFX, "_");
                    final Integer valId = this.getPrefId(paramName, PREF_VAL_PRFX + prefId + "_");
                    
                    final String value = StringUtils.trim(request.getParameter(paramName));

                    final PreferenceBuilder pref = this.getPref(prefId, prefNameMap);
                    pref.addValue(valId, value);
                }
                else if (paramName.startsWith(PREF_ID_PRFX)) {
                    final Integer prefId = this.getPrefId(paramName, PREF_ID_PRFX);
                    
                    final String realPrefIdStr = StringUtils.trim(request.getParameter(paramName));
                    try {
                        final Long realPrefId = Long.valueOf(realPrefIdStr);

                        final PreferenceBuilder pref = this.getPref(prefId, prefNameMap);
                        pref.setId(realPrefId.longValue());
                    }
                    catch (NumberFormatException nfe) {
                        throw new ConfigurationException(paramName, "Invalid value '" + realPrefIdStr + "' for parameter '" + paramName + "'");
                    }
                }
            }
            
            //See if an action was requested on a preference, update the preference model
            final String action = StringUtils.trimToNull(request.getParameter(REFRESH));
            if (action != null) {
                if (action.startsWith(ADD_VAL_PRFX)) {
                    final Integer prefId = this.getPrefId(action, ADD_VAL_PRFX, "_");

                    final PreferenceBuilder pref = this.getPref(prefId, prefNameMap);
                    pref.appendValue("");
                }
                else if (action.startsWith(DEL_VAL_PRFX)) {
                    final Integer prefId = this.getPrefId(action, DEL_VAL_PRFX, "_");
                    final Integer valId = this.getPrefId(action, DEL_VAL_PRFX + prefId + "_");

                    final PreferenceBuilder pref = this.getPref(prefId, prefNameMap);
                    pref.removeValue(valId);
                }
                else if (action.startsWith(DEL_PREF_PRFX)) {
                    final Integer prefId = this.getPrefId(action, DEL_PREF_PRFX);

                    prefNameMap.remove(prefId);
                    
                }
            }
            
              
            //Copy the modified pref model into the actuall def preferences
            prefs.getPreferences().clear();
            for (final Iterator prefItr = prefNameMap.values().iterator(); prefItr.hasNext();) {
                final IPreference pref = (IPreference)prefItr.next();
                if (pref.getName() != null) {
                	IPreference newPref = new PreferenceImpl(pref);
                	newPref.setId(PreferenceImpl.UNPERSISTED_ID); // force DAO to treat as an insert
                	prefs.addPreference(newPref);
                }
                else {
                    LOG.warn("Preference with null name built from submitted data '" + pref + "' Ignoring for persistence.");
                }
            }
        }
    }

    public Map isCompleted(ActionRequest request, ActionResponse response) throws IOException, PortletException {
        return Collections.EMPTY_MAP;
    }
    
    
    /**
     * Gets/Creates the PreferenceBuilder for the specified key using the specified Map.
     */
    private PreferenceBuilder getPref(Integer prefId, Map prefs) {
        PreferenceBuilder pref = (PreferenceBuilder)prefs.get(prefId);
        
        if (pref == null) {
            pref = new PreferenceBuilder();
            prefs.put(prefId, pref);
        }
        
        return pref;
    }
    
    /**
     * Gets a preference ID from a param string by removing the prefix. 
     */
    private Integer getPrefId(String paramName, String prefix) throws ConfigurationException {
        return this.getPrefId(paramName, prefix, null);
    }
    
    /**
     * Gets a preference ID from a param string by remove the prefix and trimming everything after
     * the first occurance of the endString.
     */
    private Integer getPrefId(String paramName, String prefix, String endString) throws ConfigurationException {
        if (paramName.length() < prefix.length()) {
            throw new IllegalArgumentException("paramName must be longer than prefix");
        }
        
        final String prefIdStr;
        if (endString == null) {
            prefIdStr = paramName.substring(prefix.length());
        }
        else {
            final int endIndex = paramName.indexOf(endString, prefix.length());
            if (endIndex < prefix.length()) {
                throw new IllegalArgumentException("endString does not exist or overlaps with prefix");
            }
            
            prefIdStr = paramName.substring(prefix.length(), endIndex);
        }
        final Integer prefId;
        
        try {
            prefId = Integer.valueOf(prefIdStr);
        }
        catch (NumberFormatException nfe) {
            throw new ConfigurationException(paramName, "Invalid format for parameter with prefix '" + prefix + "'");
        }
        
        return prefId;
    }
    

    /**
     * Used to track changes to the preferences from the UI.
     */
    private class PreferenceBuilder implements IPreference {
        private long id = -1;
        private String name;
        private boolean readOnly;
        private SortedMap values = new TreeMap();

        public long getId() {
            return id;
        }
        public void setId(long id) {
            this.id = id;
        }

        public String getName() {
            return this.name;
        }

        public List getValues() {
            return new ArrayList(this.values.values());
        }

        public boolean isReadOnly() {
            return this.readOnly;
        }

        public void setName(String name) {
            this.name = name;
        }

        public void setReadOnly(boolean readOnly) {
            this.readOnly = readOnly;
        }

        public void setValues(List values) {
            this.values.clear();
            int i = 1;
            for (final Iterator valItr = values.iterator(); valItr.hasNext();) {
                this.values.put(new Integer(i), valItr.next());
                i++;
            }
        }
        
        public void addValue(Integer index, String value) {
            this.values.put(index, value);
        }

        public void removeValue(Integer index) {
            this.values.remove(index);
        }
        
        public void appendValue(String value) {
            this.values.put(new Integer(this.values.size() + 1000), value);
        }
    }
}
