/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.manager.web.pages.portlet;

import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;

import org.apache.commons.lang.StringUtils;
import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.definition.ILocalizedPortletDefinition;
import org.jasig.portal.portlet.definition.ILocalizedPortletDefinitionControl;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.definition.LocalizedPortletDefinitionImpl;
import org.jasig.portal.portlet.registry.general.IPortletDefinitionRegistry;
import org.jasig.portal.portlets.utils.ConfigurationException;


/**
 * Displayes the Subscription information wizard page for portlet
 * definition mangement.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class SubscriptionWizardPage extends BasePortletWizardPage {
    private static final String NAME = "NAME";
    private static final String FNAME = "FNAME";
    private static final String DESC = "DESC";
    private static final String KEYWORDS = "KEYWORDS";
    
    private IPortletDefinitionRegistry portletDefinitionRegistry;
    private ILocaleManager localeManager;
    
    /**
     * @return Returns the portletDefinitionRegistry.
     */
    public IPortletDefinitionRegistry getPortletDefinitionRegistry() {
        return this.portletDefinitionRegistry;
    }
    /**
     * @param portletDefinitionRegistry The portletDefinitionRegistry to set.
     */
    public void setPortletDefinitionRegistry(IPortletDefinitionRegistry portletDefinitionRegistry) {
        this.portletDefinitionRegistry = portletDefinitionRegistry;
    }
    
    /**
     * @return Returns the localeManager.
     */
    public ILocaleManager getLocaleManager() {
        return this.localeManager;
    }
    /**
     * @param localeManager The localeManager to set.
     */
    public void setLocaleManager(ILocaleManager localeManager) {
        this.localeManager = localeManager;
    }
    
    
    public String getTitle() {
        return "Subscription";
    }

    public void processResults(ActionRequest request, ActionResponse response) throws IOException, PortletException, ConfigurationException {
        final PortletSession session = request.getPortletSession();
        final ILocalizedPortletDefinitionControl localPortDef = (ILocalizedPortletDefinitionControl)session.getAttribute(PortletDefinitionWizard.PARAM_PORT_DEF);
        
        if (localPortDef == null) {
            throw new IllegalStateException("ILocalizedPortletDefinitionControl cannot be found in session for key '" + PortletDefinitionWizard.PARAM_PORT_DEF + "'");
        }
        
        this.validateRequest(request);
        
        final String dispName = StringUtils.trimToNull(request.getParameter(NAME));
        final String fName = StringUtils.trimToNull(request.getParameter(FNAME));
        final String desc = StringUtils.trimToEmpty(request.getParameter(DESC));
        final String keywords = StringUtils.trimToEmpty(request.getParameter(KEYWORDS));
        
        localPortDef.setLocalDisplayName(dispName);
        localPortDef.setFunctionalName(fName);
        localPortDef.setLocalDescription(desc);
        localPortDef.getPortletInfo().setKeywordsString(keywords);
    }
    
    public Map isCompleted(ActionRequest request, ActionResponse response) throws IOException, PortletException {
        final Map errors = new HashMap();
        
        final PortletSession session = request.getPortletSession();
        final ILocalizedPortletDefinitionControl localPortDef = (ILocalizedPortletDefinitionControl)session.getAttribute(PortletDefinitionWizard.PARAM_PORT_DEF);
        
        if (localPortDef != null) {
            if (localPortDef.getLocalDisplayName() == null) {
                errors.put(NAME, "No name specified for the PortletDefinition");
            }

            if (localPortDef.getFunctionalName() == null) {
                errors.put(FNAME, "No name specified for the PortletDefinition");
            }
            else {
                final String msg = checkDuplicateFuncName(localPortDef.getFunctionalName(), session);
                if (msg != null) {
                    errors.put(FNAME, msg);
                }
            }
        }
        else {
            errors.put("", "No PortletDefinition in the session.");
        }
        
        return Collections.unmodifiableMap(errors);
    }

    /**
     * Validates the request parameters for this wizard page. Throwing a ConfigurationException for
     * an invalid request.
     * 
     * @param request The request to get the parameters to validate from.
     * @throws ConfigurationException If the parameter values are invalid.
     */
    private void validateRequest(ActionRequest request) throws ConfigurationException {
        final Map dataErrors = new HashMap();
        
        final String dispName = StringUtils.trimToNull(request.getParameter(NAME));
        final String fName = StringUtils.trimToNull(request.getParameter(FNAME));
        
        if (dispName == null) {
            dataErrors.put(NAME, "A 'Name' must be specified.");
        }
        if (fName == null) {
            dataErrors.put(FNAME, "A 'Functional Portlet Name' must be specified.");
        }
        else {
            final PortletSession session = request.getPortletSession();
            final String msg = checkDuplicateFuncName(fName, session);
            if (msg != null) {
                dataErrors.put(FNAME, msg);
            }
        }

        if (dataErrors.size() > 0) {
            throw new ConfigurationException(dataErrors);
        }
    }

    /**
     * Checks for a duplicate fName in the definition registry.
     * 
     * @param fName The fName to check.
     * @param session The session to get info about the portlet def control from. 
     * @return An error message if a duplicate fname exists, null if there is no problem.
     */
    private String checkDuplicateFuncName(final String fName, final PortletSession session) {
        final IPortletDefinition existingPortDef = this.portletDefinitionRegistry.getPortletDefinitionByFunctionalName(fName);
        if (existingPortDef != null) {
            final PortletDefinitionId basePortDefId = (PortletDefinitionId)session.getAttribute(PortletDefinitionWizard.PARAM_PORT_DEF_ID);
            
            if (basePortDefId == null || !basePortDefId.equals(existingPortDef.getPortletDefinitionId())) {
                ILocalizedPortletDefinition localPortDef = new LocalizedPortletDefinitionImpl(existingPortDef, this.localeManager);
                
                return "Specified functional name '" + fName + "' conflicts with an existing PortletDefintion named '" + localPortDef.getLocalDisplayName() + "'.";
            }
        }
        
        return null;
    }
}
