/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlets.passwd;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.acegisecurity.userdetails.UserDetails;
import org.acegisecurity.userdetails.UserDetailsService;
import org.acegisecurity.providers.dao.SaltSource;
import org.acegisecurity.providers.encoding.PasswordEncoder;
import org.acegisecurity.providers.encoding.PlaintextPasswordEncoder;

import org.jasig.portal.security.acegi.IPasswordSetter;

/**
 * Portlet bean for Acegi Passwd portlet 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class AcegiPasswdPortletBean extends GenericPortlet {
    private static final String DEFAULT_VIEW = "/WEB-INF/jsp/passwd/passwd.jsp";

    // injected properties
    PasswordEncoder passwordEncoder=new PlaintextPasswordEncoder();
    UserDetailsService userDetailsService;
    // optional injected properties
    SaltSource saltSource=null;
    IPasswordSetter passwordSetter;
    
    
    public final static String USERNAME_PARAM_NAME="username"; 
    public final static String OLD_PASSWORD_PARAM_NAME="existing_password";
    public final static String NEW_PASSWORD_PARAM_NAME="new_password";
    public final static String NEW_PASSWORD_REPEAT_PARAM_NAME="new_password_repeat";
    
    public final static String ERROR_MESSAGE_ATTRIBUTE_NAME="errors";
    public final static String INFO_MESSAGE_ATTRIBUTE_NAME="info";
    
    public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {
        // clear old messages
        PortletSession session=request.getPortletSession();
        clearErrorMessages(session);
        clearInfoMessages(session);
        
        String userName=request.getParameter(USERNAME_PARAM_NAME);
        String oldPassword=request.getParameter(OLD_PASSWORD_PARAM_NAME);
        String newPassword=request.getParameter(NEW_PASSWORD_PARAM_NAME);
        String newPasswordRepeat=request.getParameter(NEW_PASSWORD_REPEAT_PARAM_NAME);
        
        
        boolean validInput=checkSimpleValidity("username",userName,session);
        validInput&=checkSimpleValidity("current password",oldPassword,session);
        validInput&=checkSimpleValidity("new password",newPassword,session);
        if(validInput) {
            if(!newPassword.equals(newPasswordRepeat)) {
                addErrorMessage("two copies of the new password do not match",session);
            } else {
                // retrieve user details
                UserDetails userDetails=userDetailsService.loadUserByUsername(userName);
                // validate old password
                Object salt=null;
                if(saltSource!=null) {
                    salt=saltSource.getSalt(userDetails);
                }
                if(!passwordEncoder.isPasswordValid(userDetails.getPassword(),oldPassword,salt)) {
                    addErrorMessage("current password is invalid",session);
                } else {
                    // encode new password
                    String encodedPassword=passwordEncoder.encodePassword(newPassword,salt);
                    // persist new password
                    passwordSetter.setPassword(userName,encodedPassword);
                    addInfoMessage("password changed",session);
                }
            }
        }
        
    }
    
    boolean checkSimpleValidity(String paramName, String paramValue, PortletSession session) {
        if(paramValue==null || paramValue.length()==0) {
            addErrorMessage(paramName+" can not be empty!",session);
            return false;
        } else {
            return true;
        }
    }

    void addErrorMessage(String message, PortletSession session) {
        addMessage(message,session,ERROR_MESSAGE_ATTRIBUTE_NAME);
    }
    
    void clearErrorMessages(PortletSession session) {
        session.removeAttribute(ERROR_MESSAGE_ATTRIBUTE_NAME);
    }
    
    void clearInfoMessages(PortletSession session) {
        session.removeAttribute(INFO_MESSAGE_ATTRIBUTE_NAME);
    }
    
    void addInfoMessage(String message, PortletSession session) {
       addMessage(message,session,INFO_MESSAGE_ATTRIBUTE_NAME);
    }
    
    void addMessage(String message,PortletSession session,String sessionAttributeName) {
        List messageList=(List) session.getAttribute(sessionAttributeName);
        if(messageList==null) {
            messageList=new ArrayList();
            session.setAttribute(sessionAttributeName,messageList);
        }
        messageList.add(message);
    }
    
    protected void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        PortletSession porletSession=request.getPortletSession();
        request.setAttribute(ERROR_MESSAGE_ATTRIBUTE_NAME,porletSession.getAttribute(ERROR_MESSAGE_ATTRIBUTE_NAME));
        request.setAttribute(INFO_MESSAGE_ATTRIBUTE_NAME,porletSession.getAttribute(INFO_MESSAGE_ATTRIBUTE_NAME));
        
        PortletContext context = getPortletContext();
        PortletRequestDispatcher prd = context.getRequestDispatcher(DEFAULT_VIEW);
        response.setContentType("text/html");
        prd.include(request,response);
    }

    public void setUserDetailsService(UserDetailsService userDetailsService) {
        this.userDetailsService = userDetailsService;
    }

    public void setPasswordEncoder(PasswordEncoder passwordEncoder) {
        this.passwordEncoder = passwordEncoder;
    }

    public void setPasswordSetter(IPasswordSetter passwordSetter) {
        this.passwordSetter = passwordSetter;
    }

    public void setSaltSource(SaltSource saltSource) {
        this.saltSource = saltSource;
    }

    
}
