/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.portlets.permissions;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.jasig.gap.permissions.IAuthorizationService;
import org.jasig.gap.permissions.IPermission;
import org.jasig.gap.permissions.IUpdatingPermissionManager;
import org.springframework.webflow.core.collection.ParameterMap;
import org.springframework.webflow.execution.Event;
import org.springframework.webflow.execution.RequestContext;

public class UpdatePermissionsAction extends AuthorizingAction {
    
    private String principalActivitySeparator;
    private String typeKeySeparator;

    @Override
    protected Event doExecute(RequestContext context) throws Exception {
        
        final ParameterMap parameterMap = context.getExternalContext().getRequestParameterMap();
        
        //GenericEntityBean ownerBean =  (GenericEntityBean)context.getFlowScope().get(ownerBeanKey);
        GenericEntityBean ownerBean =  (GenericEntityBean)context.getExternalContext().getSessionMap().get(ownerBeanKey);
        if (ownerBean == null) {
            return error();
        }
        
        //GenericEntityBean targetBean = (GenericEntityBean)context.getFlowScope().get(targetBeanKey);
        GenericEntityBean targetBean = (GenericEntityBean)context.getExternalContext().getSessionMap().get(targetBeanKey);
        if (targetBean == null) {
            return error();
        }
        
        String[] principals = parameterMap.getArray(submittedPrincipalsKey);
        Map<String, Object> params = parameterMap.asMap();
        
        IAuthorizationService as = getAuthorizationService();
        IUpdatingPermissionManager upm = as.newUpdatingPermissionManager(ownerBean.getId());
        List<IPermission> addedPermissions = new LinkedList<IPermission>();
        List<IPermission> removedPermissions = new LinkedList<IPermission>();
        List<IPermission> updatedPermissions = new LinkedList<IPermission>();
        
        for (int i=0; principals != null && i<principals.length; i++) {
            for (String name : params.keySet()) {
                if (name.startsWith(buildNamePrefix(principals[i]))) {
                    String value = parameterMap.get(name);
                    String[] activitySplit = name.split(principalActivitySeparator);
                    if (activitySplit.length != 2) {
                        // TODO log error
                        return error();
                    }
                    String[] principalSplit = activitySplit[0].split(typeKeySeparator);
                    if (principalSplit.length != 2) {
                        // TODO log error
                        return error();
                    }
                    String type = principalSplit[0];
                    String key = principalSplit[1];
                    String activity = activitySplit[1];
                    IAuthorizationPrincipal ap = as.newPrincipal(key, Class.forName(type));
                    
                    boolean hasPermission = hasPermissions(ap, ownerBean, targetBean, activity);
                    IPermission p = getPermission(upm, ap, ownerBean, targetBean, activity, value);
                    if (p != null) {
                        if (hasPermission) {
                            if (grantOrDeny(value)) {
		                        updatedPermissions.add(p);
                            } else {
                                removedPermissions.add(p);
                            }
                        } else {
                            if (grantOrDeny(value)) {
		                        addedPermissions.add(p);
                            }
                        }
                    }
                }
            }
        }
        
        // now get principals that have been removed
        //List<PrincipalBean> removedPrincipals = (List<PrincipalBean>)context.getFlowScope().get(removedPrincipalsKey);
        List<PrincipalBean> removedPrincipals = (List<PrincipalBean>)context.getExternalContext().getSessionMap().get(removedPrincipalsKey);
        if (removedPrincipals != null) {
	        for (PrincipalBean pb : removedPrincipals) {
	            IAuthorizationPrincipal ap = as.newPrincipal(pb.getId(), pb.getType());
	            IPermission[] permissions = as.getPermissionsForPrincipal(ap, ownerBean.getId(), null, targetBean.getId());
	            for (int i=0; permissions != null && i<permissions.length; i++) {
	                removedPermissions.add(permissions[i]);
	            }
	        }
        }
        
        if (addedPermissions.size() > 0) {
	        upm.addPermissions(addedPermissions.toArray(new IPermission[addedPermissions.size()]));
        }
        if (removedPermissions.size() > 0) {
	        upm.removePermissions(removedPermissions.toArray(new IPermission[removedPermissions.size()]));
        }
        if (updatedPermissions.size() > 0) {
	        upm.updatePermissions(updatedPermissions.toArray(new IPermission[updatedPermissions.size()]));
        }
        
        return success();
    }

    private boolean grantOrDeny(String value) {
        return IPermission.PERMISSION_TYPE_GRANT.equals(value) ||
            IPermission.PERMISSION_TYPE_DENY.equals(value);
    }

    private boolean hasPermissions(IAuthorizationPrincipal ap,
        GenericEntityBean ownerBean, GenericEntityBean targetBean,
        String activity) {
        
        String owner = ownerBean.getId();
        String target = targetBean.getId();
        IPermission[] permissions = getAuthorizationService().getPermissionsForPrincipal(ap, owner, activity, target);
        if (permissions == null) {
            return false;
        }
        return permissions.length > 0;
    }

    private IPermission getPermission(IUpdatingPermissionManager pm,
        IAuthorizationPrincipal ap, GenericEntityBean ownerBean,
        GenericEntityBean targetBean, String activity, String value) {
        
        String owner = ownerBean.getId();
        String target = targetBean.getId();
        
        IPermission[] permissions = getAuthorizationService().getPermissionsForUpdate(ap, owner, activity, target);
        
        if (permissions == null || permissions.length == 0) {
            permissions = new IPermission[1];
            permissions[0] = pm.newPermission(ap);
            permissions[0].setActivity(activity);
            permissions[0].setTarget(target);
        }
        
        if (permissions.length > 1) {
            // TODO log error
            return null;
        }
        
        permissions[0].setType(value);
        
        return permissions[0];
    }
        

    private String buildNamePrefix(String s) {
        return new StringBuffer(s).append(principalActivitySeparator).toString();
    }

    public void setPrincipalActivitySeparator(String principalActivitySeparator) {
        this.principalActivitySeparator = principalActivitySeparator;
    }

    public void setTypeKeySeparator(String typeKeySeparator) {
        this.typeKeySeparator = typeKeySeparator;
    }
    
}
