/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlets.tests.servant;

import java.io.IOException;
import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.portlet.rendering.IPortletRenderContent;
import org.jasig.portal.portlet.servant.IPortletServant;
import org.jasig.portal.portlet.servant.IPortletServantLocator;
import org.jasig.portal.portlet.servant.ServantLocatorAccessor;

/**
 *  A consumer portlet for testing servant functionality 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class ServantConsumerTestPortlet extends GenericPortlet {
    protected final Log logger = LogFactory.getLog(this.getClass());

    private static final String DEFAULT_VIEW = "/WEB-INF/jsp/tests/servant/consumer.jsp";

    static final String SERVANT_INSTANCE_ATTRIBUTE = "servant";
    static final String SERVANT_CONTENT_ATTRIBUTE = "servantContent";
    static final String FNAME_ATTRIBUTE = "servantFName";
    static final String VALUE_ATTRIBUTE = "servantAttributeValue";
    
    static final String COMPLETE_ATTRIBUTE = "servantComplete";
    static final String CANCELED_ATTRIBUTE = "servantCanceled";
    static final String RESULT_ATTRIBUTE = "servantResult";
    
    static final String SERVANT_RUNNING = "servantRunning";
    
    static final String START_COMMAND = "startServant";
    static final String STOP_COMMAND = "stopServant";
    static final String SET_ATTR_COMMAND = "setServantAttribute";
    

    protected void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        final PortletSession session = request.getPortletSession();
        
        final String fname = this.getServantFuncName(request);
        request.setAttribute(FNAME_ATTRIBUTE, fname);

        final boolean servantRunning = this.isServantRunning(session);
        request.setAttribute(SERVANT_RUNNING, servantRunning);
        
        if (servantRunning) {
            // obtain servant
            final IPortletServant servant = this.getServant(request, fname);
            if (servant != null) {
                //Add a reference to the servant for the JSP
                request.setAttribute(SERVANT_INSTANCE_ATTRIBUTE, servant);
                
                //Pass an attribute to the servant
                final String servantValue = request.getParameter(VALUE_ATTRIBUTE);
                servant.setAttribute(request, VALUE_ATTRIBUTE, servantValue);

                //Render the servant and add its output
                final IPortletRenderContent content = servant.getRenderContent(request, response);
                request.setAttribute(SERVANT_CONTENT_ATTRIBUTE, content);
                
                //Pass servant related data to the JSP
                final Object value = servant.getAttribute(request, ServantTestPortlet.VALUE_ATTR);
                request.setAttribute(VALUE_ATTRIBUTE, value);
                
                final boolean cancelled = servant.isCancelled(request);
                request.setAttribute(CANCELED_ATTRIBUTE, cancelled);
                
                final boolean complete = servant.isComplete(request);
                request.setAttribute(COMPLETE_ATTRIBUTE, complete);
                
                if (complete) {
                    final Object result = servant.getResult(request);
                    request.setAttribute(RESULT_ATTRIBUTE, result);
                }

                //caching has to be disabled for the screens that include servants
                response.setProperty(RenderResponse.EXPIRATION_CACHE, "0");
            }
            else {
                this.logger.error("unable to obtain instance of running servant: fname=" + fname);
            }
        }

        PortletContext context = this.getPortletContext();
        PortletRequestDispatcher prd = context.getRequestDispatcher(DEFAULT_VIEW);
        prd.include(request, response);
    }

    public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {
        final PortletSession session = request.getPortletSession();
        
        final Map parameterMap = request.getParameterMap();
        if (parameterMap.containsKey(START_COMMAND)) {
            final String reqFname = request.getParameter(FNAME_ATTRIBUTE);
            session.setAttribute(FNAME_ATTRIBUTE, reqFname);
            
            if (logger.isDebugEnabled()) {
                logger.debug("starting servant : fname=" + reqFname);
            }
            
            //Initialize the servant
            this.getServant(request, reqFname);
            this.setServantRunning(session, true);
        }
        else if (parameterMap.containsKey(STOP_COMMAND)) {
            final String fname = this.getServantFuncName(request);
            if (logger.isDebugEnabled()) {
                logger.debug("stopping servlet : fname=" + fname);
            }
            
            final IPortletServant servant = this.getServant(request, fname);
            servant.reset(request);

            this.setServantRunning(session, false);
        }
        else if (this.isServantRunning(session) && parameterMap.containsKey(SET_ATTR_COMMAND)) {
            final String servantValue = request.getParameter(VALUE_ATTRIBUTE);
            response.setRenderParameter(VALUE_ATTRIBUTE, servantValue);
        }
    }
    
    protected String getServantFuncName(PortletRequest request) {
        final PortletSession session = request.getPortletSession();
        
        String fname = (String)session.getAttribute(FNAME_ATTRIBUTE);
        if (fname == null) {
            final PortletPreferences preferences = request.getPreferences();
            fname = preferences.getValue(FNAME_ATTRIBUTE, null);
            
            if (fname != null) {
                session.setAttribute(FNAME_ATTRIBUTE, fname);
            }
        }
        
        return fname;
    }
    
    protected boolean isServantRunning(PortletSession session) {
        final Boolean running = (Boolean)session.getAttribute(SERVANT_RUNNING);
        return running != null && running;
    }
    
    protected void setServantRunning(PortletSession session, Boolean servantRunning) {
        session.setAttribute(SERVANT_RUNNING, servantRunning);
    }

    protected IPortletServant getServant(PortletRequest request, String fname) {
        if (fname == null) {
            this.logger.error("null or empty fName has been specified for the servant");
            return null;
        }
        
        final IPortletServantLocator servantLocator = ServantLocatorAccessor.getServantLocator();
        if (servantLocator == null) {
            logger.error("unable to obtain servant locator");
            return null;
        }
        
        final IPortletServant servant = servantLocator.getServant(fname, request, true);
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("Found IPortletServant='" + servant + "' for fname=" + fname + "");
        }
        
        return servant;
    }
}
