/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.utils;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.collections.Predicate;
import org.apache.commons.collections.functors.InstanceofPredicate;
import org.apache.commons.collections.map.PredicatedMap;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.BeanInitializationException;

import java.util.Collections;


/**
 * Provides the framework for navigating between {@link IActionPage}s using
 * {@link PageAction}s. This class delegates the actual rendering and action
 * handling to the curent {@link IActionPage}.
 * <br>
 * <br>
 * Configuration:
 * <table border="1">
 *     <tr>
 *         <th align="left">Property</th>
 *         <th align="left">Description</th>
 *         <th align="left">Required</th>
 *         <th align="left">Default</th>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">currentPageParam</td>
 *         <td>
 *             The request parameter used to specify the current IActionPage
 *             which rendering is being delegated to.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">CURRENT_PAGE</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">defaultPageName</td>
 *         <td>
 *             The name of the default IActionPage. This page will be the
 *             page resolved if no page is specified using the currentPageParam
 *             request parameter. 
 *         </td>
 *         <td valign="top">Yes</td>
 *         <td valign="top"></td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">pages</td>
 *         <td>
 *             A Map of String page names to IActionPage pages. Used to resolve
 *             the IActionPage names specified in the PageActions.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top"></td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">failOnInvalidPage</td>
 *         <td>
 *             If an exception should be thrown when looking up an
 *             IActionPage from the pages property if it does not exist.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">false</td>
 *     </tr>
 * </table>
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class ActionPageHandler extends BaseActionPage {
    private static final Predicate PRED_STRING_TYPE = new InstanceofPredicate(String.class);
    private static final Predicate PRED_IACTIONPAGE_TYPE = new InstanceofPredicate(IActionPage.class);
    
    private static final Class[] ACTION_PARAMETERS = new Class[] {ActionRequest.class, ActionResponse.class};
    
    private String currentPageParam = "CURRENT_PAGE";
    private String defaultPageName;
    private Map pages;
    private boolean failOnInvalidPage = true;
    
    //********** Getters and Setters **********
    
    /**
     * @return Returns the currentPageParam.
     */
    public String getCurrentPageParam() {
        return this.currentPageParam;
    }
    /**
     * @param currentPageParam The currentPageParam to set.
     */
    public void setCurrentPageParam(String currentPageParam) {
        if (currentPageParam == null) {
            throw new IllegalArgumentException("currentPageParam may not be null");
        }
        currentPageParam = currentPageParam.trim();
        if (currentPageParam.length() == 0) {
            throw new IllegalArgumentException("currentPageParam must have length > 0");
        }
    
        this.currentPageParam = currentPageParam;
    }
    
    /**
     * @return Returns the failOnInvalidPage.
     */
    public boolean isFailOnInvalidPage() {
        return this.failOnInvalidPage;
    }
    /**
     * @param failOnInvalidPage The failOnInvalidPage to set.
     */
    public void setFailOnInvalidPage(boolean failOnInvalidPage) {
        this.failOnInvalidPage = failOnInvalidPage;
    }
    
    /**
     * @return Returns the defaultPageName.
     */
    public String getDefaultPageName() {
        return this.defaultPageName;
    }
    /**
     * @param defaultPageName The defaultPageName to set.
     */
    public void setDefaultPageName(String defaultPageName) {
        if (defaultPageName == null) {
            throw new IllegalArgumentException("defaultPageName may not be null.");
        }
    
        this.defaultPageName = defaultPageName;
    }
    
    /**
     * @return Returns the pages.
     */
    public Map getPages() {
        return this.pages;
    }
    /**
     * @param pages The pages to set.
     */
    public void setPages(Map pages) {
        if (pages == null) {
            throw new IllegalArgumentException("pages may not be null");
        }
        if (pages.size() < 1) {
            throw new IllegalArgumentException("pages property must contain at least one entry");
        }
        
        //Defensive copy
        pages = new HashMap(pages);
        //Predicate for type safety
        pages = PredicatedMap.decorate(pages, PRED_STRING_TYPE, PRED_IACTIONPAGE_TYPE);
        //Make unmodifiable
        this.pages = Collections.unmodifiableMap(pages);
    }
    
    
    //********** InitializingBean Methods **********

    public void afterPropertiesSet() throws Exception {
        if (this.pages == null) {
            throw new BeanInitializationException("pages may not be null");
        }
        if (this.defaultPageName == null) {
            throw new BeanInitializationException("defaultPageName may not be null.");
        }
        
        super.afterPropertiesSet();
    }    
    
    
    //********** IPage Methods **********
    
    public void renderPage(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        final String currentPageName = this.resolveCurrentPageName(request);
        final IActionPage currentPage = this.getPageUsingDefault(currentPageName);
        currentPage.renderPage(request, response);
    }
    
    public void processResults(ActionRequest request, ActionResponse response) throws PortletException, IOException {
        final String currentPageName = this.resolveCurrentPageName(request);
        final IActionPage currentPage = this.getPageUsingDefault(currentPageName);
        final PageAction action = currentPage.getAction(request);
        
        if (action == null) {
            currentPage.processResults(request, response);
        }
        else {
            final String nextPageName = action.getNextPageName();
            response.setRenderParameter(this.currentPageParam, nextPageName);
            
            final String actionMethodName = action.getActionMethodName();
            
            if (actionMethodName == null) {
                final IActionPage nextPage = this.getPage(nextPageName);
                nextPage.processResults(request, response);
            }
            else {
                final String actionTargetPageName = action.getActionTargetPageName();

                final IActionPage actionTargetPage;
                final String actualActionTargetPageName;
                if (actionTargetPageName != null) {
                    actionTargetPage = this.getPage(actionTargetPageName);
                    actualActionTargetPageName = actionTargetPageName;
                }
                else {
                    actionTargetPage = currentPage;
                    actualActionTargetPageName = nextPageName;
                }
                
                try {
                    final Method actionMethod = actionTargetPage.getClass().getMethod(actionMethodName, ACTION_PARAMETERS);
                    actionMethod.invoke(actionTargetPage, new Object[] {request, response});
                }
                catch (NoSuchMethodException nsme) {
                    final IllegalStateException ise = new IllegalStateException("Specified action '" + actionMethodName + "' does not exist for page '" + actualActionTargetPageName + "'.");
                    ise.initCause(nsme);
                    throw ise;
                }
                catch (InvocationTargetException ite) {
                    final IllegalStateException ise = new IllegalStateException("Error invoking specified action '" + actionMethodName + "' for page '" + actualActionTargetPageName + "'.");
                    ise.initCause(ite);
                    throw ise;
                }
                catch (IllegalAccessException iae) {
                    final IllegalStateException ise = new IllegalStateException("Specified action '" + actionMethodName + "' does is not accessable on page '" + actualActionTargetPageName + "'.");
                    ise.initCause(iae);
                    throw ise;
                }
            }
        }
    }
    
    
    //********** Utilitiy Methods **********
    
    /**
     * Resolves the current action page name from the request
     * using the currentPageParam.
     * 
     * @param request The request to resolve the action page name from.
     * @return The IActionPage resolved. null if none was specified or if isFailOnInvalidPage is false and the specified page doesn't exist in pages.
     */
    protected String resolveCurrentPageName(PortletRequest request) {
        String currentPageName = StringUtils.trimToNull(request.getParameter(this.currentPageParam));
        
        final PortletSession session = request.getPortletSession();
        
        if (currentPageName != null) {
            session.setAttribute(this.currentPageParam, currentPageName);
        }
        else {
            currentPageName = (String)session.getAttribute(this.currentPageParam);
        }
        
        if (currentPageName != null) {
            if (this.pages.containsKey(currentPageName)) {
                return currentPageName;
            }
            else if (this.isFailOnInvalidPage()) {
                throw new IllegalArgumentException("'" + currentPageName + "' is an unknown current page, specified by the '" + this.currentPageParam + "' parameter.");
            }
            else {
                return null;
            }
        }
    
        return currentPageName;
    }
    
    /**
     * Returns the IActionPage mapped to the specified pageName. The pageName
     * may not be null. This method will throw an exception if failOnInvalidPage
     * is true and no mapped page is found, otherwise null would be returned.
     * 
     * @param pageName The name of the page to find a mapping for, may not be null.
     * @return The mapped IActionPage, may be null.
     */
    protected IActionPage getPage(String pageName) {
        if (pageName == null) {
            throw new IllegalArgumentException("pageName may not be null");
        }
        
        IActionPage page = (IActionPage)this.pages.get(pageName);
            
        if (page == null && this.failOnInvalidPage) {
            throw new IllegalArgumentException("'" + pageName + "' is an unknown current page, specified by the '" + this.currentPageParam + "' parameter.");
        }
        
        return page;
    }
    
    /**
     * Gets the mapped IActionPage but will never return null. If a
     * mapped page is not found the default page is returned.
     * 
     * @param pageName The name of the page to find a mapping for, may be null.
     * @return The mapped IActionPage, will not be null.
     */
    protected IActionPage getPageUsingDefault(String pageName) {
        if (pageName == null) {
            pageName = this.defaultPageName;
        }
        
        IActionPage page = this.getPage(pageName);
            
        if (page == null) {
            page = (IActionPage)this.pages.get(this.defaultPageName);
            
            if (page == null) {
                throw new IllegalStateException("Default page '" + this.defaultPageName + "' must exist in pages Map");
            }
        }
        
        return page;
    }
}
