/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.utils;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.portlet.PortletRequest;

import org.apache.commons.collections.Predicate;
import org.apache.commons.collections.functors.InstanceofPredicate;
import org.apache.commons.collections.map.PredicatedMap;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.BeanInitializationException;
import org.springframework.beans.factory.InitializingBean;

/**
 * A BaseActionPage provides the framework for resolving PageActions.
 * 
 * <br>
 * <br>
 * Configuration:
 * <table border="1">
 *     <tr>
 *         <th align="left">Property</th>
 *         <th align="left">Description</th>
 *         <th align="left">Required</th>
 *         <th align="left">Default</th>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">actionParamName</td>
 *         <td>
 *             The name of the request parameter used to get the name of the
 *             action to return.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">ACTION</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">useParamNameAsAction</td>
 *         <td>
 *             If true the <code>actionParamName</code> and <code>failOnInvalidAction</code>
 *             properties are ignored. Instead the action keys are resolved by matching the
 *             name of the action to the name of a submitted parameter.
 *             <br>
 *             <br>
 *             ex: If the submitted parameters contain 'ACTION_NEXT_PAGE=Next Page' and there is
 *             an action whos key is ACTION_NEXT_PAGE that action will be resolved.
 *             <br>
 *             <br>
 *             Note that this does require O(n) time for the {@link #getAction(PortletRequest)} method
 *             where N is the number of actions in the action property.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">false</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">actions</td>
 *         <td>
 *             The map of action names to PageActions.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">java.util.Collections.EMPTY_MAP</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">failOnInvalidAction</td>
 *         <td>
 *             If the getAction method should fail with an exception if
 *             an action is specified in the request by the actionParamName
 *             parameter but does not exist in the actions Map.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">false</td>
 *     </tr>
 * </table>
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public abstract class BaseActionPage implements IActionPage, InitializingBean {
    private static final Predicate PRED_STRING_TYPE = new InstanceofPredicate(String.class);
    private static final Predicate PRED_ACTION_TYPE = new InstanceofPredicate(PageAction.class);
    
    private String actionParamName = "ACTION";
    private boolean useParamNameAsAction = false;
    private Map actions = Collections.EMPTY_MAP;
    private boolean failOnInvalidAction = false;
    
    /**
     * @return Returns the actionParamName.
     */
    public String getActionParamName() {
        return this.actionParamName;
    }
    /**
     * @param actionParamName The actionParamName to set.
     */
    public void setActionParamName(String actionParamName) {
        if (actionParamName == null) {
            throw new IllegalArgumentException("actionParamName may not be null");
        }

        this.actionParamName = actionParamName;
    }

    /**
     * @return Returns the useParamNameAsAction.
     */
    public boolean isUseParamNameAsAction() {
        return this.useParamNameAsAction;
    }
    /**
     * @param useParamNameAsAction The useParamNameAsAction to set.
     */
    public void setUseParamNameAsAction(boolean useParamNameAsAction) {
        this.useParamNameAsAction = useParamNameAsAction;
    }

    /**
     * @return Returns the actions.
     */
    public Map getActions() {
        return this.actions;
    }
    /**
     * @param actions The actions to set.
     */
    public void setActions(Map actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions may not be null");
        }
        
        //Defensive copy
        actions = new HashMap(actions);
        //Predicate for type safety
        actions = PredicatedMap.decorate(actions, PRED_STRING_TYPE, PRED_ACTION_TYPE);
        //Make unmodifiable
        this.actions = Collections.unmodifiableMap(actions);
    }
    
    /**
     * @return Returns the failOnInvalidAction.
     */
    public boolean isFailOnInvalidAction() {
        return this.failOnInvalidAction;
    }
    /**
     * @param failOnInvalidAction The failOnInvalidAction to set.
     */
    public void setFailOnInvalidAction(boolean failOnInvalidAction) {
        this.failOnInvalidAction = failOnInvalidAction;
    }

    /**
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    public void afterPropertiesSet() throws Exception {
        if (this.actionParamName == null) {
            throw new BeanInitializationException("actionParamName may not be null");
        }
    }
    
    
    public PageAction getAction(PortletRequest request) {
        if (this.useParamNameAsAction) {
            final Map paramMap = request.getParameterMap();
            for (final Iterator actionItr = this.actions.entrySet().iterator(); actionItr.hasNext();) {
                final Map.Entry entry = (Map.Entry)actionItr.next();
                final String actionKey = (String)entry.getKey();
                
                if (paramMap.containsKey(actionKey)) {
                    return (PageAction)entry.getValue();
                }
            }
            
            return null;
        }
        else {
            final String actionName = StringUtils.trimToNull(request.getParameter(this.actionParamName));
            
            if (actionName == null) {
                return null;
            }
            else {
                final PageAction action = (PageAction)this.actions.get(actionName);
                
                if (action == null && this.failOnInvalidAction) {
                    throw new IllegalStateException("No action found for '" + actionName + "'");
                }
                
                return action;
            }
        }
    }
}
