/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.utils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.collections.Predicate;
import org.apache.commons.collections.functors.AndPredicate;
import org.apache.commons.collections.functors.EqualPredicate;
import org.apache.commons.collections.functors.InstanceofPredicate;
import org.apache.commons.collections.functors.NotPredicate;
import org.apache.commons.collections.list.PredicatedList;
import org.apache.commons.collections.map.PredicatedMap;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.BeanInitializationException;

import java.util.Collections;


/**
 * The BaseWizard allows a set of {@link IWizardPage}s to be displayed
 * with wizard style control. This class can display header and footer JSPs
 * and display a wizard page in-between. It supports a next page parameter
 * which tells it the next {@link IWizardPage} to display. During any page
 * change the currently display {@link IWizardPage} has it's {@link IWizardPage#processResults(ActionRequest, ActionResponse)}
 * method called to perform parameter processing.
 * <br>
 * <br>
 * Configuration:
 * <table border="1">
 *     <tr>
 *         <th align="left">Property</th>
 *         <th align="left">Description</th>
 *         <th align="left">Required</th>
 *         <th align="left">Default</th>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">pages</td>
 *         <td>
 *             Name to IWizardPage mapings. The name of the page given here is
 *             the key used for navigating between wizard pages by this class.
 *         </td>
 *         <td valign="top">Yes</td>
 *         <td valign="top"></td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">pageOrder</td>
 *         <td>
 *             The page names that also exist as keys in the pages property. This
 *             property is needed to ensure wizard pages are displayed in the
 *             correct order.
 *         </td>
 *         <td valign="top">Yes</td>
 *         <td valign="top"></td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">headerJspPath</td>
 *         <td>
 *             The path to the JSP to render as the header for the wizard. This
 *             path must be accessible by a portlet request dispatcher. NULL
 *             implies that no header should be rendered.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">null</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">footerJspPath</td>
 *         <td>
 *             The path to the JSP to render as the footer for the wizard. This
 *             path must be accessible by a portlet request dispatcher. NULL
 *             implies that no footer should be rendered.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">null</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">summaryHeaderJspPath</td>
 *         <td>
 *             The path to the JSP to render as the header for the summry screen
 *             the wizard can display. This header is rendered after the wizard's
 *             header and before the first IWizardPage summary. This path must be
 *             accessible by a portlet request dispatcher. NULL implies that no
 *             header should be rendered.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">null</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">summarySeperatorJspPath</td>
 *         <td>
 *             The path to the JSP to render as the seperator for the summry screen
 *             the wizard can display. This seperator is rendered in between the
 *             summaries rendered by the IWizardPages. This path must be accessible
 *             by a portlet request dispatcher. NULL implies that no header should
 *             be rendered.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">null</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">summaryFooterJspPath</td>
 *         <td>
 *             The path to the JSP to render as the footer for the summry screen
 *             the wizard can display. This footer is rendered after the wizard's
 *             footer and before the first IWizardPage summary. This path must be
 *             accessible by a portlet request dispatcher. NULL implies that no
 *             footer should be rendered.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">null</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">wizardPageParam</td>
 *         <td>
 *             The request parameter/attribute that is used to determine what
 *             IWizardPage is currently being displayed using the name in the
 *             pages Map.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">WIZ_PAGE</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">prevWizardPageParam</td>
 *         <td>
 *             The request parameter/attribute that is used to determine the previous
 *             IWizardPage to display. The value of the request parameter must
 *             be a name in the pages Map.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">WIZ_PREV_PAGE</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">nextWizardPageParam</td>
 *         <td>
 *             The request parameter/attribute that is used to determine the next
 *             IWizardPage to display. The value of the request parameter must
 *             be a name in the pages Map.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">WIZ_NEXT_PAGE</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">changePrevActionParam</td>
 *         <td>
 *             If this parameter is submitted, regardless of its value, the wizard
 *             will try to change to the page specified by the <code>prevWizardPageParam</code>.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">CHANGE_PREV</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">changeNextActionParam</td>
 *         <td>
 *             If this parameter is submitted, regardless of its value, the wizard
 *             will try to change to the page specified by the <code>nextWizardPageParam</code>.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">CHANGE_NEXT</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">changeSummaryActionParam</td>
 *         <td>
 *             If this parameter is submitted, regardless of its value, the wizard
 *             will try to change to the summary page.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">CHANGE_SUMMARY</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">pageOrderAttr</td>
 *         <td>
 *             The request attribute that is used to pass the ordered list of
 *             page names to the JSPs and pages rendered by the wizard.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">WIZ_PAGE_ORDER</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">pagesAttr</td>
 *         <td>
 *             The request attribute that is used to pass the Map of page names
 *             to IWizardPages to the JSPs and pages rendered by the wizard.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">WIZ_PAGES</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">sumaryPageName</td>
 *         <td>
 *             The name of the summary sudo-IWizardPage. When this value is passed
 *             in as the next wizard page (nextWizardPageParam parameter) the summary
 *             UI is rendered instead of a configured IWizardPage.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">WIZ_SUMMARY</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">configExceptionAttr</td>
 *         <td>
 *             The request attribute that will contain the problem fields {@link Map} from
 *             the {@link ConfigurationException} thrown by the last
 *             {@link IWizardPage#processResults(ActionRequest, ActionResponse)} call
 *             if a ConfigurationException occured.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">WIZ_CONFIG_EXCP</td>
 *     </tr>
 * </table>
 * 
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class BaseWizard extends BaseActionPage {
    private static final Predicate PRED_PAGE_NAME = new AndPredicate(new InstanceofPredicate(String.class), new NotPredicate(new EqualPredicate("")));
    private static final Predicate PRED_IWIZARDPAGE_TYPE = new InstanceofPredicate(IWizardPage.class);

    
    private List pageOrder;
    private Map pages;
    
    private String headerJspPath = null;
    private String footerJspPath = null;
    private String summaryHeaderJspPath = null;
    private String summarySeperatorJspPath = null;
    private String summaryFooterJspPath = null;
    
    private String wizardPageParam = "WIZ_PAGE";
    private String prevWizardPageParam = "WIZ_PREV_PAGE";
    private String nextWizardPageParam = "WIZ_NEXT_PAGE";
    private String changePrevActionParam = "CHANGE_PREV";
    private String changeNextActionParam = "CHANGE_NEXT";
    private String changeSummaryActionParam = "CHANGE_SUMMARY";
    private String pageOrderAttr = "WIZ_PAGE_ORDER";
    private String pagesAttr = "WIZ_PAGES";
    private String configExceptionAttr = "WIZ_CONFIG_EXCP";
    
    private String sumaryPageName = "WIZ_SUMMARY";
    
    //********** Getters and Setters **********
    /**
     * @return Returns the pageOrder.
     */
    public List getPageOrder() {
        return this.pageOrder;
    }
    /**
     * @param pageOrder The pageOrder to set.
     */
    public void setPageOrder(List pageOrder) {
        if (pageOrder == null) {
            throw new IllegalArgumentException("pageOrder may not be null");
        }
        if (pageOrder.size() < 1) {
            throw new IllegalArgumentException("pageOrder property must contain at least one entry");
        }
        
        //Defensive copy
        pageOrder = new ArrayList(pageOrder);
        //Predicate for type safety
        pageOrder = PredicatedList.decorate(pageOrder, PRED_PAGE_NAME);
        //Make unmodifiable
        this.pageOrder = Collections.unmodifiableList(pageOrder);
    }

    /**
     * @return Returns the pages.
     */
    public Map getPages() {
        return this.pages;
    }
    /**
     * @param pages The pages to set.
     */
    public void setPages(Map pages) {
        if (pages == null) {
            throw new IllegalArgumentException("pages may not be null");
        }
        if (pages.size() < 1) {
            throw new IllegalArgumentException("pages property must contain at least one entry");
        }
        
        //Defensive copy
        pages = new HashMap(pages);
        //Predicate for type safety
        pages = PredicatedMap.decorate(pages, PRED_PAGE_NAME, PRED_IWIZARDPAGE_TYPE);
        //Make unmodifiable
        this.pages = Collections.unmodifiableMap(pages);
    }
    
    /**
     * @return Returns the footerJspPath.
     */
    public String getFooterJspPath() {
        return this.footerJspPath;
    }
    /**
     * @param footerJspPath The footerJspPath to set.
     */
    public void setFooterJspPath(String footerJspPath) {
        this.footerJspPath = footerJspPath;
    }
    
    /**
     * @return Returns the headerJspPath.
     */
    public String getHeaderJspPath() {
        return this.headerJspPath;
    }
    /**
     * @param headerJspPath The headerJspPath to set.
     */
    public void setHeaderJspPath(String headerJspPath) {
        this.headerJspPath = headerJspPath;
    }
    
    /**
     * @return Returns the summaryFooterJspPath.
     */
    public String getSummaryFooterJspPath() {
        return this.summaryFooterJspPath;
    }
    /**
     * @param summaryFooterJspPath The summaryFooterJspPath to set.
     */
    public void setSummaryFooterJspPath(String summaryFooterJspPath) {
        this.summaryFooterJspPath = summaryFooterJspPath;
    }
    
    /**
     * @return Returns the summaryHeaderJspPath.
     */
    public String getSummaryHeaderJspPath() {
        return this.summaryHeaderJspPath;
    }
    /**
     * @param summaryHeaderJspPath The summaryHeaderJspPath to set.
     */
    public void setSummaryHeaderJspPath(String summaryHeaderJspPath) {
        this.summaryHeaderJspPath = summaryHeaderJspPath;
    }
    
    /**
     * @return Returns the summarySeperatorJspPath.
     */
    public String getSummarySeperatorJspPath() {
        return this.summarySeperatorJspPath;
    }
    /**
     * @param summarySeperatorJspPath The summarySeperatorJspPath to set.
     */
    public void setSummarySeperatorJspPath(String summarySeperatorJspPath) {
        this.summarySeperatorJspPath = summarySeperatorJspPath;
    }
    
    /**
     * @return Returns the prevWizardPageParam.
     */
    public String getPrevWizardPageParam() {
        return this.prevWizardPageParam;
    }
    /**
     * @param prevWizardPageParam The prevWizardPageParam to set.
     */
    public void setPrevWizardPageParam(String prevWizardPageParam) {
        if (prevWizardPageParam == null) {
            throw new IllegalArgumentException("prevWizardPageParam may not be null");
        }
        prevWizardPageParam = prevWizardPageParam.trim();
        if (prevWizardPageParam.length() == 0) {
            throw new IllegalArgumentException("prevWizardPageParam must have a lenth > 0");
        }
        
        this.prevWizardPageParam = prevWizardPageParam;
    }
    
    /**
     * @return Returns the nextWizardPageParam.
     */
    public String getNextWizardPageParam() {
        return this.nextWizardPageParam;
    }
    /**
     * @param nextWizardPageParam The nextWizardPageParam to set.
     */
    public void setNextWizardPageParam(String nextWizardPageParam) {
        if (nextWizardPageParam == null) {
            throw new IllegalArgumentException("nextWizardPageParam may not be null");
        }
        nextWizardPageParam = nextWizardPageParam.trim();
        if (nextWizardPageParam.length() == 0) {
            throw new IllegalArgumentException("nextWizardPageParam must have a lenth > 0");
        }
        
        this.nextWizardPageParam = nextWizardPageParam;
    }
    
    /**
     * @return Returns the changePrevActionParam.
     */
    public String getChangePrevActionParam() {
        return this.changePrevActionParam;
    }
    /**
     * @param changePrevActionParam The changePrevActionParam to set.
     */
    public void setChangePrevActionParam(String changePrevActionParam) {
        if (changePrevActionParam == null) {
            throw new IllegalArgumentException("changePrevActionParam may not be null");
        }
        changePrevActionParam = changePrevActionParam.trim();
        if (changePrevActionParam.length() == 0) {
            throw new IllegalArgumentException("changePrevActionParam must have a lenth > 0");
        }
        
        this.changePrevActionParam = changePrevActionParam;
    }
    
    /**
     * @return Returns the changeNextActionParam.
     */
    public String getChangeNextActionParam() {
        return this.changeNextActionParam;
    }
    /**
     * @param changeNextActionParam The changeNextActionParam to set.
     */
    public void setChangeNextActionParam(String changeNextActionParam) {
        if (changeNextActionParam == null) {
            throw new IllegalArgumentException("changeNextActionParam may not be null");
        }
        changeNextActionParam = changeNextActionParam.trim();
        if (changeNextActionParam.length() == 0) {
            throw new IllegalArgumentException("changeNextActionParam must have a lenth > 0");
        }
        
        this.changeNextActionParam = changeNextActionParam;
    }
    
    /**
     * @return Returns the changeSummaryActionParam.
     */
    public String getChangeSummaryActionParam() {
        return this.changeSummaryActionParam;
    }
    /**
     * @param changeSummaryActionParam The changeSummaryActionParam to set.
     */
    public void setChangeSummaryActionParam(String changeSummaryActionParam) {
        if (changeSummaryActionParam == null) {
            throw new IllegalArgumentException("changeSummaryActionParam may not be null");
        }
        changeSummaryActionParam = changeSummaryActionParam.trim();
        if (changeSummaryActionParam.length() == 0) {
            throw new IllegalArgumentException("changeSummaryActionParam must have a lenth > 0");
        }
        
        this.changeSummaryActionParam = changeSummaryActionParam;
    }
    
    /**
     * @return Returns the wizardPageParam.
     */
    public String getWizardPageParam() {
        return this.wizardPageParam;
    }
    /**
     * @param wizardPageParam The wizardPageParam to set.
     */
    public void setWizardPageParam(String wizardPageParam) {
        if (wizardPageParam == null) {
            throw new IllegalArgumentException("wizardPageParam may not be null");
        }
        wizardPageParam = wizardPageParam.trim();
        if (wizardPageParam.length() == 0) {
            throw new IllegalArgumentException("wizardPageParam must have a lenth > 0");
        }

        this.wizardPageParam = wizardPageParam;
    }
    
    /**
     * @return Returns the pageOrderAttr.
     */
    public String getPageOrderAttr() {
        return this.pageOrderAttr;
    }
    /**
     * @param pageOrderAttr The pageOrderAttr to set.
     */
    public void setPageOrderAttr(String pageOrderAttr) {
        if (pageOrderAttr == null) {
            throw new IllegalArgumentException("pageOrderAttr may not be null");
        }
        pageOrderAttr = pageOrderAttr.trim();
        if (pageOrderAttr.length() == 0) {
            throw new IllegalArgumentException("pageOrderAttr must have a lenth > 0");
        }

        this.pageOrderAttr = pageOrderAttr;
    }
    
    /**
     * @return Returns the pagesAttr.
     */
    public String getPagesAttr() {
        return this.pagesAttr;
    }
    /**
     * @param pagesAttr The pagesAttr to set.
     */
    public void setPagesAttr(String pagesAttr) {
        if (pagesAttr == null) {
            throw new IllegalArgumentException("pagesAttr may not be null");
        }
        pagesAttr = pagesAttr.trim();
        if (pagesAttr.length() == 0) {
            throw new IllegalArgumentException("pagesAttr must have a lenth > 0");
        }

        this.pagesAttr = pagesAttr;
    }
    
    /**
     * @return Returns the configExceptionAttr.
     */
    public String getConfigExceptionAttr() {
        return this.configExceptionAttr;
    }
    /**
     * @param configExceptionAttr The configExceptionAttr to set.
     */
    public void setConfigExceptionAttr(String configExceptionAttr) {
        if (configExceptionAttr == null) {
            throw new IllegalArgumentException("configExceptionAttr may not be null");
        }
        configExceptionAttr = configExceptionAttr.trim();
        if (configExceptionAttr.length() == 0) {
            throw new IllegalArgumentException("configExceptionAttr must have a lenth > 0");
        }

        this.configExceptionAttr = configExceptionAttr;
    }
    
    /**
     * @return Returns the sumaryPageName.
     */
    public String getSumaryPageName() {
        return this.sumaryPageName;
    }
    /**
     * @param sumaryPageName The sumaryPageName to set.
     */
    public void setSumaryPageName(String sumaryPageName) {
        this.sumaryPageName = sumaryPageName;
    }
    
    
    //********** InitializingBean Methods **********
    public void afterPropertiesSet() throws Exception {
        if (this.pages == null) {
            throw new BeanInitializationException("pages may not be null");
        }
        if (this.pageOrder == null) {
            throw new BeanInitializationException("pageOrder may not be null");
        }
        
        super.afterPropertiesSet();
    }    
    
    
    //********** IPage Methods **********
    
    public void renderPage(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        String currentPageName = this.resolveCurrentPageName(request);
        
        final IWizardPage currentPage;
        if (this.sumaryPageName.equals(currentPageName)) {
            currentPage = null;
        }
        else {
            currentPage = this.getPage(currentPageName);
            
            if (currentPageName == null) {
                currentPageName = this.getPageName(currentPage);
            }
        }
        
        final PortletSession session = request.getPortletSession(true);
        final Map problemFields = (Map)session.getAttribute(this.configExceptionAttr);
        if (problemFields != null) {
            session.removeAttribute(this.configExceptionAttr);
            request.setAttribute(this.configExceptionAttr, problemFields);
        }

        request.setAttribute(this.pageOrderAttr, this.pageOrder);
        request.setAttribute(this.pagesAttr, this.pages);
        request.setAttribute(this.wizardPageParam, currentPageName);

        if (this.headerJspPath != null) {
            RenderUtils.dispatch(request, response, this.headerJspPath);
        }

        if (currentPage != null) {
            currentPage.renderPage(request, response);            
        }
        else {
            //Assume summary page is currentPage is null

            if (this.summaryHeaderJspPath != null) {
                RenderUtils.dispatch(request, response, this.summaryHeaderJspPath);
            }
            
            for (final Iterator pageNameItr = this.pageOrder.iterator(); pageNameItr.hasNext();) {
                final String pageName = (String)pageNameItr.next();
                final IWizardPage page = (IWizardPage)this.pages.get(pageName);
                page.renderSummary(request, response);
                
                if (this.summarySeperatorJspPath != null && pageNameItr.hasNext()) {
                    RenderUtils.dispatch(request, response, this.summarySeperatorJspPath);
                }
            }

            if (this.summaryFooterJspPath != null) {
                RenderUtils.dispatch(request, response, this.summaryFooterJspPath);
            }
        }
        
        if (this.footerJspPath != null) {
            RenderUtils.dispatch(request, response, this.footerJspPath);
        }
    }
    
    public void processResults(ActionRequest request, ActionResponse response) throws PortletException, IOException {
        final String currentPageName = this.resolveCurrentPageName(request);
        String nextPageName = this.resolveNextPageName(request);
        
        if (!this.sumaryPageName.equals(currentPageName)) {
            final IWizardPage currentPage = this.getPage(currentPageName);
            
            try {
                currentPage.processResults(request, response);
            }
            catch (ConfigurationException ce) {
                final PortletSession session = request.getPortletSession();
                session.setAttribute(this.configExceptionAttr, ce.getProblemFields());
                nextPageName = currentPageName;
            }
        }

        if (this.sumaryPageName.equals(nextPageName)) {
            //Check to see if all the pages are ready for the summary
            for (final Iterator pageNameItr = this.pageOrder.iterator(); pageNameItr.hasNext();) {
                final String pageName = (String)pageNameItr.next();
                final IWizardPage page = (IWizardPage)this.pages.get(pageName);
                final Map errors  = page.isCompleted(request, response);
                
                if (errors.size() > 0) {
                    final PortletSession session = request.getPortletSession();
                    session.setAttribute(this.configExceptionAttr, errors);
                    nextPageName = pageName;
                    
                    break;
                }
            }
        }

        if (nextPageName != null) {
            response.setRenderParameter(this.wizardPageParam, nextPageName);
        }
        else {
            response.setRenderParameter(this.wizardPageParam, currentPageName);
        }
    }



    //********** Utilitiy Methods **********

    /**
     * Resolves the current page name from the request. Uses the 
     * wizardPageParam property. Checks to ensure the name is valid
     * before returning.
     * 
     * @param request The request to get the current page from.
     * @return The name of the current page, null if one was not specified.
     */
    protected String resolveCurrentPageName(PortletRequest request) {
        return this.resolvePageNameForKey(request, this.wizardPageParam);
    }
    
    /**
     * Resolves the current page name from the request. Checks to ensure
     * the name is valid before returning.
     * 
     * @param request The request to get the next page from.
     * @return The name of the current page, null if one was not specified.
     */
    protected String resolveNextPageName(PortletRequest request) {
        final Map paramMap = request.getParameterMap();
        if (paramMap.containsKey(this.changePrevActionParam)) {
            return this.resolvePageNameForKey(request, this.prevWizardPageParam);
        }
        else if (paramMap.containsKey(this.changeNextActionParam)) {
            return this.resolvePageNameForKey(request, this.nextWizardPageParam);
        }
        else if (paramMap.containsKey(this.changeSummaryActionParam)) {
            return this.sumaryPageName;
        }
        else {
            return null;
        }
    }
    
    /**
     * @return The default name defined by the first name in the pageOrder List.
     */
    protected String getDefaultPageName() {
        return (String)this.pageOrder.get(0);
    }
    
    /**
     * @return The default page defined by {@link #getDefaultPageName()}
     */
    protected IWizardPage getDefaultPage() {
        final String defPageName = this.getDefaultPageName();
        return (IWizardPage)this.pages.get(defPageName);
    }
    
    /**
     * Returns the page for the specified name or the default name if
     * the specified page is not found.
     * 
     * @param pageName The name of the page to retrieve.
     * @return The named page or the default page if the pageName is null or does not have a corresponding page.
     */
    protected IWizardPage getPage(String pageName) {
        if (pageName == null) {
            return this.getDefaultPage();
        }
        
        IWizardPage page = (IWizardPage)this.pages.get(pageName);
            
        if (page == null) {
            page = this.getDefaultPage();
        }
        
        return page;
    }
    
    /**
     * Returns the page name for the page. Warning, this method
     * is O(N) where N is the number of pages. Null is returned
     * if the page is not configured in this wizard.
     * 
     * @param page The page to find a name for, may not be null.
     * @return The name of the specified page, null if not found.
     */
    protected String getPageName(IWizardPage page) {
        if (page == null) {
            throw new IllegalArgumentException("page may not be null");
        }
        
        for (final Iterator entryItr = this.pages.entrySet().iterator(); entryItr.hasNext();) {
            final Map.Entry entry = (Map.Entry)entryItr.next();
            if (page.equals(entry.getValue())) {
                return (String)entry.getKey();
            }
        }
        
        return null;
    }
    
    private String resolvePageNameForKey(PortletRequest request, String key) {
        final String currentPageName = StringUtils.trimToNull(request.getParameter(key));
        
        if (currentPageName != null) {
            if (this.sumaryPageName.equals(currentPageName) || this.pages.containsKey(currentPageName)) {
                return currentPageName;
            }
            else {
                throw new IllegalArgumentException("'" + currentPageName + "' is an unknown wizard page, specified by the '" + key + "' parameter.");
            }
        }
    
        return currentPageName;
    }
}
