/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlets.utils;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.portlet.PortletException;

import org.apache.commons.collections.Predicate;
import org.apache.commons.collections.functors.InstanceofPredicate;
import org.apache.commons.collections.map.PredicatedMap;

/**
 * Represents a problem configuring an object from data submitted
 * to a {@link org.jasig.portal.portlets.utils.IWizardPage}.
 * The Map returned by {@link #getProblemFields()} contains a map of
 * field names to error messages for that field. 
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class ConfigurationException extends PortletException {
    private static final Predicate PRED_STRING_TYPE = new InstanceofPredicate(String.class);

    private final Map problemFields;

    /**
     * Create a new ConfigurationException with the specified problem fields.
     * 
     * @param problemFields Map of field names to error messages.
     */
    public ConfigurationException(Map problemFields) {
        this(problemFields, null);
    }
    
    /**
     * Create a new ConfigurationException with the specified problem fields.
     * 
     * @param problemFields Map of field names to error messages.
     * @param cause The cause of this exception.
     */
    public ConfigurationException(Map problemFields, Throwable cause) {
        super(cause);
        
        problemFields = new HashMap(problemFields);
        problemFields = PredicatedMap.decorate(problemFields, PRED_STRING_TYPE, PRED_STRING_TYPE);
        this.problemFields = Collections.unmodifiableMap(problemFields); 
    }
    
    /**
     * Create a new ConfigurationException with a single problem field.
     * 
     * @param fieldName The name of the problem field.
     * @param errorMessage The error message for that field.
     * @param cause The cause of this exception.
     */
    public ConfigurationException(String fieldName, String errorMessage, Throwable cause) {
        super(cause);

        final Map problemFields = PredicatedMap.decorate(new HashMap(), PRED_STRING_TYPE, PRED_STRING_TYPE);
        problemFields.put(fieldName, errorMessage);
        this.problemFields = Collections.unmodifiableMap(problemFields); 
    }
    
    /**
     * Create a new ConfigurationException with a single problem field.
     * 
     * @param fieldName The name of the problem field.
     * @param errorMessage The error message for that field.
     */
    public ConfigurationException(String fieldName, String errorMessage) {
        this(fieldName, errorMessage, null);
    }

    /**
     * @return Map of field names to error messages for that field.
     */
    public Map getProblemFields() {
        return this.problemFields;
    }
}
