/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlets.xslt;

import java.io.IOException;
import java.net.URI;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.core.exception.PortalRuntimeException;
import org.jasig.portal.portlet.url.impl.PortletUrlXalanElements;
import org.jasig.portal.rendering.sax.xslt.ITransformerHelper;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 *  A generic XSLT portlet implementation.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class GenericXsltBean extends GenericPortlet {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(GenericXsltBean.class);

    protected ITransformerHelper transformerHelper;
    protected ISimpleCacheProviderFacade documentCache;
    
    protected final static String CONTEXT_PATH_PARAM_NAME="contextPath";
    
    public void processAction(ActionRequest req, ActionResponse res) throws PortletException, IOException {
        // record params into the session
        PortletSession session=req.getPortletSession();
        for(Enumeration e=req.getParameterNames();e.hasMoreElements();) {
            String paramName=(String) e.nextElement();
            session.setAttribute(paramName,req.getParameter(paramName));
        }
    }
    
    protected void processXslt ( URI xslUri, Source xmlSource,  Map xslParams, RenderRequest request, RenderResponse response ) throws PortletException, IOException {
        try {
            Transformer transformer=transformerHelper.getTransformer(xslUri);
            // add render response to the transformer parameters to enable extension element url construction
            transformer.setParameter(PortletUrlXalanElements.RENDER_RESPONSE_PARAMETER_NAME,response);
            
            // set context path param
            transformer.setParameter(CONTEXT_PATH_PARAM_NAME,request.getContextPath());
            
            // TODO: add support for stylesheet sets. Among other things, the content type would be determined from them.
            response.setContentType("text/html");
            StreamResult streamResult=new StreamResult(response.getWriter());
            
            if ( xslParams != null ) {
                for ( Iterator names = xslParams.keySet().iterator(); names.hasNext(); ) {
                 String paramName = (String) names.next();
                 transformer.setParameter(paramName,xslParams.get(paramName));
                }
            }
            
            try {
                transformer.transform(xmlSource,streamResult);
            } catch (TransformerException e) {
                throw new PortletException("unable to transform",e);
            }
        } catch (PortalException e) {
            throw new PortletException("unable to obtain transformer for xmlSource="+xmlSource!=null?xmlSource.toString():null,e);
        }
        
    }

    public void render (RenderRequest req, RenderResponse res) throws PortletException, IOException {
        Map xslParams = new HashMap();
        // forward session parameters (those set by action request)
        PortletSession session=req.getPortletSession(false);
        if ( session != null ) {
            for(Enumeration e=session.getAttributeNames();e.hasMoreElements();) {
                String paramName=(String) e.nextElement();
                xslParams.put(paramName,session.getAttribute(paramName));
            }
        }
        
        //  forward render request parameters
        for(Enumeration e=req.getParameterNames();e.hasMoreElements();) {
            String paramName=(String) e.nextElement();
            xslParams.put(paramName,req.getParameter(paramName));
        }
       
        processXslt(getXslUri(req),new DOMSource(getXmlDoc(req)),xslParams,req,res);
    }
    
    protected URI getXmlUri(RenderRequest req) {
        return getFileUri("xmlUri",req);
    }
    
    protected URI getXslUri(RenderRequest req) {
        return getFileUri("xslUri",req);
    }

    /**
     * Obtain and parse XML document, using cache if possible
     * @param req current http request
     * @return a parsed xml document
     */
    protected Document getXmlDoc(RenderRequest req) {
       Document doc=null;
       URI uri=getXmlUri(req);
       if(documentCache!=null) {
           doc=(Document) documentCache.getFromCache(uri.toString());
       }
       if(doc==null) {
           // retrieve and parse
           DocumentBuilder documentBuilder;
        try {
            documentBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            doc=documentBuilder.parse(uri.toString());
            if(documentCache!=null) {
                documentCache.putInCache(uri.toString(),doc);
            }
        } catch (ParserConfigurationException e) {
            throw new PortalRuntimeException("unable to obtain XML parser",e);
        } catch (FactoryConfigurationError e) {
            throw new PortalRuntimeException("invalid XML builder factory configuration",e);
        } catch (SAXException e) {
            throw new PortalRuntimeException("unable to parse XML document uri=\""+uri.toString()+"\"",e);
        } catch (IOException e) {
            throw new PortalRuntimeException("unable to parse XML document uri=\""+uri.toString()+"\"",e);
        }
       }
       return doc;
    }

    protected URI getFileUri(String parameterName, RenderRequest req) {
        URI uri;
        String str_uri = req.getPreferences().getValue(parameterName, null);
        try {
            // see if this is a well-formed URL
            uri = URI.create(str_uri);
            if (!uri.isAbsolute()) {
                uri = URI.create(req.getPortletSession()
                        .getPortletContext()
                        .getResource(str_uri)
                        .toString());
            }
        }
        catch (Exception any) {
            final RuntimeException e = new RuntimeException("malformed " + parameterName + " value specified : str_xslUri=" + str_uri, any);
            logger.error(e.getMessage(), e);
            throw e;
        }
        return uri;
    }

    
    public void setTransformerHelper(ITransformerHelper transformerHelper) {
        this.transformerHelper = transformerHelper;
    }

    public void setDocumentCache(ISimpleCacheProviderFacade documentCache) {
        this.documentCache = documentCache;
    }

}
