/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.xslt.common;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletMode;
import javax.portlet.PortletURL;
import javax.portlet.RenderResponse;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.xml.serialize.XHTMLSerializer;
import org.w3c.dom.Document;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 * The generic class for portlets that use XSLT. 
 * @author <a href="mailto:mvi@immagic.com">Michael Ivanov</a>
 * @version $Revision: 11911 $
 */
public class XsltPortlet extends GenericPortlet {
	
  public static final String PORTLET_PARAM = "portlet-param";
  public static final String ACTION_URL = "actionURL";

  private static TransformerFactory transformerFactory;
  protected static DocumentBuilder domBuilder;
  private static Map templatesMap;
  static {
    try {
    templatesMap = new Hashtable();
   	transformerFactory = TransformerFactory.newInstance();
   	domBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
    } catch ( Exception e ) {
  	  throw new RuntimeException(e.toString());
    }
   }

  public XsltPortlet() {
    super();
  }

   private static class SAXElement {

	 	public static final String START_MODE = "start";
	 	public static final String END_MODE = "end";
	 	public static final String DATA_MODE = "data";

	 	private String mode;
	 	private String uri;
	 	private String localName;
	 	private String qName;
	 	private Attributes atts;

	 	// characters data
	 	private String data;

	 	protected SAXElement ( String mode, String uri, String localName, String qName, Attributes atts ) {
	 		this.mode = mode;
	 		this.localName = localName;
	 		this.qName = qName;
	 		this.atts = atts;
	 		data = null;
	 	}

	 	protected SAXElement ( String data ) {
	 		this.mode = DATA_MODE;
	 		this.data = data;
	 		localName = null;
	 		qName = null;
	 		atts = null;
	 	}

	 	public static SAXElement createStartElement ( String uri, String localName, String qName, Attributes atts ) {
	 	   return new SAXElement(START_MODE,uri,localName,qName,atts);
	 	}

	 	public static SAXElement createEndElement ( String uri, String localName, String qName ) {
		   return new SAXElement(END_MODE,uri,localName,qName,null);
		}

	 	public static SAXElement createDataElement ( String data ) {
		   return new SAXElement(data);
	    }

	 	public String getMode() {
	 		return mode;
	 	}

	 	public String getUri() {
	 		return uri;
	 	}

	 	public String getLocalName() {
	 		return localName;
	 	}

	 	public String getQName() {
	 		return qName;
	 	}

	 	public Attributes getAttributes() {
	 		return atts;
	 	}

	 	public String toString() {

	 	  if ( data != null )
	 	  	return data;

	 	  if ( mode.equals(START_MODE) )
	 	  	return "<" + qName + ">";
	 	  else if ( mode.equals(END_MODE) )
	 	  	return "<" + qName + "/>";

	 	  return null;
	 	}
   }

  private class PortletContentSerializer extends XHTMLSerializer {

  	 private RenderResponse response;
  	 private PortletURL actionUrl;
  	 private Map urlParams;
  	 // Link mode
  	 private boolean actionMode;
  	 private AttributesImpl actionAtts;
  	 private List elements;


	 public PortletContentSerializer(RenderResponse response, Map urlParams ) throws IOException {
		super(response.getWriter(),null);
		this.response = response;
		this.urlParams = urlParams;
		actionMode = false;
		elements = new ArrayList();
	 }

	 private PortletURL createActionURL() {
	   PortletURL actionUrl = response.createActionURL();
	   if ( urlParams != null && !urlParams.isEmpty() ) {
	   	for ( Iterator params = urlParams.keySet().iterator(); params.hasNext(); ) {
	   		String paramName = (String) params.next();
	   		String paramValue = (String) urlParams.get(paramName);
	   		if ( paramName != null && paramValue != null )
	   		 actionUrl.setParameter(paramName,paramValue);
	   	}
	   }
	      return actionUrl;
	 }

	 public void startElement (String uri, String localName, String qName,  Attributes atts) throws SAXException {

	 	boolean actionLink = qName.equalsIgnoreCase("a") && ACTION_URL.equals(atts.getValue("href"));
	 	if ( actionLink ) {
		  actionAtts = new AttributesImpl(atts);
		  actionMode = true;
		} else if ( qName.equalsIgnoreCase("form") && ACTION_URL.equals(atts.getValue("action")) ) {
		   AttributesImpl attsImpl = new AttributesImpl(atts);
		   attsImpl.removeAttribute(attsImpl.getIndex("action"));
		   attsImpl.addAttribute(uri,"action","action","CDATA",createActionURL().toString());
	 	   super.startElement(uri,localName,qName,attsImpl);
		} else if ( qName.equalsIgnoreCase(PORTLET_PARAM) ) {
		   if ( actionMode ) {
		    if ( actionUrl == null )
		      actionUrl = createActionURL();
		    actionUrl.setParameter(atts.getValue("name"),atts.getValue("value"));
		   }
		} else if ( actionMode )
			elements.add(SAXElement.createStartElement(uri,localName,qName,new AttributesImpl(atts)));
		  else
			super.startElement(uri,localName,qName,atts);
	 }

	 public void characters (char c[], int start, int length) throws SAXException   {

        if ( actionMode ) {
        	elements.add(SAXElement.createDataElement(new String(c,start,length)));
        } else {
            super.characters(c,start,length);
        }
     }

	 public void endElement (String uri, String localName, String qName) throws SAXException {

	  	if ( actionMode && qName.equalsIgnoreCase("a") ) {
	 		actionAtts.removeAttribute(actionAtts.getIndex("href"));
	 	 	actionAtts.addAttribute(uri,"href","href","CDATA",(actionUrl!=null)?actionUrl.toString():"");
	 		super.startElement(uri,localName,qName,actionAtts);
	 		for ( int i = 0; i < elements.size(); i++ ) {
	 		  SAXElement element = (SAXElement) elements.get(i);
	 		  if ( SAXElement.START_MODE.equals(element.getMode()) )
	 		  	super.startElement(element.getUri(),element.getLocalName(),element.getQName(),element.getAttributes());
	 		  else if ( SAXElement.DATA_MODE.equals(element.getMode()) ) {
	 		  	char[] chars = element.toString().toCharArray();
	 		  	super.characters(chars,0,chars.length);
	 		  } else if ( SAXElement.END_MODE.equals(element.getMode()) )
	 		  	super.endElement(element.getUri(),element.getLocalName(),element.getQName());
	 		}
	 		super.endElement(uri,localName,qName);
            actionUrl = null;
            actionMode = false;
            elements.clear();
	    } else if ( !qName.equalsIgnoreCase(PORTLET_PARAM) ) {
	  	    if ( actionMode )
	  		  elements.add(SAXElement.createEndElement(uri,localName,qName));
	  	    else
	 		  super.endElement(uri,localName,qName);
	      }

	 }

  }

  protected void processXslt ( String xslUrl, Document xml,  RenderResponse response, Map xslParams, Map urlParams ) throws Exception {
      
  	response.setContentType("text/html");
    
    Templates templates = (Templates) templatesMap.get(xslUrl);
    if ( templates == null ) {
      StreamSource xsltSource = new StreamSource(getPortletContext().getResourceAsStream(xslUrl));
      templates = transformerFactory.newTemplates(xsltSource);
      templatesMap.put(xslUrl,templates);
    }

    Transformer transformer = templates.newTransformer();

    // Adding parameters
    if ( xslParams != null ) {
     for ( Iterator names = xslParams.keySet().iterator(); names.hasNext(); ) {
      String paramName = (String) names.next();
      transformer.setParameter(paramName,xslParams.get(paramName));
     }
    }

    //if (urlParams != null) {
      PortletContentSerializer filter = new PortletContentSerializer(response,urlParams);
      SAXResult result = new SAXResult(filter);
      transformer.transform(new DOMSource(xml), result);
    /*} else {
      PrintWriter writer = response.getWriter();
      transformer.transform( new DOMSource(xml), new StreamResult(writer));
    }*/
  }

  protected void processXslt ( String xslUrl, Document xml,  RenderResponse response, Map xslParams ) throws Exception {
    processXslt( xslUrl, xml, response, xslParams, null);
  }
  
  // The generic processAction() method
  public void processAction (ActionRequest request, ActionResponse response) throws PortletException, IOException {

  	String mode = request.getParameter("mode");
 	PortletMode currentMode = ( mode != null ) ? new PortletMode(mode) : request.getPortletMode();
 	
  	if ( PortletMode.VIEW.equals(currentMode) ) {
        processViewAction(request,response);
    } else if ( PortletMode.EDIT.equals(currentMode) ) {
    	processEditAction(request,response);
    } else if ( PortletMode.HELP.equals(currentMode) ) {
    	processHelpAction(request,response);
    }

  }
  
  // Helper-methods for performing actions in different portlet modes
  protected void processViewAction (ActionRequest request, ActionResponse response) throws PortletException, IOException { }
  protected void processEditAction (ActionRequest request, ActionResponse response) throws PortletException, IOException { }
  protected void processHelpAction (ActionRequest request, ActionResponse response) throws PortletException, IOException { }
  
  
}