/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.problem;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.lang.NullArgumentException;

/**
 * A ProblemsManaager implemented to be a FIFO cache of recently reported Throwables.
 * @author andrew.petro@yale.edu
 * $Revision: 11917 $ $Date: 2007-04-02 07:18:26 -0500 (Mon, 02 Apr 2007) $ 
 */
public class RecentProblemsManager implements IProblemsManager {

    private static final int DEFAULT_SIZE = 100;
    
    /**
     * The number of Throwables that have been reported to me.
     */
    private long count = 0;
    
    /**
     * Internal cache of recent Throwables.
     */
    private LinkedList recentErrors = new LinkedList();
    
    /**
     * The number of recent Throwables to remember.
     */
    private int size = DEFAULT_SIZE;
    
    /* (non-Javadoc)
     * @see org.jasig.portal.exception.problemsmanager.IProblemsManager#reportThrowable(java.lang.Throwable)
     */
    public synchronized void reportThrowable(Throwable throwable) {
        if (throwable == null)
            throw new NullArgumentException("throwable");
        // synchronized to avoid concurrent modification exceptions.
        this.recentErrors.addFirst(throwable);
        this.count++;
        while (this.recentErrors.size() > this.size) {
            this.recentErrors.removeLast();
        }
    }
    
    /**
     * Set the number of throwables to remember.
     * Reducing the size below the number of currently stored
     * Throwables will take effect upon next reportThrowable() call.
     * @param desiredSize - the number of recent Throwables to remember.
     */
    public void setSize(int desiredSize){
        if (desiredSize < 1) {
            throw new IllegalArgumentException("desiredSize Must be greater than zero.");
        }
        this.size = desiredSize;
    }
    
    /**
     * Get the number of recent Throwables this manager
     * will remember.
     * @return the number of Throwables I will remember.
     */
    public int getSize() {
        return this.size;
    }

    /**
     * Get an unmodifiable List of recently
     * reported Throwables.
     * List is in order from most recently reported back to
     * oldest remembered Throwable.
     * @return an unmodifiable List of recent Throwables.
     */
    public List getRecentThrowables() {
        List shallowCopy = new ArrayList();
            synchronized(this){
                shallowCopy = (List) this.recentErrors.clone();
            }
        
        return Collections.unmodifiableList(shallowCopy);   
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.exception.problemsmanager.IProblemsManager#getCount()
     */
    public long getCount() {
        return this.count;
    }
    
}
