/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering;

import java.util.ArrayList;
import java.util.List;

import org.jasig.portal.caching.CacheKeyListImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheKeyList;
import org.jasig.portal.caching.ICacheable;

/**
 * A base helper class to help pipelines implement caching.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public abstract class PipelineCacheManagerImpl implements IPipelineCacheManager {

    /**
     * Checks if a pipeline element at a given position is a properly configured
     * cache filter, and if so, creates a {@link AbstractCacheEntryPoint} appropriate for
     * this filter.
     * @param pipelineElements rendering pipeline elements
     * @param pipelinePosition position in the pipeline to check
     * @return an array of cache entry point corresponding to a current position
     */
    public abstract AbstractCacheEntryPoint[] checkAndCreateCacheEntryPoints(List pipelineElements, int pipelinePosition);
    
    /**
     * Determine optimal cache entry point for the current rendering cycle.
     * The method will start from the beginning of a pipeline, finding all
     * {@link ICacheFilter}s, setting current cache keys and validating cache
     * entries these filters may report. The method will pick an optimal cache
     * entry point.
     * @param pipelineElements
     * @return a valid {@link AbstractCacheEntryPoint} or <code>null</code>.
     */
    public AbstractCacheEntryPoint determineOptimalCacheEntryPoint(List pipelineElements) {
        List currentCacheKeys=new ArrayList();
        List cacheEntryPoints=new ArrayList();

    	for(int pipelinePosition=0; pipelinePosition<pipelineElements.size(); pipelinePosition++) {
        	Object filter=pipelineElements.get(pipelinePosition);
        	ICacheKey key=null;
        	if(filter instanceof ICacheable) {
        	    key=((ICacheable)filter).generateKey();
        	}
        	
        	if(key!=null) {        	    
        	    if(filter instanceof ICacheFilter) {
            	    ICacheFilter cacheFilter=(ICacheFilter)filter;
            	    ICacheKeyList stageKey=new CacheKeyListImpl(currentCacheKeys);
            	    cacheFilter.setCurrentCacheKey(stageKey);
            	    AbstractCacheEntryPoint[] entryPoints=this.checkAndCreateCacheEntryPoints(pipelineElements,pipelinePosition);
            	    if(entryPoints!=null && entryPoints.length>0) {
                        for (int i = 0; i < entryPoints.length; i++) {
                            cacheEntryPoints.add(entryPoints[i]);
                        }
            	    }
            	}
        	    currentCacheKeys.add(key);
        	} else {
        	    // we reached a point in pipeline at which component
        	    // is not cacheable. There can be no valid cache entry
        	    // points after this pipeline position.
        	    break;
        	}        	
        }
    	
    	// validate cache entry points
    	// validation is performed in reverse order
    	for(int cacheEntryIndex=cacheEntryPoints.size()-1; cacheEntryIndex>=0; cacheEntryIndex--) {
    	    AbstractCacheEntryPoint entryPoint=(AbstractCacheEntryPoint) cacheEntryPoints.get(cacheEntryIndex);
    	    if(entryPoint.isValid(pipelineElements)) {
    	        return entryPoint;
    	    }
    	}
    	// if no valid entry point has been found, return null
        return null;
    }
}
