/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.jasig.portal.core.exception.PortalException;

/**
 * An abstract implementation of the rendering component manager for linear rendering pipelines.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class PipelineComponentManager implements IRenderingComponentManager, IRenderingPipelineEventListener {
    IPortalRenderer firstPipelineElement;
    List downstreamElements;
    List pipelineElements;
    
    Set pipelineEventListeners=Collections.synchronizedSet(new HashSet());;
    
    IPipelineCacheManager cacheManager=null;
    
    public IPortalRenderer getRootElement() {
        return firstPipelineElement;
    }

    public Iterator getElementsIterator() {
        return getPipelineElements().iterator();
    }

    public ICacheEntryPoint determineOptimalCacheEntryPoint() {
        if(cacheManager!=null) {
            return cacheManager.determineOptimalCacheEntryPoint(getPipelineElements());
        } else {
            return null;
        }
    }
    
    /**
     * Returns a list of all pipeline elements, including the first and the last component, in order. 
     * @return List of all pipeline lements
     */
    protected List getPipelineElements() {
        if(pipelineElements==null) { // construct pipeline elements
            pipelineElements=new ArrayList();
            pipelineElements.add(getRootElement());
            pipelineElements.addAll(getDownstreamElements());
        }
        return pipelineElements;
    }
    
    /**
     * Used to reset internal pipeline elements list
     * when pipeline is modified.
     */
    private void clearPipelineElements() {
        pipelineElements=null;
    }

    /**
     * @return Returns the cacheManager.
     */
    public IPipelineCacheManager getCacheManager() {
        return cacheManager;
    }
    /**
     * @return Returns the downstreamElements.
     */
    protected List getDownstreamElements() {
        return downstreamElements;
    }
    /**
     * @param downstreamElements The downstreamElements to set.
     */
    protected void setDownstreamElements(List downstreamElements) {
        if(this.downstreamElements!=null) {
            for (Iterator iter = this.downstreamElements.iterator(); iter.hasNext();) {
                pipelineEventListeners.remove(iter.next());
            }
        }
        
        this.downstreamElements = downstreamElements;
        
        for (Iterator iter = downstreamElements.iterator(); iter.hasNext();) {
            Object element = iter.next();
            if (element instanceof IRenderingPipelineEventListener) {
                pipelineEventListeners.add((IRenderingPipelineEventListener) element);   
            }
            
        }
        
        clearPipelineElements();
    }
    /**
     * @param firstElement The firstPipelineElement to set.
     */
    protected void setFirstPipelineElement(IPortalRenderer firstElement) {
        if(this.firstPipelineElement!=null) {
            pipelineEventListeners.remove(this.firstPipelineElement);
        }
        
        this.firstPipelineElement = firstElement;
        
        if(firstElement instanceof IRenderingPipelineEventListener) {
            pipelineEventListeners.add(firstElement);
        }
        clearPipelineElements();
    }
    /**
     * @param cacheManager The cacheManager to set.
     */
    public void setCacheManager(IPipelineCacheManager cacheManager) {
        this.cacheManager = cacheManager;
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.IRenderingPipelineEventListener#reveiveSAXPipelineEvent(org.jasig.portal.rendering.RenderingPipelineEvent)
     */
    public void receiveSAXPipelineEvent(RenderingPipelineEvent ev) throws PortalException {
        for (Iterator iter = pipelineEventListeners.iterator(); iter.hasNext();) {
            IRenderingPipelineEventListener element = (IRenderingPipelineEventListener) iter.next();
            element.receiveSAXPipelineEvent(ev);
        }
    }
}
