/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering;

import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jasig.portal.core.exception.PortalException;

import java.io.Serializable;

/**
 * A tree (really any structure with a root) renderer with caching support.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class TreeRenderer implements IPortalRenderer, Serializable {
    IRenderingComponentManager componentManager;
    Set listeners=Collections.synchronizedSet(new HashSet());
    boolean synchronousRendering = false;

    public void render(HttpServletRequest req, HttpServletResponse res) throws PortalException {
    	if (synchronousRendering) {
    		synchronized (this) {
    			renderingCycle(req, res);
    		}
    	} else {
			renderingCycle(req, res);
    	}
    }
    
    private void renderingCycle(HttpServletRequest req, HttpServletResponse res) throws PortalException {
        try {
            // signal rendering start
            distributeEvent(new RenderingPipelineEvent(this,RenderingPipelineEvent.startRenderingCycleEventType,req,res));
            // determine optimal cache entry point
            ICacheEntryPoint optimalCacheEntryPoint=componentManager.determineOptimalCacheEntryPoint();
            
            if(optimalCacheEntryPoint!=null) {
                // replay cache
                if(!optimalCacheEntryPoint.replayCache()) {
                    componentManager.getRootElement().render(req,res);
                }
            } else {
                componentManager.getRootElement().render(req,res);  
            }

            // signal rendering end
            distributeEvent(new RenderingPipelineEvent(this,RenderingPipelineEvent.endRenderingCycleEventType,req,res));
        } catch (PortalException pe) {
            throw pe;
        } catch (Exception e) {
            e.printStackTrace();
            throw new PortalException(e);
        }
    }
    
    public void addRenderingPipelineEventListener(IRenderingPipelineEventListener listener) {
        listeners.add(listener);
    }
    
    public void removeRenderingPipelineEventListener(IRenderingPipelineEventListener listener) {
        listeners.remove(listener);
    }

    /**
     * Distribute pipeline event
     * @param event the event object
     */
    void distributeEvent(RenderingPipelineEvent event) throws PortalException {
        for(Iterator i=this.listeners.iterator();i.hasNext();) {
            ((IRenderingPipelineEventListener)i.next()).receiveSAXPipelineEvent(event);
        }
    }

    /**
     * @return the componentManager.
     */
    public IRenderingComponentManager getComponentManager() {
        return componentManager;
    }

    /**
     * @param componentManager the componentManager to set.
     */
    public void setComponentManager(IRenderingComponentManager componentManager) {
        this.componentManager = componentManager;
        if(componentManager instanceof IRenderingPipelineEventListener) {
            addRenderingPipelineEventListener((IRenderingPipelineEventListener)componentManager);
        }
    }

    /**
     * @return Whether the rendering methods are synchronous
     */
	public boolean isSynchronousRendering() {
		return synchronousRendering;
	}
	
	/**
	 * Sets whether the rendering methods are synchronous.
	 * @param synchronousRendering
	 */
	public void setSynchronousRendering(boolean synchronousRendering) {
		this.synchronousRendering = synchronousRendering;
	}
    
    
}
