/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Map;

import javax.xml.transform.Transformer;

import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.rendering.sax.SaxFilterImpl;
import org.jasig.portal.rendering.sax.xslt.IRenderingAttributeInjector;
import org.jasig.portal.rendering.sax.xslt.ITransformerPreferences;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 * Injects rendering attributes as child elements.  
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class ElementRenderingAttributeInjector extends SaxFilterImpl implements IRenderingAttributeInjector {
    IRenderingAttributeProvider renderingAttributeProvider;
    ITransformerPreferences transformerPreferences;
    String elementName="attribute";
    String providerNameAttribute="provider";
    String attributeNameAttribute="name";
    String attributeValueAttribute="value";
    boolean outputProviderName=true;
    boolean outputValueInBody=true;
    boolean useXsltParams=false;
    
    /**
     * Monitors for the first element since the start of the document
     */
    private boolean firstElement=false;
    
    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
        super.startElement(uri, localName, qName, atts);
        if(firstElement) {
            // output params
            Map params=renderingAttributeProvider.getParameterValues();
            if(params!=null && !params.isEmpty()) {
                for (Iterator iter = params.entrySet().iterator(); iter.hasNext();) {
                    Map.Entry e = (Map.Entry) iter.next();
                    outputAttribute((String)e.getKey(),(String)e.getValue());
                }
            }
            firstElement=false;
        }
        
        Map attributes=renderingAttributeProvider.getAttributeValues(uri,localName,qName,atts);
        if(attributes!=null && !attributes.isEmpty()) {
            for (Iterator iter = attributes.entrySet().iterator(); iter.hasNext();) {
                Map.Entry e = (Map.Entry) iter.next();
                outputAttribute((String)e.getKey(),(String)e.getValue());
            }
        }
    }
    
    protected void outputAttribute(String attributeName,String attributeValue) throws SAXException {
        AttributesImpl ai=new AttributesImpl();
        ai.addAttribute(null,attributeNameAttribute,attributeNameAttribute,"CDATA",attributeName);
        if(outputProviderName) {
            ai.addAttribute(null,providerNameAttribute,providerNameAttribute,"CDATA",renderingAttributeProvider.getProviderName());
        }
        if(outputValueInBody) {
            super.startElement("",elementName,elementName,ai);
            super.characters(attributeValue.toCharArray(),0,attributeValue.length());
            super.endElement("",attributeValueAttribute,attributeValueAttribute);
        } else {
            ai.addAttribute("",attributeValueAttribute,attributeValueAttribute,"CDATA",attributeValue);
            super.startElement("",elementName,elementName,ai);
            super.endElement("",attributeValueAttribute,attributeValueAttribute);
        }
    }
    
    public void startDocument() throws SAXException {
        super.startDocument();
        this.firstElement=true;
    }

    public ICacheKey generateKey() {
        return renderingAttributeProvider.generateKey();
    }
    public boolean isCacheValid(Serializable validityObject) {
        return renderingAttributeProvider.isCacheValid(validityObject);
    }
    /**
     * Specify rendering attribute provider
     * @param renderingAttributeProvider
     */
    public void setRenderingAttributeProvider(IRenderingAttributeProvider renderingAttributeProvider) {
        this.renderingAttributeProvider = renderingAttributeProvider;
    }
    /**
     * Specify the name of the element used to output rendering attributes (defaults to "attribute")
     * @param elementName
     */
    public void setElementName(String elementName) {
        this.elementName = elementName;
    }
    /**
     * Specify element attribute name which will be used to output rendering attribute name (defaults to "name")
     * @param attributeNameAttribute
     */
    public void setAttributeNameAttribute(String attributeNameAttribute) {
        this.attributeNameAttribute = attributeNameAttribute;
    }
    /**
     * Specify element attribute name whilch will be used to output rendering attribute value if <code>outputValueInBody</code> is set to <code>false</code> (defaults to "value")
     * @param attributeValueAttribute
     */
    public void setAttributeValueAttribute(String attributeValueAttribute) {
        this.attributeValueAttribute = attributeValueAttribute;
    }
    /**
     * Specify whether attribute values should be written out as attributes or within the element body (defaults to false).
     * @param outputValueInBody
     */
    public void setOutputValueInBody(boolean outputValueInBody) {
        this.outputValueInBody = outputValueInBody;
    }
    /**
     * Specify element attribute name which will be used to output provider name if <code>outputProviderName</code> is set to <code>true</code>
     * @param providerNameAttribute
     */
    public void setProviderNameAttribute(String providerNameAttribute) {
        this.providerNameAttribute = providerNameAttribute;
    }
    /**
     * Specify whether to include provider name in the output
     * @param outputProviderName
     */
    public void setOutputProviderName(boolean outputProviderName) {
        this.outputProviderName = outputProviderName;
    }

    public void prepareTransformer(Transformer transformer) {
        if(useXsltParams) {
            Map params=renderingAttributeProvider.getParameterValues();
            if(params!=null && !params.isEmpty()) {
                for (Iterator iter = params.entrySet().iterator(); iter.hasNext();) {
                    Map.Entry e = (Map.Entry) iter.next();
                    transformer.setParameter((String)e.getKey(),(String)e.getValue());
                }
            }
        }
    }
    
    public ITransformerPreferences getTransformerPreferences() {
        return transformerPreferences;
    }

    public void setTransformerPreferences(ITransformerPreferences transformerPreferences) {
        this.transformerPreferences = transformerPreferences;
        this.renderingAttributeProvider=transformerPreferences.getRenderingAttributeProvider();
    }

    /**
     * Specify that params should be injected using XSLT params, and not as top-level elements
     * @param useXsltParams
     */
    public void setUseXsltParams(boolean useXsltParams) {
        this.useXsltParams = useXsltParams;
    }
}
