/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.CacheKeyListImpl;
import org.jasig.portal.caching.ICacheKey;
import org.xml.sax.Attributes;

/**
 * A default implementation of {@link org.jasig.portal.rendering.attributes.IHierarchicalRenderingAttributeProvider}
 * 
 * Parent and child providers are queried on reading atribute and parameter values. Parent providers have the first say as to what the values should be, followed by the
 * main provider itself, followed by the child providers (that get a chance to override any of the values supplied earlier). 
 * Here is the detailed priority order (in the order of decreasing importance):
 * 1. First child provider, second child provider, ... nth child provider
 * 2. The main provider
 * 3. First parent provider, second parent provider, .... nth parent provider
 * 4. Default values from the main provider (if turned on)
 * 
 * Clear and remove parameter/attribute commands are passed down to parents/children only if <code>propagateDeleteCommands</code> flag is set to true (default is <code>false</code>).
 * Setting of attributes is propagated only if <code>propagateSetAttributesCommands</code> flag is set.
 * Setting of parameters is not propagated.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class HierarchicalRenderingAttributeProvider extends RenderingAttributeProviderImpl implements IHierarchicalRenderingAttributeProvider {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(HierarchicalRenderingAttributeProvider.class);

    private boolean internalOutputDefaultValues=true;
    protected List<IRenderingAttributeProvider> parentProviders=Collections.EMPTY_LIST;
    protected List<IRenderingAttributeProvider> childProviders=Collections.EMPTY_LIST;
    boolean propagateDeleteCommands=false;
    boolean propagateSetAttributesCommands=false;
    
    public HierarchicalRenderingAttributeProvider() {
        // default value output will be performed explicitly, in this class
        super.setOutputDefaultValues(false);
    }
    
    public void clearAttributes() {
        super.clearAttributes();
        if(propagateDeleteCommands) {
            for (IRenderingAttributeProvider parentProvider: parentProviders) {
                parentProvider.clearAttributes();
            }
            for (IRenderingAttributeProvider childProvider: childProviders) {
                childProvider.clearAttributes();
            }
        }
    }

    public void clearAttributes(String namespaceUri, String localName, String qName, Attributes atts) {
        super.clearAttributes(namespaceUri, localName, qName, atts);
        if(propagateDeleteCommands) {
            for (IRenderingAttributeProvider parentProvider: parentProviders) {
                parentProvider.clearAttributes(namespaceUri,localName,qName,atts);
            }
            for (IRenderingAttributeProvider childProvider: childProviders) {
                childProvider.clearAttributes(namespaceUri,localName,qName,atts);
            }
        }
    }

    public void clearParameters() {
        super.clearParameters();
        if(propagateDeleteCommands) {
            for (IRenderingAttributeProvider parentProvider: parentProviders) {
                parentProvider.clearParameters();
            }
            for (IRenderingAttributeProvider childProvider: childProviders) {
                childProvider.clearParameters();
            }
        }
    }

    public Map getAttributeValues(String namespaceUri, String localName, String qName, Attributes atts) {
        Map allValues=new HashMap();
        if(internalOutputDefaultValues) {
            // start with the default values
            allValues.putAll(getDefaultAttributeValueMap());
        }
        // query parent providers
        for (IRenderingAttributeProvider parentProvider: parentProviders) {
            Map parentValues=parentProvider.getAttributeValues(namespaceUri, localName, qName, atts);
            if(parentValues!=null) {
                allValues.putAll(parentValues);
            }
        }
        Map svalues=super.getAttributeValues(namespaceUri, localName, qName, atts);
        if(svalues!=null) {
            allValues.putAll(svalues);
        }
        
        // query child providers
        for (IRenderingAttributeProvider childProvider: childProviders) {
            Map childValues=childProvider.getAttributeValues(namespaceUri, localName, qName, atts);
            if(childValues!=null) {
                allValues.putAll(childValues);
            }
        }
        
        return allValues;
    }

    public Map getParameterValues() {
        Map allValues=new HashMap();
        if(internalOutputDefaultValues) {
            // start with the default values
            allValues.putAll(getDefaultParamValueMap());
        }
        // query parent providers
        for (IRenderingAttributeProvider parentProvider: parentProviders) {
            Map parentValues=parentProvider.getParameterValues();
            if(parentValues!=null) {
                allValues.putAll(parentValues);
            }
        }
        Map svalues=super.getParameterValues();
        if(svalues!=null) {
            allValues.putAll(svalues);
        }
        
        // query child providers
        for (IRenderingAttributeProvider childProvider: childProviders) {
            Map childValues=childProvider.getParameterValues();
            if(childValues!=null) {
                allValues.putAll(childValues);
            }
        }
        return allValues;
    }

    public void setAttributeValue(String attributeName, String attributeValue, String namespaceUri, String localName, String qName, Attributes atts) {
        if(propagateSetAttributesCommands) {
            for (IRenderingAttributeProvider parentProvider: parentProviders) {
                parentProvider.setAttributeValue(attributeName,attributeValue,namespaceUri,localName,qName,atts);
            }
            for (IRenderingAttributeProvider childProvider: childProviders) {
                childProvider.setAttributeValue(attributeName,attributeValue,namespaceUri,localName,qName,atts);
            }
        }
        super.setAttributeValue(attributeName, attributeValue, namespaceUri, localName, qName, atts);
    }

    public boolean removeAttribute(String attributeName, String namespaceUri, String localName, String qName, Attributes atts) {
        boolean removed=false;
        if(propagateDeleteCommands) {
            for (IRenderingAttributeProvider parentProvider: parentProviders) {
                removed=parentProvider.removeAttribute(attributeName,namespaceUri,localName,qName,atts) || removed;
            }
            for (IRenderingAttributeProvider childProvider: childProviders) {
                removed=childProvider.removeAttribute(attributeName,namespaceUri,localName,qName,atts) || removed;
            }
        }
        return super.removeAttribute(attributeName, namespaceUri, localName, qName, atts) || removed;
    }

    public boolean removeParameter(String parameterName) {
        boolean removed=false;
        if(propagateDeleteCommands) {
            for (IRenderingAttributeProvider parentProvider: parentProviders) {
                removed=parentProvider.removeParameter(parameterName) || removed;
            }
            for (IRenderingAttributeProvider childProvider: childProviders) {
                removed=childProvider.removeParameter(parameterName) || removed;
            }
        }
        return super.removeParameter(parameterName) || removed;
    }

    public void setOutputDefaultValues(boolean outputDefaultValues) {
        this.internalOutputDefaultValues=outputDefaultValues;
    }

    public List<IRenderingAttributeProvider> getParentProviders() {
        List nl=new ArrayList<IRenderingAttributeProvider>(parentProviders);
        Collections.reverse(nl);
        return nl;
    }

    /**
     * Specify parent provider list.
     * The provider priority is such that the values provided by the first provider can overwrite all
     * subsequent provider values.
     * @param parentProviders
     */
    public void setParentProviders(List<IRenderingAttributeProvider> parentProviders) {
        this.parentProviders = parentProviders;
        Collections.reverse(this.parentProviders);
    }
    
    /**
     * Specify child provider list.
     * The provider priority is such that the values provided by the first provider can overwrite all
     * subsequent provider values.
     * @param childProviders
     */
    public void setChildProviders(List<IRenderingAttributeProvider> childProviders) {
        this.childProviders = childProviders;
        Collections.reverse(this.childProviders);
    }
    
    public void setPropagateSetAttributesCommands(boolean propagateSetCommands) {
        this.propagateSetAttributesCommands = propagateSetCommands;
    }

    public void setPropagateDeleteCommands(boolean propagateDeleteCommands) {
        this.propagateDeleteCommands = propagateDeleteCommands;
    }
    
    public ICacheKey generateKey() {
        CacheKeyImpl localKey=new CacheKeyImpl(providerName+Boolean.toString(outputDefaultValues));
        List<ICacheKey> cacheKeys=new LinkedList<ICacheKey>();
        cacheKeys.add(localKey);
        ICacheKey ownValuesKey=renderingAttributeValues.generateKey();
        if(ownValuesKey==null) {
            return null;
        }
        cacheKeys.add(ownValuesKey);

        for (IRenderingAttributeProvider parentProvider: parentProviders) {
            ICacheKey key=parentProvider.generateKey();
            if(key==null) {
                return null;
            }
            cacheKeys.add(key);
        }
        
        for (IRenderingAttributeProvider childProvider: childProviders) {
            ICacheKey key=childProvider.generateKey();
            if(key==null) {
                return null;
            }
            cacheKeys.add(key);
        }
        
        return new CacheKeyListImpl(cacheKeys);
    }

    public boolean isCacheValid(Serializable validityObject) {
        Serializable[] os=(Serializable[]) validityObject;
        // validate own values
        if(os.length<2) {
            if (logger.isWarnEnabled()) {
                logger.warn("isCacheValid(Serializable) - too few validation objects passed : n=" + os.length);
            }
            return false;
        }
        if(!renderingAttributeValues.isCacheValid(os[1])) {
            return false;
        }
        int i=2;
        if(os.length<i+parentProviders.size()) {
            if (logger.isWarnEnabled()) {
                logger.warn("isCacheValid(Serializable) - too few validation objects passed given the number of parents ("+parentProviders.size()+") : n=" + os.length);
            }
            return false; 
        }
        for (IRenderingAttributeProvider provider: parentProviders) {
            if(!provider.isCacheValid(os[i])) { 
                return false; 
            }
            i++;
        }
        if(os.length<i+childProviders.size()) { 
            if(os.length<i+parentProviders.size()) {
                if (logger.isWarnEnabled()) {
                    logger.warn("isCacheValid(Serializable) - too few validation objects passed given the number of parents ("+parentProviders.size()+") and children ("+childProviders.size()+") : n=" + os.length);
                }
                return false; 
            }
            return false; 
        }
        for (IRenderingAttributeProvider provider: childProviders) {
            if(!provider.isCacheValid(os[i])) { return false; }
            i++;
        }
        return true;
    }
}
