/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.caching.CacheKeyListImpl;
import org.jasig.portal.caching.ICacheKey;

/**
 * A layered implementation of the rendering attribute values.
 * The decision on whether a particular parameter or attribute
 * is maintained at a current or parent level is delegated to
 * attributeLevelResolver. 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class LayeredRenderingAttributeValues extends RenderingAttributeValuesImpl {
    IAttributeLevelResolver attributeLevelResolver;
    IRenderingAttributeValues parentAttributeValues;


    public Map getElementAttributeValues(ElementResolverKey resolverKey) {
        Map allMap=new HashMap();
        Map parentMap=parentAttributeValues.getElementAttributeValues(resolverKey);
        if(parentMap!=null) { allMap.putAll(parentMap); }
        Map localMap=super.getElementAttributeValues(resolverKey);
        if(localMap!=null) { 
            allMap.putAll(localMap);
        }
        return allMap;
    }
    
    public Set getElementResolverKeys() {
        Set allSet=new HashSet();
        Set parentSet=parentAttributeValues.getElementResolverKeys();
        if(parentSet!=null) { allSet.addAll(parentSet); }
        Set localSet=super.getElementResolverKeys();
        if(localSet!=null) { allSet.addAll(localSet); }
        return allSet;
    }

    public Map getParameterValues() {
        Map allMap=new HashMap();
        Map parentMap=parentAttributeValues.getParameterValues();
        if(parentMap!=null) { allMap.putAll(parentMap); }
        Map localMap=super.getParameterValues();
        if(localMap!=null) { allMap.putAll(localMap); }
        return allMap;
    }

    public boolean removeElementAttributes(ElementResolverKey resolverKey) {
        return parentAttributeValues.removeElementAttributes(resolverKey) || super.removeElementAttributes(resolverKey);
    }

    public boolean removeElementAttributeValue(ElementResolverKey resolverKey, String attributeName) {
        if(attributeLevelResolver.isElementAttributeInLevel(resolverKey,attributeName)) {
            return super.removeElementAttributeValue(resolverKey, attributeName);
        } else {
            return parentAttributeValues.removeElementAttributeValue(resolverKey,attributeName);
        }
    }

    public boolean removeParameter(String parameterName) {
        if(attributeLevelResolver.isParameterInLevel(parameterName)) {
            return super.removeParameter(parameterName);    
        } else {
            return parentAttributeValues.removeParameter(parameterName);
        }
    }

    public void setElementAttributeValue(ElementResolverKey resolverKey, String attributeName, String attributeValue) {
        if(attributeLevelResolver.isElementAttributeInLevel(resolverKey,attributeName)) {
            super.setElementAttributeValue(resolverKey, attributeName, attributeValue);
        } else {
            parentAttributeValues.setElementAttributeValue(resolverKey, attributeName, attributeValue);
        }
    }

    public void setParameterValue(String parameterName, String parameterValue) {
        if(attributeLevelResolver.isParameterInLevel(parameterName)) {
            super.setParameterValue(parameterName, parameterValue);
        } else {
            parentAttributeValues.setParameterValue(parameterName, parameterValue);
        }
    }

    public void clearElementAttributes() {
        parentAttributeValues.clearElementAttributes();
        super.clearElementAttributes();
    }

    public void clearParameters() {
        parentAttributeValues.clearParameters();
        super.clearParameters();
    }

    public IAttributeLevelResolver getAttributeLevelResolver() {
        return attributeLevelResolver;
    }

    public void setAttributeLevelResolver(IAttributeLevelResolver attributeLevelResolver) {
        this.attributeLevelResolver = attributeLevelResolver;
    }

    public IRenderingAttributeValues getParentAttributeValues() {
        return parentAttributeValues;
    }

    public void setParentAttributeValues(IRenderingAttributeValues parentAttributeValues) {
        this.parentAttributeValues = parentAttributeValues;
    }

    public ICacheKey generateKey() {
        ICacheKey[] keys={super.generateKey(),parentAttributeValues.generateKey()};
        return new CacheKeyListImpl(keys);
    }
}
