/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.CacheKeyListImpl;
import org.jasig.portal.caching.ICacheKey;
import org.springframework.beans.factory.annotation.Required;
import org.xml.sax.Attributes;

/**
 * A container for a list of rendering attribute providers. Providers are evaluated in
 * the order supplied, that is the last provider is able to override values of all previous providers.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class ListRenderingAttributeProvider implements IHierarchicalRenderingAttributeProvider {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(ListRenderingAttributeProvider.class);

    List<IRenderingAttributeProvider> parentProviders;
    String providerName;

    public List<IRenderingAttributeProvider> getParentProviders() {
        return parentProviders;
    }

    public void clearAttributes() {
        for (IRenderingAttributeProvider provider: parentProviders) {
            provider.clearAttributes();
        }
    }

    public void clearAttributes(String namespaceUri, String localName, String qName, Attributes atts) {
        for (IRenderingAttributeProvider provider: parentProviders) {
            provider.clearAttributes(namespaceUri,localName,qName,atts);
        }
        
    }

    public void clearParameters() {
        for (IRenderingAttributeProvider provider: parentProviders) {
            provider.clearParameters();
        }
    }

    public Map getAttributeValues(String namespaceUri, String localName, String qName, Attributes atts) {
        Map values=new HashMap();
        for (IRenderingAttributeProvider provider: parentProviders) {
            Map pvalues=provider.getAttributeValues(namespaceUri,localName,qName,atts);
            if(pvalues!=null) {
                values.putAll(pvalues);
            }
        }
        return values;
    }

    public Map getParameterValues() {
        Map values=new HashMap();
        for (IRenderingAttributeProvider provider: parentProviders) {
            Map pvalues=provider.getParameterValues();
            if(pvalues!=null) {
                values.putAll(pvalues);
            }
        }
        return values;
    }

    public String getProviderName() {
        return this.providerName;
    }

    public boolean removeAttribute(String attributeName, String namespaceUri, String localName, String qName, Attributes atts) {
        boolean success=false;
        for (IRenderingAttributeProvider provider: parentProviders) {
            success=provider.removeAttribute(attributeName,namespaceUri,localName,qName,atts) || success;
        }
        return success;
    }

    public boolean removeParameter(String parameterName) {
        boolean success=false;
        for (IRenderingAttributeProvider provider: parentProviders) {
            success=provider.removeParameter(parameterName) || success;
        }
        return success;
    }

    public void setAttributeValue(String attributeName, String attributeValue, String namespaceUri, String localName, String qName, Attributes atts) {
        for (IRenderingAttributeProvider provider: parentProviders) {
            provider.setAttributeValue(attributeName,attributeValue,namespaceUri,localName,qName,atts);
        }
    }

    public void setParameterValue(String parameterName, String parameterValue) {
        for (IRenderingAttributeProvider provider: parentProviders) {
            provider.setParameterValue(parameterName,parameterValue);
        }
    }

    public ICacheKey generateKey() {
        List cacheKeys=new ArrayList();
        for (IRenderingAttributeProvider provider: parentProviders) {
            ICacheKey cacheKey=provider.generateKey();
            if(cacheKey==null) {
                if (logger.isInfoEnabled()) {
                    logger.info("clearAttributes() - a parent provider returned null cache key");
                }
                return null;
            }
            cacheKeys.add(provider.generateKey());
        }
        return new CacheKeyListImpl(cacheKeys);
    }

    public boolean isCacheValid(Serializable validityObject) {
        boolean valid=true;
        Serializable[] vo=(Serializable[]) validityObject;
        int i=0;
        for (IRenderingAttributeProvider provider: parentProviders) {
            valid=valid || provider.isCacheValid(vo[i]);
            if(!valid) {
                break;
            } else {
                i++;
            }
        }
        return valid;
    }

    @Required
    public void setParentProviders(List<IRenderingAttributeProvider> parentProviders) {
        this.parentProviders = parentProviders;
    }

    public void setProviderName(String providerName) {
        this.providerName = providerName;
    }
}
