/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.CacheKeyListImpl;
import org.jasig.portal.caching.ICacheKey;
import org.xml.sax.Attributes;

/**
 * A basic implementation of the rendering attribute provider.
 * 
 * The implementation is configured with {@link org.jasig.portal.rendering.attributes.IRenderingAttributesDescription},
 * {@link org.jasig.portal.rendering.attributes.IElementResolver} and appropriate {@link org.jasig.portal.rendering.attributes.IRenderingAttributeValues}.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class RenderingAttributeProviderImpl implements IRenderingAttributeProvider {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(RenderingAttributeProviderImpl.class);

    protected IRenderingAttributesDescription renderingAttributeDescription;
    protected IElementResolver elementResolver=new NoElementResolver();
    protected String providerName;
    protected IRenderingAttributeValues renderingAttributeValues;
    protected boolean outputDefaultValues=true;
    protected boolean omitRecordingDefaultValues=false;
    
    private Map defaultParamValues=null;
    private Map defaultAttributeValues=null;
    
    
    public Map getAttributeValues(String namespaceUri, String localName, String qName, Attributes atts) {
        ElementResolverKey elementKey=elementResolver.resolveElementKey(namespaceUri,localName,qName,atts);
        if(elementKey!=null) {
            Map currentAttributeValueMap=renderingAttributeValues.getElementAttributeValues(elementKey);
            if(outputDefaultValues) {
                // start with the default values
                Map attributeValueMap=new HashMap(getDefaultAttributeValueMap());
                if(currentAttributeValueMap!=null) {
                    attributeValueMap.putAll(currentAttributeValueMap);
                }
                return attributeValueMap;
            } else {
                return currentAttributeValueMap;
            }
        } else {
            return null;
        }
    }
    
    /**
     * @return a map of default attribute values
     */
    protected Map getDefaultAttributeValueMap() {
        if(defaultAttributeValues==null) {
            Map values=new HashMap();
            for (Iterator iter = renderingAttributeDescription.getElementAttributeNames().iterator(); iter.hasNext();) {
                String attributeName = (String) iter.next();
                IAttributeDescription attributeDesc=renderingAttributeDescription.getElementAttributeDescription(attributeName);
                String attributeValue=attributeDesc.getDefaultValue();
                if(attributeValue!=null) {
                    values.put(attributeName,attributeValue);
                }
            }
            defaultAttributeValues=values;
        }
        return defaultAttributeValues;
    }
    
    /**
     * @return a map of default param values
     */
    protected Map getDefaultParamValueMap() {
        if(defaultParamValues==null) {
            Map values=new HashMap();
            for (Iterator iter = renderingAttributeDescription.getParamNames().iterator(); iter.hasNext();) {
                String paramName = (String) iter.next();
                IAttributeDescription paramDesc=renderingAttributeDescription.getParamDescription(paramName);
                String paramValue=paramDesc.getDefaultValue();
                if(paramValue!=null) {
                    values.put(paramName,paramValue);
                }
            }
            defaultParamValues=values;
        }
        return defaultParamValues;
    }

    public Map getParameterValues() {
        Map localValues=renderingAttributeValues.getParameterValues();
        if(outputDefaultValues) {
            Map allValues=new HashMap(getDefaultParamValueMap());
            if(localValues!=null) {
                allValues.putAll(localValues);
            }
            return allValues;
        } else {
            return localValues;
        }
    }

    public void setAttributeValue(String attributeName, String attributeValue, String namespaceUri, String localName, String qName, Attributes atts) {
        if(attributeValue==null) {
            // don't support null values, remove instead
            removeAttribute(attributeName,namespaceUri,localName,qName,atts);
        } else {
            if(omitRecordingDefaultValues) {
                String defaultValue=null;
                IAttributeDescription description=renderingAttributeDescription.getElementAttributeDescription(attributeName);
                if(description!=null) {
                    defaultValue=description.getDefaultValue();
                }
                if(attributeValue.equals(defaultValue)) {
                    // remove instead
                    removeAttribute(attributeName,namespaceUri,localName,qName,atts);
                    return;
                }
            }
            ElementResolverKey elementKey=elementResolver.resolveElementKey(namespaceUri,localName,qName,atts);
            if(elementKey!=null) {
                renderingAttributeValues.setElementAttributeValue(elementKey,attributeName,attributeValue);
            }
        }
    }

    public void setParameterValue(String parameterName, String parameterValue) {
        if(parameterValue==null) {
            // dont' support null values, remove instead
            removeParameter(parameterName);
        } else {
            if(omitRecordingDefaultValues) {
                String defaultValue=null;
                IAttributeDescription description=renderingAttributeDescription.getParamDescription(parameterName);
                if(description!=null) {
                    defaultValue=description.getDefaultValue();
                }
                if(parameterValue.equals(defaultValue)) {
                    // remove instead
                    removeParameter(parameterName);
                    return;
                }
            }
            renderingAttributeValues.setParameterValue(parameterName,parameterValue);
        }
    }

    public void clearAttributes() {
        renderingAttributeValues.clearElementAttributes();
    }

    public void clearAttributes(String namespaceUri, String localName, String qName, Attributes atts) {
        ElementResolverKey elementKey=elementResolver.resolveElementKey(namespaceUri,localName,qName,atts);
        if(elementKey!=null) {
            renderingAttributeValues.removeElementAttributes(elementKey);
        }
    }

    public void clearParameters() {
        renderingAttributeValues.clearParameters();
    }

    public boolean removeAttribute(String attributeName, String namespaceUri, String localName, String qName, Attributes atts) {
        ElementResolverKey elementKey=elementResolver.resolveElementKey(namespaceUri,localName,qName,atts);
        if(elementKey!=null) {
            return renderingAttributeValues.removeElementAttributeValue(elementKey,attributeName);
        } else {
            return false;
        }
    }

    public boolean removeParameter(String parameterName) {
        return renderingAttributeValues.removeParameter(parameterName);
    }

    public void setElementResolver(IElementResolver elementResolver) {
        this.elementResolver = elementResolver;
    }

    public void setRenderingAttributesDescription(IRenderingAttributesDescription renderingAttributeDescription) {
        this.renderingAttributeDescription = renderingAttributeDescription;
    }

    public String getProviderName() {
        return providerName;
    }

    public void setProviderName(String providerName) {
        this.providerName = providerName;
    }

    public void setRenderingAttributeValues(IRenderingAttributeValues renderingAttributeValues) {
        this.renderingAttributeValues = renderingAttributeValues;
        defaultParamValues=null;
        defaultAttributeValues=null;
    }

    /**
     * Specify whether default values from the attribute descriptions should be
     * incorporated into the returns. (defaults to <code>true</code>)
     * @param outputDefaultValues
     */
    public void setOutputDefaultValues(boolean outputDefaultValues) {
        this.outputDefaultValues = outputDefaultValues;
    }

    /**
     * Specify whether parameter and attribute values equal to the default values
     * should be recorded. (defaults to <code>false</code>)
     * @param omitRecordingDefaultValues
     */
    public void setOmitRecordingDefaultValues(boolean omitRecordingDefaultValues) {
        this.omitRecordingDefaultValues = omitRecordingDefaultValues;
    }

    public ICacheKey generateKey() {
        CacheKeyImpl localKey=new CacheKeyImpl(providerName+Boolean.toString(outputDefaultValues));
        ICacheKey[] cacheKeys={localKey,renderingAttributeValues.generateKey()};
        return new CacheKeyListImpl(cacheKeys);
    }

    public boolean isCacheValid(Serializable validityObject) {
        if(validityObject instanceof Serializable[]) {
            Serializable[] os=(Serializable[]) validityObject;
            if(os.length==2) {
                return renderingAttributeValues.isCacheValid(os[1]);
            } else {
                logger.warn("isCacheValid(Serializable) - validity object array is not of size 2 as expected. size= : os=" + os.length);
                return false;
            }
        } else {
            logger.warn("isCacheValid(Serializable) - validity object is not a Serializable[] as expected");
            return false;
        }
    }
    
}
