/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.CacheKeyListImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.context.common.user.IUserControllerEventListener;
import org.jasig.portal.context.common.user.UserControllerEvent;

/**
 * Default implementation of {@link org.jasig.portal.rendering.attributes.IRenderingAttributeValues}.
 * If injected with {@link org.jasig.portal.context.common.user.IUserController} will clear
 * attribute values upon controller event. 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class RenderingAttributeValuesImpl implements IRenderingAttributeValues, IUserControllerEventListener {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(RenderingAttributeValuesImpl.class);

    Map parameterValues;
    Map elementAttributeValues;
    Long id;
    IUserController userController;
    
    protected static final ICacheKey emptyCacheKey=new CacheKeyImpl("");
    
    public Map getElementAttributeValues(ElementResolverKey resolverKey) {
        if(elementAttributeValues==null) { return null; }
        ElementAttributeValues values=(ElementAttributeValues) elementAttributeValues.get(resolverKey);
        if(values==null) {
            return null;
        } else {
            return values.getAttributeValues();
        }
    }
    public Set getElementResolverKeys() {
        if(elementAttributeValues==null) { return null; }
        return elementAttributeValues.keySet();
    }
    
    public Map getParameterValues() {
        return parameterValues;
    }
    
    public boolean removeElementAttributes(ElementResolverKey resolverKey) {
        if(elementAttributeValues==null) { return false; }
        return elementAttributeValues.remove(resolverKey)==null;
    }
    
    public boolean removeElementAttributeValue(ElementResolverKey resolverKey, String attributeName) {
        if(elementAttributeValues==null) { return false; }
        ElementAttributeValues values=(ElementAttributeValues) elementAttributeValues.get(resolverKey);
        if(values==null) {
            return false;
        } else {
            return values.getAttributeValues().remove(attributeName)==null;
        }
    }
    public boolean removeParameter(String parameterName) {
        if(parameterValues==null) { return false; }
        return parameterValues.remove(parameterName)==null;
    }
    public void setElementAttributeValue(ElementResolverKey resolverKey, String attributeName, String attributeValue) {
        if(elementAttributeValues==null) { elementAttributeValues=new HashMap(); }
        ElementAttributeValues values=(ElementAttributeValues) elementAttributeValues.get(resolverKey);
        if(values==null) {
            values=new ElementAttributeValues();
            values.setAttributeValues(new HashMap());
            elementAttributeValues.put(resolverKey,values);
        }
        values.getAttributeValues().put(attributeName,attributeValue);
    }
    
    public void setParameterValue(String parameterName, String parameterValue) {
        if(parameterValues==null) { parameterValues=new HashMap(); }
        parameterValues.put(parameterName,parameterValue);
    }

    public Long getId() {
        return id;
    }
    public void setId(Long id) {
        this.id = id;
    }
    public Map getElementAttributeValues() {
        return elementAttributeValues;
    }
    public void setElementAttributeValues(Map elementAttributeValues) {
        this.elementAttributeValues = elementAttributeValues;
    }
    public void setParameterValues(Map parameterValues) {
        this.parameterValues = parameterValues;
    }
    public void clearElementAttributes() {
        if(elementAttributeValues!=null) {
            elementAttributeValues.clear();
        }
    }
    public void clearParameters() {
        if(parameterValues!=null) {
            parameterValues.clear();
        }
    }
    
    protected ICacheKey generateParameterCacheKey() {
        if(parameterValues!=null && !parameterValues.isEmpty()) {
            StringBuffer sb=new StringBuffer();
            for (Iterator iter = parameterValues.entrySet().iterator(); iter.hasNext();) {
                Map.Entry element = (Map.Entry) iter.next();
                sb.append(element.getKey()).append(element.getValue());
            }
            return new CacheKeyImpl(sb.toString());
        } else {
            return emptyCacheKey;
        }
    }
    
    public ICacheKey generateKey() {
       List cacheKeys=new ArrayList();
       cacheKeys.add(generateParameterCacheKey());
       if(elementAttributeValues!=null) {
           StringBuffer sb=new StringBuffer();
           for (Iterator iter = elementAttributeValues.entrySet().iterator(); iter.hasNext();) {
               Map.Entry element = (Map.Entry) iter.next();
               sb.append(element.getKey());
               cacheKeys.add(((ElementAttributeValues)element.getValue()).generateKey());
           }
           cacheKeys.add(new CacheKeyImpl(sb.toString()));
       }
       return new CacheKeyListImpl(cacheKeys);
    }
    
    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }
    /**
     * Specify user controller, so that values can be cleared when new user logs in.
     * @param userController
     */
    public void setUserController(IUserController userController) {
        this.userController = userController;
        userController.addUserControllerEventListener(this);
    }
    
    public void receiveUserControllerEvent(UserControllerEvent event) {
        // clear current values
        if(elementAttributeValues!=null) {
            elementAttributeValues.clear();
        }
        if(parameterValues!=null) { parameterValues.clear(); }
        
        if (logger.isDebugEnabled()) {
            logger.debug("receiveUserControllerEvent(UserControllerEvent) - cleared existing values because of the user change event");
        }
    }
    
    
    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof RenderingAttributeValuesImpl)) {
            return false;
        }
        RenderingAttributeValuesImpl rhs = (RenderingAttributeValuesImpl)object;
        return new EqualsBuilder()
                .append(this.parameterValues, rhs.parameterValues)
                .append(this.elementAttributeValues, rhs.elementAttributeValues)
                .isEquals();
    }
    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(1271243703, 1686813055)
                .append(this.parameterValues)
                .append(this.elementAttributeValues)
                .toHashCode();
    }
    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
            .appendSuper(super.toString())
            .append("id", this.id)
            .append("parameterValues", this.parameterValues)
            .append("elementAttributeValues", this.elementAttributeValues)
            .toString();
    }
}
