/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes.url;

import java.io.Serializable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheable;
import org.jasig.portal.url.IPortalUrl;
import org.jasig.portal.url.IRequestParameterProcessor;
import org.jasig.portal.url.IServletPathExtender;
import org.jasig.portal.url.IUrlDecorator;
import org.jasig.portal.url.IWritableHttpServletRequest;

/**
 * Decorates URL by appending path elements, corresponding to the values of
 * specified rendering parameters (attribute). Also implements appropriate request parameter processor
 * to read out the attribute values from incoming request and set pass it to the rendering
 * attribute providers. 
 * 
 * Note: requires {@link org.jasig.portal.url.IServletPathExtender} URL constructor to be configured in the list of url constructors.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class AttributeDecoratorAndProcessor implements IRequestParameterProcessor, IUrlDecorator, ICacheable {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(AttributeDecoratorAndProcessor.class);
    
    protected String missingValueString="";

    List<AttributeDecoratorConfig> attributes;
    int pathElementOffset=1;

    public void decorateUrl(IPortalUrl url) {
        IServletPathExtender constructor=(IServletPathExtender) url.getUrlConstructor(IServletPathExtender.class.getName());
        for (Iterator<AttributeDecoratorConfig> iter = attributes.iterator(); iter.hasNext();) {
            AttributeDecoratorConfig attribute = iter.next();
            Map parameterValues=attribute.getProvider().getParameterValues();
            if(parameterValues!=null) {
                String parameterValue=(String) parameterValues.get(attribute.getName());
                if(parameterValue!=null) {
                    if (logger.isDebugEnabled()) {
                        logger.debug("decorateUrl(IPortalUrl) - adding current value to the path : parameterName=" + attribute.getName()+", value="+parameterValue);
                    }
                    constructor.addPathElement(parameterValue);
                    continue;
                }
            }
            if(attribute.getDefaultValue()!=null) {
                if (logger.isDebugEnabled()) {
                    logger.debug("decorateUrl(IPortalUrl) - adding default value to the path : parameterName=" + attribute.getName());
                }
                constructor.addPathElement(attribute.getDefaultValue());
            } else {
                // output empty value
                constructor.addPathElement(missingValueString);
            }
        }
    }

    public boolean canEvaluateVariables() {
        return false;
    }

    public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res) {
        String servletPath=req.getServletPath();
        int realPathElementOffset=getPathElementOffset(req);
        if(servletPath.startsWith("/")) { realPathElementOffset++; }
        String[] pathElements=servletPath.split("/");
        if(pathElements.length > (realPathElementOffset+attributes.size())) {
            int i=0;
            for (Iterator<AttributeDecoratorConfig> iter = attributes.iterator(); iter.hasNext();) {
                AttributeDecoratorConfig attribute = iter.next();
                if(!missingValueString.equals(pathElements[realPathElementOffset+i])) {
                    attribute.getProvider().setParameterValue(attribute.getName(),pathElements[realPathElementOffset+i]);
                    if (logger.isDebugEnabled()) {
                        logger.debug("processParameters(IWritableHttpServletRequest, HttpServletResponse) - setting path-defined param value : name="+attribute.getName()+", value=" + pathElements[realPathElementOffset+i]);
                    }
                }
                i++;
            }
        } else {
            if (logger.isDebugEnabled()) {
                logger.debug("processParameters(IWritableHttpServletRequest, HttpServletResponse) - too few path elements : servletPath=" + servletPath);
            }
        }
        return true;
    }

    public ICacheKey generateKey() {
        StringBuffer sb=new StringBuffer();
        for (Iterator<AttributeDecoratorConfig> iter = attributes.iterator(); iter.hasNext();) {
            AttributeDecoratorConfig attribute = iter.next();
            Map parameterValues=attribute.getProvider().getParameterValues();
            if(parameterValues!=null) { 
                String parameterValue=(String) parameterValues.get(attribute.getName());
                sb.append(parameterValue);
            }
            sb.append(":");
        }
        return new CacheKeyImpl(sb.toString());
    }

    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }


    /**
     * Specify offset at which the param will appear in the servlet path.
     * Default value is 1, which means that the param will appear as a second path element (e.g. after portal context name).
     * @param pathElementOffset number of elements prior to the location of the param
     */
    public void setPathElementOffset(int pathElementOffset) {
        this.pathElementOffset = pathElementOffset;
    }

    /**
     * Set the string that will be used to represent missing attribute value (defaults to nothing - "");
     * @param missingValueString
     */
    public void setMissingValueString(String missingValueString) {
        this.missingValueString = missingValueString;
    }

    /**
     * Specify an list of attribute configurations, in the order in which they should be incorporated into the servlet path 
     * @param attributes
     */
    public void setAttributes(List<AttributeDecoratorConfig> attributes) {
        this.attributes = attributes;
    }

    protected int getPathElementOffset(HttpServletRequest req) {
        return pathElementOffset;
    }

}
