/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes.url;

import java.util.Stack;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xalan.extensions.XSLProcessorContext;
import org.apache.xalan.templates.ElemExtensionCall;
import org.apache.xalan.transformer.TransformerImpl;
import org.apache.xpath.XPath;
import org.apache.xpath.XPathContext;
import org.apache.xpath.objects.XObject;
import org.jasig.portal.rendering.attributes.IRenderingAttributeProviderCommands;
import org.jasig.portal.url.IPortalUrl;
import org.jasig.portal.url.PortalUrlXalanElements;
import org.jasig.portal.utils.XmlUtils;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Xalan extension elements for constructing URLs with {@link org.jasig.portal.rendering.attributes.IRenderingAttributeProviderCommands} 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11917 $
 */
public class RenderingAttributeProviderXalanElements {
    final static String PROVIDER_ATTRIBUTE="provider";
    
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(RenderingAttributeProviderXalanElements.class);

    /**
     * usage <rendering:parameter source="structure" name="nodeID" value="12"/>
     * 
     * @param context
     * @param elem
     * @throws TransformerException
     */
    public void parameter(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl=context.getTransformer();
        Node contextNode=context.getContextNode();
        IRenderingAttributeProviderCommands rapc=getRenderingAttributeProviderCommands(transformerImpl);
        String source=elem.getAttribute(PROVIDER_ATTRIBUTE,contextNode,transformerImpl);
        String parameterName=elem.getAttribute("name",contextNode,transformerImpl);
        String parameterValue=elem.getAttribute("value",contextNode,transformerImpl);
        rapc.setParameterValue(source,parameterName,parameterValue);
    }
    
    
    /**
     * usage: applying to the current context node 
     *  <rendering:attribute source="theme" name="columnWidth" value="100"/>
     * usage: applying to a nodeset
     *  <rendering:attribute select="/folder[@column='true']" source="theme" name="columnWidth" value="100"/>
     * 
     * @param context
     * @param elem
     * @throws TransformerException
     */
    public void attribute(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl=context.getTransformer();
        Node contextNode=context.getContextNode();
        IRenderingAttributeProviderCommands rapc=getRenderingAttributeProviderCommands(transformerImpl);
        String source=elem.getAttribute(PROVIDER_ATTRIBUTE,contextNode,transformerImpl);
        String parameterName=elem.getAttribute("name",contextNode,transformerImpl);
        String parameterValue=elem.getAttribute("value",contextNode,transformerImpl);
        // determine attribute element
        String select=elem.getAttribute("select");
        if(select!=null && !select.equals("")) {
            // select nodeset with XPath
            XPathContext xctxt = context.getTransformer().getXPathContext();
            XPath myxpath = new XPath(select, elem, xctxt.getNamespaceContext(), XPath.SELECT);
            XObject value = myxpath.execute(xctxt, context.getContextNode(), elem);
            
            if(value.getType() != XObject.CLASS_NODESET) {
                logger.error("attribute(XSLProcessorContext, ElemExtensionCall) - select statement of rendering:attribute extension element does not evaluate to a node set : source=" + source + ", parameterName=" + parameterName + ", parameterValue=" + parameterValue + ", select=" + select, null);
            }
            NodeList nodelist = value.nodelist();
            int n = nodelist.getLength();
            for(int i = 0; i < n; i++) {
              Node node = nodelist.item(i);
              if(node.getNodeType()==Node.ELEMENT_NODE) {
                  rapc.setAttributeValue(source,parameterName,parameterValue,node.getNamespaceURI(),node.getLocalName(),node.getNodeName(),XmlUtils.getAttributes(node));
              } else {
                  logger.error("attribute(XSLProcessorContext, ElemExtensionCall) - selected nodeset for rendering:attribute extension element contains a non-element node : source=" + source + ", parameterName=" + parameterName + ", parameterValue=" + parameterValue + ", select=" + select, null);  
              }
            }
        } else {
            // use current context node
            if(contextNode.getNodeType()==Node.ELEMENT_NODE) {
                rapc.setAttributeValue(source,parameterName,parameterValue,contextNode.getNamespaceURI(),contextNode.getLocalName(),contextNode.getNodeName(),XmlUtils.getAttributes(contextNode));
            } else {
                logger.error("attribute(XSLProcessorContext, ElemExtensionCall) - context node for rendering:attribute extension element call is not an element, and no select attribute was given : source=" + source + ", parameterName=" + parameterName + ", parameterValue=" + parameterValue, null);
            }
        }
    }
    
    protected IRenderingAttributeProviderCommands getRenderingAttributeProviderCommands(Transformer transformer) {
        Stack urlStack=PortalUrlXalanElements.getUrlStack(transformer);
        if(urlStack!=null) {
            IPortalUrl portalUrl=(IPortalUrl) urlStack.peek();
            if(portalUrl!=null) {
                IRenderingAttributeProviderCommands rapc=(IRenderingAttributeProviderCommands) portalUrl.getUrlConstructor(IRenderingAttributeProviderCommands.class.getName());
                if(rapc==null) {
                    final IllegalStateException ise = new IllegalStateException("rendering attribute provider commands url constructor must be registered");
                    logger.error("unable to obtain rendering attribute provider commands constructor", ise);
                    throw ise;
                } else {
                    return rapc;
                }
            } else {
                final IllegalStateException ise = new IllegalStateException("url stack on the transformer is empty");
                logger.error("unable to obtain portal url from the url stack", ise);
                throw ise;
            }
        } else {
            final IllegalStateException ise = new IllegalStateException("url stack is null");
            logger.error("unable to obtain url stack from the transformer", ise);
            throw ise;
        }
    }
}
