/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes.url;

import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;

import org.jasig.portal.rendering.attributes.IRenderingAttributeProviderCommands;
import org.jasig.portal.url.BaseUrlConstructorImpl;
import org.jasig.portal.url.syntax.IUrlBasicSyntaxProvider;
import org.jasig.portal.url.syntax.UrlBasicSyntaxProviderImpl;
import org.jasig.portal.utils.IStringConcatenator;
import org.jasig.portal.utils.UrlEncodingStringConcatenator;
import org.xml.sax.Attributes;

/**
 * A generic implementation of rendering attribute url constructor,
 * that relies on downstream ElementResolvers being able to resolve based
 * on the qName and a pre-specified list of attributes (as determined by attributeNames
 * property, defaults to just ID attribute).
 * 
 * URL format: "provider attName attValue elementName elementAttribute(s)", where
 * provider is the name of one of the registered providers
 * attName, attValue - name and value of the attribute being set
 * elementName - element qname
 * elementAttribute(s) - space separated element attribute name value pairs.  
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11917 $
 */
public class RenderingAttributeUrlConstructorImpl extends BaseUrlConstructorImpl implements IRenderingAttributeProviderCommands {
    
    String processorPrefix=RenderingAttributeRequestParameterProcessor.DEFAULT_PROCESSOR_PREFIX;
    
    public static final String SET_ATTRIBUTE_COMMAND="sa";
    public static final String SET_PARAMETER_COMMAND="sp";
    
    
    IUrlBasicSyntaxProvider basicSyntaxProvider=new UrlBasicSyntaxProviderImpl();
    IStringConcatenator stringConcatenator=new UrlEncodingStringConcatenator();
    Set attributeNames;
    
    
    public RenderingAttributeUrlConstructorImpl() {
        attributeNames=new HashSet();
        attributeNames.add("ID");
    }

    public void setAttributeValue(String providerName, String attributeName, String attributeValue, String namespaceUri, String localName, String qName, Attributes atts) {
        String elementName=localName;
        // recognize either qName or localName - whichever is not null
        if(elementName==null) { elementName=qName; }
        
        // construct value
        LinkedList l=new LinkedList();
        l.add(providerName);
        l.add(attributeName);
        l.add(attributeValue);
        l.add(elementName);
        for (Iterator iter = attributeNames.iterator(); iter.hasNext();) {
            String attName = (String) iter.next();
            String attValue=atts.getValue(attName);
            if(attValue!=null) {
                l.add(attName);
                l.add(attValue);
            }
        }
        
        String parameterValue=stringConcatenator.concatenate(l);
        requestParameters.add(new StringTuple(basicSyntaxProvider.incorporateProcessorPrefix(processorPrefix,SET_ATTRIBUTE_COMMAND),parameterValue));
    }


    
    public void setParameterValue(String providerName, String parameterName, String parameterValue) {
        LinkedList l=new LinkedList();
        l.add(providerName);
        l.add(parameterName);
        l.add(parameterValue);
        requestParameters.add(new StringTuple(basicSyntaxProvider.incorporateProcessorPrefix(processorPrefix,SET_PARAMETER_COMMAND),stringConcatenator.concatenate(l)));
    }

    public void clearAttributes(String providerName, String namespaceUri, String localName, String qName, Attributes atts) {
        throw new UnsupportedOperationException("method not implemented - no legacy syntax exisits for this command");
    }

    public void clearAttributes(String providerName) {
        throw new UnsupportedOperationException("method not implemented - no legacy syntax exisits for this command");
    }

    public void clearParameters(String providerName) {
        throw new UnsupportedOperationException("method not implemented - no legacy syntax exisits for this command");
    }

    public boolean removeAttribute(String providerName, String attributeName, String namespaceUri, String localName, String qName, Attributes atts) {
        throw new UnsupportedOperationException("method not implemented - no legacy syntax exisits for this command");
    }

    public boolean removeParameter(String providerName, String parameterName) {
        throw new UnsupportedOperationException("method not implemented - no legacy syntax exisits for this command");
    }

    public void setStringConcatenator(IStringConcatenator stringConcatenator) {
        this.stringConcatenator = stringConcatenator;
    }

    /**
     * Set names of the element attributes that should be included in the URL. Defaults to just ID attribute.
     * The set of attributs should be sufficient for the downstream ElementResolvers to recognize the element appropriately.
     * @param attributeNames set of Strings
     */
    public void setAttributeNames(Set attributeNames) {
        this.attributeNames = attributeNames;
    }

    /**
     * Set processor prefix for the request attribute request parameter processor.
     * Defaults to <code>RenderingAttributeRequestParameterProcessor.DEFAULT_PROCESSOR_PREFIX</code>
     * @param processorPrefix
     */
    public void setProcessorPrefix(String processorPrefix) {
        this.processorPrefix = processorPrefix;
    }
}
