/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes.url;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.Serializable;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheable;
import org.jasig.portal.rendering.attributes.IRenderingAttributeProvider;
import org.jasig.portal.url.IPortalUrl;
import org.jasig.portal.url.IRequestParameterProcessor;
import org.jasig.portal.url.IServletPathExtender;
import org.jasig.portal.url.IUrlDecorator;
import org.jasig.portal.url.IWritableHttpServletRequest;

/**
 * Decorates URL by appending another path element, corresponding to the value of
 * a specified rendering parameter (attribute). Also implements appropriate request parameter processor
 * to read out the attribute value from incoming request and set pass it to the rendering
 * attribute provider. 
 * 
 * Note: requires {@link org.jasig.portal.url.IServletPathExtender} URL constructor to be configured in the list of url constructors.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class SingleAttributeDecoratorAndProcessor implements IRequestParameterProcessor, IUrlDecorator, ICacheable {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(SingleAttributeDecoratorAndProcessor.class);

    IRenderingAttributeProvider renderingAttributeProvider;
    String parameterName;
    String defaultParameterValue=null;
    int pathElementOffset=1;

    public void decorateUrl(IPortalUrl url) {
        Map parameterValues=renderingAttributeProvider.getParameterValues();
        if(parameterValues!=null) {
            String parameterValue=(String) parameterValues.get(parameterName);
            IServletPathExtender constructor=(IServletPathExtender) url.getUrlConstructor(IServletPathExtender.class.getName());
            if(parameterValue!=null) {
                if (logger.isDebugEnabled()) {
                    logger.debug("decorateUrl(IPortalUrl) - adding current value to the path : parameterName=" + parameterName+", value="+parameterValue);
                }
                constructor.addPathElement(parameterValue);
            } else {
                if(defaultParameterValue!=null) {
                    if (logger.isDebugEnabled()) {
                        logger.debug("decorateUrl(IPortalUrl) - adding default value to the path : parameterName=" + parameterName);
                    }

                    constructor.addPathElement(defaultParameterValue);
                }
            }
        }
    }

    public boolean canEvaluateVariables() {
        return false;
    }

    public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res) {
        String servletPath=req.getServletPath();
        int realPathElementOffset=pathElementOffset;
        if(servletPath.startsWith("/")) { realPathElementOffset++; }
        String[] pathElements=servletPath.split("/");
        if(pathElements.length > (realPathElementOffset+1)) {
            if (logger.isDebugEnabled()) {
                logger.debug("processParameters(IWritableHttpServletRequest, HttpServletResponse) - setting path-defined param value : name="+parameterName+", value=" + pathElements[realPathElementOffset]);
            }
            renderingAttributeProvider.setParameterValue(parameterName,pathElements[realPathElementOffset]);
        } else {
            if (logger.isDebugEnabled()) {
                logger.debug("processParameters(IWritableHttpServletRequest, HttpServletResponse) - too few path elements : servletPath=" + servletPath);
            }
        }
        return true;
    }

    public ICacheKey generateKey() {
        Map parameterValues=renderingAttributeProvider.getParameterValues();
        if(parameterValues!=null) {
            String parameterValue=(String) parameterValues.get(parameterName);
            if(parameterValue!=null) {
                return new CacheKeyImpl(parameterValue);
            }
        }
        return CacheKeyImpl.EMPTY_KEY;
    }

    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }

    /**
     * Specify rendering attribute provider
     * @param renderingAttributeProvider
     */
    public void setRenderingAttributeProvider(IRenderingAttributeProvider renderingAttributeProvider) {
        this.renderingAttributeProvider = renderingAttributeProvider;
    }

    /**
     * Name of the rendering parameter that should be incorporated into the url
     * @param parameterName
     */
    public void setParameterName(String parameterName) {
        this.parameterName = parameterName;
    }

    /**
     * Default value of the specified parameter.
     * @param defaultParameterValue or <code>null</code> if no additions to the path should be made
     * when the parameter is not set.
     */
    public void setDefaultParameterValue(String defaultParameterValue) {
        this.defaultParameterValue = defaultParameterValue;
    }

    /**
     * Specify offset at which the param will appear in the servlet path.
     * Default value is 1, which means that the param will appear as a second path element.
     * @param pathElementOffset number of elements prior to the location of the param
     */
    public void setPathElementOffset(int pathElementOffset) {
        this.pathElementOffset = pathElementOffset;
    }

}
