/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.hybrid;

import java.io.IOException;
import java.io.Serializable;
import java.io.StringWriter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheKeyList;
import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.rendering.IRenderingPipelineEventListener;
import org.jasig.portal.rendering.RenderingPipelineEvent;
import org.jasig.portal.rendering.sax.ISaxConsumer;
import org.jasig.portal.rendering.string.IStringCacheFilter;
import org.jasig.portal.rendering.string.IStringPipelineDownstreamComponent;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.ext.LexicalHandler;

/**
 * A cache filter that accepts SAX streams, serializing them and caching as strings. 
 * The normal usage would be to put it downstream of a SAX-generating element, and upstream of string-receiving elements.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public class SaxToStringCacheFilter implements IStringCacheFilter, IRenderingPipelineEventListener, ISaxConsumer, LexicalHandler {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(SaxToStringCacheFilter.class);

    boolean cacheOnStringInput;
    ISimpleCacheProviderFacade cache;
    
    public final ICacheKey dummyKey=new CacheKeyImpl("");
    
    private StringWriter currentWriter;
    private ContentHandler serializerContentHandler;
    private LexicalHandler serialzierLexicalHandler;
    
    protected XMLSerializer xmlSerializer;
    
    
    IStringPipelineDownstreamComponent stringHandler;
    
    ICacheKeyList currentKey=null;
    CacheEntry currentEntry=null;
    
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.string.IStringCacheFilter#getStringCacheValidityObjects()
     */
    public Serializable[] getStringCacheValidityObjects() {
        CacheEntry cacheEntry=this.getCurrentCacheEntry();
        if(cacheEntry!=null) {
            return cacheEntry.getValidityObjects();
        } else {
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.string.IStringPipelineUpstreamComponent#setStringHandler(org.jasig.portal.rendering.string.IStringPipelineDownstreamComponent)
     */
    public void setStringHandler(IStringPipelineDownstreamComponent downstreamFilter) {
        this.stringHandler=downstreamFilter;
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.ICacheFilter#setCurrentCacheKey(org.jasig.portal.caching.ICacheKeyList)
     */
    public void setCurrentCacheKey(ICacheKeyList cacheKey) {
        currentKey=cacheKey;
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.string.IStringCacheFilter#replayStringCache()
     */
    public boolean replayStringCache() {
        CacheEntry cacheEntry=this.getCurrentCacheEntry();
        if(cacheEntry!=null) {
            if (logger.isDebugEnabled()) {
                logger.debug("replayStringCache() - replaying string cache for key \""+this.currentKey.getKey().toString()+"\"");
            }

            stringHandler.filterRendering(cacheEntry.getContent());
            return true;
        } else {
            return false;
        }
    }
    /**
     * Sets if the filter should cache content that is give to it
     * in a string form (i.e. through String rendering' filter() method) and not in a form of sax.
     * @param cacheOnStringInput true if incoming strings should also be cached, false if only sax-based serialization should be cached
     */
    public void setCacheOnStringInput(boolean cacheOnStringInput) {
        this.cacheOnStringInput = cacheOnStringInput;
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.string.IStringFilter#canRenderString()
     */
    public boolean canRenderString() {
        return (stringHandler!=null && stringHandler.canRenderString());
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.string.IStringPipelineDownstreamComponent#filterRendering(java.lang.String)
     */
    public void filterRendering(String input) {
        // cache hasn't been requested, string has been supplied.
        // we could cache at this stage, but it's probably unnecessary
        if(cacheOnStringInput && currentKey!=null) {
            // cache
            CacheEntry entry=new CacheEntry(input,currentKey.getValidityObjectList());
            cache.putInCache(currentKey.getKey(),entry);
        }
        

    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.ISaxPipelineComponent#canRenderSax()
     */
    public boolean canRenderSax() {
        return canRenderString();
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.caching.ICacheable#generateKey()
     */
    public ICacheKey generateKey() {
        return dummyKey;
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.caching.ICacheable#isCacheValid(java.io.Serializable)
     */
    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }
    /**
     * @return Returns the cache.
     */
    public ISimpleCacheProviderFacade getCache() {
        return cache;
    }
    /**
     * @param cache The cache to set.
     */
    public void setCache(ISimpleCacheProviderFacade cache) {
        this.cache = cache;
    }
    
    CacheEntry getCurrentCacheEntry() {
        if(currentEntry==null) {
            Object currentCacheObject=cache.getFromCache(currentKey.getKey());
            if(currentCacheObject!=null) {
               currentEntry=(CacheEntry)currentCacheObject;
            }
        }
        return currentEntry;
    }
    
    class CacheEntry implements Serializable {

        Serializable[] validityObjects;
        String content;
        public CacheEntry(String content, Serializable[] validityObjects){
            this.content=content;
            this.validityObjects=validityObjects;
        }
        /**
         * @return Returns the content.
         */
        public String getContent() {
            return content;
        }
        /**
         * @param content The content to set.
         */
        public void setContent(String content) {
            this.content = content;
        }
        /**
         * @return Returns the validityObject.
         */
        public Serializable[] getValidityObjects() {
            return validityObjects;
        }
        /**
         * @param validityObjects The validityObject to set.
         */
        public void setValidityObjects(Serializable[] validityObjects) {
            this.validityObjects = validityObjects;
        }
    }
    
    void reset() throws PortalException {
        currentKey=null;
        currentEntry=null;
        currentWriter=new StringWriter();
        xmlSerializer.setOutputCharStream(currentWriter);
        try {
            serializerContentHandler=xmlSerializer.asContentHandler();
        } catch (IOException ioe) {
            throw new PortalException("unable to obtain content handler from serializer",ioe);
        }
        if(serializerContentHandler instanceof LexicalHandler) {
            serialzierLexicalHandler=(LexicalHandler) serializerContentHandler;
        } else {
            serialzierLexicalHandler=null;
        }
        
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.IRenderingPipelineEventListener#reveiveSAXPipelineEvent(org.jasig.portal.rendering.RenderingPipelineEvent)
     */
    public void receiveSAXPipelineEvent(RenderingPipelineEvent ev) throws PortalException {
        if(ev.getEventType()==RenderingPipelineEvent.startRenderingCycleEventType) {
            this.reset();
        }
    }
    
    /**
     * @param ch
     * @param start
     * @param length
     * @throws org.xml.sax.SAXException
     */
    public void characters(char[] ch, int start, int length) throws SAXException {
        serializerContentHandler.characters(ch, start, length);
    }
    /**
     * @throws org.xml.sax.SAXException
     */
    public void endDocument() throws SAXException {
        serializerContentHandler.endDocument();
        // extract serialized content
        String serializedContent=currentWriter.toString();
        // cache content
        if(currentKey!=null) {
            // cache
            CacheEntry entry=new CacheEntry(serializedContent,currentKey.getValidityObjectList());
            cache.putInCache(currentKey.getKey(),entry);
        }
        // pass content down the pipeline
        stringHandler.filterRendering(serializedContent);
    }
    /**
     * @param namespaceURI
     * @param localName
     * @param qName
     * @throws org.xml.sax.SAXException
     */
    public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
        serializerContentHandler.endElement(namespaceURI, localName, qName);
    }
    /**
     * @param prefix
     * @throws org.xml.sax.SAXException
     */
    public void endPrefixMapping(String prefix) throws SAXException {
        serializerContentHandler.endPrefixMapping(prefix);
    }
    /**
     * @param ch
     * @param start
     * @param length
     * @throws org.xml.sax.SAXException
     */
    public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
        serializerContentHandler.ignorableWhitespace(ch, start, length);
    }
    /**
     * @param target
     * @param data
     * @throws org.xml.sax.SAXException
     */
    public void processingInstruction(String target, String data) throws SAXException {
        serializerContentHandler.processingInstruction(target, data);
    }
    /**
     * @param locator
     */
    public void setDocumentLocator(Locator locator) {
        serializerContentHandler.setDocumentLocator(locator);
    }
    /**
     * @param name
     * @throws org.xml.sax.SAXException
     */
    public void skippedEntity(String name) throws SAXException {
        serializerContentHandler.skippedEntity(name);
    }
    /**
     * @throws org.xml.sax.SAXException
     */
    public void startDocument() throws SAXException {
        serializerContentHandler.startDocument();
    }
    /**
     * @param namespaceURI
     * @param localName
     * @param qName
     * @param atts
     * @throws org.xml.sax.SAXException
     */
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
        serializerContentHandler.startElement(namespaceURI, localName, qName, atts);
    }
    /**
     * @param prefix
     * @param uri
     * @throws org.xml.sax.SAXException
     */
    public void startPrefixMapping(String prefix, String uri) throws SAXException {
        serializerContentHandler.startPrefixMapping(prefix, uri);
    }
    /**
     * @param ch
     * @param start
     * @param length
     * @throws org.xml.sax.SAXException
     */
    public void comment(char[] ch, int start, int length) throws SAXException {
        if(serialzierLexicalHandler!=null) {
            serialzierLexicalHandler.comment(ch, start, length);
        }
    }
    /**
     * @throws org.xml.sax.SAXException
     */
    public void endCDATA() throws SAXException {
        if(serialzierLexicalHandler!=null) {
            serialzierLexicalHandler.endCDATA();
        }
    }
    /**
     * @throws org.xml.sax.SAXException
     */
    public void endDTD() throws SAXException {
        if(serialzierLexicalHandler!=null) {
            serialzierLexicalHandler.endDTD();
        }
    }
    /**
     * @param name
     * @throws org.xml.sax.SAXException
     */
    public void endEntity(String name) throws SAXException {
        if(serialzierLexicalHandler!=null) {
            serialzierLexicalHandler.endEntity(name);
        }
    }
    /**
     * @throws org.xml.sax.SAXException
     */
    public void startCDATA() throws SAXException {
        if(serialzierLexicalHandler!=null) {
            serialzierLexicalHandler.startCDATA();
        }
    }
    /**
     * @param name
     * @param publicId
     * @param systemId
     * @throws org.xml.sax.SAXException
     */
    public void startDTD(String name, String publicId, String systemId) throws SAXException {
        if(serialzierLexicalHandler!=null) {
            serialzierLexicalHandler.startDTD(name, publicId, systemId);
        }
    }
    /**
     * @param name
     * @throws org.xml.sax.SAXException
     */
    public void startEntity(String name) throws SAXException {
        if(serialzierLexicalHandler!=null) {
            serialzierLexicalHandler.startEntity(name);
        }
    }

    public void setOutputFormat(OutputFormat outputFormat) {
        this.xmlSerializer=new XMLSerializer(outputFormat);
    }
}
