/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.sax;

import java.io.IOException;
import java.io.Serializable;
import java.io.StringWriter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheable;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.ext.LexicalHandler;

/**
 * A filter that will log the passing content upon document end. 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class LoggingFilter extends SaxFilterImpl implements ICacheable {
    // injected properties
    /**
     * A message to be displayed before XML output in the log statement
     */
    String message="";
    
    
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(LoggingFilter.class);
    XMLSerializer xmlSerializer=new XMLSerializer(new OutputFormat("XML","US-ASCII",true));
    StringWriter currentWriter;
    ContentHandler serializerContentHandler;
    LexicalHandler serializerLexicalHandler;
    
    

    /**
     * Set XML output format
     * @param outputFormat
     */
    public void setOutputFormat(OutputFormat outputFormat) {
        this.xmlSerializer=new XMLSerializer(outputFormat);
    }


    /**
     * Set a method which should be written out before the XML output in the log statement
     * @param message
     */
    public void setMessage(String message) {
        this.message = message;
    }
    
    protected void initSerializer() throws SAXException {
        currentWriter=new StringWriter();
        xmlSerializer.setOutputCharStream(currentWriter);
        try {
            serializerContentHandler=xmlSerializer.asContentHandler();
            if(serializerContentHandler instanceof LexicalHandler) {
                serializerLexicalHandler=(LexicalHandler) serializerContentHandler;
            } else {
                serializerLexicalHandler=null;
            }
        } catch (IOException e) {
            throw new SAXException("unable to obtain content handler from the serializer",e);
        }
    }
    
    
    public void startDocument() throws SAXException {
        initSerializer();
        serializerContentHandler.startDocument();
        super.startDocument();
    }

    public void endDocument() throws SAXException {
        logger.debug(message+currentWriter.toString());
        super.endDocument();
        
    }

    public void characters(char[] ch, int start, int length) throws SAXException {
        serializerContentHandler.characters(ch,start,length);
        super.characters(ch, start, length);
    }



    public void comment(char[] ch, int start, int length) throws SAXException {
        if(serializerLexicalHandler!=null) {
            serializerLexicalHandler.comment(ch,start,length);
        }
        super.comment(ch, start, length);
    }



    public void endCDATA() throws SAXException {
        if(serializerLexicalHandler!=null) {
            serializerLexicalHandler.endCDATA();
        }
        super.endCDATA();
    }



    public void endDTD() throws SAXException {
        if(serializerLexicalHandler!=null) {
            lexicalHandler.endDTD();
        }
        super.endDTD();
    }



    public void endElement(String uri, String localName, String qName) throws SAXException {
        serializerContentHandler.endElement(uri,localName,qName);
        super.endElement(uri, localName, qName);
    }



    public void endEntity(String name) throws SAXException {
        if(serializerLexicalHandler!=null) {
            lexicalHandler.endEntity(name);
        }
        super.endEntity(name);
    }



    public void endPrefixMapping(String prefix) throws SAXException {
        serializerContentHandler.endPrefixMapping(prefix);
        super.endPrefixMapping(prefix);
    }



    public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
        serializerContentHandler.ignorableWhitespace(ch,start,length);
        super.ignorableWhitespace(ch, start, length);
    }



    public void processingInstruction(String target, String data) throws SAXException {
        serializerContentHandler.processingInstruction(target,data);
        super.processingInstruction(target, data);
    }



    public void skippedEntity(String name) throws SAXException {
        serializerContentHandler.skippedEntity(name);
        super.skippedEntity(name);
    }



    public void startCDATA() throws SAXException {
        if(serializerLexicalHandler!=null) {
            serializerLexicalHandler.startCDATA();
        }
        super.startCDATA();
    }


    public void startDTD(String name, String publicId, String systemId) throws SAXException {
        if(serializerLexicalHandler!=null) {
            serializerLexicalHandler.startDTD(name,publicId,systemId);
        }
        super.startDTD(name, publicId, systemId);
    }



    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
        serializerContentHandler.startElement(uri,localName,qName,atts);
        super.startElement(uri, localName, qName, atts);
    }



    public void startEntity(String name) throws SAXException {
        if(serializerLexicalHandler!=null) {
            lexicalHandler.startEntity(name);   
        }
        super.startEntity(name);
    }

    public void startPrefixMapping(String prefix, String uri) throws SAXException {
        serializerContentHandler.startPrefixMapping(prefix,uri);
        super.startPrefixMapping(prefix, uri);
    }


    public ICacheKey generateKey() {
        return CacheKeyImpl.EMPTY_KEY;
    }
    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }

}
