/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.sax;

import java.io.Serializable;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheable;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 * Injects static pattern in place of portlet title, to be replaced
 * by the real title later in the rendering pipeline.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 1.0$
 */
public class MockPortletTitleInjector extends SaxFilterImpl implements ICacheable {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(MockPortletTitleInjector.class);
    
    // pieces of the default title pattern
    static final String DEFAULT_TITLE_PATTERN_WINID="([\\w\\.]+)";
    static final String DEFAULT_TITLE_PATTERN_PREFIX="nM03Fvq7.";
    static final String DEFAULT_TITLE_PATTERN_POSTFIX=".aU6vu2N";
    
    /**
     * Default pattern for finding mock title string with encoded portlet window id
     */
    public static final String DEFAULT_TITLE_PATTERN=DEFAULT_TITLE_PATTERN_PREFIX+DEFAULT_TITLE_PATTERN_WINID+DEFAULT_TITLE_PATTERN_POSTFIX;

    String titlePattern=DEFAULT_TITLE_PATTERN;
    String titlePatternPrefix=DEFAULT_TITLE_PATTERN_PREFIX;
    String titlePatternPostfix=DEFAULT_TITLE_PATTERN_POSTFIX;
    
    String portletElementName="portlet";
    String portletWindowIdAttributeName="portletWindowId";
    String titleAttributeName="title";

    @Override
    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
        String elementName=localName==null?qName:localName;
        if(portletElementName.equals(elementName)) {
            String portletWindowId=atts.getValue(portletWindowIdAttributeName);
            if(portletWindowId!=null) {
                String windowTitle=titlePatternPrefix+portletWindowId+titlePatternPostfix;
                AttributesImpl ai=new AttributesImpl(atts);
                int titleIndex=atts.getIndex(titleAttributeName);
                if(titleIndex!=-1) {
                    // some title already exists, need to copy attributes one by one
                    ai.setAttribute(titleIndex,"",titleAttributeName,titleAttributeName,"CDATA",windowTitle);                    
                } else {
                    ai.addAttribute("",titleAttributeName,titleAttributeName,"CDATA",windowTitle);
                }
                super.startElement(uri,localName,qName,ai);
            } else {
                String attString="(";
                for(int i=0;i<atts.getLength();i++) {
                    attString+=atts.getQName(i)+"="+atts.getValue(i)+"  ";
                }
                attString+=")";
                logger.warn("startElement(String, String, String, Attributes) - unable determine portlet window id for a portlet element : uri=" + uri + ", localName=" + localName + ", qName=" + qName + ", atts=" + attString, null);
                super.startElement(uri, localName, qName, atts);
            }
        } else {
            super.startElement(uri, localName, qName, atts);
        }
    }

    public ICacheKey generateKey() {
        return CacheKeyImpl.EMPTY_KEY;
    }

    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }

    /**
     * Specify name of the portlet element (qName or localName). Defaults to "portlet".
     * @param portletElementName
     */
    public void setPortletElementName(String portletElementName) {
        this.portletElementName = portletElementName;
    }

    /**
     * Specify portlet window id attribute name on the portlet elements. Defaults to "portletWindowId"
     * @param portletWindowIdAttributeName
     */
    public void setPortletWindowIdAttributeName(String portletWindowIdAttributeName) {
        this.portletWindowIdAttributeName = portletWindowIdAttributeName;
    }

    public void setTitleAttributeName(String titleAttributeName) {
        this.titleAttributeName = titleAttributeName;
    }

    /**
     * Specify pattern to be used for representing portlet title with encoded portlet window id.
     * Defaults to <code>DEFAULT_TITLE_PATTERN</code>
     * @param titlePattern
     */
    public void setTitlePattern(String titlePattern) {
        this.titlePattern = titlePattern;
        
        // identify location of a variable match (portlet window id match)
        // to determine prefix and suffix strings
        int start=titlePattern.indexOf("(");
        int end=titlePattern.lastIndexOf(")");
        if(start==-1 || end==-1 || start>=end) {
            throw new IllegalArgumentException("titlePattern unable to find pranthesized variable");
        }
        
        this.titlePatternPrefix=titlePattern.substring(0,start);
        this.titlePatternPostfix=titlePattern.substring(end+1);
        
        if (logger.isDebugEnabled()) {
            logger.debug("setTitlePattern(String) - determined prefix and postfix : titlePattern=" + titlePattern+", prefix="+this.titlePatternPrefix+", postfix="+this.titlePatternPostfix);
        }
        
    }
    
    
}
