/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.sax;

import java.util.LinkedList;
import java.util.List;

import org.jasig.portal.rendering.sax.events.AbstractSaxEvent;
import org.jasig.portal.rendering.sax.events.content.CharactersEvent;
import org.jasig.portal.rendering.sax.events.content.EndDocumentEvent;
import org.jasig.portal.rendering.sax.events.content.EndElementEvent;
import org.jasig.portal.rendering.sax.events.content.EndPrefixMappingEvent;
import org.jasig.portal.rendering.sax.events.content.IgnorableWhitespaceEvent;
import org.jasig.portal.rendering.sax.events.content.ProcessingInstructionEvent;
import org.jasig.portal.rendering.sax.events.content.SkippedEntityEvent;
import org.jasig.portal.rendering.sax.events.content.StartDocumentEvent;
import org.jasig.portal.rendering.sax.events.content.StartElementEvent;
import org.jasig.portal.rendering.sax.events.content.StartPrefixMappingEvent;
import org.jasig.portal.rendering.sax.events.dtd.NotationDeclEvent;
import org.jasig.portal.rendering.sax.events.dtd.UnparsedEntityDeclEvent;
import org.jasig.portal.rendering.sax.events.error.ErrorEvent;
import org.jasig.portal.rendering.sax.events.error.FatalErrorEvent;
import org.jasig.portal.rendering.sax.events.error.WarningEvent;
import org.jasig.portal.rendering.sax.events.lexical.CommentEvent;
import org.jasig.portal.rendering.sax.events.lexical.EndCDATAEvent;
import org.jasig.portal.rendering.sax.events.lexical.EndDTDEvent;
import org.jasig.portal.rendering.sax.events.lexical.EndEntityEvent;
import org.jasig.portal.rendering.sax.events.lexical.StartCDATAEvent;
import org.jasig.portal.rendering.sax.events.lexical.StartDTDEvent;
import org.jasig.portal.rendering.sax.events.lexical.StartEntityEvent;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.DTDHandler;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.ext.LexicalHandler;

/**
 * A basic XML buffer implementation.
 *
 * @author <a href="mailto:pkharchenko@interactivebusiness.com">Peter Kharchenko</a>
 * @version $Revision: 11911 $
 */
public class SaxBufferImpl extends SaxFilterImpl {
    // holding collection
    protected final List<AbstractSaxEvent> events;

    // control flags
    protected boolean buffering;
    protected boolean outputAtDocumentEnd;
    protected boolean copyCharBlock = false;

    // constructors

    /**
     * Construct an empty XML buffer, with no parent.
     *
     * <p>This filter will have no parent: you must assign a parent
     * before you start a parse or do any configuration with
     * setFeature or setProperty.</p>
     * <p>By default, buffering is on, outputAtDocumentEnd is off.</p>
     *
     * @see org.xml.sax.XMLReader#setFeature
     * @see org.xml.sax.XMLReader#setProperty
     */
    public SaxBufferImpl() {
        super();
        buffering = true;
        outputAtDocumentEnd = false;
        events = new LinkedList<AbstractSaxEvent>();
    }

    /**
     * Construct an XML filter with the specified parent.
     * <p> Same default flag settings as with empty constructor</p>
     */
    public SaxBufferImpl(XMLReader parent) {
        this();
        setParent(parent);
    }

    /**
     * Construct an XML filter with the specified children.
     * <p> Same default flag settings as with empty constructor</p>
     */
    public SaxBufferImpl(ContentHandler ch, EntityResolver er, ErrorHandler eh, LexicalHandler lh, DTDHandler dh) {
        super(ch, er, eh, lh, dh);
        buffering = true;
        outputAtDocumentEnd = false;
        events = new LinkedList<AbstractSaxEvent>();
    }

    /**
     * Construct an XML filter based on a unified handler.
     * <p>
     * Same default flag settings as with empty constructor.
     */
    public SaxBufferImpl(ContentHandler ch) {
        super(ch);
        buffering = true;
        outputAtDocumentEnd=false;
        events = new LinkedList<AbstractSaxEvent>();
    }

    /**
     * Tells buffer to automatically output itself once
     * an end of a document is reached.
     * @param setting a <code>boolean</code> value
     */
    public void setOutputAtDocumentEnd(boolean setting) {
        this.outputAtDocumentEnd=setting;
    }

    public synchronized void clearBuffer() {
        events.clear();
    }

    public synchronized void stopBuffering () {
        buffering = false;
    }

    public synchronized void startBuffering ()  {
        buffering = true;
    }

    public synchronized boolean isEmpty() {
        return events.isEmpty();
    }


    /**
     * Outputs buffer's content to a current set of handlers.
     * Please note that this method is not thread-safe if
     * handlers are being reset by different threads.
     * Use the other {@link #outputBuffer(ContentHandler)} method in such cases.
     *
     * @exception SAXException if an error occurs
     */
    public void outputBuffer() throws SAXException {
        this.outputBuffer(this.contentHandler, this.dtdHandler, this.errorHandler, this.lexicalHandler);
    }


    /**
     * Outputs buffer's content to a specified content handler.
     * Please note that the contant handler can also represent
     * lexical handler, dtd handler and error handler.
     * This method is thread-safe.
     *
     * @param contentHandler a <code>ContenteHandler</code> value
     * @exception SAXException if an error occurs
     */
    public void outputBuffer(ContentHandler contentHandler) throws SAXException {
        if (contentHandler == null) {
            final IllegalStateException ise = new IllegalStateException("Trying to ouput buffer to a null ContentHandler");
            this.logger.error(ise.getMessage(), ise);
            return;
        }
        
        final DTDHandler dtdHandler;
        if (contentHandler instanceof DTDHandler) {
            dtdHandler = (DTDHandler)contentHandler;
        }
        else {
            dtdHandler = null;
        }

        final ErrorHandler errorHandler;
        if (contentHandler instanceof ErrorHandler) {
            errorHandler = (ErrorHandler)contentHandler;
        }
        else {
            errorHandler = null;
        }

        final LexicalHandler lexicalHandler;
        if (contentHandler instanceof LexicalHandler) {
            lexicalHandler = (LexicalHandler)contentHandler;
        }
        else {
            lexicalHandler = null;
        }
        
        this.outputBuffer(contentHandler, dtdHandler, errorHandler, lexicalHandler);
    }
    
    
    /**
     * Outputs buffer's content to a specified handlers. The ContentHandler is required, all others
     * may be null.
     *
     * This method is thread-safe. As long as the buffer is not being modified while this method is running.
     *
     * @exception SAXException if an error occurs
     */
    public void outputBuffer(ContentHandler contentHandler, DTDHandler dtdHandler, ErrorHandler errorHandler, LexicalHandler lexicalHandler) throws SAXException {
        if (contentHandler == null) {
            final IllegalStateException ise = new IllegalStateException("Trying to ouput buffer to a null ContentHandler");
            this.logger.error(ise.getMessage(), ise);
            return;
        }
        
        for (final AbstractSaxEvent event : this.events) {
            final AbstractSaxEvent.SaxEventType eventType = event.getEventType();
            
            switch (eventType) {
                // ContentHandler events
                case START_DOCUMENT:
                    contentHandler.startDocument();
                    break;
                case END_DOCUMENT:
                    contentHandler.endDocument();
                    break;
                case START_PREFIX_MAPPING:
                    final StartPrefixMappingEvent spme = (StartPrefixMappingEvent)event;
                    contentHandler.startPrefixMapping(spme.getPrefix(), spme.getUri());
                    break;
                case END_PREFIX_MAPPING:
                    final EndPrefixMappingEvent epme = (EndPrefixMappingEvent)event;
                    contentHandler.endPrefixMapping(epme.getPrefix());
                    break;
                case START_ELEMENT:
                    final StartElementEvent see = (StartElementEvent)event;
                    contentHandler.startElement(see.getUri(), see.getLocalName(), see.getQName(), see.getAttributes());
                    break;
                case SKIPPED_ENTITY:
                    final SkippedEntityEvent skee = (SkippedEntityEvent)event;
                    contentHandler.skippedEntity(skee.getName());
                    break;
                case END_ELEMENT:
                    final EndElementEvent eee = (EndElementEvent)event;
                    contentHandler.endElement(eee.getUri(), eee.getLocalName(), eee.getQName());
                    break;
                case CHARACTERS:
                    final CharactersEvent ce = (CharactersEvent)event;
                    contentHandler.characters(ce.getCharacters(), ce.getStart(), ce.getLength());
                    break;
                case IGNORABLE_WHITESPACE:
                    final IgnorableWhitespaceEvent iwe = (IgnorableWhitespaceEvent)event;
                    contentHandler.ignorableWhitespace(iwe.getCharacters(), iwe.getStart(), iwe.getLength());
                    break;
                case PROCESSING_INSTRUCTION:
                    final ProcessingInstructionEvent pie = (ProcessingInstructionEvent)event;
                    contentHandler.processingInstruction(pie.getTarget(), pie.getData());
                    break;

                //DTDHandler events
                case NOTATION_DECL:
                    if (dtdHandler != null) {
                        final NotationDeclEvent nde = (NotationDeclEvent)event;
                        dtdHandler.notationDecl(nde.getName(), nde.getPublicId(), nde.getSystemId());
                    }
                    break;
                case UNPARSED_ENTITY_DECL:
                    if (dtdHandler != null) {
                        final UnparsedEntityDeclEvent uede = (UnparsedEntityDeclEvent)event;
                        dtdHandler.unparsedEntityDecl(uede.getName(), uede.getPublicId(), uede.getSystemId(), uede.getNotationName());
                    }
                    break;
                
                //ErrorHandler events
                case WARNING:
                    if (errorHandler != null) {
                        final WarningEvent we = (WarningEvent)event;
                        errorHandler.warning(we.getSaxException());
                    }
                    break;
                case ERROR:
                    if (errorHandler != null) {
                        final ErrorEvent ee = (ErrorEvent)event;
                        errorHandler.error(ee.getSaxException());
                    }
                    break;
                case FATAL_ERROR:
                    if (errorHandler != null) {
                        final FatalErrorEvent fee = (FatalErrorEvent)event;
                        errorHandler.fatalError(fee.getSaxException());
                    }
                    break;
                
                // LexicalHandler events
                case START_DTD:
                    if (lexicalHandler != null) {
                        final StartDTDEvent sde = (StartDTDEvent)event;
                        lexicalHandler.startDTD(sde.getName(), sde.getPublicId(), sde.getSystemId());
                    }
                    break;
                case END_DTD:
                    if (lexicalHandler != null) {
                        lexicalHandler.endDTD();
                    }
                    break;
                case START_ENTITY:
                    if (lexicalHandler != null) {
                        final StartEntityEvent seev = (StartEntityEvent)event;
                        lexicalHandler.startEntity(seev.getName());
                    }
                    break;
                case END_ENTITY:
                    if (lexicalHandler != null) {
                        final EndEntityEvent eeev = (EndEntityEvent)event;
                        lexicalHandler.endEntity(eeev.getName());
                    }
                    break;
                case START_CDATA:
                    if (lexicalHandler != null) {
                        lexicalHandler.startCDATA();
                    }
                    break;
                case END_CDATA:
                    if (lexicalHandler != null) {
                        lexicalHandler.endCDATA();
                    }
                    break;
                case COMMENT:
                    if (lexicalHandler != null) {
                        final CommentEvent cev = (CommentEvent)event;
                        lexicalHandler.comment(cev.getCharacters(), cev.getStart(), cev.getLength());
                    }
                    break;
            }
        }
    }
    
    // Implementation of org.xml.sax.ext.LexicalHandler

    public void startDTD(String name, String publicId, String systemId) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new StartDTDEvent(name, publicId, systemId);
            this.events.add(event);
        }
        else if (lexicalHandler != null) {
            lexicalHandler.startDTD(name, publicId, systemId);
        }
    }

    public void endDTD() throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new EndDTDEvent();
            this.events.add(event);
        }
        else if (lexicalHandler != null) {
            lexicalHandler.endDTD();
        }
    }

    public void startEntity(String name) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new StartEntityEvent(name);
            this.events.add(event);
        }
        else if (lexicalHandler != null) {
            lexicalHandler.startEntity(name);
        }
    }

    public void endEntity(String name) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new EndEntityEvent(name);
            this.events.add(event);
        }
        else if (lexicalHandler != null) {
            lexicalHandler.endEntity(name);
        }
    }

    public void startCDATA() throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new StartCDATAEvent();
            this.events.add(event);
        }
        else if (lexicalHandler != null) {
            lexicalHandler.startCDATA();
        }
    }

    public void endCDATA() throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new EndCDATAEvent();
            this.events.add(event);
        }
        else if (lexicalHandler != null) {
            lexicalHandler.endCDATA();
        }
    }

    public void comment(char ch[], int start, int length) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new CommentEvent(ch, start, length);
            this.events.add(event);
        }
        else if (lexicalHandler != null) {
            lexicalHandler.comment(ch, start, length);
        }
    }

    // Implementation of org.xml.sax.DTDHandler
    public void notationDecl(String name, String publicId, String systemId) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new NotationDeclEvent(name, publicId, systemId);
            this.events.add(event);
        }
        else if (dtdHandler != null) {
            dtdHandler.notationDecl(name, publicId, systemId);
        }
    }

    public void unparsedEntityDecl(String name, String publicId, String systemId, String notationName) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new UnparsedEntityDeclEvent(name, publicId, systemId, notationName);
            this.events.add(event);
        }
        else if (dtdHandler != null) {
            dtdHandler.unparsedEntityDecl(name, publicId, systemId, notationName);
        }
    }

    // Implementation of org.xml.sax.ContentHandler.
    public void setDocumentLocator(Locator locator) {
        this.locator = locator;
        if (!buffering) {
            if (contentHandler != null) {
                contentHandler.setDocumentLocator(locator);
            }
        }
    }

    public void startDocument() throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new StartDocumentEvent();
            this.events.add(event);
        }
        else if (contentHandler != null) {
            contentHandler.startDocument();
        }
    }

    public void endDocument() throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new EndDocumentEvent();
            this.events.add(event);

            if (outputAtDocumentEnd) {
                this.stopBuffering();
                this.outputBuffer();
            }
        }
        else if (contentHandler != null) {
            contentHandler.endDocument();
        }
    }

    public void startPrefixMapping(String prefix, String uri) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new StartPrefixMappingEvent(prefix, uri);
            this.events.add(event);
        }
        else if (contentHandler != null) {
            contentHandler.startPrefixMapping(prefix, uri);
        }
    }

    public void endPrefixMapping(String prefix) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new EndPrefixMappingEvent(prefix);
            this.events.add(event);
        }
        else if (contentHandler != null) {
            contentHandler.endPrefixMapping(prefix);
        }
    }

    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new StartElementEvent(uri, localName, qName, atts);
            this.events.add(event);
        }
        else if (contentHandler != null) {
            contentHandler.startElement(uri, localName, qName, atts);
        }
    }

    public void endElement(String uri, String localName, String qName) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new EndElementEvent(uri, localName, qName);
            this.events.add(event);
        }
        else if (contentHandler != null) {
            contentHandler.endElement(uri, localName, qName);
        }
    }

    public void characters(char ch[], int start, int length) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new CharactersEvent(ch, start, length);
            this.events.add(event);
        }
        else if (contentHandler != null) {
            contentHandler.characters(ch, start, length);
        }
    }

    public void ignorableWhitespace(char ch[], int start, int length) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new IgnorableWhitespaceEvent(ch, start, length);
            this.events.add(event);
        }
        else if (contentHandler != null) {
            contentHandler.ignorableWhitespace(ch, start, length);
        }
    }

    public void processingInstruction(String target, String data) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new ProcessingInstructionEvent(target, data);
            this.events.add(event);
        }
        else if (contentHandler != null) {
            contentHandler.processingInstruction(target, data);
        }
    }

    public void skippedEntity(String name) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new SkippedEntityEvent(name);
            this.events.add(event);
        }
        else if (contentHandler != null) {
            contentHandler.skippedEntity(name);
        }
    }

    // Implementation of org.xml.sax.ErrorHandler.

    public void warning(SAXParseException e) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new WarningEvent(e);
            this.events.add(event);
        }
        else if (errorHandler != null) {
            errorHandler.warning(e);
        }
    }

    public void error(SAXParseException e) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new ErrorEvent(e);
            this.events.add(event);
        }
        else if (errorHandler != null) {
            errorHandler.error(e);
        }
    }

    public void fatalError(SAXParseException e) throws SAXException {
        if (buffering) {
            final AbstractSaxEvent event = new FatalErrorEvent(e);
            this.events.add(event);
        }
        else if (errorHandler != null) {
            errorHandler.fatalError(e);
        }
    }
}
