/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.sax;

import java.io.IOException;
import java.io.Serializable;

import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheKeyList;
import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.rendering.IRenderingPipelineEventListener;
import org.jasig.portal.rendering.RenderingPipelineEvent;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.DTDHandler;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.SAXParseException;
import org.xml.sax.ext.LexicalHandler;

/**
 * Default implementation of ISaxCacheFilter with buffering support.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class SaxCacheFilterImpl implements IRenderingPipelineEventListener, ISaxCacheFilter, EntityResolver, DTDHandler, ContentHandler, ErrorHandler, LexicalHandler {

    public final ICacheKey dummyKey=new CacheKeyImpl("");
    
    private SaxBufferImpl buffer=new SaxBufferImpl();
    
    ISimpleCacheProviderFacade cache;
    
    ICacheKeyList currentKey=null;
    CacheEntry currentEntry=null;
    boolean contentDemanded=false;
    
    // handlers
    ContentHandler contentHandler=null;
    DTDHandler dtdHandler=null;
    LexicalHandler lexicalHandler=null;
    ErrorHandler errorHandler=null;
    EntityResolver entityResolver=null;
    
    ISaxConsumer consumer=null;
    
    CacheEntry getCurrentCacheEntry() {
        if(currentEntry==null) {
            Object currentCacheObject=cache.getFromCache(currentKey.getKey());
            if(currentCacheObject!=null) {
               currentEntry=(CacheEntry)currentCacheObject;
            }
        }
        return currentEntry;
    }

    public Serializable[] getSaxCacheValidityObjects() {
        CacheEntry cacheEntry=this.getCurrentCacheEntry();
        if(cacheEntry!=null) {
            return cacheEntry.getValidityObjects();
        } else {
            return null;
        }
    }

    public boolean replaySaxCache() throws SAXException {
        CacheEntry cacheEntry=this.getCurrentCacheEntry();
        if(cacheEntry!=null) {
            contentDemanded=true;
            cacheEntry.getContent().outputBuffer(this.contentHandler);
            return true;
        } else {
            return false;
        }
    }

    public void setCurrentCacheKey(ICacheKeyList cacheKey) {
        currentKey=cacheKey;
    }

    public ICacheKey generateKey() {
        return this.dummyKey;
    }

    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }

    /**
     * @param buffer The buffer to set.
     */
    void setBuffer(SaxBufferImpl buffer) {
        this.buffer = buffer;
    }

    /**
     * @return Returns the buffer.
     */
    SaxBufferImpl getBuffer() {
        return buffer;
    }
    
    // ISaxProducer methods
    /**
     * @param handler
     */
    public void setContentHandler(ContentHandler handler) {
        this.contentHandler=handler;
    }
    
    /**
     * @param handler
     */
    public void setDTDHandler(DTDHandler handler) {
        this.dtdHandler=handler;
    }
    
    /**
     * @param resolver
     */
    public void setEntityResolver(EntityResolver resolver) {
        this.entityResolver=resolver;
    }
    
    /**
     * @param handler
     */
    public void setErrorHandler(ErrorHandler handler) {
        this.errorHandler=handler;
    }
    
    /**
     * @param handler
     */
    public void setLexicalHandler(LexicalHandler handler) {
        this.lexicalHandler=handler;
    }

    public boolean isContenteHandlerSet() {
        return this.contentHandler==null;
    }
    
    /**
     * @param ch
     */
    public void setAllHandlers(ContentHandler ch) {
        setContentHandler(ch);
        if(ch instanceof EntityResolver) {
            setEntityResolver((EntityResolver)ch);
        }
        if(ch instanceof ErrorHandler) {
            setErrorHandler((ErrorHandler)ch);
        }
        if(ch instanceof DTDHandler) {
            setDTDHandler((DTDHandler)ch);
        }
        if(ch instanceof LexicalHandler) {
            setLexicalHandler((LexicalHandler)ch);
        }
        buffer.setAllHandlers(ch);
        buffer.setOutputAtDocumentEnd(true);
    }
        
    public void setSaxConsumer(ISaxConsumer consumer) {
        this.consumer=consumer;
        this.setAllHandlers(consumer);
    }

    public boolean canRenderSax() {
        return (consumer!=null && consumer.canRenderSax());
    }

    /**
     * @param ch
     * @param start
     * @param length
     * @throws org.xml.sax.SAXException
     */
    public void characters(char[] ch, int start, int length) throws SAXException {
        getBuffer().characters(ch, start, length);
    }

    /**
     * @param ch
     * @param start
     * @param length
     * @throws org.xml.sax.SAXException
     */
    public void comment(char[] ch, int start, int length) throws SAXException {
        getBuffer().comment(ch, start, length);
    }

    /**
     * @throws org.xml.sax.SAXException
     */
    public void endCDATA() throws SAXException {
        getBuffer().endCDATA();
    }

    /**
     * @throws org.xml.sax.SAXException
     */
    public void endDocument() throws SAXException {
        getBuffer().endDocument();
    }

    /**
     * @throws org.xml.sax.SAXException
     */
    public void endDTD() throws SAXException {
        getBuffer().endDTD();
    }

    /**
     * @param namespaceURI
     * @param localName
     * @param qName
     * @throws org.xml.sax.SAXException
     */
    public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
        getBuffer().endElement(namespaceURI, localName, qName);
    }

    /**
     * @param name
     * @throws org.xml.sax.SAXException
     */
    public void endEntity(String name) throws SAXException {
        getBuffer().endEntity(name);
    }

    /**
     * @param prefix
     * @throws org.xml.sax.SAXException
     */
    public void endPrefixMapping(String prefix) throws SAXException {
        getBuffer().endPrefixMapping(prefix);
    }

    /**
     * @param exception
     * @throws org.xml.sax.SAXException
     */
    public void error(SAXParseException exception) throws SAXException {
        getBuffer().error(exception);
    }

    /**
     * @param exception
     * @throws org.xml.sax.SAXException
     */
    public void fatalError(SAXParseException exception) throws SAXException {
        getBuffer().fatalError(exception);
    }

    /**
     * @param ch
     * @param start
     * @param length
     * @throws org.xml.sax.SAXException
     */
    public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
        getBuffer().ignorableWhitespace(ch, start, length);
    }

    /**
     * @param name
     * @param publicId
     * @param systemId
     * @throws org.xml.sax.SAXException
     */
    public void notationDecl(String name, String publicId, String systemId) throws SAXException {
        getBuffer().notationDecl(name, publicId, systemId);
    }

    /**
     * @param target
     * @param data
     * @throws org.xml.sax.SAXException
     */
    public void processingInstruction(String target, String data) throws SAXException {
        getBuffer().processingInstruction(target, data);
    }

    /**
     * @param publicId
     * @param systemId
     * @throws SAXException
     * @throws IOException
     */
    public InputSource resolveEntity(String publicId, String systemId) throws SAXException, IOException {
        return getBuffer().resolveEntity(publicId, systemId);
    }

    /**
     * @param locator
     */
    public void setDocumentLocator(Locator locator) {
        getBuffer().setDocumentLocator(locator);
    }

    /**
     * @param name
     * @param state
     * @throws SAXNotRecognizedException
     * @throws SAXNotSupportedException
     */
    public void setFeature(String name, boolean state) throws SAXNotRecognizedException, SAXNotSupportedException {
        getBuffer().setFeature(name, state);
    }

    /**
     * @param name
     * @throws org.xml.sax.SAXException
     */
    public void skippedEntity(String name) throws SAXException {
        getBuffer().skippedEntity(name);
    }

    /**
     * @throws org.xml.sax.SAXException
     */
    public void startCDATA() throws SAXException {
        getBuffer().startCDATA();
    }

    /**
     * @throws org.xml.sax.SAXException
     */
    public void startDocument() throws SAXException {
        getBuffer().startDocument();
    }

    /**
     * @param name
     * @param publicId
     * @param systemId
     * @throws org.xml.sax.SAXException
     */
    public void startDTD(String name, String publicId, String systemId) throws SAXException {
        getBuffer().startDTD(name, publicId, systemId);
    }

    /**
     * @param namespaceURI
     * @param localName
     * @param qName
     * @param atts
     * @throws org.xml.sax.SAXException
     */
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
        getBuffer().startElement(namespaceURI, localName, qName, atts);
    }

    /**
     * @param name
     * @throws org.xml.sax.SAXException
     */
    public void startEntity(String name) throws SAXException {
        getBuffer().startEntity(name);
    }

    /**
     * @param prefix
     * @param uri
     * @throws org.xml.sax.SAXException
     */
    public void startPrefixMapping(String prefix, String uri) throws SAXException {
        getBuffer().startPrefixMapping(prefix, uri);
    }

    /**
     * @param name
     * @param publicId
     * @param systemId
     * @param notationName
     * @throws org.xml.sax.SAXException
     */
    public void unparsedEntityDecl(String name, String publicId, String systemId, String notationName) throws SAXException {
        getBuffer().unparsedEntityDecl(name, publicId, systemId, notationName);
    }

    /**
     * @param exception
     * @throws org.xml.sax.SAXException
     */
    public void warning(SAXParseException exception) throws SAXException {
        getBuffer().warning(exception);
    }

    /**
     * @param cache The cache to set.
     */
    public void setCache(ISimpleCacheProviderFacade cache) {
        this.cache = cache;
    }
    
    public void receiveSAXPipelineEvent(RenderingPipelineEvent ev) throws PortalException {
        if(ev.getEventType()==RenderingPipelineEvent.startRenderingCycleEventType) {
            this.reset();
        } else if(ev.getEventType()==RenderingPipelineEvent.endRenderingCycleEventType) {
            // decide if cache should be recorded
            if(!buffer.isEmpty() && this.currentKey!=null) {
                CacheEntry entry=new CacheEntry(this.buffer,currentKey.getValidityObjectList());
                this.buffer=new SaxBufferImpl();
                buffer.setOutputAtDocumentEnd(true);
                buffer.setAllHandlers(contentHandler);
                cache.putInCache(currentKey.getKey(),entry);
            }
        }
    }
    
    void reset() {
        currentKey=null;
        currentEntry=null;
        contentDemanded=false;
        this.buffer.clearBuffer();
    }
    
    class CacheEntry {
        Serializable[] validityObjects;
        SaxBufferImpl content;
        public CacheEntry(SaxBufferImpl content, Serializable[] validityObjects){
            this.content=content;
            this.validityObjects=validityObjects;
        }
        /**
         * @return Returns the content.
         */
        public SaxBufferImpl getContent() {
            return content;
        }
        /**
         * @param content The content to set.
         */
        public void setContent(SaxBufferImpl content) {
            this.content = content;
        }
        /**
         * @return Returns the validityObject.
         */
        public Serializable[] getValidityObjects() {
            return validityObjects;
        }
        /**
         * @param validityObjects The validityObject to set.
         */
        public void setValidityObjects(Serializable[] validityObjects) {
            this.validityObjects = validityObjects;
        }
    }
}
