/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.sax;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jasig.portal.rendering.PipelineComponentManager;

/**
 * An implementation of the pipeline component manager for configuring
 * pure SAX rendering pipelines.
 * The pipeline is explicitly separted into three parts: first element (which must
 * implement {@link org.jasig.portal.rendering.sax.ISaxRenderer}, a list of
 * sax filters {@link org.jasig.portal.rendering.sax.ISaxFilter}, and a last
 * component, typically a serializer, which must implement {@link org.jasig.portal.rendering.sax.ISaxConsumer}.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class SaxPipelineComponentManager extends PipelineComponentManager {
    protected List pipelineFilters=null;
    protected ISaxRenderer saxSource=null;
    protected ISaxConsumer saxSerializer=null;
    
    
    /**
     * Set the first element of the rendering pipeline.
     * @param saxSource first element
     */
    public void setSaxSource(ISaxRenderer saxSource) {
        super.setFirstPipelineElement(saxSource);
        this.saxSource=saxSource;
    }

    /**
     * Set the last element of the rendering pipeline.
     * @param saxSerializer
     */
    public void setSaxSerializer(ISaxConsumer saxSerializer) {
        this.saxSerializer = saxSerializer;
        connectAndReportDownstreamElements();
    }

    /**
     * Set a list of pipeline filters. All filters must implement {@link ISaxFilter}.
     * @param pipelineFilters pipeline filters, can be left unset, or set to empty.
     */
    public void setPipelineFilters(List pipelineFilters) {
        this.pipelineFilters = pipelineFilters;
        if(saxSerializer!=null) {
            // last element has been set, so we can report all downstream elements to the super
            connectAndReportDownstreamElements();
        }
    }
    
    /**
     * The method connects components together and reports all of the downstream
     * components to the super implementation.
     */
    protected void connectAndReportDownstreamElements() {
        // assemble components
        List downstreamElements=new ArrayList();
        if(pipelineFilters!=null) {
            downstreamElements.addAll(pipelineFilters);
        }
        downstreamElements.add(saxSerializer);
        
        // connect components
        Iterator iter=downstreamElements.iterator();
        ISaxConsumer currentComponent=(ISaxConsumer)iter.next();
        saxSource.setSaxConsumer(currentComponent);
        while(iter.hasNext()) {
            ISaxProducer previousComponent=(ISaxProducer)currentComponent;
            currentComponent=(ISaxConsumer)iter.next();
            previousComponent.setSaxConsumer(currentComponent);
        }
  
        super.setDownstreamElements(downstreamElements);
    }

    /**
     * @return Returns the pipelineFilters.
     */
    public List getPipelineFilters() {
        return pipelineFilters;
    }

    /**
     * @return Returns the saxSerializer.
     */
    public ISaxConsumer getSaxSerializer() {
        return saxSerializer;
    }

    /**
     * @return Returns the saxSource.
     */
    public ISaxRenderer getSaxSource() {
        return saxSource;
    }
}
