/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.sax.serialize;

import java.io.IOException;
import java.util.Map;

import javax.servlet.ServletResponse;

import org.apache.xml.serialize.Serializer;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.rendering.IRenderingPipelineEventListener;
import org.jasig.portal.rendering.RenderingPipelineEvent;
import org.jasig.portal.rendering.sax.ISaxConsumer;
import org.springframework.beans.factory.BeanFactory;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.DTDHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.ext.LexicalHandler;

/**
 * A default implementation of a response serializer.
 * This implementation is configured by defining a map between
 * response MIME types and bean names pointing to the <code>org.apache.xml.serialize.Serializer</code>
 * implementations. A default MIME type can also be configured.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public class ResponseSerializerImpl implements ISaxConsumer, ContentHandler, LexicalHandler, DTDHandler, IRenderingPipelineEventListener {
    Map serializerMimeTypeMapping;
    BeanFactory factory;
	ServletResponse res;
	
    ContentHandler currentContentHandler;
    LexicalHandler currentLexicalhandler;
    DTDHandler currentDtdHandler;
	String currentSerializerName;
	Serializer currentSerializer;
    
	String defaultMimeType;
	String desiredMimeType;
	

    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.IRenderingPipelineEventListener#reveiveSAXPipelineEvent(org.jasig.portal.rendering.RenderingPipelineEvent)
     */
    public void receiveSAXPipelineEvent(RenderingPipelineEvent ev) throws PortalException {
        if(ev.getEventType()==RenderingPipelineEvent.startRenderingCycleEventType) {
            this.setServletResponse(ev.getHttpServletResponse());
        } else if(ev.getEventType()==RenderingPipelineEvent.endRenderingCycleEventType) {
            this.setServletResponse(null);
        }
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.ISaxPipelineComponent#canRenderSax()
     */
    public boolean canRenderSax() {
        return res!=null;
    }
    /**
     * In Servlet 2.4 standard, the mime type can be obtained
     * from the response object itself, however in 2.3 we have to 
     * rely on one of the preceeding pipeline elements to report
     * the target mime type directly to the response serializer.
     * 
     * @param suppliedMimeType The desired Mime Type to set.
     */
    public void setDesiredMimeType(String suppliedMimeType) {
        this.desiredMimeType = suppliedMimeType;
    }
    /**
     * @return Returns the desiredMimeTYpe.
     */
    String getDesiredMimeType() {
        return desiredMimeType;
    }
    /**
     * Define the mapping between MIME types <code>String</code>s and relative
     * bean names (also <code>String</code>) of the <code>IBaseSAXSerializer</code> implementations.
     * 
     * @param serializerNames a <code>String</code> to <code>String</code>
     */
    public void setSerializerMimeTypeMapping(Map serializerNames) {
        this.serializerMimeTypeMapping = serializerNames;
    }

    /**
     * @return Returns the currentSerializer.
     */
    ContentHandler getCurrentContentHandler() throws SAXException {
        if(currentSerializer==null) {
            // determine what serializer should be used for the new mime type
            String desiredSerializer=null;
            if(getDesiredMimeType()!=null) {
                desiredSerializer=(String)serializerMimeTypeMapping.get(getDesiredMimeType());
            }
            if(desiredSerializer==null) {
                desiredSerializer=(String)serializerMimeTypeMapping.get(defaultMimeType);
            }
            if(desiredSerializer==null) {
                throw new SAXException("Unable to locate default serializer (default MIME type "+defaultMimeType);
            }
            // check if the new serializer should be obtained, or the old one reset
            currentSerializer=(Serializer) factory.getBean(desiredSerializer, Serializer.class);
            currentSerializerName=desiredSerializer;
            
            // set the output stream
            try {
                currentSerializer.setOutputCharStream(res.getWriter());
            } catch (IOException e) {
                throw new SAXException("Unable to set the output stream for the serializer",e);
            }
            
            try {
                currentContentHandler=currentSerializer.asContentHandler();
            } catch (IOException e1) {
                throw new SAXException("unable to obtain content hander",e1);
            }
            
            if(currentContentHandler instanceof DTDHandler) {
                currentDtdHandler=(DTDHandler) currentContentHandler;
            } else {
                currentDtdHandler=null;
            }
            
            if(currentContentHandler instanceof LexicalHandler) {
                currentLexicalhandler=(LexicalHandler) currentContentHandler;
            } else {
                currentLexicalhandler=null;
            }
            
            if(currentSerializer==null) {
                throw new SAXException("Unable to instantiate serializer under the relative name \""+desiredSerializer+"\"");
            }

        }
	            
        return currentContentHandler;
    }
    
    LexicalHandler getCurrentLexicalHandler() throws SAXException {
        getCurrentContentHandler();
        return currentLexicalhandler;
    }
    
    DTDHandler getCurrentDtdHandler() throws SAXException {
        getCurrentContentHandler();
        return currentDtdHandler;
    }
    
    /**
     * @return Returns the existingSerializerName.
     */
    String getCurrentSerializerName() {
        return currentSerializerName;
    }
    /**
     * Sets the factory in which the ResponseSerializerImpl is declared.
     * All serializer names will be resolved relative to this factory
     * @param factory The factory to set.
     */
    public void setFactory(BeanFactory factory) {
        this.factory = factory;
    }
    
    /**
     * Sets the MIME type to be used when a serializer mapping
     * for the desired MIME type is not available
     * @param defaultMimeType The defaultMimeType to set.
     */
    public void setDefaultMimeType(String defaultMimeType) {
        this.defaultMimeType = defaultMimeType;
    }
	
	/* (non-Javadoc)
	 * @see org.jasig.portal.rendering.sax.serialize.IResponseSerializer#setServletResponse(javax.servlet.ServletResponse)
	 */
	public void setServletResponse(ServletResponse res) {
		this.res=res;
		currentSerializer=null;
	}
	
	public void startDocument() throws SAXException {
        // proceed with serialization
        this.getCurrentContentHandler().startDocument();
	}

	
	// delegate methods
	
	public void characters(char[] arg0, int arg1, int arg2) throws SAXException {
	    if(currentContentHandler==null) {
		      throw new SAXException("characters() called before startDocument()!");   
		}
		currentContentHandler.characters(arg0, arg1, arg2);
	}

	public void endDocument() throws SAXException {
	    if(currentContentHandler==null) {
		      throw new SAXException("endDocument() called before startDocument()!");   
		}
		currentContentHandler.endDocument();
	}

	public void endElement(String arg0, String arg1, String arg2) throws SAXException {
	    if(currentContentHandler==null) {
		      throw new SAXException("endElement() called before startDocument()!");   
		}
		currentContentHandler.endElement(arg0, arg1, arg2);
	}

	public void endPrefixMapping(String arg0) throws SAXException {
	    if(currentContentHandler==null) {
		      throw new SAXException("endPrefixMapping() called before startDocument()!");   
		}
		currentContentHandler.endPrefixMapping(arg0);
	}

	public void ignorableWhitespace(char[] arg0, int arg1, int arg2) throws SAXException {
		this.getCurrentContentHandler().ignorableWhitespace(arg0, arg1, arg2);
	}

	public void processingInstruction(String arg0, String arg1) throws SAXException {
		this.getCurrentContentHandler().processingInstruction(arg0, arg1);
	}

	public void setDocumentLocator(Locator arg0) {
	    try {
	        this.getCurrentContentHandler().setDocumentLocator(arg0);
	    } catch (SAXException se) {};
	}

	public void skippedEntity(String arg0) throws SAXException {
		this.getCurrentContentHandler().skippedEntity(arg0);
	}

	public void startElement(String arg0, String arg1, String arg2, Attributes arg3) throws SAXException {
	    if(currentContentHandler==null) {
		      throw new SAXException("startElement() called before startDocument()!");   
		}
        currentContentHandler.startElement(arg0, arg1, arg2, arg3);
	}

	public void startPrefixMapping(String arg0, String arg1) throws SAXException {
		this.getCurrentContentHandler().startPrefixMapping(arg0, arg1);
	}

	public void notationDecl(String arg0, String arg1, String arg2) throws SAXException {
		this.getCurrentDtdHandler().notationDecl(arg0, arg1, arg2);
	}

	public void unparsedEntityDecl(String arg0, String arg1, String arg2, String arg3) throws SAXException {
		this.getCurrentDtdHandler().unparsedEntityDecl(arg0, arg1, arg2, arg3);
	}

	public void comment(char[] arg0, int arg1, int arg2) throws SAXException {
		this.getCurrentLexicalHandler().comment(arg0, arg1, arg2);
	}

	public void endCDATA() throws SAXException {
	    if(currentLexicalhandler==null) {
	      throw new SAXException("endCDATA() called before startDocument()!");   
	    }
        currentLexicalhandler.endCDATA();
	}

	public void endDTD() throws SAXException {
	    if(currentLexicalhandler==null) {
		      throw new SAXException("endDTD() called before startDocument()!");   
		}
        currentLexicalhandler.endDTD();
	}

	public void endEntity(String arg0) throws SAXException {
	    if(currentLexicalhandler==null) {
		      throw new SAXException("endEntity() called before startDocument()!");   
		}	    
        currentLexicalhandler.endEntity(arg0);
	}

	public void startCDATA() throws SAXException {
	    if(currentContentHandler==null) {
		      throw new SAXException("startCDATA() called before startDocument()!");   
		}	    
        currentLexicalhandler.startCDATA();
	}

	public void startDTD(String arg0, String arg1, String arg2) throws SAXException {
		this.getCurrentLexicalHandler().startDTD(arg0, arg1, arg2);
	}

	public void startEntity(String arg0) throws SAXException {
		this.getCurrentLexicalHandler().startEntity(arg0);
	}

}
