/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.sax.xslt;

import java.io.Serializable;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.CacheKeyListImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.context.common.profile.IProfile;
import org.jasig.portal.context.common.profile.IProfileController;
import org.jasig.portal.context.common.profile.IProfileControllerEventListener;
import org.jasig.portal.context.common.profile.ProfileId;
import org.jasig.portal.context.common.profile.ProfileControllerEvent;
import org.jasig.portal.core.exception.PortalRuntimeException;
import org.jasig.portal.rendering.attributes.IRenderingAttributeProvider;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanFactoryAware;
import org.springframework.beans.factory.annotation.Required;

/**
 * A facade that determines {@link org.jasig.portal.rendering.sax.xslt.ITransformerPreferences} based
 * on the current profile.
 * 
 * The name of the transformer preferences property (setTransformerPreferencesPropertyName) is used to
 * retrieve a transformer preferences bean name. The bean is acquired from the same BeanFactory in which
 * the ProfileTransformerPreferences itself is configured.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class ProfileTransformerPreferences implements ITransformerPreferences, IProfileControllerEventListener, BeanFactoryAware {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(ProfileTransformerPreferences.class);

    String transformerPreferencesPropertyName;
    IProfileController profileController;
    List<String> paramNames=null;  
    
    BeanFactory ownBeanFactory=null;
    
    ITransformerPreferences currentTransformerPreferences=null;
    
    protected ITransformerPreferences getCurrentTransformerPreferences() {
        if(currentTransformerPreferences==null) {
            IProfile currentProfile=profileController.getCurrentProfile();
            if(currentProfile!=null) {
                String transformerPreferencesBeanName;
                try {
                    transformerPreferencesBeanName=(String) currentProfile.getProperty(transformerPreferencesPropertyName);
                    if (logger.isDebugEnabled()) {
                        logger.debug("getCurrentTransformerPreferences() - acquired profile-mapped transformer preferences : profileId=" + currentProfile.getId() + ", transformerPreferencesBeanName=" + transformerPreferencesBeanName);
                    }
                } catch (Exception e) {
                    logger.error("getCurrentTransformerPreferences() - unable to obtain transformer preferences bean name from the current profile. profileId="+currentProfile.getId()+", transformerPreferencesPropertyName="+transformerPreferencesPropertyName, e);
                    throw new PortalRuntimeException("unable to obtain transformer preferences bean name from the current profile. profileId="+currentProfile.getId()+", transformerPreferencesPropertyName="+transformerPreferencesPropertyName,e);
                }
                try {
                    currentTransformerPreferences=(ITransformerPreferences) ownBeanFactory.getBean(transformerPreferencesBeanName);
                    // set params
                    IRenderingAttributeProvider provider=currentTransformerPreferences.getRenderingAttributeProvider();
                    if(paramNames!=null) {
                        if(provider!=null) {
                            for(String param: paramNames) {
                                String value=(String) currentProfile.getProperty(param);
                                if(value!=null) {
                                    provider.setParameterValue(param,value);
                                    if (logger.isDebugEnabled()) {
                                        logger.debug("getCurrentTransformerPreferences() - set specified param value : paramName="+param+" value="+value+" profileId=" + currentProfile.getId() + ", transformerPreferencesBeanName=" + transformerPreferencesBeanName);
                                    }
                                } else {
                                    if (logger.isDebugEnabled()) {
                                        logger.debug("getCurrentTransformerPreferences() - no value specified for param in the profile : paramName="+param+"  profileId=" + currentProfile.getId() + ", transformerPreferencesBeanName=" + transformerPreferencesBeanName);
                                    }
                                }
                            }
                        } else {
                            if (logger.isInfoEnabled()) {
                                logger.info("getCurrentTransformerPreferences() - unable to obtain rendering attribute provider : currentProfileId=" + currentProfile.getId() + ", transformerPreferencesBeanName=" + transformerPreferencesBeanName);
                            }
                        }
                    }
                } catch (Exception e) {
                    logger.error("getCurrentTransformerPreferences() - unable to obtain transformer preferences bean from the BeanFactory : transformerPreferencesBeanName=" + transformerPreferencesBeanName, e);
                    throw new PortalRuntimeException("unable to obtain transformer preferences bean from the BeanFactory : transformerPreferencesBeanName=" + transformerPreferencesBeanName,e);
                }
                
            } else {
                logger.error("getCurrentTransformerPreferences() - unable to obtain current profile", null);
            }
        }
        return currentTransformerPreferences;
    }

    /**
     * Specify profile property name that will be used to look up transformer preferences instance bean
     * name.
     * @param transformerPreferencesPropertyName profile property name that should be used to lookup
     * transformer preferences bean.
     */
    @Required
    public void setTransformerPreferencesPropertyName(String transformerPreferencesPropertyName) {
        this.transformerPreferencesPropertyName = transformerPreferencesPropertyName;
    }

    // listener methods
    public void setBeanFactory(BeanFactory arg0) throws BeansException {
        this.ownBeanFactory=arg0;
    }

    public void receiveProfileControllerEvent(ProfileControllerEvent event) {
        // reload transformer preferences
        this.currentTransformerPreferences=null;
    }
    
    public ICacheKey generateKey() {
        // append profile id to the cache key
        ICacheKey tpCachekey=getCurrentTransformerPreferences().generateKey();
        if(tpCachekey==null) {
            return null;
        }
        IProfile currentProfile=this.profileController.getCurrentProfile();
        if(currentProfile!=null) {
            ProfileId profileId=currentProfile.getId();
            if(profileId!=null) {
                ICacheKey profileCacheKey=new CacheKeyImpl(profileId.toString());
                ICacheKey[] cka={tpCachekey,profileCacheKey};
                return new CacheKeyListImpl(cka);
            }
        }
        return tpCachekey;
    }

    // delegate methods
    public IRenderingAttributeProvider getRenderingAttributeProvider() {
        return getCurrentTransformerPreferences().getRenderingAttributeProvider();
    }


    public IStylesheetDescription getStylesheetDescription() {
        return getCurrentTransformerPreferences().getStylesheetDescription();
    }


    public boolean isCacheValid(Serializable validityObject) {
        return getCurrentTransformerPreferences().isCacheValid(validityObject);
    }

    @Required
    public void setProfileController(IProfileController profileController) {
        if(this.profileController!=null) {
            this.profileController.removeProfileControllerEventListener(this);
        }
        this.profileController = profileController;
        if(this.profileController!=null) {
            this.profileController.addProfileControllerEventListener(this);
        }
    }

    /**
     * Specify a list of rendering attribute param names that should be set upon switching to a new profile. 
     * The values will be determined from the profile properties directly, using param name as a property key
     * @param paramNames
     */
    public void setParamNames(List<String> paramNames) {
        this.paramNames = paramNames;
    }
}
