/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.sax.xslt;

import java.io.Serializable;
import java.net.URI;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.transform.Transformer;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.sax.TransformerHandler;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.CacheKeyListImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheable;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.rendering.IRenderingPipelineEventListener;
import org.jasig.portal.rendering.RenderingPipelineEvent;
import org.jasig.portal.rendering.sax.ContentHandlerConsumer;
import org.jasig.portal.rendering.sax.ISaxConsumer;
import org.jasig.portal.rendering.sax.ISaxFilter;
import org.jasig.portal.url.IPortalUrl;
import org.jasig.portal.url.IUrlConstructorProviderFactoryService;
import org.jasig.portal.url.IUrlConstructorProviderFactoryServiceAccessor;
import org.jasig.portal.url.IUrlDecorator;
import org.jasig.portal.url.IUrlDecoratorListAccessor;
import org.jasig.portal.url.PortalUrlImpl;
import org.jasig.portal.url.PortalUrlXalanElements;
import org.jasig.portal.url.UrlCacheUtils;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.DTDHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.ext.LexicalHandler;

/**
 * A configurable XSL transformer bean.
 * Transformer can be configured with a particular stylesheet,
 * a set of XSL params and Element attributes that should be injected
 * prior to the transformation.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public class TransformationFilter implements IRenderingPipelineEventListener, ISaxFilter, ICacheable, ContentHandler, DTDHandler, ErrorHandler, LexicalHandler {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(TransformationFilter.class);

    /**
     * XSLT param name which will be used to pass current servlet context path. This can be used
     * to construct mediaPath that would not depend on the rest of the servlet path.
     */
    public final static String SERVLET_CONTEXT_PATH_PARAM_NAME="contextPath";
    
    
    // injected properties
    IRenderingAttributeInjector renderingAttributeInjector;
    ITransformerHelper transformerHelper;
    // optional injected properties
    IUrlConstructorProviderFactoryServiceAccessor constructorProviderFactoryServiceAccessor;
    IUrlDecoratorListAccessor urlDecoratorListAccessor;
    ICacheKey urlCacheKeys;
    String baseActionUrlParamName;

    // internal transformer handler
    TransformerHandler transformerHandler=null;
    ITransformerPreferences transformerPreferences=null;
    ContentHandler internalContentHandler=null;
    
    ErrorHandler errorInputHandler;
    ErrorHandler errorHandler=null;
    LexicalHandler lexicalHandler;
    ContentHandler contentHandler;
    
    ISaxConsumer consumer=null;
    
    /* (non-Javadoc)
     * @see org.jasig.portal.caching.ICacheable#generateKey()
     */
    public ICacheKey generateKey() {
        if(renderingAttributeInjector!=null) {
            ICacheKey[] cacheKeys=new ICacheKey[2];
            cacheKeys[0]=urlCacheKeys;
            cacheKeys[1]=renderingAttributeInjector.generateKey();
            return new CacheKeyListImpl(cacheKeys);
        } else {
            return urlCacheKeys;
        }
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.caching.ICacheable#isCacheValid(java.io.Serializable)
     */
    public boolean isCacheValid(Serializable validityObject) {
        if(renderingAttributeInjector==null) {
            return true;
        } else {
            if(validityObject instanceof Serializable[]) {
                Serializable[] obj=(Serializable[]) validityObject;
                if(obj.length==2) {
                    return this.renderingAttributeInjector.isCacheValid(obj[1]);
                } else {
                    logger.warn("isCacheValid(Serializable) - passed set of validity objects has the wrong size (expecting size 2, received size "+obj.length+")", null);
                    return false;
                }
            } else {
                logger.warn("isCacheValid(Serializable) - validityObject is not a an array, as expected when an injector is configured", null);
                return false;
            }
        }
    }
	/* (non-Javadoc)
	 * @see org.jasig.portal.rendering.sax.IRenderingPipelineEventListener#reveiveSAXPipelineEvent(org.jasig.portal.rendering.sax.RenderingPipelineEvent)
	 */
	public void receiveSAXPipelineEvent(RenderingPipelineEvent ev) throws PortalException {
        if (ev == null) {
            throw new IllegalArgumentException("Cannot receive a null event.");
        }
        
		// start rendering event
		if(ev.getEventType()==RenderingPipelineEvent.startRenderingCycleEventType) {
			// reset the transformer
            
            if (transformerPreferences == null) {
                throw new IllegalStateException("transformerPreferences must not be null.");
            }
            
            IStylesheetDescription stylesheetDescription = transformerPreferences.getStylesheetDescription();
            
            if (stylesheetDescription == null) {
                throw new IllegalStateException("Stylesheet description must not be null.");
            }
            
            URI stylesheetLocationUri = stylesheetDescription.getStylesheetLocationUri();
            
            if (stylesheetLocationUri == null) {
                throw new IllegalStateException("Stylesheet location URI must not be null.");
            }
            
            if (transformerHelper == null) {
                throw new IllegalStateException("transformerHelper was null.");
            }
            
            transformerHandler=transformerHelper.getTransformerHandler(stylesheetLocationUri);
            
            setUpTransformerHandlerHandlers();
            
            // set the mime type if the stylesheet determines it
            String stylesheetMimeType=transformerPreferences.getStylesheetDescription().getMimeType();
            if(stylesheetMimeType!=null) {
                HttpServletResponse res=ev.getHttpServletResponse();
                res.setContentType(stylesheetMimeType);
            }
            setupTransformer(ev.getHttpServletRequest());
        }
	}
    
    protected void setupTransformer(HttpServletRequest request) {
        // set up various transformer params, url decorators, url cache keys
        // Note: rendering attribute params are configured at startDocument() event
        Transformer transformer=transformerHandler.getTransformer();
        
        // set url constructors and decorators for URL construction Xalan extension elements
        IUrlConstructorProviderFactoryService cpfs=null;
        if(constructorProviderFactoryServiceAccessor!=null) {
            cpfs=constructorProviderFactoryServiceAccessor.getUrlConstructorProviderFactoryService(request);
            transformer.setParameter(PortalUrlXalanElements.URL_CONSTRUCTOR_CONFIGURATION_PARAMETER_NAME,cpfs);
        }
        List<IUrlDecorator> urlDecorators=null;
        if(urlDecoratorListAccessor!=null) {
            urlDecorators=urlDecoratorListAccessor.getUrlDecorators(request);
            transformer.setParameter(PortalUrlXalanElements.URL_DECORATOR_CONFIGURATION_PARAMETER_NAME,urlDecorators);
        }
        
        // set base action url param
        if(baseActionUrlParamName!=null) {
            // TODO: this functionality could be generalized into a list of
            // injected objects that can operate on the transformer, e.g.:
            // prepareTransformer(transformer,req,res);
            IPortalUrl portalUrl=new PortalUrlImpl(cpfs,urlDecorators);
            String str_portalUrl=portalUrl.toString();
            transformer.setParameter(baseActionUrlParamName,str_portalUrl);
        }
        
        urlCacheKeys=UrlCacheUtils.getCacheKey(cpfs,urlDecorators);
        
        transformer.setParameter(SERVLET_CONTEXT_PATH_PARAM_NAME,request.getContextPath());
    }
    // configure handlers
    private void setUpTransformerHandlerHandlers() {
        if(renderingAttributeInjector!=null) {
            renderingAttributeInjector.setSaxConsumer(new ContentHandlerConsumer(transformerHandler));
            internalContentHandler=renderingAttributeInjector;
        } else {
            internalContentHandler=transformerHandler;
        }
        if(transformerHandler!=null) {
            if(contentHandler!=null) {
                SAXResult r=new SAXResult(contentHandler);
                if(lexicalHandler!=null) {
                    r.setLexicalHandler(lexicalHandler);
                }
                transformerHandler.setResult(r);
            }
            if(transformerHandler instanceof ErrorHandler) {
                // redirect errors to transformer if it supports it
                errorInputHandler=(ErrorHandler)transformerHandler;
            } else { // otherwise redirect errors to the next filter stage
                if(errorHandler!=null) {
                    errorInputHandler=errorHandler;
                }
            }
        }
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.ISAXProducer#setAllHandlers(org.xml.sax.ContentHandler)
     */
    public void setAllHandlers(ContentHandler ch) {
        contentHandler=ch;
        
        if(ch instanceof LexicalHandler) {
            lexicalHandler=(LexicalHandler)ch;
        } else {
            lexicalHandler=null;
        }
        
        if(ch instanceof ErrorHandler) {
            errorHandler=(ErrorHandler)ch;
        } else {
            errorHandler=null;
        }
        
        setUpTransformerHandlerHandlers();
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.ISaxProducer#setSaxConsumer(org.jasig.portal.rendering.sax.ISaxConsumer)
     */
    public void setSaxConsumer(ISaxConsumer consumer) {
        this.consumer=consumer;
        this.setAllHandlers(consumer);
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.ISaxPipelineComponent#canRenderSax()
     */
    public boolean canRenderSax() {
        return (consumer!=null && consumer.canRenderSax());
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.ISaxProducer#isContenteHandlerSet()
     */
    public boolean isContenteHandlerSet() {
        return this.contentHandler!=null;
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.ISAXProducer#setDTDHandler(org.xml.sax.DTDHandler)
     */
    public void setDTDHandler(DTDHandler handler) {}
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.ISAXProducer#setContentHandler(org.xml.sax.ContentHandler)
     */
    public void setContentHandler(ContentHandler handler) {
        this.contentHandler=handler;
        setUpTransformerHandlerHandlers();
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.ISAXProducer#setErrorHandler(org.xml.sax.ErrorHandler)
     */
    public void setErrorHandler(ErrorHandler handler) {
        this.errorHandler=handler;
        setUpTransformerHandlerHandlers();
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.ISAXProducer#setLexicalHandler(org.xml.sax.ext.LexicalHandler)
     */
    public void setLexicalHandler(LexicalHandler handler) {
        this.lexicalHandler=handler;
        setUpTransformerHandlerHandlers();
    }
    
    public void startElement(String namespaceUri, String localName, String qName, Attributes atts) throws SAXException {
    	internalContentHandler.startElement(namespaceUri, localName, qName, atts);
    }

    
    public void characters(char[] arg0, int arg1, int arg2) throws SAXException {
        internalContentHandler.characters(arg0, arg1, arg2);
    }
    
    public void comment(char[] arg0, int arg1, int arg2) throws SAXException {
        transformerHandler.comment(arg0, arg1, arg2);
    }
    
    public void endCDATA() throws SAXException {
        transformerHandler.endCDATA();
    }
    
    public void endDocument() throws SAXException {
        internalContentHandler.endDocument();
    }
    
    public void endDTD() throws SAXException {
        transformerHandler.endDTD();
    }
    
    public void endElement(String arg0, String arg1, String arg2) throws SAXException {
        internalContentHandler.endElement(arg0, arg1, arg2);
    }
    
    public void endEntity(String arg0) throws SAXException {
        transformerHandler.endEntity(arg0);
    }
    
    public void endPrefixMapping(String arg0) throws SAXException {
        internalContentHandler.endPrefixMapping(arg0);
    }
    
    public void ignorableWhitespace(char[] arg0, int arg1, int arg2) throws SAXException {
        internalContentHandler.ignorableWhitespace(arg0, arg1, arg2);
    }
    
    public void notationDecl(String arg0, String arg1, String arg2) throws SAXException {
        transformerHandler.notationDecl(arg0, arg1, arg2);
    }
    
    public void processingInstruction(String arg0, String arg1) throws SAXException {
        internalContentHandler.processingInstruction(arg0, arg1);
    }

    public void setDocumentLocator(Locator arg0) {
        internalContentHandler.setDocumentLocator(arg0);
    }

    public void skippedEntity(String arg0) throws SAXException {
        internalContentHandler.skippedEntity(arg0);
    }

    public void startCDATA() throws SAXException {
        transformerHandler.startCDATA();
    }

    public void startDocument() throws SAXException {
        // set up XSLT params that may come from rendering attribute injector
        Transformer transformer=transformerHandler.getTransformer();
        if(renderingAttributeInjector!=null) {
            renderingAttributeInjector.prepareTransformer(transformer);
        }
        internalContentHandler.startDocument();
    }

    public void startDTD(String arg0, String arg1, String arg2) throws SAXException {
        transformerHandler.startDTD(arg0, arg1, arg2);
    }

    public void startEntity(String arg0) throws SAXException {
        transformerHandler.startEntity(arg0);
    }

    public void startPrefixMapping(String arg0, String arg1) throws SAXException {
        internalContentHandler.startPrefixMapping(arg0, arg1);
    }

    public void unparsedEntityDecl(String arg0, String arg1, String arg2, String arg3) throws SAXException {
        transformerHandler.unparsedEntityDecl(arg0, arg1, arg2, arg3);
    }
    /* (non-Javadoc)
     * @see org.xml.sax.ErrorHandler#error(org.xml.sax.SAXParseException)
     */
    public void error(SAXParseException arg0) throws SAXException {
        if(errorInputHandler!=null) {
            errorInputHandler.error(arg0);
        }
    }
    /* (non-Javadoc)
     * @see org.xml.sax.ErrorHandler#fatalError(org.xml.sax.SAXParseException)
     */
    public void fatalError(SAXParseException arg0) throws SAXException {
        if(errorInputHandler!=null) {
            errorInputHandler.fatalError(arg0);
        }
    }
    /* (non-Javadoc)
     * @see org.xml.sax.ErrorHandler#warning(org.xml.sax.SAXParseException)
     */
    public void warning(SAXParseException arg0) throws SAXException {
        if(errorInputHandler!=null) {
            errorInputHandler.warning(arg0);
        }
    }
    
    /**
     * Provide transformer helper to be used
     * @param transformerHelper The transformerHelper to set.
     */
    public void setTransformerHelper(ITransformerHelper transformerHelper) {
        this.transformerHelper = transformerHelper;
    }
    /**
     * @return Returns the transformerHandler.
     */
    public TransformerHandler getTransformerHandler() {
        return transformerHandler;
    }
    public void setRenderingAttributeInjector(IRenderingAttributeInjector renderingAttributeInjector) {
        this.renderingAttributeInjector = renderingAttributeInjector;
    }
    public void setTransformerPreferences(ITransformerPreferences transformerPreferences) {
        this.transformerPreferences = transformerPreferences;
    }
    /**
     * Set constructor provider factory service accessor (required for URL construction through Xalan extension elements within the stylesheets) 
     * @param constructorProviderFactoryServiceAccessor
     */
    public void setConstructorProviderFactoryServiceAccessor(IUrlConstructorProviderFactoryServiceAccessor constructorProviderFactoryServiceAccessor) {
        this.constructorProviderFactoryServiceAccessor = constructorProviderFactoryServiceAccessor;
    }
    /**
     * Set url decorator list accessor (required for URL construction through Xalan extension elements within the stylesheets)
     * @param urlDecoratorListAccessor
     */
    public void setUrlDecoratorListAccessor(IUrlDecoratorListAccessor urlDecoratorIteratorAccessor) {
        this.urlDecoratorListAccessor = urlDecoratorIteratorAccessor;
    }
    /**
     * Set param name for passing base action url. Turned off by default.
     * @param baseActionUrlParamName param name for passing base action url, or <code>null</code> if no base action should be passed.
     */
    public void setBaseActionUrlParamName(String baseActionUrlParamName) {
        this.baseActionUrlParamName = baseActionUrlParamName;
    }
}
