/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.sax.xslt.legacy;

import javax.servlet.http.HttpServletResponse;

import org.jasig.portal.rendering.attributes.IRenderingAttributeProvider;
import org.jasig.portal.url.IRequestParameterProcessor;
import org.jasig.portal.url.IWritableHttpServletRequest;
import org.xml.sax.helpers.AttributesImpl;

/**
 * TransformationFilterLegacyRequestParameterProcessor is responsible for processing request parameter syntax
 * for the {@link org.jasig.portal.rendering.sax.xslt.TransformationFilter}. It replicates the parameter
 * syntax used to relate stylesheet params and attributes in uPortal 2.x implementations. The filter should be
 * configured with rendering attribute providers corresponding to structure and theme transformations.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 *
 */
public class TransformationFilterLegacyRequestParameterProcessor implements IRequestParameterProcessor {
    public final static String STRUCTURE_STYLESHEET_PROCESSOR_NAME="s";
    public final static String THEME_STYLESHEET_PROCESSOR_NAME="t";

    protected IRenderingAttributeProvider structureRenderingAttributeProvider;
    protected IRenderingAttributeProvider themeRenderingAttributeProvider;
    

    public final static String UP_PREFIX="uP_";
    public final static String PARAM_POSTFIX="param";
    public final static String FOLDER_ATTRIBUTE_POSTFIX="fattr";
    public final static String CHANNEL_ATTRIBUTE_POSTFIX="cattr";
    public final static String FOLDERID_POSTFIX="_folderId";
    public final static String CHANNELID_POSTFIX="_channelId";
    public final static String VALUE_POSTFIX="_value";
    public final static String NAME_ID_SEPARATOR="_";
    public final static String FOCUS_COMMAND="uP_root";
    public final static String FOCUS_PARAM_NAME="userLayoutRoot";
    
    public final static String FOLDER_QNAME="folder";
    public final static String CHANNEL_QNAME="channel";
    public final static String ID_ATTRIBUTE_NAME="ID";
    
    public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res) {
        processParameters(structureRenderingAttributeProvider,STRUCTURE_STYLESHEET_PROCESSOR_NAME,req,res);
        processParameters(themeRenderingAttributeProvider,THEME_STYLESHEET_PROCESSOR_NAME,req,res);
        return true;
    }
    
    public void processParameters(IRenderingAttributeProvider provider, String processorPrefix, IWritableHttpServletRequest req, HttpServletResponse res) {
        // params
        String[] paramNames=req.getParameterValues(UP_PREFIX+processorPrefix+PARAM_POSTFIX);
        if(paramNames!=null) {
            for (int i = 0; i < paramNames.length; i++) {
                String paramValue=req.getParameter(paramNames[i]);
                if(paramValue!=null) {
                    this.setStylesheetParam(paramNames[i],paramValue,provider);
                }
            }
        }
        
        // channel attributes
        String[] cattrNames=req.getParameterValues(UP_PREFIX+processorPrefix+CHANNEL_ATTRIBUTE_POSTFIX);
        if(cattrNames!=null) {
            for (int i = 0; i < cattrNames.length; i++) {
                String[] channelIds=req.getParameterValues(cattrNames[i]+CHANNELID_POSTFIX);
                if(channelIds!=null) {
                    for(String channelId: channelIds) {
                        String cattrValue=req.getParameter(cattrNames[i]+NAME_ID_SEPARATOR+channelId+VALUE_POSTFIX);
                        if(cattrValue!=null) {
                            this.setChannelAttribute(channelId,cattrNames[i],cattrValue,provider);
                        }
                    }
                }
            }
        }
        
        // folder attributes
        if(STRUCTURE_STYLESHEET_PROCESSOR_NAME.equals(processorPrefix)) { // folder attributes only exist for structure transformation
            String[] fattrNames=req.getParameterValues(UP_PREFIX+STRUCTURE_STYLESHEET_PROCESSOR_NAME+FOLDER_ATTRIBUTE_POSTFIX);
            if(fattrNames!=null) {
                for (int i = 0; i < fattrNames.length; i++) {
                    String[] folderIds=req.getParameterValues(fattrNames[i]+FOLDERID_POSTFIX);
                    if(folderIds!=null) {
                        for(String folderId: folderIds) {
                            String fattrValue=req.getParameter(fattrNames[i]+NAME_ID_SEPARATOR+folderId+VALUE_POSTFIX);
                            if(fattrValue!=null) {
                                this.setFolderAttribute(folderId,fattrNames[i],fattrValue,provider);
                            }  
                        }
                    }
                }
            } 
        }
        
        String focusId=req.getParameter(FOCUS_COMMAND);
        if(focusId!=null) {
            // TODO: add support for the "me" value (extracting portlet window id from the base of the file spec)
            this.setStylesheetParam(FOCUS_PARAM_NAME,focusId,provider);
        }
    }

    public void setChannelAttribute(String channelId, String attributeName, String attributeValue, IRenderingAttributeProvider provider) {
        AttributesImpl ai=new AttributesImpl();
        ai.addAttribute(null,ID_ATTRIBUTE_NAME,ID_ATTRIBUTE_NAME,"ID",channelId);
        provider.setAttributeValue(attributeName,attributeValue,null,CHANNEL_QNAME,CHANNEL_QNAME,ai);
    }
    
    public void setFolderAttribute(String folderId, String attributeName, String attributeValue, IRenderingAttributeProvider provider) {
        AttributesImpl ai=new AttributesImpl();
        ai.addAttribute(null,ID_ATTRIBUTE_NAME,ID_ATTRIBUTE_NAME,"ID",folderId);
        provider.setAttributeValue(attributeName,attributeValue,null,FOLDER_QNAME,FOLDER_QNAME,ai);
    }
    
    public void setStylesheetParam(String paramName, String paramValue, IRenderingAttributeProvider provider) {
        provider.setParameterValue(paramName,paramValue);
    }


    public boolean canEvaluateVariables() {
        return false;
    }
    
    public void setStructureRenderingAttributeProvider(IRenderingAttributeProvider structureRenderingAttributeProvider) {
        this.structureRenderingAttributeProvider = structureRenderingAttributeProvider;
    }

    public void setThemeRenderingAttributeProvider(IRenderingAttributeProvider themeRenderingAttributeProvider) {
        this.themeRenderingAttributeProvider = themeRenderingAttributeProvider;
    }
}
