/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.rendering.sax.xslt.legacy.persistance;

import java.io.IOException;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.net.URL;

import org.apache.commons.digester.AbstractObjectCreationFactory;
import org.apache.commons.digester.Digester;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.core.exception.PortalRuntimeException;
import org.jasig.portal.rendering.attributes.AttributeDescriptionImpl;
import org.jasig.portal.rendering.attributes.AttributePersistenceScope;
import org.jasig.portal.rendering.sax.xslt.legacy.ILegacyStylesheetDescription;
import org.jasig.portal.rendering.sax.xslt.legacy.LegacyStylesheetDescription;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * LegacyStylesheetDescriptionDao description
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public class LegacyStylesheetDescriptionDao implements ILegacyStylesheetDescriptionDao {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(LegacyStylesheetDescriptionDao.class);

    public ILegacyStylesheetDescription getLegacyStylesheetDescription(String stylesheetDescriptionLocation, String stylesheetLocation) {
        URL stylesheetDescriptionUrl=this.getClass().getResource(stylesheetDescriptionLocation);
        if(stylesheetDescriptionUrl!=null) {
            try {
                return readSdfFile(stylesheetDescriptionUrl.openStream(),stylesheetLocation);
            } catch (IOException e) {
                logger.error("getLegacyStylesheetDescription() - unable to open stylesheet description file : URL stylesheetDescriptionUrl = " + stylesheetDescriptionUrl, e);
                throw new PortalRuntimeException("unable to open stylesheet description file: URL stylesheetDescriptionUrl = " + stylesheetDescriptionUrl,e);
            } catch (SAXException e) {
                logger.error("getLegacyStylesheetDescription() - unable to parse stylesheet description file : URL stylesheetDescriptionUrl = " + stylesheetDescriptionUrl, e);
                throw new PortalRuntimeException("unable to parse stylesheet description file: URL stylesheetDescriptionUrl = " + stylesheetDescriptionUrl,e);
            } catch (URISyntaxException e) {
                logger.error("getLegacyStylesheetDescription() - unable to parse stylesheet location : URL stylesheetUrl = " + stylesheetLocation, e);
                throw new PortalRuntimeException("unable to parse stylesheet location: URL stylesheetDescriptionUrl = " + stylesheetLocation,e);
            }
        }
        return null;
    }
    
    /**
     * Reads uP2 stylesheet description file, creating legacy stylesheet description
     * @return legacy stylesheet description instance
     * @throws IOException
     * @throws SAXException
     * @throws URISyntaxException 
     */
    protected LegacyStylesheetDescription readSdfFile(InputStream inputStream, String stylesheetLocation) throws IOException, SAXException, URISyntaxException {
        Digester digester = new Digester();
        digester.setValidating(false);       
        
        // push description
        digester.addFactoryCreate("stylesheetdescription",new AbstractObjectCreationFactory() {
            public Object createObject(Attributes attributes) throws Exception {
                LegacyStylesheetDescription d=new LegacyStylesheetDescription();
                return d;
            }
        });
        digester.addCallMethod("stylesheetdescription/name","setName",0);
        digester.addCallMethod("stylesheetdescription/parentStructureStylesheet","setParentStructureStylesheet",0);
        digester.addCallMethod("stylesheetdescription/mimeType","setMimeType",0);
        digester.addCallMethod("stylesheetdescription/serializer","setSerializer",0);
        digester.addCallMethod("stylesheetdescription/deviceType","setDeviceType",0);
        digester.addCallMethod("stylesheetdescription/samplePictureURI","setSamplePictureUri",0);
        digester.addCallMethod("stylesheetdescription/description","setDescription",0);
        digester.addCallMethod("stylesheetdescription/sampleIconURI","setSampleIconUri",0);
        digester.addCallMethod("stylesheetdescription/userPreferencesModuleClass","setUserPreferencesModuleClass",0);


        // parameters processing
        digester.addObjectCreate("stylesheetdescription/parameters", "java.util.HashSet");
        digester.addFactoryCreate("stylesheetdescription/parameters/parameter",new AbstractObjectCreationFactory() {
            public Object createObject(Attributes attributes) throws Exception {
                AttributeDescriptionImpl d=new AttributeDescriptionImpl();
                d.setPersistenceScope(AttributePersistenceScope.permanentScope);
                return d;
            }
        });
        digester.addSetProperties("stylesheetdescription/parameters/parameter");
        digester.addCallMethod("stylesheetdescription/parameters/parameter/description","setDescription",0);
        digester.addCallMethod("stylesheetdescription/parameters/parameter/defaultvalue","setDefaultValue",0);
        digester.addSetNext("stylesheetdescription/parameters/parameter","add");
        digester.addSetNext("stylesheetdescription/parameters","setParamDescriptions");
        
        // channel attributes processing
        digester.addObjectCreate("stylesheetdescription/channelattributes", "java.util.HashSet");
        digester.addFactoryCreate("stylesheetdescription/channelattributes/attribute",new AbstractObjectCreationFactory() {
            public Object createObject(Attributes attributes) throws Exception {
                AttributeDescriptionImpl d=new AttributeDescriptionImpl();
                d.setPersistenceScope(AttributePersistenceScope.permanentScope);
                return d;
            }
        });
        digester.addSetProperties("stylesheetdescription/channelattributes/attribute");
        digester.addCallMethod("stylesheetdescription/channelattributes/attribute/description","setDescription",0);
        digester.addCallMethod("stylesheetdescription/channelattributes/attribute/defaultvalue","setDefaultValue",0);
        digester.addSetNext("stylesheetdescription/channelattributes/attribute","add");
        digester.addSetNext("stylesheetdescription/channelattributes","setChannelAttributeDescriptions");

        // folder attribute processing
        digester.addObjectCreate("stylesheetdescription/folderattributes", "java.util.HashSet");
        digester.addFactoryCreate("stylesheetdescription/folderattributes/attribute",new AbstractObjectCreationFactory() {
            public Object createObject(Attributes attributes) throws Exception {
                AttributeDescriptionImpl d=new AttributeDescriptionImpl();
                //d.setElementQNames(folderQNames);
                d.setPersistenceScope(AttributePersistenceScope.permanentScope);
                return d;
            }
        });
        digester.addSetProperties("stylesheetdescription/folderattributes/attribute");
        digester.addCallMethod("stylesheetdescription/folderattributes/attribute/description","setDescription",0);
        digester.addCallMethod("stylesheetdescription/folderattributes/attribute/defaultvalue","setDefaultValue",0);
        digester.addSetNext("stylesheetdescription/folderattributes/attribute","add");
        digester.addSetNext("stylesheetdescription/folderattributes","setFolderAttributeDescriptions");
        
        LegacyStylesheetDescription description=(LegacyStylesheetDescription) digester.parse(inputStream);
        description.setStylesheetLocation(stylesheetLocation);
        return description;
    }
}
