/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.sax.xslt.legacy.spring;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import org.jasig.portal.context.common.persistence.IPersistenceController;
import org.jasig.portal.rendering.attributes.ElementResolverKeyFactoryImpl;
import org.jasig.portal.rendering.attributes.IElementResolver;
import org.jasig.portal.rendering.attributes.IElementResolverKeyFactory;
import org.jasig.portal.rendering.attributes.IRenderingAttributeProvider;
import org.jasig.portal.rendering.attributes.IRenderingAttributeValues;
import org.jasig.portal.rendering.attributes.IRenderingAttributesDescription;
import org.jasig.portal.rendering.attributes.LayeredRenderingAttributeValues;
import org.jasig.portal.rendering.attributes.ListRenderingAttributeProvider;
import org.jasig.portal.rendering.attributes.PersistentRenderingAttributeValues;
import org.jasig.portal.rendering.attributes.RenderingAttributeProviderImpl;
import org.jasig.portal.rendering.attributes.RenderingAttributesDescriptionImpl;
import org.jasig.portal.rendering.attributes.SessionAttributeLevelResolver;
import org.jasig.portal.rendering.attributes.SingleElementResolver;
import org.jasig.portal.rendering.attributes.dao.IRenderingAttributesDao;
import org.jasig.portal.rendering.sax.xslt.legacy.ILegacyStylesheetDescription;
import org.jasig.portal.rendering.sax.xslt.legacy.persistance.LegacyRenderingAttributeDescriptionAdapter;
import org.springframework.beans.factory.BeanInitializationException;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Required;

/**
 * A utility factory bean to configure entire rendering attribute provider
 * for uP2 legacy stylesheets. 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class LegacyRenderingAttributeProviderFactoryBean implements FactoryBean, InitializingBean {
    protected static final String CHANNEL_ELEMENT_NAME="channel";
    protected static final String FOLDER_ELEMENT_NAME="folder";
    protected static final String ID_ATTRIBUTE="ID";
    
    // injected properties
    ILegacyStylesheetDescription legacyStylesheetDescription;
    IRenderingAttributesDao renderingAttributesDao;
    IPersistenceController persistenceController;
    IRenderingAttributeValues renderingAttributeValues=null;
    
    String providerName;
    
    // optional injected properties
    IElementResolverKeyFactory elementResolverKeyFactory=new ElementResolverKeyFactoryImpl();
    IElementResolver folderElementResolver=new SingleElementResolver(FOLDER_ELEMENT_NAME,ID_ATTRIBUTE,elementResolverKeyFactory);
    IElementResolver channelElementResolver=new SingleElementResolver(CHANNEL_ELEMENT_NAME,ID_ATTRIBUTE,elementResolverKeyFactory);
    List<IRenderingAttributeProvider> childProviders=null;
    List<IRenderingAttributeProvider> additionalParentProviders=null;
    
    
    public Object getObject() throws Exception {
        
        // param and folder attribute provider
        RenderingAttributeProviderImpl folderProvider=new RenderingAttributeProviderImpl();
        folderProvider.setProviderName(providerName+".param.and.folder");
        folderProvider.setElementResolver(folderElementResolver);
        HashSet<IRenderingAttributesDescription> s=new HashSet<IRenderingAttributesDescription>();
        s.add(legacyStylesheetDescription.getFolderAttributesDescription());
        s.add(legacyStylesheetDescription.getParamsDescription());;
        IRenderingAttributesDescription desc=new RenderingAttributesDescriptionImpl(s);
        folderProvider.setRenderingAttributesDescription(desc);
        
        IRenderingAttributeValues values=this.renderingAttributeValues;
        if(values==null) {
            // construct rendering attribute values
            LayeredRenderingAttributeValues layeredRenderingAttributeValues=new LayeredRenderingAttributeValues();
            // session-level resolver
            SessionAttributeLevelResolver attributeLevelResolver=new SessionAttributeLevelResolver();
            LegacyRenderingAttributeDescriptionAdapter combinedDescriptions=new LegacyRenderingAttributeDescriptionAdapter();
            combinedDescriptions.setLegacyStylesheetDescription(legacyStylesheetDescription);
            attributeLevelResolver.setRenderingAttributesDescription(combinedDescriptions);
            layeredRenderingAttributeValues.setAttributeLevelResolver(attributeLevelResolver);
            
            // persistent values - parent values
            PersistentRenderingAttributeValues persistentValues=new PersistentRenderingAttributeValues();
            persistentValues.setRenderingAttributesDao(this.renderingAttributesDao);
            persistentValues.setPersistenceController(this.persistenceController);
            layeredRenderingAttributeValues.setParentAttributeValues(persistentValues);
            values=layeredRenderingAttributeValues;
        }
        
        
        folderProvider.setRenderingAttributeValues(values);
        
        // channel attribute provider
        RenderingAttributeProviderImpl channelProvider=new RenderingAttributeProviderImpl();
        channelProvider.setProviderName(providerName+".channel");
        channelProvider.setElementResolver(channelElementResolver);
        channelProvider.setRenderingAttributesDescription(legacyStylesheetDescription.getChannelAttributesDescription());
        channelProvider.setRenderingAttributeValues(values);

        List<IRenderingAttributeProvider> parentProviders=new ArrayList<IRenderingAttributeProvider>();
        if(this.additionalParentProviders!=null) {
            parentProviders.addAll(this.additionalParentProviders);
        }
        parentProviders.add(channelProvider);
        parentProviders.add(folderProvider);
        
        if(this.childProviders!=null) {
            parentProviders.addAll(this.childProviders);
        }
        
        ListRenderingAttributeProvider listProvider=new ListRenderingAttributeProvider();
        listProvider.setParentProviders(parentProviders);
        listProvider.setProviderName(providerName);
        return listProvider;
    }

    public Class getObjectType() {
        return IRenderingAttributeProvider.class;
    }

    public boolean isSingleton() {
        return true;
    }

    @Required
    public void setLegacyStylesheetDescription(ILegacyStylesheetDescription legacyStylesheetDescription) {
        this.legacyStylesheetDescription = legacyStylesheetDescription;
    }

    /**
     * Specify element resolver key factory (optional config parameter)
     * @param elementResolverKeyFactory
     */
    public void setElementResolverKeyFactory(IElementResolverKeyFactory elementResolverKeyFactory) {
        this.elementResolverKeyFactory = elementResolverKeyFactory;
    }

    /**
     * Specify folder element resolver (optional config parameter)
     * @param folderElementResolver
     */
    public void setFolderElementResolver(IElementResolver folderElementResolver) {
        this.folderElementResolver = folderElementResolver;
    }

    /**
     * Specify channel element resolver (optional config parameter)
     * @param channelElementResolver
     */
    public void setChannelElementResolver(IElementResolver channelElementResolver) {
        this.channelElementResolver = channelElementResolver;
    }

    /**
     * Specify provider name
     * @param providerName
     */
    public void setProviderName(String providerName) {
        this.providerName = providerName;
    }

    /**
     * Specify child providers that should be included in the hierarchical provider configuration.
     * (optional config parameter)
     * @param childProviders
     */
    public void setChildProviders(List<IRenderingAttributeProvider> additionalChildProviders) {
        this.childProviders = additionalChildProviders;
    }

    /**
     * Specify additional parent providers that should be included in the configuration.
     * If specified, the providers will be included on top of the list (giving them higher priority).
     * (optional config parameter)
     * @param additionalParentProviders
     */
    public void setAdditionalParentProviders(List<IRenderingAttributeProvider> additionalParentProviders) {
        this.additionalParentProviders = additionalParentProviders;
    }

    /**
     * Persistence controller - required if rendering attribute values are not supplied
     * @param persistenceController
     */
    public void setPersistenceController(IPersistenceController persistenceController) {
        this.persistenceController = persistenceController;
    }

    
    /**
     * Specify DAO to be used in storing persistent rendering attributes.
     * Note: required if rendering attribute values are not supplied 
     * @param renderingAttributesDao
     */
    public void setRenderingAttributesDao(IRenderingAttributesDao renderingAttributesDao) {
        this.renderingAttributesDao = renderingAttributesDao;
    }

    /**
     * Values object to store rendering attributes in. If not injected, the factory will
     * construct a layered rendering attribute values object that persists SDF-defiend attributes, 
     * while maintaining the rest in the session. In that case, renderingAttributesDao and persistenceController
     * must be injected.
     * @param renderingAttributeValues
     */
    public void setRenderingAttributeValues(IRenderingAttributeValues renderingAttributeValues) {
        this.renderingAttributeValues = renderingAttributeValues;
    }

    public void afterPropertiesSet() throws Exception {
        if(renderingAttributeValues==null) {
            if(persistenceController==null) {
                throw new BeanInitializationException("persistenceController must be provided if renderingAttributeValues are not");
            }
            if(renderingAttributesDao==null) {
                throw new BeanInitializationException("renderingAttributesDao must be provided if renderingAttributeValues are not");
            }
        }
        
    }

}
