/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.rendering.string;

import org.apache.oro.text.regex.MalformedPatternException;
import org.apache.oro.text.regex.MatchResult;
import org.apache.oro.text.regex.Pattern;
import org.apache.oro.text.regex.PatternCompiler;
import org.apache.oro.text.regex.PatternMatcherInput;
import org.apache.oro.text.regex.Perl5Compiler;
import org.apache.oro.text.regex.Perl5Matcher;
import org.jasig.portal.caching.ISimpleCacheProviderFacade;

/**
 * RegexStringFilter is an abstract base class for string filters that filter regular-expression defined content patterns.
 * Subclasses must implement <code>processMatch()</code> method.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public abstract class RegexStringFilter implements IStringFilter {
    protected static final LocalMatcher localMatcher = new LocalMatcher();
    
    Pattern pattern;
    ISimpleCacheProviderFacade patternCache;
    IStringPipelineDownstreamComponent downstreamComponent;
    
    StringBuffer contentBuffer;
    String content;

    public void filterRendering(String input) {
        contentBuffer=null;
        content=input;
        // match regular expression
        Perl5Matcher matcher=(Perl5Matcher)localMatcher.get();
        PatternMatcherInput matcherInput=new PatternMatcherInput(content);
        while(matcher.contains(matcherInput,pattern)) {
            // construct a matchresult with corrected offsetts, if the buffer is present
            if(contentBuffer!=null && contentBuffer.length()!=content.length()) {
                processMatch(new CorrectedOffsetMatch(matcher.getMatch(),content.length()-contentBuffer.length()));
            } else {
                processMatch(matcher.getMatch());
            }
        }
        
        if(contentBuffer!=null) {
            downstreamComponent.filterRendering(contentBuffer.toString());
        } else {
            downstreamComponent.filterRendering(content);
        }
    }
    
    /**
     * Provide regular expression to match
     * 
     * @param regex regular expression
     */
    public void setRegex(String regex) throws MalformedPatternException {
        // compile regular expression
        PatternCompiler compiler=new Perl5Compiler();            
        // check the cache for compiled patterns
        if(patternCache!=null) {
            Pattern compiledRegex=(Pattern) patternCache.getFromCache(regex);
            if(compiledRegex==null) { // compile and store in cache
                compiledRegex=compiler.compile(regex);
                patternCache.putInCache(regex,compiledRegex);
            }
            this.pattern=compiledRegex;
        } else {
            this.pattern=compiler.compile(regex);
        }
    }
    
    /**
     * A method called upon encountering a match. Content can be modified by operating on the
     * buffer (@see getContentBuffer()).
     * 
     * Note: implementation uses the end of the string to calculate offsets. Any modifications to the input
     * buffer must not affect the region of the string beyond the end of the current match.
     * 
     * @param match match result
     */
    public abstract void processMatch(MatchResult match);
    
    /**
     * Acquire a buffer with current version of the rendering content.
     * The content can be modified only according to the rules described in the <code>processMatch()</code> javadoc.
     * 
     * @return a buffer containing current version of the content
     */
    public StringBuffer getContentBuffer() {
        if(this.contentBuffer==null) {
            this.contentBuffer=new StringBuffer(content);
        }
        return this.contentBuffer;
    }
    
    protected static class LocalMatcher extends ThreadLocal {
        protected Object initialValue() {
            return new Perl5Matcher();
        }
    }

    /**
     * @return Returns a {@link org.apache.oro.text.regex.Pattern} instance.
     */
    protected Pattern getPattern() {
        return pattern;
    }

    /**
     * Provides cache to store compiled regex pattern instances.
     * @param patternCache The patternCache to set.
     */
    public void setPatternCache(ISimpleCacheProviderFacade patternCache) {
        this.patternCache = patternCache;
    }

    public void setStringHandler(IStringPipelineDownstreamComponent downstreamFilter) {
        this.downstreamComponent=downstreamFilter;
    }

    public boolean canRenderString() {
        return downstreamComponent!=null;
    }
    
    /**
     * CorrectedOffsetMatch corrects match offsets under
     * an assumption that content before current match start could've been modified (extended, shortened),
     * but content after current match end is left intact.
     *
     */
    protected class CorrectedOffsetMatch implements MatchResult {
        MatchResult mr;
        int ld;
        
        /**
         * Construct a corrected offset match
         * @param mr origianl match result
         * @param lengthDifference difference between original and current string lengths
         */
        public CorrectedOffsetMatch(MatchResult mr,int lengthDifference) {
            this.mr=mr;
            this.ld=lengthDifference;
        }
 
        public int begin(int arg0) {
            return mr.begin(arg0);
        }
 
        public int beginOffset(int arg0) {
            int oo=mr.beginOffset(arg0);
            if(oo!=-1) {
                return oo-ld;
            } else {
                return oo;
            }
        }

        public int end(int arg0) {
            return mr.end(arg0);
        }

        public int endOffset(int arg0) {
            int oo=mr.endOffset(arg0);
            if(oo!=-1) {
                return oo-ld;
            } else {
                return oo;
            }
        }

        public String group(int arg0) {
            return mr.group(arg0);
        }

        public int groups() {
            return mr.groups();
        }

        public int length() {
            return mr.length();
        }
        
        public String toString() {
            return mr.toString();
        }
    }
}
