/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.string;

import java.io.Serializable;

import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheKeyList;
import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.rendering.IRenderingPipelineEventListener;
import org.jasig.portal.rendering.RenderingPipelineEvent;

/**
 * Default string cache filter implementation.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class StringCacheFilterImpl implements IStringCacheFilter, IRenderingPipelineEventListener {
    ISimpleCacheProviderFacade cache;
    IStringPipelineDownstreamComponent stringHandler;
    
    ICacheKeyList currentKey=null;
    CacheEntry currentEntry=null;
    boolean contentDemanded=false;

    /**
     * Since {@link IStringCacheFilter} has to implement {@link org.jasig.portal.caching.ICacheable}
     * interface, it has to return a key. Because the cache filter has no internal state to contribute
     * it reports a constant (empty) cache key. The <code>dummyKey</code> variable defines that key.
     */
    public final ICacheKey dummyKey=new CacheKeyImpl("");
    
     
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.string.IStringCacheFilter#getStringCacheValidityObjects()
     */
    public Serializable[] getStringCacheValidityObjects() {
        CacheEntry cacheEntry=this.getCurrentCacheEntry();
        if(cacheEntry!=null) {
            return cacheEntry.getValidityObjects();
        } else {
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.string.IStringPipelineUpstreamComponent#setStringHandler(org.jasig.portal.rendering.string.IStringPipelineDownstreamComponent)
     */
    public void setStringHandler(IStringPipelineDownstreamComponent downstreamFilter) {
        this.stringHandler=downstreamFilter;
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.ICacheFilter#setCurrentCacheKey(org.jasig.portal.caching.ICacheKeyList)
     */
    public void setCurrentCacheKey(ICacheKeyList cacheKey) {
        currentKey=cacheKey;
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.string.IStringFilter#filterRendering(java.lang.String, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public void filterRendering(String input) {
        // this is where decision about whether or not current content should be cached
        // is made. Currently, the implementation checks that cache key is set and
        // that getStringCache() has not been called before caching the content.
        // This assumes behavior of the cahce managers that call getStringCache() iff
        // cache entry passed validation.
        if(currentKey!=null && !contentDemanded) {
            // cache
            CacheEntry entry=new CacheEntry(input,currentKey.getValidityObjectList());
            cache.putInCache(currentKey.getKey(),entry);
        }
        stringHandler.filterRendering(input);
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.string.IStringCacheFilter#replayStringCache()
     */
    public boolean replayStringCache() {
        CacheEntry cacheEntry=this.getCurrentCacheEntry();
        if(cacheEntry!=null) {
            contentDemanded=true;
            stringHandler.filterRendering(cacheEntry.getContent());
            return true;
        } else {
            return false;
        }
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.string.IStringFilter#canRenderString()
     */
    public boolean canRenderString() {
        return (stringHandler!=null && stringHandler.canRenderString());
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.caching.ICacheable#generateKey()
     */
    public ICacheKey generateKey() {
        return dummyKey;
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.caching.ICacheable#isCacheValid(java.io.Serializable)
     */
    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }
    /**
     * @return Returns the cache.
     */
    public ISimpleCacheProviderFacade getCache() {
        return cache;
    }
    /**
     * @param cache The cache to set.
     */
    public void setCache(ISimpleCacheProviderFacade cache) {
        this.cache = cache;
    }
    
    CacheEntry getCurrentCacheEntry() {
        if(currentEntry==null) {
            Object currentCacheObject=cache.getFromCache(currentKey.getKey());
            if(currentCacheObject!=null) {
               currentEntry=(CacheEntry)currentCacheObject;
            }
        }
        return currentEntry;
    }
    
    class CacheEntry {
        Serializable[] validityObjects;
        String content;
        public CacheEntry(String content, Serializable[] validityObjects){
            this.content=content;
            this.validityObjects=validityObjects;
        }
        /**
         * @return Returns the content.
         */
        public String getContent() {
            return content;
        }
        /**
         * @param content The content to set.
         */
        public void setContent(String content) {
            this.content = content;
        }
        /**
         * @return Returns the validityObject.
         */
        public Serializable[] getValidityObjects() {
            return validityObjects;
        }
        /**
         * @param validityObjects The validityObject to set.
         */
        public void setValidityObjects(Serializable[] validityObjects) {
            this.validityObjects = validityObjects;
        }
    }
    
    void reset() {
        currentKey=null;
        currentEntry=null;
        contentDemanded=false;
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.IRenderingPipelineEventListener#reveiveSAXPipelineEvent(org.jasig.portal.rendering.RenderingPipelineEvent)
     */
    public void receiveSAXPipelineEvent(RenderingPipelineEvent ev) throws PortalException {
        if(ev.getEventType()==RenderingPipelineEvent.startRenderingCycleEventType) {
            this.reset();
        }
    }
}
