/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.string;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jasig.portal.rendering.PipelineComponentManager;

/**
 * A pipeline component mangager for configuring string rendering pipelines.
 * The pipeline is broken down explicitly into first element, last element and
 * a list of filters in the middle. First and last elements must not be <code>null</code>,
 * howerver the list of filters can be either <code>null</code> or empty. 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class StringPipelineComponentManager extends PipelineComponentManager {
    List pipelineFilters=null;
    IStringRenderer firstElement=null;
    IStringPipelineDownstreamComponent lastElement=null;
    
    
    /**
     * Set the first element of the rendering pipeline.
     * @param firstElement first element
     */
    public void setFirstElement(IStringRenderer firstElement) {
        super.setFirstPipelineElement(firstElement);
        this.firstElement=firstElement;
    }
    /**
     * Set the last element of the rendering pipeline.
     * @param lastElement last element.
     */
    public void setLastElement(IStringPipelineDownstreamComponent lastElement) {
        this.lastElement = lastElement;
        connectAndReportDownstreamElements();
    }
    /**
     * Set a list of pipeline filters. All filters must implement {@link IStringFilter}.
     * @param pipelineFilters pipeline filters.
     */
    public void setPipelineFilters(List pipelineFilters) {
        this.pipelineFilters = pipelineFilters;
        if(lastElement!=null) {
            // last element has been set, so we can report all downstream elements to the super
            connectAndReportDownstreamElements();
        }
    }
    
    /**
     * The method connects components together and reports all of the downstream
     * components to the super implementation.
     */
    protected void connectAndReportDownstreamElements() {
        // assemble components
        List downstreamElements=new ArrayList();
        if(pipelineFilters!=null) {
            downstreamElements.addAll(pipelineFilters);
        }
        downstreamElements.add(lastElement);
        
        // connect components
        Iterator iter=downstreamElements.iterator();
        IStringPipelineDownstreamComponent currentComponent=(IStringPipelineDownstreamComponent)iter.next();
        firstElement.setStringHandler(currentComponent);
        while(iter.hasNext()) {
            IStringPipelineUpstreamComponent previousComponent=(IStringPipelineUpstreamComponent)currentComponent;
            currentComponent=(IStringPipelineDownstreamComponent)iter.next();
            previousComponent.setStringHandler(currentComponent);
        }
  
        super.setDownstreamElements(downstreamElements);
    }
    /**
     * @return Returns the firstElement.
     */
    public IStringRenderer getFirstElement() {
        return firstElement;
    }
    /**
     * @return Returns the lastElement.
     */
    public IStringPipelineDownstreamComponent getLastElement() {
        return lastElement;
    }
    /**
     * @return Returns the pipelineFilters.
     */
    public List getPipelineFilters() {
        return pipelineFilters;
    }
}
