/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.security.acegi;

import java.util.Iterator;

import org.acegisecurity.Authentication;
import org.acegisecurity.ConfigAttribute;
import org.acegisecurity.ConfigAttributeDefinition;
import org.acegisecurity.vote.AccessDecisionVoter;
import org.aopalliance.intercept.MethodInvocation;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.jasig.gap.permissions.IAuthorizationService;
import org.springframework.beans.factory.annotation.Required;

/**
 * An ACEGI {@link org.acegisecurity.vote.AccessDecisionVoter} for
 * uPortal permissions. 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class PermissionVoter implements AccessDecisionVoter {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PermissionVoter.class);

    private IPermissionTargetResolver targetResolver;
    private String permissionOwner;
    private IAuthorizationService authorizationService;
    private IAuthorizationPrincipalAccessor authorizationPrincipalAccessor;
    private String activityPrefix=null;

    /**
     * This implementation supports only {@link MethodInvocation}
     */
    public boolean supports(Class clazz) {
        if (MethodInvocation.class.isAssignableFrom(clazz)) {
            return true;
        } else {
            return false;
        }
    }

    public boolean supports(ConfigAttribute ca) {
        return true;
    }
    
    public boolean isActivityAttribute(ConfigAttribute ca) {
        if(ca!=null) {
            if(activityPrefix!=null && !activityPrefix.equals("")) {
                // check prefix
                return ca.getAttribute().startsWith(activityPrefix);
            } else {
                return true;
            }
        } else {
            return false;
        }
    }
    
    public int vote(Authentication auth, Object secureObject, ConfigAttributeDefinition config) {
            int result = ACCESS_ABSTAIN;
            Iterator iter = config.getConfigAttributes();
            MethodInvocation invocation;
            
            if (secureObject instanceof MethodInvocation) {
                invocation = (MethodInvocation) secureObject;
            } else {
                if (logger.isDebugEnabled()) {
                    logger.debug("vote(Authentication, Object, ConfigAttributeDefinition) - secure object is not an instance of MethodInvocation, and is not supported. secure object="+secureObject);
                }
                return result;
            }
            
            IAuthorizationPrincipal principal=authorizationPrincipalAccessor.getAuthorizationPrincipal(auth,authorizationService);

            while (iter.hasNext()) {
                ConfigAttribute attribute = (ConfigAttribute) iter.next();

                if (this.isActivityAttribute(attribute)) {
                    result = ACCESS_DENIED;
                    
                    // resolve activity
                    String activity=attribute.getAttribute();
                    if(activityPrefix!=null && !activityPrefix.equals("")) {
                        // parse out activity prefix
                        activity=activity.substring(activityPrefix.length());
                    }
                    try {
                        String target=targetResolver.getTarget(invocation,config);
                        
                        if(authorizationService.doesPrincipalHavePermission(principal,this.permissionOwner,activity,target)) {
                            return ACCESS_GRANTED;
                        }
                    } catch (TargetResolutionException tre) {
                        // log and abstain
                        if (logger.isInfoEnabled()) {
                            logger.info("vote(Authentication, Object, ConfigAttributeDefinition) - unable to identify permission target : authorization principal=" + principal + ", attribute=" + attribute + ", activity=" + activity,tre);
                        }
                        return ACCESS_ABSTAIN;
                    }
                }
            }

            return result;
    }

    /**
     * Specify resolver to be used to determine permission target from method invokation
     * @param targetResolver
     */
    public void setTargetResolver(IPermissionTargetResolver targetResolver) {
        this.targetResolver = targetResolver;
    }

    /**
     * Specify prefix that should preceed activity config attributes. Example:
     * "org.jasig.portal.Class.publishPortl*=activity_Publish" in method interceptor config
     * would have "activity_" as the prefix.
     * Defaults to <code>null</code>, which means all config attributes are interpreted as activities.
     * @param activityPrefix
     */
    public void setActivityPrefix(String activityPrefix) {
        this.activityPrefix = activityPrefix;
    }

    /**
     * Specify owner for the permissions being checked.
     * @param permissionOwner
     */
    @Required
    public void setPermissionOwner(String permissionOwner) {
        this.permissionOwner = permissionOwner;
    }

    @Required
    public void setAuthorizationService(IAuthorizationService authorizationService) {
        this.authorizationService = authorizationService;
    }

    @Required
    public void setAuthorizationPrincipalAccessor(IAuthorizationPrincipalAccessor authorizationPrincipalAccessor) {
        this.authorizationPrincipalAccessor = authorizationPrincipalAccessor;
    }

}
