/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.security.acegi;

import java.util.Iterator;

import org.acegisecurity.ConfigAttribute;
import org.acegisecurity.ConfigAttributeDefinition;
import org.aopalliance.intercept.MethodInvocation;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.definition.PortletDefinitionId;

/**
 * A resolver that determines portlet definition id from one of the method arguments.
 * 
 * The resolver will resolve string version of portlet definition id from the argument
 * that is either {@link org.jasig.portal.portlet.definition.IPortletDefinition} or
 * {@link org.jasig.portal.portlet.definition.PortletDefinitionId} instance.
 * 
 * If <code>searchConfigurationAttributes</code> is <code>true</code> and <code>targetPrefix</code>
 * is not empty or <code>null</code>, the resolver will first examine config attributes for target specification.
 *
 * 
 * This can be used to do application-level authorization checks on a specific portlet definition.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletDefinitionIdResolver implements IPermissionTargetResolver {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PortletDefinitionIdResolver.class);
    
    String targetPrefix=null;
    boolean searchConfigurationAttributes=true;
    boolean allowMissingTarget=false;
    boolean returnTargetWithPrefix=true;
    

    public String getTarget(MethodInvocation invocation, ConfigAttributeDefinition config) throws TargetResolutionException{
        // check configuration attributes
        if(searchConfigurationAttributes && targetPrefix!=null && !targetPrefix.equals("")) {
            for(Iterator ci=config.getConfigAttributes(); ci.hasNext();) {
                final ConfigAttribute attribute = (ConfigAttribute) ci.next();
                final String value=attribute.getAttribute();
                if(value!=null && value.startsWith(targetPrefix)) {
                    // target identified
                    if (logger.isDebugEnabled()) {
                        logger.debug("getTarget(MethodInvocation, ConfigAttributeDefinition) - located permission target in the config attributes : value=" + value);
                    }

                    if(returnTargetWithPrefix) {
                        return value;
                    } else {
                        return value.substring(targetPrefix.length());
                    }
                }
            }
        }
        
        // search in method invocation arguments
        Class[] params = invocation.getMethod().getParameterTypes();
        for (int i = 0; i < params.length; i++) {
            if (IPortletDefinition.class.isAssignableFrom(params[i])) {
                final Object[] args = invocation.getArguments();
                final IPortletDefinition def=(IPortletDefinition) args[i];
                if(def!=null) {
                    final PortletDefinitionId defId=def.getPortletDefinitionId();
                    if(defId!=null) {
                        if (logger.isDebugEnabled()) {
                            logger.debug("getTarget(MethodInvocation, ConfigAttributeDefinition) - located permission target : defId=" + defId);
                        }
                        return defId.toString();
                    } else {
                        if(allowMissingTarget) {
                            return null;
                        } else {
                            throw new TargetResolutionException("null portletDefinitionId obtained from a non-nul portletDefinition object");
                        }
                    }
                } else {
                    if(allowMissingTarget) {
                        return null;
                    } else {
                        throw new TargetResolutionException("null portletDefinition argument found");
                    }
                }
            } else if(PortletDefinitionId.class.isAssignableFrom(params[i])) {
                final Object[] args = invocation.getArguments();
                final PortletDefinitionId defId=(PortletDefinitionId) args[i];
                if(defId!=null) {
                    if (logger.isDebugEnabled()) {
                        logger.debug("getTarget(MethodInvocation, ConfigAttributeDefinition) - located permission target : defId=" + defId);
                    }

                    if(returnTargetWithPrefix && targetPrefix!=null) {
                           return targetPrefix+defId.toString();
                    } else {
                        return defId.toString();
                    }
                } else {
                    if(allowMissingTarget) {
                        return null;
                    } else {
                        throw new TargetResolutionException("null portletDefinitionId obtained from a non-null portletDefinition object");
                    }
                }
            }
        }
        if(allowMissingTarget) {
            return null;
        } else {
            throw new TargetResolutionException("No target specified and allowMissingTarget=false");
        }
    }

    /**
     * Specify if the target returned by this resolver should include specified target prefix, or if it should be stripped out
     * prior to return.
     * @param returnTargetWithPrefix
     */
    public void setReturnTargetWithPrefix(boolean returnTargetWithPrefix) {
        this.returnTargetWithPrefix = returnTargetWithPrefix;
    }

    /**
     * String prefix that should preceed target specification in the configuration attributes.
     * Note: prefix can not be null if one would like to specify target in the config attributes.
     * Note: use <code>returnTargetWithPrefix</code> to control if the prefix should be stripped out prior to return.
     * @param targetPrefix a valid string prefix, or <code>null</code> or empty string for no prefix
     */
    public void setTargetPrefix(String targetPrefix) {
        this.targetPrefix = targetPrefix;
    }

    /**
     * Determine whether configuration attributes of the method interceptor should be searched for the target,
     * or if the resolver should only attempt to find the target in the method arguments.
     * (defaults to <code>true</code>)
     * 
     * Note: if set to <code>true</code>, make sure <code>targetPrefix</code> is not null or empty string, otherwise
     *  config attribute processing will be skipped.
     * @param searchConfigurationAttributes <code>true</code> if resolver should attempt to find permission target in the
     *  configuration attributes of the method interceptor prior to looking at the method arguments, <code>false</code> if
     *  the resolver should look only in the method arguments. 
     */
    public void setSearchConfigurationAttributes(boolean searchConfigurationAttributes) {
        this.searchConfigurationAttributes = searchConfigurationAttributes;
    }

    /**
     * Specify whether resolver should return <code>null</code> or throw an exception when the permission target
     * is not specified.
     * @param allowMissingTarget <code>true</code> if resolver should return <code>null</code> when permission target
     * has not been found, <code>false</code> if resolver should throw {@link TargetResolutionException} in such cases.
     */
    public void setAllowMissingTarget(boolean allowNullTarget) {
        this.allowMissingTarget = allowNullTarget;
    }
}
