/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.servants.groups.commands;

import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;

import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.jasig.gap.permissions.IAuthorizationService;
import org.jasig.gap.permissions.IPermission;
import org.jasig.gap.permissions.IUpdatingPermissionManager;
import org.jasig.portal.servants.groups.GroupsManagerConstants;
import org.jasig.portal.servants.groups.GroupsManagerXml;
import org.jasig.portal.servants.groups.GroupsSessionData;
import org.jasig.portal.servants.groups.IGroupsManagerCommand;
import org.jasig.portal.servants.groups.Utility;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * This class is the parent class of all other command classes.
 * @author Don Fracapane
 * @version $Revision: 11911 $
 */
public abstract class GroupsManagerCommand implements IGroupsManagerCommand, GroupsManagerConstants {
    
    private ICompositeGroupService compositeGroupService;
    
   /**
    * GroupsManagerCommand is the parent of all Groups Manager commands. It
    * hold the commone functionality of all commands.
    */
   public GroupsManagerCommand (ICompositeGroupService compositeGroupService) {
       this.compositeGroupService = compositeGroupService;
   }

   /**
    * This is the public method
    * @param sessionData
    * @throws Exception
    */
   abstract public void execute (GroupsSessionData sessionData, GroupsManagerXml groupsManagerXml,IAuthorizationService authorizationService) throws Exception;

   /**
    * clear out the selection list
    * @param sessionData
    */
   protected void clearSelected (GroupsSessionData sessionData) {
      Element rootElem = getXmlDoc(sessionData).getDocumentElement();
      NodeList nGroupList = rootElem.getElementsByTagName(GROUP_TAGNAME);
      NodeList nPersonList = rootElem.getElementsByTagName(ENTITY_TAGNAME);
      NodeList nList = nGroupList;
      for (int i = 0; i < nList.getLength(); i++) {
         Element elem = (org.w3c.dom.Element)nList.item(i);
         elem.setAttribute("selected", "false");
      }
      nList = nPersonList;
      for (int i = 0; i < nList.getLength(); i++) {
         Element elem = (org.w3c.dom.Element)nList.item(i);
         elem.setAttribute("selected", "false");
      }
      return;
   }

   /**
    * Removes all of the permissions for a GroupMember. We need to get permissions
    * for the group as a principal and as a target. I am merging the 2 arrays into a
    * single array in order to use the transaction management in the RDBMPermissionsImpl.
    * If an exception is generated, I do not delete the group or anything else.
    * Possible Exceptions: AuthorizationException and GroupsException
    * @param grpMbr
    * @throws Exception
    */
   protected void deletePermissions (IGroupMember grpMbr,IAuthorizationService authorizationService) throws Exception{
      try {
         String grpKey = grpMbr.getKey();
         // first we retrieve all permissions for which the group is the principal
         IAuthorizationPrincipal iap = authorizationService.newPrincipal(grpMbr);
         IPermission[] perms1 = iap.getPermissions();

         // next we retrieve all permissions for which the group is the target
         IUpdatingPermissionManager upm = authorizationService.newUpdatingPermissionManager(OWNER);
         IPermission[] perms2 = upm.getPermissions(null, grpKey);

         // merge the permissions
         IPermission[] allPerms = new IPermission[perms1.length + perms2.length];
         System.arraycopy(perms1,0,allPerms,0,perms1.length);
         System.arraycopy(perms2,0,allPerms,perms1.length,perms2.length);

         upm.removePermissions(allPerms);
      }
      catch (Exception e) {
         String errMsg = "DeleteGroup::deletePermissions(): Error removing permissions for " + grpMbr;
         Utility.logMessage("ERROR", errMsg, e);
         throw new Exception(errMsg);
      }
   }

   /**
    * Answers if the parentGroupId has been set. If it has not been set, this
    * would indicate that Groups Manager is in Servant mode.
    * @param staticData
    * @return boolean
    */
   protected boolean hasParentId (GroupsSessionData session) {
      String pk = getParentId(session);
      if (pk == null) {
         return  false;
      }
      if (pk.equals("")) {
         return  false;
      }
      Utility.logMessage("Debug", "GroupsManagerCommand::hasParentId: Value is set to default: "
            + pk);
      return  true;
   }

   /**
    * Returns the grpCommand parameter from runtimeData
    * @param runtimeData
    * @return String
    */
   protected String getCommand (PortletRequest request) {
      return request.getParameter(GROUP_COMMAND);
   }

   /**
    * Returns the grpCommandIds parameter from runtimeData. The string usually
    * hold one element ID but could contain a string of delimited ids. (See
    * RemoveMember command).
    * @param runtimeData
    * @return String
    */
   protected String getCommandArg (PortletRequest request) {
      String commandArg = request.getParameter(GROUP_COMMAND_ARG);
      // chained command args are set using the portlet request attributes
      if (commandArg == null) {
         commandArg = (String)request.getAttribute(GROUP_COMMAND_ARG);
      }
      return commandArg;
   }
   
   /**
    * Returns the groupParentId parameter from staticData
    * @param staticData
    * @return String
    */
   protected String getParentId (GroupsSessionData session) {
      return  (String) session.getAttribute("groupParentId");
   }

   /**
    * Returns the userID from the user object
    * @param sessionData
    * @return String
    */
   protected String getUserID (GroupsSessionData sessionData) {
      return  String.valueOf(sessionData.getUserId());
   }

   /**
    * Returns the cached xml document from staticData
    * @param sessionData
    * @return Document
    */
   protected Document getXmlDoc (GroupsSessionData sessionData) {
      return  sessionData.getModel();
   }

   /**
    * Set the CommandArg value, useful for commands which would like to chain
    * other commands
    * @param runtimeData
    * @param arg String
    */
   protected void setCommandArg (PortletRequest request, String arg) {
	   request.setAttribute(GROUP_COMMAND_ARG, arg);
   }

public ICompositeGroupService getCompositeGroupService() {
    return compositeGroupService;
}

}
