package org.jasig.portal.servants.permissions;

import java.util.Map;

import org.jasig.portal.groups.security.IPermissible;

/**
 * Maintains a collection of <code>IPermissible</code> classes that elect to
 * have some or all of their permissions managed externally. Registration
 * can be of simple Class<IPermissible> objects, which can get instantiated
 * by the permissible registry or more complex concrete IPermissible objects.
 */
public interface IPermissibleRegistry {

    /**
     * Adds a <code>IPermissible</code> to this registry.
     * 
     * @param permissible class which allows for certain permissions to be 
     *        managed externally.
     */
    public void register(IPermissible permissible);
    
    
    /**
     * Removes specified <code>IPermissible</code> from this registry. No-op if
     * the permissible was never registered.
     * 
     * @param permissible class which allows for certain permissions to be 
     *        managed externally.
     */
    public void unRegister(IPermissible permissible);
    
    /**
     * Returns all registered permissibles.
     * 
     * @return a <code>Map</code> of all permissible entities within the portal,
     *         keyed by class name. 
     */
    public Map<String, IPermissible> getAllPermissibles();
    
    /**
     * Returns the <code>IPermissible</code> identified by the owner key.
     * 
     * @param owner token from the <code>IPermissible</code>.
     * @return <code>IPermissible</code> identified by the owner token, or null if
     *         no permissible is registered with token.
     */
    public IPermissible getPermissible(String owner);
    
}
