package org.jasig.portal.servants.permissions.impl;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.groups.security.IPermissible;
import org.jasig.portal.servants.permissions.IPermissibleRegistry;

/**
 * <code>IPermissibleRegistry</code> implementation that stores registered 
 * permissibles in memory only. Registration can be threw 
 */
public class JvmPermissibleRegistry implements IPermissibleRegistry {
    
    private Map<String, IPermissible> owners = 
            new ConcurrentHashMap<String, IPermissible>();

    /**
     * @see org.jasig.portal.servants.permissions.IPermissibleRegistry#getAllPermissibles()
     */
    public Map<String, IPermissible> getAllPermissibles() {
        return Collections.unmodifiableMap(owners);
    }


    /**
     * @see org.jasig.portal.servants.permissions.IPermissibleRegistry#register(IPermissible)
     */
    public void register(IPermissible permissible) {
        if (permissible == null) {
            final String msg = "Argument 'permissible' cannot by null."; 
            throw new IllegalArgumentException(msg);
        }
        this.owners.put(permissible.getOwnerName(), permissible);
    }


    /**
     * @see org.jasig.portal.servants.permissions.IPermissibleRegistry#unRegister(IPermissible)
     */
    public void unRegister(IPermissible permissible) {
        if (permissible == null) {
            final String msg = 
                    "Argument 'permissible' of type Class cannot by null."; 
            throw new IllegalArgumentException(msg);
        }
        this.owners.remove(permissible.getOwnerName());
    }

    
    /**
     * @see org.jasig.portal.servants.permissions.IPermissibleRegistry#getPermissible(String)
     */
    public IPermissible getPermissible(String owner) {
        return owners.get(owner);
    }


    /**
     * Adds owner classes to the registry. This does not remove existing owners, but it
     * will replace any with identical keys.
     *   
     * @param owners <code>Map</code> of permissible classes keyed by class 
     *        name.
     */
    public void setOwners(List<IPermissible> owners) {
        if (owners == null) {
            final String msg = "Argument 'owners' of type Map cannot by null."; 
            throw new IllegalArgumentException(msg);
        }
        
        for (IPermissible p : owners) {
	        this.owners.put(p.getOwnerToken(), p);
        }
    }
}
