/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.spring;

import java.util.List;
import java.util.ListIterator;

import org.jasig.portal.utils.NormalizedPath;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.HierarchicalBeanFactory;
import org.springframework.beans.factory.NoSuchBeanDefinitionException;

/**
 * Implements context resolution methods used by both bean factory and application contexts.
 *
 * @author Peter Kharchenko, <pkharchenko at unicon.com>
 * @version $Revision: 11911 $
 */
public class ContextResolverImpl implements IContextResolver {

    public ContextResolverAnswer resolveFactoryAndName(IContextResolvingBeanFactory factory, String beanName) throws NoSuchBeanDefinitionException {
        if (beanName == null || beanName.length() == 0) {
            throw new IllegalArgumentException("Invalid bean name: '" + beanName + "'");
        }

        final NormalizedPath path = new NormalizedPath(beanName);
        final List pathElements = path.getPathElements();
        final ListIterator pathElementItr = pathElements.listIterator();
        
        final StringBuffer parsedNameBuff = new StringBuffer(path.toString().length());
        
        BeanFactory currentFactory = factory;
        
        //If the path starts with a seperator the currentFactory needs to point
        //at the root of the bean factory hierarchy
        if (pathElementItr.hasNext()) {
            if (NormalizedPath.PATH_SEPARATOR.equals(pathElementItr.next())) {
                parsedNameBuff.append(NormalizedPath.PATH_SEPARATOR);
                
                while (currentFactory instanceof HierarchicalBeanFactory && ((HierarchicalBeanFactory)currentFactory).getParentBeanFactory() != null) {
                    currentFactory = ((HierarchicalBeanFactory)currentFactory).getParentBeanFactory();
                }
            }
            else {
                pathElementItr.previous();
            }
        }
        
        //Iterate through the rest of the parsed path, navigating up and down the
        //bean factory hierarchy based on the path contents
        while (pathElementItr.hasNext()) {
            final String currentElement = (String)pathElementItr.next();
            parsedNameBuff.append(currentElement);
            
            if (NormalizedPath.PARENT_ELEMENT_NAME.equals(currentElement)) {
                if (currentFactory instanceof HierarchicalBeanFactory) {
                    currentFactory = ((HierarchicalBeanFactory)currentFactory).getParentBeanFactory();
                }
                else {
                    throw new NoSuchBeanDefinitionException(beanName, "context '" + parsedNameBuff + "' is not a HierarchicalBeanFactory");
                }
            }
            else if (pathElementItr.hasNext() &&
                    !NormalizedPath.PATH_SEPARATOR.equals(currentElement) &&
                    !NormalizedPath.CURRENT_ELEMENT_NAME.equals(currentElement)) {

                currentFactory = (BeanFactory)currentFactory.getBean(currentElement, BeanFactory.class);
            }
            
            //NormalizedPath.PATH_SEPARATOR and NormalizedPath.CURRENT_ELEMENT_NAME are ignored
        }
        
        //Get the last element from the path
        final String lastElement = (String)pathElements.get(pathElements.size() - 1);
        
        //If it is a special string the current BeanFactory is the target of the
        //path so CURRENT_ELEMENT_NAME is returned
        if (NormalizedPath.PATH_SEPARATOR.equals(lastElement) || 
                NormalizedPath.PARENT_ELEMENT_NAME.equals(lastElement) || 
                NormalizedPath.CURRENT_ELEMENT_NAME.equals(lastElement)) {

            //If the resolved factory is a IContextResolvingBeanFactory return the 
            if (currentFactory instanceof IContextResolvingBeanFactory) {
                return new ContextResolverAnswer(NormalizedPath.CURRENT_ELEMENT_NAME, ((IContextResolvingBeanFactory)currentFactory).getItself());
            }
            else {
                return new ContextResolverAnswer(NormalizedPath.CURRENT_ELEMENT_NAME, currentFactory);
            }
        }
        else {
            //If it is not a special element it is the name of the target of the
            //path, return it with BeanFactory that owns the bean.
            return new ContextResolverAnswer(lastElement, currentFactory);
        }
    }
}
