/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.spring;

import org.jasig.portal.utils.NormalizedPath;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanNotOfRequiredTypeException;
import org.springframework.beans.factory.NoSuchBeanDefinitionException;
import org.springframework.beans.factory.support.AbstractBeanFactory;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;

/**
 * ContextResolvingDefaultListableBeanFactory that supports hierarchical bean references.
 * <br>
 * <br>
 * Be aware that the methods of this class that override the methods defined in {@link org.springframework.beans.factory.BeanFactory}
 * have duplicated implementations in {@link ContextResolvingXmlBeanFactory}.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class ContextResolvingDefaultListableBeanFactory extends DefaultListableBeanFactory implements IContextResolvingBeanFactory {
    private final IContextResolvingBeanFactory outsideContext;
    private IContextResolver contextResolver = null;

    /**
     * Empty constructor
     */
    public ContextResolvingDefaultListableBeanFactory() {
        super();
        this.outsideContext = null;
    }

    /**
     * Parent-based constructor
     * @param parentBeanFactory
     */
    public ContextResolvingDefaultListableBeanFactory(BeanFactory parentBeanFactory) {
        this(null, parentBeanFactory);
    }
    
    /**
     * Parent-based constructor that specifies outside application context
     * @param parentBeanFactory
     */
    public ContextResolvingDefaultListableBeanFactory(IContextResolvingBeanFactory applicationContext, BeanFactory parentBeanFactory) {
        super(parentBeanFactory);
        this.outsideContext = applicationContext;
    }
    
    
    /**
     * @return Returns the contextResolver.
     */
    public IContextResolver getContextResolver() {
        return this.contextResolver;
    }
    /**
     * @param contextResolver The contextResolver to set.
     */
    public void setContextResolver(IContextResolver contextResolver) {
        this.contextResolver = contextResolver;
    }

    
    /* (non-Javadoc)
     * @see org.jasig.portal.spring.ContextResolvingBeanFactory#getItself()
     */
    public IContextResolvingBeanFactory getItself() {
        if (this.outsideContext != null) {
            return this.outsideContext;
        }
        else {
            return this;
        }
    }



    /**
     * @see org.springframework.beans.factory.BeanFactory#containsBean(java.lang.String)
     */
    public boolean containsBean(String name) {
        final ContextResolverAnswer answer;
        try {
            final IContextResolver contextResolver = this.getContextResolver();
            answer = contextResolver.resolveFactoryAndName(this, name);
        }
        catch (BeansException nsbde) {
            return false;
        }

        final String leafBeanName = answer.getLeafBeanName();

        if (NormalizedPath.CURRENT_ELEMENT_NAME.equals(leafBeanName)) {
            return answer.getOwnerFactory() != null;
        }
        else {
            final BeanFactory factory = answer.getOwnerFactory();
            
            if (this == factory) {
                return super.containsBean(leafBeanName);
            }
            else {
                return factory.containsBean(leafBeanName);
            }
        }
    }

    /**
     * @see org.springframework.beans.factory.BeanFactory#getAliases(java.lang.String)
     */
    public String[] getAliases(String name) throws NoSuchBeanDefinitionException {
        final IContextResolver contextResolver = this.getContextResolver();
        final ContextResolverAnswer answer = contextResolver.resolveFactoryAndName(this, name);

        final String leafBeanName = answer.getLeafBeanName();

        if (NormalizedPath.CURRENT_ELEMENT_NAME.equals(leafBeanName)) {
            return new String[0];
        }
        else {
            final BeanFactory factory = answer.getOwnerFactory();
            
            if (this == factory) {
                return super.getAliases(leafBeanName);
            }
            else {
                return factory.getAliases(leafBeanName);
            }
        }
    }

    /**
     * @see org.springframework.beans.factory.BeanFactory#getBean(java.lang.String)
     */
    public Object getBean(String name) throws BeansException {
        final IContextResolver contextResolver = this.getContextResolver();
        final ContextResolverAnswer answer = contextResolver.resolveFactoryAndName(this, name);
        final String leafBeanName = answer.getLeafBeanName();

        if (NormalizedPath.CURRENT_ELEMENT_NAME.equals(leafBeanName)) {
            return answer.getOwnerFactory();
        }
        else {
            final BeanFactory factory = answer.getOwnerFactory();
            
            if (this == factory) {
                return super.getBean(leafBeanName);
            }
            else {
                return factory.getBean(leafBeanName);
            }
        }
    }

    /**
     * @see org.springframework.beans.factory.BeanFactory#getBean(java.lang.String, java.lang.Class)
     */
    public Object getBean(String name, Class requiredType) throws BeansException {
        final IContextResolver contextResolver = this.getContextResolver();
        final ContextResolverAnswer answer = contextResolver.resolveFactoryAndName(this, name);
        final String leafBeanName = answer.getLeafBeanName();

        if (NormalizedPath.CURRENT_ELEMENT_NAME.equals(leafBeanName)) {
            final BeanFactory bean = answer.getOwnerFactory();
            
            if (requiredType != null && !requiredType.isAssignableFrom(bean.getClass())) {
                throw new BeanNotOfRequiredTypeException(name, requiredType, bean.getClass());
            }
            
            return bean;
        }
        else {
            final BeanFactory factory = answer.getOwnerFactory();
            
            if (this == factory) {
                return super.getBean(leafBeanName, requiredType);
            }
            else {
                return factory.getBean(leafBeanName, requiredType);
            }
        }
    }
    
    /**
     * @see org.springframework.beans.factory.suppurt.AbstractBeanFactory#getBean(java.lang.String, java.lang.Class, java.lang.Object[])
     */
    public Object getBean(String name, Class requiredType, Object[] args) throws BeansException {
        final IContextResolver contextResolver = this.getContextResolver();
        final ContextResolverAnswer answer = contextResolver.resolveFactoryAndName(this, name);
        final String leafBeanName = answer.getLeafBeanName();

        if (NormalizedPath.CURRENT_ELEMENT_NAME.equals(leafBeanName)) {
            final BeanFactory bean = answer.getOwnerFactory();
            
            if (requiredType != null && !requiredType.isAssignableFrom(bean.getClass())) {
                throw new BeanNotOfRequiredTypeException(name, requiredType, bean.getClass());
            }
            
            return bean;
        }
        else {
            final BeanFactory factory = answer.getOwnerFactory();
            
            if (this == factory) {
                return super.getBean(leafBeanName, requiredType, args);
            }
            else {
                
                if (factory instanceof AbstractBeanFactory) {
                    // Delegation to parent with args only possible for AbstractBeanFactory.
                    return ((AbstractBeanFactory) factory).getBean(leafBeanName, requiredType, args);
                }
                else if (args == null) {
                    // No args -> delegate to standard getBean method.
                    return factory.getBean(leafBeanName, requiredType);
                }
                else {
                    throw new NoSuchBeanDefinitionException(leafBeanName,
                            "Cannot delegate to parent BeanFactory because it does not supported passed-in arguments");
                }
            }
        }
    }

    /**
     * @see org.springframework.beans.factory.BeanFactory#getType(java.lang.String)
     */
    public Class getType(String name) throws NoSuchBeanDefinitionException {
        final IContextResolver contextResolver = this.getContextResolver();
        final ContextResolverAnswer answer = contextResolver.resolveFactoryAndName(this, name);
        final String leafBeanName = answer.getLeafBeanName();

        if (NormalizedPath.CURRENT_ELEMENT_NAME.equals(leafBeanName)) {
            return answer.getOwnerFactory().getClass();
        }
        else {
            final BeanFactory factory = answer.getOwnerFactory();
            
            if (this == factory) {
                return super.getType(leafBeanName);
            }
            else {
                return factory.getType(leafBeanName);
            }
        }
    }

    /**
     * @see org.springframework.beans.factory.BeanFactory#isSingleton(java.lang.String)
     */
    public boolean isSingleton(String name) throws NoSuchBeanDefinitionException {
        final IContextResolver contextResolver = this.getContextResolver();
        final ContextResolverAnswer answer = contextResolver.resolveFactoryAndName(this, name);
        final String leafBeanName = answer.getLeafBeanName();

        if (NormalizedPath.CURRENT_ELEMENT_NAME.equals(leafBeanName)) {
            return true;
        }
        else {
            final BeanFactory factory = answer.getOwnerFactory();
            
            if (this == factory) {
                return super.isSingleton(leafBeanName);
            }
            else {
                return factory.isSingleton(leafBeanName);
            }
        }
    }
}
