/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.spring;

import java.io.InputStream;

import org.jasig.portal.utils.NormalizedPath;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanNotOfRequiredTypeException;
import org.springframework.beans.factory.NoSuchBeanDefinitionException;
import org.springframework.beans.factory.xml.XmlBeanFactory;
import org.springframework.core.io.InputStreamResource;
import org.springframework.core.io.Resource;

/**
 * An extension of the XmlBeanFactory that resolves hierarchical reference syntax.
 * For example it is possible to specify <code>&lt;ref bean="../context/beanName"/&gt;</code>
 * <br>
 * <br>
 * Be aware that the methods of this class that override the methods defined in {@link org.springframework.beans.factory.BeanFactory}
 * have duplicated implementations in {@link ContextResolvingDefaultListableBeanFactory}.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class ContextResolvingXmlBeanFactory extends XmlBeanFactory implements IContextResolvingBeanFactory {
    private IContextResolver contextResolver = null;
	
	public ContextResolvingXmlBeanFactory(Resource resource, BeanFactory parentBeanFactory) throws BeansException {
        super(resource, parentBeanFactory);
    }

    public ContextResolvingXmlBeanFactory(InputStream is) throws BeansException {
        super(new InputStreamResource(is));
    }

    public ContextResolvingXmlBeanFactory(Resource resource) throws BeansException {
        super(resource);
    }
    

    /**
     * @return Returns the contextResolver.
     */
    public IContextResolver getContextResolver() {
        if (this.contextResolver == null) {
            this.contextResolver = new ContextResolverImpl();
        }
        
        return this.contextResolver;
    }
    /**
     * @param contextResolver The contextResolver to set.
     */
    public void setContextResolver(IContextResolver contextResolver) {
        this.contextResolver = contextResolver;
    }

    
    /* (non-Javadoc)
     * @see org.jasig.portal.spring.ContextResolvingBeanFactory#getItself()
     */
    public IContextResolvingBeanFactory getItself() {
        return this;
    }



    /**
     * @see org.springframework.beans.factory.BeanFactory#containsBean(java.lang.String)
     */
    public boolean containsBean(String name) {
        final ContextResolverAnswer answer;
        try {
            final IContextResolver contextResolver = this.getContextResolver();
            answer = contextResolver.resolveFactoryAndName(this, name);
        }
        catch (BeansException nsbde) {
            return false;
        }

        final String leafBeanName = answer.getLeafBeanName();

        if (NormalizedPath.CURRENT_ELEMENT_NAME.equals(leafBeanName)) {
            return answer.getOwnerFactory() != null;
        }
        else {
            final BeanFactory factory = answer.getOwnerFactory();
            
            if (this == factory) {
                return super.containsBean(leafBeanName);
            }
            else {
                return factory.containsBean(leafBeanName);
            }
        }
    }

    /**
     * @see org.springframework.beans.factory.BeanFactory#getAliases(java.lang.String)
     */
    public String[] getAliases(String name) throws NoSuchBeanDefinitionException {
        final IContextResolver contextResolver = this.getContextResolver();
        final ContextResolverAnswer answer = contextResolver.resolveFactoryAndName(this, name);

        final String leafBeanName = answer.getLeafBeanName();

        if (NormalizedPath.CURRENT_ELEMENT_NAME.equals(leafBeanName)) {
            return new String[0];
        }
        else {
            final BeanFactory factory = answer.getOwnerFactory();
            
            if (this == factory) {
                return super.getAliases(leafBeanName);
            }
            else {
                return factory.getAliases(leafBeanName);
            }
        }
    }

    /**
     * @see org.springframework.beans.factory.BeanFactory#getBean(java.lang.String)
     */
    public Object getBean(String name) throws BeansException {
        final IContextResolver contextResolver = this.getContextResolver();
        final ContextResolverAnswer answer = contextResolver.resolveFactoryAndName(this, name);
        final String leafBeanName = answer.getLeafBeanName();

        if (NormalizedPath.CURRENT_ELEMENT_NAME.equals(leafBeanName)) {
            return answer.getOwnerFactory();
        }
        else {
            final BeanFactory factory = answer.getOwnerFactory();
            
            if (this == factory) {
                return super.getBean(leafBeanName);
            }
            else {
                return factory.getBean(leafBeanName);
            }
        }
    }

    /**
     * @see org.springframework.beans.factory.BeanFactory#getBean(java.lang.String, java.lang.Class)
     */
    public Object getBean(String name, Class requiredType) throws BeansException {
        final IContextResolver contextResolver = this.getContextResolver();
        final ContextResolverAnswer answer = contextResolver.resolveFactoryAndName(this, name);
        final String leafBeanName = answer.getLeafBeanName();

        if (NormalizedPath.CURRENT_ELEMENT_NAME.equals(leafBeanName)) {
            final BeanFactory bean = answer.getOwnerFactory();
            
            if (requiredType != null && !requiredType.isAssignableFrom(bean.getClass())) {
                throw new BeanNotOfRequiredTypeException(name, requiredType, bean.getClass());
            }
            
            return bean;
        }
        else {
            final BeanFactory factory = answer.getOwnerFactory();
            
            if (this == factory) {
                return super.getBean(leafBeanName, requiredType);
            }
            else {
                return factory.getBean(leafBeanName, requiredType);
            }
        }
    }

    /**
     * @see org.springframework.beans.factory.BeanFactory#getType(java.lang.String)
     */
    public Class getType(String name) throws NoSuchBeanDefinitionException {
        final IContextResolver contextResolver = this.getContextResolver();
        final ContextResolverAnswer answer = contextResolver.resolveFactoryAndName(this, name);
        final String leafBeanName = answer.getLeafBeanName();

        if (NormalizedPath.CURRENT_ELEMENT_NAME.equals(leafBeanName)) {
            return answer.getOwnerFactory().getClass();
        }
        else {
            final BeanFactory factory = answer.getOwnerFactory();
            
            if (this == factory) {
                return super.getType(leafBeanName);
            }
            else {
                return factory.getType(leafBeanName);
            }
        }
    }

    /**
     * @see org.springframework.beans.factory.BeanFactory#isSingleton(java.lang.String)
     */
    public boolean isSingleton(String name) throws NoSuchBeanDefinitionException {
        final IContextResolver contextResolver = this.getContextResolver();
        final ContextResolverAnswer answer = contextResolver.resolveFactoryAndName(this, name);
        final String leafBeanName = answer.getLeafBeanName();

        if (NormalizedPath.CURRENT_ELEMENT_NAME.equals(leafBeanName)) {
            return true;
        }
        else {
            final BeanFactory factory = answer.getOwnerFactory();
            
            if (this == factory) {
                return super.isSingleton(leafBeanName);
            }
            else {
                return factory.isSingleton(leafBeanName);
            }
        }
    }
}
