/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.spring;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;


/**
 * An implementation of HttpServlet which reads an init param with the name
 * {@link #CONFIG_PROVIDER_BEAN_NAME} to get the name of a spring bean that
 * implements {@link IServletConfigProviderBean}.
 * <br>
 * Subclasses should be sure to call super.init() if they override the
 * init() method. 
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public abstract class ServletConfigAwareProcessorServlet extends HttpServlet {
    private static final Log logger = LogFactory.getLog(ServletConfigAwareProcessorServlet.class);
    
    /**
     * The name of the init-param to get to get the bean name to configure from
     */
    public static final String CONFIG_PROVIDER_BEAN_NAME = "ServletConfigProviderBean";
    
    /**
     * Init method which tries to configure the specified bean with the ServletConfig
     * 
     * @see javax.servlet.GenericServlet#init()
     */
    public void init() throws ServletException {
        super.init();
        
        final ApplicationContext appContext = WebApplicationContextUtils.getWebApplicationContext(getServletContext());
        
        final ServletConfig conf = this.getServletConfig();
        

        final String beanName = conf.getInitParameter(CONFIG_PROVIDER_BEAN_NAME);
        
        try {
            final IServletConfigProviderBean bean = (IServletConfigProviderBean)appContext.getBean(beanName, IServletConfigProviderBean.class);
            bean.setServletConfig(conf);
        }
        catch (final Exception e) {
            logger.error("Error injecting ServletConfig into bean " + beanName + ". Processing will continute.", e);
        }
    }
}
