/*
 * Copyright 2007 The JA-SIG Collaborative. All rights reserved. See license distributed with this
 * file and available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.spring;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.collections.map.ReferenceMap;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.DestructionAwareBeanPostProcessor;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

/**
 * Listens to bean creation and descruction events in the context, all singleton beans are registered
 * in a map by type and name. This allows O(1) time access to beans of a particular type.
 * 
 * @author Eric Dalquist
 * @version $Revision$
 */
public class TypeAwareBeanRegistryPostProcessor implements TypeAwareBeanRegistry<Object>, DestructionAwareBeanPostProcessor, ApplicationContextAware {
    private final Map<Class<? extends Object>, Map<String, Object>> beanRegistry = new HashMap<Class<? extends Object>, Map<String, Object>>();
    private ApplicationContext                                      applicationContext;
    
    
    /**
     * @see org.jasig.portal.spring.TypeAwareBeanRegistry#getBeansOfType(java.lang.Class)
     */
    public Map<String, Object> getBeansOfType(Class clazz) {
        final Map<String, Object> beans = this.beanRegistry.get(clazz);

        if (beans == null) {
            return Collections.emptyMap();
        }
        else {
            return Collections.unmodifiableMap(beans);
        }
    }

    /**
     * @see org.springframework.context.ApplicationContextAware#setApplicationContext(org.springframework.context.ApplicationContext)
     */
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }

    /**
     * @see org.springframework.beans.factory.config.DestructionAwareBeanPostProcessor#postProcessBeforeDestruction(java.lang.Object, java.lang.String)
     */
    @SuppressWarnings("unchecked")
    public void postProcessBeforeDestruction(Object bean, String beanName) throws BeansException {
        Class<? extends Object> clazz = bean.getClass();

        final Class<? extends Object>[] interfaces = clazz.getInterfaces();
        for (final Class<? extends Object> interfc : interfaces) {
            this.deregisterBean(interfc, beanName);
        }
        
        while (!Object.class.equals(clazz)) {
            this.deregisterBean(clazz, beanName);
            clazz = clazz.getSuperclass();
        }
    }

    /**
     * @see org.springframework.beans.factory.config.BeanPostProcessor#postProcessAfterInitialization(java.lang.Object,
     *      java.lang.String)
     */
    @SuppressWarnings("unchecked")
    public Object postProcessAfterInitialization(Object bean, String beanName) throws BeansException {
        Class<? extends Object> clazz = bean.getClass();

        if (this.shouldProcess(beanName)) {
            final Class<? extends Object>[] interfaces = clazz.getInterfaces();
            for (final Class<? extends Object> interfc : interfaces) {
                this.registerBean(interfc, beanName, bean);
            }
        }
        
        while (!Object.class.equals(clazz)) {
            this.registerBean(clazz, beanName, bean);
            clazz = clazz.getSuperclass();
        }

        return bean;
    }

    /**
     * @see org.springframework.beans.factory.config.BeanPostProcessor#postProcessBeforeInitialization(java.lang.Object,
     *      java.lang.String)
     */
    public Object postProcessBeforeInitialization(Object bean, String beanName) throws BeansException {
        return bean; // This is a no-op method
    }

    protected boolean shouldProcess(String beanName) {
        try {
            return this.applicationContext.isSingleton(beanName);
        }
        catch (BeansException nsbde) {
            return false;
        }
    }

    private void registerBean(Class<? extends Object> clazz, String beanName, Object bean) {
        final Map<String, Object> beans = this.getOrCreateBeans(clazz);
        beans.put(beanName, bean);
    }

    private void deregisterBean(Class<? extends Object> clazz, String beanName) {
        final Map<String, Object> beans = this.getOrCreateBeans(clazz);
        beans.remove(beanName);
    }

    @SuppressWarnings("unchecked")
    private synchronized Map<String, Object> getOrCreateBeans(final Class<? extends Object> clazz) {
        Map<String, Object> beans = this.beanRegistry.get(clazz);
        if (beans == null) {
            beans = new ReferenceMap(ReferenceMap.HARD, ReferenceMap.WEAK);
            this.beanRegistry.put(clazz, beans);
        }

        return beans;
    }
}
