package org.jasig.portal.tools.dbloader;

import java.sql.Types;

import org.jasig.portal.tools.dbloader.utils.DbUtils;

/**
 * Maps from Integers representing JDBC types to {@link IColumnTargetType}s. 
 * 
 * @author Dan McCallum, dmccallum@unicon.net
 *
 */
public class ColumnMetaDataTypeMapping {

	private Integer columnMetaDataType;
	private IColumnTargetType columnTargetType;
	
	/**
	 * Identical to invoking 
	 * <code>new ColumnMetaDataTypeMapping(DbUtils.getJavaSqlType(String)), IColumnTargetType)</code>.
	 * 
	 * @param columnMetaDataType logical name of a JDBC type resolvable by 
	 *   {@link org.jasig.portal.tools.dbloader.utils.DbUtils#getJavaSqlType(String)}.
	 * @param columnTargetType represents the JDBC type(s) to which <code>columnMetaDataType</code> 
	 *   maps
	 */
	public ColumnMetaDataTypeMapping(String columnMetaDataType, IColumnTargetType columnTargetType) {
		
		this(new Integer(DbUtils.getJavaSqlType(columnMetaDataType)), columnTargetType);
		
	}
	
	/**
	 * Initialize this <code>ColumnMetaDataTypeMapping</code>, specifying the JDBC type
	 * to map and the target mapping.
	 * 
	 * @param columnMetaDataType
	 * @param columnTargetType
	 * @throws IllegalArgumentException if <code>columnMetaDataType</code> is null or otherwise
	 *   invalid, or if <code>columnTargetType</code> is null.
	 */
	public ColumnMetaDataTypeMapping(Integer columnMetaDataType, IColumnTargetType columnTargetType) {
		
		if ( columnMetaDataType == null ) {
			throw new IllegalArgumentException("must specify a column meta data type");
		}

		if ( columnMetaDataType.equals(new Integer(Types.NULL)) ) { // the default return value, which makes no sense for our purposes
			throw new IllegalArgumentException("Must specify a valid JDBC column type. Received [" + columnMetaDataType + "]");
		}
		
		if ( columnTargetType == null ) {
			throw new IllegalArgumentException("must specify a target column type");
		}
		
		
		this.columnMetaDataType = columnMetaDataType;
		this.columnTargetType = columnTargetType;
		
	}
	
	/**
	 * Access the target JDBC type for this mapping, potentially using a name 
	 * associated with the original type as a discriminator. Implemented as 
	 * a pass-through to the underlying IColumnTargetType.
	 * 
	 * @param columnMetaDataTypeName the name of the jdbc type to be mapped. In
	 *   some cases, type names returned with column meta data can be used to 
	 *   disambiguate type codes.
	 * @return the target JDBC type or null of no match
	 */
	public Integer getTargetType(String columnMetaDataTypeName) {
		return columnTargetType.getTargetType(columnMetaDataTypeName);
	}

	/**
	 * Access the JDBC type <em>from which</em> this <code>ColumnMetaDataTypeMapping</code>
	 * will map to another JDBC type.
	 * 
	 * @return an Integer representing a JDBC type
	 */
	public Integer getColumnMetaDataType() {
		return columnMetaDataType;
	}
	
}
