package org.jasig.portal.tools.dbloader;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Collects {@link ColumnMetaDataTypeMapping} instances. 
 * 
 * 
 * @author Dan McCallum, dmccallum@unicon.net
 *
 */
public class ColumnMetaDataTypeMappings {

	private Map mappings;

	/**
	 * Initialize this <code>ColumnMetaDataTypeMappings</code> with
	 * an empty collection of {@link ColumnMetaDataTypeMapping}s.
	 *
	 */
	public ColumnMetaDataTypeMappings() {
		// no-op
	}
	
	/**
	 * See {@link #setColumnMetaDataTypeMappings(Collection)}
	 */
	public ColumnMetaDataTypeMappings(Collection seedMappings) {
		setColumnMetaDataTypeMappings(seedMappings);
	}
	
	/**
	 * Assigns the specified <code>Collection</code> of {@link ColumnMetaDataTypeMapping}.
	 * 
	 * @param seedMappings a <code>Collection</code> of {@link ColumnMetaDataTypeMapping}
	 * @throws ClassCastException if <code>seedMappings</code> contains non-
	 *   ColumnMetaDataTypeMapping objects.
	 */
	public void setColumnMetaDataTypeMappings(Collection seedMappings) {
		if ( seedMappings != null ) {
			  mappings = new HashMap();
			  for (final Iterator mappingsItr = seedMappings.iterator(); mappingsItr.hasNext(); ) {
				  ColumnMetaDataTypeMapping mapping = (ColumnMetaDataTypeMapping)mappingsItr.next();
				  mappings.put(mapping.getColumnMetaDataType(), mapping);
			  }
		} else {
			mappings = null;
		}
	}
	
	
	/**
	 * Maps a given JDBC data type to another JDBC data type. The String parameter
	 * can be leveraged as a discriminator if the database meta data uses a
	 * type name to distinguish between multiple native type sharing the same
	 * JDBC type. Echoes the requested type if any of the following are true:
	 * 
	 * <ol>
	 *   <li>No mappings have been assigned to this DbTypeMapping</li>
	 *   <li>No mappings exist for the specified type</li>
	 *   <li>Mulitple mappings exist for the specified type, but none for
	 *   the specified type name</li>
	 * </ol>
	 * 
	 * 
	 * @param typeToMap an Integer representing a data type enumerated by java.sql.Types 
	 * @param typeToMapName the data type name reported by querying db meta-data
	 * @return the resolved type, or <code>typeToMap</code> if mapping failed.  
	 */
	public Integer mapColumnMetaDataType(Integer typeToMap, String typeToMapName) {
		
		if ( mappings == null ) {
			return typeToMap;
		}
		
		ColumnMetaDataTypeMapping mapping = 
			(ColumnMetaDataTypeMapping)mappings.get(typeToMap);
		
		if ( mapping == null ) {
			return typeToMap;
		}
		
		Integer targetType = mapping.getTargetType(typeToMapName);
		if ( targetType == null ) {
			return typeToMap;
		}
		
		return targetType;
		
	}
	
}
