/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.tools.dbloader;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.StringTokenizer;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.tools.dbloader.utils.DbUtils;
import org.springframework.jdbc.core.JdbcTemplate;

/**
 * Holds all configuration values used in DbLoader as part of loading a database.
 *
 * @author Ken Weiner, kweiner@unicon.net
 * @author Mark Boyd <mark.boyd@engineer.com>
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @author Dan McCallum, dmccallum@unicon.net
 * @version $Revision: 11911 $
 */
public class Configuration
{
    public static final String TYPE_MAPPING_JDBC = "JDBC";
    public static final String TYPE_MAPPING_XML  = "XML";
    
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private DataSource dataSource;
    private DbInfo dbInfo;
    private DbTypeMappings dbTypeMappings;
    private JdbcTemplate template;
    
    
    private boolean recoverExceptions = true;
    
    private boolean dropTables = true;
    private boolean createTables = true;
    private boolean populateTables = true;
    private boolean createForeignKeys = true;
    private boolean dropForeignKeys = true;
    
    private String tablesXml;
    private String tablesXsl = "tables.xsl";
    private String dataXml;
    private String dataXsl = "data.xsl";
    
    private boolean createScript = false;
    private String scriptFile = "dbloader.script";
    private String sqlStatementTerminator = ";";
    private PrintWriter scriptWriter= null;
    
    private String upgradeVersion;
    private int upgradeMajor;
    private int upgradeMinor;
    
    private boolean validateXml = false;
    
    private String typeMappingMethod = TYPE_MAPPING_JDBC;
    
    
    
    /**
     * @return The DbInfo object for the configured DataSource;
     */
    public DbInfo getDbInfo() {
        if (this.dbInfo == null) {
            this.dbInfo = DbUtils.getDatabaseInfo(this.getDataSource());
        }
        
        return this.dbInfo;
    }
    /**
     * @return The PrintWriter to write the SQL script to, null if script logging is disabled
     */
    public PrintWriter getScriptWriter() {
        if (createScript && this.scriptWriter == null) {
            try {
                final File outputFile = new File(this.getScriptFile());
                this.logger.debug("Writing script to: " + outputFile.toURL());
                final FileWriter writer = new FileWriter(outputFile); 
                this.scriptWriter = new PrintWriter(writer);
            }
            catch (IOException ioe) {
                this.logger.error("Error creating FileWriter for '" + this.getScriptFile() + "'", ioe);
            }
        }

        return this.scriptWriter;
    }
    

    /**
     * @param upgradeVersion The upgradeVersion to set.
     */
    public void setUpgradeVersion(String upgradeVersion) {
        this.upgradeVersion = upgradeVersion;
        
        final StringTokenizer versionParts = new StringTokenizer(upgradeVersion, ".");
        if (versionParts.hasMoreElements()) {
            final String versionString = (String)versionParts.nextElement();
            final int version = Integer.parseInt(versionString);
            this.upgradeMajor = version;
        }
        else if (versionParts.hasMoreElements()) {
            final String versionString = (String)versionParts.nextElement();
            final int version = Integer.parseInt(versionString);
            this.upgradeMinor = version;
        }
    }
    /**
     * @param mappingMethod The type mapping method to use {@link #TYPE_MAPPING_XML} or {@link #TYPE_MAPPING_JDBC}. 
     */
    public void setTypeMappingMethod(String mappingMethod) {
        if (TYPE_MAPPING_XML.equalsIgnoreCase(mappingMethod))
            this.typeMappingMethod = TYPE_MAPPING_XML;
        else
            this.typeMappingMethod = TYPE_MAPPING_JDBC;
    }


    /**
     * @return Returns the createForeignKeys.
     */
    public boolean isCreateForeignKeys() {
        return this.createForeignKeys;
    }
    /**
     * @return Returns the createScript.
     */
    public boolean isCreateScript() {
        return this.createScript;
    }
    /**
     * @return Returns the createTables.
     */
    public boolean isCreateTables() {
        return this.createTables;
    }
    /**
     * @return Returns the dataSource.
     */
    public DataSource getDataSource() {
        return this.dataSource;
    }
    /**
     * @return Returns the dataXml.
     */
    public String getDataXml() {
        return this.dataXml;
    }
    /**
     * @return Returns the dataXsl.
     */
    public String getDataXsl() {
        return this.dataXsl;
    }
    /**
     * @return Returns the dbTypeMappings.
     */
    public DbTypeMappings getDbTypeMappings() {
        return this.dbTypeMappings;
    }
    /**
     * @return Returns the dropForeignKeys.
     */
    public boolean isDropForeignKeys() {
        return this.dropForeignKeys;
    }
    /**
     * @return Returns the dropTables.
     */
    public boolean isDropTables() {
        return this.dropTables;
    }
    /**
     * @return Returns the populateTables.
     */
    public boolean isPopulateTables() {
        return this.populateTables;
    }
    /**
     * @return Returns the recoverExceptions.
     */
    public boolean isRecoverExceptions() {
        return this.recoverExceptions;
    }
    /**
     * @return Returns the scriptFile.
     */
    public String getScriptFile() {
        return this.scriptFile;
    }
    /**
     * @return Returns the sqlStatementTerminator.
     */
    public String getSqlStatementTerminator() {
        return this.sqlStatementTerminator;
    }
    /**
     * @return Returns the tablesXml.
     */
    public String getTablesXml() {
        return this.tablesXml;
    }
    /**
     * @return Returns the tablesXsl.
     */
    public String getTablesXsl() {
        return this.tablesXsl;
    }
    /**
     * @return Returns the template.
     */
    public JdbcTemplate getTemplate() {
        return this.template;
    }
    /**
     * @return Returns the upgradeMajor.
     */
    public int getUpgradeMajor() {
        return this.upgradeMajor;
    }
    /**
     * @return Returns the upgradeMinor.
     */
    public int getUpgradeMinor() {
        return this.upgradeMinor;
    }
    /**
     * @return Returns the upgradeVersion.
     */
    public String getUpgradeVersion() {
        return this.upgradeVersion;
    }
    /**
     * @return Returns the validateXml.
     */
    public boolean isValidateXml() {
        return this.validateXml;
    }
    /**
     * @return Returns the typeMappingMethod;
     */
    public String getTypeMappingMethod() {
        return this.typeMappingMethod;
    }


    /**
     * @param createForeignKeys The createForeignKeys to set.
     */
    public void setCreateForeignKeys(boolean createForeignKeys) {
        this.createForeignKeys = createForeignKeys;
    }
    /**
     * @param createScript The createScript to set.
     */
    public void setCreateScript(boolean createScript) {
        this.createScript = createScript;
    }
    /**
     * @param createTables The createTables to set.
     */
    public void setCreateTables(boolean createTables) {
        this.createTables = createTables;
    }
    /**
     * @param dataSource The dataSource to set.
     */
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }
    /**
     * @param dataXml The dataXml to set.
     */
    public void setDataXml(String dataXml) {
        this.dataXml = dataXml;
    }
    /**
     * @param dataXsl The dataXsl to set.
     */
    public void setDataXsl(String dataXsl) {
        this.dataXsl = dataXsl;
    }
    /**
     * @param dbTypeMappings The dbTypeMappings to set.
     */
    public void setDbTypeMappings(DbTypeMappings dbTypeMappings) {
        this.dbTypeMappings = dbTypeMappings;
    }
    /**
     * @param dropForeignKeys The dropForeignKeys to set.
     */
    public void setDropForeignKeys(boolean dropForeignKeys) {
        this.dropForeignKeys = dropForeignKeys;
    }
    /**
     * @param dropTables The dropTables to set.
     */
    public void setDropTables(boolean dropTables) {
        this.dropTables = dropTables;
    }
    /**
     * @param populateTables The populateTables to set.
     */
    public void setPopulateTables(boolean populateTables) {
        this.populateTables = populateTables;
    }
    /**
     * @param recoverExceptions The recoverExceptions to set.
     */
    public void setRecoverExceptions(boolean recoverExceptions) {
        this.recoverExceptions = recoverExceptions;
    }
    /**
     * @param scriptFile The scriptFile to set.
     */
    public void setScriptFile(String scriptFile) {
        this.scriptFile = scriptFile;
    }
    /**
     * @param scriptWriter The scriptWriter to set.
     */
    public void setScriptWriter(PrintWriter scriptWriter) {
        this.scriptWriter = scriptWriter;
    }
    /**
     * @param sqlStatementTerminator The sqlStatementTerminator to set.
     */
    public void setSqlStatementTerminator(String sqlStatementTerminator) {
        this.sqlStatementTerminator = sqlStatementTerminator;
    }
    /**
     * @param tablesXml The tablesXml to set.
     */
    public void setTablesXml(String tablesXml) {
        this.tablesXml = tablesXml;
    }
    /**
     * @param tablesXsl The tablesXsl to set.
     */
    public void setTablesXsl(String tablesXsl) {
        this.tablesXsl = tablesXsl;
    }
    /**
     * @param template The template to set.
     */
    public void setTemplate(JdbcTemplate template) {
        this.template = template;
    }
    /**
     * @param validateXml The validateXml to set.
     */
    public void setValidateXml(boolean validateXml) {
        this.validateXml = validateXml;
    }
    
    /**
     * Matches the given table name against an exclude
     * pattern assigned to the DbMapping for the current
     * DbInfo.
     * 
     * <p>This was introducted primarily for Oracle to avoid
     * problems with BIN tables when retrieving column meta-data.
     * 
     * @param tableName
     */
    public boolean isExcludedTableName(String tableName) {
    	
        return getCurrentDbTypeMapping().isExcludedTableName(tableName);
        
    }
    
    /**
     * Executes the given Statement using the currently assigned
     * JdbcTemplate, possibly modifying the arg and type sets.
     * Arg and type filtering is delegated to the current DbTypeMapping's
     * DmlFilter.
     */
    public void execUpdate(String statement, Object[] args, int[] types) {
    	
    	Dml dml = new Dml(statement, args, types);
    	dml = getCurrentDbTypeMapping().getDmlFilter().filter(dml);
    	getTemplate().update(dml.getStatement(), dml.getArgs(), dml.getTypes());
    	
    }
    
    /**
     * Find the DbTypeMapping instance that corresponds to the currently
     * selected DbInfo instance.
     * 
     * @return a DbTypeMapping for the current DbInfo
     */
    public DbTypeMapping getCurrentDbTypeMapping() {
    	return getDbTypeMappings().getMappingForDb(getDbInfo());
    }
}