/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.tools.dbloader;

import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamResult;

import org.jasig.portal.utils.XmlLocationStack;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLFilter;
import org.xml.sax.XMLReader;
import org.xml.sax.ext.LexicalHandler;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.XMLFilterImpl;
import org.xml.sax.helpers.XMLReaderFactory;


/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class DataSequenceAnnotator {


    public static void main(String[] args) throws Exception {
        final URL dataXmlUrl = DataSequenceAnnotator.class.getResource("/properties/db/data.xml");
        
        final TransformerFactory transFactory = TransformerFactory.newInstance();
        final Transformer trans = transFactory.newTransformer();
        
        
        final XMLReader reader = XMLReaderFactory.createXMLReader();
        
        //The type filter sits between the XSLT transformer and the XMLReader
        //it maps the database column types as the XML document is parsed
        final XMLFilter typeFilter = new DataSequenceAnnotatingFilter(reader);
        
        //The sources based on the SAX filters
        final InputSource xmlStreamSource = new InputSource(dataXmlUrl.openStream());
        final Source xmlSource = new SAXSource(typeFilter, xmlStreamSource);
        
        //Use a SAX result piping the events to our TableHandler which executes the SQL
        final Result transResult = new StreamResult(System.out);
        
        //Execute the transformation, this triggers the reading of the XML
        //document which runs it through the filters, performs the XSLT and
        //sends the output events to the table handler
        trans.transform(xmlSource, transResult);
    }
    
    private static class DataSequenceAnnotatingFilter extends XMLFilterImpl {
        private static final String DATA_LOC            = "/data";
        private static final String SEQUENCE_LOC        = "/data/table/rows/row/column/sequence";
        private static final String VALUE_ELEMENT_NAME  = "value";
        private static final String ID_ATTRIBUTE_NAME   = "id";
        
        private final XmlLocationStack location = new XmlLocationStack();
        private final Map seqValues = new HashMap();
        private final Map namedSeqValues = new HashMap(); 
        
        private StringBuffer chars = null;
        private String seqName = null;
        
        /**
         * Creates a new filter, specifies the reader to pass on events too and the
         * configuration to use.
         * 
         * @param parent The XMLReader to pass events on to.
         */
        public DataSequenceAnnotatingFilter(final XMLReader parent) {
            super(parent);
        }
        
        
        /**
         * For sequence elements the chars buffer is created, the sequence name
         * is pulled fromt the id attribute and a value element is started.
         * 
         * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
         */
        public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
            final String locationPath = this.location.startElement(qName);
            
            if (SEQUENCE_LOC.equalsIgnoreCase(locationPath)) {
                this.chars = new StringBuffer(256);
                this.seqName = atts.getValue(ID_ATTRIBUTE_NAME);
            }
            else
                super.startElement(uri, localName, qName, atts);
        }
        
        /**
         * If in a sequence element buffer the character data.
         * 
         * @see org.xml.sax.ContentHandler#characters(char[], int, int)
         */
        public void characters(char[] ch, int start, int length) throws SAXException {
            if (this.chars != null)
                this.chars.append(ch, start, length);
            else
                super.characters(ch, start, length);
        }

        /**
         * If ending a sequnce element the value key is computed and the cache is
         * consulted to see if a value for that sequence key exists. If it does it
         * is used if not a new sequence number is generated and stored in the cache.
         * 
         * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
         */
        public void endElement(String uri, String localName, String qName) throws SAXException {
            final String locationPath = this.location.endElement(qName);
            
            if (SEQUENCE_LOC.equalsIgnoreCase(locationPath)) {
                final String seqId = this.chars.toString().trim();
                final String valueKey = this.seqName + "." + seqId;
                
                Integer value = (Integer)namedSeqValues.get(valueKey);
                if (value == null) {
                    value = (Integer)seqValues.get(this.seqName);
                    
                    if (value == null)
                        value = new Integer(0);
                        
                    value = new Integer(value.intValue() + 1);
                    seqValues.put(this.seqName, value);
                    namedSeqValues.put(valueKey, value);
                }
                
                final XMLReader parent = this.getParent();
                final LexicalHandler lexHandler = (LexicalHandler)parent.getProperty("http://xml.org/sax/properties/lexical-handler");
                
                if (lexHandler != null) {
                    final String comment = " " + this.seqName + " : " + seqId + " ";
                    lexHandler.comment(comment.toCharArray(), 0, comment.length());
                }
                
                final String valueStr = value.toString();

                super.startElement(uri, VALUE_ELEMENT_NAME, VALUE_ELEMENT_NAME, new AttributesImpl());
                this.characters(valueStr);
                super.endElement(uri, VALUE_ELEMENT_NAME, VALUE_ELEMENT_NAME);
                
                if (lexHandler == null) {
                    final AttributesImpl attrs = new AttributesImpl();
                    attrs.addAttribute(uri, "name", "name", "string", this.seqName);
                    attrs.addAttribute(uri, "name", "name", "string", seqId);
                    
                    super.startElement(uri, "seqhint", "seqhint", attrs);
                    super.endElement(uri, "seqhint", "seqhint");
                }
                
                this.chars = null;
            }
            else if (DATA_LOC.equalsIgnoreCase(locationPath)) {
                final Attributes emptyAttrs = new AttributesImpl();
                
                this.ignorableWhitespace("\n    \n    ");
                
                super.startElement(uri, "table", "table", emptyAttrs);
                this.ignorableWhitespace("\n        ");
                
                super.startElement(uri, "name", "name", emptyAttrs);
                this.characters("UP_SEQ");
                super.endElement(uri, "name", "name");
                this.ignorableWhitespace("\n        ");
                
                super.startElement(uri, "rows", "rows", emptyAttrs);
                this.ignorableWhitespace("\n            ");
                
                for (final Iterator seqItr = this.seqValues.keySet().iterator(); seqItr.hasNext();) {
                    final String seqKey = (String)seqItr.next();
                    
                    super.startElement(uri, "row", "row", emptyAttrs);
                    this.ignorableWhitespace("\n                ");
                    
                        super.startElement(uri, "column", "column", emptyAttrs);
                            super.startElement(uri, "name", "name", emptyAttrs);
                            this.characters("SEQ_NM");
                            super.endElement(uri, "name", "name");
                            
                            super.startElement(uri, "value", "value", emptyAttrs);
                            this.characters(seqKey);
                            super.endElement(uri, "value", "value");
                        super.endElement(uri, "column", "column");
                        this.ignorableWhitespace("\n                ");
                        
                        super.startElement(uri, "column", "column", emptyAttrs);
                            super.startElement(uri, "name", "name", emptyAttrs);
                            this.characters("SEQ_VL");
                            super.endElement(uri, "name", "name");
                            
                            super.startElement(uri, "value", "value", emptyAttrs);
                            this.characters(this.seqValues.get(seqKey).toString());
                            super.endElement(uri, "value", "value");
                        super.endElement(uri, "column", "column");
                        this.ignorableWhitespace("\n            ");
                    
                    super.endElement(uri, "row", "row");
                    this.ignorableWhitespace("\n        ");
                    
                    if (seqItr.hasNext())
                        this.ignorableWhitespace("    ");
                }
                
                super.endElement(uri, "rows", "rows");
                this.ignorableWhitespace("\n    ");
                
                
                super.endElement(uri, "table", "table");
                this.ignorableWhitespace("\n");
                
                super.endElement(uri, localName, qName);
            }
            else
                super.endElement(uri, localName, qName);
            
            
        }
        
        
        public void endDocument() throws SAXException {
            super.endDocument();
        }
        
        private void ignorableWhitespace(String wsp) throws SAXException {
            super.ignorableWhitespace(wsp.toCharArray(), 0, wsp.length());
        }
        
        private void characters(String chars) throws SAXException {
            super.characters(chars.toCharArray(), 0, chars.length());
        }
    }
}
