/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.tools.dbloader;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.sql.SQLException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.tools.dbloader.filters.JdbcTypeMaskFilter;
import org.jasig.portal.tools.dbloader.filters.MappingTypesFilter;
import org.jasig.portal.tools.dbloader.filters.SequenceFilter;
import org.jasig.portal.tools.dbloader.filters.XmlTypeMaskFilter;
import org.jasig.portal.tools.dbloader.handlers.DataHandler;
import org.jasig.portal.tools.dbloader.handlers.TableHandler;
import org.springframework.beans.factory.config.PropertyPlaceholderConfigurer;
import org.springframework.beans.factory.xml.XmlBeanFactory;
import org.springframework.core.io.UrlResource;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.XMLFilter;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;


/**
 * <p>A tool to set up a uPortal database. This tool was created so that uPortal
 * developers would only have to maintain a single set of xml documents to define
 * the uPortal database schema and data.  Previously it was necessary to maintain
 * different scripts for each database we wanted to support.</p>
 *
 * <p>DbLoader reads the generic types that are specified in tables.xml and
 * tries to map them to local types by querying the database metadata via methods
 * implemented by the JDBC driver.  Fallback mappings can be supplied in
 * dbloader.xml for cases where the JDBC driver is not able to determine the
 * appropriate mapping.  Such cases will be reported to standard out.</p>
 *
 * <p>An xsl transformation is used to produce the DROP TABLE and CREATE TABLE
 * SQL statements. These statements can be altered by modifying tables.xsl</p>
 *
 * <p>Generic data types (as defined in java.sql.Types) which may be specified
 * in tables.xml include:
 * <code>BIT, TINYINT, SMALLINT, INTEGER, BIGINT, FLOAT, REAL, DOUBLE,
 * NUMERIC, DECIMAL, CHAR, VARCHAR, LONGVARCHAR, DATE, TIME, TIMESTAMP,
 * BINARY, VARBINARY, LONGVARBINARY, NULL, OTHER, JAVA_OBJECT, DISTINCT,
 * STRUCT, ARRAY, BLOB, CLOB, REF, DATALINK, BOOLEAN</code>
 *
 * <p><strong>WARNING: YOU MAY WANT TO MAKE A BACKUP OF YOUR DATABASE BEFORE RUNNING DbLoader</strong></p>
 *
 * <p>DbLoader will perform the following steps:
 * <ol>
 * <li>Read configurable properties from dbloader.xml</li>
 * <li>Get database connection from RDBMServices
 *     (reads JDBC database settings from database.properties).</li>
 * <li>Read tables.xml and issue corresponding DROP TABLE and CREATE TABLE SQL statements.</li>
 * <li>Read data.xml and issue corresponding INSERT/UPDATE/DELETE SQL statements.</li>
 * </ol>
 * </p>
 *
 * @author Ken Weiner, kweiner@unicon.net
 * @author Mark Boyd, mboyd@sungardsct.com
 * @author Eric Dalquist, edalquist@unicon.net
 * @version $Revision: 11911 $
 * @see java.sql.Types
 * @since uPortal 2.0
 */
public class DbLoader {
    
    private static final String DBLOADER_CONTEXT_XML = "/properties/db/dbLoaderContext.xml";
    
    public static final String RDBM_PROPERTIES = "/database.properties";
    
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    /**
     * Uses the Spring bean config for dbLoader to get a fully configured
     * instance. The configuration options may be changed before calling
     * process.
     * 
     * @return A fully configured DbLoader.
     */
    public static DbLoader newConfiguredInstance() {
        //Load the bean factory
        final URL contextXmlUrl = DbLoaderRunner.class.getResource(DBLOADER_CONTEXT_XML);
        final XmlBeanFactory beanFactory = new XmlBeanFactory(new UrlResource(contextXmlUrl));

        //Decorate the bean factory with the rdbm configuration
        final PropertyPlaceholderConfigurer cfg = new PropertyPlaceholderConfigurer();
        final URL url = DbLoaderRunner.class.getResource(RDBM_PROPERTIES);
        cfg.setLocation(new UrlResource(url));
        try {
            cfg.postProcessBeanFactory(beanFactory);
        } catch (Exception e) {
            System.err.println("Could not locate properties in the given application context");
        }
        
        //Get the dbLoader bean
        final DbLoader loader = (DbLoader)beanFactory.getBean("dbLoader", DbLoader.class);
        
        return loader;
    }
    
    private Configuration config = null;
    private boolean processing = false;
        
    /**
     * @return Returns the config.
     */
    public synchronized Configuration getConfig() {
        if (this.config == null)
            this.config = new Configuration();
        
        return this.config;
    }
    /**
     * @param config The config to set.
     */
    public void setConfig(Configuration config) {
        if (config == null)
            throw new IllegalArgumentException("Configuration cannot be null");

        this.config = config;
    }
    
    /**
     * Takes a file path and tries to resolve it as a file system path. If that does
     * not work the class loader is used to load it as a resource. If that does
     * not work null is returned.
     * 
     * @param filePathName The file to try and get a URL to.
     * @return The URL to the file or null if one could not be resolved.
     */
    private URL resolveResource(String filePathName) {
        final File file = new File(filePathName);
        if (file.exists()) {
            try {
                return file.toURL();
            }
            catch (MalformedURLException mue) {
            }
        }
        
        try {
            final URL fileUrl = new URL(filePathName);
            return fileUrl;
        }
        catch (MalformedURLException mue) {
        }
        
        final URL fileUrl = this.getClass().getResource(filePathName);
        
        if (fileUrl == null) {
            this.logger.warn("Could not locate " + filePathName + " in file system, URL or class path.");
        }
        
        return fileUrl;
    }
    
    /**
     * Runs the DbLoader process.
     */
    public void process() throws SQLException, IOException, SAXException, 
        ParserConfigurationException, TransformerConfigurationException,
        TransformerException {
        
        synchronized (this) {
            if (processing) {
                throw new IllegalStateException("This instance of DbLoader is already running");
            }
            
            processing = true;
        }
        
        try {
            final long startTime = System.currentTimeMillis();
            this.logger.info("Starting DbLoader...");
            this.logger.debug(config.getDbInfo());

            if (config.isCreateTables() || config.isDropTables() || config.isCreateForeignKeys() || config.isDropForeignKeys()) {
                //Get table resources
                final URL tablesXmlUrl = this.resolveResource(this.config.getTablesXml());
                final URL tablesXslUrl = this.resolveResource(this.config.getTablesXsl());
                this.logger.debug("Getting tables from: " + tablesXmlUrl);
                this.logger.debug("Getting tables XSL from: " + tablesXslUrl);
    
                if (tablesXmlUrl == null)
                    throw new IllegalStateException("Could not find tables XML [" + this.config.getTablesXml() + "]");
                if (tablesXslUrl == null)
                    throw new IllegalStateException("Could not find tables XSL [" + this.config.getTablesXsl() + "]");
                
                //Get the type mapping information
                final DbInfo info = this.getConfig().getDbInfo();
                final DbTypeMapping typeMapping = this.config.getDbTypeMappings().getMappingForDb(info);
    
                if (typeMapping == null)
                    this.logger.warn("No mapping found for database with information:\n" + info);
                
                //TODO consolidate XSLT declaration code between tables & data work
                //Create the XSL source and get a Transformer for it
                final Source transSource = new StreamSource(tablesXslUrl.openStream());
                final TransformerFactory transFactory = TransformerFactory.newInstance();
                final Transformer trans = transFactory.newTransformer(transSource);
                
                
                //The XMLReader will read in the XML document
                final XMLReader reader = XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");
                
                reader.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
                
                if (this.config.isValidateXml()) {
                    try {
                        reader.setFeature("http://apache.org/xml/features/validation/dynamic", true);     
                        reader.setFeature("http://apache.org/xml/features/validation/schema", true);
                        
                        reader.setProperty("http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation", "/org/jasig/portal/tools/dbloader/tables.xsd");
                    }
                    catch (SAXNotRecognizedException snre) {
                        this.logger.warn("Could not enable XSD validation", snre);
                    }
                }
                
                //The type filter sits between the XSLT transformer and the XMLReader
                //it maps the database column types as the XML document is parsed
                final XMLFilter typeFilter = new MappingTypesFilter(reader, typeMapping);
                
                //The sources based on the SAX filters
                final InputSource xmlStreamSource = new InputSource(tablesXmlUrl.openStream());
                final Source xmlSource = new SAXSource(typeFilter, xmlStreamSource);
                
                //Use a SAX result piping the events to our TableHandler which executes the SQL
                final Result transResult = new SAXResult(new TableHandler(this.config));
                
                if (this.config.getUpgradeVersion() != null) {
                    trans.setParameter("upgradeMajor", Integer.toString(this.config.getUpgradeMajor()));
                    trans.setParameter("upgradeMinor", Integer.toString(this.config.getUpgradeMinor()));
                }
                
                //Execute the transformation, this triggers the reading of the XML
                //document which runs it through the filters, performs the XSLT and
                //sends the output events to the table handler
                trans.transform(xmlSource, transResult);
            }
            else {
                this.logger.debug("Table modifications...disabled.");
            }
            
            if (config.isPopulateTables()) {
                //Get data resources
                final URL dataXmlUrl = this.resolveResource(this.config.getDataXml());
                final URL dataXslUrl = this.resolveResource(this.config.getDataXsl());
                this.logger.debug("Getting data from: " + dataXmlUrl);
                this.logger.debug("Getting data XSL from: " + dataXslUrl);
    
                if (dataXmlUrl == null)
                    throw new IllegalStateException("Could not find data XML [" + this.config.getDataXml() + "]");
                if (dataXslUrl == null)
                    throw new IllegalStateException("Could not find data XSL [" + this.config.getDataXsl() + "]");
                
                //Create the XSL source and get a Transformer for it
                final Source transSource = new StreamSource(dataXslUrl.openStream());
                final TransformerFactory transFactory = TransformerFactory.newInstance();
                final Transformer trans = transFactory.newTransformer(transSource);
                
                //The XMLReader will read in the XML document
                final XMLReader reader = XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");
                
                if (this.config.isValidateXml()) {
                    try {
                        reader.setFeature("http://apache.org/xml/features/validation/dynamic", true);     
                        reader.setFeature("http://apache.org/xml/features/validation/schema", true);
                        
                        reader.setProperty("http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation", "/org/jasig/portal/tools/dbloader/tables.xsd");
                    }
                    catch (SAXNotRecognizedException snre) {
                        this.logger.warn("Could not enable XSD validation", snre);
                    }
                }
                
                //TODO make this injected!
                final XMLFilter typeFilter;
                if (Configuration.TYPE_MAPPING_JDBC.equals(this.config.getTypeMappingMethod())) {
                    this.logger.debug("Using JDBC for column.table type mapping...");
                    typeFilter = new JdbcTypeMaskFilter(reader, this.config);
                }
                else {
                    final URL tablesXmlUrl = this.resolveResource(this.config.getTablesXml());
                    this.logger.debug("Getting tables for column.table type mapping from: " + tablesXmlUrl);
                    typeFilter = new XmlTypeMaskFilter(reader, tablesXmlUrl);
                }
                    
                final XMLFilter seqFilter = new SequenceFilter(typeFilter, this.config);
                
                //The sources based on the SAX filters
                final InputSource xmlStreamSource = new InputSource(dataXmlUrl.openStream());
                final Source xmlSource = new SAXSource(seqFilter, xmlStreamSource);
                
                //Use a SAX result piping the events to our DataHandler which executes the SQL
                final Result transResult = new SAXResult(new DataHandler(this.config));
                
                if (this.config.getUpgradeVersion() != null) {
                    trans.setParameter("upgradeMajor", Integer.toString(this.config.getUpgradeMajor()));
                    trans.setParameter("upgradeMinor", Integer.toString(this.config.getUpgradeMinor()));
                }
                
                //Execute the transformation, this triggers the reading of the XML
                //document which runs it through the filters, performs the XSLT and
                //sends the output events to the table handler
                trans.transform(xmlSource, transResult);
            }
            else {
                this.logger.debug("Populating tables...disabled.");
            }

            this.logger.info("Done!");
            long endTime = System.currentTimeMillis();
            this.logger.info("Elapsed time: " + ((endTime - startTime) / 1000f) + " seconds");
        }
        finally {
            processing = false;
            
            if (config.getScriptWriter() != null)
                config.getScriptWriter().close();
        }
    }
}