/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.tools.dbloader;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jasig.portal.tools.dbloader.utils.DbUtils;


/**
 * Associates a list of DbInfo objects with a Map of generic column
 * types to specific column types for the databases and drivers described
 * by the DbInfo objects.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @author Dan McCallum, dmccallum@unicon.net
 * @version $Revision: 11911 $
 */
public class DbTypeMapping {
    private List databases;
    private Map typeMappings;
    private ColumnMetaDataTypeMappings columnMetaDataTypeMappings;
    private String tableExcludePattern;
    private IDmlFilter dmlFilter;
    
    public DbTypeMapping() {
    	dmlFilter = new EchoDmlFilterImpl();
    }

    /**
     * @return Returns the databases.
     */
    public List getDatabases() {
        return this.databases;
    }
    /**
     * @return Returns the typeMappings.
     */
    public Map getTypeMappings() {
        return this.typeMappings;
    }
    
    /**
     * @param databases The databases to set.
     */
    public void setDatabases(List databases) {
        this.databases = databases;
    }
    /**
     * @param typeMappings The typeMappings to set.
     */
    public void setTypeMappings(Map typeMappings) {
        final Map newMapping = new HashMap((int)(typeMappings.size() * 1.5));
        
        for (final Iterator typeItr = typeMappings.keySet().iterator(); typeItr.hasNext();) {
            final String type = (String)typeItr.next();
            newMapping.put(type.toLowerCase().trim(), typeMappings.get(type));
        }
        
        this.typeMappings = newMapping;
    }
    
    /**
     * 
     * @return the current table name exclude pattern. May be null.
     */
	public String getTableExcludePattern() {
		return tableExcludePattern;
	}
	
	/**
	 * Assign a regexp which, when matched, will cause DbLoader to skip
	 * handling of a given table. Nulls and empty strings treated as
	 * wildcards.
	 * 
	 * @param tableExcludePattern
	 */
	public void setTableExcludePattern(String tableExcludePattern) {
		this.tableExcludePattern = tableExcludePattern;
	}
	
	/**
	 * Tests if a table name exclusion pattern has been assigned.
	 * 
	 * @return true unless the assigned pattern is null or zero-length
	 */
    public boolean isTableExcludePatternAssigned() {
		return this.tableExcludePattern != null && this.tableExcludePattern.length() > 0;
	}
	
	/**
	 * Matches the given table name against the current exclude
     * pattern.
	 * 
	 * @param tableName
	 */
	public boolean isExcludedTableName(String tableName) {
		
		if ( tableName == null ) {
			throw new IllegalArgumentException("cannot test a null table name");
		}
		
		if ( !(isTableExcludePatternAssigned()) ) {
			return false;
		}
		
		return tableName.matches(this.tableExcludePattern);
		
	}
	
	/**
	 * Access the currently assigned IDmlFilter. Should not return null.
	 * 
	 * @return the currently assigned IDmlFilter.
	 */
	public IDmlFilter getDmlFilter() {
		return dmlFilter;
	}
	
	/**
	 * Assign this DbTypeMapping's IDmlFilter. If null, will
	 * cache a default instance ({@link EchoDmlFilterImpl}). 
	 * 
	 * @param dmlFilter an IDmlFilter to cache. 
	 */
	public void setDmlFilter(IDmlFilter dmlFilter) {
		this.dmlFilter = dmlFilter == null ? new EchoDmlFilterImpl() : dmlFilter;
	}

	/**
	 * Access the data type mapping overrides consulted when using
	 * database meta-data to determine types for data load statements. 
	 * 
	 * @return a direct reference to current column meta data mappings
	 */
	public ColumnMetaDataTypeMappings getColumnMetaDataTypeMappings() {
		return columnMetaDataTypeMappings;
	}

	/**
	 * Assign the data type mapping overrides consulted when using
	 * database meta-data to determine types for data load statements.
	 * 
	 * @param columnMetaDataTypeMappings a set of JDBC type mappings
	 */
	public void setColumnMetaDataTypeMappings(ColumnMetaDataTypeMappings columnMetaDataTypeMappings) {
		
		this.columnMetaDataTypeMappings = columnMetaDataTypeMappings;
		
	}
	
	/**
	 * See {@see ColumnMetaDataTypeMappings.mapColumnMetaDataType(Integer,String)}
	 * 
	 * 
	 * @param typeToMap an Integer representing a data type enumerated by java.sql.Types 
	 * @param typeToMapName the data type name reported by querying db meta-data
	 * @return the resolved type, or <code>typeToMap</code> if mapping failed.  
	 */
	public Integer mapColumnMetaDataType(Integer typeToMap, String typeToMapName) {
		
		if ( columnMetaDataTypeMappings == null ) {
			return typeToMap;
		}
		
		return columnMetaDataTypeMappings.mapColumnMetaDataType(typeToMap, typeToMapName);
		
	}
	
	
}
