package org.jasig.portal.tools.dbloader;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

/**
 * A dumb data bean representing the inputs to a prepared
 * DML statement.
 * 
 * @author Dan McCallum, dmccallum@unicon.net
 */
public class Dml {

	private String statement;
	private Object[] args;
	private int[] types;
	
	/**
	 * No-op no-arg constructor.
	 *
	 */
	public Dml() {
		//
	}
	
	/**
	 * Initialize this Dml
	 * 
	 * @param statement raw SQL
	 * @param args parameter values
	 * @param types parameter types (java.sql.Types values)
	 */
	public Dml(String statement, Object[] args, int[] types) {
		this.statement = statement;
		this.args = args;
		this.types = types;
	}
	
	/**
	 * Access the current statement's parameter value set.
	 * 
	 * @return a direct reference to the current parameter value
	 *   array
	 */
	public Object[] getArgs() {
		return args;
	}
	
	/**
	 * Assign the current statement's parameter value set
	 * 
	 * @param args an Object[] representing parameter values. Replaces
	 *   the currently assigned Object[], if any.
	 */
	public void setArgs(Object[] args) {
		this.args = args;
	}
	
	/**
	 * Access the current statement as a raw SQL string
	 * 
	 * @return a SQL string
	 */
	public String getStatement() {
		return statement;
	}
	
	/**
	 * Assign the current statement as a raw SQL string
	 * 
	 * @param statement a SQL string
	 */
	public void setStatement(String statement) {
		this.statement = statement;
	}
	
	/**
	 * Access the current statement's parameter value type set.
	 * Parameter values and types are correlated by ordinal
	 * position.
	 * 
	 * @return an array of java.sql.Types values. 
	 */
	public int[] getTypes() {
		return types;
	}
	
	/**
	 * Assign the java.sql.Types for the current parameter value
	 * set. Parameter values and types are correlated by ordinal
	 * position.
	 * 
	 * @param types an int[] representing parameter JDBC types. Replaces
	 *   the currently assigned int[], if any.
	 */
	public void setTypes(int[] types) {
		this.types = types;
	}
	
	public String toString() {
		return new ToStringBuilder(this, ToStringStyle.MULTI_LINE_STYLE)
		    .append("statement", statement)
		    .append("args", args)
		    .append("types", types)
		    .toString();
	}
	
	public int hashCode() {
		final int classCode = this.getClass().hashCode() | 1;
        final int classNameCode = this.getClass().getName().hashCode() | 1;
		
		return new HashCodeBuilder(classCode, classNameCode)
		    .append(statement)
		    .append(args)
		    .append(types)
		    .toHashCode();
	}
	
	public boolean equals(Object other) {
		
		if ( other == null ) {
			return false;
		}
		
		if ( other == this ) {
			return true;
		}
		
		if ( other.getClass() != getClass() ) {
			return false;
		}
		
		Dml otherDml = (Dml)other;
		
		return new EqualsBuilder()
		    .append(this.statement, otherDml.statement)
		    .append(this.args, otherDml.args)
		    .append(this.types, otherDml.args)
		    .isEquals();
		
	}
	
}
