package org.jasig.portal.tools.dbloader;

import java.sql.Types;

import org.springframework.jdbc.core.support.SqlLobValue;
import org.springframework.jdbc.support.lob.OracleLobHandler;

/**
 * Provides Oracle-specific Dml filtering. Currently applies special
 * handling to CLOB types and booleans encoded as English string
 * literals (true,false).
 * 
 * 
 * @author Dan McCallum, dmccallum@unicon.net
 *
 */
public class OracleDmlFilterImpl implements IDmlFilter {
	
	public static final String DEFAULT_FALSE_REGEXP = "false|FALSE";
	public static final String DEFAULT_TRUE_REGEXP = "true|TRUE";
	private OracleLobHandler lobHandler;
	private String trueRegexp = DEFAULT_TRUE_REGEXP;
	private String falseRegexp = DEFAULT_FALSE_REGEXP;
	
	/**
	 * Intercepts boolean and CLOB parameters. Replaces Strings destined
	 * for CLOB fields with SqlLobValues. (Note that we do not support
	 * BLOBs, but this shouldn't be an issue for DbLoader.) Replaces
	 * DECIMAL types that look like String-encoded booleans with String
	 * representations of 1 and 0.
	 * 
	 * @param dml the Dml to filter
	 * @return the received Dml, possibly modified
	 */
	public Dml filter(Dml dml) {
		
		int[] types = dml.getTypes();
		if ( types == null || types.length == 0 ) {
			return dml;
		}
		
		for ( int j = 0; j < types.length; j++ ) {
			if ( types[j] == Types.CLOB ) {
				Object[] args = dml.getArgs();
				args[j] = new SqlLobValue((String)args[j], lobHandler);
			} else if ( types[j] == Types.DECIMAL ) {
				Object[] args = dml.getArgs();
				String arg = (String)args[j];
				if ( arg != null ) {
					if ( arg.matches(trueRegexp) ) {
						args[j] = "1";
					} else if ( arg.matches(falseRegexp) ) {
						args[j] = "0";
					}
				}
			}
		}
		
		return dml;
		
	}

	public OracleLobHandler getLobHandler() {
		return lobHandler;
	}

	public void setLobHandler(OracleLobHandler lobHandler) {
		this.lobHandler = lobHandler;
	}

	public String getTrueRegexp() {
		return trueRegexp;
	}
	
	/**
	 * Assign the regexp used in {@link #filter(Dml)} to determine
	 * if a parameter of type java.sql.Types.DECIMAL should actually
	 * be considered a boolean true value. Note that we accept nulls and do
	 * not revert to the {@link #DEFAULT_TRUE_REGEXP} when they
	 * are received.
	 * 
	 * @param booleanRegexp a regexp used to identify boolean trues
	 *   encoded as Strings
	 */
	public void setTrueRegexp(String regexp) {
        trueRegexp = regexp;
	}
	
	public String getFalseRegexp() {
		return falseRegexp;
	}

	/**
	 * Assign the regexp used in {@link #filter(Dml)} to determine
	 * if a parameter of type java.sql.Types.DECIMAL should actually
	 * be considered a boolean false value. Note that we accept nulls and do
	 * not revert to the {@link #DEFAULT_FALSE_REGEXP} when they
	 * are received.
	 * 
	 * @param booleanRegexp a regexp used to identify boolean falses
	 *   encoded as Strings
	 */
	public void setFalseRegexp(String booleanRegexp) {
		this.falseRegexp = booleanRegexp;
	}

}
