/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.tools.dbloader.filters;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.tools.dbloader.Configuration;
import org.jasig.portal.tools.dbloader.DbTypeMapping;
import org.springframework.jdbc.support.SQLErrorCodeSQLExceptionTranslator;
import org.xml.sax.XMLReader;

/**
 * Loads the table.column type mappings using the DatabaseMetaData object.
 * 
 * For databases that support this API this is the recomended way of performing
 * type mapping as it is the most accurate. 
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @author Dan McCallum, dmccallum@unicon.net
 * @version $Revision: 11911 $
 */
public class JdbcTypeMaskFilter extends TypeMaskFilter {
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private final Map<String, Integer> tableColumnTypeMapping;

    /**
     * Create a new filter, pre-load all the mappings during construction.
     * 
     * @param reader The reader to pass events to.
     * @param configuration The configuration object to get the logger and data source from.
     * @see TypeMaskFilter#TypeMaskFilter(XMLReader)
     */
    public JdbcTypeMaskFilter(final XMLReader reader, final Configuration configuration) {
        super(reader);
       //Pre-Load all the column types, this is used when executing the prepared statements
        this.logger.debug("Loading Table.Column -> Type mappings");
        final Map<String, Integer> mappingBuilder = new HashMap<String, Integer>();     
        final DataSource ds = configuration.getDataSource();
        DbTypeMapping dbConfig = configuration.getCurrentDbTypeMapping();
        Connection conn = null;
        ResultSet columnsRs = null;
        ResultSet tablesRs = null;
        try {
        	
        	conn = ds.getConnection();
        	final DatabaseMetaData dbmd = conn.getMetaData();
            
            tablesRs = dbmd.getTables(null, null, "%", new String[] { "TABLE" });
            while (tablesRs.next()) {
                final String tableName = tablesRs.getString("TABLE_NAME");
                                            
                // TableHandler and DataHandler apply the same table name filter. Need 
                // to duplicate that work here b/c special charactes in Oracle  
                // tables can break the the getColumns() call below. Hopefully the
                // user has configured DbLoader to exclude such tables.
                if ( configuration.isExcludedTableName(tableName) ) {
                	this.logger.debug("  Skipping: " + tableName);
                	continue;
                } else {
                	this.logger.debug("  " + tableName);
                }
                                
                columnsRs = dbmd.getColumns(null, null, tableName, "%");
                while (columnsRs.next()) {
                    final String columnName = columnsRs.getString("COLUMN_NAME");
                    final int columnType = columnsRs.getInt("DATA_TYPE");
                    final String columnTypeName = columnsRs.getString("TYPE_NAME");
                    Integer mappedColumnType = 
                    	dbConfig.mapColumnMetaDataType(new Integer(columnType), columnTypeName);
                       
                    if (this.logger.isTraceEnabled()) {
                        this.logger.trace("    " + columnName + " -> " + mappedColumnType);
                    }
                    
                    final String key = this.generateKey(tableName, columnName);
                    mappingBuilder.put(key, mappedColumnType);
                }
                
                columnsRs.close();
            }
        }
        catch (final SQLException sqle) {
            final SQLErrorCodeSQLExceptionTranslator translator = new SQLErrorCodeSQLExceptionTranslator(configuration.getDataSource());
            throw translator.translate("Loading table-column type mapping.", null, sqle);
        } finally {
        	try {
        		if ( tablesRs != null ) {
        			tablesRs.close();
        		}
        	} catch ( SQLException e ) {
        		if ( this.logger.isDebugEnabled() ) {
        	        this.logger.debug("Failed to close table meta-data result set", e);
        		}
        	}
        	try {
        		if ( columnsRs != null ) {
        			columnsRs.close();
        		}
        	} catch ( SQLException e ) {
        		if ( this.logger.isDebugEnabled() ) {
        	        this.logger.debug("Failed to close column meta-data result set", e);
        		}
        	}
        	try {
        		if ( conn != null ) {
        			conn.close();
        		}
        	} catch ( SQLException e ) {
        		if ( this.logger.isDebugEnabled() ) {
        	        this.logger.debug("Failed to close db meta-data connection", e);
        		}
        	}
        	
        }
        
        this.tableColumnTypeMapping = Collections.unmodifiableMap(mappingBuilder);
    }

    /**
     * @see org.jasig.portal.tools.dbloader.filters.TypeMaskFilter#getType(java.lang.String, java.lang.String)
     */
    protected int getType(String tableName, String columnName) {
        final String key = this.generateKey(tableName, columnName);
        final Integer type = tableColumnTypeMapping.get(key);
        
        if (type != null)
            return type.intValue();
        else
            return Types.NULL;
    }

    /**
     * Generates the mapping key for a table and column name.
     * 
     * @param tableName The name of the table to generate a key from.
     * @param columnName The name of the column to generate a key from.
     * @return The key for the table and column name.
     */
    private String generateKey(String tableName, String columnName) {
        return tableName.toUpperCase().trim() + "." + columnName.toUpperCase().trim();
    }
    
}