/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.tools.dbloader.filters;

import java.util.Map;

import org.jasig.portal.tools.dbloader.DbTypeMapping;
import org.jasig.portal.utils.XmlLocationStack;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLFilterImpl;


/**
 * XMLFilter that maps column types for a database using a DbTypeMapping
 * object.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class MappingTypesFilter extends XMLFilterImpl {
    private static final String COLUMN_TYPE_LOC = "/tables/table/columns/column/type";
    
    private final XmlLocationStack location = new XmlLocationStack();
    private final Map typeMapping;

    public StringBuffer chars = null;
    
    /**
     * Creates a new filter, specifies the reader to pass on events too and the
     * DbTypeMapping to try to map column types with.
     * 
     * @param parent The XMLReader to pass events on to.
     * @param mapping The object to get the type mappings from, if null no mapping will be done.
     */
    public MappingTypesFilter(final XMLReader parent, final DbTypeMapping mapping) {
        super(parent);
        
        if (mapping == null)
            this.typeMapping = null;
        else
            this.typeMapping = mapping.getTypeMappings();
    }

    /**
     * Tracks when a "type" element starts, resets the character buffer
     * 
     * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
        final String locationPath = this.location.startElement(qName);
        
        if (this.typeMapping != null && COLUMN_TYPE_LOC.equalsIgnoreCase(locationPath)) {
            this.chars = new StringBuffer(256);
        }

        super.startElement(uri, localName, qName, atts);
    }

    /**
     * If in a "type" element buffer the character data.
     * 
     * @see org.xml.sax.ContentHandler#characters(char[], int, int)
     */
    public void characters(char[] ch, int start, int length) throws SAXException {
        if (this.typeMapping != null && this.chars != null)
            this.chars.append(ch, start, length);
        else
            super.characters(ch, start, length);
    }

    /**
     * If ending a "type" element try to map the value stored in the
     * character buffer. Write out the appropriate character data
     * 
     * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
     */
    public void endElement(String uri, String localName, String qName) throws SAXException {
        final String locationPath = this.location.endElement(qName);
        
        if (this.typeMapping != null && COLUMN_TYPE_LOC.equalsIgnoreCase(locationPath)) {
            final String type = this.chars.toString().trim();
            final String mappedType = (String)this.typeMapping.get(type.toLowerCase());
            
            //If a mapping was found, send the mapped type out for the character data
            if (mappedType != null)
                super.characters(mappedType.toCharArray(), 0, mappedType.length());
            //If no mapping was found, send the original type
            else
                super.characters(type.toCharArray(), 0, type.length());

            this.chars = null;
        }

        super.endElement(uri, localName, qName);
    }
}
