/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.tools.dbloader.filters;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Pattern;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.portlet.dao.IncrementerImpl;
import org.jasig.portal.tools.dbloader.Configuration;
import org.jasig.portal.utils.IIncrementer;
import org.jasig.portal.utils.XmlLocationStack;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.XMLFilterImpl;

/**
 * This filter replaces the sequence tags in the incoming data xml document with
 * values generated by the uPortal IncrementerImpl class.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class SequenceFilter extends XMLFilterImpl {
    private static final String SEQUENCE_LOC        = "/data/table/rows/row/.*sequence";
    private static final String VALUE_ELEMENT_NAME  = "value";
    private static final String ID_ATTRIBUTE_NAME   = "id";
    
    private final XmlLocationStack location = new XmlLocationStack();
    private final Map<String, String> namedSeqValues = new HashMap<String, String>(); 
    private final Configuration config;
    private final IIncrementer incrementer;
    
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private StringBuffer chars = null;
    private String seqName = null;
    
    /**
     * Creates a new filter, specifies the reader to pass on events too and the
     * configuration to use.
     * 
     * @param parent The XMLReader to pass events on to.
     * @param configuration The configuration to get setting for the filter from.
     */
    public SequenceFilter(final XMLReader parent, final Configuration configuration) {
        super(parent);
        
        this.config = configuration;
        
        this.incrementer = new IncrementerImpl();
        this.incrementer.setDataSource(this.config.getDataSource());
    }
    
    /**
     * @see org.xml.sax.ContentHandler#startDocument()
     */
    public void startDocument() throws SAXException {
        this.logger.debug("Converting sequence elements to values...");
        super.startDocument();
    }

    /**
     * For sequence elements the chars buffer is created, the sequence name
     * is pulled fromt the id attribute and a value element is started.
     * 
     * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
        final String locationPath = this.location.startElement(qName);
        if (Pattern.matches(SEQUENCE_LOC,locationPath.toLowerCase())) {
            this.chars = new StringBuffer(256);
            this.seqName = atts.getValue(ID_ATTRIBUTE_NAME);
            super.startElement(uri, VALUE_ELEMENT_NAME, VALUE_ELEMENT_NAME, new AttributesImpl());
        }
        else
            super.startElement(uri, localName, qName, atts);
    }
    
    /**
     * If in a sequence element buffer the character data.
     * 
     * @see org.xml.sax.ContentHandler#characters(char[], int, int)
     */
    public void characters(char[] ch, int start, int length) throws SAXException {
        if (this.chars != null)
            this.chars.append(ch, start, length);
        else
            super.characters(ch, start, length);
    }

    /**
     * If ending a sequnce element the value key is computed and the cache is
     * consulted to see if a value for that sequence key exists. If it does it
     * is used if not a new sequence number is generated and stored in the cache.
     * 
     * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
     */
    public void endElement(String uri, String localName, String qName) throws SAXException {
        final String locationPath = this.location.endElement(qName);
        
        if (Pattern.matches(SEQUENCE_LOC,locationPath.toLowerCase())) {
            final String seqId = this.chars.toString().trim();
            final String valueKey = this.seqName + "." + seqId;
            
            String value = namedSeqValues.get(valueKey);
            if (value == null) {
                this.logger.debug("  SEQ: " + this.seqName);

                synchronized (this.incrementer) {
                    this.incrementer.setIncrementerName(this.seqName);
                    
                    if (!this.incrementer.existsInDb())
                        this.incrementer.createIncrementer(0);
                    
                    value = Long.toString(this.incrementer.getNextKey());
                }
                
                namedSeqValues.put(valueKey, value);
            }
            
            super.characters(value.toCharArray(), 0, value.length());
            super.endElement(uri, VALUE_ELEMENT_NAME, VALUE_ELEMENT_NAME);
            
            this.chars = null;
        }
        else
            super.endElement(uri, localName, qName);
    }
}