/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.tools.dbloader.filters;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.utils.XmlLocationStack;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.XMLFilterImpl;


/**
 * Adds type elements to the column elements of the data XML. These hints are
 * used by the DataHandler to create the appropriate prepared statements for the
 * inserts.
 * 
 * The implementing class must implement the getType method which returns a value
 * from the java.sql.Types constants class for a table and column.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public abstract class TypeMaskFilter extends XMLFilterImpl {
    private static final String TABLE_LOC           = "/data/table";
    private static final String TABLE_NAME_LOC      = "/data/table/name";
    private static final String COLUMN_LOC          = "/data/table/rows/row/column";
    private static final String COLUMN_NAME_LOC     = "/data/table/rows/row/column/name";
    private static final String TYPE_ELEMENT_NAME   = "type";
    
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private final XmlLocationStack location = new XmlLocationStack();
    
    private StringBuffer chars = null;
    private String tableName = null;
    private String columnName = null;
    
    /**
     * Creates a new filter, usnig the parent reader to pass events to.
     * 
     * @param parent The filter to pass events to.
     * @param logger The logger to use.
     */
    public TypeMaskFilter(final XMLReader parent) {
        super(parent);
    }
    
    /**
     * Takes a table name and column name and returns the java.sql.Types type
     * for the column.
     * 
     * @param tableName The name of the table that the column is in.
     * @param columnName The name of the column.
     * @return The java type of the column.
     */
    protected abstract int getType(String tableName, String columnName);

    /**
     * @see org.xml.sax.ContentHandler#startDocument()
     */
    public void startDocument() throws SAXException {
        this.logger.debug("Adding type elements to XML...");
        super.startDocument();
    }
    
    /**
     * Tracks the location within the XML document. Flags table, column and name
     * elements to gather the needed information.
     * 
     * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
        final String locationPath = this.location.startElement(qName);
        
        if (TABLE_NAME_LOC.equalsIgnoreCase(locationPath) || COLUMN_NAME_LOC.equalsIgnoreCase(locationPath)) {
            this.chars = new StringBuffer(1024);
        }

        super.startElement(uri, localName, qName, atts);
    }

    /**
     * Tracks the character data if the buffer isn't null.
     * 
     * @see org.xml.sax.ContentHandler#characters(char[], int, int)
     */
    public void characters(char[] ch, int start, int length) throws SAXException {
        if (this.chars != null)
            this.chars.append(ch, start, length);
        
        super.characters(ch, start, length);
    }

    /**
     * Handles the ending of elements. When a column element ends a type element
     * is inserted before the end tag with the appropriate value.
     * 
     * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
     */
    public void endElement(String uri, String localName, String qName) throws SAXException {
        final String locationPath = this.location.endElement(qName);
        
        if (TABLE_LOC.equalsIgnoreCase(locationPath)) {
            this.tableName = null;
        }
        else if (COLUMN_LOC.equalsIgnoreCase(locationPath)) {
            //Get the mapped type.
            final int type = this.getType(this.tableName, this.columnName);
            final String typeStr = Integer.toString(type);

            //Insert the type element
            super.startElement(uri, TYPE_ELEMENT_NAME, TYPE_ELEMENT_NAME, new AttributesImpl());
            super.characters(typeStr.toCharArray(), 0, typeStr.length());
            super.endElement(uri, TYPE_ELEMENT_NAME, TYPE_ELEMENT_NAME);
            
            this.columnName = null;
        }
        else if (COLUMN_NAME_LOC.equalsIgnoreCase(locationPath)) {
            this.columnName = this.chars.toString().toUpperCase();
        }
        else if (TABLE_NAME_LOC.equalsIgnoreCase(locationPath)) {
            this.tableName = this.chars.toString().toUpperCase();
        }
        
        super.endElement(uri, localName, qName);
    }
}
