/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.tools.dbloader.filters;

import java.io.IOException;
import java.net.URL;
import java.sql.Types;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.tools.dbloader.utils.DbUtils;
import org.jasig.portal.utils.XmlLocationStack;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;


/**
 * Reads the tables XML and creates the table.column type mapping.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class XmlTypeMaskFilter extends TypeMaskFilter {
    private final Map<String, Integer> tableColumnTypeMapping;
    
    public XmlTypeMaskFilter(XMLReader parent, URL dataXmlUrl) throws SAXException, IOException {
        super(parent);
        
        final XMLReader reader = XMLReaderFactory.createXMLReader();
        final XmlTableColumnTypeMappingHandler mappingHandler = new XmlTableColumnTypeMappingHandler();
        reader.setContentHandler(mappingHandler);
        reader.parse(new InputSource(dataXmlUrl.openStream()));

        this.tableColumnTypeMapping = Collections.unmodifiableMap(mappingHandler.getMappings());
    }

    /**
     * @see org.jasig.portal.tools.dbloader.filters.TypeMaskFilter#getType(java.lang.String, java.lang.String)
     */
    protected int getType(String tableName, String columnName) {
        final String key = this.generateKey(tableName, columnName);
        final Integer type = tableColumnTypeMapping.get(key);
        
        if (type != null)
            return type.intValue();
        else
            return Types.NULL;
    }

    /**
     * Generates the mapping key for a table and column name.
     * 
     * @param tableName The name of the table to generate a key from.
     * @param columnName The name of the column to generate a key from.
     * @return The key for the table and column name.
     */
    private String generateKey(String tableName, String columnName) {
        return tableName.toUpperCase().trim() + "." + columnName.toUpperCase().trim();
    }
    
    /**
     * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
     * @version $Revision: 11911 $
     */
    private class XmlTableColumnTypeMappingHandler implements ContentHandler {
        protected final Log logger = LogFactory.getLog(this.getClass());
        
        private static final String TABLE_LOC       = "/tables/table";
        private static final String TABLE_NAME_LOC  = "/tables/table/name";
        private static final String COLUMN_LOC      = "/tables/table/columns/column";
        private static final String COLUMN_NAME_LOC = "/tables/table/columns/column/name";
        private static final String COLUMN_TYPE_LOC = "/tables/table/columns/column/type";
        
        private final Map<String, Integer> tableColumnTypeMappings = new HashMap<String, Integer>();
        private final XmlLocationStack location = new XmlLocationStack();
        
        private StringBuffer chars = null;
        private String tableName = null;
        private String columnName = null;
        private String columnType = null;
        
        public Map<String, Integer> getMappings() {
            return this.tableColumnTypeMappings;
        }

        /**
         * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
         */
        public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
            final String locationPath = this.location.startElement(qName);
            
            try {
                if (TABLE_NAME_LOC.equalsIgnoreCase(locationPath) || 
                        COLUMN_NAME_LOC.equalsIgnoreCase(locationPath) ||
                        COLUMN_TYPE_LOC.equalsIgnoreCase(locationPath)) {
                    this.chars = new StringBuffer(1024);
                }
            }
            catch (RuntimeException e) {
                this.logger.error("Error during startElement", e);
                throw e;
            }
        }

        /**
         * @see org.xml.sax.ContentHandler#characters(char[], int, int)
         */
        public void characters(char[] ch, int start, int length) throws SAXException {
            try {
                if (this.chars != null)
                    this.chars.append(ch, start, length);
            }
            catch (RuntimeException e) {
                this.logger.error("Error during characters", e);
                throw e;
            }
        }
        
        /**
         * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
         */
        public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
            try {
                final String locationPath = this.location.endElement(qName);
                
                if (TABLE_LOC.equalsIgnoreCase(locationPath)) {
                    this.tableName = null;
                }
                else if (COLUMN_LOC.equalsIgnoreCase(locationPath)) {
                    final String key = generateKey(this.tableName, this.columnName);
                    
                    final int type = DbUtils.getJavaSqlType(this.columnType);
                    this.tableColumnTypeMappings.put(key, new Integer(type));

                    this.columnName = null;
                    this.columnType = null;
                }
                else if (TABLE_NAME_LOC.equalsIgnoreCase(locationPath)) {
                    this.tableName = this.chars.toString().toUpperCase();
                }
                else if (COLUMN_NAME_LOC.equalsIgnoreCase(locationPath)) {
                    this.columnName = this.chars.toString().toUpperCase();
                }
                else if (COLUMN_TYPE_LOC.equalsIgnoreCase(locationPath)) {
                    this.columnType = this.chars.toString().toUpperCase();
                }
            }
            catch (RuntimeException e) {
                this.logger.error("Error during endElement", e);
                throw e;
            }
        }

        
        
        //NOOP Methods
        public void endDocument() throws SAXException {
        }
        public void startDocument() throws SAXException {
        }
        public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
        }
        public void endPrefixMapping(String prefix) throws SAXException {
        }
        public void skippedEntity(String name) throws SAXException {
        }
        public void setDocumentLocator(Locator locator) {
        }
        public void processingInstruction(String target, String data) throws SAXException {
        }
        public void startPrefixMapping(String prefix, String uri) throws SAXException {
        }
    }
}
