/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.tools.dbloader.handlers;

import java.util.LinkedList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.tools.dbloader.Configuration;
import org.jasig.portal.tools.dbloader.utils.DbUtils;
import org.jasig.portal.utils.XmlLocationStack;
import org.springframework.dao.DataAccessException;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * Handles SAX events resulting from parsing of the data.xml file and executes
 * the SQL contained in the XML.
 *
 * @author Ken Weiner, kweiner@unicon.net
 * @author Mark Boyd <mark.boyd@engineer.com>
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @author Dan McCallum, dmccallum@unicon.net
 * @version $Revision: 11911 $
 */
public class DataHandler implements ContentHandler {
    private static final String STATEMENT_LOC   = "/statements/statement";
    private static final String SQL_LOC         = "/statements/statement/sql";
    private static final String PARAMETERS_LOC  = "/statements/statement/parameters";
    private static final String PARAMETER_LOC   = "/statements/statement/parameters/parameter";
    private static final String NAME_ATTR       = "name";
    private static final String TYPE_ATTR       = "type";
    
    private final XmlLocationStack location = new XmlLocationStack();
    private final Configuration config;
    
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private StringBuffer chars = null;
    
    private String tableName = null;
    private String sql = null;
    private List<Integer> types = null;
    private List<String> values = null;
    
    /**
     * @param configuration The Configuration object to use for the handler.
     */
    public DataHandler(final Configuration configuration) {
        this.config = configuration;
    }
    
    /**
     * Log the start of the processing.
     * 
     * @see org.xml.sax.ContentHandler#startDocument()
     */
    public void startDocument() throws SAXException {
        this.logger.info("Populating tables...");
    }
    
    /**
     * Tracks the start of the statements and other elements
     * 
     * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
        final String locationPath = this.location.startElement(qName);
        
        if (STATEMENT_LOC.equalsIgnoreCase(locationPath)) {
            this.tableName = atts.getValue(NAME_ATTR);
        }
        else if (SQL_LOC.equalsIgnoreCase(locationPath)) {
            this.chars = new StringBuffer(1024);
        }
        else if (PARAMETERS_LOC.equalsIgnoreCase(locationPath)) {
            this.types = new LinkedList<Integer>();
            this.values = new LinkedList<String>();
        }
        else if (PARAMETER_LOC.equalsIgnoreCase(locationPath)) {
            this.chars = new StringBuffer(1024);
            
            final String type = atts.getValue(TYPE_ATTR);
            this.types.add(new Integer(type));
        }
        
    }

    /**
     * Buffer character data if the buffer isn't null.
     * 
     * @see org.xml.sax.ContentHandler#characters(char[], int, int)
     */
    public void characters(char[] ch, int start, int length) throws SAXException {
        if (chars != null)
            chars.append(ch, start, length);
    }
    
    /**
     * At the end of the statement element all the info should be avaiable
     * to execute the SQL
     * 
     * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
     */
    public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
        final String locationPath = this.location.endElement(qName);
       
        if (STATEMENT_LOC.equalsIgnoreCase(locationPath)) {
        	
        	if ( this.config.isExcludedTableName(this.tableName) ) {
        		
        		if (this.logger.isTraceEnabled()) {
                    this.logger.trace("  Skipping statement for excluded table [" + this.tableName + "] : " + this.sql + " " + this.values + " " + this.types);
        		} else if (this.logger.isDebugEnabled()) {
                    this.logger.debug("  Skipping statement for excluded table [" + this.tableName + "]");
        		}
        		
        	} else {
        	
        		final int[] types = new int[this.types.size()];
        		int index = 0;
                for (final Integer typeInt : this.types) {
        			types[index++] = typeInt.intValue();
        		}

        		if (this.logger.isTraceEnabled())
        			this.logger.trace("  " + this.tableName + ": " + this.sql + " " + this.values + " " + this.types);
        		else if (this.logger.isDebugEnabled())
        			this.logger.debug("  " + this.tableName);

        		try {
        			DbUtils.executePreparedStatement(this.config, this.sql, this.values.toArray(), types);
        		}
        		catch (DataAccessException dae) {
        			if (this.config.isRecoverExceptions())
        				this.logger.warn("Error executing SQL [" + this.sql + "] with values " + this.values + " of type " + this.types, dae);
        			else
        				throw dae;
        		}
        	}
            
            this.tableName = null;
            this.sql = null;
            this.values = null;
            this.types = null;
        }
        else if (SQL_LOC.equalsIgnoreCase(locationPath)) {
            this.sql = this.chars.toString();
        }
        else if (PARAMETER_LOC.equalsIgnoreCase(locationPath)) {
            this.values.add(this.chars.toString());
        }
    }
    

    // NOOP Methods
    public void endDocument() throws SAXException {
    }
    public void endPrefixMapping(String prefix) throws SAXException {
    }
    public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
    }
    public void processingInstruction(String target, String data) throws SAXException {
    }
    public void setDocumentLocator(Locator locator) {
    }
    public void skippedEntity(String name) throws SAXException {
    }
    public void startPrefixMapping(String prefix, String uri) throws SAXException {
    }
}
