/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.tools.dbloader.handlers;


import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.tools.dbloader.Configuration;
import org.jasig.portal.tools.dbloader.utils.DbUtils;
import org.springframework.dao.DataAccessException;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * Class for handling SAX events during parsing of tables.xml.
 *
 * @author Ken Weiner, kweiner@unicon.net
 * @author Mark Boyd <mark.boyd@engineer.com>
 * @author Dan McCallum, dmccallum@unicon.net
 * @version $Revision: 11911 $
 */
public class TableHandler implements ContentHandler {

    private Configuration config = null;

    private static final String FK_DROP    = "fkDrop";
    private static final String TBL_DROP   = "tblDrop";
    private static final String TBL_CREATE = "tblCreate";
    private static final String IDX_CREATE = "idxCreate";
    private static final String FK_CREATE  = "fkCreate";
    
    private static final String STATEMENT_ELEMENT   = "statement";
    private static final String TYPE_ATTR           = "type";
    private static final String NAME_ATTR           = "name";
    
    protected final Log logger = LogFactory.getLog(this.getClass());

    private String lastStmtType = null;

    private StringBuffer stmtBuffer;
    private String tableName = null;

    /**
     * Initialises the table handler using the specified Configuration.
     * 
     * @param config The Configuration to use.
     */
    public TableHandler(Configuration config) {
        this.config = config;
    }

    /**
     * If for a statement element the statement buffer is created
     * and appropriate message logged.
     *  
     * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts) {
        if (STATEMENT_ELEMENT.equalsIgnoreCase(qName)) {
            final String statementType = atts.getValue(TYPE_ATTR);
            this.tableName = atts.getValue(NAME_ATTR);
            
            if (!statementType.equals(this.lastStmtType)) {
                final StringBuffer message = new StringBuffer(64);
                
                if (FK_DROP.equals(statementType)) {
                    message.append("Dropping foreign keys...");
                    
                    if (config.isDropForeignKeys() || config.isDropTables()) {
                        this.stmtBuffer = new StringBuffer(1024);
                    }
                    else {
                        message.append("disabled");
                        this.stmtBuffer = null;
                    }
                }
                else if (TBL_DROP.equals(statementType)) {
                    message.append("Dropping tables...");
                    
                    if (config.isDropTables()) {
                        this.stmtBuffer = new StringBuffer(1024);
                    }
                    else {
                        message.append("disabled.");
                        this.stmtBuffer = null;
                    }
                }
                else if (TBL_CREATE.equals(statementType)) {
                    message.append("Creating tables...");
                    
                    if (config.isCreateTables()) {
                        this.stmtBuffer = new StringBuffer(1024);
                    }
                    else {
                        message.append("disabled.");
                        this.stmtBuffer = null;
                    }
                }
                else if (IDX_CREATE.equals(statementType)) {
                    message.append("Creating indicies...");
                    
                    if (config.isCreateTables()) {
                        this.stmtBuffer = new StringBuffer(1024);
                    }
                    else {
                        message.append("disabled.");
                        this.stmtBuffer = null;
                    }
                }
                else if (FK_CREATE.equals(statementType)) {
                    message.append("Creating foreign keys...");
                    
                    if (config.isCreateForeignKeys()) {
                        this.stmtBuffer = new StringBuffer(1024);
                    }
                    else {
                        message.append("disabled.");
                        this.stmtBuffer = null;
                    }
                }
                else {
                    message.append("Unknown statement type: " + statementType);
                }

                this.logger.info(message);
                this.lastStmtType = statementType;
            }
            else {
                if (this.stmtBuffer != null)
                    this.stmtBuffer = new StringBuffer(1024);
            }
        }
    }

    /**
     * If the buffer exists write the character data to it.
     * 
     * @see org.xml.sax.ContentHandler#characters(char[], int, int)
     */
    public void characters(char ch[], int start, int length) {
        if (this.stmtBuffer != null)
            this.stmtBuffer.append(ch, start, length);
    }

    /**
     * If ending a statement element dump the buffer to a string a execute
     * the SQL. Returns without executing the SQL if the current table name
     * matches an exclude pattern in the current Configuration.
     * 
     * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
     */
    public void endElement(String namespaceURI, String localName, String qName) {
        if (STATEMENT_ELEMENT.equalsIgnoreCase(qName) && this.stmtBuffer != null) {
        	
        	final String sqlStatement = this.stmtBuffer.toString();
        	
        	if ( this.config.isExcludedTableName(this.tableName) ) {
        		
        		if (this.logger.isTraceEnabled()) {
                    this.logger.trace("  Skipping statement for excluded table [" + this.tableName + "]: " + sqlStatement);
        		} else if (this.logger.isDebugEnabled()) {
                    this.logger.debug("  Skipping statement for excluded table [" + this.tableName + "]");
        		}
        		
            } else {
            
            	if (this.logger.isTraceEnabled())
            		this.logger.trace("  " + this.tableName + ": " + sqlStatement);
            	else if (this.logger.isDebugEnabled())
            		this.logger.debug("  " + this.tableName);

            	try {
            		DbUtils.executeUpdate(this.config, sqlStatement);
            	}
            	catch (DataAccessException dae) {
            		if (FK_DROP.equals(lastStmtType) || TBL_DROP.equals(lastStmtType)) {
            			this.logger.debug("Error executing SQL [" + sqlStatement + "]", dae);
            		}
            		else if (this.config.isRecoverExceptions()) {
            			this.logger.warn("Error executing SQL [" + sqlStatement + "]", dae);
            		}
            		else {
            			throw dae;
            		}
            	}
            }
        }
    }

    //NOOP methods
    public void startDocument() {
    }

    public void endDocument() {
    }

    public void setDocumentLocator(Locator locator) {
    }

    public void processingInstruction(String target, String data) {
    }

    public void ignorableWhitespace(char[] ch, int start, int length) {
    }

    public void startPrefixMapping(String prefix, String uri) throws SAXException {
    }

    public void endPrefixMapping(String prefix) throws SAXException {
    }

    public void skippedEntity(String name) throws SAXException {
    }
}