/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.tools.dbloader.utils;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.tools.dbloader.Configuration;
import org.jasig.portal.tools.dbloader.DbInfo;
import org.springframework.jdbc.core.JdbcTemplate;

/**
 * Utility class for centralizing various functions performed on the database.
 *
 * @author Ken Weiner, kweiner@unicon.net
 * @author Mark Boyd <mark.boyd@engineer.com>
 * @version $Revision: 11911 $
 */
public class DbUtils {
    protected static final Log LOG = LogFactory.getLog(DbUtils.class);

    /**
     * Takes a DataSource and attempts to load the
     * meta information about the remote DB.
     * 
     * @param ds The DataSource to load the meta data for.
     * @param logger Used to log any messages
     * @return A DbInfo object representing the DataSource or null if there was a problem
     */
    public static DbInfo getDatabaseInfo(DataSource ds) {
        try {
            final Connection conn = ds.getConnection();
            final DatabaseMetaData dbmd = conn.getMetaData();
            
            final DbInfo info = new DbInfo();
            info.setDbName(dbmd.getDatabaseProductName());
            info.setDbVersion(dbmd.getDatabaseProductVersion());
            info.setDriverName(dbmd.getDriverName());
            info.setDriverVersion(dbmd.getDriverVersion());
            info.setJdbcUrl(dbmd.getURL());
            info.setJdbcUser(dbmd.getUserName());
            
            return info;
        }
        catch (SQLException sqle) {
            LOG.error("Error creating DbInfo object", sqle);
            return null;
        }
    }

    /**
     * Uses the Spring JdbcTemplate class to execute add-hoc SQL statements. If
     * script creation is enabled the statement will be logged to the script.
     * 
     * @param config Configuration object to get the script, template and data source from.
     * @param statement The SQL to execute
     */
    public static void executeUpdate(final Configuration config, final String statement) {
        if (config.isCreateScript()) {
            config.getScriptWriter().println(statement + config.getSqlStatementTerminator());
            config.getScriptWriter().flush();
        }

        synchronized (config) {
            if (config.getTemplate() == null) {
                config.setTemplate(new JdbcTemplate(config.getDataSource()));
            }
        }
        
        config.getTemplate().execute(statement);
    }
    
    /**
     * Compiles and executes a prepared statement. If script creation is
     * enabled the statement will be logged to the script.
     *  
     * @param config Configuration object to get the script, template and data source from.
     * @param statement The SQL in prepared statement format.
     * @param args The argument values.
     * @param types The argument types as defined in java.sql.Types
     */
    public static void executePreparedStatement(final Configuration config, final String statement, final Object[] args, final int[] types) {
        if (config.isCreateScript()) {
            final StringBuffer fullStatement = new StringBuffer(statement);
            
            int strIndex = fullStatement.indexOf("?");
            int argIndex = 0;
            while (strIndex >= 0) {
                String argValue = args[argIndex].toString();
                if (types[argIndex] != Types.INTEGER && types[argIndex] != Types.DECIMAL) {
                    argValue = "'" + argValue + "'";
                }
                    
                fullStatement.replace(strIndex, strIndex + 1, argValue);
                argIndex++;
                strIndex = fullStatement.indexOf("?");
            }
            
            config.getScriptWriter().println(fullStatement + config.getSqlStatementTerminator());
            config.getScriptWriter().flush();
        }

        synchronized (config) {
            if (config.getTemplate() == null) {
                config.setTemplate(new JdbcTemplate(config.getDataSource()));
            }
        }
        
        // config potentially has knowledge of certain type handling
        // gymnastics, e.g. Oracle LOBs, so statement execution is 
        // delegated to it.
        config.execUpdate(statement, args, types);
        
    }
    
    //Uses reflection to create a map of Types names to values
    private static final Map<String, Integer> typeMappings;
    static {
        final Map<String, Integer> mappingBuilder = new HashMap<String, Integer>();
        
        try {
            final Field[] fields = Types.class.getFields();
            for (int index = 0; index < fields.length; index++) {
                if (Modifier.isPublic(fields[index].getModifiers())) {
                    final Class fType = fields[index].getType();
                    
                    if (Integer.TYPE.equals(fType)) {
                        final int value = fields[index].getInt(null);
                        mappingBuilder.put(fields[index].getName().toUpperCase(), new Integer(value));
                    }   
                }
            }
        }
        catch (Exception e) {
            System.err.println("Error while loading java.sql.Types mapping");
            e.printStackTrace(System.err);
        }
        
        typeMappings = Collections.unmodifiableMap(mappingBuilder);
    }
    
    
    
    /**
     * Map string representation of the java.sql.Types value to the actual
     * int value.
     * 
     * @param genericDataTypeName
     * @return The int value associated with the string name.
     */
    public static int getJavaSqlType(String genericDataTypeName) {
        final Integer value = typeMappings.get(genericDataTypeName);
        
        if (value != null)
            return value.intValue();
        else
            return Types.NULL;
    }
}