/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.url;

import java.io.Serializable;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheable;

/**
 * Prepends "/servletName" to the constructed URLs (instead of relying on relative URLs).
 * This decorator is necessary if path will be used to carry information other than servlet name.
 * Additional N currentPathElements (from the servlet path) can be appended by setting "additionalElements" property to N (not by done by default).
 * 
 * The implementation also has to be placed on a list of request parameter processors (to listen to the current servlet path)
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class AbsoluteUrlDecorator implements IRequestParameterProcessor, IUrlDecorator, ICacheable {
    int additionalElements=0;
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(AbsoluteUrlDecorator.class);
    private String currentPathElements=null;
    

    public void decorateUrl(IPortalUrl url) {
        if(currentPathElements!=null) {
            IServletPathExtender constructor=(IServletPathExtender) url.getUrlConstructor(IServletPathExtender.class.getName());

            if (logger.isDebugEnabled()) {
                logger.debug("decorateUrl(IPortalUrl) - setting servlet path="+currentPathElements);
            }

            constructor.addPathElement(currentPathElements);
        }
            
           
    }

    public boolean canEvaluateVariables() {
        return false;
    }

    public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res) {
        String currentContextPath=req.getContextPath();
        String currentServletPath=req.getServletPath();
        
        StringBuffer sb=new StringBuffer(currentContextPath);
        
        // append additional currentPathElements, if specified
        if(additionalElements>0 && currentServletPath!=null) {
            String cleanPath=currentServletPath;
            if(cleanPath.startsWith("/")) { cleanPath=cleanPath.substring(1); }
            String[] pathElements=cleanPath.split("/");
            if(pathElements.length>1) {
                int nElements=additionalElements;
                if(nElements > pathElements.length-1) {
                    nElements=pathElements.length-1;
                    if (logger.isDebugEnabled()) {
                        logger.debug("processParameters(IWritableHttpServletRequest,HttpServletResponse) - fewer path currentPathElements found in the sevletPath than specified. servletPath="+currentServletPath+", nElements="+nElements);
                    }
                }
                for(int i=0;i<nElements;i++) {
                    if (logger.isDebugEnabled()) {
                        logger.debug("processParameters(IWritableHttpServletRequest,HttpServletResponse) - adding path element="+pathElements[i]);
                    }

                    sb.append("/").append(pathElements[i]);
                }
            } else {
                if (logger.isDebugEnabled()) {
                    logger.debug("processParameters(IWritableHttpServletRequest,HttpServletResponse) - too few path currentPathElements found in the sevletPath. servletPath="+currentServletPath);
                }
            }
        }
        currentPathElements=sb.toString();
        return true;
    }

    /**
     * Specify number of path currentPathElements from servletPath to be appended
     * @param additionalElements number of path currentPathElements from servletPath to be appended, defaults to 0.
     */
    public void setAdditionalElements(int additionalElements) {
        this.additionalElements = additionalElements;
    }

    public ICacheKey generateKey() {
        return new CacheKeyImpl(currentPathElements);
    }

    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }

}
