/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.url;

import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.core.exception.PortalRuntimeException;

/**
 * UrlInfo is a utility class facilitating construction of request URL with structured path, file and parameter elements.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11918 $
 */
public class UrlInfo {
	
	private List pathElements=new ArrayList(1);
	private List fileElements=new ArrayList(1);
	private Map parameters=new Hashtable();
    private String host=null;
    private String port=null;
    private String protocol=null;
    private String fragment=null;
	
	public final static String PATH_SEPARATOR="/";
	public final static String PARAMETER_START="?";
	public final static String PARAMETER_SEPARATOR="&";
	public final static String PARAMETER_ASSIGNMENT="=";
	public final static String DEFAULT_FILE_SEPARATOR=".";
	public final static String DEFAULT_FILE_SUFFIX="";
    public final static String PORT_SERPARATOR=":";
    public final static String FRAGMENT_SERPARATOR="#";
    public final static String PROTOCOL_SERPARATOR="://";
    
    public final static String HTTPS_PROTOCOL="https";
    public final static String HTTP_PROTOCOL="http";
	
	private String fileSeparator=DEFAULT_FILE_SEPARATOR;
	private String fileSuffix=DEFAULT_FILE_SUFFIX;
	
	
	/**
	 * @return Returns suffix of the file element
	 * <code>null</code> implies that no suffix will be printed
	 */
	public String getFileSuffix() {
		return fileSuffix;
	}
	/**
	 * @param fileSuffix the suffix of the file element
	 * <code>null</code> implies that no suffix will be printed
	 */
	public void setFileSuffix(String fileSuffix) {
		this.fileSuffix = fileSuffix;
	}
	/**
	 * @return Returns the file separator string.
	 */
	public String getFileSeparator() {
		return fileSeparator;
	}
	/**
	 * @param fileSeparator sequence of characters to be used in separating fields in the file element.
	 */
	public void setFileSeparator(String fileSeparator) {
		this.fileSeparator = fileSeparator;
	}
	/**
	 * Returns a list of elements composing file filed. 
	 * @return unmodifiable <code>List</code> of <code>String</code> file elements.
	 */
	public List getFileElements() {
		return Collections.unmodifiableList(fileElements);
	}
	
	/**
	 * Append a new file element to the end of the existing file elements
	 * @param element - a string to be apppeded to the end of file field (after a separater)
	 */
	public void appendFileElement(String element) {
		fileElements.add(element);
	}
	
	/**
	 * Insert file element into a specified position
	 * @param index - index of the position at which the element should be inserted
	 * @param element - the string element ot be inserted
	 */
	public void insertFileElement(int index,String element) {
		fileElements.add(index,element);
	}
	
	/**
	 * Remove file element at a specified position
	 * @param index - index of an element to be removed
	 */
	public void removeFileElement(int index) {
		fileElements.remove(index);
	}
	
	/**
	 * Remove a specified file element
	 * @param element - the element to be removed
	 */
	public void removeFileElement(String element) {
		fileElements.remove(element);
	}

	/**
	 * Provides a set of parameter names currently set
	 * @return a <code>Set</code> of <code>String</code> names
	 */
	public Set getParameterNames() {
		return parameters.keySet();
	}
	
	/**
	 * Returns a set of values for a given parameter
	 * @param parameterName - parameter name
	 * @return an unmodifiable <code>Set</code> of <code>String</code> parameter values
	 */
	public Set getParameterValues(String parameterName) {
		return Collections.unmodifiableSet((Set)parameters.get(parameterName));
	}
	
	/**
	 * Add a value to a given parameter name.
	 * The method can be used to set multi-valued parameters
	 * @param parameterName
	 * @param parameterValue
	 */
	public void addParameterValue(String parameterName, String parameterValue) {
		Set s=(Set)parameters.get(parameterName);
		if(s==null) {
			s=new HashSet();
			parameters.put(parameterName,s);
		}
		s.add(parameterValue);
	}
	
	/**
	 * Set value of a parameter
	 * This method call implies single-valued parameter. The method will reset all existing parameter values
	 * @param parameterName
	 * @param parameterValue
	 */
	public void setParameterValue(String parameterName, String parameterValue) {
		Set s=(Set)parameters.get(parameterName);
		if(s==null) {
			s=new HashSet();
			parameters.put(parameterName,s);
		}
		s.clear();
		s.add(parameterValue);
	}
	
	/**
	 * Remove a certain value of a given parameter
	 * @param parameterName
	 * @param parameterValue
	 * @return true If the url contained the specified parameter.
	 */
	public boolean removeParameterValue(String parameterName, String parameterValue) {
		Set s=(Set)parameters.get(parameterName);
		if(s!=null) {
			return s.remove(parameterValue);
		} else {
			return false;
		}
	}
	
	/**
	 * Remove all values of a given parameter
	 * @param parameterName
	 */
	public void clearParameterValues(String parameterName) {
		parameters.remove(parameterName);
	}
	
	/**
	 * Returns a list of path elements
	 * @return unmodifiable <code>List</code> of <code>String</code> path elements.
	 */
	public List getPathElements() {
		return Collections.unmodifiableList(pathElements);
	}
	
	/**
	 * Append a new path element to the end of the existing path elements
	 * @param element - a string to be apppeded to the URL path after a separator
	 */
	public void appendPathElement(String element) {
		pathElements.add(element);
	}
	
	/**
	 * Insert path element into a specified position
	 * @param index - index of the position at which the element should be inserted
	 * @param element - the string element ot be inserted
	 */
	public void insertPathElement(int index,String element) {
		pathElements.add(index,element);
	}
	
	/**
	 * Remove path element at a specified position
	 * @param index - index of an element to be removed
	 */
	public void removePathElement(int index) {
		pathElements.remove(index);
	}
	
	/**
	 * Returns a string representation of the constructed URL
	 * @return complete serialized URL
	 */
	public String toString() {
        StringBuffer sb=new StringBuffer(10);
        
        // output host
        if(host!=null) {
            if(protocol!=null) {
                sb.append(protocol);
            } else {
                sb.append(HTTP_PROTOCOL);
            }
            sb.append(PROTOCOL_SERPARATOR);
            sb.append(host);
            if(port!=null) {
                sb.append(PORT_SERPARATOR);
                sb.append(port);
            }
        }
        
        
		// append file path
		for(Iterator i=pathElements.iterator();i.hasNext();) {
			sb.append((String) i.next());
			sb.append(PATH_SEPARATOR);
		}
			
		// append file content
		for(Iterator j=fileElements.iterator();j.hasNext();) {
			sb.append((String)j.next());
			if(j.hasNext()) {
				sb.append(this.fileSeparator);
			}
		}
		
		// append suffix
		if(this.fileSuffix!=null) {
			sb.append(this.fileSeparator);
			sb.append(this.fileSuffix);
		}
		
		try {
		 // append parameters
		 if(!parameters.isEmpty()) {
			sb.append(PARAMETER_START);
			for(Iterator k=parameters.keySet().iterator();k.hasNext();) {
				String pName=(String)k.next();
				// obtain list of parameter values
				Set pValues=(Set)parameters.get(pName);
				String pAssignment=URLEncoder.encode(pName,"UTF-8")+PARAMETER_ASSIGNMENT;
				// output parameter values
				for(Iterator l=pValues.iterator();l.hasNext();) {
					String pValue = (String) l.next();
					if ( pValue == null ) continue;
					sb.append(pAssignment);
					sb.append(URLEncoder.encode(pValue,"UTF-8"));
					if(l.hasNext()) {
						sb.append(PARAMETER_SEPARATOR);
					}
				}
				if(k.hasNext()) {
					sb.append(PARAMETER_SEPARATOR);
				}
			}
		  }
		} catch ( UnsupportedEncodingException e ) {
			throw new PortalRuntimeException(e);
		}
        // append fragment
        if(fragment!=null) {
            sb.append(FRAGMENT_SERPARATOR);
            sb.append(fragment);
        }
		
		return sb.toString();
	}
	
	/**
	 * Returns complete URL
	 * @return URL containing all specified elements
	 * @throws MalformedURLException
	 */
	public URL toURL() throws MalformedURLException {
		return new URL(this.toString()); 
	}
	
	/**
	 * Returns complete URI
	 * @return URI containing all specified elements
	 * @throws URISyntaxException
	 */
	public URI toURI() throws URISyntaxException {
		return new URI(this.toString());
	}
    /**
     * Set host field. As in "<code>localhost</code>" in "<code>http://localhost/uP3/</code>"
     * @param host
     */
    public void setHost(String host) {
        this.host = host;
    }
    /**
     * Set port. As in "<code>8080</code>" in "<code>http://localhost:8080/uP3/</code>"
     * Note: port will be set only if host is specified
     * @param port
     */
    public void setPort(String port) {
        this.port = port;
    }
    /**
     * Set protocol, as in "<code>http</code>" (default) or "<code>https</code>"
     * Note: will be set only if host is specified
     * @param protocol
     */
    public void setProtocol(String protocol) {
        this.protocol = protocol;
    }
    /**
     * Set fragment, as in "<code>pagemark</code>" in "<code>http://localhost/uP3/render.uP#pagemark</code>"
     * @param fragment
     */
    public void setFragment(String fragment) {
        this.fragment = fragment;
    }
}
