/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.url.syntax;

/**
 * An interface describing a number of utility methods that help to define common syntax conventions for different
 * url constructors/processors.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public interface IUrlBasicSyntaxProvider {
    /**
     * Default encoding to use by various URL-related classes (defaults to UTF-8)
     */
    public final static String DEFAULT_ENCODING="UTF-8";
    
    // variables
    /**
     * Check if a given string contains reference to a variable
     * @param s text (could be request parameter value, name or a part of 
     * @return <code>true</code> if the text contains unescaped reference to a variable
     */
    public boolean containsVariable(String s);
    
    /**
     * Checks if text contains a variable with a particular name
     * @param s the text
     * @param variableName variable name
     * @return <code>true</code> if text contains variable, otherwise <code>false</code>
     */
    public boolean containsVariable(String s, String variableName);
    
    /**
     * Checks if text contains variables for a particular processor
     * @param s the text
     * @param processorName
     * @return <code>true</code> if text contains variables, otherwise <code>false</code>
     */
    public boolean containsVariableForProcessor(String s,String processorName);
    
    /**
     * Parses a string, extracting variable names
     * @param s text
     * @return an array of variable names. If no variables around found in the string, an empty array is returned.
     */
    public String[] getVaribleNames(String s);
    
    /**
     * Extract variables addressed to a particular processor 
     * @param s text
     * @param processorName component name
     * @return variable names (stripped out of component affiliation)
     */
    public String[] getVariableNamesForProcessor(String s, String processorName);
    
    /**
     * Replace variable reference with the actual value
     * @param variableName variable reference name
     * @param value value
     * @param text the text containing variable
     * @return text with substituted variable
     */
    public String replaceVariableWithValue(String variableName, String value, String text);
    
    
    // processor affiliation
    /**
     * Returns the prefix of the component specified in the parameter name
     * @param fullParameterName request parameter or a variable name
     * @return an array of two elements, containing processor prefix followed by
     * the rest of the name, or <code>null</code> if no processor has been specified.
     */
    public String[] separateProcessorPrefix(String fullParameterName);
    
    /**
     * Get bare parameter name (without processor affiliation syntax) for a given processor.
     * @param processorPrefix name of the processor for which parameter should be addressed
     * @param fullParameterName full parameter name, containing processor affiliation syntax
     * @return bare parameter name, or <code>null</code> if the parameter was not affiliated with the specified
     * processor.
     * 
     * Note: if you can come up with a better name, go ahead and change this :)
     */
    public String getProcessorBareParameterName(String processorPrefix, String fullParameterName);
    
    /**
     * Incorporates processor prefix in a parameter or variable name
     * @param processorName processor prefix
     * @param parameterName request parameter or variable
     * @return a compound representation containing both processor prefix and original parameter (variable) name
     */
    public String incorporateProcessorPrefix(String processorName,String parameterName);

}
