/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.utils;

import java.lang.reflect.Array;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;

import org.apache.commons.collections.iterators.IteratorChain;


/**
 * Provides a layered view of two Collections. Only the targetCollection is mutable.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class LayeredCollection implements Collection {
    private final Collection targetCollection;
    private final Collection baseCollection;


    public LayeredCollection(Collection targetCollection, Collection baseCollection) {
        if (targetCollection == null) {
            throw new IllegalArgumentException("targetCollection may not be null");
        }
        if (baseCollection == null) {
            throw new IllegalArgumentException("baseCollection may not be null");
        }
        
        this.targetCollection = targetCollection;
        this.baseCollection = Collections.unmodifiableCollection(baseCollection);
    }

    /**
     * @see java.util.Collection#size()
     */
    public int size() {
        return this.targetCollection.size() + this.baseCollection.size();
    }

    /**
     * @see java.util.Collection#clear()
     */
    public void clear() {
        this.targetCollection.clear();
    }

    /**
     * @see java.util.Collection#isEmpty()
     */
    public boolean isEmpty() {
        return this.targetCollection.isEmpty() && this.baseCollection.isEmpty();
    }

    /**
     * @see java.util.Collection#toArray()
     */
    public Object[] toArray() {
        return this.toArray(new Object[this.size()]);
    }

    /**
     * @see java.util.Collection#add(java.lang.Object)
     */
    public boolean add(Object o) {
        return this.targetCollection.add(o);
    }

    /**
     * @see java.util.Collection#contains(java.lang.Object)
     */
    public boolean contains(Object o) {
        return this.targetCollection.contains(o) || this.baseCollection.contains(o);
    }

    /**
     * @see java.util.Collection#remove(java.lang.Object)
     */
    public boolean remove(Object o) {
        return this.targetCollection.remove(o);
    }

    /**
     * @see java.util.Collection#addAll(java.util.Collection)
     */
    public boolean addAll(Collection c) {
        return this.targetCollection.addAll(c);
    }

    /**
     * @see java.util.Collection#containsAll(java.util.Collection)
     */
    public boolean containsAll(Collection c) {
        for (final Iterator cItr = c.iterator(); cItr.hasNext();) {
            if (!this.contains(cItr.next())) {
                return false;
            }
        }
        
        return true;
    }

    /**
     * @see java.util.Collection#removeAll(java.util.Collection)
     */
    public boolean removeAll(Collection c) {
        return this.targetCollection.removeAll(c);
    }

    /**
     * @see java.util.Collection#retainAll(java.util.Collection)
     */
    public boolean retainAll(Collection c) {
        return this.targetCollection.retainAll(c);
    }

    /**
     * @see java.util.Collection#iterator()
     */
    public Iterator iterator() {
        return new IteratorChain(this.targetCollection.iterator(), this.baseCollection.iterator());
    }

    /**
     * @see java.util.Collection#toArray(java.lang.Object[])
     */
    public Object[] toArray(Object[] array) {
        if (array.length < this.size()) {
            array = (Object[])Array.newInstance(array.getClass().getComponentType(), this.size());
        }

        final Object[] targetArray = this.targetCollection.toArray();
        final Object[] baseArray = this.baseCollection.toArray();
        
        System.arraycopy(targetArray, 0, array, 0, targetArray.length);
        System.arraycopy(baseArray, 0, array, targetArray.length, baseArray.length);
        
        if (array.length > this.size()) {
            array[this.size()] = null;
        }
        
        return array;
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (!(obj instanceof Collection)) {
            return false;
        }
        
        final Collection other = (Collection)obj;
        return (this.size() == other.size()) && this.containsAll(other);
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        int hash = 0;
        
        for (final Iterator itr = this.iterator(); itr.hasNext();) {
            final Object o = itr.next();
            if (o != null) {
                hash += o.hashCode();
            }
        }
        
        return hash;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        final StringBuffer buf = new StringBuffer(this.size() * 3);
        buf.append("[");
        for (final Iterator itr = this.iterator(); itr.hasNext();) {
            buf.append(itr.next());
            if (itr.hasNext()) {
                buf.append(", ");
            }
        }
        buf.append("]");
        return  buf.toString();
    }
}
