/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.utils;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;


/**
 * Provides a layered view of two Maps. Only the targetMap is mutable.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class LayeredMap<K, V> implements Map<K, V> {
    private final Map<K, V> targetMap;
    private final Map<K, V> baseMap;
    private final Map<K, V> compositeMap = new HashMap<K, V>();
    
    public LayeredMap(Map<K, V> targetMap, Map<K, V> baseMap) {
        if (targetMap == null) {
            throw new IllegalArgumentException("targetMap may not be null");
        }
        if (baseMap == null) {
            throw new IllegalArgumentException("baseMap may not be null");
        }
        
        this.targetMap = targetMap;
        this.baseMap = Collections.unmodifiableMap(baseMap);
        this.updateCompositeMap();
    }
    
    /**
     * @see java.util.Map#size()
     */
    public int size() {
        return this.compositeMap.size();
    }

    /**
     * @see java.util.Map#clear()
     */
    public void clear() {
        this.targetMap.clear();
        this.updateCompositeMap();
    }

    /**
     * @see java.util.Map#isEmpty()
     */
    public boolean isEmpty() {
        return this.compositeMap.isEmpty();
    }

    /**
     * @see java.util.Map#containsKey(java.lang.Object)
     */
    public boolean containsKey(Object key) {
        return this.compositeMap.containsKey(key);
    }

    /**
     * @see java.util.Map#containsValue(java.lang.Object)
     */
    public boolean containsValue(Object value) {
        return this.compositeMap.containsValue(value);
    }

    /**
     * @see java.util.Map#values()
     */
    public Collection<V> values() {
        return new MapLayeredCollection<V>(this.compositeMap.values(), this.targetMap.values());
    }

    /**
     * @see java.util.Map#putAll(java.util.Map)
     */
    public void putAll(Map<? extends K, ? extends V> t) {
        this.targetMap.putAll(t);
        this.compositeMap.putAll(t);
    }

    /**
     * @see java.util.Map#entrySet()
     */
    public Set<Map.Entry<K, V>> entrySet() {
        return new MapLayeredSet<Map.Entry<K, V>>(this.compositeMap.entrySet(), this.targetMap.entrySet());
    }

    /**
     * @see java.util.Map#keySet()
     */
    public Set<K> keySet() {
        return new MapLayeredSet<K>(this.compositeMap.keySet(), this.targetMap.keySet());
    }

    /**
     * @see java.util.Map#get(java.lang.Object)
     */
    public V get(Object key) {
        return this.compositeMap.get(key);
    }

    /**
     * @see java.util.Map#remove(java.lang.Object)
     */
    public V remove(Object key) {
        final V removed = this.targetMap.remove(key);
        this.updateCompositeMap();
        return removed;
    }

    /**
     * @see java.util.Map#put(java.lang.Object, java.lang.Object)
     */
    public V put(K key, V value) {
        this.compositeMap.put(key, value);
        return this.targetMap.put(key, value);
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (!(obj instanceof Map)) {
            return false;
        }
        
        final Map other = (Map)obj;
        return this.entrySet().equals(other.entrySet());
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        int hash = 0;
        for (final Iterator entryItr = this.entrySet().iterator(); entryItr.hasNext();) {
            final Map.Entry entry = (Map.Entry)entryItr.next();
            hash += entry.hashCode();
        }
        return hash;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return this.compositeMap.toString();
    }

    private void updateCompositeMap() {
        this.compositeMap.clear();
        this.compositeMap.putAll(this.baseMap);
        this.compositeMap.putAll(this.targetMap);
    }
    

    private class MapLayeredSet<E> implements Set<E> {
        private final Set<E> compositeSet;
        private final Set<E> targetSet;
        
        public MapLayeredSet(Set<E> compositeSet, Set<E> targetSet) {
            this.compositeSet = compositeSet;
            this.targetSet = targetSet;
        }

        public boolean add(E o) {
            //No action as this is not supported by the Map interface
            return this.compositeSet.add(o);
        }

        public boolean addAll(Collection<? extends E> c) {
            //No action as this is not supported by the Map interface
            return this.compositeSet.addAll(c);
        }

        public void clear() {
            this.targetSet.clear();
            LayeredMap.this.updateCompositeMap();
        }

        public boolean contains(Object o) {
            return this.compositeSet.contains(o);
        }

        public boolean containsAll(Collection c) {
            return this.compositeSet.containsAll(c);
        }

        public boolean equals(Object o) {
            return this.compositeSet.equals(o);
        }

        public int hashCode() {
            return this.compositeSet.hashCode();
        }

        public boolean isEmpty() {
            return this.compositeSet.isEmpty();
        }

        public Iterator<E> iterator() {
            return new MapLayeredIterator<E>(this.compositeSet.iterator(), this.targetSet);
        }

        public boolean remove(Object o) {
            final boolean changed = this.targetSet.remove(o);
            LayeredMap.this.updateCompositeMap();
            return changed;
        }

        public boolean removeAll(Collection<?> c) {
            final boolean changed = this.targetSet.removeAll(c);
            LayeredMap.this.updateCompositeMap();
            return changed;
        }

        public boolean retainAll(Collection<?> c) {
            final boolean changed = this.targetSet.retainAll(c);
            LayeredMap.this.updateCompositeMap();
            return changed;
        }

        public int size() {
            return this.compositeSet.size();
        }

        public Object[] toArray() {
            return this.compositeSet.toArray();
        }

        public<T> T[] toArray(T[] a) {
            return this.compositeSet.toArray(a);
        }
    }
    
    private class MapLayeredCollection<E> implements Collection<E> {
        private final Collection<E> compositeCollection;
        private final Collection<E> targetCollection;
        
        public MapLayeredCollection(Collection<E> compositeCollection, Collection<E> targetCollection) {
            this.compositeCollection = compositeCollection;
            this.targetCollection = targetCollection;
        }

        public boolean add(E o) {
            //No action as this is not supported by the Map interface
            return this.compositeCollection.add(o);
        }

        public boolean addAll(Collection<? extends E> c) {
            //No action as this is not supported by the Map interface
            return this.compositeCollection.addAll(c);
        }

        public void clear() {
            this.targetCollection.clear();
            LayeredMap.this.updateCompositeMap();
        }

        public boolean contains(Object o) {
            return this.compositeCollection.contains(o);
        }

        public boolean containsAll(Collection<?> c) {
            return this.compositeCollection.containsAll(c);
        }

        public boolean equals(Object o) {
            return this.compositeCollection.equals(o);
        }

        public int hashCode() {
            return this.compositeCollection.hashCode();
        }

        public boolean isEmpty() {
            return this.compositeCollection.isEmpty();
        }

        public Iterator<E> iterator() {
            return new MapLayeredIterator<E>(this.compositeCollection.iterator(), this.targetCollection);
        }

        public boolean remove(Object o) {
            final boolean changed = this.targetCollection.remove(o);
            LayeredMap.this.updateCompositeMap();
            return changed;
        }

        public boolean removeAll(Collection<?> c) {
            final boolean changed = this.targetCollection.removeAll(c);
            LayeredMap.this.updateCompositeMap();
            return changed;
        }

        public boolean retainAll(Collection<?> c) {
            final boolean changed = this.targetCollection.retainAll(c);
            LayeredMap.this.updateCompositeMap();
            return changed;
        }

        public int size() {
            return this.compositeCollection.size();
        }

        public Object[] toArray() {
            return this.compositeCollection.toArray();
        }

        public<T> T[] toArray(T[] a) {
            return this.compositeCollection.toArray(a);
        }
    }

    private class MapLayeredIterator<E> implements Iterator<E> {
        private final Iterator<E> compositeIterator;
        private final Collection<E> targetSet;
        private E prevObj = null;
        private boolean canRemove = false;
        
        public MapLayeredIterator(Iterator<E> compositeIterator, Collection<E> targetSet) {
            this.compositeIterator = compositeIterator;
            this.targetSet = targetSet;
        }

        /**
         * @see java.util.Iterator#remove()
         */
        public void remove() {
            if (!this.canRemove) {
                throw new IllegalStateException("next() must be called before remove()");
            }
            
            boolean remove = false;
            
            remove = this.targetSet.remove(this.prevObj);
            
            if (remove) {
                this.compositeIterator.remove();
            }
            
            this.prevObj = null;
            this.canRemove = false;
        }

        /**
         * @see java.util.Iterator#hasNext()
         */
        public boolean hasNext() {
            return this.compositeIterator.hasNext();
        }

        /**
         * @see java.util.Iterator#next()
         */
        public E next() {
            final E o = this.compositeIterator.next();
            this.prevObj = o;
            this.canRemove = true;
            return o;
        }
    }
}
