/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.utils;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;


/**
 * Provides a layered view of two Sets. Only the targetSet is mutable. Note
 * this implementation is NOT syncrhonized and the synchronization on Sets
 * composing this Set does not matter.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class LayeredSet implements Set {
    private final Set targetSet;
    private final Set baseSet;
    private final Set compositeSet = new HashSet();
    
    public LayeredSet(Set targetSet, Set baseSet) {
        if (targetSet == null) {
            throw new IllegalArgumentException("targetSet may not be null");
        }
        if (baseSet == null) {
            throw new IllegalArgumentException("baseSet may not be null");
        }
        
        this.targetSet = targetSet;
        this.baseSet = Collections.unmodifiableSet(baseSet);
        
        this.updateCompositeSet();
    }

    /**
     * @see java.util.Set#add(java.lang.Object)
     */
    public boolean add(Object o) {
        this.compositeSet.add(o);
        return this.targetSet.add(o);
    }

    /**
     * @see java.util.Set#addAll(java.util.Collection)
     */
    public boolean addAll(Collection c) {
        this.compositeSet.addAll(c);
        return this.targetSet.addAll(c);
    }

    /**
     * @see java.util.Set#clear()
     */
    public void clear() {
        this.targetSet.clear();
        this.updateCompositeSet();
    }

    /**
     * @see java.util.Set#contains(java.lang.Object)
     */
    public boolean contains(Object o) {
        return this.compositeSet.contains(o);
    }

    /**
     * @see java.util.Set#containsAll(java.util.Collection)
     */
    public boolean containsAll(Collection c) {
        return this.compositeSet.containsAll(c);
    }

    /**
     * @see java.util.Set#isEmpty()
     */
    public boolean isEmpty() {
        return this.compositeSet.isEmpty();
    }

    /**
     * @see java.util.Set#iterator()
     */
    public Iterator iterator() {
        return new SetLayeredIterator(this.compositeSet.iterator(), this.targetSet);
    }

    /**
     * @see java.util.Set#remove(java.lang.Object)
     */
    public boolean remove(Object o) {
        final boolean removed = this.targetSet.remove(o);
        this.updateCompositeSet();
        return removed;
    }

    /**
     * @see java.util.Set#removeAll(java.util.Collection)
     */
    public boolean removeAll(Collection c) {
        final boolean changed = this.targetSet.removeAll(c);
        this.updateCompositeSet();
        return changed;
    }

    /**
     * @see java.util.Set#retainAll(java.util.Collection)
     */
    public boolean retainAll(Collection c) {
        final boolean changed = this.targetSet.retainAll(c);
        this.updateCompositeSet();
        return changed;
    }

    /**
     * @see java.util.Set#size()
     */
    public int size() {
        return this.compositeSet.size();
    }

    /**
     * @see java.util.Set#toArray()
     */
    public Object[] toArray() {
        return this.compositeSet.toArray();
    }

    /**
     * @see java.util.Set#toArray(java.lang.Object[])
     */
    public Object[] toArray(Object[] a) {
        return this.compositeSet.toArray(a);
    }
    
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof Set)) {
            return false;
        }

        final Set other = (Set)obj;
        if (other.size() != this.size()) {
            return false;
        }
        
        return this.containsAll(other);
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        int hash = 0;
        
        for (final Iterator itr = this.iterator(); itr.hasNext();) {
            final Object obj = itr.next();
            
            if (obj != null) {
                hash += obj.hashCode();
            }
        }

        return hash;
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return this.compositeSet.toString();
    }

    private void updateCompositeSet() {
        this.compositeSet.clear();
        this.compositeSet.addAll(this.baseSet);
        this.compositeSet.addAll(this.targetSet);
    }
    
    
    private class SetLayeredIterator implements Iterator {
        private final Iterator compositeIterator;
        private final Set targetSet;
        private Object prevObj = null;
        private boolean canRemove = false;
        
        public SetLayeredIterator(Iterator compositeIterator, Set targetSet) {
            this.compositeIterator = compositeIterator;
            this.targetSet = targetSet;
        }

        /**
         * @see java.util.Iterator#remove()
         */
        public void remove() {
            if (!this.canRemove) {
                throw new IllegalStateException("next() must be called before remove()");
            }
            
            boolean remove = false;
            
            remove = this.targetSet.remove(this.prevObj);
            
            if (remove) {
                this.compositeIterator.remove();
            }
            
            this.prevObj = null;
            this.canRemove = false;
        }

        /**
         * @see java.util.Iterator#hasNext()
         */
        public boolean hasNext() {
            return this.compositeIterator.hasNext();
        }

        /**
         * @see java.util.Iterator#next()
         */
        public Object next() {
            final Object o = this.compositeIterator.next();
            this.prevObj = o;
            this.canRemove = true;
            return o;
        }
    }
}

