/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.utils;

import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

/**
 * A Set that holds listeners using WeakReference and allows concurrent modification
 * by using <code>CopyOnWriteArraySet</code> for the underlying implementaiton.
 * Useful for keeping track of listeners  
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 1.0$
 */
public class ListenerSet<T> implements Set<T> {
    CopyOnWriteArraySet<EqualWeakReference<T>> set=new CopyOnWriteArraySet<EqualWeakReference<T>>();

    public boolean add(T arg0) {
        return set.add(new EqualWeakReference<T>(arg0));
    }

    public boolean addAll(Collection<? extends T> arg0) {
        boolean changed=false;
        for(T e: arg0) {
            changed=changed || set.add(new EqualWeakReference<T>(e));
        }
        return changed;
    }

    public void clear() {
        set.clear();
    }

    public boolean contains(Object arg0) {
        return set.contains(new EqualWeakReference(arg0));
    }

    public boolean containsAll(Collection<?> arg0) {
        for(Object o:arg0) {
            if(!set.contains(o)) {
                return false;
            }
        }
        return true;
    }



    public boolean equals(Object arg0) {
        return set.equals(arg0);
    }



    public int hashCode() {
        return set.hashCode();
    }



    public boolean isEmpty() {
        return set.isEmpty();
    }



    public Iterator<T> iterator() {
        return new DereferencingIterator<T>(set.iterator(),set);
    }



    public boolean remove(Object arg0) {
        return set.remove(new EqualWeakReference(arg0));
    }



    public boolean removeAll(Collection<?> arg0) {
        boolean changed=false;
        for(Object o: arg0) {
            changed=changed || set.remove(new EqualWeakReference(arg0));
        }
        return changed;
    }



    public boolean retainAll(Collection<?> arg0) {
        boolean changed=false;
        for(Iterator<EqualWeakReference<T>> i=set.iterator();i.hasNext();) {
            EqualWeakReference<T> r=i.next();
            T o=r.get();
            if(o!=null) {
                if(!arg0.contains(o)) {
                    set.remove(r);
                    changed=true;
                }
            } else {
                set.remove(r);
                changed=true;
            }
        }
        return changed;
    }



    public int size() {
        return set.size();
    }



    public Object[] toArray() {
        List l=new ArrayList(set.size());
        for(EqualWeakReference<T> r: set) {
            T o=r.get();
            if(o!=null) {
                l.add(o);
            }
        }
        return l.toArray();
    }



    public <E> E[] toArray(E[] arg0) {
        List<E> l=new ArrayList(set.size());
        for(EqualWeakReference<T> r: set) {
            T o=r.get();
            if(o!=null) {
                l.add((E) o);
            }
        }
        return l.toArray(arg0);
    }



    public String toString() {
        return set.toString();
    }
    
    /**
     * An extension of a weak reference class that can be compared
     * based on the referenced object.
     *
     * @author Peter Kharchenko: pkharchenko at unicon.net
     * @version $Revision: 1.0$
     */
    static class EqualWeakReference<E> extends WeakReference<E> {
        int hashCode;
        
        
        public EqualWeakReference(E o) {
            super(o);
            this.hashCode=o.hashCode();
        }
        
        public EqualWeakReference(E o, ReferenceQueue<? super E> q) {
            super(o,q);
            this.hashCode=o.hashCode();
        }

        @Override
        public boolean equals(Object o) {
            if(this==o) { return true; }
            if(!(o instanceof EqualWeakReference)) { return false; }
            E to=this.get();
            Object oo=((EqualWeakReference)o).get();
            if(to==null || oo==null) { return false; }
            if(to==oo) { return true; }
            return(to.equals(oo));
        }

        @Override
        public int hashCode() {
            return this.hashCode;
        }
    }
    
    static class DereferencingIterator<E> implements Iterator<E> {
        Iterator<EqualWeakReference<E>> it;
        Set<EqualWeakReference<E>> set;
        EqualWeakReference<E> r=null;

        public <T> DereferencingIterator(Iterator<EqualWeakReference<E>> it, Set<EqualWeakReference<E>> set) {
            this.it=it;
            this.set=set;
        }
        
        public boolean hasNext() {
            return it.hasNext();
        }

        public E next() {
            r=it.next();
            if(r!=null) {
                E o=r.get();
                if(o==null) {
                    set.remove(r);
                }
                return o;
            } else {
                return null;
            }
        }

        public void remove() {
            if(r!=null) {
                set.remove(r);
            }
        }
    }
}
