package org.jasig.portal.utils;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

/**
 * Takes a absolute or relative path and normalizes it.
 * <br>
 * <br>
 * Examples:
 * <pre>
 * / -> /
 * /foo -> /foo
 * /foo/bar -> /foo/bar
 * /foo/../bar -> /bar
 * ../bar/foo -> ../bar/foo
 * ./bar/foo -> bar/foo
 * . -> .
 * //.//./../foo -> Invalid path. Cannot escape above the root element
 * /foo/b.ar/../.// -> /foo
 * /foo/bar/../.. -> /
 * /foo/bar/../../ -> /
 * foo/bar/../.. -> .
 * foo/bar/../../ -> .
 * foo/bar/../../foo/bar -> foo/bar
 * foo/bar/../../../foo/bar -> ../foo/bar
 * /foo/bar/../../foo/bar -> /foo/bar
 * /foo/bar/../../../foo/bar -> Invalid path. Cannot escape above the root element
 * </pre>
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class NormalizedPath {
    public final static String PATH_SEPARATOR = "/";
    public final static String PARENT_ELEMENT_NAME = "..";
    public final static String CURRENT_ELEMENT_NAME = ".";
    
    private final static Pattern PATH_PATTERN = Pattern.compile("(" + PATH_SEPARATOR + "?)([^" + PATH_SEPARATOR + "]*)");
    
    private final List pathElements;
    private final String normalPath;
    
    /**
     * Creates a new normalized path from the specified path.
     * 
     * @param basePath The path to normalize, may not be null.
     */
    public NormalizedPath(String basePath) {
        if (basePath == null) {
            throw new IllegalArgumentException("path may not be null");
        }

        final List elements = this.parsePathString(basePath);
        this.pathElements = Collections.unmodifiableList(new ArrayList(elements));
        
        final StringBuffer normalPathBuff = new StringBuffer();
        for (final Iterator elementItr = this.pathElements.iterator(); elementItr.hasNext();) {
            final String element = (String)elementItr.next();
            normalPathBuff.append(element);
        }
        this.normalPath = normalPathBuff.toString();
    }
    
    /**
     * Gets a List representation of the Path. The List contains String
     * objects with one String per path element.
     * <br>
     * <br>
     * Example:
     * <pre>
     * ../foo/bar -> [.., /, foo, /, bar]
     * </pre>
     * 
     * @return Immutable List of path elements, will not be null and will have size >= 1.
     */
    public List getPathElements() {
        return this.pathElements;
    }
    
    /**
     * Responsible for taking the basePath String and returning the normalized
     * List of path elements. 
     * 
     * @param basePath The path to normalize, will not be null.
     * @return The normalized path elements, may not be null.
     */
    protected List parsePathString(String basePath) {
        final Matcher matches = PATH_PATTERN.matcher(basePath);
        final List currentPath = new ArrayList();
        
        //Iterate through all the match groups in the path
        while (matches.find()) {
            //Add the slash to the path if it exists
            final String slash = matches.group(1);
            if (slash.length() > 0) {
                this.addElement(currentPath, slash);
            }
            
            //Add the element to the path if it exists
            final String element = matches.group(2);
            if (element.length() > 0) {
                this.addElement(currentPath, element);
            }
        }
        
        if (currentPath.size() > 1 && PATH_SEPARATOR.equals(currentPath.get(currentPath.size() - 1))) {
            currentPath.remove(currentPath.size() - 1);
        }
        if (currentPath.size() == 0) {
            currentPath.add(".");
        }
        
        return currentPath;
    }
    
    /**
     * Does the normalization work. Called by the default {@link #parsePathString(String)} for
     * each path element. The method updates the currentPath list appropriatly based on the element
     * being added to the list.
     * 
     * @param currentPath The current normalized path representation, may not be null.
     * @param element The element to add to the path, may not be null.
     */
    protected void addElement(List currentPath, String element) {
        //If working with an empty path just add the element
        if (currentPath.size() == 0) {
            currentPath.add(element);
        }
        else {
            if (PATH_SEPARATOR.equals(element)) {
                if (currentPath.size() > 0) {
                    if (CURRENT_ELEMENT_NAME.equals(currentPath.get(currentPath.size() - 1))) {
                        //If a seperator and the previous element was the current element string
                        //remove it ./ is the same as nothing
                        currentPath.remove(currentPath.size() - 1);
                    }
                    else if (!PATH_SEPARATOR.equals(currentPath.get(currentPath.size() - 1))) {
                        //If the previous element wasn't a seperator add this seperator
                        //prevents //
                        currentPath.add(element);
                    } 
                }
            }
            else if (PARENT_ELEMENT_NAME.equals(element)) {
                if (currentPath.size() <= 1) {
                    //Trying to escape up from the root element
                    throw new IllegalArgumentException("Invalid path. Cannot escape above the root element");
                }
                else if (currentPath.size() > 1 && !PARENT_ELEMENT_NAME.equals(currentPath.get(currentPath.size() - 2))) {
                    //Escaping the previous elements if 2 or more exist and the previous element
                    //wasn't a parent string.
                    currentPath.remove(currentPath.size() - 1);
                    currentPath.remove(currentPath.size() - 1);
                    
                    //If escaped to the root of the path add the current element string.
                    if (currentPath.size() == 0) {
                        currentPath.add(CURRENT_ELEMENT_NAME);
                    }
                }
                else {
                    //Add the element
                    currentPath.add(element);
                }
            }
            else if (!CURRENT_ELEMENT_NAME.equals(element)) {
                //Add the element
                currentPath.add(element);
            }
        }
    }
    
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object arg0) {
        if (arg0 == this) {
            return true;
        }
        if (!(arg0 instanceof NormalizedPath)) {
            return false;
        }
        
        return this.normalPath.equals(((NormalizedPath)arg0).normalPath);
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return this.normalPath.hashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return this.normalPath;
    }
}
