/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.utils;

import java.io.IOException;
import java.lang.ref.WeakReference;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;


/**
 * This filter should be the first filter in the chain for this context.<br><br>
 * 
 * It looks for a HttpServletRequest to get a HttpSession from. If found it is put in
 * a ThreadLocal accessible via the static {@link #getSession()} method. A finally block
 * in after the request is passed up the chain is used to remove the session from the 
 * ThreadLocal.<br><br>
 * 
 * The HttpSession is also wrapped in a WeakReference. This ensures that a failure to remove
 * the session from the ThreadLocal for some reason won't result in a memory leak.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class SessionLocalBindingFilter implements Filter {
    private static final ThreadLocal<WeakReference<HttpSession>> SESSION_LOCAL = new TrackingThreadLocal<WeakReference<HttpSession>>();

    /**
     * Retrieves the HttpSession stored for the thread calling this method.
     * 
     * @return The HttpSession stored for the thread calling this method.
     */
    public static HttpSession getSession() {
        final WeakReference<HttpSession> ref = SESSION_LOCAL.get();
        if (ref != null) {
            return ref.get();
        }
        else {
            return null;
        }
    }

    /**
     * Sets the HttpSession for this thread. Only accessible by package classes for security.
     * 
     * @param session The HttpSession for this thread.
     */
    static void setSession(HttpSession session) {
        if (session != null) {
            SESSION_LOCAL.set(new WeakReference<HttpSession>(session));
        }
        else {
            SESSION_LOCAL.set(null);
        }
    }

    /**
     * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
     */
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
            throws IOException, ServletException {
        try {
            //Only HttpServletRequests have sessions
            if (request instanceof HttpServletRequest) {
                //Cast the request
                final HttpServletRequest httpRequest = (HttpServletRequest)request;
                //Get the session (creating it if it doesn't exist)
                final HttpSession session = httpRequest.getSession();
                //Bind the session to this thread
                setSession(session);
            }
            else {
                //No session, clear the thread local
                setSession(null);
            }

            //Pass the request up the chain
            chain.doFilter(request, response);
        }
        finally {
            //Clear the thread local after the request is processed
            setSession(null);
        }
    }

    /**
     * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
     */
    public void init(FilterConfig filterConfig) throws ServletException {
    }

    /**
     * @see javax.servlet.Filter#destroy()
     */
    public void destroy() {
    }
}
