/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.utils;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * This class workes with the {@link TrackingThreadLocal} class to copy ThreadLocal
 * values between threads.<br><br>
 * 
 * When this class is instantiated it uses {@link TrackingThreadLocal#getInstances()}
 * to iterate over all of the existing TrackingThreadLocal objects. The value of each
 * is stored in a class member Map keyed using the ThreadLocal instance.<br><br>
 * 
 * When {@link #run()} is called the ThreadLocal values stored in the member map are
 * stored in the ThreadLocals for the Thread calling {@link #run()}. The wrapped Runnable's
 * run method is then called. When it returns the all ThreadLocals that were populated
 * prior to the delgate call have their value set to null.<br><br>
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class ThreadLocalCopyRunnable implements Runnable {
    private static final Log LOG = LogFactory.getLog(ThreadLocalCopyRunnable.class);
    
    private final Runnable wrappedRunnable;
    private final boolean runOnce;
    private final Map<ThreadLocal, Object> localCopies = new HashMap<ThreadLocal, Object>();
    
    /**
     * Using this constructor {@link #run()} may be called any number of times.
     * Each call takes the same action of populating the calling Thread's ThreadLocals,
     * delegating the call and then clearing the ThreadLocals. This means this class
     * keeps strong references to all the TrackingThreadLocal instances that existed
     * when it was created along with their values at the time it was created.<br><br>
     * 
     * This is the same as calling {@link #ThreadLocalCopyRunnable(Runnable, boolean)}
     * with <code>false</code> as the second argument.
     * 
     * @param wrappedRunnable The Runnable to delegate to.
     * @throws IllegalArgumentException If wrappedRunnable is null.
     */
    public ThreadLocalCopyRunnable(Runnable wrappedRunnable) {
        this(wrappedRunnable, false);
    }

    /**
     * Using this constructor and passing <code>true</code> will clear the local
     * references to the TrackingThreadLocal instances and their value objects after
     * the first call to {@link #run()}.<br><br>
     * 
     * When {@link #run()} is called and after the values are copied to the calling
     * Thread's ThreadLocals the values referenced by the local Map are cleared.<br><br>
     * 
     * After the delegates run method returns and the ThreadLocals are cleared the local
     * Map is cleared so no references to the TrackingThreadLocals exist.<br><br>
     * 
     * Subsiquent calls to {@link #run()} will still call the delegates run method but
     * no ThreadLocal initialization will occure.
     * 
     * @param wrappedRunnable The Runnable to delegate to,
     * @param runOnce true if the references should be cleared after the first {@link #run()}
     * call. Passing <code>false</code> for this method is the same as calling
     * {@link #ThreadLocalCopyRunnable(Runnable)}
     * @throws IllegalArgumentException If wrappedRunnable is null.
     */
    public ThreadLocalCopyRunnable(Runnable wrappedRunnable, boolean runOnce) {
        if (wrappedRunnable == null) {
            throw new IllegalArgumentException("wrappedRunnable may not be null");
        }

        this.wrappedRunnable = wrappedRunnable;
        this.runOnce = runOnce;
        
        //Iterate through all TrackingTreadLocals storing references and their values
        //in a local map (not local to this thread)
        final Set<TrackingThreadLocal> locals = TrackingThreadLocal.getInstances();
        for (final TrackingThreadLocal local : locals) {
            localCopies.put(local, local.get());
        }
    }

    /**
     * @see java.lang.Runnable#run()
     */
    @SuppressWarnings("unchecked")
    public void run() {
        try {
            //Iterate through the local map of ThreadLocals and values. Setting the value
            //on each ThreadLocal.
            for (Map.Entry<ThreadLocal, Object> entry : this.localCopies.entrySet()) {
                final ThreadLocal<Object> local = entry.getKey();
                final Object value = entry.getValue();
                local.set(value);
                
                //If runOnce clear the value references.
                if (this.runOnce) {
                    entry.setValue(null);
                }
            }
            
            this.wrappedRunnable.run();
        }
        finally {
            //Iterate through the local map of ThreadLocals, clearing the value of each.
            //This happens in a finaly to try and avoid memory leaks.
            for (Map.Entry<ThreadLocal, Object> entry : this.localCopies.entrySet()) {
                try {
                    final ThreadLocal<Object> local = entry.getKey();
                    local.set(null);
                }
                catch (Exception e) {
                    LOG.error("Error clearing ThreadLocal from Map.Entry='" + entry + "'", e);
                }
            }
            
            //If runOnce clear the map.
            if (this.runOnce) {
                this.localCopies.clear();
            }
        }
    }
    
    protected boolean isRunOnce() {
    	return runOnce;
    }
}
