/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.utils;

import java.util.Collections;
import java.util.Set;


/**
 * A ThreadLocal subclass that keeps track of all created instances. Using this ThreadLocal
 * subclass in uPortal allows any {@link java.lang.Runnable} wrapped by
 * {@link org.jasig.portal.utils.ThreadLocalCopyRunnable} to have the values copied into the
 * appropriate ThreadLocals in the child thread when the Runable is run. This avoids having
 * to write explicit code for each ThreadLocal that needs to be copied.<br><br>  
 * 
 * The references kept the the ThreadLocal instances is done using a weak set so this tracking
 * subclass will not cause leaks by holding on to thread local instances that are no longer
 * referenced. The downside of this is if you set a value through a ThreadLocal that you don't
 * hold a strong reference to there may be no way to retrieve or remove that value from the
 * Thread.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class TrackingThreadLocal<T> extends ThreadLocal<T> {
    private static Set<TrackingThreadLocal> INSTANCES = new WeakHashSet<TrackingThreadLocal>();
    
    /**
     * Gets an umodifiable version of the Set of TrackingThreadLocal's that have
     * been created and are still referenced strongly somewhere in the JVM.
     * 
     * @return An unmodifiable set of TrackingThreadLocals
     */
    public static Set<TrackingThreadLocal> getInstances() {
        return Collections.unmodifiableSet(INSTANCES);
    }
    
    /**
     * Create a new TrackingThreadLocal and add to the the instances Set.
     */
    public TrackingThreadLocal() {
        INSTANCES.add(this);
    }
}
