/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.utils;



/**
 * Generates a string representation of the current path
 * location in the XML document. 
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class XmlLocationStack {
    private final StringBuffer location = new StringBuffer();
    
    /**
     * Starts an element, appending it to the path string.
     * 
     * @param name The name of the element to append.
     * @return The location after the element has been appended.
     */
    public synchronized String startElement(String name) {
        this.location.append(this.getPathElement(name));
        return this.toString();
    }
    
    /**
     * Ends an element, removing it from the end of the path string. The
     * specified name must match the element at the end of the path or an
     * IllegalStateException will be thrown.
     * 
     * @param name The name of the element to remove from the path.
     * @return The location before the element removal.
     */
    public synchronized String endElement(String name) {
        final String currentPath = this.location.toString();
        final String pathElement = this.getPathElement(name).toString();
        
        final int locationBaseLength = this.location.length() - pathElement.length();
        if (locationBaseLength < 0)
            throw new IllegalStateException("Cannot remove '" + name + "' from '" + this.location + "'");
        
        final String currLocElement = this.location.substring(locationBaseLength);
        
        if (!pathElement.equals(currLocElement))
            throw new IllegalStateException("Cannot remove '" + name + "' from '" + this.location + "'");
        
        this.location.delete(locationBaseLength, this.location.length());
        
        return currentPath;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public synchronized String toString() {
        return this.location.toString();
    }
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        return obj instanceof XmlLocationStack && this.toString().equals(obj.toString());
    }
    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return this.location.hashCode();
    }
    
    private StringBuffer getPathElement(String name) {
        return new StringBuffer("/").append(name);
    }
}
